if(typeof Math.imul == "undefined" || (Math.imul(0xffffffff,5) == 0)) {
    Math.imul = function (a, b) {
        var ah  = (a >>> 16) & 0xffff;
        var al = a & 0xffff;
        var bh  = (b >>> 16) & 0xffff;
        var bl = b & 0xffff;
        // the shift by 0 fixes the sign on the high part
        // the final |0 converts the unsigned value into a signed value
        return ((al * bl) + (((ah * bl + al * bh) << 16) >>> 0)|0);
    }
}

/**
 * React v15.3.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e();else if("function"==typeof define&&define.amd)define([],e);else{var t;t="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,t.React=e()}}(function(){return function e(t,n,r){function o(i,s){if(!n[i]){if(!t[i]){var u="function"==typeof require&&require;if(!s&&u)return u(i,!0);if(a)return a(i,!0);var l=new Error("Cannot find module '"+i+"'");throw l.code="MODULE_NOT_FOUND",l}var c=n[i]={exports:{}};t[i][0].call(c.exports,function(e){var n=t[i][1][e];return o(n?n:e)},c,c.exports,e,t,n,r)}return n[i].exports}for(var a="function"==typeof require&&require,i=0;i<r.length;i++)o(r[i]);return o}({1:[function(e,t,n){"use strict";var r=e(40),o=e(148),a={focusDOMComponent:function(){o(r.getNodeFromInstance(this))}};t.exports=a},{148:148,40:40}],2:[function(e,t,n){"use strict";function r(){var e=window.opera;return"object"==typeof e&&"function"==typeof e.version&&parseInt(e.version(),10)<=12}function o(e){return(e.ctrlKey||e.altKey||e.metaKey)&&!(e.ctrlKey&&e.altKey)}function a(e){switch(e){case k.topCompositionStart:return M.compositionStart;case k.topCompositionEnd:return M.compositionEnd;case k.topCompositionUpdate:return M.compositionUpdate}}function i(e,t){return e===k.topKeyDown&&t.keyCode===_}function s(e,t){switch(e){case k.topKeyUp:return C.indexOf(t.keyCode)!==-1;case k.topKeyDown:return t.keyCode!==_;case k.topKeyPress:case k.topMouseDown:case k.topBlur:return!0;default:return!1}}function u(e){var t=e.detail;return"object"==typeof t&&"data"in t?t.data:null}function l(e,t,n,r){var o,l;if(E?o=a(e):R?s(e,n)&&(o=M.compositionEnd):i(e,n)&&(o=M.compositionStart),!o)return null;N&&(R||o!==M.compositionStart?o===M.compositionEnd&&R&&(l=R.getData()):R=v.getPooled(r));var c=g.getPooled(o,t,n,r);if(l)c.data=l;else{var p=u(n);null!==p&&(c.data=p)}return h.accumulateTwoPhaseDispatches(c),c}function c(e,t){switch(e){case k.topCompositionEnd:return u(t);case k.topKeyPress:var n=t.which;return n!==w?null:(S=!0,P);case k.topTextInput:var r=t.data;return r===P&&S?null:r;default:return null}}function p(e,t){if(R){if(e===k.topCompositionEnd||s(e,t)){var n=R.getData();return v.release(R),R=null,n}return null}switch(e){case k.topPaste:return null;case k.topKeyPress:return t.which&&!o(t)?String.fromCharCode(t.which):null;case k.topCompositionEnd:return N?null:t.data;default:return null}}function d(e,t,n,r){var o;if(o=T?c(e,n):p(e,n),!o)return null;var a=y.getPooled(M.beforeInput,t,n,r);return a.data=o,h.accumulateTwoPhaseDispatches(a),a}var f=e(16),h=e(20),m=e(140),v=e(21),g=e(95),y=e(99),b=e(158),C=[9,13,27,32],_=229,E=m.canUseDOM&&"CompositionEvent"in window,x=null;m.canUseDOM&&"documentMode"in document&&(x=document.documentMode);var T=m.canUseDOM&&"TextEvent"in window&&!x&&!r(),N=m.canUseDOM&&(!E||x&&x>8&&x<=11),w=32,P=String.fromCharCode(w),k=f.topLevelTypes,M={beforeInput:{phasedRegistrationNames:{bubbled:b({onBeforeInput:null}),captured:b({onBeforeInputCapture:null})},dependencies:[k.topCompositionEnd,k.topKeyPress,k.topTextInput,k.topPaste]},compositionEnd:{phasedRegistrationNames:{bubbled:b({onCompositionEnd:null}),captured:b({onCompositionEndCapture:null})},dependencies:[k.topBlur,k.topCompositionEnd,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]},compositionStart:{phasedRegistrationNames:{bubbled:b({onCompositionStart:null}),captured:b({onCompositionStartCapture:null})},dependencies:[k.topBlur,k.topCompositionStart,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]},compositionUpdate:{phasedRegistrationNames:{bubbled:b({onCompositionUpdate:null}),captured:b({onCompositionUpdateCapture:null})},dependencies:[k.topBlur,k.topCompositionUpdate,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]}},S=!1,R=null,O={eventTypes:M,extractEvents:function(e,t,n,r){return[l(e,t,n,r),d(e,t,n,r)]}};t.exports=O},{140:140,158:158,16:16,20:20,21:21,95:95,99:99}],3:[function(e,t,n){"use strict";function r(e,t){return e+t.charAt(0).toUpperCase()+t.substring(1)}var o={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridRow:!0,gridColumn:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},a=["Webkit","ms","Moz","O"];Object.keys(o).forEach(function(e){a.forEach(function(t){o[r(t,e)]=o[e]})});var i={background:{backgroundAttachment:!0,backgroundColor:!0,backgroundImage:!0,backgroundPositionX:!0,backgroundPositionY:!0,backgroundRepeat:!0},backgroundPosition:{backgroundPositionX:!0,backgroundPositionY:!0},border:{borderWidth:!0,borderStyle:!0,borderColor:!0},borderBottom:{borderBottomWidth:!0,borderBottomStyle:!0,borderBottomColor:!0},borderLeft:{borderLeftWidth:!0,borderLeftStyle:!0,borderLeftColor:!0},borderRight:{borderRightWidth:!0,borderRightStyle:!0,borderRightColor:!0},borderTop:{borderTopWidth:!0,borderTopStyle:!0,borderTopColor:!0},font:{fontStyle:!0,fontVariant:!0,fontWeight:!0,fontSize:!0,lineHeight:!0,fontFamily:!0},outline:{outlineWidth:!0,outlineStyle:!0,outlineColor:!0}},s={isUnitlessNumber:o,shorthandPropertyExpansions:i};t.exports=s},{}],4:[function(e,t,n){"use strict";var r=e(3),o=e(140),a=(e(66),e(142),e(113)),i=e(153),s=e(159),u=(e(161),s(function(e){return i(e)})),l=!1,c="cssFloat";if(o.canUseDOM){var p=document.createElement("div").style;try{p.font=""}catch(e){l=!0}void 0===document.documentElement.style.cssFloat&&(c="styleFloat")}var d={createMarkupForStyles:function(e,t){var n="";for(var r in e)if(e.hasOwnProperty(r)){var o=e[r];null!=o&&(n+=u(r)+":",n+=a(r,o,t)+";")}return n||null},setValueForStyles:function(e,t,n){var o=e.style;for(var i in t)if(t.hasOwnProperty(i)){var s=a(i,t[i],n);if("float"!==i&&"cssFloat"!==i||(i=c),s)o[i]=s;else{var u=l&&r.shorthandPropertyExpansions[i];if(u)for(var p in u)o[p]="";else o[i]=""}}}};t.exports=d},{113:113,140:140,142:142,153:153,159:159,161:161,3:3,66:66}],5:[function(e,t,n){"use strict";function r(){this._callbacks=null,this._contexts=null}var o=e(132),a=e(162),i=e(25);e(154);a(r.prototype,{enqueue:function(e,t){this._callbacks=this._callbacks||[],this._contexts=this._contexts||[],this._callbacks.push(e),this._contexts.push(t)},notifyAll:function(){var e=this._callbacks,t=this._contexts;if(e){e.length!==t.length?o("24"):void 0,this._callbacks=null,this._contexts=null;for(var n=0;n<e.length;n++)e[n].call(t[n]);e.length=0,t.length=0}},checkpoint:function(){return this._callbacks?this._callbacks.length:0},rollback:function(e){this._callbacks&&(this._callbacks.length=e,this._contexts.length=e)},reset:function(){this._callbacks=null,this._contexts=null},destructor:function(){this.reset()}}),i.addPoolingTo(r),t.exports=r},{132:132,154:154,162:162,25:25}],6:[function(e,t,n){"use strict";function r(e){var t=e.nodeName&&e.nodeName.toLowerCase();return"select"===t||"input"===t&&"file"===e.type}function o(e){var t=T.getPooled(S.change,O,e,N(e));C.accumulateTwoPhaseDispatches(t),x.batchedUpdates(a,t)}function a(e){b.enqueueEvents(e),b.processEventQueue(!1)}function i(e,t){R=e,O=t,R.attachEvent("onchange",o)}function s(){R&&(R.detachEvent("onchange",o),R=null,O=null)}function u(e,t){if(e===M.topChange)return t}function l(e,t,n){e===M.topFocus?(s(),i(t,n)):e===M.topBlur&&s()}function c(e,t){R=e,O=t,I=e.value,D=Object.getOwnPropertyDescriptor(e.constructor.prototype,"value"),Object.defineProperty(R,"value",U),R.attachEvent?R.attachEvent("onpropertychange",d):R.addEventListener("propertychange",d,!1)}function p(){R&&(delete R.value,R.detachEvent?R.detachEvent("onpropertychange",d):R.removeEventListener("propertychange",d,!1),R=null,O=null,I=null,D=null)}function d(e){if("value"===e.propertyName){var t=e.srcElement.value;t!==I&&(I=t,o(e))}}function f(e,t){if(e===M.topInput)return t}function h(e,t,n){e===M.topFocus?(p(),c(t,n)):e===M.topBlur&&p()}function m(e,t){if((e===M.topSelectionChange||e===M.topKeyUp||e===M.topKeyDown)&&R&&R.value!==I)return I=R.value,O}function v(e){return e.nodeName&&"input"===e.nodeName.toLowerCase()&&("checkbox"===e.type||"radio"===e.type)}function g(e,t){if(e===M.topClick)return t}var y=e(16),b=e(17),C=e(20),_=e(140),E=e(40),x=e(88),T=e(97),N=e(121),w=e(128),P=e(129),k=e(158),M=y.topLevelTypes,S={change:{phasedRegistrationNames:{bubbled:k({onChange:null}),captured:k({onChangeCapture:null})},dependencies:[M.topBlur,M.topChange,M.topClick,M.topFocus,M.topInput,M.topKeyDown,M.topKeyUp,M.topSelectionChange]}},R=null,O=null,I=null,D=null,A=!1;_.canUseDOM&&(A=w("change")&&(!("documentMode"in document)||document.documentMode>8));var L=!1;_.canUseDOM&&(L=w("input")&&(!("documentMode"in document)||document.documentMode>11));var U={get:function(){return D.get.call(this)},set:function(e){I=""+e,D.set.call(this,e)}},F={eventTypes:S,extractEvents:function(e,t,n,o){var a,i,s=t?E.getNodeFromInstance(t):window;if(r(s)?A?a=u:i=l:P(s)?L?a=f:(a=m,i=h):v(s)&&(a=g),a){var c=a(e,t);if(c){var p=T.getPooled(S.change,c,n,o);return p.type="change",C.accumulateTwoPhaseDispatches(p),p}}i&&i(e,s,t)}};t.exports=F},{121:121,128:128,129:129,140:140,158:158,16:16,17:17,20:20,40:40,88:88,97:97}],7:[function(e,t,n){"use strict";function r(e,t){return Array.isArray(t)&&(t=t[1]),t?t.nextSibling:e.firstChild}function o(e,t,n){c.insertTreeBefore(e,t,n)}function a(e,t,n){Array.isArray(t)?s(e,t[0],t[1],n):v(e,t,n)}function i(e,t){if(Array.isArray(t)){var n=t[1];t=t[0],u(e,t,n),e.removeChild(n)}e.removeChild(t)}function s(e,t,n,r){for(var o=t;;){var a=o.nextSibling;if(v(e,o,r),o===n)break;o=a}}function u(e,t,n){for(;;){var r=t.nextSibling;if(r===n)break;e.removeChild(r)}}function l(e,t,n){var r=e.parentNode,o=e.nextSibling;o===t?n&&v(r,document.createTextNode(n),o):n?(m(o,n),u(r,o,t)):u(r,e,t)}var c=e(8),p=e(12),d=e(70),f=(e(40),e(66),e(112)),h=e(134),m=e(135),v=f(function(e,t,n){e.insertBefore(t,n)}),g=p.dangerouslyReplaceNodeWithMarkup,y={dangerouslyReplaceNodeWithMarkup:g,replaceDelimitedText:l,processUpdates:function(e,t){for(var n=0;n<t.length;n++){var s=t[n];switch(s.type){case d.INSERT_MARKUP:o(e,s.content,r(e,s.afterNode));break;case d.MOVE_EXISTING:a(e,s.fromNode,r(e,s.afterNode));break;case d.SET_MARKUP:h(e,s.content);break;case d.TEXT_CONTENT:m(e,s.content);break;case d.REMOVE_NODE:i(e,s.fromNode)}}}};t.exports=y},{112:112,12:12,134:134,135:135,40:40,66:66,70:70,8:8}],8:[function(e,t,n){"use strict";function r(e){if(v){var t=e.node,n=e.children;if(n.length)for(var r=0;r<n.length;r++)g(t,n[r],null);else null!=e.html?p(t,e.html):null!=e.text&&f(t,e.text)}}function o(e,t){e.parentNode.replaceChild(t.node,e),r(t)}function a(e,t){v?e.children.push(t):e.node.appendChild(t.node)}function i(e,t){v?e.html=t:p(e.node,t)}function s(e,t){v?e.text=t:f(e.node,t)}function u(){return this.node.nodeName}function l(e){return{node:e,children:[],html:null,text:null,toString:u}}var c=e(9),p=e(134),d=e(112),f=e(135),h=1,m=11,v="undefined"!=typeof document&&"number"==typeof document.documentMode||"undefined"!=typeof navigator&&"string"==typeof navigator.userAgent&&/\bEdge\/\d/.test(navigator.userAgent),g=d(function(e,t,n){t.node.nodeType===m||t.node.nodeType===h&&"object"===t.node.nodeName.toLowerCase()&&(null==t.node.namespaceURI||t.node.namespaceURI===c.html)?(r(t),e.insertBefore(t.node,n)):(e.insertBefore(t.node,n),r(t))});l.insertTreeBefore=g,l.replaceChildWithTree=o,l.queueChild=a,l.queueHTML=i,l.queueText=s,t.exports=l},{112:112,134:134,135:135,9:9}],9:[function(e,t,n){"use strict";var r={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};t.exports=r},{}],10:[function(e,t,n){"use strict";function r(e,t){return(e&t)===t}var o=e(132),a=(e(154),{MUST_USE_PROPERTY:1,HAS_BOOLEAN_VALUE:4,HAS_NUMERIC_VALUE:8,HAS_POSITIVE_NUMERIC_VALUE:24,HAS_OVERLOADED_BOOLEAN_VALUE:32,injectDOMPropertyConfig:function(e){var t=a,n=e.Properties||{},i=e.DOMAttributeNamespaces||{},u=e.DOMAttributeNames||{},l=e.DOMPropertyNames||{},c=e.DOMMutationMethods||{};e.isCustomAttribute&&s._isCustomAttributeFunctions.push(e.isCustomAttribute);for(var p in n){s.properties.hasOwnProperty(p)?o("48",p):void 0;var d=p.toLowerCase(),f=n[p],h={attributeName:d,attributeNamespace:null,propertyName:p,mutationMethod:null,mustUseProperty:r(f,t.MUST_USE_PROPERTY),hasBooleanValue:r(f,t.HAS_BOOLEAN_VALUE),hasNumericValue:r(f,t.HAS_NUMERIC_VALUE),hasPositiveNumericValue:r(f,t.HAS_POSITIVE_NUMERIC_VALUE),hasOverloadedBooleanValue:r(f,t.HAS_OVERLOADED_BOOLEAN_VALUE)};if(h.hasBooleanValue+h.hasNumericValue+h.hasOverloadedBooleanValue<=1?void 0:o("50",p),u.hasOwnProperty(p)){var m=u[p];h.attributeName=m}i.hasOwnProperty(p)&&(h.attributeNamespace=i[p]),l.hasOwnProperty(p)&&(h.propertyName=l[p]),c.hasOwnProperty(p)&&(h.mutationMethod=c[p]),s.properties[p]=h}}}),i=":A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD",s={ID_ATTRIBUTE_NAME:"data-reactid",ROOT_ATTRIBUTE_NAME:"data-reactroot",ATTRIBUTE_NAME_START_CHAR:i,ATTRIBUTE_NAME_CHAR:i+"\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040",properties:{},getPossibleStandardName:null,_isCustomAttributeFunctions:[],isCustomAttribute:function(e){for(var t=0;t<s._isCustomAttributeFunctions.length;t++){var n=s._isCustomAttributeFunctions[t];if(n(e))return!0}return!1},injection:a};t.exports=s},{132:132,154:154}],11:[function(e,t,n){"use strict";function r(e){return!!l.hasOwnProperty(e)||!u.hasOwnProperty(e)&&(s.test(e)?(l[e]=!0,!0):(u[e]=!0,!1))}function o(e,t){return null==t||e.hasBooleanValue&&!t||e.hasNumericValue&&isNaN(t)||e.hasPositiveNumericValue&&t<1||e.hasOverloadedBooleanValue&&t===!1}var a=e(10),i=(e(40),e(66),e(131)),s=(e(161),new RegExp("^["+a.ATTRIBUTE_NAME_START_CHAR+"]["+a.ATTRIBUTE_NAME_CHAR+"]*$")),u={},l={},c={createMarkupForID:function(e){return a.ID_ATTRIBUTE_NAME+"="+i(e)},setAttributeForID:function(e,t){e.setAttribute(a.ID_ATTRIBUTE_NAME,t)},createMarkupForRoot:function(){return a.ROOT_ATTRIBUTE_NAME+'=""'},setAttributeForRoot:function(e){e.setAttribute(a.ROOT_ATTRIBUTE_NAME,"")},createMarkupForProperty:function(e,t){var n=a.properties.hasOwnProperty(e)?a.properties[e]:null;if(n){if(o(n,t))return"";var r=n.attributeName;return n.hasBooleanValue||n.hasOverloadedBooleanValue&&t===!0?r+'=""':r+"="+i(t)}return a.isCustomAttribute(e)?null==t?"":e+"="+i(t):null},createMarkupForCustomAttribute:function(e,t){return r(e)&&null!=t?e+"="+i(t):""},setValueForProperty:function(e,t,n){var r=a.properties.hasOwnProperty(t)?a.properties[t]:null;if(r){var i=r.mutationMethod;if(i)i(e,n);else{if(o(r,n))return void this.deleteValueForProperty(e,t);if(r.mustUseProperty)e[r.propertyName]=n;else{var s=r.attributeName,u=r.attributeNamespace;u?e.setAttributeNS(u,s,""+n):r.hasBooleanValue||r.hasOverloadedBooleanValue&&n===!0?e.setAttribute(s,""):e.setAttribute(s,""+n)}}}else if(a.isCustomAttribute(t))return void c.setValueForAttribute(e,t,n)},setValueForAttribute:function(e,t,n){r(t)&&(null==n?e.removeAttribute(t):e.setAttribute(t,""+n))},deleteValueForAttribute:function(e,t){e.removeAttribute(t)},deleteValueForProperty:function(e,t){var n=a.properties.hasOwnProperty(t)?a.properties[t]:null;if(n){var r=n.mutationMethod;if(r)r(e,void 0);else if(n.mustUseProperty){var o=n.propertyName;n.hasBooleanValue?e[o]=!1:e[o]=""}else e.removeAttribute(n.attributeName)}else a.isCustomAttribute(t)&&e.removeAttribute(t)}};t.exports=c},{10:10,131:131,161:161,40:40,66:66}],12:[function(e,t,n){"use strict";var r=e(132),o=e(8),a=e(140),i=e(145),s=e(146),u=(e(154),{dangerouslyReplaceNodeWithMarkup:function(e,t){if(a.canUseDOM?void 0:r("56"),t?void 0:r("57"),"HTML"===e.nodeName?r("58"):void 0,"string"==typeof t){var n=i(t,s)[0];e.parentNode.replaceChild(n,e)}else o.replaceChildWithTree(e,t)}});t.exports=u},{132:132,140:140,145:145,146:146,154:154,8:8}],13:[function(e,t,n){"use strict";var r=e(158),o=[r({ResponderEventPlugin:null}),r({SimpleEventPlugin:null}),r({TapEventPlugin:null}),r({EnterLeaveEventPlugin:null}),r({ChangeEventPlugin:null}),r({SelectEventPlugin:null}),r({BeforeInputEventPlugin:null})];t.exports=o},{158:158}],14:[function(e,t,n){"use strict";var r={onClick:!0,onDoubleClick:!0,onMouseDown:!0,onMouseMove:!0,onMouseUp:!0,onClickCapture:!0,onDoubleClickCapture:!0,onMouseDownCapture:!0,onMouseMoveCapture:!0,onMouseUpCapture:!0},o={getHostProps:function(e,t){if(!t.disabled)return t;var n={};for(var o in t)!r[o]&&t.hasOwnProperty(o)&&(n[o]=t[o]);return n}};t.exports=o},{}],15:[function(e,t,n){"use strict";var r=e(16),o=e(20),a=e(40),i=e(101),s=e(158),u=r.topLevelTypes,l={mouseEnter:{registrationName:s({onMouseEnter:null}),dependencies:[u.topMouseOut,u.topMouseOver]},mouseLeave:{registrationName:s({onMouseLeave:null}),dependencies:[u.topMouseOut,u.topMouseOver]}},c={eventTypes:l,extractEvents:function(e,t,n,r){if(e===u.topMouseOver&&(n.relatedTarget||n.fromElement))return null;if(e!==u.topMouseOut&&e!==u.topMouseOver)return null;var s;if(r.window===r)s=r;else{var c=r.ownerDocument;s=c?c.defaultView||c.parentWindow:window}var p,d;if(e===u.topMouseOut){p=t;var f=n.relatedTarget||n.toElement;d=f?a.getClosestInstanceFromNode(f):null}else p=null,d=t;if(p===d)return null;var h=null==p?s:a.getNodeFromInstance(p),m=null==d?s:a.getNodeFromInstance(d),v=i.getPooled(l.mouseLeave,p,n,r);v.type="mouseleave",v.target=h,v.relatedTarget=m;var g=i.getPooled(l.mouseEnter,d,n,r);return g.type="mouseenter",g.target=m,g.relatedTarget=h,o.accumulateEnterLeaveDispatches(v,g,p,d),[v,g]}};t.exports=c},{101:101,158:158,16:16,20:20,40:40}],16:[function(e,t,n){"use strict";var r=e(157),o=r({bubbled:null,captured:null}),a=r({topAbort:null,topAnimationEnd:null,topAnimationIteration:null,topAnimationStart:null,topBlur:null,topCanPlay:null,topCanPlayThrough:null,topChange:null,topClick:null,topCompositionEnd:null,topCompositionStart:null,topCompositionUpdate:null,topContextMenu:null,topCopy:null,topCut:null,topDoubleClick:null,topDrag:null,topDragEnd:null,topDragEnter:null,topDragExit:null,topDragLeave:null,topDragOver:null,topDragStart:null,topDrop:null,topDurationChange:null,topEmptied:null,topEncrypted:null,topEnded:null,topError:null,topFocus:null,topInput:null,topInvalid:null,topKeyDown:null,topKeyPress:null,topKeyUp:null,topLoad:null,topLoadedData:null,topLoadedMetadata:null,topLoadStart:null,topMouseDown:null,topMouseMove:null,topMouseOut:null,topMouseOver:null,topMouseUp:null,topPaste:null,topPause:null,topPlay:null,topPlaying:null,topProgress:null,topRateChange:null,topReset:null,topScroll:null,topSeeked:null,topSeeking:null,topSelectionChange:null,topStalled:null,topSubmit:null,topSuspend:null,topTextInput:null,topTimeUpdate:null,topTouchCancel:null,topTouchEnd:null,topTouchMove:null,topTouchStart:null,topTransitionEnd:null,topVolumeChange:null,topWaiting:null,topWheel:null}),i={topLevelTypes:a,PropagationPhases:o};t.exports=i},{157:157}],17:[function(e,t,n){"use strict";var r=e(132),o=e(18),a=e(19),i=e(58),s=e(108),u=e(117),l=(e(154),{}),c=null,p=function(e,t){e&&(a.executeDispatchesInOrder(e,t),e.isPersistent()||e.constructor.release(e))},d=function(e){return p(e,!0)},f=function(e){return p(e,!1)},h=function(e){return"."+e._rootNodeID},m={injection:{injectEventPluginOrder:o.injectEventPluginOrder,injectEventPluginsByName:o.injectEventPluginsByName},putListener:function(e,t,n){"function"!=typeof n?r("94",t,typeof n):void 0;var a=h(e),i=l[t]||(l[t]={});i[a]=n;var s=o.registrationNameModules[t];s&&s.didPutListener&&s.didPutListener(e,t,n)},getListener:function(e,t){var n=l[t],r=h(e);return n&&n[r]},deleteListener:function(e,t){var n=o.registrationNameModules[t];n&&n.willDeleteListener&&n.willDeleteListener(e,t);var r=l[t];if(r){var a=h(e);delete r[a]}},deleteAllListeners:function(e){var t=h(e);for(var n in l)if(l.hasOwnProperty(n)&&l[n][t]){var r=o.registrationNameModules[n];r&&r.willDeleteListener&&r.willDeleteListener(e,n),delete l[n][t]}},extractEvents:function(e,t,n,r){for(var a,i=o.plugins,u=0;u<i.length;u++){var l=i[u];if(l){var c=l.extractEvents(e,t,n,r);c&&(a=s(a,c))}}return a},enqueueEvents:function(e){e&&(c=s(c,e))},processEventQueue:function(e){var t=c;c=null,e?u(t,d):u(t,f),c?r("95"):void 0,i.rethrowCaughtError()},__purge:function(){l={}},__getListenerBank:function(){return l}};t.exports=m},{108:108,117:117,132:132,154:154,18:18,19:19,58:58}],18:[function(e,t,n){"use strict";function r(){if(s)for(var e in u){var t=u[e],n=s.indexOf(e);if(n>-1?void 0:i("96",e),!l.plugins[n]){t.extractEvents?void 0:i("97",e),l.plugins[n]=t;var r=t.eventTypes;for(var a in r)o(r[a],t,a)?void 0:i("98",a,e)}}}function o(e,t,n){l.eventNameDispatchConfigs.hasOwnProperty(n)?i("99",n):void 0,l.eventNameDispatchConfigs[n]=e;var r=e.phasedRegistrationNames;if(r){for(var o in r)if(r.hasOwnProperty(o)){var s=r[o];a(s,t,n)}return!0}return!!e.registrationName&&(a(e.registrationName,t,n),!0)}function a(e,t,n){l.registrationNameModules[e]?i("100",e):void 0,l.registrationNameModules[e]=t,l.registrationNameDependencies[e]=t.eventTypes[n].dependencies}var i=e(132),s=(e(154),null),u={},l={plugins:[],eventNameDispatchConfigs:{},registrationNameModules:{},registrationNameDependencies:{},possibleRegistrationNames:null,injectEventPluginOrder:function(e){s?i("101"):void 0,s=Array.prototype.slice.call(e),r()},injectEventPluginsByName:function(e){var t=!1;for(var n in e)if(e.hasOwnProperty(n)){var o=e[n];u.hasOwnProperty(n)&&u[n]===o||(u[n]?i("102",n):void 0,u[n]=o,t=!0)}t&&r()},getPluginModuleForEvent:function(e){var t=e.dispatchConfig;if(t.registrationName)return l.registrationNameModules[t.registrationName]||null;for(var n in t.phasedRegistrationNames)if(t.phasedRegistrationNames.hasOwnProperty(n)){var r=l.registrationNameModules[t.phasedRegistrationNames[n]];if(r)return r}return null},_resetEventPlugins:function(){s=null;for(var e in u)u.hasOwnProperty(e)&&delete u[e];l.plugins.length=0;var t=l.eventNameDispatchConfigs;for(var n in t)t.hasOwnProperty(n)&&delete t[n];var r=l.registrationNameModules;for(var o in r)r.hasOwnProperty(o)&&delete r[o]}};t.exports=l},{132:132,154:154}],19:[function(e,t,n){"use strict";function r(e){return e===y.topMouseUp||e===y.topTouchEnd||e===y.topTouchCancel}function o(e){return e===y.topMouseMove||e===y.topTouchMove}function a(e){return e===y.topMouseDown||e===y.topTouchStart}function i(e,t,n,r){var o=e.type||"unknown-event";e.currentTarget=b.getNodeFromInstance(r),t?v.invokeGuardedCallbackWithCatch(o,n,e):v.invokeGuardedCallback(o,n,e),e.currentTarget=null}function s(e,t){var n=e._dispatchListeners,r=e._dispatchInstances;if(Array.isArray(n))for(var o=0;o<n.length&&!e.isPropagationStopped();o++)i(e,t,n[o],r[o]);else n&&i(e,t,n,r);e._dispatchListeners=null,e._dispatchInstances=null}function u(e){var t=e._dispatchListeners,n=e._dispatchInstances;if(Array.isArray(t)){for(var r=0;r<t.length&&!e.isPropagationStopped();r++)if(t[r](e,n[r]))return n[r]}else if(t&&t(e,n))return n;return null}function l(e){var t=u(e);return e._dispatchInstances=null,e._dispatchListeners=null,t}function c(e){var t=e._dispatchListeners,n=e._dispatchInstances;Array.isArray(t)?h("103"):void 0,e.currentTarget=t?b.getNodeFromInstance(n):null;var r=t?t(e):null;return e.currentTarget=null,e._dispatchListeners=null,e._dispatchInstances=null,r}function p(e){return!!e._dispatchListeners}var d,f,h=e(132),m=e(16),v=e(58),g=(e(154),e(161),{injectComponentTree:function(e){d=e},injectTreeTraversal:function(e){f=e}}),y=m.topLevelTypes,b={isEndish:r,isMoveish:o,isStartish:a,executeDirectDispatch:c,executeDispatchesInOrder:s,executeDispatchesInOrderStopAtTrue:l,hasDispatches:p,getInstanceFromNode:function(e){return d.getInstanceFromNode(e)},getNodeFromInstance:function(e){return d.getNodeFromInstance(e)},isAncestor:function(e,t){return f.isAncestor(e,t)},getLowestCommonAncestor:function(e,t){return f.getLowestCommonAncestor(e,t)},getParentInstance:function(e){return f.getParentInstance(e)},traverseTwoPhase:function(e,t,n){return f.traverseTwoPhase(e,t,n)},traverseEnterLeave:function(e,t,n,r,o){return f.traverseEnterLeave(e,t,n,r,o)},injection:g};t.exports=b},{132:132,154:154,16:16,161:161,58:58}],20:[function(e,t,n){"use strict";function r(e,t,n){var r=t.dispatchConfig.phasedRegistrationNames[n];return b(e,r)}function o(e,t,n){var o=t?y.bubbled:y.captured,a=r(e,n,o);a&&(n._dispatchListeners=v(n._dispatchListeners,a),n._dispatchInstances=v(n._dispatchInstances,e))}function a(e){e&&e.dispatchConfig.phasedRegistrationNames&&m.traverseTwoPhase(e._targetInst,o,e)}function i(e){if(e&&e.dispatchConfig.phasedRegistrationNames){var t=e._targetInst,n=t?m.getParentInstance(t):null;m.traverseTwoPhase(n,o,e)}}function s(e,t,n){if(n&&n.dispatchConfig.registrationName){var r=n.dispatchConfig.registrationName,o=b(e,r);o&&(n._dispatchListeners=v(n._dispatchListeners,o),n._dispatchInstances=v(n._dispatchInstances,e))}}function u(e){e&&e.dispatchConfig.registrationName&&s(e._targetInst,null,e)}function l(e){g(e,a)}function c(e){g(e,i)}function p(e,t,n,r){m.traverseEnterLeave(n,r,s,e,t)}function d(e){g(e,u)}var f=e(16),h=e(17),m=e(19),v=e(108),g=e(117),y=(e(161),f.PropagationPhases),b=h.getListener,C={accumulateTwoPhaseDispatches:l,accumulateTwoPhaseDispatchesSkipTarget:c,accumulateDirectDispatches:d,accumulateEnterLeaveDispatches:p};t.exports=C},{108:108,117:117,16:16,161:161,17:17,19:19}],21:[function(e,t,n){"use strict";function r(e){this._root=e,this._startText=this.getText(),this._fallbackText=null}var o=e(162),a=e(25),i=e(125);o(r.prototype,{destructor:function(){this._root=null,this._startText=null,this._fallbackText=null},getText:function(){return"value"in this._root?this._root.value:this._root[i()]},getData:function(){if(this._fallbackText)return this._fallbackText;var e,t,n=this._startText,r=n.length,o=this.getText(),a=o.length;for(e=0;e<r&&n[e]===o[e];e++);var i=r-e;for(t=1;t<=i&&n[r-t]===o[a-t];t++);var s=t>1?1-t:void 0;return this._fallbackText=o.slice(e,s),this._fallbackText}}),a.addPoolingTo(r),t.exports=r},{125:125,162:162,25:25}],22:[function(e,t,n){"use strict";var r=e(10),o=r.injection.MUST_USE_PROPERTY,a=r.injection.HAS_BOOLEAN_VALUE,i=r.injection.HAS_NUMERIC_VALUE,s=r.injection.HAS_POSITIVE_NUMERIC_VALUE,u=r.injection.HAS_OVERLOADED_BOOLEAN_VALUE,l={isCustomAttribute:RegExp.prototype.test.bind(new RegExp("^(data|aria)-["+r.ATTRIBUTE_NAME_CHAR+"]*$")),Properties:{accept:0,acceptCharset:0,accessKey:0,action:0,allowFullScreen:a,allowTransparency:0,alt:0,async:a,autoComplete:0,autoPlay:a,capture:a,cellPadding:0,cellSpacing:0,charSet:0,challenge:0,checked:o|a,cite:0,classID:0,className:0,cols:s,colSpan:0,content:0,contentEditable:0,contextMenu:0,controls:a,coords:0,crossOrigin:0,data:0,dateTime:0,default:a,defer:a,dir:0,disabled:a,download:u,draggable:0,encType:0,form:0,formAction:0,formEncType:0,formMethod:0,formNoValidate:a,formTarget:0,frameBorder:0,headers:0,height:0,hidden:a,high:0,href:0,hrefLang:0,htmlFor:0,httpEquiv:0,icon:0,id:0,inputMode:0,integrity:0,is:0,keyParams:0,keyType:0,kind:0,label:0,lang:0,list:0,loop:a,low:0,manifest:0,marginHeight:0,marginWidth:0,max:0,maxLength:0,media:0,mediaGroup:0,method:0,min:0,minLength:0,multiple:o|a,muted:o|a,name:0,nonce:0,noValidate:a,open:a,optimum:0,pattern:0,placeholder:0,poster:0,preload:0,profile:0,radioGroup:0,readOnly:a,referrerPolicy:0,rel:0,required:a,reversed:a,role:0,rows:s,rowSpan:i,sandbox:0,scope:0,scoped:a,scrolling:0,seamless:a,selected:o|a,shape:0,size:s,sizes:0,span:s,spellCheck:0,src:0,srcDoc:0,srcLang:0,srcSet:0,start:i,step:0,style:0,summary:0,tabIndex:0,target:0,title:0,type:0,useMap:0,value:0,width:0,wmode:0,wrap:0,about:0,datatype:0,inlist:0,prefix:0,property:0,resource:0,typeof:0,vocab:0,autoCapitalize:0,autoCorrect:0,autoSave:0,color:0,itemProp:0,itemScope:a,itemType:0,itemID:0,itemRef:0,results:0,security:0,unselectable:0},DOMAttributeNames:{acceptCharset:"accept-charset",className:"class",htmlFor:"for",httpEquiv:"http-equiv"},DOMPropertyNames:{}};t.exports=l},{10:10}],23:[function(e,t,n){"use strict";function r(e){var t=/[=:]/g,n={"=":"=0",":":"=2"},r=(""+e).replace(t,function(e){return n[e]});return"$"+r}function o(e){var t=/(=0|=2)/g,n={"=0":"=","=2":":"},r="."===e[0]&&"$"===e[1]?e.substring(2):e.substring(1);return(""+r).replace(t,function(e){return n[e]})}var a={escape:r,unescape:o};t.exports=a},{}],24:[function(e,t,n){"use strict";function r(e){null!=e.checkedLink&&null!=e.valueLink?s("87"):void 0}function o(e){r(e),null!=e.value||null!=e.onChange?s("88"):void 0}function a(e){r(e),null!=e.checked||null!=e.onChange?s("89"):void 0}function i(e){if(e){var t=e.getName();if(t)return" Check the render method of `"+t+"`."}return""}var s=e(132),u=e(76),l=e(75),c=e(77),p=(e(154),e(161),{button:!0,checkbox:!0,image:!0,hidden:!0,radio:!0,reset:!0,submit:!0}),d={value:function(e,t,n){return!e[t]||p[e.type]||e.onChange||e.readOnly||e.disabled?null:new Error("You provided a `value` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultValue`. Otherwise, set either `onChange` or `readOnly`.")},checked:function(e,t,n){return!e[t]||e.onChange||e.readOnly||e.disabled?null:new Error("You provided a `checked` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultChecked`. Otherwise, set either `onChange` or `readOnly`.")},onChange:u.func},f={},h={checkPropTypes:function(e,t,n){for(var r in d){if(d.hasOwnProperty(r))var o=d[r](t,r,e,l.prop,null,c);o instanceof Error&&!(o.message in f)&&(f[o.message]=!0,i(n))}},getValue:function(e){return e.valueLink?(o(e),e.valueLink.value):e.value},getChecked:function(e){return e.checkedLink?(a(e),e.checkedLink.value):e.checked},executeOnChange:function(e,t){return e.valueLink?(o(e),e.valueLink.requestChange(t.target.value)):e.checkedLink?(a(e),e.checkedLink.requestChange(t.target.checked)):e.onChange?e.onChange.call(void 0,t):void 0}};t.exports=h},{132:132,154:154,161:161,75:75,76:76,77:77}],25:[function(e,t,n){"use strict";var r=e(132),o=(e(154),function(e){var t=this;if(t.instancePool.length){var n=t.instancePool.pop();return t.call(n,e),n}return new t(e)}),a=function(e,t){var n=this;if(n.instancePool.length){var r=n.instancePool.pop();return n.call(r,e,t),r}return new n(e,t)},i=function(e,t,n){var r=this;if(r.instancePool.length){var o=r.instancePool.pop();return r.call(o,e,t,n),o}return new r(e,t,n)},s=function(e,t,n,r){var o=this;if(o.instancePool.length){var a=o.instancePool.pop();return o.call(a,e,t,n,r),a}return new o(e,t,n,r)},u=function(e,t,n,r,o){var a=this;if(a.instancePool.length){var i=a.instancePool.pop();return a.call(i,e,t,n,r,o),i}return new a(e,t,n,r,o)},l=function(e){var t=this;e instanceof t?void 0:r("25"),e.destructor(),t.instancePool.length<t.poolSize&&t.instancePool.push(e)},c=10,p=o,d=function(e,t){var n=e;return n.instancePool=[],n.getPooled=t||p,n.poolSize||(n.poolSize=c),n.release=l,n},f={addPoolingTo:d,oneArgumentPooler:o,twoArgumentPooler:a,threeArgumentPooler:i,fourArgumentPooler:s,fiveArgumentPooler:u};t.exports=f},{132:132,154:154}],26:[function(e,t,n){"use strict";var r=e(162),o=e(29),a=e(31),i=e(78),s=e(30),u=e(43),l=e(56),c=e(76),p=e(89),d=e(130),f=(e(161),l.createElement),h=l.createFactory,m=l.cloneElement,v=r,g={Children:{map:o.map,forEach:o.forEach,count:o.count,toArray:o.toArray,only:d},Component:a,PureComponent:i,createElement:f,cloneElement:m,isValidElement:l.isValidElement,PropTypes:c,createClass:s.createClass,createFactory:h,createMixin:function(e){return e},DOM:u,version:p,__spread:v};t.exports=g},{130:130,161:161,162:162,29:29,30:30,31:31,43:43,56:56,76:76,78:78,89:89}],27:[function(e,t,n){"use strict";function r(e){return Object.prototype.hasOwnProperty.call(e,v)||(e[v]=h++,d[e[v]]={}),d[e[v]]}var o,a=e(162),i=e(16),s=e(18),u=e(59),l=e(107),c=e(126),p=e(128),d={},f=!1,h=0,m={
topAbort:"abort",topAnimationEnd:c("animationend")||"animationend",topAnimationIteration:c("animationiteration")||"animationiteration",topAnimationStart:c("animationstart")||"animationstart",topBlur:"blur",topCanPlay:"canplay",topCanPlayThrough:"canplaythrough",topChange:"change",topClick:"click",topCompositionEnd:"compositionend",topCompositionStart:"compositionstart",topCompositionUpdate:"compositionupdate",topContextMenu:"contextmenu",topCopy:"copy",topCut:"cut",topDoubleClick:"dblclick",topDrag:"drag",topDragEnd:"dragend",topDragEnter:"dragenter",topDragExit:"dragexit",topDragLeave:"dragleave",topDragOver:"dragover",topDragStart:"dragstart",topDrop:"drop",topDurationChange:"durationchange",topEmptied:"emptied",topEncrypted:"encrypted",topEnded:"ended",topError:"error",topFocus:"focus",topInput:"input",topKeyDown:"keydown",topKeyPress:"keypress",topKeyUp:"keyup",topLoadedData:"loadeddata",topLoadedMetadata:"loadedmetadata",topLoadStart:"loadstart",topMouseDown:"mousedown",topMouseMove:"mousemove",topMouseOut:"mouseout",topMouseOver:"mouseover",topMouseUp:"mouseup",topPaste:"paste",topPause:"pause",topPlay:"play",topPlaying:"playing",topProgress:"progress",topRateChange:"ratechange",topScroll:"scroll",topSeeked:"seeked",topSeeking:"seeking",topSelectionChange:"selectionchange",topStalled:"stalled",topSuspend:"suspend",topTextInput:"textInput",topTimeUpdate:"timeupdate",topTouchCancel:"touchcancel",topTouchEnd:"touchend",topTouchMove:"touchmove",topTouchStart:"touchstart",topTransitionEnd:c("transitionend")||"transitionend",topVolumeChange:"volumechange",topWaiting:"waiting",topWheel:"wheel"},v="_reactListenersID"+String(Math.random()).slice(2),g=a({},u,{ReactEventListener:null,injection:{injectReactEventListener:function(e){e.setHandleTopLevel(g.handleTopLevel),g.ReactEventListener=e}},setEnabled:function(e){g.ReactEventListener&&g.ReactEventListener.setEnabled(e)},isEnabled:function(){return!(!g.ReactEventListener||!g.ReactEventListener.isEnabled())},listenTo:function(e,t){for(var n=t,o=r(n),a=s.registrationNameDependencies[e],u=i.topLevelTypes,l=0;l<a.length;l++){var c=a[l];o.hasOwnProperty(c)&&o[c]||(c===u.topWheel?p("wheel")?g.ReactEventListener.trapBubbledEvent(u.topWheel,"wheel",n):p("mousewheel")?g.ReactEventListener.trapBubbledEvent(u.topWheel,"mousewheel",n):g.ReactEventListener.trapBubbledEvent(u.topWheel,"DOMMouseScroll",n):c===u.topScroll?p("scroll",!0)?g.ReactEventListener.trapCapturedEvent(u.topScroll,"scroll",n):g.ReactEventListener.trapBubbledEvent(u.topScroll,"scroll",g.ReactEventListener.WINDOW_HANDLE):c===u.topFocus||c===u.topBlur?(p("focus",!0)?(g.ReactEventListener.trapCapturedEvent(u.topFocus,"focus",n),g.ReactEventListener.trapCapturedEvent(u.topBlur,"blur",n)):p("focusin")&&(g.ReactEventListener.trapBubbledEvent(u.topFocus,"focusin",n),g.ReactEventListener.trapBubbledEvent(u.topBlur,"focusout",n)),o[u.topBlur]=!0,o[u.topFocus]=!0):m.hasOwnProperty(c)&&g.ReactEventListener.trapBubbledEvent(c,m[c],n),o[c]=!0)}},trapBubbledEvent:function(e,t,n){return g.ReactEventListener.trapBubbledEvent(e,t,n)},trapCapturedEvent:function(e,t,n){return g.ReactEventListener.trapCapturedEvent(e,t,n)},ensureScrollValueMonitoring:function(){if(void 0===o&&(o=document.createEvent&&"pageX"in document.createEvent("MouseEvent")),!o&&!f){var e=l.refreshScrollValues;g.ReactEventListener.monitorScrollValue(e),f=!0}}});t.exports=g},{107:107,126:126,128:128,16:16,162:162,18:18,59:59}],28:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r){var o=void 0===e[n];null!=t&&o&&(e[n]=a(t,!0))}var o=e(80),a=e(127),i=(e(23),e(136)),s=e(137);e(161);"undefined"!=typeof n&&n.env,1;var u={instantiateChildren:function(e,t,n,o){if(null==e)return null;var a={};return s(e,r,a),a},updateChildren:function(e,t,n,r,s,u,l,c,p){if(t||e){var d,f;for(d in t)if(t.hasOwnProperty(d)){f=e&&e[d];var h=f&&f._currentElement,m=t[d];if(null!=f&&i(h,m))o.receiveComponent(f,m,s,c),t[d]=f;else{f&&(r[d]=o.getHostNode(f),o.unmountComponent(f,!1));var v=a(m,!0);t[d]=v;var g=o.mountComponent(v,s,u,l,c,p);n.push(g)}}for(d in e)!e.hasOwnProperty(d)||t&&t.hasOwnProperty(d)||(f=e[d],r[d]=o.getHostNode(f),o.unmountComponent(f,!1))}},unmountChildren:function(e,t){for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];o.unmountComponent(r,t)}}};t.exports=u}).call(this,void 0)},{127:127,136:136,137:137,161:161,23:23,80:80}],29:[function(e,t,n){"use strict";function r(e){return(""+e).replace(C,"$&/")}function o(e,t){this.func=e,this.context=t,this.count=0}function a(e,t,n){var r=e.func,o=e.context;r.call(o,t,e.count++)}function i(e,t,n){if(null==e)return e;var r=o.getPooled(t,n);g(e,a,r),o.release(r)}function s(e,t,n,r){this.result=e,this.keyPrefix=t,this.func=n,this.context=r,this.count=0}function u(e,t,n){var o=e.result,a=e.keyPrefix,i=e.func,s=e.context,u=i.call(s,t,e.count++);Array.isArray(u)?l(u,o,n,v.thatReturnsArgument):null!=u&&(m.isValidElement(u)&&(u=m.cloneAndReplaceKey(u,a+(!u.key||t&&t.key===u.key?"":r(u.key)+"/")+n)),o.push(u))}function l(e,t,n,o,a){var i="";null!=n&&(i=r(n)+"/");var l=s.getPooled(t,i,o,a);g(e,u,l),s.release(l)}function c(e,t,n){if(null==e)return e;var r=[];return l(e,r,null,t,n),r}function p(e,t,n){return null}function d(e,t){return g(e,p,null)}function f(e){var t=[];return l(e,t,null,v.thatReturnsArgument),t}var h=e(25),m=e(56),v=e(146),g=e(137),y=h.twoArgumentPooler,b=h.fourArgumentPooler,C=/\/+/g;o.prototype.destructor=function(){this.func=null,this.context=null,this.count=0},h.addPoolingTo(o,y),s.prototype.destructor=function(){this.result=null,this.keyPrefix=null,this.func=null,this.context=null,this.count=0},h.addPoolingTo(s,b);var _={forEach:i,map:c,mapIntoWithKeyPrefixInternal:l,count:d,toArray:f};t.exports=_},{137:137,146:146,25:25,56:56}],30:[function(e,t,n){"use strict";function r(e,t){var n=E.hasOwnProperty(t)?E[t]:null;T.hasOwnProperty(t)&&(n!==C.OVERRIDE_BASE?p("73",t):void 0),e&&(n!==C.DEFINE_MANY&&n!==C.DEFINE_MANY_MERGED?p("74",t):void 0)}function o(e,t){if(t){"function"==typeof t?p("75"):void 0,h.isValidElement(t)?p("76"):void 0;var n=e.prototype,o=n.__reactAutoBindPairs;t.hasOwnProperty(b)&&x.mixins(e,t.mixins);for(var a in t)if(t.hasOwnProperty(a)&&a!==b){var i=t[a],l=n.hasOwnProperty(a);if(r(l,a),x.hasOwnProperty(a))x[a](e,i);else{var c=E.hasOwnProperty(a),d="function"==typeof i,f=d&&!c&&!l&&t.autobind!==!1;if(f)o.push(a,i),n[a]=i;else if(l){var m=E[a];!c||m!==C.DEFINE_MANY_MERGED&&m!==C.DEFINE_MANY?p("77",m,a):void 0,m===C.DEFINE_MANY_MERGED?n[a]=s(n[a],i):m===C.DEFINE_MANY&&(n[a]=u(n[a],i))}else n[a]=i}}}}function a(e,t){if(t)for(var n in t){var r=t[n];if(t.hasOwnProperty(n)){var o=n in x;o?p("78",n):void 0;var a=n in e;a?p("79",n):void 0,e[n]=r}}}function i(e,t){e&&t&&"object"==typeof e&&"object"==typeof t?void 0:p("80");for(var n in t)t.hasOwnProperty(n)&&(void 0!==e[n]?p("81",n):void 0,e[n]=t[n]);return e}function s(e,t){return function(){var n=e.apply(this,arguments),r=t.apply(this,arguments);if(null==n)return r;if(null==r)return n;var o={};return i(o,n),i(o,r),o}}function u(e,t){return function(){e.apply(this,arguments),t.apply(this,arguments)}}function l(e,t){var n=t.bind(e);return n}function c(e){for(var t=e.__reactAutoBindPairs,n=0;n<t.length;n+=2){var r=t[n],o=t[n+1];e[r]=l(e,o)}}var p=e(132),d=e(162),f=e(31),h=e(56),m=(e(75),e(74),e(72)),v=e(147),g=(e(154),e(157)),y=e(158),b=(e(161),y({mixins:null})),C=g({DEFINE_ONCE:null,DEFINE_MANY:null,OVERRIDE_BASE:null,DEFINE_MANY_MERGED:null}),_=[],E={mixins:C.DEFINE_MANY,statics:C.DEFINE_MANY,propTypes:C.DEFINE_MANY,contextTypes:C.DEFINE_MANY,childContextTypes:C.DEFINE_MANY,getDefaultProps:C.DEFINE_MANY_MERGED,getInitialState:C.DEFINE_MANY_MERGED,getChildContext:C.DEFINE_MANY_MERGED,render:C.DEFINE_ONCE,componentWillMount:C.DEFINE_MANY,componentDidMount:C.DEFINE_MANY,componentWillReceiveProps:C.DEFINE_MANY,shouldComponentUpdate:C.DEFINE_ONCE,componentWillUpdate:C.DEFINE_MANY,componentDidUpdate:C.DEFINE_MANY,componentWillUnmount:C.DEFINE_MANY,updateComponent:C.OVERRIDE_BASE},x={displayName:function(e,t){e.displayName=t},mixins:function(e,t){if(t)for(var n=0;n<t.length;n++)o(e,t[n])},childContextTypes:function(e,t){e.childContextTypes=d({},e.childContextTypes,t)},contextTypes:function(e,t){e.contextTypes=d({},e.contextTypes,t)},getDefaultProps:function(e,t){e.getDefaultProps?e.getDefaultProps=s(e.getDefaultProps,t):e.getDefaultProps=t},propTypes:function(e,t){e.propTypes=d({},e.propTypes,t)},statics:function(e,t){a(e,t)},autobind:function(){}},T={replaceState:function(e,t){this.updater.enqueueReplaceState(this,e),t&&this.updater.enqueueCallback(this,t,"replaceState")},isMounted:function(){return this.updater.isMounted(this)}},N=function(){};d(N.prototype,f.prototype,T);var w={createClass:function(e){var t=function(e,n,r){this.__reactAutoBindPairs.length&&c(this),this.props=e,this.context=n,this.refs=v,this.updater=r||m,this.state=null;var o=this.getInitialState?this.getInitialState():null;"object"!=typeof o||Array.isArray(o)?p("82",t.displayName||"ReactCompositeComponent"):void 0,this.state=o};t.prototype=new N,t.prototype.constructor=t,t.prototype.__reactAutoBindPairs=[],_.forEach(o.bind(null,t)),o(t,e),t.getDefaultProps&&(t.defaultProps=t.getDefaultProps()),t.prototype.render?void 0:p("83");for(var n in E)t.prototype[n]||(t.prototype[n]=null);return t},injection:{injectMixin:function(e){_.push(e)}}};t.exports=w},{132:132,147:147,154:154,157:157,158:158,161:161,162:162,31:31,56:56,72:72,74:74,75:75}],31:[function(e,t,n){"use strict";function r(e,t,n){this.props=e,this.context=t,this.refs=i,this.updater=n||a}var o=e(132),a=e(72),i=(e(110),e(147));e(154),e(161);r.prototype.isReactComponent={},r.prototype.setState=function(e,t){"object"!=typeof e&&"function"!=typeof e&&null!=e?o("85"):void 0,this.updater.enqueueSetState(this,e),t&&this.updater.enqueueCallback(this,t,"setState")},r.prototype.forceUpdate=function(e){this.updater.enqueueForceUpdate(this),e&&this.updater.enqueueCallback(this,e,"forceUpdate")};t.exports=r},{110:110,132:132,147:147,154:154,161:161,72:72}],32:[function(e,t,n){"use strict";var r=e(7),o=e(45),a={processChildrenUpdates:o.dangerouslyProcessChildrenUpdates,replaceNodeWithMarkup:r.dangerouslyReplaceNodeWithMarkup};t.exports=a},{45:45,7:7}],33:[function(e,t,n){"use strict";var r=e(132),o=(e(154),!1),a={replaceNodeWithMarkup:null,processChildrenUpdates:null,injection:{injectEnvironment:function(e){o?r("104"):void 0,a.replaceNodeWithMarkup=e.replaceNodeWithMarkup,a.processChildrenUpdates=e.processChildrenUpdates,o=!0}}};t.exports=a},{132:132,154:154}],34:[function(e,t,n){"use strict";function r(e){}function o(e,t){}function a(e){return!(!e.prototype||!e.prototype.isReactComponent)}function i(e){return!(!e.prototype||!e.prototype.isPureReactComponent)}var s=e(132),u=e(162),l=e(33),c=e(35),p=e(56),d=e(58),f=e(65),h=(e(66),e(71)),m=(e(75),e(80)),v=e(111),g=e(147),y=(e(154),e(160)),b=e(136),C=(e(161),{ImpureClass:0,PureClass:1,StatelessFunctional:2});r.prototype.render=function(){var e=f.get(this)._currentElement.type,t=e(this.props,this.context,this.updater);return o(e,t),t};var _=1,E={construct:function(e){this._currentElement=e,this._rootNodeID=0,this._compositeType=null,this._instance=null,this._hostParent=null,this._hostContainerInfo=null,this._updateBatchNumber=null,this._pendingElement=null,this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1,this._renderedNodeType=null,this._renderedComponent=null,this._context=null,this._mountOrder=0,this._topLevelWrapper=null,this._pendingCallbacks=null,this._calledComponentWillUnmount=!1},mountComponent:function(e,t,n,u){this._context=u,this._mountOrder=_++,this._hostParent=t,this._hostContainerInfo=n;var l,c=this._currentElement.props,d=this._processContext(u),h=this._currentElement.type,m=e.getUpdateQueue(),v=a(h),y=this._constructComponent(v,c,d,m);v||null!=y&&null!=y.render?i(h)?this._compositeType=C.PureClass:this._compositeType=C.ImpureClass:(l=y,o(h,l),null===y||y===!1||p.isValidElement(y)?void 0:s("105",h.displayName||h.name||"Component"),y=new r(h),this._compositeType=C.StatelessFunctional),y.props=c,y.context=d,y.refs=g,y.updater=m,this._instance=y,f.set(y,this);var b=y.state;void 0===b&&(y.state=b=null),"object"!=typeof b||Array.isArray(b)?s("106",this.getName()||"ReactCompositeComponent"):void 0,this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1;var E;return E=y.unstable_handleError?this.performInitialMountWithErrorHandling(l,t,n,e,u):this.performInitialMount(l,t,n,e,u),y.componentDidMount&&e.getReactMountReady().enqueue(y.componentDidMount,y),E},_constructComponent:function(e,t,n,r){return this._constructComponentWithoutOwner(e,t,n,r)},_constructComponentWithoutOwner:function(e,t,n,r){var o,a=this._currentElement.type;return o=e?new a(t,n,r):a(t,n,r)},performInitialMountWithErrorHandling:function(e,t,n,r,o){var a,i=r.checkpoint();try{a=this.performInitialMount(e,t,n,r,o)}catch(s){r.rollback(i),this._instance.unstable_handleError(s),this._pendingStateQueue&&(this._instance.state=this._processPendingState(this._instance.props,this._instance.context)),i=r.checkpoint(),this._renderedComponent.unmountComponent(!0),r.rollback(i),a=this.performInitialMount(e,t,n,r,o)}return a},performInitialMount:function(e,t,n,r,o){var a=this._instance;a.componentWillMount&&(a.componentWillMount(),this._pendingStateQueue&&(a.state=this._processPendingState(a.props,a.context))),void 0===e&&(e=this._renderValidatedComponent());var i=h.getType(e);this._renderedNodeType=i;var s=this._instantiateReactComponent(e,i!==h.EMPTY);this._renderedComponent=s;var u=0,l=m.mountComponent(s,r,t,n,this._processChildContext(o),u);return l},getHostNode:function(){return m.getHostNode(this._renderedComponent)},unmountComponent:function(e){if(this._renderedComponent){var t=this._instance;if(t.componentWillUnmount&&!t._calledComponentWillUnmount)if(t._calledComponentWillUnmount=!0,e){var n=this.getName()+".componentWillUnmount()";d.invokeGuardedCallback(n,t.componentWillUnmount.bind(t))}else t.componentWillUnmount();this._renderedComponent&&(m.unmountComponent(this._renderedComponent,e),this._renderedNodeType=null,this._renderedComponent=null,this._instance=null),this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1,this._pendingCallbacks=null,this._pendingElement=null,this._context=null,this._rootNodeID=0,this._topLevelWrapper=null,f.remove(t)}},_maskContext:function(e){var t=this._currentElement.type,n=t.contextTypes;if(!n)return g;var r={};for(var o in n)r[o]=e[o];return r},_processContext:function(e){var t=this._maskContext(e);return t},_processChildContext:function(e){var t=this._currentElement.type,n=this._instance,r=n.getChildContext&&n.getChildContext();if(r){"object"!=typeof t.childContextTypes?s("107",this.getName()||"ReactCompositeComponent"):void 0;for(var o in r)o in t.childContextTypes?void 0:s("108",this.getName()||"ReactCompositeComponent",o);return u({},e,r)}return e},_checkContextTypes:function(e,t,n){v(e,t,n,this.getName(),null,this._debugID)},receiveComponent:function(e,t,n){var r=this._currentElement,o=this._context;this._pendingElement=null,this.updateComponent(t,r,e,o,n)},performUpdateIfNecessary:function(e){null!=this._pendingElement?m.receiveComponent(this,this._pendingElement,e,this._context):null!==this._pendingStateQueue||this._pendingForceUpdate?this.updateComponent(e,this._currentElement,this._currentElement,this._context,this._context):this._updateBatchNumber=null},updateComponent:function(e,t,n,r,o){var a=this._instance;null==a?s("136",this.getName()||"ReactCompositeComponent"):void 0;var i,u=!1;this._context===o?i=a.context:(i=this._processContext(o),u=!0);var l=t.props,c=n.props;t!==n&&(u=!0),u&&a.componentWillReceiveProps&&a.componentWillReceiveProps(c,i);var p=this._processPendingState(c,i),d=!0;this._pendingForceUpdate||(a.shouldComponentUpdate?d=a.shouldComponentUpdate(c,p,i):this._compositeType===C.PureClass&&(d=!y(l,c)||!y(a.state,p))),this._updateBatchNumber=null,d?(this._pendingForceUpdate=!1,this._performComponentUpdate(n,c,p,i,e,o)):(this._currentElement=n,this._context=o,a.props=c,a.state=p,a.context=i)},_processPendingState:function(e,t){var n=this._instance,r=this._pendingStateQueue,o=this._pendingReplaceState;if(this._pendingReplaceState=!1,this._pendingStateQueue=null,!r)return n.state;if(o&&1===r.length)return r[0];for(var a=u({},o?r[0]:n.state),i=o?1:0;i<r.length;i++){var s=r[i];u(a,"function"==typeof s?s.call(n,a,e,t):s)}return a},_performComponentUpdate:function(e,t,n,r,o,a){var i,s,u,l=this._instance,c=Boolean(l.componentDidUpdate);c&&(i=l.props,s=l.state,u=l.context),l.componentWillUpdate&&l.componentWillUpdate(t,n,r),this._currentElement=e,this._context=a,l.props=t,l.state=n,l.context=r,this._updateRenderedComponent(o,a),c&&o.getReactMountReady().enqueue(l.componentDidUpdate.bind(l,i,s,u),l)},_updateRenderedComponent:function(e,t){var n=this._renderedComponent,r=n._currentElement,o=this._renderValidatedComponent();if(b(r,o))m.receiveComponent(n,o,e,this._processChildContext(t));else{var a=m.getHostNode(n);m.unmountComponent(n,!1);var i=h.getType(o);this._renderedNodeType=i;var s=this._instantiateReactComponent(o,i!==h.EMPTY);this._renderedComponent=s;var u=0,l=m.mountComponent(s,e,this._hostParent,this._hostContainerInfo,this._processChildContext(t),u);this._replaceNodeWithMarkup(a,l,n)}},_replaceNodeWithMarkup:function(e,t,n){l.replaceNodeWithMarkup(e,t,n)},_renderValidatedComponentWithoutOwnerOrContext:function(){var e=this._instance,t=e.render();return t},_renderValidatedComponent:function(){var e;if(this._compositeType!==C.StatelessFunctional){c.current=this;try{e=this._renderValidatedComponentWithoutOwnerOrContext()}finally{c.current=null}}else e=this._renderValidatedComponentWithoutOwnerOrContext();return null===e||e===!1||p.isValidElement(e)?void 0:s("109",this.getName()||"ReactCompositeComponent"),e},attachRef:function(e,t){var n=this.getPublicInstance();null==n?s("110"):void 0;var r=t.getPublicInstance(),o=n.refs===g?n.refs={}:n.refs;o[e]=r},detachRef:function(e){var t=this.getPublicInstance().refs;delete t[e]},getName:function(){var e=this._currentElement.type,t=this._instance&&this._instance.constructor;return e.displayName||t&&t.displayName||e.name||t&&t.name||null},getPublicInstance:function(){var e=this._instance;return this._compositeType===C.StatelessFunctional?null:e},_instantiateReactComponent:null},x={Mixin:E};t.exports=x},{111:111,132:132,136:136,147:147,154:154,160:160,161:161,162:162,33:33,35:35,56:56,58:58,65:65,66:66,71:71,75:75,80:80}],35:[function(e,t,n){"use strict";var r={current:null};t.exports=r},{}],36:[function(e,t,n){"use strict";var r=e(40),o=e(55),a=e(68),i=e(80),s=e(88),u=e(89),l=e(115),c=e(122),p=e(133);e(161);o.inject();var d={findDOMNode:l,render:a.render,unmountComponentAtNode:a.unmountComponentAtNode,version:u,unstable_batchedUpdates:s.batchedUpdates,unstable_renderSubtreeIntoContainer:p};"undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.inject&&__REACT_DEVTOOLS_GLOBAL_HOOK__.inject({ComponentTree:{getClosestInstanceFromNode:r.getClosestInstanceFromNode,getNodeFromInstance:function(e){return e._renderedComponent&&(e=c(e)),e?r.getNodeFromInstance(e):null}},Mount:a,Reconciler:i});t.exports=d},{115:115,122:122,133:133,161:161,40:40,55:55,68:68,80:80,88:88,89:89}],37:[function(e,t,n){"use strict";var r=e(14),o={getHostProps:r.getHostProps};t.exports=o},{14:14}],38:[function(e,t,n){"use strict";function r(e){if(e){var t=e._currentElement._owner||null;if(t){var n=t.getName();if(n)return" This DOM node was rendered by `"+n+"`."}}return""}function o(e,t){t&&($[e._tag]&&(null!=t.children||null!=t.dangerouslySetInnerHTML?m("137",e._tag,e._currentElement._owner?" Check the render method of "+e._currentElement._owner.getName()+".":""):void 0),null!=t.dangerouslySetInnerHTML&&(null!=t.children?m("60"):void 0,"object"==typeof t.dangerouslySetInnerHTML&&K in t.dangerouslySetInnerHTML?void 0:m("61")),null!=t.style&&"object"!=typeof t.style?m("62",r(e)):void 0)}function a(e,t,n,r){if(!(r instanceof A)){var o=e._hostContainerInfo,a=o._node&&o._node.nodeType===z,s=a?o._node:o._ownerDocument;B(t,s),r.getReactMountReady().enqueue(i,{inst:e,registrationName:t,listener:n})}}function i(){var e=this;T.putListener(e.inst,e.registrationName,e.listener)}function s(){var e=this;S.postMountWrapper(e)}function u(){var e=this;I.postMountWrapper(e)}function l(){var e=this;R.postMountWrapper(e)}function c(){var e=this;e._rootNodeID?void 0:m("63");var t=j(e);switch(t?void 0:m("64"),e._tag){case"iframe":case"object":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topLoad,"load",t)];break;case"video":case"audio":e._wrapperState.listeners=[];for(var n in X)X.hasOwnProperty(n)&&e._wrapperState.listeners.push(w.trapBubbledEvent(x.topLevelTypes[n],X[n],t));break;case"source":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topError,"error",t)];break;case"img":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topError,"error",t),w.trapBubbledEvent(x.topLevelTypes.topLoad,"load",t)];break;case"form":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topReset,"reset",t),w.trapBubbledEvent(x.topLevelTypes.topSubmit,"submit",t)];break;case"input":case"select":case"textarea":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topInvalid,"invalid",t)]}}function p(){O.postUpdateWrapper(this)}function d(e){ee.call(J,e)||(Z.test(e)?void 0:m("65",e),J[e]=!0)}function f(e,t){return e.indexOf("-")>=0||null!=t.is}function h(e){var t=e.type;d(t),this._currentElement=e,this._tag=t.toLowerCase(),this._namespaceURI=null,this._renderedChildren=null,this._previousStyle=null,this._previousStyleCopy=null,this._hostNode=null,this._hostParent=null,this._rootNodeID=0,this._domID=0,this._hostContainerInfo=null,this._wrapperState=null,this._topLevelWrapper=null,this._flags=0}var m=e(132),v=e(162),g=e(1),y=e(4),b=e(8),C=e(9),_=e(10),E=e(11),x=e(16),T=e(17),N=e(18),w=e(27),P=e(37),k=e(39),M=e(40),S=e(46),R=e(47),O=e(48),I=e(52),D=(e(66),e(69)),A=e(84),L=(e(146),e(114)),U=(e(154),e(128),e(158)),F=(e(160),e(138),e(161),k),V=T.deleteListener,j=M.getNodeFromInstance,B=w.listenTo,W=N.registrationNameModules,H={string:!0,number:!0},q=U({style:null}),K=U({__html:null}),Y={children:null,dangerouslySetInnerHTML:null,suppressContentEditableWarning:null},z=11,X={topAbort:"abort",topCanPlay:"canplay",topCanPlayThrough:"canplaythrough",topDurationChange:"durationchange",topEmptied:"emptied",topEncrypted:"encrypted",topEnded:"ended",topError:"error",topLoadedData:"loadeddata",topLoadedMetadata:"loadedmetadata",topLoadStart:"loadstart",topPause:"pause",topPlay:"play",topPlaying:"playing",topProgress:"progress",topRateChange:"ratechange",topSeeked:"seeked",topSeeking:"seeking",topStalled:"stalled",topSuspend:"suspend",topTimeUpdate:"timeupdate",topVolumeChange:"volumechange",topWaiting:"waiting"},G={area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},Q={listing:!0,pre:!0,textarea:!0},$=v({menuitem:!0},G),Z=/^[a-zA-Z][a-zA-Z:_\.\-\d]*$/,J={},ee={}.hasOwnProperty,te=1;h.displayName="ReactDOMComponent",h.Mixin={mountComponent:function(e,t,n,r){this._rootNodeID=te++,this._domID=n._idCounter++,this._hostParent=t,this._hostContainerInfo=n;var a=this._currentElement.props;switch(this._tag){case"audio":case"form":case"iframe":case"img":case"link":case"object":case"source":case"video":this._wrapperState={listeners:null},e.getReactMountReady().enqueue(c,this);break;case"button":a=P.getHostProps(this,a,t);break;case"input":S.mountWrapper(this,a,t),a=S.getHostProps(this,a),e.getReactMountReady().enqueue(c,this);break;case"option":R.mountWrapper(this,a,t),a=R.getHostProps(this,a);break;case"select":O.mountWrapper(this,a,t),a=O.getHostProps(this,a),e.getReactMountReady().enqueue(c,this);break;case"textarea":I.mountWrapper(this,a,t),a=I.getHostProps(this,a),e.getReactMountReady().enqueue(c,this)}o(this,a);var i,p;null!=t?(i=t._namespaceURI,p=t._tag):n._tag&&(i=n._namespaceURI,p=n._tag),(null==i||i===C.svg&&"foreignobject"===p)&&(i=C.html),i===C.html&&("svg"===this._tag?i=C.svg:"math"===this._tag&&(i=C.mathml)),this._namespaceURI=i;var d;if(e.useCreateElement){var f,h=n._ownerDocument;if(i===C.html)if("script"===this._tag){var m=h.createElement("div"),v=this._currentElement.type;m.innerHTML="<"+v+"></"+v+">",f=m.removeChild(m.firstChild)}else f=a.is?h.createElement(this._currentElement.type,a.is):h.createElement(this._currentElement.type);else f=h.createElementNS(i,this._currentElement.type);M.precacheNode(this,f),this._flags|=F.hasCachedChildNodes,this._hostParent||E.setAttributeForRoot(f),this._updateDOMProperties(null,a,e);var y=b(f);this._createInitialChildren(e,a,r,y),d=y}else{var _=this._createOpenTagMarkupAndPutListeners(e,a),x=this._createContentMarkup(e,a,r);d=!x&&G[this._tag]?_+"/>":_+">"+x+"</"+this._currentElement.type+">"}switch(this._tag){case"input":e.getReactMountReady().enqueue(s,this),a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"textarea":e.getReactMountReady().enqueue(u,this),a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"select":a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"button":a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"option":e.getReactMountReady().enqueue(l,this)}return d},_createOpenTagMarkupAndPutListeners:function(e,t){var n="<"+this._currentElement.type;for(var r in t)if(t.hasOwnProperty(r)){var o=t[r];if(null!=o)if(W.hasOwnProperty(r))o&&a(this,r,o,e);else{r===q&&(o&&(o=this._previousStyleCopy=v({},t.style)),o=y.createMarkupForStyles(o,this));var i=null;null!=this._tag&&f(this._tag,t)?Y.hasOwnProperty(r)||(i=E.createMarkupForCustomAttribute(r,o)):i=E.createMarkupForProperty(r,o),i&&(n+=" "+i)}}return e.renderToStaticMarkup?n:(this._hostParent||(n+=" "+E.createMarkupForRoot()),n+=" "+E.createMarkupForID(this._domID))},_createContentMarkup:function(e,t,n){var r="",o=t.dangerouslySetInnerHTML;if(null!=o)null!=o.__html&&(r=o.__html);else{var a=H[typeof t.children]?t.children:null,i=null!=a?null:t.children;if(null!=a)r=L(a);else if(null!=i){var s=this.mountChildren(i,e,n);r=s.join("")}}return Q[this._tag]&&"\n"===r.charAt(0)?"\n"+r:r},_createInitialChildren:function(e,t,n,r){var o=t.dangerouslySetInnerHTML;if(null!=o)null!=o.__html&&b.queueHTML(r,o.__html);else{var a=H[typeof t.children]?t.children:null,i=null!=a?null:t.children;if(null!=a)b.queueText(r,a);else if(null!=i)for(var s=this.mountChildren(i,e,n),u=0;u<s.length;u++)b.queueChild(r,s[u])}},receiveComponent:function(e,t,n){var r=this._currentElement;this._currentElement=e,this.updateComponent(t,r,e,n)},updateComponent:function(e,t,n,r){var a=t.props,i=this._currentElement.props;switch(this._tag){case"button":a=P.getHostProps(this,a),i=P.getHostProps(this,i);break;case"input":a=S.getHostProps(this,a),i=S.getHostProps(this,i);break;case"option":a=R.getHostProps(this,a),i=R.getHostProps(this,i);break;case"select":a=O.getHostProps(this,a),i=O.getHostProps(this,i);break;case"textarea":a=I.getHostProps(this,a),i=I.getHostProps(this,i)}switch(o(this,i),this._updateDOMProperties(a,i,e),this._updateDOMChildren(a,i,e,r),this._tag){case"input":S.updateWrapper(this);break;case"textarea":I.updateWrapper(this);break;case"select":e.getReactMountReady().enqueue(p,this)}},_updateDOMProperties:function(e,t,n){var r,o,i;for(r in e)if(!t.hasOwnProperty(r)&&e.hasOwnProperty(r)&&null!=e[r])if(r===q){var s=this._previousStyleCopy;for(o in s)s.hasOwnProperty(o)&&(i=i||{},i[o]="");this._previousStyleCopy=null}else W.hasOwnProperty(r)?e[r]&&V(this,r):f(this._tag,e)?Y.hasOwnProperty(r)||E.deleteValueForAttribute(j(this),r):(_.properties[r]||_.isCustomAttribute(r))&&E.deleteValueForProperty(j(this),r);for(r in t){var u=t[r],l=r===q?this._previousStyleCopy:null!=e?e[r]:void 0;if(t.hasOwnProperty(r)&&u!==l&&(null!=u||null!=l))if(r===q)if(u?u=this._previousStyleCopy=v({},u):this._previousStyleCopy=null,l){for(o in l)!l.hasOwnProperty(o)||u&&u.hasOwnProperty(o)||(i=i||{},i[o]="");for(o in u)u.hasOwnProperty(o)&&l[o]!==u[o]&&(i=i||{},i[o]=u[o])}else i=u;else if(W.hasOwnProperty(r))u?a(this,r,u,n):l&&V(this,r);else if(f(this._tag,t))Y.hasOwnProperty(r)||E.setValueForAttribute(j(this),r,u);else if(_.properties[r]||_.isCustomAttribute(r)){var c=j(this);null!=u?E.setValueForProperty(c,r,u):E.deleteValueForProperty(c,r)}}i&&y.setValueForStyles(j(this),i,this)},_updateDOMChildren:function(e,t,n,r){var o=H[typeof e.children]?e.children:null,a=H[typeof t.children]?t.children:null,i=e.dangerouslySetInnerHTML&&e.dangerouslySetInnerHTML.__html,s=t.dangerouslySetInnerHTML&&t.dangerouslySetInnerHTML.__html,u=null!=o?null:e.children,l=null!=a?null:t.children,c=null!=o||null!=i,p=null!=a||null!=s;null!=u&&null==l?this.updateChildren(null,n,r):c&&!p&&this.updateTextContent(""),null!=a?o!==a&&this.updateTextContent(""+a):null!=s?i!==s&&this.updateMarkup(""+s):null!=l&&this.updateChildren(l,n,r)},getHostNode:function(){return j(this)},unmountComponent:function(e){switch(this._tag){case"audio":case"form":case"iframe":case"img":case"link":case"object":case"source":case"video":var t=this._wrapperState.listeners;if(t)for(var n=0;n<t.length;n++)t[n].remove();break;case"html":case"head":case"body":m("66",this._tag)}this.unmountChildren(e),M.uncacheNode(this),T.deleteAllListeners(this),this._rootNodeID=0,this._domID=0,this._wrapperState=null},getPublicInstance:function(){return j(this)}},v(h.prototype,h.Mixin,D.Mixin),t.exports=h},{1:1,10:10,11:11,114:114,128:128,132:132,138:138,146:146,154:154,158:158,16:16,160:160,161:161,162:162,17:17,18:18,27:27,37:37,39:39,4:4,40:40,46:46,47:47,48:48,52:52,66:66,69:69,8:8,84:84,9:9}],39:[function(e,t,n){"use strict";var r={hasCachedChildNodes:1};t.exports=r},{}],40:[function(e,t,n){"use strict";function r(e){for(var t;t=e._renderedComponent;)e=t;return e}function o(e,t){var n=r(e);n._hostNode=t,t[m]=n}function a(e){var t=e._hostNode;t&&(delete t[m],e._hostNode=null)}function i(e,t){if(!(e._flags&h.hasCachedChildNodes)){var n=e._renderedChildren,a=t.firstChild;e:for(var i in n)if(n.hasOwnProperty(i)){var s=n[i],u=r(s)._domID;if(0!==u){for(;null!==a;a=a.nextSibling)if(1===a.nodeType&&a.getAttribute(f)===String(u)||8===a.nodeType&&a.nodeValue===" react-text: "+u+" "||8===a.nodeType&&a.nodeValue===" react-empty: "+u+" "){o(s,a);continue e}c("32",u)}}e._flags|=h.hasCachedChildNodes}}function s(e){if(e[m])return e[m];for(var t=[];!e[m];){if(t.push(e),!e.parentNode)return null;e=e.parentNode}for(var n,r;e&&(r=e[m]);e=t.pop())n=r,t.length&&i(r,e);return n}function u(e){var t=s(e);return null!=t&&t._hostNode===e?t:null}function l(e){if(void 0===e._hostNode?c("33"):void 0,e._hostNode)return e._hostNode;for(var t=[];!e._hostNode;)t.push(e),e._hostParent?void 0:c("34"),e=e._hostParent;for(;t.length;e=t.pop())i(e,e._hostNode);return e._hostNode}var c=e(132),p=e(10),d=e(39),f=(e(154),p.ID_ATTRIBUTE_NAME),h=d,m="__reactInternalInstance$"+Math.random().toString(36).slice(2),v={getClosestInstanceFromNode:s,getInstanceFromNode:u,getNodeFromInstance:l,precacheChildNodes:i,precacheNode:o,uncacheNode:a};t.exports=v},{10:10,132:132,154:154,39:39}],41:[function(e,t,n){"use strict";function r(e,t){var n={_topLevelWrapper:e,_idCounter:1,_ownerDocument:t?t.nodeType===o?t:t.ownerDocument:null,_node:t,_tag:t?t.nodeName.toLowerCase():null,_namespaceURI:t?t.namespaceURI:null};return n}var o=(e(138),9);t.exports=r},{138:138}],42:[function(e,t,n){"use strict";var r=e(162),o=e(8),a=e(40),i=function(e){this._currentElement=null,this._hostNode=null,this._hostParent=null,this._hostContainerInfo=null,this._domID=0};r(i.prototype,{mountComponent:function(e,t,n,r){var i=n._idCounter++;this._domID=i,this._hostParent=t,this._hostContainerInfo=n;var s=" react-empty: "+this._domID+" ";if(e.useCreateElement){var u=n._ownerDocument,l=u.createComment(s);return a.precacheNode(this,l),
o(l)}return e.renderToStaticMarkup?"":"<!--"+s+"-->"},receiveComponent:function(){},getHostNode:function(){return a.getNodeFromInstance(this)},unmountComponent:function(){a.uncacheNode(this)}}),t.exports=i},{162:162,40:40,8:8}],43:[function(e,t,n){"use strict";var r=e(56),o=r.createFactory,a={a:o("a"),abbr:o("abbr"),address:o("address"),area:o("area"),article:o("article"),aside:o("aside"),audio:o("audio"),b:o("b"),base:o("base"),bdi:o("bdi"),bdo:o("bdo"),big:o("big"),blockquote:o("blockquote"),body:o("body"),br:o("br"),button:o("button"),canvas:o("canvas"),caption:o("caption"),cite:o("cite"),code:o("code"),col:o("col"),colgroup:o("colgroup"),data:o("data"),datalist:o("datalist"),dd:o("dd"),del:o("del"),details:o("details"),dfn:o("dfn"),dialog:o("dialog"),div:o("div"),dl:o("dl"),dt:o("dt"),em:o("em"),embed:o("embed"),fieldset:o("fieldset"),figcaption:o("figcaption"),figure:o("figure"),footer:o("footer"),form:o("form"),h1:o("h1"),h2:o("h2"),h3:o("h3"),h4:o("h4"),h5:o("h5"),h6:o("h6"),head:o("head"),header:o("header"),hgroup:o("hgroup"),hr:o("hr"),html:o("html"),i:o("i"),iframe:o("iframe"),img:o("img"),input:o("input"),ins:o("ins"),kbd:o("kbd"),keygen:o("keygen"),label:o("label"),legend:o("legend"),li:o("li"),link:o("link"),main:o("main"),map:o("map"),mark:o("mark"),menu:o("menu"),menuitem:o("menuitem"),meta:o("meta"),meter:o("meter"),nav:o("nav"),noscript:o("noscript"),object:o("object"),ol:o("ol"),optgroup:o("optgroup"),option:o("option"),output:o("output"),p:o("p"),param:o("param"),picture:o("picture"),pre:o("pre"),progress:o("progress"),q:o("q"),rp:o("rp"),rt:o("rt"),ruby:o("ruby"),s:o("s"),samp:o("samp"),script:o("script"),section:o("section"),select:o("select"),small:o("small"),source:o("source"),span:o("span"),strong:o("strong"),style:o("style"),sub:o("sub"),summary:o("summary"),sup:o("sup"),table:o("table"),tbody:o("tbody"),td:o("td"),textarea:o("textarea"),tfoot:o("tfoot"),th:o("th"),thead:o("thead"),time:o("time"),title:o("title"),tr:o("tr"),track:o("track"),u:o("u"),ul:o("ul"),var:o("var"),video:o("video"),wbr:o("wbr"),circle:o("circle"),clipPath:o("clipPath"),defs:o("defs"),ellipse:o("ellipse"),g:o("g"),image:o("image"),line:o("line"),linearGradient:o("linearGradient"),mask:o("mask"),path:o("path"),pattern:o("pattern"),polygon:o("polygon"),polyline:o("polyline"),radialGradient:o("radialGradient"),rect:o("rect"),stop:o("stop"),svg:o("svg"),text:o("text"),tspan:o("tspan")};t.exports=a},{56:56}],44:[function(e,t,n){"use strict";var r={useCreateElement:!0};t.exports=r},{}],45:[function(e,t,n){"use strict";var r=e(7),o=e(40),a={dangerouslyProcessChildrenUpdates:function(e,t){var n=o.getNodeFromInstance(e);r.processUpdates(n,t)}};t.exports=a},{40:40,7:7}],46:[function(e,t,n){"use strict";function r(){this._rootNodeID&&d.updateWrapper(this)}function o(e){var t=this._currentElement.props,n=l.executeOnChange(t,e);p.asap(r,this);var o=t.name;if("radio"===t.type&&null!=o){for(var i=c.getNodeFromInstance(this),s=i;s.parentNode;)s=s.parentNode;for(var u=s.querySelectorAll("input[name="+JSON.stringify(""+o)+'][type="radio"]'),d=0;d<u.length;d++){var f=u[d];if(f!==i&&f.form===i.form){var h=c.getInstanceFromNode(f);h?void 0:a("90"),p.asap(r,h)}}}return n}var a=e(132),i=e(162),s=e(14),u=e(11),l=e(24),c=e(40),p=e(88),d=(e(154),e(161),{getHostProps:function(e,t){var n=l.getValue(t),r=l.getChecked(t),o=i({type:void 0,step:void 0,min:void 0,max:void 0},s.getHostProps(e,t),{defaultChecked:void 0,defaultValue:void 0,value:null!=n?n:e._wrapperState.initialValue,checked:null!=r?r:e._wrapperState.initialChecked,onChange:e._wrapperState.onChange});return o},mountWrapper:function(e,t){var n=t.defaultValue;e._wrapperState={initialChecked:null!=t.checked?t.checked:t.defaultChecked,initialValue:null!=t.value?t.value:n,listeners:null,onChange:o.bind(e)}},updateWrapper:function(e){var t=e._currentElement.props,n=t.checked;null!=n&&u.setValueForProperty(c.getNodeFromInstance(e),"checked",n||!1);var r=c.getNodeFromInstance(e),o=l.getValue(t);if(null!=o){var a=""+o;a!==r.value&&(r.value=a)}else null==t.value&&null!=t.defaultValue&&(r.defaultValue=""+t.defaultValue),null==t.checked&&null!=t.defaultChecked&&(r.defaultChecked=!!t.defaultChecked)},postMountWrapper:function(e){var t=e._currentElement.props,n=c.getNodeFromInstance(e);switch(t.type){case"submit":case"reset":break;case"color":case"date":case"datetime":case"datetime-local":case"month":case"time":case"week":n.value="",n.value=n.defaultValue;break;default:n.value=n.value}var r=n.name;""!==r&&(n.name=""),n.defaultChecked=!n.defaultChecked,n.defaultChecked=!n.defaultChecked,""!==r&&(n.name=r)}});t.exports=d},{11:11,132:132,14:14,154:154,161:161,162:162,24:24,40:40,88:88}],47:[function(e,t,n){"use strict";function r(e){var t="";return a.forEach(e,function(e){null!=e&&("string"==typeof e||"number"==typeof e?t+=e:u||(u=!0))}),t}var o=e(162),a=e(29),i=e(40),s=e(48),u=(e(161),!1),l={mountWrapper:function(e,t,n){var o=null;if(null!=n){var a=n;"optgroup"===a._tag&&(a=a._hostParent),null!=a&&"select"===a._tag&&(o=s.getSelectValueContext(a))}var i=null;if(null!=o){var u;if(u=null!=t.value?t.value+"":r(t.children),i=!1,Array.isArray(o)){for(var l=0;l<o.length;l++)if(""+o[l]===u){i=!0;break}}else i=""+o===u}e._wrapperState={selected:i}},postMountWrapper:function(e){var t=e._currentElement.props;if(null!=t.value){var n=i.getNodeFromInstance(e);n.setAttribute("value",t.value)}},getHostProps:function(e,t){var n=o({selected:void 0,children:void 0},t);null!=e._wrapperState.selected&&(n.selected=e._wrapperState.selected);var a=r(t.children);return a&&(n.children=a),n}};t.exports=l},{161:161,162:162,29:29,40:40,48:48}],48:[function(e,t,n){"use strict";function r(){if(this._rootNodeID&&this._wrapperState.pendingUpdate){this._wrapperState.pendingUpdate=!1;var e=this._currentElement.props,t=u.getValue(e);null!=t&&o(this,Boolean(e.multiple),t)}}function o(e,t,n){var r,o,a=l.getNodeFromInstance(e).options;if(t){for(r={},o=0;o<n.length;o++)r[""+n[o]]=!0;for(o=0;o<a.length;o++){var i=r.hasOwnProperty(a[o].value);a[o].selected!==i&&(a[o].selected=i)}}else{for(r=""+n,o=0;o<a.length;o++)if(a[o].value===r)return void(a[o].selected=!0);a.length&&(a[0].selected=!0)}}function a(e){var t=this._currentElement.props,n=u.executeOnChange(t,e);return this._rootNodeID&&(this._wrapperState.pendingUpdate=!0),c.asap(r,this),n}var i=e(162),s=e(14),u=e(24),l=e(40),c=e(88),p=(e(161),!1),d={getHostProps:function(e,t){return i({},s.getHostProps(e,t),{onChange:e._wrapperState.onChange,value:void 0})},mountWrapper:function(e,t){var n=u.getValue(t);e._wrapperState={pendingUpdate:!1,initialValue:null!=n?n:t.defaultValue,listeners:null,onChange:a.bind(e),wasMultiple:Boolean(t.multiple)},void 0===t.value||void 0===t.defaultValue||p||(p=!0)},getSelectValueContext:function(e){return e._wrapperState.initialValue},postUpdateWrapper:function(e){var t=e._currentElement.props;e._wrapperState.initialValue=void 0;var n=e._wrapperState.wasMultiple;e._wrapperState.wasMultiple=Boolean(t.multiple);var r=u.getValue(t);null!=r?(e._wrapperState.pendingUpdate=!1,o(e,Boolean(t.multiple),r)):n!==Boolean(t.multiple)&&(null!=t.defaultValue?o(e,Boolean(t.multiple),t.defaultValue):o(e,Boolean(t.multiple),t.multiple?[]:""))}};t.exports=d},{14:14,161:161,162:162,24:24,40:40,88:88}],49:[function(e,t,n){"use strict";function r(e,t,n,r){return e===n&&t===r}function o(e){var t=document.selection,n=t.createRange(),r=n.text.length,o=n.duplicate();o.moveToElementText(e),o.setEndPoint("EndToStart",n);var a=o.text.length,i=a+r;return{start:a,end:i}}function a(e){var t=window.getSelection&&window.getSelection();if(!t||0===t.rangeCount)return null;var n=t.anchorNode,o=t.anchorOffset,a=t.focusNode,i=t.focusOffset,s=t.getRangeAt(0);try{s.startContainer.nodeType,s.endContainer.nodeType}catch(e){return null}var u=r(t.anchorNode,t.anchorOffset,t.focusNode,t.focusOffset),l=u?0:s.toString().length,c=s.cloneRange();c.selectNodeContents(e),c.setEnd(s.startContainer,s.startOffset);var p=r(c.startContainer,c.startOffset,c.endContainer,c.endOffset),d=p?0:c.toString().length,f=d+l,h=document.createRange();h.setStart(n,o),h.setEnd(a,i);var m=h.collapsed;return{start:m?f:d,end:m?d:f}}function i(e,t){var n,r,o=document.selection.createRange().duplicate();void 0===t.end?(n=t.start,r=n):t.start>t.end?(n=t.end,r=t.start):(n=t.start,r=t.end),o.moveToElementText(e),o.moveStart("character",n),o.setEndPoint("EndToStart",o),o.moveEnd("character",r-n),o.select()}function s(e,t){if(window.getSelection){var n=window.getSelection(),r=e[c()].length,o=Math.min(t.start,r),a=void 0===t.end?o:Math.min(t.end,r);if(!n.extend&&o>a){var i=a;a=o,o=i}var s=l(e,o),u=l(e,a);if(s&&u){var p=document.createRange();p.setStart(s.node,s.offset),n.removeAllRanges(),o>a?(n.addRange(p),n.extend(u.node,u.offset)):(p.setEnd(u.node,u.offset),n.addRange(p))}}}var u=e(140),l=e(124),c=e(125),p=u.canUseDOM&&"selection"in document&&!("getSelection"in window),d={getOffsets:p?o:a,setOffsets:p?i:s};t.exports=d},{124:124,125:125,140:140}],50:[function(e,t,n){"use strict";var r=e(55),o=e(83),a=e(89);r.inject();var i={renderToString:o.renderToString,renderToStaticMarkup:o.renderToStaticMarkup,version:a};t.exports=i},{55:55,83:83,89:89}],51:[function(e,t,n){"use strict";var r=e(132),o=e(162),a=e(7),i=e(8),s=e(40),u=e(114),l=(e(154),e(138),function(e){this._currentElement=e,this._stringText=""+e,this._hostNode=null,this._hostParent=null,this._domID=0,this._mountIndex=0,this._closingComment=null,this._commentNodes=null});o(l.prototype,{mountComponent:function(e,t,n,r){var o=n._idCounter++,a=" react-text: "+o+" ",l=" /react-text ";if(this._domID=o,this._hostParent=t,e.useCreateElement){var c=n._ownerDocument,p=c.createComment(a),d=c.createComment(l),f=i(c.createDocumentFragment());return i.queueChild(f,i(p)),this._stringText&&i.queueChild(f,i(c.createTextNode(this._stringText))),i.queueChild(f,i(d)),s.precacheNode(this,p),this._closingComment=d,f}var h=u(this._stringText);return e.renderToStaticMarkup?h:"<!--"+a+"-->"+h+"<!--"+l+"-->"},receiveComponent:function(e,t){if(e!==this._currentElement){this._currentElement=e;var n=""+e;if(n!==this._stringText){this._stringText=n;var r=this.getHostNode();a.replaceDelimitedText(r[0],r[1],n)}}},getHostNode:function(){var e=this._commentNodes;if(e)return e;if(!this._closingComment)for(var t=s.getNodeFromInstance(this),n=t.nextSibling;;){if(null==n?r("67",this._domID):void 0,8===n.nodeType&&" /react-text "===n.nodeValue){this._closingComment=n;break}n=n.nextSibling}return e=[this._hostNode,this._closingComment],this._commentNodes=e,e},unmountComponent:function(){this._closingComment=null,this._commentNodes=null,s.uncacheNode(this)}}),t.exports=l},{114:114,132:132,138:138,154:154,162:162,40:40,7:7,8:8}],52:[function(e,t,n){"use strict";function r(){this._rootNodeID&&p.updateWrapper(this)}function o(e){var t=this._currentElement.props,n=u.executeOnChange(t,e);return c.asap(r,this),n}var a=e(132),i=e(162),s=e(14),u=e(24),l=e(40),c=e(88),p=(e(154),e(161),{getHostProps:function(e,t){null!=t.dangerouslySetInnerHTML?a("91"):void 0;var n=i({},s.getHostProps(e,t),{value:void 0,defaultValue:void 0,children:""+e._wrapperState.initialValue,onChange:e._wrapperState.onChange});return n},mountWrapper:function(e,t){var n=u.getValue(t),r=n;if(null==n){var i=t.defaultValue,s=t.children;null!=s&&(null!=i?a("92"):void 0,Array.isArray(s)&&(s.length<=1?void 0:a("93"),s=s[0]),i=""+s),null==i&&(i=""),r=i}e._wrapperState={initialValue:""+r,listeners:null,onChange:o.bind(e)}},updateWrapper:function(e){var t=e._currentElement.props,n=l.getNodeFromInstance(e),r=u.getValue(t);if(null!=r){var o=""+r;o!==n.value&&(n.value=o),null==t.defaultValue&&(n.defaultValue=o)}null!=t.defaultValue&&(n.defaultValue=t.defaultValue)},postMountWrapper:function(e){var t=l.getNodeFromInstance(e);t.value=t.textContent}});t.exports=p},{132:132,14:14,154:154,161:161,162:162,24:24,40:40,88:88}],53:[function(e,t,n){"use strict";function r(e,t){"_hostNode"in e?void 0:u("33"),"_hostNode"in t?void 0:u("33");for(var n=0,r=e;r;r=r._hostParent)n++;for(var o=0,a=t;a;a=a._hostParent)o++;for(;n-o>0;)e=e._hostParent,n--;for(;o-n>0;)t=t._hostParent,o--;for(var i=n;i--;){if(e===t)return e;e=e._hostParent,t=t._hostParent}return null}function o(e,t){"_hostNode"in e?void 0:u("35"),"_hostNode"in t?void 0:u("35");for(;t;){if(t===e)return!0;t=t._hostParent}return!1}function a(e){return"_hostNode"in e?void 0:u("36"),e._hostParent}function i(e,t,n){for(var r=[];e;)r.push(e),e=e._hostParent;var o;for(o=r.length;o-- >0;)t(r[o],!1,n);for(o=0;o<r.length;o++)t(r[o],!0,n)}function s(e,t,n,o,a){for(var i=e&&t?r(e,t):null,s=[];e&&e!==i;)s.push(e),e=e._hostParent;for(var u=[];t&&t!==i;)u.push(t),t=t._hostParent;var l;for(l=0;l<s.length;l++)n(s[l],!0,o);for(l=u.length;l-- >0;)n(u[l],!1,a)}var u=e(132);e(154);t.exports={isAncestor:o,getLowestCommonAncestor:r,getParentInstance:a,traverseTwoPhase:i,traverseEnterLeave:s}},{132:132,154:154}],54:[function(e,t,n){"use strict";function r(){this.reinitializeTransaction()}var o=e(162),a=e(88),i=e(106),s=e(146),u={initialize:s,close:function(){d.isBatchingUpdates=!1}},l={initialize:s,close:a.flushBatchedUpdates.bind(a)},c=[l,u];o(r.prototype,i.Mixin,{getTransactionWrappers:function(){return c}});var p=new r,d={isBatchingUpdates:!1,batchedUpdates:function(e,t,n,r,o,a){var i=d.isBatchingUpdates;d.isBatchingUpdates=!0,i?e(t,n,r,o,a):p.perform(e,null,t,n,r,o,a)}};t.exports=d},{106:106,146:146,162:162,88:88}],55:[function(e,t,n){"use strict";function r(){E||(E=!0,g.EventEmitter.injectReactEventListener(v),g.EventPluginHub.injectEventPluginOrder(i),g.EventPluginUtils.injectComponentTree(p),g.EventPluginUtils.injectTreeTraversal(f),g.EventPluginHub.injectEventPluginsByName({SimpleEventPlugin:_,EnterLeaveEventPlugin:s,ChangeEventPlugin:a,SelectEventPlugin:C,BeforeInputEventPlugin:o}),g.HostComponent.injectGenericComponentClass(c),g.HostComponent.injectTextComponentClass(h),g.DOMProperty.injectDOMPropertyConfig(u),g.DOMProperty.injectDOMPropertyConfig(b),g.EmptyComponent.injectEmptyComponentFactory(function(e){return new d(e)}),g.Updates.injectReconcileTransaction(y),g.Updates.injectBatchingStrategy(m),g.Component.injectEnvironment(l))}var o=e(2),a=e(6),i=e(13),s=e(15),u=e(22),l=e(32),c=e(38),p=e(40),d=e(42),f=e(53),h=e(51),m=e(54),v=e(60),g=e(63),y=e(79),b=e(90),C=e(91),_=e(92),E=!1;t.exports={inject:r}},{13:13,15:15,2:2,22:22,32:32,38:38,40:40,42:42,51:51,53:53,54:54,6:6,60:60,63:63,79:79,90:90,91:91,92:92}],56:[function(e,t,n){"use strict";function r(e){return void 0!==e.ref}function o(e){return void 0!==e.key}var a=e(162),i=e(35),s=(e(161),e(110),Object.prototype.hasOwnProperty),u="function"==typeof Symbol&&Symbol.for&&Symbol.for("react.element")||60103,l={key:!0,ref:!0,__self:!0,__source:!0},c=function(e,t,n,r,o,a,i){var s={$$typeof:u,type:e,key:t,ref:n,props:i,_owner:a};return s};c.createElement=function(e,t,n){var a,u={},p=null,d=null,f=null,h=null;if(null!=t){r(t)&&(d=t.ref),o(t)&&(p=""+t.key),f=void 0===t.__self?null:t.__self,h=void 0===t.__source?null:t.__source;for(a in t)s.call(t,a)&&!l.hasOwnProperty(a)&&(u[a]=t[a])}var m=arguments.length-2;if(1===m)u.children=n;else if(m>1){for(var v=Array(m),g=0;g<m;g++)v[g]=arguments[g+2];u.children=v}if(e&&e.defaultProps){var y=e.defaultProps;for(a in y)void 0===u[a]&&(u[a]=y[a])}return c(e,p,d,f,h,i.current,u)},c.createFactory=function(e){var t=c.createElement.bind(null,e);return t.type=e,t},c.cloneAndReplaceKey=function(e,t){var n=c(e.type,t,e.ref,e._self,e._source,e._owner,e.props);return n},c.cloneElement=function(e,t,n){var u,p=a({},e.props),d=e.key,f=e.ref,h=e._self,m=e._source,v=e._owner;if(null!=t){r(t)&&(f=t.ref,v=i.current),o(t)&&(d=""+t.key);var g;e.type&&e.type.defaultProps&&(g=e.type.defaultProps);for(u in t)s.call(t,u)&&!l.hasOwnProperty(u)&&(void 0===t[u]&&void 0!==g?p[u]=g[u]:p[u]=t[u])}var y=arguments.length-2;if(1===y)p.children=n;else if(y>1){for(var b=Array(y),C=0;C<y;C++)b[C]=arguments[C+2];p.children=b}return c(e.type,d,f,h,m,v,p)},c.isValidElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===u},c.REACT_ELEMENT_TYPE=u,t.exports=c},{110:110,161:161,162:162,35:35}],57:[function(e,t,n){"use strict";var r,o={injectEmptyComponentFactory:function(e){r=e}},a={create:function(e){return r(e)}};a.injection=o,t.exports=a},{}],58:[function(e,t,n){"use strict";function r(e,t,n,r){try{return t(n,r)}catch(e){return void(null===o&&(o=e))}}var o=null,a={invokeGuardedCallback:r,invokeGuardedCallbackWithCatch:r,rethrowCaughtError:function(){if(o){var e=o;throw o=null,e}}};t.exports=a},{}],59:[function(e,t,n){"use strict";function r(e){o.enqueueEvents(e),o.processEventQueue(!1)}var o=e(17),a={handleTopLevel:function(e,t,n,a){var i=o.extractEvents(e,t,n,a);r(i)}};t.exports=a},{17:17}],60:[function(e,t,n){"use strict";function r(e){for(;e._hostParent;)e=e._hostParent;var t=p.getNodeFromInstance(e),n=t.parentNode;return p.getClosestInstanceFromNode(n)}function o(e,t){this.topLevelType=e,this.nativeEvent=t,this.ancestors=[]}function a(e){var t=f(e.nativeEvent),n=p.getClosestInstanceFromNode(t),o=n;do e.ancestors.push(o),o=o&&r(o);while(o);for(var a=0;a<e.ancestors.length;a++)n=e.ancestors[a],m._handleTopLevel(e.topLevelType,n,e.nativeEvent,f(e.nativeEvent))}function i(e){var t=h(window);e(t)}var s=e(162),u=e(139),l=e(140),c=e(25),p=e(40),d=e(88),f=e(121),h=e(151);s(o.prototype,{destructor:function(){this.topLevelType=null,this.nativeEvent=null,this.ancestors.length=0}}),c.addPoolingTo(o,c.twoArgumentPooler);var m={_enabled:!0,_handleTopLevel:null,WINDOW_HANDLE:l.canUseDOM?window:null,setHandleTopLevel:function(e){m._handleTopLevel=e},setEnabled:function(e){m._enabled=!!e},isEnabled:function(){return m._enabled},trapBubbledEvent:function(e,t,n){var r=n;return r?u.listen(r,t,m.dispatchEvent.bind(null,e)):null},trapCapturedEvent:function(e,t,n){var r=n;return r?u.capture(r,t,m.dispatchEvent.bind(null,e)):null},monitorScrollValue:function(e){var t=i.bind(null,e);u.listen(window,"scroll",t)},dispatchEvent:function(e,t){if(m._enabled){var n=o.getPooled(e,t);try{d.batchedUpdates(a,n)}finally{o.release(n)}}}};t.exports=m},{121:121,139:139,140:140,151:151,162:162,25:25,40:40,88:88}],61:[function(e,t,n){"use strict";var r={logTopLevelRenders:!1};t.exports=r},{}],62:[function(e,t,n){"use strict";function r(e){return u?void 0:i("111",e.type),new u(e)}function o(e){return new c(e)}function a(e){return e instanceof c}var i=e(132),s=e(162),u=(e(154),null),l={},c=null,p={injectGenericComponentClass:function(e){u=e},injectTextComponentClass:function(e){c=e},injectComponentClasses:function(e){s(l,e)}},d={createInternalComponent:r,createInstanceForText:o,isTextComponent:a,injection:p};t.exports=d},{132:132,154:154,162:162}],63:[function(e,t,n){"use strict";var r=e(10),o=e(17),a=e(19),i=e(33),s=e(30),u=e(57),l=e(27),c=e(62),p=e(88),d={Component:i.injection,Class:s.injection,DOMProperty:r.injection,EmptyComponent:u.injection,EventPluginHub:o.injection,EventPluginUtils:a.injection,EventEmitter:l.injection,HostComponent:c.injection,Updates:p.injection};t.exports=d},{10:10,17:17,19:19,27:27,30:30,33:33,57:57,62:62,88:88}],64:[function(e,t,n){"use strict";function r(e){return a(document.documentElement,e)}var o=e(49),a=e(143),i=e(148),s=e(149),u={hasSelectionCapabilities:function(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&("input"===t&&"text"===e.type||"textarea"===t||"true"===e.contentEditable)},getSelectionInformation:function(){var e=s();return{focusedElem:e,selectionRange:u.hasSelectionCapabilities(e)?u.getSelection(e):null}},restoreSelection:function(e){var t=s(),n=e.focusedElem,o=e.selectionRange;t!==n&&r(n)&&(u.hasSelectionCapabilities(n)&&u.setSelection(n,o),i(n))},getSelection:function(e){var t;if("selectionStart"in e)t={start:e.selectionStart,end:e.selectionEnd};else if(document.selection&&e.nodeName&&"input"===e.nodeName.toLowerCase()){var n=document.selection.createRange();n.parentElement()===e&&(t={start:-n.moveStart("character",-e.value.length),end:-n.moveEnd("character",-e.value.length)})}else t=o.getOffsets(e);return t||{start:0,end:0}},setSelection:function(e,t){var n=t.start,r=t.end;if(void 0===r&&(r=n),"selectionStart"in e)e.selectionStart=n,e.selectionEnd=Math.min(r,e.value.length);else if(document.selection&&e.nodeName&&"input"===e.nodeName.toLowerCase()){var a=e.createTextRange();a.collapse(!0),a.moveStart("character",n),a.moveEnd("character",r-n),a.select()}else o.setOffsets(e,t)}};t.exports=u},{143:143,148:148,149:149,49:49}],65:[function(e,t,n){"use strict";var r={remove:function(e){e._reactInternalInstance=void 0},get:function(e){return e._reactInternalInstance},has:function(e){return void 0!==e._reactInternalInstance},set:function(e,t){e._reactInternalInstance=t}};t.exports=r},{}],66:[function(e,t,n){"use strict";var r=null;t.exports={debugTool:r}},{}],67:[function(e,t,n){"use strict";var r=e(109),o=/\/?>/,a=/^<\!\-\-/,i={CHECKSUM_ATTR_NAME:"data-react-checksum",addChecksumToMarkup:function(e){var t=r(e);return a.test(e)?e:e.replace(o," "+i.CHECKSUM_ATTR_NAME+'="'+t+'"$&')},canReuseMarkup:function(e,t){var n=t.getAttribute(i.CHECKSUM_ATTR_NAME);n=n&&parseInt(n,10);var o=r(e);return o===n}};t.exports=i},{109:109}],68:[function(e,t,n){"use strict";function r(e,t){for(var n=Math.min(e.length,t.length),r=0;r<n;r++)if(e.charAt(r)!==t.charAt(r))return r;return e.length===t.length?-1:n}function o(e){return e?e.nodeType===D?e.documentElement:e.firstChild:null}function a(e){return e.getAttribute&&e.getAttribute(R)||""}function i(e,t,n,r,o){var a;if(_.logTopLevelRenders){var i=e._currentElement.props,s=i.type;a="React mount: "+("string"==typeof s?s:s.displayName||s.name),console.time(a)}var u=T.mountComponent(e,n,null,y(e,t),o,0);a&&console.timeEnd(a),e._renderedComponent._topLevelWrapper=e,V._mountImageIntoNode(u,t,e,r,n)}function s(e,t,n,r){var o=w.ReactReconcileTransaction.getPooled(!n&&b.useCreateElement);o.perform(i,null,e,t,o,n,r),w.ReactReconcileTransaction.release(o)}function u(e,t,n){for(T.unmountComponent(e,n),t.nodeType===D&&(t=t.documentElement);t.lastChild;)t.removeChild(t.lastChild)}function l(e){var t=o(e);if(t){var n=g.getInstanceFromNode(t);return!(!n||!n._hostParent)}}function c(e){return!(!e||e.nodeType!==I&&e.nodeType!==D&&e.nodeType!==A)}function p(e){var t=o(e),n=t&&g.getInstanceFromNode(t);return n&&!n._hostParent?n:null}function d(e){var t=p(e);return t?t._hostContainerInfo._topLevelWrapper:null}var f=e(132),h=e(8),m=e(10),v=e(27),g=(e(35),e(40)),y=e(41),b=e(44),C=e(56),_=e(61),E=e(65),x=(e(66),e(67)),T=e(80),N=e(87),w=e(88),P=e(147),k=e(127),M=(e(154),e(134)),S=e(136),R=(e(161),m.ID_ATTRIBUTE_NAME),O=m.ROOT_ATTRIBUTE_NAME,I=1,D=9,A=11,L={},U=1,F=function(){this.rootID=U++};F.prototype.isReactComponent={},F.prototype.render=function(){return this.props};var V={TopLevelWrapper:F,_instancesByReactRootID:L,scrollMonitor:function(e,t){t()},_updateRootComponent:function(e,t,n,r,o){return V.scrollMonitor(r,function(){N.enqueueElementInternal(e,t,n),o&&N.enqueueCallbackInternal(e,o)}),e},_renderNewRootComponent:function(e,t,n,r){c(t)?void 0:f("37"),v.ensureScrollValueMonitoring();var o=k(e,!1);w.batchedUpdates(s,o,t,n,r);var a=o._instance.rootID;return L[a]=o,o},renderSubtreeIntoContainer:function(e,t,n,r){return null!=e&&E.has(e)?void 0:f("38"),V._renderSubtreeIntoContainer(e,t,n,r)},_renderSubtreeIntoContainer:function(e,t,n,r){N.validateCallback(r,"ReactDOM.render"),C.isValidElement(t)?void 0:f("39","string"==typeof t?" Instead of passing a string like 'div', pass React.createElement('div') or <div />.":"function"==typeof t?" Instead of passing a class like Foo, pass React.createElement(Foo) or <Foo />.":null!=t&&void 0!==t.props?" This may be caused by unintentionally loading two independent copies of React.":"");var i,s=C(F,null,null,null,null,null,t);if(e){var u=E.get(e);i=u._processChildContext(u._context)}else i=P;var c=d(n);if(c){var p=c._currentElement,h=p.props;if(S(h,t)){var m=c._renderedComponent.getPublicInstance(),v=r&&function(){r.call(m)};return V._updateRootComponent(c,s,i,n,v),m}V.unmountComponentAtNode(n)}var g=o(n),y=g&&!!a(g),b=l(n),_=y&&!c&&!b,x=V._renderNewRootComponent(s,n,_,i)._renderedComponent.getPublicInstance();return r&&r.call(x),x},render:function(e,t,n){return V._renderSubtreeIntoContainer(null,e,t,n)},unmountComponentAtNode:function(e){c(e)?void 0:f("40");var t=d(e);return t?(delete L[t._instance.rootID],w.batchedUpdates(u,t,e,!1),!0):(l(e),1===e.nodeType&&e.hasAttribute(O),!1)},_mountImageIntoNode:function(e,t,n,a,i){if(c(t)?void 0:f("41"),a){var s=o(t);if(x.canReuseMarkup(e,s))return void g.precacheNode(n,s);var u=s.getAttribute(x.CHECKSUM_ATTR_NAME);s.removeAttribute(x.CHECKSUM_ATTR_NAME);var l=s.outerHTML;s.setAttribute(x.CHECKSUM_ATTR_NAME,u);var p=e,d=r(p,l),m=" (client) "+p.substring(d-20,d+20)+"\n (server) "+l.substring(d-20,d+20);t.nodeType===D?f("42",m):void 0}if(t.nodeType===D?f("43"):void 0,i.useCreateElement){for(;t.lastChild;)t.removeChild(t.lastChild);h.insertTreeBefore(t,e,null)}else M(t,e),g.precacheNode(n,t.firstChild)}};t.exports=V},{10:10,127:127,132:132,134:134,136:136,147:147,154:154,161:161,27:27,35:35,40:40,41:41,44:44,56:56,61:61,65:65,66:66,67:67,8:8,80:80,87:87,88:88}],69:[function(e,t,n){"use strict";function r(e,t,n){return{type:d.INSERT_MARKUP,content:e,fromIndex:null,fromNode:null,toIndex:n,afterNode:t}}function o(e,t,n){return{type:d.MOVE_EXISTING,content:null,fromIndex:e._mountIndex,fromNode:f.getHostNode(e),toIndex:n,afterNode:t}}function a(e,t){return{type:d.REMOVE_NODE,content:null,fromIndex:e._mountIndex,fromNode:t,toIndex:null,afterNode:null}}function i(e){return{type:d.SET_MARKUP,content:e,fromIndex:null,fromNode:null,toIndex:null,afterNode:null}}function s(e){return{type:d.TEXT_CONTENT,content:e,fromIndex:null,fromNode:null,toIndex:null,afterNode:null}}function u(e,t){return t&&(e=e||[],e.push(t)),e}function l(e,t){p.processChildrenUpdates(e,t)}var c=e(132),p=e(33),d=(e(65),e(66),e(70)),f=(e(35),e(80)),h=e(28),m=(e(146),e(116)),v=(e(154),{Mixin:{_reconcilerInstantiateChildren:function(e,t,n){return h.instantiateChildren(e,t,n)},_reconcilerUpdateChildren:function(e,t,n,r,o,a){var i,s=0;return i=m(t,s),h.updateChildren(e,i,n,r,o,this,this._hostContainerInfo,a,s),i},mountChildren:function(e,t,n){var r=this._reconcilerInstantiateChildren(e,t,n);this._renderedChildren=r;var o=[],a=0;for(var i in r)if(r.hasOwnProperty(i)){var s=r[i],u=0,l=f.mountComponent(s,t,this,this._hostContainerInfo,n,u);s._mountIndex=a++,o.push(l)}return o},updateTextContent:function(e){var t=this._renderedChildren;h.unmountChildren(t,!1);for(var n in t)t.hasOwnProperty(n)&&c("118");var r=[s(e)];l(this,r)},updateMarkup:function(e){var t=this._renderedChildren;h.unmountChildren(t,!1);for(var n in t)t.hasOwnProperty(n)&&c("118");var r=[i(e)];l(this,r)},updateChildren:function(e,t,n){this._updateChildren(e,t,n)},_updateChildren:function(e,t,n){var r=this._renderedChildren,o={},a=[],i=this._reconcilerUpdateChildren(r,e,a,o,t,n);if(i||r){var s,c=null,p=0,d=0,h=0,m=null;for(s in i)if(i.hasOwnProperty(s)){var v=r&&r[s],g=i[s];v===g?(c=u(c,this.moveChild(v,m,p,d)),d=Math.max(v._mountIndex,d),v._mountIndex=p):(v&&(d=Math.max(v._mountIndex,d)),c=u(c,this._mountChildAtIndex(g,a[h],m,p,t,n)),h++),p++,m=f.getHostNode(g)}for(s in o)o.hasOwnProperty(s)&&(c=u(c,this._unmountChild(r[s],o[s])));c&&l(this,c),this._renderedChildren=i}},unmountChildren:function(e){var t=this._renderedChildren;h.unmountChildren(t,e),this._renderedChildren=null},moveChild:function(e,t,n,r){if(e._mountIndex<r)return o(e,t,n)},createChild:function(e,t,n){return r(n,t,e._mountIndex)},removeChild:function(e,t){return a(e,t)},_mountChildAtIndex:function(e,t,n,r,o,a){return e._mountIndex=r,this.createChild(e,n,t)},_unmountChild:function(e,t){var n=this.removeChild(e,t);return e._mountIndex=null,n}}});t.exports=v},{116:116,132:132,146:146,154:154,28:28,33:33,35:35,65:65,66:66,70:70,80:80}],70:[function(e,t,n){"use strict";var r=e(157),o=r({INSERT_MARKUP:null,MOVE_EXISTING:null,REMOVE_NODE:null,SET_MARKUP:null,TEXT_CONTENT:null});t.exports=o},{157:157}],71:[function(e,t,n){"use strict";var r=e(132),o=e(56),a=(e(154),{HOST:0,COMPOSITE:1,EMPTY:2,getType:function(e){return null===e||e===!1?a.EMPTY:o.isValidElement(e)?"function"==typeof e.type?a.COMPOSITE:a.HOST:void r("26",e)}});t.exports=a},{132:132,154:154,56:56}],72:[function(e,t,n){"use strict";function r(e,t){}var o=(e(161),{isMounted:function(e){return!1},enqueueCallback:function(e,t){},enqueueForceUpdate:function(e){r(e,"forceUpdate")},enqueueReplaceState:function(e,t){r(e,"replaceState")},enqueueSetState:function(e,t){r(e,"setState")}});t.exports=o},{161:161}],73:[function(e,t,n){"use strict";var r=e(132),o=(e(154),{isValidOwner:function(e){return!(!e||"function"!=typeof e.attachRef||"function"!=typeof e.detachRef)},addComponentAsRefTo:function(e,t,n){o.isValidOwner(n)?void 0:r("119"),n.attachRef(t,e)},removeComponentAsRefFrom:function(e,t,n){o.isValidOwner(n)?void 0:r("120");var a=n.getPublicInstance();a&&a.refs[t]===e.getPublicInstance()&&n.detachRef(t)}});t.exports=o},{132:132,154:154}],74:[function(e,t,n){"use strict";var r={};t.exports=r},{}],75:[function(e,t,n){"use strict";var r=e(157),o=r({prop:null,context:null,childContext:null});t.exports=o},{157:157}],76:[function(e,t,n){"use strict";function r(e,t){return e===t?0!==e||1/e===1/t:e!==e&&t!==t}function o(e){this.message=e,this.stack=""}function a(e){function t(t,n,r,a,i,s,u){if(a=a||w,s=s||r,null==n[r]){var l=E[i];return t?new o("Required "+l+" `"+s+"` was not specified in "+("`"+a+"`.")):null}return e(n,r,a,i,s)}var n=t.bind(null,!1);return n.isRequired=t.bind(null,!0),n}function i(e){function t(t,n,r,a,i,s){var u=t[n],l=y(u);if(l!==e){var c=E[a],p=b(u);return new o("Invalid "+c+" `"+i+"` of type "+("`"+p+"` supplied to `"+r+"`, expected ")+("`"+e+"`."))}return null}return a(t)}function s(){return a(T.thatReturns(null))}function u(e){function t(t,n,r,a,i){if("function"!=typeof e)return new o("Property `"+i+"` of component `"+r+"` has invalid PropType notation inside arrayOf.");var s=t[n];if(!Array.isArray(s)){var u=E[a],l=y(s);return new o("Invalid "+u+" `"+i+"` of type "+("`"+l+"` supplied to `"+r+"`, expected an array."))}for(var c=0;c<s.length;c++){var p=e(s,c,r,a,i+"["+c+"]",x);if(p instanceof Error)return p}return null}return a(t)}function l(){function e(e,t,n,r,a){var i=e[t];if(!_.isValidElement(i)){var s=E[r],u=y(i);return new o("Invalid "+s+" `"+a+"` of type "+("`"+u+"` supplied to `"+n+"`, expected a single ReactElement."))}return null}return a(e)}function c(e){function t(t,n,r,a,i){if(!(t[n]instanceof e)){var s=E[a],u=e.name||w,l=C(t[n]);return new o("Invalid "+s+" `"+i+"` of type "+("`"+l+"` supplied to `"+r+"`, expected ")+("instance of `"+u+"`."))}return null}return a(t)}function p(e){function t(t,n,a,i,s){for(var u=t[n],l=0;l<e.length;l++)if(r(u,e[l]))return null;var c=E[i],p=JSON.stringify(e);return new o("Invalid "+c+" `"+s+"` of value `"+u+"` "+("supplied to `"+a+"`, expected one of "+p+"."))}return Array.isArray(e)?a(t):T.thatReturnsNull}function d(e){function t(t,n,r,a,i){if("function"!=typeof e)return new o("Property `"+i+"` of component `"+r+"` has invalid PropType notation inside objectOf.");var s=t[n],u=y(s);if("object"!==u){var l=E[a];return new o("Invalid "+l+" `"+i+"` of type "+("`"+u+"` supplied to `"+r+"`, expected an object."))}for(var c in s)if(s.hasOwnProperty(c)){var p=e(s,c,r,a,i+"."+c,x);if(p instanceof Error)return p}return null}return a(t)}function f(e){function t(t,n,r,a,i){for(var s=0;s<e.length;s++){var u=e[s];if(null==u(t,n,r,a,i,x))return null}var l=E[a];return new o("Invalid "+l+" `"+i+"` supplied to "+("`"+r+"`."))}return Array.isArray(e)?a(t):T.thatReturnsNull}function h(){function e(e,t,n,r,a){if(!v(e[t])){var i=E[r];return new o("Invalid "+i+" `"+a+"` supplied to "+("`"+n+"`, expected a ReactNode."))}return null}return a(e)}function m(e){function t(t,n,r,a,i){var s=t[n],u=y(s);if("object"!==u){var l=E[a];return new o("Invalid "+l+" `"+i+"` of type `"+u+"` "+("supplied to `"+r+"`, expected `object`."));
}for(var c in e){var p=e[c];if(p){var d=p(s,c,r,a,i+"."+c,x);if(d)return d}}return null}return a(t)}function v(e){switch(typeof e){case"number":case"string":case"undefined":return!0;case"boolean":return!e;case"object":if(Array.isArray(e))return e.every(v);if(null===e||_.isValidElement(e))return!0;var t=N(e);if(!t)return!1;var n,r=t.call(e);if(t!==e.entries){for(;!(n=r.next()).done;)if(!v(n.value))return!1}else for(;!(n=r.next()).done;){var o=n.value;if(o&&!v(o[1]))return!1}return!0;default:return!1}}function g(e,t){return"symbol"===e||"Symbol"===t["@@toStringTag"]||"function"==typeof Symbol&&t instanceof Symbol}function y(e){var t=typeof e;return Array.isArray(e)?"array":e instanceof RegExp?"object":g(t,e)?"symbol":t}function b(e){var t=y(e);if("object"===t){if(e instanceof Date)return"date";if(e instanceof RegExp)return"regexp"}return t}function C(e){return e.constructor&&e.constructor.name?e.constructor.name:w}var _=e(56),E=e(74),x=e(77),T=e(146),N=e(123),w=(e(161),"<<anonymous>>"),P={array:i("array"),bool:i("boolean"),func:i("function"),number:i("number"),object:i("object"),string:i("string"),symbol:i("symbol"),any:s(),arrayOf:u,element:l(),instanceOf:c,node:h(),objectOf:d,oneOf:p,oneOfType:f,shape:m};o.prototype=Error.prototype,t.exports=P},{123:123,146:146,161:161,56:56,74:74,77:77}],77:[function(e,t,n){"use strict";var r="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";t.exports=r},{}],78:[function(e,t,n){"use strict";function r(e,t,n){this.props=e,this.context=t,this.refs=u,this.updater=n||s}function o(){}var a=e(162),i=e(31),s=e(72),u=e(147);o.prototype=i.prototype,r.prototype=new o,r.prototype.constructor=r,a(r.prototype,i.prototype),r.prototype.isPureReactComponent=!0,t.exports=r},{147:147,162:162,31:31,72:72}],79:[function(e,t,n){"use strict";function r(e){this.reinitializeTransaction(),this.renderToStaticMarkup=!1,this.reactMountReady=a.getPooled(null),this.useCreateElement=e}var o=e(162),a=e(5),i=e(25),s=e(27),u=e(64),l=(e(66),e(106)),c=e(87),p={initialize:u.getSelectionInformation,close:u.restoreSelection},d={initialize:function(){var e=s.isEnabled();return s.setEnabled(!1),e},close:function(e){s.setEnabled(e)}},f={initialize:function(){this.reactMountReady.reset()},close:function(){this.reactMountReady.notifyAll()}},h=[p,d,f],m={getTransactionWrappers:function(){return h},getReactMountReady:function(){return this.reactMountReady},getUpdateQueue:function(){return c},checkpoint:function(){return this.reactMountReady.checkpoint()},rollback:function(e){this.reactMountReady.rollback(e)},destructor:function(){a.release(this.reactMountReady),this.reactMountReady=null}};o(r.prototype,l.Mixin,m),i.addPoolingTo(r),t.exports=r},{106:106,162:162,25:25,27:27,5:5,64:64,66:66,87:87}],80:[function(e,t,n){"use strict";function r(){o.attachRefs(this,this._currentElement)}var o=e(81),a=(e(66),e(161),{mountComponent:function(e,t,n,o,a,i){var s=e.mountComponent(t,n,o,a,i);return e._currentElement&&null!=e._currentElement.ref&&t.getReactMountReady().enqueue(r,e),s},getHostNode:function(e){return e.getHostNode()},unmountComponent:function(e,t){o.detachRefs(e,e._currentElement),e.unmountComponent(t)},receiveComponent:function(e,t,n,a){var i=e._currentElement;if(t!==i||a!==e._context){var s=o.shouldUpdateRefs(i,t);s&&o.detachRefs(e,i),e.receiveComponent(t,n,a),s&&e._currentElement&&null!=e._currentElement.ref&&n.getReactMountReady().enqueue(r,e)}},performUpdateIfNecessary:function(e,t,n){e._updateBatchNumber===n&&e.performUpdateIfNecessary(t)}});t.exports=a},{161:161,66:66,81:81}],81:[function(e,t,n){"use strict";function r(e,t,n){"function"==typeof e?e(t.getPublicInstance()):a.addComponentAsRefTo(t,e,n)}function o(e,t,n){"function"==typeof e?e(null):a.removeComponentAsRefFrom(t,e,n)}var a=e(73),i={};i.attachRefs=function(e,t){if(null!==t&&t!==!1){var n=t.ref;null!=n&&r(n,e,t._owner)}},i.shouldUpdateRefs=function(e,t){var n=null===e||e===!1,r=null===t||t===!1;return n||r||t.ref!==e.ref||"string"==typeof t.ref&&t._owner!==e._owner},i.detachRefs=function(e,t){if(null!==t&&t!==!1){var n=t.ref;null!=n&&o(n,e,t._owner)}},t.exports=i},{73:73}],82:[function(e,t,n){"use strict";var r={isBatchingUpdates:!1,batchedUpdates:function(e){}};t.exports=r},{}],83:[function(e,t,n){"use strict";function r(e,t){var n;try{return h.injection.injectBatchingStrategy(d),n=f.getPooled(t),g++,n.perform(function(){var r=v(e,!0),o=p.mountComponent(r,n,null,s(),m,0);return t||(o=c.addChecksumToMarkup(o)),o},null)}finally{g--,f.release(n),g||h.injection.injectBatchingStrategy(u)}}function o(e){return l.isValidElement(e)?void 0:i("46"),r(e,!1)}function a(e){return l.isValidElement(e)?void 0:i("47"),r(e,!0)}var i=e(132),s=e(41),u=e(54),l=e(56),c=(e(66),e(67)),p=e(80),d=e(82),f=e(84),h=e(88),m=e(147),v=e(127),g=(e(154),0);t.exports={renderToString:o,renderToStaticMarkup:a}},{127:127,132:132,147:147,154:154,41:41,54:54,56:56,66:66,67:67,80:80,82:82,84:84,88:88}],84:[function(e,t,n){"use strict";function r(e){this.reinitializeTransaction(),this.renderToStaticMarkup=e,this.useCreateElement=!1,this.updateQueue=new s(this)}var o=e(162),a=e(25),i=e(106),s=(e(66),e(85)),u=[],l={enqueue:function(){}},c={getTransactionWrappers:function(){return u},getReactMountReady:function(){return l},getUpdateQueue:function(){return this.updateQueue},destructor:function(){},checkpoint:function(){},rollback:function(){}};o(r.prototype,i.Mixin,c),a.addPoolingTo(r),t.exports=r},{106:106,162:162,25:25,66:66,85:85}],85:[function(e,t,n){"use strict";function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){}var a=e(87),i=(e(106),e(161),function(){function e(t){r(this,e),this.transaction=t}return e.prototype.isMounted=function(e){return!1},e.prototype.enqueueCallback=function(e,t,n){this.transaction.isInTransaction()&&a.enqueueCallback(e,t,n)},e.prototype.enqueueForceUpdate=function(e){this.transaction.isInTransaction()?a.enqueueForceUpdate(e):o(e,"forceUpdate")},e.prototype.enqueueReplaceState=function(e,t){this.transaction.isInTransaction()?a.enqueueReplaceState(e,t):o(e,"replaceState")},e.prototype.enqueueSetState=function(e,t){this.transaction.isInTransaction()?a.enqueueSetState(e,t):o(e,"setState")},e}());t.exports=i},{106:106,161:161,87:87}],86:[function(e,t,n){"use strict";var r=e(162),o=e(36),a=e(50),i=e(26),s=r({__SECRET_DOM_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:o,__SECRET_DOM_SERVER_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:a},i);t.exports=s},{162:162,26:26,36:36,50:50}],87:[function(e,t,n){"use strict";function r(e){u.enqueueUpdate(e)}function o(e){var t=typeof e;if("object"!==t)return t;var n=e.constructor&&e.constructor.name||t,r=Object.keys(e);return r.length>0&&r.length<20?n+" (keys: "+r.join(", ")+")":n}function a(e,t){var n=s.get(e);return n?n:null}var i=e(132),s=(e(35),e(65)),u=(e(66),e(88)),l=(e(154),e(161),{isMounted:function(e){var t=s.get(e);return!!t&&!!t._renderedComponent},enqueueCallback:function(e,t,n){l.validateCallback(t,n);var o=a(e);return o?(o._pendingCallbacks?o._pendingCallbacks.push(t):o._pendingCallbacks=[t],void r(o)):null},enqueueCallbackInternal:function(e,t){e._pendingCallbacks?e._pendingCallbacks.push(t):e._pendingCallbacks=[t],r(e)},enqueueForceUpdate:function(e){var t=a(e,"forceUpdate");t&&(t._pendingForceUpdate=!0,r(t))},enqueueReplaceState:function(e,t){var n=a(e,"replaceState");n&&(n._pendingStateQueue=[t],n._pendingReplaceState=!0,r(n))},enqueueSetState:function(e,t){var n=a(e,"setState");if(n){var o=n._pendingStateQueue||(n._pendingStateQueue=[]);o.push(t),r(n)}},enqueueElementInternal:function(e,t,n){e._pendingElement=t,e._context=n,r(e)},validateCallback:function(e,t){e&&"function"!=typeof e?i("122",t,o(e)):void 0}});t.exports=l},{132:132,154:154,161:161,35:35,65:65,66:66,88:88}],88:[function(e,t,n){"use strict";function r(){P.ReactReconcileTransaction&&_?void 0:c("123")}function o(){this.reinitializeTransaction(),this.dirtyComponentsLength=null,this.callbackQueue=d.getPooled(),this.reconcileTransaction=P.ReactReconcileTransaction.getPooled(!0)}function a(e,t,n,o,a,i){r(),_.batchedUpdates(e,t,n,o,a,i)}function i(e,t){return e._mountOrder-t._mountOrder}function s(e){var t=e.dirtyComponentsLength;t!==g.length?c("124",t,g.length):void 0,g.sort(i),y++;for(var n=0;n<t;n++){var r=g[n],o=r._pendingCallbacks;r._pendingCallbacks=null;var a;if(h.logTopLevelRenders){var s=r;r._currentElement.props===r._renderedComponent._currentElement&&(s=r._renderedComponent),a="React update: "+s.getName(),console.time(a)}if(m.performUpdateIfNecessary(r,e.reconcileTransaction,y),a&&console.timeEnd(a),o)for(var u=0;u<o.length;u++)e.callbackQueue.enqueue(o[u],r.getPublicInstance())}}function u(e){return r(),_.isBatchingUpdates?(g.push(e),void(null==e._updateBatchNumber&&(e._updateBatchNumber=y+1))):void _.batchedUpdates(u,e)}function l(e,t){_.isBatchingUpdates?void 0:c("125"),b.enqueue(e,t),C=!0}var c=e(132),p=e(162),d=e(5),f=e(25),h=e(61),m=e(80),v=e(106),g=(e(154),[]),y=0,b=d.getPooled(),C=!1,_=null,E={initialize:function(){this.dirtyComponentsLength=g.length},close:function(){this.dirtyComponentsLength!==g.length?(g.splice(0,this.dirtyComponentsLength),N()):g.length=0}},x={initialize:function(){this.callbackQueue.reset()},close:function(){this.callbackQueue.notifyAll()}},T=[E,x];p(o.prototype,v.Mixin,{getTransactionWrappers:function(){return T},destructor:function(){this.dirtyComponentsLength=null,d.release(this.callbackQueue),this.callbackQueue=null,P.ReactReconcileTransaction.release(this.reconcileTransaction),this.reconcileTransaction=null},perform:function(e,t,n){return v.Mixin.perform.call(this,this.reconcileTransaction.perform,this.reconcileTransaction,e,t,n)}}),f.addPoolingTo(o);var N=function(){for(;g.length||C;){if(g.length){var e=o.getPooled();e.perform(s,null,e),o.release(e)}if(C){C=!1;var t=b;b=d.getPooled(),t.notifyAll(),d.release(t)}}},w={injectReconcileTransaction:function(e){e?void 0:c("126"),P.ReactReconcileTransaction=e},injectBatchingStrategy:function(e){e?void 0:c("127"),"function"!=typeof e.batchedUpdates?c("128"):void 0,"boolean"!=typeof e.isBatchingUpdates?c("129"):void 0,_=e}},P={ReactReconcileTransaction:null,batchedUpdates:a,enqueueUpdate:u,flushBatchedUpdates:N,injection:w,asap:l};t.exports=P},{106:106,132:132,154:154,162:162,25:25,5:5,61:61,80:80}],89:[function(e,t,n){"use strict";t.exports="15.3.1"},{}],90:[function(e,t,n){"use strict";var r={xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace"},o={accentHeight:"accent-height",accumulate:0,additive:0,alignmentBaseline:"alignment-baseline",allowReorder:"allowReorder",alphabetic:0,amplitude:0,arabicForm:"arabic-form",ascent:0,attributeName:"attributeName",attributeType:"attributeType",autoReverse:"autoReverse",azimuth:0,baseFrequency:"baseFrequency",baseProfile:"baseProfile",baselineShift:"baseline-shift",bbox:0,begin:0,bias:0,by:0,calcMode:"calcMode",capHeight:"cap-height",clip:0,clipPath:"clip-path",clipRule:"clip-rule",clipPathUnits:"clipPathUnits",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",contentScriptType:"contentScriptType",contentStyleType:"contentStyleType",cursor:0,cx:0,cy:0,d:0,decelerate:0,descent:0,diffuseConstant:"diffuseConstant",direction:0,display:0,divisor:0,dominantBaseline:"dominant-baseline",dur:0,dx:0,dy:0,edgeMode:"edgeMode",elevation:0,enableBackground:"enable-background",end:0,exponent:0,externalResourcesRequired:"externalResourcesRequired",fill:0,fillOpacity:"fill-opacity",fillRule:"fill-rule",filter:0,filterRes:"filterRes",filterUnits:"filterUnits",floodColor:"flood-color",floodOpacity:"flood-opacity",focusable:0,fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",format:0,from:0,fx:0,fy:0,g1:0,g2:0,glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",glyphRef:"glyphRef",gradientTransform:"gradientTransform",gradientUnits:"gradientUnits",hanging:0,horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",ideographic:0,imageRendering:"image-rendering",in:0,in2:0,intercept:0,k:0,k1:0,k2:0,k3:0,k4:0,kernelMatrix:"kernelMatrix",kernelUnitLength:"kernelUnitLength",kerning:0,keyPoints:"keyPoints",keySplines:"keySplines",keyTimes:"keyTimes",lengthAdjust:"lengthAdjust",letterSpacing:"letter-spacing",lightingColor:"lighting-color",limitingConeAngle:"limitingConeAngle",local:0,markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",markerHeight:"markerHeight",markerUnits:"markerUnits",markerWidth:"markerWidth",mask:0,maskContentUnits:"maskContentUnits",maskUnits:"maskUnits",mathematical:0,mode:0,numOctaves:"numOctaves",offset:0,opacity:0,operator:0,order:0,orient:0,orientation:0,origin:0,overflow:0,overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pathLength:"pathLength",patternContentUnits:"patternContentUnits",patternTransform:"patternTransform",patternUnits:"patternUnits",pointerEvents:"pointer-events",points:0,pointsAtX:"pointsAtX",pointsAtY:"pointsAtY",pointsAtZ:"pointsAtZ",preserveAlpha:"preserveAlpha",preserveAspectRatio:"preserveAspectRatio",primitiveUnits:"primitiveUnits",r:0,radius:0,refX:"refX",refY:"refY",renderingIntent:"rendering-intent",repeatCount:"repeatCount",repeatDur:"repeatDur",requiredExtensions:"requiredExtensions",requiredFeatures:"requiredFeatures",restart:0,result:0,rotate:0,rx:0,ry:0,scale:0,seed:0,shapeRendering:"shape-rendering",slope:0,spacing:0,specularConstant:"specularConstant",specularExponent:"specularExponent",speed:0,spreadMethod:"spreadMethod",startOffset:"startOffset",stdDeviation:"stdDeviation",stemh:0,stemv:0,stitchTiles:"stitchTiles",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",string:0,stroke:0,strokeDasharray:"stroke-dasharray",strokeDashoffset:"stroke-dashoffset",strokeLinecap:"stroke-linecap",strokeLinejoin:"stroke-linejoin",strokeMiterlimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",surfaceScale:"surfaceScale",systemLanguage:"systemLanguage",tableValues:"tableValues",targetX:"targetX",targetY:"targetY",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",textLength:"textLength",to:0,transform:0,u1:0,u2:0,underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicode:0,unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",values:0,vectorEffect:"vector-effect",version:0,vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",viewBox:"viewBox",viewTarget:"viewTarget",visibility:0,widths:0,wordSpacing:"word-spacing",writingMode:"writing-mode",x:0,xHeight:"x-height",x1:0,x2:0,xChannelSelector:"xChannelSelector",xlinkActuate:"xlink:actuate",xlinkArcrole:"xlink:arcrole",xlinkHref:"xlink:href",xlinkRole:"xlink:role",xlinkShow:"xlink:show",xlinkTitle:"xlink:title",xlinkType:"xlink:type",xmlBase:"xml:base",xmlns:0,xmlnsXlink:"xmlns:xlink",xmlLang:"xml:lang",xmlSpace:"xml:space",y:0,y1:0,y2:0,yChannelSelector:"yChannelSelector",z:0,zoomAndPan:"zoomAndPan"},a={Properties:{},DOMAttributeNamespaces:{xlinkActuate:r.xlink,xlinkArcrole:r.xlink,xlinkHref:r.xlink,xlinkRole:r.xlink,xlinkShow:r.xlink,xlinkTitle:r.xlink,xlinkType:r.xlink,xmlBase:r.xml,xmlLang:r.xml,xmlSpace:r.xml},DOMAttributeNames:{}};Object.keys(o).forEach(function(e){a.Properties[e]=0,o[e]&&(a.DOMAttributeNames[e]=o[e])}),t.exports=a},{}],91:[function(e,t,n){"use strict";function r(e){if("selectionStart"in e&&l.hasSelectionCapabilities(e))return{start:e.selectionStart,end:e.selectionEnd};if(window.getSelection){var t=window.getSelection();return{anchorNode:t.anchorNode,anchorOffset:t.anchorOffset,focusNode:t.focusNode,focusOffset:t.focusOffset}}if(document.selection){var n=document.selection.createRange();return{parentElement:n.parentElement(),text:n.text,top:n.boundingTop,left:n.boundingLeft}}}function o(e,t){if(_||null==y||y!==p())return null;var n=r(y);if(!C||!h(C,n)){C=n;var o=c.getPooled(g.select,b,e,t);return o.type="select",o.target=y,i.accumulateTwoPhaseDispatches(o),o}return null}var a=e(16),i=e(20),s=e(140),u=e(40),l=e(64),c=e(97),p=e(149),d=e(129),f=e(158),h=e(160),m=a.topLevelTypes,v=s.canUseDOM&&"documentMode"in document&&document.documentMode<=11,g={select:{phasedRegistrationNames:{bubbled:f({onSelect:null}),captured:f({onSelectCapture:null})},dependencies:[m.topBlur,m.topContextMenu,m.topFocus,m.topKeyDown,m.topMouseDown,m.topMouseUp,m.topSelectionChange]}},y=null,b=null,C=null,_=!1,E=!1,x=f({onSelect:null}),T={eventTypes:g,extractEvents:function(e,t,n,r){if(!E)return null;var a=t?u.getNodeFromInstance(t):window;switch(e){case m.topFocus:(d(a)||"true"===a.contentEditable)&&(y=a,b=t,C=null);break;case m.topBlur:y=null,b=null,C=null;break;case m.topMouseDown:_=!0;break;case m.topContextMenu:case m.topMouseUp:return _=!1,o(n,r);case m.topSelectionChange:if(v)break;case m.topKeyDown:case m.topKeyUp:return o(n,r)}return null},didPutListener:function(e,t,n){t===x&&(E=!0)}};t.exports=T},{129:129,140:140,149:149,158:158,16:16,160:160,20:20,40:40,64:64,97:97}],92:[function(e,t,n){"use strict";function r(e){return"."+e._rootNodeID}var o=e(132),a=e(16),i=e(139),s=e(20),u=e(40),l=e(93),c=e(94),p=e(97),d=e(98),f=e(100),h=e(101),m=e(96),v=e(102),g=e(103),y=e(104),b=e(105),C=e(146),_=e(118),E=(e(154),e(158)),x=a.topLevelTypes,T={abort:{phasedRegistrationNames:{bubbled:E({onAbort:!0}),captured:E({onAbortCapture:!0})}},animationEnd:{phasedRegistrationNames:{bubbled:E({onAnimationEnd:!0}),captured:E({onAnimationEndCapture:!0})}},animationIteration:{phasedRegistrationNames:{bubbled:E({onAnimationIteration:!0}),captured:E({onAnimationIterationCapture:!0})}},animationStart:{phasedRegistrationNames:{bubbled:E({onAnimationStart:!0}),captured:E({onAnimationStartCapture:!0})}},blur:{phasedRegistrationNames:{bubbled:E({onBlur:!0}),captured:E({onBlurCapture:!0})}},canPlay:{phasedRegistrationNames:{bubbled:E({onCanPlay:!0}),captured:E({onCanPlayCapture:!0})}},canPlayThrough:{phasedRegistrationNames:{bubbled:E({onCanPlayThrough:!0}),captured:E({onCanPlayThroughCapture:!0})}},click:{phasedRegistrationNames:{bubbled:E({onClick:!0}),captured:E({onClickCapture:!0})}},contextMenu:{phasedRegistrationNames:{bubbled:E({onContextMenu:!0}),captured:E({onContextMenuCapture:!0})}},copy:{phasedRegistrationNames:{bubbled:E({onCopy:!0}),captured:E({onCopyCapture:!0})}},cut:{phasedRegistrationNames:{bubbled:E({onCut:!0}),captured:E({onCutCapture:!0})}},doubleClick:{phasedRegistrationNames:{bubbled:E({onDoubleClick:!0}),captured:E({onDoubleClickCapture:!0})}},drag:{phasedRegistrationNames:{bubbled:E({onDrag:!0}),captured:E({onDragCapture:!0})}},dragEnd:{phasedRegistrationNames:{bubbled:E({onDragEnd:!0}),captured:E({onDragEndCapture:!0})}},dragEnter:{phasedRegistrationNames:{bubbled:E({onDragEnter:!0}),captured:E({onDragEnterCapture:!0})}},dragExit:{phasedRegistrationNames:{bubbled:E({onDragExit:!0}),captured:E({onDragExitCapture:!0})}},dragLeave:{phasedRegistrationNames:{bubbled:E({onDragLeave:!0}),captured:E({onDragLeaveCapture:!0})}},dragOver:{phasedRegistrationNames:{bubbled:E({onDragOver:!0}),captured:E({onDragOverCapture:!0})}},dragStart:{phasedRegistrationNames:{bubbled:E({onDragStart:!0}),captured:E({onDragStartCapture:!0})}},drop:{phasedRegistrationNames:{bubbled:E({onDrop:!0}),captured:E({onDropCapture:!0})}},durationChange:{phasedRegistrationNames:{bubbled:E({onDurationChange:!0}),captured:E({onDurationChangeCapture:!0})}},emptied:{phasedRegistrationNames:{bubbled:E({onEmptied:!0}),captured:E({onEmptiedCapture:!0})}},encrypted:{phasedRegistrationNames:{bubbled:E({onEncrypted:!0}),captured:E({onEncryptedCapture:!0})}},ended:{phasedRegistrationNames:{bubbled:E({onEnded:!0}),captured:E({onEndedCapture:!0})}},error:{phasedRegistrationNames:{bubbled:E({onError:!0}),captured:E({onErrorCapture:!0})}},focus:{phasedRegistrationNames:{bubbled:E({onFocus:!0}),captured:E({onFocusCapture:!0})}},input:{phasedRegistrationNames:{bubbled:E({onInput:!0}),captured:E({onInputCapture:!0})}},invalid:{phasedRegistrationNames:{bubbled:E({onInvalid:!0}),captured:E({onInvalidCapture:!0})}},keyDown:{phasedRegistrationNames:{bubbled:E({onKeyDown:!0}),captured:E({onKeyDownCapture:!0})}},keyPress:{phasedRegistrationNames:{bubbled:E({onKeyPress:!0}),captured:E({onKeyPressCapture:!0})}},keyUp:{phasedRegistrationNames:{bubbled:E({onKeyUp:!0}),captured:E({onKeyUpCapture:!0})}},load:{phasedRegistrationNames:{bubbled:E({onLoad:!0}),captured:E({onLoadCapture:!0})}},loadedData:{phasedRegistrationNames:{bubbled:E({onLoadedData:!0}),captured:E({onLoadedDataCapture:!0})}},loadedMetadata:{phasedRegistrationNames:{bubbled:E({onLoadedMetadata:!0}),captured:E({onLoadedMetadataCapture:!0})}},loadStart:{phasedRegistrationNames:{bubbled:E({onLoadStart:!0}),captured:E({onLoadStartCapture:!0})}},mouseDown:{phasedRegistrationNames:{bubbled:E({onMouseDown:!0}),captured:E({onMouseDownCapture:!0})}},mouseMove:{phasedRegistrationNames:{bubbled:E({onMouseMove:!0}),captured:E({onMouseMoveCapture:!0})}},mouseOut:{phasedRegistrationNames:{bubbled:E({onMouseOut:!0}),captured:E({onMouseOutCapture:!0})}},mouseOver:{phasedRegistrationNames:{bubbled:E({onMouseOver:!0}),captured:E({onMouseOverCapture:!0})}},mouseUp:{phasedRegistrationNames:{bubbled:E({onMouseUp:!0}),captured:E({onMouseUpCapture:!0})}},paste:{phasedRegistrationNames:{bubbled:E({onPaste:!0}),captured:E({onPasteCapture:!0})}},pause:{phasedRegistrationNames:{bubbled:E({onPause:!0}),captured:E({onPauseCapture:!0})}},play:{phasedRegistrationNames:{bubbled:E({onPlay:!0}),captured:E({onPlayCapture:!0})}},playing:{phasedRegistrationNames:{bubbled:E({onPlaying:!0}),captured:E({onPlayingCapture:!0})}},progress:{phasedRegistrationNames:{bubbled:E({onProgress:!0}),captured:E({onProgressCapture:!0})}},rateChange:{phasedRegistrationNames:{bubbled:E({onRateChange:!0}),captured:E({onRateChangeCapture:!0})}},reset:{phasedRegistrationNames:{bubbled:E({onReset:!0}),captured:E({onResetCapture:!0})}},scroll:{phasedRegistrationNames:{bubbled:E({onScroll:!0}),captured:E({onScrollCapture:!0})}},seeked:{phasedRegistrationNames:{bubbled:E({onSeeked:!0}),captured:E({onSeekedCapture:!0})}},seeking:{phasedRegistrationNames:{bubbled:E({onSeeking:!0}),captured:E({onSeekingCapture:!0})}},stalled:{phasedRegistrationNames:{bubbled:E({onStalled:!0}),captured:E({onStalledCapture:!0})}},submit:{phasedRegistrationNames:{bubbled:E({onSubmit:!0}),captured:E({onSubmitCapture:!0})}},suspend:{phasedRegistrationNames:{bubbled:E({onSuspend:!0}),captured:E({onSuspendCapture:!0})}},timeUpdate:{phasedRegistrationNames:{bubbled:E({onTimeUpdate:!0}),captured:E({onTimeUpdateCapture:!0})}},touchCancel:{phasedRegistrationNames:{bubbled:E({onTouchCancel:!0}),captured:E({onTouchCancelCapture:!0})}},touchEnd:{phasedRegistrationNames:{bubbled:E({onTouchEnd:!0}),captured:E({onTouchEndCapture:!0})}},touchMove:{phasedRegistrationNames:{bubbled:E({onTouchMove:!0}),captured:E({onTouchMoveCapture:!0})}},touchStart:{phasedRegistrationNames:{bubbled:E({onTouchStart:!0}),captured:E({onTouchStartCapture:!0})}},transitionEnd:{phasedRegistrationNames:{bubbled:E({onTransitionEnd:!0}),captured:E({onTransitionEndCapture:!0})}},volumeChange:{phasedRegistrationNames:{bubbled:E({onVolumeChange:!0}),captured:E({onVolumeChangeCapture:!0})}},waiting:{phasedRegistrationNames:{bubbled:E({onWaiting:!0}),captured:E({onWaitingCapture:!0})}},wheel:{phasedRegistrationNames:{bubbled:E({onWheel:!0}),captured:E({onWheelCapture:!0})}}},N={topAbort:T.abort,topAnimationEnd:T.animationEnd,topAnimationIteration:T.animationIteration,topAnimationStart:T.animationStart,topBlur:T.blur,topCanPlay:T.canPlay,topCanPlayThrough:T.canPlayThrough,topClick:T.click,topContextMenu:T.contextMenu,topCopy:T.copy,topCut:T.cut,topDoubleClick:T.doubleClick,topDrag:T.drag,topDragEnd:T.dragEnd,topDragEnter:T.dragEnter,topDragExit:T.dragExit,topDragLeave:T.dragLeave,topDragOver:T.dragOver,topDragStart:T.dragStart,topDrop:T.drop,topDurationChange:T.durationChange,topEmptied:T.emptied,topEncrypted:T.encrypted,topEnded:T.ended,topError:T.error,topFocus:T.focus,topInput:T.input,topInvalid:T.invalid,topKeyDown:T.keyDown,topKeyPress:T.keyPress,topKeyUp:T.keyUp,topLoad:T.load,topLoadedData:T.loadedData,topLoadedMetadata:T.loadedMetadata,topLoadStart:T.loadStart,topMouseDown:T.mouseDown,topMouseMove:T.mouseMove,topMouseOut:T.mouseOut,topMouseOver:T.mouseOver,topMouseUp:T.mouseUp,topPaste:T.paste,topPause:T.pause,topPlay:T.play,topPlaying:T.playing,topProgress:T.progress,topRateChange:T.rateChange,topReset:T.reset,topScroll:T.scroll,topSeeked:T.seeked,topSeeking:T.seeking,topStalled:T.stalled,topSubmit:T.submit,topSuspend:T.suspend,topTimeUpdate:T.timeUpdate,topTouchCancel:T.touchCancel,topTouchEnd:T.touchEnd,topTouchMove:T.touchMove,topTouchStart:T.touchStart,topTransitionEnd:T.transitionEnd,topVolumeChange:T.volumeChange,topWaiting:T.waiting,topWheel:T.wheel};for(var w in N)N[w].dependencies=[w];var P=E({onClick:null}),k={},M={eventTypes:T,extractEvents:function(e,t,n,r){var a=N[e];if(!a)return null;var i;switch(e){case x.topAbort:case x.topCanPlay:case x.topCanPlayThrough:case x.topDurationChange:case x.topEmptied:case x.topEncrypted:case x.topEnded:case x.topError:case x.topInput:case x.topInvalid:case x.topLoad:case x.topLoadedData:case x.topLoadedMetadata:case x.topLoadStart:case x.topPause:case x.topPlay:case x.topPlaying:case x.topProgress:case x.topRateChange:case x.topReset:case x.topSeeked:case x.topSeeking:case x.topStalled:case x.topSubmit:case x.topSuspend:case x.topTimeUpdate:case x.topVolumeChange:case x.topWaiting:i=p;break;case x.topKeyPress:if(0===_(n))return null;case x.topKeyDown:case x.topKeyUp:i=f;break;case x.topBlur:case x.topFocus:i=d;break;case x.topClick:if(2===n.button)return null;case x.topContextMenu:case x.topDoubleClick:case x.topMouseDown:case x.topMouseMove:case x.topMouseOut:case x.topMouseOver:case x.topMouseUp:i=h;break;case x.topDrag:case x.topDragEnd:case x.topDragEnter:case x.topDragExit:case x.topDragLeave:case x.topDragOver:case x.topDragStart:case x.topDrop:i=m;break;case x.topTouchCancel:case x.topTouchEnd:case x.topTouchMove:case x.topTouchStart:i=v;break;case x.topAnimationEnd:case x.topAnimationIteration:case x.topAnimationStart:i=l;break;case x.topTransitionEnd:i=g;break;case x.topScroll:i=y;break;case x.topWheel:i=b;break;case x.topCopy:case x.topCut:case x.topPaste:i=c}i?void 0:o("86",e);var u=i.getPooled(a,t,n,r);return s.accumulateTwoPhaseDispatches(u),u},didPutListener:function(e,t,n){if(t===P){var o=r(e),a=u.getNodeFromInstance(e);k[o]||(k[o]=i.listen(a,"click",C))}},willDeleteListener:function(e,t){if(t===P){var n=r(e);k[n].remove(),delete k[n]}}};t.exports=M},{100:100,101:101,102:102,103:103,104:104,105:105,118:118,132:132,139:139,146:146,154:154,158:158,16:16,20:20,40:40,93:93,94:94,96:96,97:97,98:98}],93:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={animationName:null,elapsedTime:null,pseudoElement:null};o.augmentClass(r,a),t.exports=r},{97:97}],94:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}};o.augmentClass(r,a),t.exports=r},{97:97}],95:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={data:null};o.augmentClass(r,a),t.exports=r},{97:97}],96:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(101),a={dataTransfer:null};o.augmentClass(r,a),t.exports=r},{101:101}],97:[function(e,t,n){"use strict";function r(e,t,n,r){this.dispatchConfig=e,this._targetInst=t,this.nativeEvent=n;var o=this.constructor.Interface;for(var a in o)if(o.hasOwnProperty(a)){var s=o[a];s?this[a]=s(n):"target"===a?this.target=r:this[a]=n[a]}var u=null!=n.defaultPrevented?n.defaultPrevented:n.returnValue===!1;return u?this.isDefaultPrevented=i.thatReturnsTrue:this.isDefaultPrevented=i.thatReturnsFalse,this.isPropagationStopped=i.thatReturnsFalse,this}var o=e(162),a=e(25),i=e(146),s=(e(161),"function"==typeof Proxy,["dispatchConfig","_targetInst","nativeEvent","isDefaultPrevented","isPropagationStopped","_dispatchListeners","_dispatchInstances"]),u={type:null,target:null,currentTarget:i.thatReturnsNull,eventPhase:null,bubbles:null,cancelable:null,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:null,isTrusted:null};o(r.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():e.returnValue=!1,this.isDefaultPrevented=i.thatReturnsTrue)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():"unknown"!=typeof e.cancelBubble&&(e.cancelBubble=!0),this.isPropagationStopped=i.thatReturnsTrue)},persist:function(){this.isPersistent=i.thatReturnsTrue},isPersistent:i.thatReturnsFalse,destructor:function(){var e=this.constructor.Interface;for(var t in e)this[t]=null;for(var n=0;n<s.length;n++)this[s[n]]=null}}),r.Interface=u,r.augmentClass=function(e,t){var n=this,r=function(){};r.prototype=n.prototype;var i=new r;o(i,e.prototype),e.prototype=i,e.prototype.constructor=e,e.Interface=o({},n.Interface,t),e.augmentClass=n.augmentClass,a.addPoolingTo(e,a.fourArgumentPooler)},a.addPoolingTo(r,a.fourArgumentPooler),t.exports=r},{146:146,161:161,162:162,25:25}],98:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a={relatedTarget:null};o.augmentClass(r,a),t.exports=r},{104:104}],99:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={data:null};o.augmentClass(r,a),t.exports=r},{97:97}],100:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(118),i=e(119),s=e(120),u={key:i,location:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,repeat:null,locale:null,getModifierState:s,charCode:function(e){return"keypress"===e.type?a(e):0},keyCode:function(e){return"keydown"===e.type||"keyup"===e.type?e.keyCode:0},which:function(e){return"keypress"===e.type?a(e):"keydown"===e.type||"keyup"===e.type?e.keyCode:0}};o.augmentClass(r,u),t.exports=r},{104:104,118:118,119:119,120:120}],101:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(107),i=e(120),s={screenX:null,screenY:null,clientX:null,clientY:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,getModifierState:i,button:function(e){var t=e.button;return"which"in e?t:2===t?2:4===t?1:0},buttons:null,relatedTarget:function(e){return e.relatedTarget||(e.fromElement===e.srcElement?e.toElement:e.fromElement)},pageX:function(e){return"pageX"in e?e.pageX:e.clientX+a.currentScrollLeft},pageY:function(e){return"pageY"in e?e.pageY:e.clientY+a.currentScrollTop}};o.augmentClass(r,s),t.exports=r},{104:104,107:107,120:120}],102:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(120),i={touches:null,targetTouches:null,changedTouches:null,altKey:null,metaKey:null,ctrlKey:null,shiftKey:null,getModifierState:a};o.augmentClass(r,i),t.exports=r},{104:104,120:120}],103:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={propertyName:null,elapsedTime:null,pseudoElement:null};o.augmentClass(r,a),t.exports=r},{97:97}],104:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a=e(121),i={view:function(e){if(e.view)return e.view;var t=a(e);if(t.window===t)return t;var n=t.ownerDocument;return n?n.defaultView||n.parentWindow:window},detail:function(e){return e.detail||0}};o.augmentClass(r,i),
t.exports=r},{121:121,97:97}],105:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(101),a={deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:null,deltaMode:null};o.augmentClass(r,a),t.exports=r},{101:101}],106:[function(e,t,n){"use strict";var r=e(132),o=(e(154),{reinitializeTransaction:function(){this.transactionWrappers=this.getTransactionWrappers(),this.wrapperInitData?this.wrapperInitData.length=0:this.wrapperInitData=[],this._isInTransaction=!1},_isInTransaction:!1,getTransactionWrappers:null,isInTransaction:function(){return!!this._isInTransaction},perform:function(e,t,n,o,a,i,s,u){this.isInTransaction()?r("27"):void 0;var l,c;try{this._isInTransaction=!0,l=!0,this.initializeAll(0),c=e.call(t,n,o,a,i,s,u),l=!1}finally{try{if(l)try{this.closeAll(0)}catch(e){}else this.closeAll(0)}finally{this._isInTransaction=!1}}return c},initializeAll:function(e){for(var t=this.transactionWrappers,n=e;n<t.length;n++){var r=t[n];try{this.wrapperInitData[n]=a.OBSERVED_ERROR,this.wrapperInitData[n]=r.initialize?r.initialize.call(this):null}finally{if(this.wrapperInitData[n]===a.OBSERVED_ERROR)try{this.initializeAll(n+1)}catch(e){}}}},closeAll:function(e){this.isInTransaction()?void 0:r("28");for(var t=this.transactionWrappers,n=e;n<t.length;n++){var o,i=t[n],s=this.wrapperInitData[n];try{o=!0,s!==a.OBSERVED_ERROR&&i.close&&i.close.call(this,s),o=!1}finally{if(o)try{this.closeAll(n+1)}catch(e){}}}this.wrapperInitData.length=0}}),a={Mixin:o,OBSERVED_ERROR:{}};t.exports=a},{132:132,154:154}],107:[function(e,t,n){"use strict";var r={currentScrollLeft:0,currentScrollTop:0,refreshScrollValues:function(e){r.currentScrollLeft=e.x,r.currentScrollTop=e.y}};t.exports=r},{}],108:[function(e,t,n){"use strict";function r(e,t){return null==t?o("30"):void 0,null==e?t:Array.isArray(e)?Array.isArray(t)?(e.push.apply(e,t),e):(e.push(t),e):Array.isArray(t)?[e].concat(t):[e,t]}var o=e(132);e(154);t.exports=r},{132:132,154:154}],109:[function(e,t,n){"use strict";function r(e){for(var t=1,n=0,r=0,a=e.length,i=a&-4;r<i;){for(var s=Math.min(r+4096,i);r<s;r+=4)n+=(t+=e.charCodeAt(r))+(t+=e.charCodeAt(r+1))+(t+=e.charCodeAt(r+2))+(t+=e.charCodeAt(r+3));t%=o,n%=o}for(;r<a;r++)n+=t+=e.charCodeAt(r);return t%=o,n%=o,t|n<<16}var o=65521;t.exports=r},{}],110:[function(e,t,n){"use strict";var r=!1;t.exports=r},{}],111:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r,u,l){for(var c in e)if(e.hasOwnProperty(c)){var p;try{"function"!=typeof e[c]?o("84",r||"React class",a[n],c):void 0,p=e[c](t,c,r,n,null,i)}catch(e){p=e}p instanceof Error&&!(p.message in s)&&(s[p.message]=!0)}}var o=e(132),a=e(74),i=e(77);e(154),e(161);"undefined"!=typeof n&&n.env,1;var s={};t.exports=r}).call(this,void 0)},{132:132,154:154,161:161,74:74,77:77}],112:[function(e,t,n){"use strict";var r=function(e){return"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(t,n,r,o){MSApp.execUnsafeLocalFunction(function(){return e(t,n,r,o)})}:e};t.exports=r},{}],113:[function(e,t,n){"use strict";function r(e,t,n){var r=null==t||"boolean"==typeof t||""===t;if(r)return"";var o=isNaN(t);return o||0===t||a.hasOwnProperty(e)&&a[e]?""+t:("string"==typeof t&&(t=t.trim()),t+"px")}var o=e(3),a=(e(161),o.isUnitlessNumber);t.exports=r},{161:161,3:3}],114:[function(e,t,n){"use strict";function r(e){var t=""+e,n=a.exec(t);if(!n)return t;var r,o="",i=0,s=0;for(i=n.index;i<t.length;i++){switch(t.charCodeAt(i)){case 34:r="&quot;";break;case 38:r="&amp;";break;case 39:r="&#x27;";break;case 60:r="&lt;";break;case 62:r="&gt;";break;default:continue}s!==i&&(o+=t.substring(s,i)),s=i+1,o+=r}return s!==i?o+t.substring(s,i):o}function o(e){return"boolean"==typeof e||"number"==typeof e?""+e:r(e)}var a=/["'&<>]/;t.exports=o},{}],115:[function(e,t,n){"use strict";function r(e){if(null==e)return null;if(1===e.nodeType)return e;var t=i.get(e);return t?(t=s(t),t?a.getNodeFromInstance(t):null):void("function"==typeof e.render?o("44"):o("45",Object.keys(e)))}var o=e(132),a=(e(35),e(40)),i=e(65),s=e(122);e(154),e(161);t.exports=r},{122:122,132:132,154:154,161:161,35:35,40:40,65:65}],116:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r){if(e&&"object"==typeof e){var o=e,a=void 0===o[n];a&&null!=t&&(o[n]=t)}}function o(e,t){if(null==e)return e;var n={};return a(e,r,n),n}var a=(e(23),e(137));e(161);"undefined"!=typeof n&&n.env,t.exports=o}).call(this,void 0)},{137:137,161:161,23:23}],117:[function(e,t,n){"use strict";function r(e,t,n){Array.isArray(e)?e.forEach(t,n):e&&t.call(n,e)}t.exports=r},{}],118:[function(e,t,n){"use strict";function r(e){var t,n=e.keyCode;return"charCode"in e?(t=e.charCode,0===t&&13===n&&(t=13)):t=n,t>=32||13===t?t:0}t.exports=r},{}],119:[function(e,t,n){"use strict";function r(e){if(e.key){var t=a[e.key]||e.key;if("Unidentified"!==t)return t}if("keypress"===e.type){var n=o(e);return 13===n?"Enter":String.fromCharCode(n)}return"keydown"===e.type||"keyup"===e.type?i[e.keyCode]||"Unidentified":""}var o=e(118),a={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},i={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"};t.exports=r},{118:118}],120:[function(e,t,n){"use strict";function r(e){var t=this,n=t.nativeEvent;if(n.getModifierState)return n.getModifierState(e);var r=a[e];return!!r&&!!n[r]}function o(e){return r}var a={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};t.exports=o},{}],121:[function(e,t,n){"use strict";function r(e){var t=e.target||e.srcElement||window;return t.correspondingUseElement&&(t=t.correspondingUseElement),3===t.nodeType?t.parentNode:t}t.exports=r},{}],122:[function(e,t,n){"use strict";function r(e){for(var t;(t=e._renderedNodeType)===o.COMPOSITE;)e=e._renderedComponent;return t===o.HOST?e._renderedComponent:t===o.EMPTY?null:void 0}var o=e(71);t.exports=r},{71:71}],123:[function(e,t,n){"use strict";function r(e){var t=e&&(o&&e[o]||e[a]);if("function"==typeof t)return t}var o="function"==typeof Symbol&&Symbol.iterator,a="@@iterator";t.exports=r},{}],124:[function(e,t,n){"use strict";function r(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function o(e){for(;e;){if(e.nextSibling)return e.nextSibling;e=e.parentNode}}function a(e,t){for(var n=r(e),a=0,i=0;n;){if(3===n.nodeType){if(i=a+n.textContent.length,a<=t&&i>=t)return{node:n,offset:t-a};a=i}n=r(o(n))}}t.exports=a},{}],125:[function(e,t,n){"use strict";function r(){return!a&&o.canUseDOM&&(a="textContent"in document.documentElement?"textContent":"innerText"),a}var o=e(140),a=null;t.exports=r},{140:140}],126:[function(e,t,n){"use strict";function r(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n["ms"+e]="MS"+t,n["O"+e]="o"+t.toLowerCase(),n}function o(e){if(s[e])return s[e];if(!i[e])return e;var t=i[e];for(var n in t)if(t.hasOwnProperty(n)&&n in u)return s[e]=t[n];return""}var a=e(140),i={animationend:r("Animation","AnimationEnd"),animationiteration:r("Animation","AnimationIteration"),animationstart:r("Animation","AnimationStart"),transitionend:r("Transition","TransitionEnd")},s={},u={};a.canUseDOM&&(u=document.createElement("div").style,"AnimationEvent"in window||(delete i.animationend.animation,delete i.animationiteration.animation,delete i.animationstart.animation),"TransitionEvent"in window||delete i.transitionend.transition),t.exports=o},{140:140}],127:[function(e,t,n){"use strict";function r(e){if(e){var t=e.getName();if(t)return" Check the render method of `"+t+"`."}return""}function o(e){return"function"==typeof e&&"undefined"!=typeof e.prototype&&"function"==typeof e.prototype.mountComponent&&"function"==typeof e.prototype.receiveComponent}function a(e,t){var n;if(null===e||e===!1)n=l.create(a);else if("object"==typeof e){var s=e;!s||"function"!=typeof s.type&&"string"!=typeof s.type?i("130",null==s.type?s.type:typeof s.type,r(s._owner)):void 0,"string"==typeof s.type?n=c.createInternalComponent(s):o(s.type)?(n=new s.type(s),n.getHostNode||(n.getHostNode=n.getNativeNode)):n=new p(s)}else"string"==typeof e||"number"==typeof e?n=c.createInstanceForText(e):i("131",typeof e);return n._mountIndex=0,n._mountImage=null,n}var i=e(132),s=e(162),u=e(34),l=e(57),c=e(62),p=(e(154),e(161),function(e){this.construct(e)});s(p.prototype,u.Mixin,{_instantiateReactComponent:a});t.exports=a},{132:132,154:154,161:161,162:162,34:34,57:57,62:62}],128:[function(e,t,n){"use strict";function r(e,t){if(!a.canUseDOM||t&&!("addEventListener"in document))return!1;var n="on"+e,r=n in document;if(!r){var i=document.createElement("div");i.setAttribute(n,"return;"),r="function"==typeof i[n]}return!r&&o&&"wheel"===e&&(r=document.implementation.hasFeature("Events.wheel","3.0")),r}var o,a=e(140);a.canUseDOM&&(o=document.implementation&&document.implementation.hasFeature&&document.implementation.hasFeature("","")!==!0),t.exports=r},{140:140}],129:[function(e,t,n){"use strict";function r(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return"input"===t?!!o[e.type]:"textarea"===t}var o={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};t.exports=r},{}],130:[function(e,t,n){"use strict";function r(e){return a.isValidElement(e)?void 0:o("143"),e}var o=e(132),a=e(56);e(154);t.exports=r},{132:132,154:154,56:56}],131:[function(e,t,n){"use strict";function r(e){return'"'+o(e)+'"'}var o=e(114);t.exports=r},{114:114}],132:[function(e,t,n){"use strict";function r(e){for(var t=arguments.length-1,n="Minified React error #"+e+"; visit http://facebook.github.io/react/docs/error-decoder.html?invariant="+e,r=0;r<t;r++)n+="&args[]="+encodeURIComponent(arguments[r+1]);n+=" for the full message or use the non-minified dev environment for full errors and additional helpful warnings.";var o=new Error(n);throw o.name="Invariant Violation",o.framesToPop=1,o}t.exports=r},{}],133:[function(e,t,n){"use strict";var r=e(68);t.exports=r.renderSubtreeIntoContainer},{68:68}],134:[function(e,t,n){"use strict";var r,o=e(140),a=e(9),i=/^[ \r\n\t\f]/,s=/<(!--|link|noscript|meta|script|style)[ \r\n\t\f\/>]/,u=e(112),l=u(function(e,t){if(e.namespaceURI!==a.svg||"innerHTML"in e)e.innerHTML=t;else{r=r||document.createElement("div"),r.innerHTML="<svg>"+t+"</svg>";for(var n=r.firstChild.childNodes,o=0;o<n.length;o++)e.appendChild(n[o])}});if(o.canUseDOM){var c=document.createElement("div");c.innerHTML=" ",""===c.innerHTML&&(l=function(e,t){if(e.parentNode&&e.parentNode.replaceChild(e,e),i.test(t)||"<"===t[0]&&s.test(t)){e.innerHTML=String.fromCharCode(65279)+t;var n=e.firstChild;1===n.data.length?e.removeChild(n):n.deleteData(0,1)}else e.innerHTML=t}),c=null}t.exports=l},{112:112,140:140,9:9}],135:[function(e,t,n){"use strict";var r=e(140),o=e(114),a=e(134),i=function(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&3===n.nodeType)return void(n.nodeValue=t)}e.textContent=t};r.canUseDOM&&("textContent"in document.documentElement||(i=function(e,t){a(e,o(t))})),t.exports=i},{114:114,134:134,140:140}],136:[function(e,t,n){"use strict";function r(e,t){var n=null===e||e===!1,r=null===t||t===!1;if(n||r)return n===r;var o=typeof e,a=typeof t;return"string"===o||"number"===o?"string"===a||"number"===a:"object"===a&&e.type===t.type&&e.key===t.key}t.exports=r},{}],137:[function(e,t,n){"use strict";function r(e,t){return e&&"object"==typeof e&&null!=e.key?l.escape(e.key):t.toString(36)}function o(e,t,n,a){var d=typeof e;if("undefined"!==d&&"boolean"!==d||(e=null),null===e||"string"===d||"number"===d||s.isValidElement(e))return n(a,e,""===t?c+r(e,0):t),1;var f,h,m=0,v=""===t?c:t+p;if(Array.isArray(e))for(var g=0;g<e.length;g++)f=e[g],h=v+r(f,g),m+=o(f,h,n,a);else{var y=u(e);if(y){var b,C=y.call(e);if(y!==e.entries)for(var _=0;!(b=C.next()).done;)f=b.value,h=v+r(f,_++),m+=o(f,h,n,a);else for(;!(b=C.next()).done;){var E=b.value;E&&(f=E[1],h=v+l.escape(E[0])+p+r(f,0),m+=o(f,h,n,a))}}else if("object"===d){var x="",T=String(e);i("31","[object Object]"===T?"object with keys {"+Object.keys(e).join(", ")+"}":T,x)}}return m}function a(e,t,n){return null==e?0:o(e,"",t,n)}var i=e(132),s=(e(35),e(56)),u=e(123),l=(e(154),e(23)),c=(e(161),"."),p=":";t.exports=a},{123:123,132:132,154:154,161:161,23:23,35:35,56:56}],138:[function(e,t,n){"use strict";var r=(e(162),e(146)),o=(e(161),r);t.exports=o},{146:146,161:161,162:162}],139:[function(e,t,n){"use strict";var r=e(146),o={listen:function(e,t,n){return e.addEventListener?(e.addEventListener(t,n,!1),{remove:function(){e.removeEventListener(t,n,!1)}}):e.attachEvent?(e.attachEvent("on"+t,n),{remove:function(){e.detachEvent("on"+t,n)}}):void 0},capture:function(e,t,n){return e.addEventListener?(e.addEventListener(t,n,!0),{remove:function(){e.removeEventListener(t,n,!0)}}):{remove:r}},registerDefault:function(){}};t.exports=o},{146:146}],140:[function(e,t,n){"use strict";var r=!("undefined"==typeof window||!window.document||!window.document.createElement),o={canUseDOM:r,canUseWorkers:"undefined"!=typeof Worker,canUseEventListeners:r&&!(!window.addEventListener&&!window.attachEvent),canUseViewport:r&&!!window.screen,isInWorker:!r};t.exports=o},{}],141:[function(e,t,n){"use strict";function r(e){return e.replace(o,function(e,t){return t.toUpperCase()})}var o=/-(.)/g;t.exports=r},{}],142:[function(e,t,n){"use strict";function r(e){return o(e.replace(a,"ms-"))}var o=e(141),a=/^-ms-/;t.exports=r},{141:141}],143:[function(e,t,n){"use strict";function r(e,t){return!(!e||!t)&&(e===t||!o(e)&&(o(t)?r(e,t.parentNode):"contains"in e?e.contains(t):!!e.compareDocumentPosition&&!!(16&e.compareDocumentPosition(t))))}var o=e(156);t.exports=r},{156:156}],144:[function(e,t,n){"use strict";function r(e){var t=e.length;if(Array.isArray(e)||"object"!=typeof e&&"function"!=typeof e?i(!1):void 0,"number"!=typeof t?i(!1):void 0,0===t||t-1 in e?void 0:i(!1),"function"==typeof e.callee?i(!1):void 0,e.hasOwnProperty)try{return Array.prototype.slice.call(e)}catch(e){}for(var n=Array(t),r=0;r<t;r++)n[r]=e[r];return n}function o(e){return!!e&&("object"==typeof e||"function"==typeof e)&&"length"in e&&!("setInterval"in e)&&"number"!=typeof e.nodeType&&(Array.isArray(e)||"callee"in e||"item"in e)}function a(e){return o(e)?Array.isArray(e)?e.slice():r(e):[e]}var i=e(154);t.exports=a},{154:154}],145:[function(e,t,n){"use strict";function r(e){var t=e.match(c);return t&&t[1].toLowerCase()}function o(e,t){var n=l;l?void 0:u(!1);var o=r(e),a=o&&s(o);if(a){n.innerHTML=a[1]+e+a[2];for(var c=a[0];c--;)n=n.lastChild}else n.innerHTML=e;var p=n.getElementsByTagName("script");p.length&&(t?void 0:u(!1),i(p).forEach(t));for(var d=Array.from(n.childNodes);n.lastChild;)n.removeChild(n.lastChild);return d}var a=e(140),i=e(144),s=e(150),u=e(154),l=a.canUseDOM?document.createElement("div"):null,c=/^\s*<(\w+)/;t.exports=o},{140:140,144:144,150:150,154:154}],146:[function(e,t,n){"use strict";function r(e){return function(){return e}}var o=function(){};o.thatReturns=r,o.thatReturnsFalse=r(!1),o.thatReturnsTrue=r(!0),o.thatReturnsNull=r(null),o.thatReturnsThis=function(){return this},o.thatReturnsArgument=function(e){return e},t.exports=o},{}],147:[function(e,t,n){"use strict";var r={};t.exports=r},{}],148:[function(e,t,n){"use strict";function r(e){try{e.focus()}catch(e){}}t.exports=r},{}],149:[function(e,t,n){"use strict";function r(){if("undefined"==typeof document)return null;try{return document.activeElement||document.body}catch(e){return document.body}}t.exports=r},{}],150:[function(e,t,n){"use strict";function r(e){return i?void 0:a(!1),d.hasOwnProperty(e)||(e="*"),s.hasOwnProperty(e)||("*"===e?i.innerHTML="<link />":i.innerHTML="<"+e+"></"+e+">",s[e]=!i.firstChild),s[e]?d[e]:null}var o=e(140),a=e(154),i=o.canUseDOM?document.createElement("div"):null,s={},u=[1,'<select multiple="true">',"</select>"],l=[1,"<table>","</table>"],c=[3,"<table><tbody><tr>","</tr></tbody></table>"],p=[1,'<svg xmlns="http://www.w3.org/2000/svg">',"</svg>"],d={"*":[1,"?<div>","</div>"],area:[1,"<map>","</map>"],col:[2,"<table><tbody></tbody><colgroup>","</colgroup></table>"],legend:[1,"<fieldset>","</fieldset>"],param:[1,"<object>","</object>"],tr:[2,"<table><tbody>","</tbody></table>"],optgroup:u,option:u,caption:l,colgroup:l,tbody:l,tfoot:l,thead:l,td:c,th:c},f=["circle","clipPath","defs","ellipse","g","image","line","linearGradient","mask","path","pattern","polygon","polyline","radialGradient","rect","stop","text","tspan"];f.forEach(function(e){d[e]=p,s[e]=!0}),t.exports=r},{140:140,154:154}],151:[function(e,t,n){"use strict";function r(e){return e===window?{x:window.pageXOffset||document.documentElement.scrollLeft,y:window.pageYOffset||document.documentElement.scrollTop}:{x:e.scrollLeft,y:e.scrollTop}}t.exports=r},{}],152:[function(e,t,n){"use strict";function r(e){return e.replace(o,"-$1").toLowerCase()}var o=/([A-Z])/g;t.exports=r},{}],153:[function(e,t,n){"use strict";function r(e){return o(e).replace(a,"-ms-")}var o=e(152),a=/^ms-/;t.exports=r},{152:152}],154:[function(e,t,n){"use strict";function r(e,t,n,r,o,a,i,s){if(!e){var u;if(void 0===t)u=new Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,r,o,a,i,s],c=0;u=new Error(t.replace(/%s/g,function(){return l[c++]})),u.name="Invariant Violation"}throw u.framesToPop=1,u}}t.exports=r},{}],155:[function(e,t,n){"use strict";function r(e){return!(!e||!("function"==typeof Node?e instanceof Node:"object"==typeof e&&"number"==typeof e.nodeType&&"string"==typeof e.nodeName))}t.exports=r},{}],156:[function(e,t,n){"use strict";function r(e){return o(e)&&3==e.nodeType}var o=e(155);t.exports=r},{155:155}],157:[function(e,t,n){"use strict";var r=e(154),o=function(e){var t,n={};e instanceof Object&&!Array.isArray(e)?void 0:r(!1);for(t in e)e.hasOwnProperty(t)&&(n[t]=t);return n};t.exports=o},{154:154}],158:[function(e,t,n){"use strict";var r=function(e){var t;for(t in e)if(e.hasOwnProperty(t))return t;return null};t.exports=r},{}],159:[function(e,t,n){"use strict";function r(e){var t={};return function(n){return t.hasOwnProperty(n)||(t[n]=e.call(this,n)),t[n]}}t.exports=r},{}],160:[function(e,t,n){"use strict";function r(e,t){return e===t?0!==e||1/e===1/t:e!==e&&t!==t}function o(e,t){if(r(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),o=Object.keys(t);if(n.length!==o.length)return!1;for(var i=0;i<n.length;i++)if(!a.call(t,n[i])||!r(e[n[i]],t[n[i]]))return!1;return!0}var a=Object.prototype.hasOwnProperty;t.exports=o},{}],161:[function(e,t,n){"use strict";var r=e(146),o=r;t.exports=o},{146:146}],162:[function(e,t,n){"use strict";function r(e){if(null===e||void 0===e)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(e)}function o(){try{if(!Object.assign)return!1;var e=new String("abc");if(e[5]="de","5"===Object.getOwnPropertyNames(e)[0])return!1;for(var t={},n=0;n<10;n++)t["_"+String.fromCharCode(n)]=n;var r=Object.getOwnPropertyNames(t).map(function(e){return t[e]});if("0123456789"!==r.join(""))return!1;var o={};return"abcdefghijklmnopqrst".split("").forEach(function(e){o[e]=e}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},o)).join("")}catch(e){return!1}}var a=Object.prototype.hasOwnProperty,i=Object.prototype.propertyIsEnumerable;t.exports=o()?Object.assign:function(e,t){for(var n,o,s=r(e),u=1;u<arguments.length;u++){n=Object(arguments[u]);for(var l in n)a.call(n,l)&&(s[l]=n[l]);if(Object.getOwnPropertySymbols){o=Object.getOwnPropertySymbols(n);for(var c=0;c<o.length;c++)i.call(n,o[c])&&(s[o[c]]=n[o[c]])}}return s}},{}]},{},[86])(86)});
/**
 * ReactDOMServer v15.2.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e(require("react"));else if("function"==typeof define&&define.amd)define(["react"],e);else{var f;f="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,f.ReactDOMServer=e(f.React)}}(function(e){return e.__SECRET_DOM_SERVER_DO_NOT_USE_OR_YOU_WILL_BE_FIRED});
/**
 * ReactDOM v15.2.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e(require("react"));else if("function"==typeof define&&define.amd)define(["react"],e);else{var f;f="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,f.ReactDOM=e(f.React)}}(function(e){return e.__SECRET_DOM_DO_NOT_USE_OR_YOU_WILL_BE_FIRED});
!function(){function n(n){return n&&(n.ownerDocument||n.document||n).documentElement}function t(n){return n&&(n.ownerDocument&&n.ownerDocument.defaultView||n.document&&n||n.defaultView)}function e(n,t){return t>n?-1:n>t?1:n>=t?0:0/0}function r(n){return null===n?0/0:+n}function u(n){return!isNaN(n)}function i(n){return{left:function(t,e,r,u){for(arguments.length<3&&(r=0),arguments.length<4&&(u=t.length);u>r;){var i=r+u>>>1;n(t[i],e)<0?r=i+1:u=i}return r},right:function(t,e,r,u){for(arguments.length<3&&(r=0),arguments.length<4&&(u=t.length);u>r;){var i=r+u>>>1;n(t[i],e)>0?u=i:r=i+1}return r}}}function o(n){return n.length}function a(n){for(var t=1;n*t%1;)t*=10;return t}function c(n,t){for(var e in t)Object.defineProperty(n.prototype,e,{value:t[e],enumerable:!1})}function l(){this._=Object.create(null)}function s(n){return(n+="")===pa||n[0]===va?va+n:n}function f(n){return(n+="")[0]===va?n.slice(1):n}function h(n){return s(n)in this._}function g(n){return(n=s(n))in this._&&delete this._[n]}function p(){var n=[];for(var t in this._)n.push(f(t));return n}function v(){var n=0;for(var t in this._)++n;return n}function d(){for(var n in this._)return!1;return!0}function m(){this._=Object.create(null)}function y(n){return n}function M(n,t,e){return function(){var r=e.apply(t,arguments);return r===t?n:r}}function x(n,t){if(t in n)return t;t=t.charAt(0).toUpperCase()+t.slice(1);for(var e=0,r=da.length;r>e;++e){var u=da[e]+t;if(u in n)return u}}function b(){}function _(){}function w(n){function t(){for(var t,r=e,u=-1,i=r.length;++u<i;)(t=r[u].on)&&t.apply(this,arguments);return n}var e=[],r=new l;return t.on=function(t,u){var i,o=r.get(t);return arguments.length<2?o&&o.on:(o&&(o.on=null,e=e.slice(0,i=e.indexOf(o)).concat(e.slice(i+1)),r.remove(t)),u&&e.push(r.set(t,{on:u})),n)},t}function S(){ta.event.preventDefault()}function k(){for(var n,t=ta.event;n=t.sourceEvent;)t=n;return t}function E(n){for(var t=new _,e=0,r=arguments.length;++e<r;)t[arguments[e]]=w(t);return t.of=function(e,r){return function(u){try{var i=u.sourceEvent=ta.event;u.target=n,ta.event=u,t[u.type].apply(e,r)}finally{ta.event=i}}},t}function A(n){return ya(n,_a),n}function N(n){return"function"==typeof n?n:function(){return Ma(n,this)}}function C(n){return"function"==typeof n?n:function(){return xa(n,this)}}function z(n,t){function e(){this.removeAttribute(n)}function r(){this.removeAttributeNS(n.space,n.local)}function u(){this.setAttribute(n,t)}function i(){this.setAttributeNS(n.space,n.local,t)}function o(){var e=t.apply(this,arguments);null==e?this.removeAttribute(n):this.setAttribute(n,e)}function a(){var e=t.apply(this,arguments);null==e?this.removeAttributeNS(n.space,n.local):this.setAttributeNS(n.space,n.local,e)}return n=ta.ns.qualify(n),null==t?n.local?r:e:"function"==typeof t?n.local?a:o:n.local?i:u}function q(n){return n.trim().replace(/\s+/g," ")}function L(n){return new RegExp("(?:^|\\s+)"+ta.requote(n)+"(?:\\s+|$)","g")}function T(n){return(n+"").trim().split(/^|\s+/)}function R(n,t){function e(){for(var e=-1;++e<u;)n[e](this,t)}function r(){for(var e=-1,r=t.apply(this,arguments);++e<u;)n[e](this,r)}n=T(n).map(D);var u=n.length;return"function"==typeof t?r:e}function D(n){var t=L(n);return function(e,r){if(u=e.classList)return r?u.add(n):u.remove(n);var u=e.getAttribute("class")||"";r?(t.lastIndex=0,t.test(u)||e.setAttribute("class",q(u+" "+n))):e.setAttribute("class",q(u.replace(t," ")))}}function P(n,t,e){function r(){this.style.removeProperty(n)}function u(){this.style.setProperty(n,t,e)}function i(){var r=t.apply(this,arguments);null==r?this.style.removeProperty(n):this.style.setProperty(n,r,e)}return null==t?r:"function"==typeof t?i:u}function U(n,t){function e(){delete this[n]}function r(){this[n]=t}function u(){var e=t.apply(this,arguments);null==e?delete this[n]:this[n]=e}return null==t?e:"function"==typeof t?u:r}function j(n){function t(){var t=this.ownerDocument,e=this.namespaceURI;return e?t.createElementNS(e,n):t.createElement(n)}function e(){return this.ownerDocument.createElementNS(n.space,n.local)}return"function"==typeof n?n:(n=ta.ns.qualify(n)).local?e:t}function F(){var n=this.parentNode;n&&n.removeChild(this)}function H(n){return{__data__:n}}function O(n){return function(){return ba(this,n)}}function I(n){return arguments.length||(n=e),function(t,e){return t&&e?n(t.__data__,e.__data__):!t-!e}}function Y(n,t){for(var e=0,r=n.length;r>e;e++)for(var u,i=n[e],o=0,a=i.length;a>o;o++)(u=i[o])&&t(u,o,e);return n}function Z(n){return ya(n,Sa),n}function V(n){var t,e;return function(r,u,i){var o,a=n[i].update,c=a.length;for(i!=e&&(e=i,t=0),u>=t&&(t=u+1);!(o=a[t])&&++t<c;);return o}}function X(n,t,e){function r(){var t=this[o];t&&(this.removeEventListener(n,t,t.$),delete this[o])}function u(){var u=c(t,ra(arguments));r.call(this),this.addEventListener(n,this[o]=u,u.$=e),u._=t}function i(){var t,e=new RegExp("^__on([^.]+)"+ta.requote(n)+"$");for(var r in this)if(t=r.match(e)){var u=this[r];this.removeEventListener(t[1],u,u.$),delete this[r]}}var o="__on"+n,a=n.indexOf("."),c=$;a>0&&(n=n.slice(0,a));var l=ka.get(n);return l&&(n=l,c=B),a?t?u:r:t?b:i}function $(n,t){return function(e){var r=ta.event;ta.event=e,t[0]=this.__data__;try{n.apply(this,t)}finally{ta.event=r}}}function B(n,t){var e=$(n,t);return function(n){var t=this,r=n.relatedTarget;r&&(r===t||8&r.compareDocumentPosition(t))||e.call(t,n)}}function W(e){var r=".dragsuppress-"+ ++Aa,u="click"+r,i=ta.select(t(e)).on("touchmove"+r,S).on("dragstart"+r,S).on("selectstart"+r,S);if(null==Ea&&(Ea="onselectstart"in e?!1:x(e.style,"userSelect")),Ea){var o=n(e).style,a=o[Ea];o[Ea]="none"}return function(n){if(i.on(r,null),Ea&&(o[Ea]=a),n){var t=function(){i.on(u,null)};i.on(u,function(){S(),t()},!0),setTimeout(t,0)}}}function J(n,e){e.changedTouches&&(e=e.changedTouches[0]);var r=n.ownerSVGElement||n;if(r.createSVGPoint){var u=r.createSVGPoint();if(0>Na){var i=t(n);if(i.scrollX||i.scrollY){r=ta.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important");var o=r[0][0].getScreenCTM();Na=!(o.f||o.e),r.remove()}}return Na?(u.x=e.pageX,u.y=e.pageY):(u.x=e.clientX,u.y=e.clientY),u=u.matrixTransform(n.getScreenCTM().inverse()),[u.x,u.y]}var a=n.getBoundingClientRect();return[e.clientX-a.left-n.clientLeft,e.clientY-a.top-n.clientTop]}function G(){return ta.event.changedTouches[0].identifier}function K(n){return n>0?1:0>n?-1:0}function Q(n,t,e){return(t[0]-n[0])*(e[1]-n[1])-(t[1]-n[1])*(e[0]-n[0])}function nt(n){return n>1?0:-1>n?qa:Math.acos(n)}function tt(n){return n>1?Ra:-1>n?-Ra:Math.asin(n)}function et(n){return((n=Math.exp(n))-1/n)/2}function rt(n){return((n=Math.exp(n))+1/n)/2}function ut(n){return((n=Math.exp(2*n))-1)/(n+1)}function it(n){return(n=Math.sin(n/2))*n}function ot(){}function at(n,t,e){return this instanceof at?(this.h=+n,this.s=+t,void(this.l=+e)):arguments.length<2?n instanceof at?new at(n.h,n.s,n.l):bt(""+n,_t,at):new at(n,t,e)}function ct(n,t,e){function r(n){return n>360?n-=360:0>n&&(n+=360),60>n?i+(o-i)*n/60:180>n?o:240>n?i+(o-i)*(240-n)/60:i}function u(n){return Math.round(255*r(n))}var i,o;return n=isNaN(n)?0:(n%=360)<0?n+360:n,t=isNaN(t)?0:0>t?0:t>1?1:t,e=0>e?0:e>1?1:e,o=.5>=e?e*(1+t):e+t-e*t,i=2*e-o,new mt(u(n+120),u(n),u(n-120))}function lt(n,t,e){return this instanceof lt?(this.h=+n,this.c=+t,void(this.l=+e)):arguments.length<2?n instanceof lt?new lt(n.h,n.c,n.l):n instanceof ft?gt(n.l,n.a,n.b):gt((n=wt((n=ta.rgb(n)).r,n.g,n.b)).l,n.a,n.b):new lt(n,t,e)}function st(n,t,e){return isNaN(n)&&(n=0),isNaN(t)&&(t=0),new ft(e,Math.cos(n*=Da)*t,Math.sin(n)*t)}function ft(n,t,e){return this instanceof ft?(this.l=+n,this.a=+t,void(this.b=+e)):arguments.length<2?n instanceof ft?new ft(n.l,n.a,n.b):n instanceof lt?st(n.h,n.c,n.l):wt((n=mt(n)).r,n.g,n.b):new ft(n,t,e)}function ht(n,t,e){var r=(n+16)/116,u=r+t/500,i=r-e/200;return u=pt(u)*Xa,r=pt(r)*$a,i=pt(i)*Ba,new mt(dt(3.2404542*u-1.5371385*r-.4985314*i),dt(-.969266*u+1.8760108*r+.041556*i),dt(.0556434*u-.2040259*r+1.0572252*i))}function gt(n,t,e){return n>0?new lt(Math.atan2(e,t)*Pa,Math.sqrt(t*t+e*e),n):new lt(0/0,0/0,n)}function pt(n){return n>.206893034?n*n*n:(n-4/29)/7.787037}function vt(n){return n>.008856?Math.pow(n,1/3):7.787037*n+4/29}function dt(n){return Math.round(255*(.00304>=n?12.92*n:1.055*Math.pow(n,1/2.4)-.055))}function mt(n,t,e){return this instanceof mt?(this.r=~~n,this.g=~~t,void(this.b=~~e)):arguments.length<2?n instanceof mt?new mt(n.r,n.g,n.b):bt(""+n,mt,ct):new mt(n,t,e)}function yt(n){return new mt(n>>16,n>>8&255,255&n)}function Mt(n){return yt(n)+""}function xt(n){return 16>n?"0"+Math.max(0,n).toString(16):Math.min(255,n).toString(16)}function bt(n,t,e){var r,u,i,o=0,a=0,c=0;if(r=/([a-z]+)\((.*)\)/i.exec(n))switch(u=r[2].split(","),r[1]){case"hsl":return e(parseFloat(u[0]),parseFloat(u[1])/100,parseFloat(u[2])/100);case"rgb":return t(kt(u[0]),kt(u[1]),kt(u[2]))}return(i=Ga.get(n.toLowerCase()))?t(i.r,i.g,i.b):(null==n||"#"!==n.charAt(0)||isNaN(i=parseInt(n.slice(1),16))||(4===n.length?(o=(3840&i)>>4,o=o>>4|o,a=240&i,a=a>>4|a,c=15&i,c=c<<4|c):7===n.length&&(o=(16711680&i)>>16,a=(65280&i)>>8,c=255&i)),t(o,a,c))}function _t(n,t,e){var r,u,i=Math.min(n/=255,t/=255,e/=255),o=Math.max(n,t,e),a=o-i,c=(o+i)/2;return a?(u=.5>c?a/(o+i):a/(2-o-i),r=n==o?(t-e)/a+(e>t?6:0):t==o?(e-n)/a+2:(n-t)/a+4,r*=60):(r=0/0,u=c>0&&1>c?0:r),new at(r,u,c)}function wt(n,t,e){n=St(n),t=St(t),e=St(e);var r=vt((.4124564*n+.3575761*t+.1804375*e)/Xa),u=vt((.2126729*n+.7151522*t+.072175*e)/$a),i=vt((.0193339*n+.119192*t+.9503041*e)/Ba);return ft(116*u-16,500*(r-u),200*(u-i))}function St(n){return(n/=255)<=.04045?n/12.92:Math.pow((n+.055)/1.055,2.4)}function kt(n){var t=parseFloat(n);return"%"===n.charAt(n.length-1)?Math.round(2.55*t):t}function Et(n){return"function"==typeof n?n:function(){return n}}function At(n){return function(t,e,r){return 2===arguments.length&&"function"==typeof e&&(r=e,e=null),Nt(t,e,n,r)}}function Nt(n,t,e,r){function u(){var n,t=c.status;if(!t&&zt(c)||t>=200&&300>t||304===t){try{n=e.call(i,c)}catch(r){return void o.error.call(i,r)}o.load.call(i,n)}else o.error.call(i,c)}var i={},o=ta.dispatch("beforesend","progress","load","error"),a={},c=new XMLHttpRequest,l=null;return!this.XDomainRequest||"withCredentials"in c||!/^(http(s)?:)?\/\//.test(n)||(c=new XDomainRequest),"onload"in c?c.onload=c.onerror=u:c.onreadystatechange=function(){c.readyState>3&&u()},c.onprogress=function(n){var t=ta.event;ta.event=n;try{o.progress.call(i,c)}finally{ta.event=t}},i.header=function(n,t){return n=(n+"").toLowerCase(),arguments.length<2?a[n]:(null==t?delete a[n]:a[n]=t+"",i)},i.mimeType=function(n){return arguments.length?(t=null==n?null:n+"",i):t},i.responseType=function(n){return arguments.length?(l=n,i):l},i.response=function(n){return e=n,i},["get","post"].forEach(function(n){i[n]=function(){return i.send.apply(i,[n].concat(ra(arguments)))}}),i.send=function(e,r,u){if(2===arguments.length&&"function"==typeof r&&(u=r,r=null),c.open(e,n,!0),null==t||"accept"in a||(a.accept=t+",*/*"),c.setRequestHeader)for(var s in a)c.setRequestHeader(s,a[s]);return null!=t&&c.overrideMimeType&&c.overrideMimeType(t),null!=l&&(c.responseType=l),null!=u&&i.on("error",u).on("load",function(n){u(null,n)}),o.beforesend.call(i,c),c.send(null==r?null:r),i},i.abort=function(){return c.abort(),i},ta.rebind(i,o,"on"),null==r?i:i.get(Ct(r))}function Ct(n){return 1===n.length?function(t,e){n(null==t?e:null)}:n}function zt(n){var t=n.responseType;return t&&"text"!==t?n.response:n.responseText}function qt(){var n=Lt(),t=Tt()-n;t>24?(isFinite(t)&&(clearTimeout(tc),tc=setTimeout(qt,t)),nc=0):(nc=1,rc(qt))}function Lt(){var n=Date.now();for(ec=Ka;ec;)n>=ec.t&&(ec.f=ec.c(n-ec.t)),ec=ec.n;return n}function Tt(){for(var n,t=Ka,e=1/0;t;)t.f?t=n?n.n=t.n:Ka=t.n:(t.t<e&&(e=t.t),t=(n=t).n);return Qa=n,e}function Rt(n,t){return t-(n?Math.ceil(Math.log(n)/Math.LN10):1)}function Dt(n,t){var e=Math.pow(10,3*ga(8-t));return{scale:t>8?function(n){return n/e}:function(n){return n*e},symbol:n}}function Pt(n){var t=n.decimal,e=n.thousands,r=n.grouping,u=n.currency,i=r&&e?function(n,t){for(var u=n.length,i=[],o=0,a=r[0],c=0;u>0&&a>0&&(c+a+1>t&&(a=Math.max(1,t-c)),i.push(n.substring(u-=a,u+a)),!((c+=a+1)>t));)a=r[o=(o+1)%r.length];return i.reverse().join(e)}:y;return function(n){var e=ic.exec(n),r=e[1]||" ",o=e[2]||">",a=e[3]||"-",c=e[4]||"",l=e[5],s=+e[6],f=e[7],h=e[8],g=e[9],p=1,v="",d="",m=!1,y=!0;switch(h&&(h=+h.substring(1)),(l||"0"===r&&"="===o)&&(l=r="0",o="="),g){case"n":f=!0,g="g";break;case"%":p=100,d="%",g="f";break;case"p":p=100,d="%",g="r";break;case"b":case"o":case"x":case"X":"#"===c&&(v="0"+g.toLowerCase());case"c":y=!1;case"d":m=!0,h=0;break;case"s":p=-1,g="r"}"$"===c&&(v=u[0],d=u[1]),"r"!=g||h||(g="g"),null!=h&&("g"==g?h=Math.max(1,Math.min(21,h)):("e"==g||"f"==g)&&(h=Math.max(0,Math.min(20,h)))),g=oc.get(g)||Ut;var M=l&&f;return function(n){var e=d;if(m&&n%1)return"";var u=0>n||0===n&&0>1/n?(n=-n,"-"):"-"===a?"":a;if(0>p){var c=ta.formatPrefix(n,h);n=c.scale(n),e=c.symbol+d}else n*=p;n=g(n,h);var x,b,_=n.lastIndexOf(".");if(0>_){var w=y?n.lastIndexOf("e"):-1;0>w?(x=n,b=""):(x=n.substring(0,w),b=n.substring(w))}else x=n.substring(0,_),b=t+n.substring(_+1);!l&&f&&(x=i(x,1/0));var S=v.length+x.length+b.length+(M?0:u.length),k=s>S?new Array(S=s-S+1).join(r):"";return M&&(x=i(k+x,k.length?s-b.length:1/0)),u+=v,n=x+b,("<"===o?u+n+k:">"===o?k+u+n:"^"===o?k.substring(0,S>>=1)+u+n+k.substring(S):u+(M?n:k+n))+e}}}function Ut(n){return n+""}function jt(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}function Ft(n,t,e){function r(t){var e=n(t),r=i(e,1);return r-t>t-e?e:r}function u(e){return t(e=n(new cc(e-1)),1),e}function i(n,e){return t(n=new cc(+n),e),n}function o(n,r,i){var o=u(n),a=[];if(i>1)for(;r>o;)e(o)%i||a.push(new Date(+o)),t(o,1);else for(;r>o;)a.push(new Date(+o)),t(o,1);return a}function a(n,t,e){try{cc=jt;var r=new jt;return r._=n,o(r,t,e)}finally{cc=Date}}n.floor=n,n.round=r,n.ceil=u,n.offset=i,n.range=o;var c=n.utc=Ht(n);return c.floor=c,c.round=Ht(r),c.ceil=Ht(u),c.offset=Ht(i),c.range=a,n}function Ht(n){return function(t,e){try{cc=jt;var r=new jt;return r._=t,n(r,e)._}finally{cc=Date}}}function Ot(n){function t(n){function t(t){for(var e,u,i,o=[],a=-1,c=0;++a<r;)37===n.charCodeAt(a)&&(o.push(n.slice(c,a)),null!=(u=sc[e=n.charAt(++a)])&&(e=n.charAt(++a)),(i=N[e])&&(e=i(t,null==u?"e"===e?" ":"0":u)),o.push(e),c=a+1);return o.push(n.slice(c,a)),o.join("")}var r=n.length;return t.parse=function(t){var r={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null},u=e(r,n,t,0);if(u!=t.length)return null;"p"in r&&(r.H=r.H%12+12*r.p);var i=null!=r.Z&&cc!==jt,o=new(i?jt:cc);return"j"in r?o.setFullYear(r.y,0,r.j):"w"in r&&("W"in r||"U"in r)?(o.setFullYear(r.y,0,1),o.setFullYear(r.y,0,"W"in r?(r.w+6)%7+7*r.W-(o.getDay()+5)%7:r.w+7*r.U-(o.getDay()+6)%7)):o.setFullYear(r.y,r.m,r.d),o.setHours(r.H+(r.Z/100|0),r.M+r.Z%100,r.S,r.L),i?o._:o},t.toString=function(){return n},t}function e(n,t,e,r){for(var u,i,o,a=0,c=t.length,l=e.length;c>a;){if(r>=l)return-1;if(u=t.charCodeAt(a++),37===u){if(o=t.charAt(a++),i=C[o in sc?t.charAt(a++):o],!i||(r=i(n,e,r))<0)return-1}else if(u!=e.charCodeAt(r++))return-1}return r}function r(n,t,e){_.lastIndex=0;var r=_.exec(t.slice(e));return r?(n.w=w.get(r[0].toLowerCase()),e+r[0].length):-1}function u(n,t,e){x.lastIndex=0;var r=x.exec(t.slice(e));return r?(n.w=b.get(r[0].toLowerCase()),e+r[0].length):-1}function i(n,t,e){E.lastIndex=0;var r=E.exec(t.slice(e));return r?(n.m=A.get(r[0].toLowerCase()),e+r[0].length):-1}function o(n,t,e){S.lastIndex=0;var r=S.exec(t.slice(e));return r?(n.m=k.get(r[0].toLowerCase()),e+r[0].length):-1}function a(n,t,r){return e(n,N.c.toString(),t,r)}function c(n,t,r){return e(n,N.x.toString(),t,r)}function l(n,t,r){return e(n,N.X.toString(),t,r)}function s(n,t,e){var r=M.get(t.slice(e,e+=2).toLowerCase());return null==r?-1:(n.p=r,e)}var f=n.dateTime,h=n.date,g=n.time,p=n.periods,v=n.days,d=n.shortDays,m=n.months,y=n.shortMonths;t.utc=function(n){function e(n){try{cc=jt;var t=new cc;return t._=n,r(t)}finally{cc=Date}}var r=t(n);return e.parse=function(n){try{cc=jt;var t=r.parse(n);return t&&t._}finally{cc=Date}},e.toString=r.toString,e},t.multi=t.utc.multi=ae;var M=ta.map(),x=Yt(v),b=Zt(v),_=Yt(d),w=Zt(d),S=Yt(m),k=Zt(m),E=Yt(y),A=Zt(y);p.forEach(function(n,t){M.set(n.toLowerCase(),t)});var N={a:function(n){return d[n.getDay()]},A:function(n){return v[n.getDay()]},b:function(n){return y[n.getMonth()]},B:function(n){return m[n.getMonth()]},c:t(f),d:function(n,t){return It(n.getDate(),t,2)},e:function(n,t){return It(n.getDate(),t,2)},H:function(n,t){return It(n.getHours(),t,2)},I:function(n,t){return It(n.getHours()%12||12,t,2)},j:function(n,t){return It(1+ac.dayOfYear(n),t,3)},L:function(n,t){return It(n.getMilliseconds(),t,3)},m:function(n,t){return It(n.getMonth()+1,t,2)},M:function(n,t){return It(n.getMinutes(),t,2)},p:function(n){return p[+(n.getHours()>=12)]},S:function(n,t){return It(n.getSeconds(),t,2)},U:function(n,t){return It(ac.sundayOfYear(n),t,2)},w:function(n){return n.getDay()},W:function(n,t){return It(ac.mondayOfYear(n),t,2)},x:t(h),X:t(g),y:function(n,t){return It(n.getFullYear()%100,t,2)},Y:function(n,t){return It(n.getFullYear()%1e4,t,4)},Z:ie,"%":function(){return"%"}},C={a:r,A:u,b:i,B:o,c:a,d:Qt,e:Qt,H:te,I:te,j:ne,L:ue,m:Kt,M:ee,p:s,S:re,U:Xt,w:Vt,W:$t,x:c,X:l,y:Wt,Y:Bt,Z:Jt,"%":oe};return t}function It(n,t,e){var r=0>n?"-":"",u=(r?-n:n)+"",i=u.length;return r+(e>i?new Array(e-i+1).join(t)+u:u)}function Yt(n){return new RegExp("^(?:"+n.map(ta.requote).join("|")+")","i")}function Zt(n){for(var t=new l,e=-1,r=n.length;++e<r;)t.set(n[e].toLowerCase(),e);return t}function Vt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+1));return r?(n.w=+r[0],e+r[0].length):-1}function Xt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e));return r?(n.U=+r[0],e+r[0].length):-1}function $t(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e));return r?(n.W=+r[0],e+r[0].length):-1}function Bt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+4));return r?(n.y=+r[0],e+r[0].length):-1}function Wt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.y=Gt(+r[0]),e+r[0].length):-1}function Jt(n,t,e){return/^[+-]\d{4}$/.test(t=t.slice(e,e+5))?(n.Z=-t,e+5):-1}function Gt(n){return n+(n>68?1900:2e3)}function Kt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.m=r[0]-1,e+r[0].length):-1}function Qt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.d=+r[0],e+r[0].length):-1}function ne(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+3));return r?(n.j=+r[0],e+r[0].length):-1}function te(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.H=+r[0],e+r[0].length):-1}function ee(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.M=+r[0],e+r[0].length):-1}function re(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.S=+r[0],e+r[0].length):-1}function ue(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+3));return r?(n.L=+r[0],e+r[0].length):-1}function ie(n){var t=n.getTimezoneOffset(),e=t>0?"-":"+",r=ga(t)/60|0,u=ga(t)%60;return e+It(r,"0",2)+It(u,"0",2)}function oe(n,t,e){hc.lastIndex=0;var r=hc.exec(t.slice(e,e+1));return r?e+r[0].length:-1}function ae(n){for(var t=n.length,e=-1;++e<t;)n[e][0]=this(n[e][0]);return function(t){for(var e=0,r=n[e];!r[1](t);)r=n[++e];return r[0](t)}}function ce(){}function le(n,t,e){var r=e.s=n+t,u=r-n,i=r-u;e.t=n-i+(t-u)}function se(n,t){n&&dc.hasOwnProperty(n.type)&&dc[n.type](n,t)}function fe(n,t,e){var r,u=-1,i=n.length-e;for(t.lineStart();++u<i;)r=n[u],t.point(r[0],r[1],r[2]);t.lineEnd()}function he(n,t){var e=-1,r=n.length;for(t.polygonStart();++e<r;)fe(n[e],t,1);t.polygonEnd()}function ge(){function n(n,t){n*=Da,t=t*Da/2+qa/4;var e=n-r,o=e>=0?1:-1,a=o*e,c=Math.cos(t),l=Math.sin(t),s=i*l,f=u*c+s*Math.cos(a),h=s*o*Math.sin(a);yc.add(Math.atan2(h,f)),r=n,u=c,i=l}var t,e,r,u,i;Mc.point=function(o,a){Mc.point=n,r=(t=o)*Da,u=Math.cos(a=(e=a)*Da/2+qa/4),i=Math.sin(a)},Mc.lineEnd=function(){n(t,e)}}function pe(n){var t=n[0],e=n[1],r=Math.cos(e);return[r*Math.cos(t),r*Math.sin(t),Math.sin(e)]}function ve(n,t){return n[0]*t[0]+n[1]*t[1]+n[2]*t[2]}function de(n,t){return[n[1]*t[2]-n[2]*t[1],n[2]*t[0]-n[0]*t[2],n[0]*t[1]-n[1]*t[0]]}function me(n,t){n[0]+=t[0],n[1]+=t[1],n[2]+=t[2]}function ye(n,t){return[n[0]*t,n[1]*t,n[2]*t]}function Me(n){var t=Math.sqrt(n[0]*n[0]+n[1]*n[1]+n[2]*n[2]);n[0]/=t,n[1]/=t,n[2]/=t}function xe(n){return[Math.atan2(n[1],n[0]),tt(n[2])]}function be(n,t){return ga(n[0]-t[0])<Ca&&ga(n[1]-t[1])<Ca}function _e(n,t){n*=Da;var e=Math.cos(t*=Da);we(e*Math.cos(n),e*Math.sin(n),Math.sin(t))}function we(n,t,e){++xc,_c+=(n-_c)/xc,wc+=(t-wc)/xc,Sc+=(e-Sc)/xc}function Se(){function n(n,u){n*=Da;var i=Math.cos(u*=Da),o=i*Math.cos(n),a=i*Math.sin(n),c=Math.sin(u),l=Math.atan2(Math.sqrt((l=e*c-r*a)*l+(l=r*o-t*c)*l+(l=t*a-e*o)*l),t*o+e*a+r*c);bc+=l,kc+=l*(t+(t=o)),Ec+=l*(e+(e=a)),Ac+=l*(r+(r=c)),we(t,e,r)}var t,e,r;qc.point=function(u,i){u*=Da;var o=Math.cos(i*=Da);t=o*Math.cos(u),e=o*Math.sin(u),r=Math.sin(i),qc.point=n,we(t,e,r)}}function ke(){qc.point=_e}function Ee(){function n(n,t){n*=Da;var e=Math.cos(t*=Da),o=e*Math.cos(n),a=e*Math.sin(n),c=Math.sin(t),l=u*c-i*a,s=i*o-r*c,f=r*a-u*o,h=Math.sqrt(l*l+s*s+f*f),g=r*o+u*a+i*c,p=h&&-nt(g)/h,v=Math.atan2(h,g);Nc+=p*l,Cc+=p*s,zc+=p*f,bc+=v,kc+=v*(r+(r=o)),Ec+=v*(u+(u=a)),Ac+=v*(i+(i=c)),we(r,u,i)}var t,e,r,u,i;qc.point=function(o,a){t=o,e=a,qc.point=n,o*=Da;var c=Math.cos(a*=Da);r=c*Math.cos(o),u=c*Math.sin(o),i=Math.sin(a),we(r,u,i)},qc.lineEnd=function(){n(t,e),qc.lineEnd=ke,qc.point=_e}}function Ae(n,t){function e(e,r){return e=n(e,r),t(e[0],e[1])}return n.invert&&t.invert&&(e.invert=function(e,r){return e=t.invert(e,r),e&&n.invert(e[0],e[1])}),e}function Ne(){return!0}function Ce(n,t,e,r,u){var i=[],o=[];if(n.forEach(function(n){if(!((t=n.length-1)<=0)){var t,e=n[0],r=n[t];if(be(e,r)){u.lineStart();for(var a=0;t>a;++a)u.point((e=n[a])[0],e[1]);return void u.lineEnd()}var c=new qe(e,n,null,!0),l=new qe(e,null,c,!1);c.o=l,i.push(c),o.push(l),c=new qe(r,n,null,!1),l=new qe(r,null,c,!0),c.o=l,i.push(c),o.push(l)}}),o.sort(t),ze(i),ze(o),i.length){for(var a=0,c=e,l=o.length;l>a;++a)o[a].e=c=!c;for(var s,f,h=i[0];;){for(var g=h,p=!0;g.v;)if((g=g.n)===h)return;s=g.z,u.lineStart();do{if(g.v=g.o.v=!0,g.e){if(p)for(var a=0,l=s.length;l>a;++a)u.point((f=s[a])[0],f[1]);else r(g.x,g.n.x,1,u);g=g.n}else{if(p){s=g.p.z;for(var a=s.length-1;a>=0;--a)u.point((f=s[a])[0],f[1])}else r(g.x,g.p.x,-1,u);g=g.p}g=g.o,s=g.z,p=!p}while(!g.v);u.lineEnd()}}}function ze(n){if(t=n.length){for(var t,e,r=0,u=n[0];++r<t;)u.n=e=n[r],e.p=u,u=e;u.n=e=n[0],e.p=u}}function qe(n,t,e,r){this.x=n,this.z=t,this.o=e,this.e=r,this.v=!1,this.n=this.p=null}function Le(n,t,e,r){return function(u,i){function o(t,e){var r=u(t,e);n(t=r[0],e=r[1])&&i.point(t,e)}function a(n,t){var e=u(n,t);d.point(e[0],e[1])}function c(){y.point=a,d.lineStart()}function l(){y.point=o,d.lineEnd()}function s(n,t){v.push([n,t]);var e=u(n,t);x.point(e[0],e[1])}function f(){x.lineStart(),v=[]}function h(){s(v[0][0],v[0][1]),x.lineEnd();var n,t=x.clean(),e=M.buffer(),r=e.length;if(v.pop(),p.push(v),v=null,r)if(1&t){n=e[0];var u,r=n.length-1,o=-1;if(r>0){for(b||(i.polygonStart(),b=!0),i.lineStart();++o<r;)i.point((u=n[o])[0],u[1]);i.lineEnd()}}else r>1&&2&t&&e.push(e.pop().concat(e.shift())),g.push(e.filter(Te))}var g,p,v,d=t(i),m=u.invert(r[0],r[1]),y={point:o,lineStart:c,lineEnd:l,polygonStart:function(){y.point=s,y.lineStart=f,y.lineEnd=h,g=[],p=[]},polygonEnd:function(){y.point=o,y.lineStart=c,y.lineEnd=l,g=ta.merge(g);var n=Fe(m,p);g.length?(b||(i.polygonStart(),b=!0),Ce(g,De,n,e,i)):n&&(b||(i.polygonStart(),b=!0),i.lineStart(),e(null,null,1,i),i.lineEnd()),b&&(i.polygonEnd(),b=!1),g=p=null},sphere:function(){i.polygonStart(),i.lineStart(),e(null,null,1,i),i.lineEnd(),i.polygonEnd()}},M=Re(),x=t(M),b=!1;return y}}function Te(n){return n.length>1}function Re(){var n,t=[];return{lineStart:function(){t.push(n=[])},point:function(t,e){n.push([t,e])},lineEnd:b,buffer:function(){var e=t;return t=[],n=null,e},rejoin:function(){t.length>1&&t.push(t.pop().concat(t.shift()))}}}function De(n,t){return((n=n.x)[0]<0?n[1]-Ra-Ca:Ra-n[1])-((t=t.x)[0]<0?t[1]-Ra-Ca:Ra-t[1])}function Pe(n){var t,e=0/0,r=0/0,u=0/0;return{lineStart:function(){n.lineStart(),t=1},point:function(i,o){var a=i>0?qa:-qa,c=ga(i-e);ga(c-qa)<Ca?(n.point(e,r=(r+o)/2>0?Ra:-Ra),n.point(u,r),n.lineEnd(),n.lineStart(),n.point(a,r),n.point(i,r),t=0):u!==a&&c>=qa&&(ga(e-u)<Ca&&(e-=u*Ca),ga(i-a)<Ca&&(i-=a*Ca),r=Ue(e,r,i,o),n.point(u,r),n.lineEnd(),n.lineStart(),n.point(a,r),t=0),n.point(e=i,r=o),u=a},lineEnd:function(){n.lineEnd(),e=r=0/0},clean:function(){return 2-t}}}function Ue(n,t,e,r){var u,i,o=Math.sin(n-e);return ga(o)>Ca?Math.atan((Math.sin(t)*(i=Math.cos(r))*Math.sin(e)-Math.sin(r)*(u=Math.cos(t))*Math.sin(n))/(u*i*o)):(t+r)/2}function je(n,t,e,r){var u;if(null==n)u=e*Ra,r.point(-qa,u),r.point(0,u),r.point(qa,u),r.point(qa,0),r.point(qa,-u),r.point(0,-u),r.point(-qa,-u),r.point(-qa,0),r.point(-qa,u);else if(ga(n[0]-t[0])>Ca){var i=n[0]<t[0]?qa:-qa;u=e*i/2,r.point(-i,u),r.point(0,u),r.point(i,u)}else r.point(t[0],t[1])}function Fe(n,t){var e=n[0],r=n[1],u=[Math.sin(e),-Math.cos(e),0],i=0,o=0;yc.reset();for(var a=0,c=t.length;c>a;++a){var l=t[a],s=l.length;if(s)for(var f=l[0],h=f[0],g=f[1]/2+qa/4,p=Math.sin(g),v=Math.cos(g),d=1;;){d===s&&(d=0),n=l[d];var m=n[0],y=n[1]/2+qa/4,M=Math.sin(y),x=Math.cos(y),b=m-h,_=b>=0?1:-1,w=_*b,S=w>qa,k=p*M;if(yc.add(Math.atan2(k*_*Math.sin(w),v*x+k*Math.cos(w))),i+=S?b+_*La:b,S^h>=e^m>=e){var E=de(pe(f),pe(n));Me(E);var A=de(u,E);Me(A);var N=(S^b>=0?-1:1)*tt(A[2]);(r>N||r===N&&(E[0]||E[1]))&&(o+=S^b>=0?1:-1)}if(!d++)break;h=m,p=M,v=x,f=n}}return(-Ca>i||Ca>i&&0>yc)^1&o}function He(n){function t(n,t){return Math.cos(n)*Math.cos(t)>i}function e(n){var e,i,c,l,s;return{lineStart:function(){l=c=!1,s=1},point:function(f,h){var g,p=[f,h],v=t(f,h),d=o?v?0:u(f,h):v?u(f+(0>f?qa:-qa),h):0;if(!e&&(l=c=v)&&n.lineStart(),v!==c&&(g=r(e,p),(be(e,g)||be(p,g))&&(p[0]+=Ca,p[1]+=Ca,v=t(p[0],p[1]))),v!==c)s=0,v?(n.lineStart(),g=r(p,e),n.point(g[0],g[1])):(g=r(e,p),n.point(g[0],g[1]),n.lineEnd()),e=g;else if(a&&e&&o^v){var m;d&i||!(m=r(p,e,!0))||(s=0,o?(n.lineStart(),n.point(m[0][0],m[0][1]),n.point(m[1][0],m[1][1]),n.lineEnd()):(n.point(m[1][0],m[1][1]),n.lineEnd(),n.lineStart(),n.point(m[0][0],m[0][1])))}!v||e&&be(e,p)||n.point(p[0],p[1]),e=p,c=v,i=d},lineEnd:function(){c&&n.lineEnd(),e=null},clean:function(){return s|(l&&c)<<1}}}function r(n,t,e){var r=pe(n),u=pe(t),o=[1,0,0],a=de(r,u),c=ve(a,a),l=a[0],s=c-l*l;if(!s)return!e&&n;var f=i*c/s,h=-i*l/s,g=de(o,a),p=ye(o,f),v=ye(a,h);me(p,v);var d=g,m=ve(p,d),y=ve(d,d),M=m*m-y*(ve(p,p)-1);if(!(0>M)){var x=Math.sqrt(M),b=ye(d,(-m-x)/y);if(me(b,p),b=xe(b),!e)return b;var _,w=n[0],S=t[0],k=n[1],E=t[1];w>S&&(_=w,w=S,S=_);var A=S-w,N=ga(A-qa)<Ca,C=N||Ca>A;if(!N&&k>E&&(_=k,k=E,E=_),C?N?k+E>0^b[1]<(ga(b[0]-w)<Ca?k:E):k<=b[1]&&b[1]<=E:A>qa^(w<=b[0]&&b[0]<=S)){var z=ye(d,(-m+x)/y);return me(z,p),[b,xe(z)]}}}function u(t,e){var r=o?n:qa-n,u=0;return-r>t?u|=1:t>r&&(u|=2),-r>e?u|=4:e>r&&(u|=8),u}var i=Math.cos(n),o=i>0,a=ga(i)>Ca,c=gr(n,6*Da);return Le(t,e,c,o?[0,-n]:[-qa,n-qa])}function Oe(n,t,e,r){return function(u){var i,o=u.a,a=u.b,c=o.x,l=o.y,s=a.x,f=a.y,h=0,g=1,p=s-c,v=f-l;if(i=n-c,p||!(i>0)){if(i/=p,0>p){if(h>i)return;g>i&&(g=i)}else if(p>0){if(i>g)return;i>h&&(h=i)}if(i=e-c,p||!(0>i)){if(i/=p,0>p){if(i>g)return;i>h&&(h=i)}else if(p>0){if(h>i)return;g>i&&(g=i)}if(i=t-l,v||!(i>0)){if(i/=v,0>v){if(h>i)return;g>i&&(g=i)}else if(v>0){if(i>g)return;i>h&&(h=i)}if(i=r-l,v||!(0>i)){if(i/=v,0>v){if(i>g)return;i>h&&(h=i)}else if(v>0){if(h>i)return;g>i&&(g=i)}return h>0&&(u.a={x:c+h*p,y:l+h*v}),1>g&&(u.b={x:c+g*p,y:l+g*v}),u}}}}}}function Ie(n,t,e,r){function u(r,u){return ga(r[0]-n)<Ca?u>0?0:3:ga(r[0]-e)<Ca?u>0?2:1:ga(r[1]-t)<Ca?u>0?1:0:u>0?3:2}function i(n,t){return o(n.x,t.x)}function o(n,t){var e=u(n,1),r=u(t,1);return e!==r?e-r:0===e?t[1]-n[1]:1===e?n[0]-t[0]:2===e?n[1]-t[1]:t[0]-n[0]}return function(a){function c(n){for(var t=0,e=d.length,r=n[1],u=0;e>u;++u)for(var i,o=1,a=d[u],c=a.length,l=a[0];c>o;++o)i=a[o],l[1]<=r?i[1]>r&&Q(l,i,n)>0&&++t:i[1]<=r&&Q(l,i,n)<0&&--t,l=i;return 0!==t}function l(i,a,c,l){var s=0,f=0;if(null==i||(s=u(i,c))!==(f=u(a,c))||o(i,a)<0^c>0){do l.point(0===s||3===s?n:e,s>1?r:t);while((s=(s+c+4)%4)!==f)}else l.point(a[0],a[1])}function s(u,i){return u>=n&&e>=u&&i>=t&&r>=i}function f(n,t){s(n,t)&&a.point(n,t)}function h(){C.point=p,d&&d.push(m=[]),S=!0,w=!1,b=_=0/0}function g(){v&&(p(y,M),x&&w&&A.rejoin(),v.push(A.buffer())),C.point=f,w&&a.lineEnd()}function p(n,t){n=Math.max(-Tc,Math.min(Tc,n)),t=Math.max(-Tc,Math.min(Tc,t));var e=s(n,t);if(d&&m.push([n,t]),S)y=n,M=t,x=e,S=!1,e&&(a.lineStart(),a.point(n,t));else if(e&&w)a.point(n,t);else{var r={a:{x:b,y:_},b:{x:n,y:t}};N(r)?(w||(a.lineStart(),a.point(r.a.x,r.a.y)),a.point(r.b.x,r.b.y),e||a.lineEnd(),k=!1):e&&(a.lineStart(),a.point(n,t),k=!1)}b=n,_=t,w=e}var v,d,m,y,M,x,b,_,w,S,k,E=a,A=Re(),N=Oe(n,t,e,r),C={point:f,lineStart:h,lineEnd:g,polygonStart:function(){a=A,v=[],d=[],k=!0},polygonEnd:function(){a=E,v=ta.merge(v);var t=c([n,r]),e=k&&t,u=v.length;(e||u)&&(a.polygonStart(),e&&(a.lineStart(),l(null,null,1,a),a.lineEnd()),u&&Ce(v,i,t,l,a),a.polygonEnd()),v=d=m=null}};return C}}function Ye(n){var t=0,e=qa/3,r=ir(n),u=r(t,e);return u.parallels=function(n){return arguments.length?r(t=n[0]*qa/180,e=n[1]*qa/180):[t/qa*180,e/qa*180]},u}function Ze(n,t){function e(n,t){var e=Math.sqrt(i-2*u*Math.sin(t))/u;return[e*Math.sin(n*=u),o-e*Math.cos(n)]}var r=Math.sin(n),u=(r+Math.sin(t))/2,i=1+r*(2*u-r),o=Math.sqrt(i)/u;return e.invert=function(n,t){var e=o-t;return[Math.atan2(n,e)/u,tt((i-(n*n+e*e)*u*u)/(2*u))]},e}function Ve(){function n(n,t){Dc+=u*n-r*t,r=n,u=t}var t,e,r,u;Hc.point=function(i,o){Hc.point=n,t=r=i,e=u=o},Hc.lineEnd=function(){n(t,e)}}function Xe(n,t){Pc>n&&(Pc=n),n>jc&&(jc=n),Uc>t&&(Uc=t),t>Fc&&(Fc=t)}function $e(){function n(n,t){o.push("M",n,",",t,i)}function t(n,t){o.push("M",n,",",t),a.point=e}function e(n,t){o.push("L",n,",",t)}function r(){a.point=n}function u(){o.push("Z")}var i=Be(4.5),o=[],a={point:n,lineStart:function(){a.point=t},lineEnd:r,polygonStart:function(){a.lineEnd=u},polygonEnd:function(){a.lineEnd=r,a.point=n},pointRadius:function(n){return i=Be(n),a},result:function(){if(o.length){var n=o.join("");return o=[],n}}};return a}function Be(n){return"m0,"+n+"a"+n+","+n+" 0 1,1 0,"+-2*n+"a"+n+","+n+" 0 1,1 0,"+2*n+"z"}function We(n,t){_c+=n,wc+=t,++Sc}function Je(){function n(n,r){var u=n-t,i=r-e,o=Math.sqrt(u*u+i*i);kc+=o*(t+n)/2,Ec+=o*(e+r)/2,Ac+=o,We(t=n,e=r)}var t,e;Ic.point=function(r,u){Ic.point=n,We(t=r,e=u)}}function Ge(){Ic.point=We}function Ke(){function n(n,t){var e=n-r,i=t-u,o=Math.sqrt(e*e+i*i);kc+=o*(r+n)/2,Ec+=o*(u+t)/2,Ac+=o,o=u*n-r*t,Nc+=o*(r+n),Cc+=o*(u+t),zc+=3*o,We(r=n,u=t)}var t,e,r,u;Ic.point=function(i,o){Ic.point=n,We(t=r=i,e=u=o)},Ic.lineEnd=function(){n(t,e)}}function Qe(n){function t(t,e){n.moveTo(t+o,e),n.arc(t,e,o,0,La)}function e(t,e){n.moveTo(t,e),a.point=r}function r(t,e){n.lineTo(t,e)}function u(){a.point=t}function i(){n.closePath()}var o=4.5,a={point:t,lineStart:function(){a.point=e},lineEnd:u,polygonStart:function(){a.lineEnd=i},polygonEnd:function(){a.lineEnd=u,a.point=t},pointRadius:function(n){return o=n,a},result:b};return a}function nr(n){function t(n){return(a?r:e)(n)}function e(t){return rr(t,function(e,r){e=n(e,r),t.point(e[0],e[1])})}function r(t){function e(e,r){e=n(e,r),t.point(e[0],e[1])}function r(){M=0/0,S.point=i,t.lineStart()}function i(e,r){var i=pe([e,r]),o=n(e,r);u(M,x,y,b,_,w,M=o[0],x=o[1],y=e,b=i[0],_=i[1],w=i[2],a,t),t.point(M,x)}function o(){S.point=e,t.lineEnd()}function c(){r(),S.point=l,S.lineEnd=s}function l(n,t){i(f=n,h=t),g=M,p=x,v=b,d=_,m=w,S.point=i}function s(){u(M,x,y,b,_,w,g,p,f,v,d,m,a,t),S.lineEnd=o,o()}var f,h,g,p,v,d,m,y,M,x,b,_,w,S={point:e,lineStart:r,lineEnd:o,polygonStart:function(){t.polygonStart(),S.lineStart=c
},polygonEnd:function(){t.polygonEnd(),S.lineStart=r}};return S}function u(t,e,r,a,c,l,s,f,h,g,p,v,d,m){var y=s-t,M=f-e,x=y*y+M*M;if(x>4*i&&d--){var b=a+g,_=c+p,w=l+v,S=Math.sqrt(b*b+_*_+w*w),k=Math.asin(w/=S),E=ga(ga(w)-1)<Ca||ga(r-h)<Ca?(r+h)/2:Math.atan2(_,b),A=n(E,k),N=A[0],C=A[1],z=N-t,q=C-e,L=M*z-y*q;(L*L/x>i||ga((y*z+M*q)/x-.5)>.3||o>a*g+c*p+l*v)&&(u(t,e,r,a,c,l,N,C,E,b/=S,_/=S,w,d,m),m.point(N,C),u(N,C,E,b,_,w,s,f,h,g,p,v,d,m))}}var i=.5,o=Math.cos(30*Da),a=16;return t.precision=function(n){return arguments.length?(a=(i=n*n)>0&&16,t):Math.sqrt(i)},t}function tr(n){var t=nr(function(t,e){return n([t*Pa,e*Pa])});return function(n){return or(t(n))}}function er(n){this.stream=n}function rr(n,t){return{point:t,sphere:function(){n.sphere()},lineStart:function(){n.lineStart()},lineEnd:function(){n.lineEnd()},polygonStart:function(){n.polygonStart()},polygonEnd:function(){n.polygonEnd()}}}function ur(n){return ir(function(){return n})()}function ir(n){function t(n){return n=a(n[0]*Da,n[1]*Da),[n[0]*h+c,l-n[1]*h]}function e(n){return n=a.invert((n[0]-c)/h,(l-n[1])/h),n&&[n[0]*Pa,n[1]*Pa]}function r(){a=Ae(o=lr(m,M,x),i);var n=i(v,d);return c=g-n[0]*h,l=p+n[1]*h,u()}function u(){return s&&(s.valid=!1,s=null),t}var i,o,a,c,l,s,f=nr(function(n,t){return n=i(n,t),[n[0]*h+c,l-n[1]*h]}),h=150,g=480,p=250,v=0,d=0,m=0,M=0,x=0,b=Lc,_=y,w=null,S=null;return t.stream=function(n){return s&&(s.valid=!1),s=or(b(o,f(_(n)))),s.valid=!0,s},t.clipAngle=function(n){return arguments.length?(b=null==n?(w=n,Lc):He((w=+n)*Da),u()):w},t.clipExtent=function(n){return arguments.length?(S=n,_=n?Ie(n[0][0],n[0][1],n[1][0],n[1][1]):y,u()):S},t.scale=function(n){return arguments.length?(h=+n,r()):h},t.translate=function(n){return arguments.length?(g=+n[0],p=+n[1],r()):[g,p]},t.center=function(n){return arguments.length?(v=n[0]%360*Da,d=n[1]%360*Da,r()):[v*Pa,d*Pa]},t.rotate=function(n){return arguments.length?(m=n[0]%360*Da,M=n[1]%360*Da,x=n.length>2?n[2]%360*Da:0,r()):[m*Pa,M*Pa,x*Pa]},ta.rebind(t,f,"precision"),function(){return i=n.apply(this,arguments),t.invert=i.invert&&e,r()}}function or(n){return rr(n,function(t,e){n.point(t*Da,e*Da)})}function ar(n,t){return[n,t]}function cr(n,t){return[n>qa?n-La:-qa>n?n+La:n,t]}function lr(n,t,e){return n?t||e?Ae(fr(n),hr(t,e)):fr(n):t||e?hr(t,e):cr}function sr(n){return function(t,e){return t+=n,[t>qa?t-La:-qa>t?t+La:t,e]}}function fr(n){var t=sr(n);return t.invert=sr(-n),t}function hr(n,t){function e(n,t){var e=Math.cos(t),a=Math.cos(n)*e,c=Math.sin(n)*e,l=Math.sin(t),s=l*r+a*u;return[Math.atan2(c*i-s*o,a*r-l*u),tt(s*i+c*o)]}var r=Math.cos(n),u=Math.sin(n),i=Math.cos(t),o=Math.sin(t);return e.invert=function(n,t){var e=Math.cos(t),a=Math.cos(n)*e,c=Math.sin(n)*e,l=Math.sin(t),s=l*i-c*o;return[Math.atan2(c*i+l*o,a*r+s*u),tt(s*r-a*u)]},e}function gr(n,t){var e=Math.cos(n),r=Math.sin(n);return function(u,i,o,a){var c=o*t;null!=u?(u=pr(e,u),i=pr(e,i),(o>0?i>u:u>i)&&(u+=o*La)):(u=n+o*La,i=n-.5*c);for(var l,s=u;o>0?s>i:i>s;s-=c)a.point((l=xe([e,-r*Math.cos(s),-r*Math.sin(s)]))[0],l[1])}}function pr(n,t){var e=pe(t);e[0]-=n,Me(e);var r=nt(-e[1]);return((-e[2]<0?-r:r)+2*Math.PI-Ca)%(2*Math.PI)}function vr(n,t,e){var r=ta.range(n,t-Ca,e).concat(t);return function(n){return r.map(function(t){return[n,t]})}}function dr(n,t,e){var r=ta.range(n,t-Ca,e).concat(t);return function(n){return r.map(function(t){return[t,n]})}}function mr(n){return n.source}function yr(n){return n.target}function Mr(n,t,e,r){var u=Math.cos(t),i=Math.sin(t),o=Math.cos(r),a=Math.sin(r),c=u*Math.cos(n),l=u*Math.sin(n),s=o*Math.cos(e),f=o*Math.sin(e),h=2*Math.asin(Math.sqrt(it(r-t)+u*o*it(e-n))),g=1/Math.sin(h),p=h?function(n){var t=Math.sin(n*=h)*g,e=Math.sin(h-n)*g,r=e*c+t*s,u=e*l+t*f,o=e*i+t*a;return[Math.atan2(u,r)*Pa,Math.atan2(o,Math.sqrt(r*r+u*u))*Pa]}:function(){return[n*Pa,t*Pa]};return p.distance=h,p}function xr(){function n(n,u){var i=Math.sin(u*=Da),o=Math.cos(u),a=ga((n*=Da)-t),c=Math.cos(a);Yc+=Math.atan2(Math.sqrt((a=o*Math.sin(a))*a+(a=r*i-e*o*c)*a),e*i+r*o*c),t=n,e=i,r=o}var t,e,r;Zc.point=function(u,i){t=u*Da,e=Math.sin(i*=Da),r=Math.cos(i),Zc.point=n},Zc.lineEnd=function(){Zc.point=Zc.lineEnd=b}}function br(n,t){function e(t,e){var r=Math.cos(t),u=Math.cos(e),i=n(r*u);return[i*u*Math.sin(t),i*Math.sin(e)]}return e.invert=function(n,e){var r=Math.sqrt(n*n+e*e),u=t(r),i=Math.sin(u),o=Math.cos(u);return[Math.atan2(n*i,r*o),Math.asin(r&&e*i/r)]},e}function _r(n,t){function e(n,t){o>0?-Ra+Ca>t&&(t=-Ra+Ca):t>Ra-Ca&&(t=Ra-Ca);var e=o/Math.pow(u(t),i);return[e*Math.sin(i*n),o-e*Math.cos(i*n)]}var r=Math.cos(n),u=function(n){return Math.tan(qa/4+n/2)},i=n===t?Math.sin(n):Math.log(r/Math.cos(t))/Math.log(u(t)/u(n)),o=r*Math.pow(u(n),i)/i;return i?(e.invert=function(n,t){var e=o-t,r=K(i)*Math.sqrt(n*n+e*e);return[Math.atan2(n,e)/i,2*Math.atan(Math.pow(o/r,1/i))-Ra]},e):Sr}function wr(n,t){function e(n,t){var e=i-t;return[e*Math.sin(u*n),i-e*Math.cos(u*n)]}var r=Math.cos(n),u=n===t?Math.sin(n):(r-Math.cos(t))/(t-n),i=r/u+n;return ga(u)<Ca?ar:(e.invert=function(n,t){var e=i-t;return[Math.atan2(n,e)/u,i-K(u)*Math.sqrt(n*n+e*e)]},e)}function Sr(n,t){return[n,Math.log(Math.tan(qa/4+t/2))]}function kr(n){var t,e=ur(n),r=e.scale,u=e.translate,i=e.clipExtent;return e.scale=function(){var n=r.apply(e,arguments);return n===e?t?e.clipExtent(null):e:n},e.translate=function(){var n=u.apply(e,arguments);return n===e?t?e.clipExtent(null):e:n},e.clipExtent=function(n){var o=i.apply(e,arguments);if(o===e){if(t=null==n){var a=qa*r(),c=u();i([[c[0]-a,c[1]-a],[c[0]+a,c[1]+a]])}}else t&&(o=null);return o},e.clipExtent(null)}function Er(n,t){return[Math.log(Math.tan(qa/4+t/2)),-n]}function Ar(n){return n[0]}function Nr(n){return n[1]}function Cr(n){for(var t=n.length,e=[0,1],r=2,u=2;t>u;u++){for(;r>1&&Q(n[e[r-2]],n[e[r-1]],n[u])<=0;)--r;e[r++]=u}return e.slice(0,r)}function zr(n,t){return n[0]-t[0]||n[1]-t[1]}function qr(n,t,e){return(e[0]-t[0])*(n[1]-t[1])<(e[1]-t[1])*(n[0]-t[0])}function Lr(n,t,e,r){var u=n[0],i=e[0],o=t[0]-u,a=r[0]-i,c=n[1],l=e[1],s=t[1]-c,f=r[1]-l,h=(a*(c-l)-f*(u-i))/(f*o-a*s);return[u+h*o,c+h*s]}function Tr(n){var t=n[0],e=n[n.length-1];return!(t[0]-e[0]||t[1]-e[1])}function Rr(){tu(this),this.edge=this.site=this.circle=null}function Dr(n){var t=el.pop()||new Rr;return t.site=n,t}function Pr(n){Xr(n),Qc.remove(n),el.push(n),tu(n)}function Ur(n){var t=n.circle,e=t.x,r=t.cy,u={x:e,y:r},i=n.P,o=n.N,a=[n];Pr(n);for(var c=i;c.circle&&ga(e-c.circle.x)<Ca&&ga(r-c.circle.cy)<Ca;)i=c.P,a.unshift(c),Pr(c),c=i;a.unshift(c),Xr(c);for(var l=o;l.circle&&ga(e-l.circle.x)<Ca&&ga(r-l.circle.cy)<Ca;)o=l.N,a.push(l),Pr(l),l=o;a.push(l),Xr(l);var s,f=a.length;for(s=1;f>s;++s)l=a[s],c=a[s-1],Kr(l.edge,c.site,l.site,u);c=a[0],l=a[f-1],l.edge=Jr(c.site,l.site,null,u),Vr(c),Vr(l)}function jr(n){for(var t,e,r,u,i=n.x,o=n.y,a=Qc._;a;)if(r=Fr(a,o)-i,r>Ca)a=a.L;else{if(u=i-Hr(a,o),!(u>Ca)){r>-Ca?(t=a.P,e=a):u>-Ca?(t=a,e=a.N):t=e=a;break}if(!a.R){t=a;break}a=a.R}var c=Dr(n);if(Qc.insert(t,c),t||e){if(t===e)return Xr(t),e=Dr(t.site),Qc.insert(c,e),c.edge=e.edge=Jr(t.site,c.site),Vr(t),void Vr(e);if(!e)return void(c.edge=Jr(t.site,c.site));Xr(t),Xr(e);var l=t.site,s=l.x,f=l.y,h=n.x-s,g=n.y-f,p=e.site,v=p.x-s,d=p.y-f,m=2*(h*d-g*v),y=h*h+g*g,M=v*v+d*d,x={x:(d*y-g*M)/m+s,y:(h*M-v*y)/m+f};Kr(e.edge,l,p,x),c.edge=Jr(l,n,null,x),e.edge=Jr(n,p,null,x),Vr(t),Vr(e)}}function Fr(n,t){var e=n.site,r=e.x,u=e.y,i=u-t;if(!i)return r;var o=n.P;if(!o)return-1/0;e=o.site;var a=e.x,c=e.y,l=c-t;if(!l)return a;var s=a-r,f=1/i-1/l,h=s/l;return f?(-h+Math.sqrt(h*h-2*f*(s*s/(-2*l)-c+l/2+u-i/2)))/f+r:(r+a)/2}function Hr(n,t){var e=n.N;if(e)return Fr(e,t);var r=n.site;return r.y===t?r.x:1/0}function Or(n){this.site=n,this.edges=[]}function Ir(n){for(var t,e,r,u,i,o,a,c,l,s,f=n[0][0],h=n[1][0],g=n[0][1],p=n[1][1],v=Kc,d=v.length;d--;)if(i=v[d],i&&i.prepare())for(a=i.edges,c=a.length,o=0;c>o;)s=a[o].end(),r=s.x,u=s.y,l=a[++o%c].start(),t=l.x,e=l.y,(ga(r-t)>Ca||ga(u-e)>Ca)&&(a.splice(o,0,new Qr(Gr(i.site,s,ga(r-f)<Ca&&p-u>Ca?{x:f,y:ga(t-f)<Ca?e:p}:ga(u-p)<Ca&&h-r>Ca?{x:ga(e-p)<Ca?t:h,y:p}:ga(r-h)<Ca&&u-g>Ca?{x:h,y:ga(t-h)<Ca?e:g}:ga(u-g)<Ca&&r-f>Ca?{x:ga(e-g)<Ca?t:f,y:g}:null),i.site,null)),++c)}function Yr(n,t){return t.angle-n.angle}function Zr(){tu(this),this.x=this.y=this.arc=this.site=this.cy=null}function Vr(n){var t=n.P,e=n.N;if(t&&e){var r=t.site,u=n.site,i=e.site;if(r!==i){var o=u.x,a=u.y,c=r.x-o,l=r.y-a,s=i.x-o,f=i.y-a,h=2*(c*f-l*s);if(!(h>=-za)){var g=c*c+l*l,p=s*s+f*f,v=(f*g-l*p)/h,d=(c*p-s*g)/h,f=d+a,m=rl.pop()||new Zr;m.arc=n,m.site=u,m.x=v+o,m.y=f+Math.sqrt(v*v+d*d),m.cy=f,n.circle=m;for(var y=null,M=tl._;M;)if(m.y<M.y||m.y===M.y&&m.x<=M.x){if(!M.L){y=M.P;break}M=M.L}else{if(!M.R){y=M;break}M=M.R}tl.insert(y,m),y||(nl=m)}}}}function Xr(n){var t=n.circle;t&&(t.P||(nl=t.N),tl.remove(t),rl.push(t),tu(t),n.circle=null)}function $r(n){for(var t,e=Gc,r=Oe(n[0][0],n[0][1],n[1][0],n[1][1]),u=e.length;u--;)t=e[u],(!Br(t,n)||!r(t)||ga(t.a.x-t.b.x)<Ca&&ga(t.a.y-t.b.y)<Ca)&&(t.a=t.b=null,e.splice(u,1))}function Br(n,t){var e=n.b;if(e)return!0;var r,u,i=n.a,o=t[0][0],a=t[1][0],c=t[0][1],l=t[1][1],s=n.l,f=n.r,h=s.x,g=s.y,p=f.x,v=f.y,d=(h+p)/2,m=(g+v)/2;if(v===g){if(o>d||d>=a)return;if(h>p){if(i){if(i.y>=l)return}else i={x:d,y:c};e={x:d,y:l}}else{if(i){if(i.y<c)return}else i={x:d,y:l};e={x:d,y:c}}}else if(r=(h-p)/(v-g),u=m-r*d,-1>r||r>1)if(h>p){if(i){if(i.y>=l)return}else i={x:(c-u)/r,y:c};e={x:(l-u)/r,y:l}}else{if(i){if(i.y<c)return}else i={x:(l-u)/r,y:l};e={x:(c-u)/r,y:c}}else if(v>g){if(i){if(i.x>=a)return}else i={x:o,y:r*o+u};e={x:a,y:r*a+u}}else{if(i){if(i.x<o)return}else i={x:a,y:r*a+u};e={x:o,y:r*o+u}}return n.a=i,n.b=e,!0}function Wr(n,t){this.l=n,this.r=t,this.a=this.b=null}function Jr(n,t,e,r){var u=new Wr(n,t);return Gc.push(u),e&&Kr(u,n,t,e),r&&Kr(u,t,n,r),Kc[n.i].edges.push(new Qr(u,n,t)),Kc[t.i].edges.push(new Qr(u,t,n)),u}function Gr(n,t,e){var r=new Wr(n,null);return r.a=t,r.b=e,Gc.push(r),r}function Kr(n,t,e,r){n.a||n.b?n.l===e?n.b=r:n.a=r:(n.a=r,n.l=t,n.r=e)}function Qr(n,t,e){var r=n.a,u=n.b;this.edge=n,this.site=t,this.angle=e?Math.atan2(e.y-t.y,e.x-t.x):n.l===t?Math.atan2(u.x-r.x,r.y-u.y):Math.atan2(r.x-u.x,u.y-r.y)}function nu(){this._=null}function tu(n){n.U=n.C=n.L=n.R=n.P=n.N=null}function eu(n,t){var e=t,r=t.R,u=e.U;u?u.L===e?u.L=r:u.R=r:n._=r,r.U=u,e.U=r,e.R=r.L,e.R&&(e.R.U=e),r.L=e}function ru(n,t){var e=t,r=t.L,u=e.U;u?u.L===e?u.L=r:u.R=r:n._=r,r.U=u,e.U=r,e.L=r.R,e.L&&(e.L.U=e),r.R=e}function uu(n){for(;n.L;)n=n.L;return n}function iu(n,t){var e,r,u,i=n.sort(ou).pop();for(Gc=[],Kc=new Array(n.length),Qc=new nu,tl=new nu;;)if(u=nl,i&&(!u||i.y<u.y||i.y===u.y&&i.x<u.x))(i.x!==e||i.y!==r)&&(Kc[i.i]=new Or(i),jr(i),e=i.x,r=i.y),i=n.pop();else{if(!u)break;Ur(u.arc)}t&&($r(t),Ir(t));var o={cells:Kc,edges:Gc};return Qc=tl=Gc=Kc=null,o}function ou(n,t){return t.y-n.y||t.x-n.x}function au(n,t,e){return(n.x-e.x)*(t.y-n.y)-(n.x-t.x)*(e.y-n.y)}function cu(n){return n.x}function lu(n){return n.y}function su(){return{leaf:!0,nodes:[],point:null,x:null,y:null}}function fu(n,t,e,r,u,i){if(!n(t,e,r,u,i)){var o=.5*(e+u),a=.5*(r+i),c=t.nodes;c[0]&&fu(n,c[0],e,r,o,a),c[1]&&fu(n,c[1],o,r,u,a),c[2]&&fu(n,c[2],e,a,o,i),c[3]&&fu(n,c[3],o,a,u,i)}}function hu(n,t,e,r,u,i,o){var a,c=1/0;return function l(n,s,f,h,g){if(!(s>i||f>o||r>h||u>g)){if(p=n.point){var p,v=t-n.x,d=e-n.y,m=v*v+d*d;if(c>m){var y=Math.sqrt(c=m);r=t-y,u=e-y,i=t+y,o=e+y,a=p}}for(var M=n.nodes,x=.5*(s+h),b=.5*(f+g),_=t>=x,w=e>=b,S=w<<1|_,k=S+4;k>S;++S)if(n=M[3&S])switch(3&S){case 0:l(n,s,f,x,b);break;case 1:l(n,x,f,h,b);break;case 2:l(n,s,b,x,g);break;case 3:l(n,x,b,h,g)}}}(n,r,u,i,o),a}function gu(n,t){n=ta.rgb(n),t=ta.rgb(t);var e=n.r,r=n.g,u=n.b,i=t.r-e,o=t.g-r,a=t.b-u;return function(n){return"#"+xt(Math.round(e+i*n))+xt(Math.round(r+o*n))+xt(Math.round(u+a*n))}}function pu(n,t){var e,r={},u={};for(e in n)e in t?r[e]=mu(n[e],t[e]):u[e]=n[e];for(e in t)e in n||(u[e]=t[e]);return function(n){for(e in r)u[e]=r[e](n);return u}}function vu(n,t){return n=+n,t=+t,function(e){return n*(1-e)+t*e}}function du(n,t){var e,r,u,i=il.lastIndex=ol.lastIndex=0,o=-1,a=[],c=[];for(n+="",t+="";(e=il.exec(n))&&(r=ol.exec(t));)(u=r.index)>i&&(u=t.slice(i,u),a[o]?a[o]+=u:a[++o]=u),(e=e[0])===(r=r[0])?a[o]?a[o]+=r:a[++o]=r:(a[++o]=null,c.push({i:o,x:vu(e,r)})),i=ol.lastIndex;return i<t.length&&(u=t.slice(i),a[o]?a[o]+=u:a[++o]=u),a.length<2?c[0]?(t=c[0].x,function(n){return t(n)+""}):function(){return t}:(t=c.length,function(n){for(var e,r=0;t>r;++r)a[(e=c[r]).i]=e.x(n);return a.join("")})}function mu(n,t){for(var e,r=ta.interpolators.length;--r>=0&&!(e=ta.interpolators[r](n,t)););return e}function yu(n,t){var e,r=[],u=[],i=n.length,o=t.length,a=Math.min(n.length,t.length);for(e=0;a>e;++e)r.push(mu(n[e],t[e]));for(;i>e;++e)u[e]=n[e];for(;o>e;++e)u[e]=t[e];return function(n){for(e=0;a>e;++e)u[e]=r[e](n);return u}}function Mu(n){return function(t){return 0>=t?0:t>=1?1:n(t)}}function xu(n){return function(t){return 1-n(1-t)}}function bu(n){return function(t){return.5*(.5>t?n(2*t):2-n(2-2*t))}}function _u(n){return n*n}function wu(n){return n*n*n}function Su(n){if(0>=n)return 0;if(n>=1)return 1;var t=n*n,e=t*n;return 4*(.5>n?e:3*(n-t)+e-.75)}function ku(n){return function(t){return Math.pow(t,n)}}function Eu(n){return 1-Math.cos(n*Ra)}function Au(n){return Math.pow(2,10*(n-1))}function Nu(n){return 1-Math.sqrt(1-n*n)}function Cu(n,t){var e;return arguments.length<2&&(t=.45),arguments.length?e=t/La*Math.asin(1/n):(n=1,e=t/4),function(r){return 1+n*Math.pow(2,-10*r)*Math.sin((r-e)*La/t)}}function zu(n){return n||(n=1.70158),function(t){return t*t*((n+1)*t-n)}}function qu(n){return 1/2.75>n?7.5625*n*n:2/2.75>n?7.5625*(n-=1.5/2.75)*n+.75:2.5/2.75>n?7.5625*(n-=2.25/2.75)*n+.9375:7.5625*(n-=2.625/2.75)*n+.984375}function Lu(n,t){n=ta.hcl(n),t=ta.hcl(t);var e=n.h,r=n.c,u=n.l,i=t.h-e,o=t.c-r,a=t.l-u;return isNaN(o)&&(o=0,r=isNaN(r)?t.c:r),isNaN(i)?(i=0,e=isNaN(e)?t.h:e):i>180?i-=360:-180>i&&(i+=360),function(n){return st(e+i*n,r+o*n,u+a*n)+""}}function Tu(n,t){n=ta.hsl(n),t=ta.hsl(t);var e=n.h,r=n.s,u=n.l,i=t.h-e,o=t.s-r,a=t.l-u;return isNaN(o)&&(o=0,r=isNaN(r)?t.s:r),isNaN(i)?(i=0,e=isNaN(e)?t.h:e):i>180?i-=360:-180>i&&(i+=360),function(n){return ct(e+i*n,r+o*n,u+a*n)+""}}function Ru(n,t){n=ta.lab(n),t=ta.lab(t);var e=n.l,r=n.a,u=n.b,i=t.l-e,o=t.a-r,a=t.b-u;return function(n){return ht(e+i*n,r+o*n,u+a*n)+""}}function Du(n,t){return t-=n,function(e){return Math.round(n+t*e)}}function Pu(n){var t=[n.a,n.b],e=[n.c,n.d],r=ju(t),u=Uu(t,e),i=ju(Fu(e,t,-u))||0;t[0]*e[1]<e[0]*t[1]&&(t[0]*=-1,t[1]*=-1,r*=-1,u*=-1),this.rotate=(r?Math.atan2(t[1],t[0]):Math.atan2(-e[0],e[1]))*Pa,this.translate=[n.e,n.f],this.scale=[r,i],this.skew=i?Math.atan2(u,i)*Pa:0}function Uu(n,t){return n[0]*t[0]+n[1]*t[1]}function ju(n){var t=Math.sqrt(Uu(n,n));return t&&(n[0]/=t,n[1]/=t),t}function Fu(n,t,e){return n[0]+=e*t[0],n[1]+=e*t[1],n}function Hu(n,t){var e,r=[],u=[],i=ta.transform(n),o=ta.transform(t),a=i.translate,c=o.translate,l=i.rotate,s=o.rotate,f=i.skew,h=o.skew,g=i.scale,p=o.scale;return a[0]!=c[0]||a[1]!=c[1]?(r.push("translate(",null,",",null,")"),u.push({i:1,x:vu(a[0],c[0])},{i:3,x:vu(a[1],c[1])})):r.push(c[0]||c[1]?"translate("+c+")":""),l!=s?(l-s>180?s+=360:s-l>180&&(l+=360),u.push({i:r.push(r.pop()+"rotate(",null,")")-2,x:vu(l,s)})):s&&r.push(r.pop()+"rotate("+s+")"),f!=h?u.push({i:r.push(r.pop()+"skewX(",null,")")-2,x:vu(f,h)}):h&&r.push(r.pop()+"skewX("+h+")"),g[0]!=p[0]||g[1]!=p[1]?(e=r.push(r.pop()+"scale(",null,",",null,")"),u.push({i:e-4,x:vu(g[0],p[0])},{i:e-2,x:vu(g[1],p[1])})):(1!=p[0]||1!=p[1])&&r.push(r.pop()+"scale("+p+")"),e=u.length,function(n){for(var t,i=-1;++i<e;)r[(t=u[i]).i]=t.x(n);return r.join("")}}function Ou(n,t){return t=(t-=n=+n)||1/t,function(e){return(e-n)/t}}function Iu(n,t){return t=(t-=n=+n)||1/t,function(e){return Math.max(0,Math.min(1,(e-n)/t))}}function Yu(n){for(var t=n.source,e=n.target,r=Vu(t,e),u=[t];t!==r;)t=t.parent,u.push(t);for(var i=u.length;e!==r;)u.splice(i,0,e),e=e.parent;return u}function Zu(n){for(var t=[],e=n.parent;null!=e;)t.push(n),n=e,e=e.parent;return t.push(n),t}function Vu(n,t){if(n===t)return n;for(var e=Zu(n),r=Zu(t),u=e.pop(),i=r.pop(),o=null;u===i;)o=u,u=e.pop(),i=r.pop();return o}function Xu(n){n.fixed|=2}function $u(n){n.fixed&=-7}function Bu(n){n.fixed|=4,n.px=n.x,n.py=n.y}function Wu(n){n.fixed&=-5}function Ju(n,t,e){var r=0,u=0;if(n.charge=0,!n.leaf)for(var i,o=n.nodes,a=o.length,c=-1;++c<a;)i=o[c],null!=i&&(Ju(i,t,e),n.charge+=i.charge,r+=i.charge*i.cx,u+=i.charge*i.cy);if(n.point){n.leaf||(n.point.x+=Math.random()-.5,n.point.y+=Math.random()-.5);var l=t*e[n.point.index];n.charge+=n.pointCharge=l,r+=l*n.point.x,u+=l*n.point.y}n.cx=r/n.charge,n.cy=u/n.charge}function Gu(n,t){return ta.rebind(n,t,"sort","children","value"),n.nodes=n,n.links=ri,n}function Ku(n,t){for(var e=[n];null!=(n=e.pop());)if(t(n),(u=n.children)&&(r=u.length))for(var r,u;--r>=0;)e.push(u[r])}function Qu(n,t){for(var e=[n],r=[];null!=(n=e.pop());)if(r.push(n),(i=n.children)&&(u=i.length))for(var u,i,o=-1;++o<u;)e.push(i[o]);for(;null!=(n=r.pop());)t(n)}function ni(n){return n.children}function ti(n){return n.value}function ei(n,t){return t.value-n.value}function ri(n){return ta.merge(n.map(function(n){return(n.children||[]).map(function(t){return{source:n,target:t}})}))}function ui(n){return n.x}function ii(n){return n.y}function oi(n,t,e){n.y0=t,n.y=e}function ai(n){return ta.range(n.length)}function ci(n){for(var t=-1,e=n[0].length,r=[];++t<e;)r[t]=0;return r}function li(n){for(var t,e=1,r=0,u=n[0][1],i=n.length;i>e;++e)(t=n[e][1])>u&&(r=e,u=t);return r}function si(n){return n.reduce(fi,0)}function fi(n,t){return n+t[1]}function hi(n,t){return gi(n,Math.ceil(Math.log(t.length)/Math.LN2+1))}function gi(n,t){for(var e=-1,r=+n[0],u=(n[1]-r)/t,i=[];++e<=t;)i[e]=u*e+r;return i}function pi(n){return[ta.min(n),ta.max(n)]}function vi(n,t){return n.value-t.value}function di(n,t){var e=n._pack_next;n._pack_next=t,t._pack_prev=n,t._pack_next=e,e._pack_prev=t}function mi(n,t){n._pack_next=t,t._pack_prev=n}function yi(n,t){var e=t.x-n.x,r=t.y-n.y,u=n.r+t.r;return.999*u*u>e*e+r*r}function Mi(n){function t(n){s=Math.min(n.x-n.r,s),f=Math.max(n.x+n.r,f),h=Math.min(n.y-n.r,h),g=Math.max(n.y+n.r,g)}if((e=n.children)&&(l=e.length)){var e,r,u,i,o,a,c,l,s=1/0,f=-1/0,h=1/0,g=-1/0;if(e.forEach(xi),r=e[0],r.x=-r.r,r.y=0,t(r),l>1&&(u=e[1],u.x=u.r,u.y=0,t(u),l>2))for(i=e[2],wi(r,u,i),t(i),di(r,i),r._pack_prev=i,di(i,u),u=r._pack_next,o=3;l>o;o++){wi(r,u,i=e[o]);var p=0,v=1,d=1;for(a=u._pack_next;a!==u;a=a._pack_next,v++)if(yi(a,i)){p=1;break}if(1==p)for(c=r._pack_prev;c!==a._pack_prev&&!yi(c,i);c=c._pack_prev,d++);p?(d>v||v==d&&u.r<r.r?mi(r,u=a):mi(r=c,u),o--):(di(r,i),u=i,t(i))}var m=(s+f)/2,y=(h+g)/2,M=0;for(o=0;l>o;o++)i=e[o],i.x-=m,i.y-=y,M=Math.max(M,i.r+Math.sqrt(i.x*i.x+i.y*i.y));n.r=M,e.forEach(bi)}}function xi(n){n._pack_next=n._pack_prev=n}function bi(n){delete n._pack_next,delete n._pack_prev}function _i(n,t,e,r){var u=n.children;if(n.x=t+=r*n.x,n.y=e+=r*n.y,n.r*=r,u)for(var i=-1,o=u.length;++i<o;)_i(u[i],t,e,r)}function wi(n,t,e){var r=n.r+e.r,u=t.x-n.x,i=t.y-n.y;if(r&&(u||i)){var o=t.r+e.r,a=u*u+i*i;o*=o,r*=r;var c=.5+(r-o)/(2*a),l=Math.sqrt(Math.max(0,2*o*(r+a)-(r-=a)*r-o*o))/(2*a);e.x=n.x+c*u+l*i,e.y=n.y+c*i-l*u}else e.x=n.x+r,e.y=n.y}function Si(n,t){return n.parent==t.parent?1:2}function ki(n){var t=n.children;return t.length?t[0]:n.t}function Ei(n){var t,e=n.children;return(t=e.length)?e[t-1]:n.t}function Ai(n,t,e){var r=e/(t.i-n.i);t.c-=r,t.s+=e,n.c+=r,t.z+=e,t.m+=e}function Ni(n){for(var t,e=0,r=0,u=n.children,i=u.length;--i>=0;)t=u[i],t.z+=e,t.m+=e,e+=t.s+(r+=t.c)}function Ci(n,t,e){return n.a.parent===t.parent?n.a:e}function zi(n){return 1+ta.max(n,function(n){return n.y})}function qi(n){return n.reduce(function(n,t){return n+t.x},0)/n.length}function Li(n){var t=n.children;return t&&t.length?Li(t[0]):n}function Ti(n){var t,e=n.children;return e&&(t=e.length)?Ti(e[t-1]):n}function Ri(n){return{x:n.x,y:n.y,dx:n.dx,dy:n.dy}}function Di(n,t){var e=n.x+t[3],r=n.y+t[0],u=n.dx-t[1]-t[3],i=n.dy-t[0]-t[2];return 0>u&&(e+=u/2,u=0),0>i&&(r+=i/2,i=0),{x:e,y:r,dx:u,dy:i}}function Pi(n){var t=n[0],e=n[n.length-1];return e>t?[t,e]:[e,t]}function Ui(n){return n.rangeExtent?n.rangeExtent():Pi(n.range())}function ji(n,t,e,r){var u=e(n[0],n[1]),i=r(t[0],t[1]);return function(n){return i(u(n))}}function Fi(n,t){var e,r=0,u=n.length-1,i=n[r],o=n[u];return i>o&&(e=r,r=u,u=e,e=i,i=o,o=e),n[r]=t.floor(i),n[u]=t.ceil(o),n}function Hi(n){return n?{floor:function(t){return Math.floor(t/n)*n},ceil:function(t){return Math.ceil(t/n)*n}}:ml}function Oi(n,t,e,r){var u=[],i=[],o=0,a=Math.min(n.length,t.length)-1;for(n[a]<n[0]&&(n=n.slice().reverse(),t=t.slice().reverse());++o<=a;)u.push(e(n[o-1],n[o])),i.push(r(t[o-1],t[o]));return function(t){var e=ta.bisect(n,t,1,a)-1;return i[e](u[e](t))}}function Ii(n,t,e,r){function u(){var u=Math.min(n.length,t.length)>2?Oi:ji,c=r?Iu:Ou;return o=u(n,t,c,e),a=u(t,n,c,mu),i}function i(n){return o(n)}var o,a;return i.invert=function(n){return a(n)},i.domain=function(t){return arguments.length?(n=t.map(Number),u()):n},i.range=function(n){return arguments.length?(t=n,u()):t},i.rangeRound=function(n){return i.range(n).interpolate(Du)},i.clamp=function(n){return arguments.length?(r=n,u()):r},i.interpolate=function(n){return arguments.length?(e=n,u()):e},i.ticks=function(t){return Xi(n,t)},i.tickFormat=function(t,e){return $i(n,t,e)},i.nice=function(t){return Zi(n,t),u()},i.copy=function(){return Ii(n,t,e,r)},u()}function Yi(n,t){return ta.rebind(n,t,"range","rangeRound","interpolate","clamp")}function Zi(n,t){return Fi(n,Hi(Vi(n,t)[2]))}function Vi(n,t){null==t&&(t=10);var e=Pi(n),r=e[1]-e[0],u=Math.pow(10,Math.floor(Math.log(r/t)/Math.LN10)),i=t/r*u;return.15>=i?u*=10:.35>=i?u*=5:.75>=i&&(u*=2),e[0]=Math.ceil(e[0]/u)*u,e[1]=Math.floor(e[1]/u)*u+.5*u,e[2]=u,e}function Xi(n,t){return ta.range.apply(ta,Vi(n,t))}function $i(n,t,e){var r=Vi(n,t);if(e){var u=ic.exec(e);if(u.shift(),"s"===u[8]){var i=ta.formatPrefix(Math.max(ga(r[0]),ga(r[1])));return u[7]||(u[7]="."+Bi(i.scale(r[2]))),u[8]="f",e=ta.format(u.join("")),function(n){return e(i.scale(n))+i.symbol}}u[7]||(u[7]="."+Wi(u[8],r)),e=u.join("")}else e=",."+Bi(r[2])+"f";return ta.format(e)}function Bi(n){return-Math.floor(Math.log(n)/Math.LN10+.01)}function Wi(n,t){var e=Bi(t[2]);return n in yl?Math.abs(e-Bi(Math.max(ga(t[0]),ga(t[1]))))+ +("e"!==n):e-2*("%"===n)}function Ji(n,t,e,r){function u(n){return(e?Math.log(0>n?0:n):-Math.log(n>0?0:-n))/Math.log(t)}function i(n){return e?Math.pow(t,n):-Math.pow(t,-n)}function o(t){return n(u(t))}return o.invert=function(t){return i(n.invert(t))},o.domain=function(t){return arguments.length?(e=t[0]>=0,n.domain((r=t.map(Number)).map(u)),o):r},o.base=function(e){return arguments.length?(t=+e,n.domain(r.map(u)),o):t},o.nice=function(){var t=Fi(r.map(u),e?Math:xl);return n.domain(t),r=t.map(i),o},o.ticks=function(){var n=Pi(r),o=[],a=n[0],c=n[1],l=Math.floor(u(a)),s=Math.ceil(u(c)),f=t%1?2:t;if(isFinite(s-l)){if(e){for(;s>l;l++)for(var h=1;f>h;h++)o.push(i(l)*h);o.push(i(l))}else for(o.push(i(l));l++<s;)for(var h=f-1;h>0;h--)o.push(i(l)*h);for(l=0;o[l]<a;l++);for(s=o.length;o[s-1]>c;s--);o=o.slice(l,s)}return o},o.tickFormat=function(n,t){if(!arguments.length)return Ml;arguments.length<2?t=Ml:"function"!=typeof t&&(t=ta.format(t));var r,a=Math.max(.1,n/o.ticks().length),c=e?(r=1e-12,Math.ceil):(r=-1e-12,Math.floor);return function(n){return n/i(c(u(n)+r))<=a?t(n):""}},o.copy=function(){return Ji(n.copy(),t,e,r)},Yi(o,n)}function Gi(n,t,e){function r(t){return n(u(t))}var u=Ki(t),i=Ki(1/t);return r.invert=function(t){return i(n.invert(t))},r.domain=function(t){return arguments.length?(n.domain((e=t.map(Number)).map(u)),r):e},r.ticks=function(n){return Xi(e,n)},r.tickFormat=function(n,t){return $i(e,n,t)},r.nice=function(n){return r.domain(Zi(e,n))},r.exponent=function(o){return arguments.length?(u=Ki(t=o),i=Ki(1/t),n.domain(e.map(u)),r):t},r.copy=function(){return Gi(n.copy(),t,e)},Yi(r,n)}function Ki(n){return function(t){return 0>t?-Math.pow(-t,n):Math.pow(t,n)}}function Qi(n,t){function e(e){return i[((u.get(e)||("range"===t.t?u.set(e,n.push(e)):0/0))-1)%i.length]}function r(t,e){return ta.range(n.length).map(function(n){return t+e*n})}var u,i,o;return e.domain=function(r){if(!arguments.length)return n;n=[],u=new l;for(var i,o=-1,a=r.length;++o<a;)u.has(i=r[o])||u.set(i,n.push(i));return e[t.t].apply(e,t.a)},e.range=function(n){return arguments.length?(i=n,o=0,t={t:"range",a:arguments},e):i},e.rangePoints=function(u,a){arguments.length<2&&(a=0);var c=u[0],l=u[1],s=n.length<2?(c=(c+l)/2,0):(l-c)/(n.length-1+a);return i=r(c+s*a/2,s),o=0,t={t:"rangePoints",a:arguments},e},e.rangeRoundPoints=function(u,a){arguments.length<2&&(a=0);var c=u[0],l=u[1],s=n.length<2?(c=l=Math.round((c+l)/2),0):(l-c)/(n.length-1+a)|0;return i=r(c+Math.round(s*a/2+(l-c-(n.length-1+a)*s)/2),s),o=0,t={t:"rangeRoundPoints",a:arguments},e},e.rangeBands=function(u,a,c){arguments.length<2&&(a=0),arguments.length<3&&(c=a);var l=u[1]<u[0],s=u[l-0],f=u[1-l],h=(f-s)/(n.length-a+2*c);return i=r(s+h*c,h),l&&i.reverse(),o=h*(1-a),t={t:"rangeBands",a:arguments},e},e.rangeRoundBands=function(u,a,c){arguments.length<2&&(a=0),arguments.length<3&&(c=a);var l=u[1]<u[0],s=u[l-0],f=u[1-l],h=Math.floor((f-s)/(n.length-a+2*c));return i=r(s+Math.round((f-s-(n.length-a)*h)/2),h),l&&i.reverse(),o=Math.round(h*(1-a)),t={t:"rangeRoundBands",a:arguments},e},e.rangeBand=function(){return o},e.rangeExtent=function(){return Pi(t.a[0])},e.copy=function(){return Qi(n,t)},e.domain(n)}function no(n,t){function i(){var e=0,r=t.length;for(a=[];++e<r;)a[e-1]=ta.quantile(n,e/r);return o}function o(n){return isNaN(n=+n)?void 0:t[ta.bisect(a,n)]}var a;return o.domain=function(t){return arguments.length?(n=t.map(r).filter(u).sort(e),i()):n},o.range=function(n){return arguments.length?(t=n,i()):t},o.quantiles=function(){return a},o.invertExtent=function(e){return e=t.indexOf(e),0>e?[0/0,0/0]:[e>0?a[e-1]:n[0],e<a.length?a[e]:n[n.length-1]]},o.copy=function(){return no(n,t)},i()}function to(n,t,e){function r(t){return e[Math.max(0,Math.min(o,Math.floor(i*(t-n))))]}function u(){return i=e.length/(t-n),o=e.length-1,r}var i,o;return r.domain=function(e){return arguments.length?(n=+e[0],t=+e[e.length-1],u()):[n,t]},r.range=function(n){return arguments.length?(e=n,u()):e},r.invertExtent=function(t){return t=e.indexOf(t),t=0>t?0/0:t/i+n,[t,t+1/i]},r.copy=function(){return to(n,t,e)},u()}function eo(n,t){function e(e){return e>=e?t[ta.bisect(n,e)]:void 0}return e.domain=function(t){return arguments.length?(n=t,e):n},e.range=function(n){return arguments.length?(t=n,e):t},e.invertExtent=function(e){return e=t.indexOf(e),[n[e-1],n[e]]},e.copy=function(){return eo(n,t)},e}function ro(n){function t(n){return+n}return t.invert=t,t.domain=t.range=function(e){return arguments.length?(n=e.map(t),t):n},t.ticks=function(t){return Xi(n,t)},t.tickFormat=function(t,e){return $i(n,t,e)},t.copy=function(){return ro(n)},t}function uo(){return 0}function io(n){return n.innerRadius}function oo(n){return n.outerRadius}function ao(n){return n.startAngle}function co(n){return n.endAngle}function lo(n){return n&&n.padAngle}function so(n,t,e,r){return(n-e)*t-(t-r)*n>0?0:1}function fo(n,t,e,r,u){var i=n[0]-t[0],o=n[1]-t[1],a=(u?r:-r)/Math.sqrt(i*i+o*o),c=a*o,l=-a*i,s=n[0]+c,f=n[1]+l,h=t[0]+c,g=t[1]+l,p=(s+h)/2,v=(f+g)/2,d=h-s,m=g-f,y=d*d+m*m,M=e-r,x=s*g-h*f,b=(0>m?-1:1)*Math.sqrt(M*M*y-x*x),_=(x*m-d*b)/y,w=(-x*d-m*b)/y,S=(x*m+d*b)/y,k=(-x*d+m*b)/y,E=_-p,A=w-v,N=S-p,C=k-v;return E*E+A*A>N*N+C*C&&(_=S,w=k),[[_-c,w-l],[_*e/M,w*e/M]]}function ho(n){function t(t){function o(){l.push("M",i(n(s),a))}for(var c,l=[],s=[],f=-1,h=t.length,g=Et(e),p=Et(r);++f<h;)u.call(this,c=t[f],f)?s.push([+g.call(this,c,f),+p.call(this,c,f)]):s.length&&(o(),s=[]);return s.length&&o(),l.length?l.join(""):null}var e=Ar,r=Nr,u=Ne,i=go,o=i.key,a=.7;return t.x=function(n){return arguments.length?(e=n,t):e},t.y=function(n){return arguments.length?(r=n,t):r},t.defined=function(n){return arguments.length?(u=n,t):u},t.interpolate=function(n){return arguments.length?(o="function"==typeof n?i=n:(i=El.get(n)||go).key,t):o},t.tension=function(n){return arguments.length?(a=n,t):a},t}function go(n){return n.join("L")}function po(n){return go(n)+"Z"}function vo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("H",(r[0]+(r=n[t])[0])/2,"V",r[1]);return e>1&&u.push("H",r[0]),u.join("")}function mo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("V",(r=n[t])[1],"H",r[0]);return u.join("")}function yo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("H",(r=n[t])[0],"V",r[1]);return u.join("")}function Mo(n,t){return n.length<4?go(n):n[1]+_o(n.slice(1,-1),wo(n,t))}function xo(n,t){return n.length<3?go(n):n[0]+_o((n.push(n[0]),n),wo([n[n.length-2]].concat(n,[n[1]]),t))}function bo(n,t){return n.length<3?go(n):n[0]+_o(n,wo(n,t))}function _o(n,t){if(t.length<1||n.length!=t.length&&n.length!=t.length+2)return go(n);var e=n.length!=t.length,r="",u=n[0],i=n[1],o=t[0],a=o,c=1;if(e&&(r+="Q"+(i[0]-2*o[0]/3)+","+(i[1]-2*o[1]/3)+","+i[0]+","+i[1],u=n[1],c=2),t.length>1){a=t[1],i=n[c],c++,r+="C"+(u[0]+o[0])+","+(u[1]+o[1])+","+(i[0]-a[0])+","+(i[1]-a[1])+","+i[0]+","+i[1];for(var l=2;l<t.length;l++,c++)i=n[c],a=t[l],r+="S"+(i[0]-a[0])+","+(i[1]-a[1])+","+i[0]+","+i[1]}if(e){var s=n[c];r+="Q"+(i[0]+2*a[0]/3)+","+(i[1]+2*a[1]/3)+","+s[0]+","+s[1]}return r}function wo(n,t){for(var e,r=[],u=(1-t)/2,i=n[0],o=n[1],a=1,c=n.length;++a<c;)e=i,i=o,o=n[a],r.push([u*(o[0]-e[0]),u*(o[1]-e[1])]);return r}function So(n){if(n.length<3)return go(n);var t=1,e=n.length,r=n[0],u=r[0],i=r[1],o=[u,u,u,(r=n[1])[0]],a=[i,i,i,r[1]],c=[u,",",i,"L",No(Cl,o),",",No(Cl,a)];for(n.push(n[e-1]);++t<=e;)r=n[t],o.shift(),o.push(r[0]),a.shift(),a.push(r[1]),Co(c,o,a);return n.pop(),c.push("L",r),c.join("")}function ko(n){if(n.length<4)return go(n);for(var t,e=[],r=-1,u=n.length,i=[0],o=[0];++r<3;)t=n[r],i.push(t[0]),o.push(t[1]);for(e.push(No(Cl,i)+","+No(Cl,o)),--r;++r<u;)t=n[r],i.shift(),i.push(t[0]),o.shift(),o.push(t[1]),Co(e,i,o);return e.join("")}function Eo(n){for(var t,e,r=-1,u=n.length,i=u+4,o=[],a=[];++r<4;)e=n[r%u],o.push(e[0]),a.push(e[1]);for(t=[No(Cl,o),",",No(Cl,a)],--r;++r<i;)e=n[r%u],o.shift(),o.push(e[0]),a.shift(),a.push(e[1]),Co(t,o,a);return t.join("")}function Ao(n,t){var e=n.length-1;if(e)for(var r,u,i=n[0][0],o=n[0][1],a=n[e][0]-i,c=n[e][1]-o,l=-1;++l<=e;)r=n[l],u=l/e,r[0]=t*r[0]+(1-t)*(i+u*a),r[1]=t*r[1]+(1-t)*(o+u*c);return So(n)}function No(n,t){return n[0]*t[0]+n[1]*t[1]+n[2]*t[2]+n[3]*t[3]}function Co(n,t,e){n.push("C",No(Al,t),",",No(Al,e),",",No(Nl,t),",",No(Nl,e),",",No(Cl,t),",",No(Cl,e))}function zo(n,t){return(t[1]-n[1])/(t[0]-n[0])}function qo(n){for(var t=0,e=n.length-1,r=[],u=n[0],i=n[1],o=r[0]=zo(u,i);++t<e;)r[t]=(o+(o=zo(u=i,i=n[t+1])))/2;return r[t]=o,r}function Lo(n){for(var t,e,r,u,i=[],o=qo(n),a=-1,c=n.length-1;++a<c;)t=zo(n[a],n[a+1]),ga(t)<Ca?o[a]=o[a+1]=0:(e=o[a]/t,r=o[a+1]/t,u=e*e+r*r,u>9&&(u=3*t/Math.sqrt(u),o[a]=u*e,o[a+1]=u*r));for(a=-1;++a<=c;)u=(n[Math.min(c,a+1)][0]-n[Math.max(0,a-1)][0])/(6*(1+o[a]*o[a])),i.push([u||0,o[a]*u||0]);return i}function To(n){return n.length<3?go(n):n[0]+_o(n,Lo(n))}function Ro(n){for(var t,e,r,u=-1,i=n.length;++u<i;)t=n[u],e=t[0],r=t[1]-Ra,t[0]=e*Math.cos(r),t[1]=e*Math.sin(r);return n}function Do(n){function t(t){function c(){v.push("M",a(n(m),f),s,l(n(d.reverse()),f),"Z")}for(var h,g,p,v=[],d=[],m=[],y=-1,M=t.length,x=Et(e),b=Et(u),_=e===r?function(){return g}:Et(r),w=u===i?function(){return p}:Et(i);++y<M;)o.call(this,h=t[y],y)?(d.push([g=+x.call(this,h,y),p=+b.call(this,h,y)]),m.push([+_.call(this,h,y),+w.call(this,h,y)])):d.length&&(c(),d=[],m=[]);return d.length&&c(),v.length?v.join(""):null}var e=Ar,r=Ar,u=0,i=Nr,o=Ne,a=go,c=a.key,l=a,s="L",f=.7;return t.x=function(n){return arguments.length?(e=r=n,t):r},t.x0=function(n){return arguments.length?(e=n,t):e},t.x1=function(n){return arguments.length?(r=n,t):r
},t.y=function(n){return arguments.length?(u=i=n,t):i},t.y0=function(n){return arguments.length?(u=n,t):u},t.y1=function(n){return arguments.length?(i=n,t):i},t.defined=function(n){return arguments.length?(o=n,t):o},t.interpolate=function(n){return arguments.length?(c="function"==typeof n?a=n:(a=El.get(n)||go).key,l=a.reverse||a,s=a.closed?"M":"L",t):c},t.tension=function(n){return arguments.length?(f=n,t):f},t}function Po(n){return n.radius}function Uo(n){return[n.x,n.y]}function jo(n){return function(){var t=n.apply(this,arguments),e=t[0],r=t[1]-Ra;return[e*Math.cos(r),e*Math.sin(r)]}}function Fo(){return 64}function Ho(){return"circle"}function Oo(n){var t=Math.sqrt(n/qa);return"M0,"+t+"A"+t+","+t+" 0 1,1 0,"+-t+"A"+t+","+t+" 0 1,1 0,"+t+"Z"}function Io(n){return function(){var t,e;(t=this[n])&&(e=t[t.active])&&(--t.count?delete t[t.active]:delete this[n],t.active+=.5,e.event&&e.event.interrupt.call(this,this.__data__,e.index))}}function Yo(n,t,e){return ya(n,Pl),n.namespace=t,n.id=e,n}function Zo(n,t,e,r){var u=n.id,i=n.namespace;return Y(n,"function"==typeof e?function(n,o,a){n[i][u].tween.set(t,r(e.call(n,n.__data__,o,a)))}:(e=r(e),function(n){n[i][u].tween.set(t,e)}))}function Vo(n){return null==n&&(n=""),function(){this.textContent=n}}function Xo(n){return null==n?"__transition__":"__transition_"+n+"__"}function $o(n,t,e,r,u){var i=n[e]||(n[e]={active:0,count:0}),o=i[r];if(!o){var a=u.time;o=i[r]={tween:new l,time:a,delay:u.delay,duration:u.duration,ease:u.ease,index:t},u=null,++i.count,ta.timer(function(u){function c(e){if(i.active>r)return s();var u=i[i.active];u&&(--i.count,delete i[i.active],u.event&&u.event.interrupt.call(n,n.__data__,u.index)),i.active=r,o.event&&o.event.start.call(n,n.__data__,t),o.tween.forEach(function(e,r){(r=r.call(n,n.__data__,t))&&v.push(r)}),h=o.ease,f=o.duration,ta.timer(function(){return p.c=l(e||1)?Ne:l,1},0,a)}function l(e){if(i.active!==r)return 1;for(var u=e/f,a=h(u),c=v.length;c>0;)v[--c].call(n,a);return u>=1?(o.event&&o.event.end.call(n,n.__data__,t),s()):void 0}function s(){return--i.count?delete i[r]:delete n[e],1}var f,h,g=o.delay,p=ec,v=[];return p.t=g+a,u>=g?c(u-g):void(p.c=c)},0,a)}}function Bo(n,t,e){n.attr("transform",function(n){var r=t(n);return"translate("+(isFinite(r)?r:e(n))+",0)"})}function Wo(n,t,e){n.attr("transform",function(n){var r=t(n);return"translate(0,"+(isFinite(r)?r:e(n))+")"})}function Jo(n){return n.toISOString()}function Go(n,t,e){function r(t){return n(t)}function u(n,e){var r=n[1]-n[0],u=r/e,i=ta.bisect(Vl,u);return i==Vl.length?[t.year,Vi(n.map(function(n){return n/31536e6}),e)[2]]:i?t[u/Vl[i-1]<Vl[i]/u?i-1:i]:[Bl,Vi(n,e)[2]]}return r.invert=function(t){return Ko(n.invert(t))},r.domain=function(t){return arguments.length?(n.domain(t),r):n.domain().map(Ko)},r.nice=function(n,t){function e(e){return!isNaN(e)&&!n.range(e,Ko(+e+1),t).length}var i=r.domain(),o=Pi(i),a=null==n?u(o,10):"number"==typeof n&&u(o,n);return a&&(n=a[0],t=a[1]),r.domain(Fi(i,t>1?{floor:function(t){for(;e(t=n.floor(t));)t=Ko(t-1);return t},ceil:function(t){for(;e(t=n.ceil(t));)t=Ko(+t+1);return t}}:n))},r.ticks=function(n,t){var e=Pi(r.domain()),i=null==n?u(e,10):"number"==typeof n?u(e,n):!n.range&&[{range:n},t];return i&&(n=i[0],t=i[1]),n.range(e[0],Ko(+e[1]+1),1>t?1:t)},r.tickFormat=function(){return e},r.copy=function(){return Go(n.copy(),t,e)},Yi(r,n)}function Ko(n){return new Date(n)}function Qo(n){return JSON.parse(n.responseText)}function na(n){var t=ua.createRange();return t.selectNode(ua.body),t.createContextualFragment(n.responseText)}var ta={version:"3.5.5"},ea=[].slice,ra=function(n){return ea.call(n)},ua=this.document;if(ua)try{ra(ua.documentElement.childNodes)[0].nodeType}catch(ia){ra=function(n){for(var t=n.length,e=new Array(t);t--;)e[t]=n[t];return e}}if(Date.now||(Date.now=function(){return+new Date}),ua)try{ua.createElement("DIV").style.setProperty("opacity",0,"")}catch(oa){var aa=this.Element.prototype,ca=aa.setAttribute,la=aa.setAttributeNS,sa=this.CSSStyleDeclaration.prototype,fa=sa.setProperty;aa.setAttribute=function(n,t){ca.call(this,n,t+"")},aa.setAttributeNS=function(n,t,e){la.call(this,n,t,e+"")},sa.setProperty=function(n,t,e){fa.call(this,n,t+"",e)}}ta.ascending=e,ta.descending=function(n,t){return n>t?-1:t>n?1:t>=n?0:0/0},ta.min=function(n,t){var e,r,u=-1,i=n.length;if(1===arguments.length){for(;++u<i;)if(null!=(r=n[u])&&r>=r){e=r;break}for(;++u<i;)null!=(r=n[u])&&e>r&&(e=r)}else{for(;++u<i;)if(null!=(r=t.call(n,n[u],u))&&r>=r){e=r;break}for(;++u<i;)null!=(r=t.call(n,n[u],u))&&e>r&&(e=r)}return e},ta.max=function(n,t){var e,r,u=-1,i=n.length;if(1===arguments.length){for(;++u<i;)if(null!=(r=n[u])&&r>=r){e=r;break}for(;++u<i;)null!=(r=n[u])&&r>e&&(e=r)}else{for(;++u<i;)if(null!=(r=t.call(n,n[u],u))&&r>=r){e=r;break}for(;++u<i;)null!=(r=t.call(n,n[u],u))&&r>e&&(e=r)}return e},ta.extent=function(n,t){var e,r,u,i=-1,o=n.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=n[i])&&r>=r){e=u=r;break}for(;++i<o;)null!=(r=n[i])&&(e>r&&(e=r),r>u&&(u=r))}else{for(;++i<o;)if(null!=(r=t.call(n,n[i],i))&&r>=r){e=u=r;break}for(;++i<o;)null!=(r=t.call(n,n[i],i))&&(e>r&&(e=r),r>u&&(u=r))}return[e,u]},ta.sum=function(n,t){var e,r=0,i=n.length,o=-1;if(1===arguments.length)for(;++o<i;)u(e=+n[o])&&(r+=e);else for(;++o<i;)u(e=+t.call(n,n[o],o))&&(r+=e);return r},ta.mean=function(n,t){var e,i=0,o=n.length,a=-1,c=o;if(1===arguments.length)for(;++a<o;)u(e=r(n[a]))?i+=e:--c;else for(;++a<o;)u(e=r(t.call(n,n[a],a)))?i+=e:--c;return c?i/c:void 0},ta.quantile=function(n,t){var e=(n.length-1)*t+1,r=Math.floor(e),u=+n[r-1],i=e-r;return i?u+i*(n[r]-u):u},ta.median=function(n,t){var i,o=[],a=n.length,c=-1;if(1===arguments.length)for(;++c<a;)u(i=r(n[c]))&&o.push(i);else for(;++c<a;)u(i=r(t.call(n,n[c],c)))&&o.push(i);return o.length?ta.quantile(o.sort(e),.5):void 0},ta.variance=function(n,t){var e,i,o=n.length,a=0,c=0,l=-1,s=0;if(1===arguments.length)for(;++l<o;)u(e=r(n[l]))&&(i=e-a,a+=i/++s,c+=i*(e-a));else for(;++l<o;)u(e=r(t.call(n,n[l],l)))&&(i=e-a,a+=i/++s,c+=i*(e-a));return s>1?c/(s-1):void 0},ta.deviation=function(){var n=ta.variance.apply(this,arguments);return n?Math.sqrt(n):n};var ha=i(e);ta.bisectLeft=ha.left,ta.bisect=ta.bisectRight=ha.right,ta.bisector=function(n){return i(1===n.length?function(t,r){return e(n(t),r)}:n)},ta.shuffle=function(n,t,e){(i=arguments.length)<3&&(e=n.length,2>i&&(t=0));for(var r,u,i=e-t;i;)u=Math.random()*i--|0,r=n[i+t],n[i+t]=n[u+t],n[u+t]=r;return n},ta.permute=function(n,t){for(var e=t.length,r=new Array(e);e--;)r[e]=n[t[e]];return r},ta.pairs=function(n){for(var t,e=0,r=n.length-1,u=n[0],i=new Array(0>r?0:r);r>e;)i[e]=[t=u,u=n[++e]];return i},ta.zip=function(){if(!(r=arguments.length))return[];for(var n=-1,t=ta.min(arguments,o),e=new Array(t);++n<t;)for(var r,u=-1,i=e[n]=new Array(r);++u<r;)i[u]=arguments[u][n];return e},ta.transpose=function(n){return ta.zip.apply(ta,n)},ta.keys=function(n){var t=[];for(var e in n)t.push(e);return t},ta.values=function(n){var t=[];for(var e in n)t.push(n[e]);return t},ta.entries=function(n){var t=[];for(var e in n)t.push({key:e,value:n[e]});return t},ta.merge=function(n){for(var t,e,r,u=n.length,i=-1,o=0;++i<u;)o+=n[i].length;for(e=new Array(o);--u>=0;)for(r=n[u],t=r.length;--t>=0;)e[--o]=r[t];return e};var ga=Math.abs;ta.range=function(n,t,e){if(arguments.length<3&&(e=1,arguments.length<2&&(t=n,n=0)),(t-n)/e===1/0)throw new Error("infinite range");var r,u=[],i=a(ga(e)),o=-1;if(n*=i,t*=i,e*=i,0>e)for(;(r=n+e*++o)>t;)u.push(r/i);else for(;(r=n+e*++o)<t;)u.push(r/i);return u},ta.map=function(n,t){var e=new l;if(n instanceof l)n.forEach(function(n,t){e.set(n,t)});else if(Array.isArray(n)){var r,u=-1,i=n.length;if(1===arguments.length)for(;++u<i;)e.set(u,n[u]);else for(;++u<i;)e.set(t.call(n,r=n[u],u),r)}else for(var o in n)e.set(o,n[o]);return e};var pa="__proto__",va="\x00";c(l,{has:h,get:function(n){return this._[s(n)]},set:function(n,t){return this._[s(n)]=t},remove:g,keys:p,values:function(){var n=[];for(var t in this._)n.push(this._[t]);return n},entries:function(){var n=[];for(var t in this._)n.push({key:f(t),value:this._[t]});return n},size:v,empty:d,forEach:function(n){for(var t in this._)n.call(this,f(t),this._[t])}}),ta.nest=function(){function n(t,o,a){if(a>=i.length)return r?r.call(u,o):e?o.sort(e):o;for(var c,s,f,h,g=-1,p=o.length,v=i[a++],d=new l;++g<p;)(h=d.get(c=v(s=o[g])))?h.push(s):d.set(c,[s]);return t?(s=t(),f=function(e,r){s.set(e,n(t,r,a))}):(s={},f=function(e,r){s[e]=n(t,r,a)}),d.forEach(f),s}function t(n,e){if(e>=i.length)return n;var r=[],u=o[e++];return n.forEach(function(n,u){r.push({key:n,values:t(u,e)})}),u?r.sort(function(n,t){return u(n.key,t.key)}):r}var e,r,u={},i=[],o=[];return u.map=function(t,e){return n(e,t,0)},u.entries=function(e){return t(n(ta.map,e,0),0)},u.key=function(n){return i.push(n),u},u.sortKeys=function(n){return o[i.length-1]=n,u},u.sortValues=function(n){return e=n,u},u.rollup=function(n){return r=n,u},u},ta.set=function(n){var t=new m;if(n)for(var e=0,r=n.length;r>e;++e)t.add(n[e]);return t},c(m,{has:h,add:function(n){return this._[s(n+="")]=!0,n},remove:g,values:p,size:v,empty:d,forEach:function(n){for(var t in this._)n.call(this,f(t))}}),ta.behavior={},ta.rebind=function(n,t){for(var e,r=1,u=arguments.length;++r<u;)n[e=arguments[r]]=M(n,t,t[e]);return n};var da=["webkit","ms","moz","Moz","o","O"];ta.dispatch=function(){for(var n=new _,t=-1,e=arguments.length;++t<e;)n[arguments[t]]=w(n);return n},_.prototype.on=function(n,t){var e=n.indexOf("."),r="";if(e>=0&&(r=n.slice(e+1),n=n.slice(0,e)),n)return arguments.length<2?this[n].on(r):this[n].on(r,t);if(2===arguments.length){if(null==t)for(n in this)this.hasOwnProperty(n)&&this[n].on(r,null);return this}},ta.event=null,ta.requote=function(n){return n.replace(ma,"\\$&")};var ma=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,ya={}.__proto__?function(n,t){n.__proto__=t}:function(n,t){for(var e in t)n[e]=t[e]},Ma=function(n,t){return t.querySelector(n)},xa=function(n,t){return t.querySelectorAll(n)},ba=function(n,t){var e=n.matches||n[x(n,"matchesSelector")];return(ba=function(n,t){return e.call(n,t)})(n,t)};"function"==typeof Sizzle&&(Ma=function(n,t){return Sizzle(n,t)[0]||null},xa=Sizzle,ba=Sizzle.matchesSelector),ta.selection=function(){return ta.select(ua.documentElement)};var _a=ta.selection.prototype=[];_a.select=function(n){var t,e,r,u,i=[];n=N(n);for(var o=-1,a=this.length;++o<a;){i.push(t=[]),t.parentNode=(r=this[o]).parentNode;for(var c=-1,l=r.length;++c<l;)(u=r[c])?(t.push(e=n.call(u,u.__data__,c,o)),e&&"__data__"in u&&(e.__data__=u.__data__)):t.push(null)}return A(i)},_a.selectAll=function(n){var t,e,r=[];n=C(n);for(var u=-1,i=this.length;++u<i;)for(var o=this[u],a=-1,c=o.length;++a<c;)(e=o[a])&&(r.push(t=ra(n.call(e,e.__data__,a,u))),t.parentNode=e);return A(r)};var wa={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};ta.ns={prefix:wa,qualify:function(n){var t=n.indexOf(":"),e=n;return t>=0&&(e=n.slice(0,t),n=n.slice(t+1)),wa.hasOwnProperty(e)?{space:wa[e],local:n}:n}},_a.attr=function(n,t){if(arguments.length<2){if("string"==typeof n){var e=this.node();return n=ta.ns.qualify(n),n.local?e.getAttributeNS(n.space,n.local):e.getAttribute(n)}for(t in n)this.each(z(t,n[t]));return this}return this.each(z(n,t))},_a.classed=function(n,t){if(arguments.length<2){if("string"==typeof n){var e=this.node(),r=(n=T(n)).length,u=-1;if(t=e.classList){for(;++u<r;)if(!t.contains(n[u]))return!1}else for(t=e.getAttribute("class");++u<r;)if(!L(n[u]).test(t))return!1;return!0}for(t in n)this.each(R(t,n[t]));return this}return this.each(R(n,t))},_a.style=function(n,e,r){var u=arguments.length;if(3>u){if("string"!=typeof n){2>u&&(e="");for(r in n)this.each(P(r,n[r],e));return this}if(2>u){var i=this.node();return t(i).getComputedStyle(i,null).getPropertyValue(n)}r=""}return this.each(P(n,e,r))},_a.property=function(n,t){if(arguments.length<2){if("string"==typeof n)return this.node()[n];for(t in n)this.each(U(t,n[t]));return this}return this.each(U(n,t))},_a.text=function(n){return arguments.length?this.each("function"==typeof n?function(){var t=n.apply(this,arguments);this.textContent=null==t?"":t}:null==n?function(){this.textContent=""}:function(){this.textContent=n}):this.node().textContent},_a.html=function(n){return arguments.length?this.each("function"==typeof n?function(){var t=n.apply(this,arguments);this.innerHTML=null==t?"":t}:null==n?function(){this.innerHTML=""}:function(){this.innerHTML=n}):this.node().innerHTML},_a.append=function(n){return n=j(n),this.select(function(){return this.appendChild(n.apply(this,arguments))})},_a.insert=function(n,t){return n=j(n),t=N(t),this.select(function(){return this.insertBefore(n.apply(this,arguments),t.apply(this,arguments)||null)})},_a.remove=function(){return this.each(F)},_a.data=function(n,t){function e(n,e){var r,u,i,o=n.length,f=e.length,h=Math.min(o,f),g=new Array(f),p=new Array(f),v=new Array(o);if(t){var d,m=new l,y=new Array(o);for(r=-1;++r<o;)m.has(d=t.call(u=n[r],u.__data__,r))?v[r]=u:m.set(d,u),y[r]=d;for(r=-1;++r<f;)(u=m.get(d=t.call(e,i=e[r],r)))?u!==!0&&(g[r]=u,u.__data__=i):p[r]=H(i),m.set(d,!0);for(r=-1;++r<o;)m.get(y[r])!==!0&&(v[r]=n[r])}else{for(r=-1;++r<h;)u=n[r],i=e[r],u?(u.__data__=i,g[r]=u):p[r]=H(i);for(;f>r;++r)p[r]=H(e[r]);for(;o>r;++r)v[r]=n[r]}p.update=g,p.parentNode=g.parentNode=v.parentNode=n.parentNode,a.push(p),c.push(g),s.push(v)}var r,u,i=-1,o=this.length;if(!arguments.length){for(n=new Array(o=(r=this[0]).length);++i<o;)(u=r[i])&&(n[i]=u.__data__);return n}var a=Z([]),c=A([]),s=A([]);if("function"==typeof n)for(;++i<o;)e(r=this[i],n.call(r,r.parentNode.__data__,i));else for(;++i<o;)e(r=this[i],n);return c.enter=function(){return a},c.exit=function(){return s},c},_a.datum=function(n){return arguments.length?this.property("__data__",n):this.property("__data__")},_a.filter=function(n){var t,e,r,u=[];"function"!=typeof n&&(n=O(n));for(var i=0,o=this.length;o>i;i++){u.push(t=[]),t.parentNode=(e=this[i]).parentNode;for(var a=0,c=e.length;c>a;a++)(r=e[a])&&n.call(r,r.__data__,a,i)&&t.push(r)}return A(u)},_a.order=function(){for(var n=-1,t=this.length;++n<t;)for(var e,r=this[n],u=r.length-1,i=r[u];--u>=0;)(e=r[u])&&(i&&i!==e.nextSibling&&i.parentNode.insertBefore(e,i),i=e);return this},_a.sort=function(n){n=I.apply(this,arguments);for(var t=-1,e=this.length;++t<e;)this[t].sort(n);return this.order()},_a.each=function(n){return Y(this,function(t,e,r){n.call(t,t.__data__,e,r)})},_a.call=function(n){var t=ra(arguments);return n.apply(t[0]=this,t),this},_a.empty=function(){return!this.node()},_a.node=function(){for(var n=0,t=this.length;t>n;n++)for(var e=this[n],r=0,u=e.length;u>r;r++){var i=e[r];if(i)return i}return null},_a.size=function(){var n=0;return Y(this,function(){++n}),n};var Sa=[];ta.selection.enter=Z,ta.selection.enter.prototype=Sa,Sa.append=_a.append,Sa.empty=_a.empty,Sa.node=_a.node,Sa.call=_a.call,Sa.size=_a.size,Sa.select=function(n){for(var t,e,r,u,i,o=[],a=-1,c=this.length;++a<c;){r=(u=this[a]).update,o.push(t=[]),t.parentNode=u.parentNode;for(var l=-1,s=u.length;++l<s;)(i=u[l])?(t.push(r[l]=e=n.call(u.parentNode,i.__data__,l,a)),e.__data__=i.__data__):t.push(null)}return A(o)},Sa.insert=function(n,t){return arguments.length<2&&(t=V(this)),_a.insert.call(this,n,t)},ta.select=function(t){var e;return"string"==typeof t?(e=[Ma(t,ua)],e.parentNode=ua.documentElement):(e=[t],e.parentNode=n(t)),A([e])},ta.selectAll=function(n){var t;return"string"==typeof n?(t=ra(xa(n,ua)),t.parentNode=ua.documentElement):(t=n,t.parentNode=null),A([t])},_a.on=function(n,t,e){var r=arguments.length;if(3>r){if("string"!=typeof n){2>r&&(t=!1);for(e in n)this.each(X(e,n[e],t));return this}if(2>r)return(r=this.node()["__on"+n])&&r._;e=!1}return this.each(X(n,t,e))};var ka=ta.map({mouseenter:"mouseover",mouseleave:"mouseout"});ua&&ka.forEach(function(n){"on"+n in ua&&ka.remove(n)});var Ea,Aa=0;ta.mouse=function(n){return J(n,k())};var Na=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;ta.touch=function(n,t,e){if(arguments.length<3&&(e=t,t=k().changedTouches),t)for(var r,u=0,i=t.length;i>u;++u)if((r=t[u]).identifier===e)return J(n,r)},ta.behavior.drag=function(){function n(){this.on("mousedown.drag",i).on("touchstart.drag",o)}function e(n,t,e,i,o){return function(){function a(){var n,e,r=t(h,v);r&&(n=r[0]-M[0],e=r[1]-M[1],p|=n|e,M=r,g({type:"drag",x:r[0]+l[0],y:r[1]+l[1],dx:n,dy:e}))}function c(){t(h,v)&&(m.on(i+d,null).on(o+d,null),y(p&&ta.event.target===f),g({type:"dragend"}))}var l,s=this,f=ta.event.target,h=s.parentNode,g=r.of(s,arguments),p=0,v=n(),d=".drag"+(null==v?"":"-"+v),m=ta.select(e(f)).on(i+d,a).on(o+d,c),y=W(f),M=t(h,v);u?(l=u.apply(s,arguments),l=[l.x-M[0],l.y-M[1]]):l=[0,0],g({type:"dragstart"})}}var r=E(n,"drag","dragstart","dragend"),u=null,i=e(b,ta.mouse,t,"mousemove","mouseup"),o=e(G,ta.touch,y,"touchmove","touchend");return n.origin=function(t){return arguments.length?(u=t,n):u},ta.rebind(n,r,"on")},ta.touches=function(n,t){return arguments.length<2&&(t=k().touches),t?ra(t).map(function(t){var e=J(n,t);return e.identifier=t.identifier,e}):[]};var Ca=1e-6,za=Ca*Ca,qa=Math.PI,La=2*qa,Ta=La-Ca,Ra=qa/2,Da=qa/180,Pa=180/qa,Ua=Math.SQRT2,ja=2,Fa=4;ta.interpolateZoom=function(n,t){function e(n){var t=n*y;if(m){var e=rt(v),o=i/(ja*h)*(e*ut(Ua*t+v)-et(v));return[r+o*l,u+o*s,i*e/rt(Ua*t+v)]}return[r+n*l,u+n*s,i*Math.exp(Ua*t)]}var r=n[0],u=n[1],i=n[2],o=t[0],a=t[1],c=t[2],l=o-r,s=a-u,f=l*l+s*s,h=Math.sqrt(f),g=(c*c-i*i+Fa*f)/(2*i*ja*h),p=(c*c-i*i-Fa*f)/(2*c*ja*h),v=Math.log(Math.sqrt(g*g+1)-g),d=Math.log(Math.sqrt(p*p+1)-p),m=d-v,y=(m||Math.log(c/i))/Ua;return e.duration=1e3*y,e},ta.behavior.zoom=function(){function n(n){n.on(q,f).on(Oa+".zoom",g).on("dblclick.zoom",p).on(R,h)}function e(n){return[(n[0]-k.x)/k.k,(n[1]-k.y)/k.k]}function r(n){return[n[0]*k.k+k.x,n[1]*k.k+k.y]}function u(n){k.k=Math.max(N[0],Math.min(N[1],n))}function i(n,t){t=r(t),k.x+=n[0]-t[0],k.y+=n[1]-t[1]}function o(t,e,r,o){t.__chart__={x:k.x,y:k.y,k:k.k},u(Math.pow(2,o)),i(d=e,r),t=ta.select(t),C>0&&(t=t.transition().duration(C)),t.call(n.event)}function a(){b&&b.domain(x.range().map(function(n){return(n-k.x)/k.k}).map(x.invert)),w&&w.domain(_.range().map(function(n){return(n-k.y)/k.k}).map(_.invert))}function c(n){z++||n({type:"zoomstart"})}function l(n){a(),n({type:"zoom",scale:k.k,translate:[k.x,k.y]})}function s(n){--z||n({type:"zoomend"}),d=null}function f(){function n(){f=1,i(ta.mouse(u),g),l(a)}function r(){h.on(L,null).on(T,null),p(f&&ta.event.target===o),s(a)}var u=this,o=ta.event.target,a=D.of(u,arguments),f=0,h=ta.select(t(u)).on(L,n).on(T,r),g=e(ta.mouse(u)),p=W(u);Dl.call(u),c(a)}function h(){function n(){var n=ta.touches(p);return g=k.k,n.forEach(function(n){n.identifier in d&&(d[n.identifier]=e(n))}),n}function t(){var t=ta.event.target;ta.select(t).on(x,r).on(b,a),_.push(t);for(var e=ta.event.changedTouches,u=0,i=e.length;i>u;++u)d[e[u].identifier]=null;var c=n(),l=Date.now();if(1===c.length){if(500>l-M){var s=c[0];o(p,s,d[s.identifier],Math.floor(Math.log(k.k)/Math.LN2)+1),S()}M=l}else if(c.length>1){var s=c[0],f=c[1],h=s[0]-f[0],g=s[1]-f[1];m=h*h+g*g}}function r(){var n,t,e,r,o=ta.touches(p);Dl.call(p);for(var a=0,c=o.length;c>a;++a,r=null)if(e=o[a],r=d[e.identifier]){if(t)break;n=e,t=r}if(r){var s=(s=e[0]-n[0])*s+(s=e[1]-n[1])*s,f=m&&Math.sqrt(s/m);n=[(n[0]+e[0])/2,(n[1]+e[1])/2],t=[(t[0]+r[0])/2,(t[1]+r[1])/2],u(f*g)}M=null,i(n,t),l(v)}function a(){if(ta.event.touches.length){for(var t=ta.event.changedTouches,e=0,r=t.length;r>e;++e)delete d[t[e].identifier];for(var u in d)return void n()}ta.selectAll(_).on(y,null),w.on(q,f).on(R,h),E(),s(v)}var g,p=this,v=D.of(p,arguments),d={},m=0,y=".zoom-"+ta.event.changedTouches[0].identifier,x="touchmove"+y,b="touchend"+y,_=[],w=ta.select(p),E=W(p);t(),c(v),w.on(q,null).on(R,t)}function g(){var n=D.of(this,arguments);y?clearTimeout(y):(v=e(d=m||ta.mouse(this)),Dl.call(this),c(n)),y=setTimeout(function(){y=null,s(n)},50),S(),u(Math.pow(2,.002*Ha())*k.k),i(d,v),l(n)}function p(){var n=ta.mouse(this),t=Math.log(k.k)/Math.LN2;o(this,n,e(n),ta.event.shiftKey?Math.ceil(t)-1:Math.floor(t)+1)}var v,d,m,y,M,x,b,_,w,k={x:0,y:0,k:1},A=[960,500],N=Ia,C=250,z=0,q="mousedown.zoom",L="mousemove.zoom",T="mouseup.zoom",R="touchstart.zoom",D=E(n,"zoomstart","zoom","zoomend");return Oa||(Oa="onwheel"in ua?(Ha=function(){return-ta.event.deltaY*(ta.event.deltaMode?120:1)},"wheel"):"onmousewheel"in ua?(Ha=function(){return ta.event.wheelDelta},"mousewheel"):(Ha=function(){return-ta.event.detail},"MozMousePixelScroll")),n.event=function(n){n.each(function(){var n=D.of(this,arguments),t=k;Tl?ta.select(this).transition().each("start.zoom",function(){k=this.__chart__||{x:0,y:0,k:1},c(n)}).tween("zoom:zoom",function(){var e=A[0],r=A[1],u=d?d[0]:e/2,i=d?d[1]:r/2,o=ta.interpolateZoom([(u-k.x)/k.k,(i-k.y)/k.k,e/k.k],[(u-t.x)/t.k,(i-t.y)/t.k,e/t.k]);return function(t){var r=o(t),a=e/r[2];this.__chart__=k={x:u-r[0]*a,y:i-r[1]*a,k:a},l(n)}}).each("interrupt.zoom",function(){s(n)}).each("end.zoom",function(){s(n)}):(this.__chart__=k,c(n),l(n),s(n))})},n.translate=function(t){return arguments.length?(k={x:+t[0],y:+t[1],k:k.k},a(),n):[k.x,k.y]},n.scale=function(t){return arguments.length?(k={x:k.x,y:k.y,k:+t},a(),n):k.k},n.scaleExtent=function(t){return arguments.length?(N=null==t?Ia:[+t[0],+t[1]],n):N},n.center=function(t){return arguments.length?(m=t&&[+t[0],+t[1]],n):m},n.size=function(t){return arguments.length?(A=t&&[+t[0],+t[1]],n):A},n.duration=function(t){return arguments.length?(C=+t,n):C},n.x=function(t){return arguments.length?(b=t,x=t.copy(),k={x:0,y:0,k:1},n):b},n.y=function(t){return arguments.length?(w=t,_=t.copy(),k={x:0,y:0,k:1},n):w},ta.rebind(n,D,"on")};var Ha,Oa,Ia=[0,1/0];ta.color=ot,ot.prototype.toString=function(){return this.rgb()+""},ta.hsl=at;var Ya=at.prototype=new ot;Ya.brighter=function(n){return n=Math.pow(.7,arguments.length?n:1),new at(this.h,this.s,this.l/n)},Ya.darker=function(n){return n=Math.pow(.7,arguments.length?n:1),new at(this.h,this.s,n*this.l)},Ya.rgb=function(){return ct(this.h,this.s,this.l)},ta.hcl=lt;var Za=lt.prototype=new ot;Za.brighter=function(n){return new lt(this.h,this.c,Math.min(100,this.l+Va*(arguments.length?n:1)))},Za.darker=function(n){return new lt(this.h,this.c,Math.max(0,this.l-Va*(arguments.length?n:1)))},Za.rgb=function(){return st(this.h,this.c,this.l).rgb()},ta.lab=ft;var Va=18,Xa=.95047,$a=1,Ba=1.08883,Wa=ft.prototype=new ot;Wa.brighter=function(n){return new ft(Math.min(100,this.l+Va*(arguments.length?n:1)),this.a,this.b)},Wa.darker=function(n){return new ft(Math.max(0,this.l-Va*(arguments.length?n:1)),this.a,this.b)},Wa.rgb=function(){return ht(this.l,this.a,this.b)},ta.rgb=mt;var Ja=mt.prototype=new ot;Ja.brighter=function(n){n=Math.pow(.7,arguments.length?n:1);var t=this.r,e=this.g,r=this.b,u=30;return t||e||r?(t&&u>t&&(t=u),e&&u>e&&(e=u),r&&u>r&&(r=u),new mt(Math.min(255,t/n),Math.min(255,e/n),Math.min(255,r/n))):new mt(u,u,u)},Ja.darker=function(n){return n=Math.pow(.7,arguments.length?n:1),new mt(n*this.r,n*this.g,n*this.b)},Ja.hsl=function(){return _t(this.r,this.g,this.b)},Ja.toString=function(){return"#"+xt(this.r)+xt(this.g)+xt(this.b)};var Ga=ta.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});Ga.forEach(function(n,t){Ga.set(n,yt(t))}),ta.functor=Et,ta.xhr=At(y),ta.dsv=function(n,t){function e(n,e,i){arguments.length<3&&(i=e,e=null);var o=Nt(n,t,null==e?r:u(e),i);return o.row=function(n){return arguments.length?o.response(null==(e=n)?r:u(n)):e},o}function r(n){return e.parse(n.responseText)}function u(n){return function(t){return e.parse(t.responseText,n)}}function i(t){return t.map(o).join(n)}function o(n){return a.test(n)?'"'+n.replace(/\"/g,'""')+'"':n}var a=new RegExp('["'+n+"\n]"),c=n.charCodeAt(0);return e.parse=function(n,t){var r;return e.parseRows(n,function(n,e){if(r)return r(n,e-1);var u=new Function("d","return {"+n.map(function(n,t){return JSON.stringify(n)+": d["+t+"]"}).join(",")+"}");r=t?function(n,e){return t(u(n),e)}:u})},e.parseRows=function(n,t){function e(){if(s>=l)return o;if(u)return u=!1,i;var t=s;if(34===n.charCodeAt(t)){for(var e=t;e++<l;)if(34===n.charCodeAt(e)){if(34!==n.charCodeAt(e+1))break;++e}s=e+2;var r=n.charCodeAt(e+1);return 13===r?(u=!0,10===n.charCodeAt(e+2)&&++s):10===r&&(u=!0),n.slice(t+1,e).replace(/""/g,'"')}for(;l>s;){var r=n.charCodeAt(s++),a=1;if(10===r)u=!0;else if(13===r)u=!0,10===n.charCodeAt(s)&&(++s,++a);else if(r!==c)continue;return n.slice(t,s-a)}return n.slice(t)}for(var r,u,i={},o={},a=[],l=n.length,s=0,f=0;(r=e())!==o;){for(var h=[];r!==i&&r!==o;)h.push(r),r=e();t&&null==(h=t(h,f++))||a.push(h)}return a},e.format=function(t){if(Array.isArray(t[0]))return e.formatRows(t);var r=new m,u=[];return t.forEach(function(n){for(var t in n)r.has(t)||u.push(r.add(t))}),[u.map(o).join(n)].concat(t.map(function(t){return u.map(function(n){return o(t[n])}).join(n)})).join("\n")},e.formatRows=function(n){return n.map(i).join("\n")},e},ta.csv=ta.dsv(",","text/csv"),ta.tsv=ta.dsv("	","text/tab-separated-values");var Ka,Qa,nc,tc,ec,rc=this[x(this,"requestAnimationFrame")]||function(n){setTimeout(n,17)};ta.timer=function(n,t,e){var r=arguments.length;2>r&&(t=0),3>r&&(e=Date.now());var u=e+t,i={c:n,t:u,f:!1,n:null};Qa?Qa.n=i:Ka=i,Qa=i,nc||(tc=clearTimeout(tc),nc=1,rc(qt))},ta.timer.flush=function(){Lt(),Tt()},ta.round=function(n,t){return t?Math.round(n*(t=Math.pow(10,t)))/t:Math.round(n)};var uc=["y","z","a","f","p","n","\xb5","m","","k","M","G","T","P","E","Z","Y"].map(Dt);ta.formatPrefix=function(n,t){var e=0;return n&&(0>n&&(n*=-1),t&&(n=ta.round(n,Rt(n,t))),e=1+Math.floor(1e-12+Math.log(n)/Math.LN10),e=Math.max(-24,Math.min(24,3*Math.floor((e-1)/3)))),uc[8+e/3]};var ic=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,oc=ta.map({b:function(n){return n.toString(2)},c:function(n){return String.fromCharCode(n)},o:function(n){return n.toString(8)},x:function(n){return n.toString(16)},X:function(n){return n.toString(16).toUpperCase()},g:function(n,t){return n.toPrecision(t)},e:function(n,t){return n.toExponential(t)},f:function(n,t){return n.toFixed(t)},r:function(n,t){return(n=ta.round(n,Rt(n,t))).toFixed(Math.max(0,Math.min(20,Rt(n*(1+1e-15),t))))}}),ac=ta.time={},cc=Date;jt.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){lc.setUTCDate.apply(this._,arguments)},setDay:function(){lc.setUTCDay.apply(this._,arguments)},setFullYear:function(){lc.setUTCFullYear.apply(this._,arguments)},setHours:function(){lc.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){lc.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){lc.setUTCMinutes.apply(this._,arguments)},setMonth:function(){lc.setUTCMonth.apply(this._,arguments)},setSeconds:function(){lc.setUTCSeconds.apply(this._,arguments)},setTime:function(){lc.setTime.apply(this._,arguments)}};var lc=Date.prototype;ac.year=Ft(function(n){return n=ac.day(n),n.setMonth(0,1),n},function(n,t){n.setFullYear(n.getFullYear()+t)},function(n){return n.getFullYear()}),ac.years=ac.year.range,ac.years.utc=ac.year.utc.range,ac.day=Ft(function(n){var t=new cc(2e3,0);return t.setFullYear(n.getFullYear(),n.getMonth(),n.getDate()),t},function(n,t){n.setDate(n.getDate()+t)},function(n){return n.getDate()-1}),ac.days=ac.day.range,ac.days.utc=ac.day.utc.range,ac.dayOfYear=function(n){var t=ac.year(n);return Math.floor((n-t-6e4*(n.getTimezoneOffset()-t.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach(function(n,t){t=7-t;var e=ac[n]=Ft(function(n){return(n=ac.day(n)).setDate(n.getDate()-(n.getDay()+t)%7),n},function(n,t){n.setDate(n.getDate()+7*Math.floor(t))},function(n){var e=ac.year(n).getDay();return Math.floor((ac.dayOfYear(n)+(e+t)%7)/7)-(e!==t)});ac[n+"s"]=e.range,ac[n+"s"].utc=e.utc.range,ac[n+"OfYear"]=function(n){var e=ac.year(n).getDay();return Math.floor((ac.dayOfYear(n)+(e+t)%7)/7)}}),ac.week=ac.sunday,ac.weeks=ac.sunday.range,ac.weeks.utc=ac.sunday.utc.range,ac.weekOfYear=ac.sundayOfYear;var sc={"-":"",_:" ",0:"0"},fc=/^\s*\d+/,hc=/^%/;ta.locale=function(n){return{numberFormat:Pt(n),timeFormat:Ot(n)}};var gc=ta.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});ta.format=gc.numberFormat,ta.geo={},ce.prototype={s:0,t:0,add:function(n){le(n,this.t,pc),le(pc.s,this.s,this),this.s?this.t+=pc.t:this.s=pc.t
},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var pc=new ce;ta.geo.stream=function(n,t){n&&vc.hasOwnProperty(n.type)?vc[n.type](n,t):se(n,t)};var vc={Feature:function(n,t){se(n.geometry,t)},FeatureCollection:function(n,t){for(var e=n.features,r=-1,u=e.length;++r<u;)se(e[r].geometry,t)}},dc={Sphere:function(n,t){t.sphere()},Point:function(n,t){n=n.coordinates,t.point(n[0],n[1],n[2])},MultiPoint:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)n=e[r],t.point(n[0],n[1],n[2])},LineString:function(n,t){fe(n.coordinates,t,0)},MultiLineString:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)fe(e[r],t,0)},Polygon:function(n,t){he(n.coordinates,t)},MultiPolygon:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)he(e[r],t)},GeometryCollection:function(n,t){for(var e=n.geometries,r=-1,u=e.length;++r<u;)se(e[r],t)}};ta.geo.area=function(n){return mc=0,ta.geo.stream(n,Mc),mc};var mc,yc=new ce,Mc={sphere:function(){mc+=4*qa},point:b,lineStart:b,lineEnd:b,polygonStart:function(){yc.reset(),Mc.lineStart=ge},polygonEnd:function(){var n=2*yc;mc+=0>n?4*qa+n:n,Mc.lineStart=Mc.lineEnd=Mc.point=b}};ta.geo.bounds=function(){function n(n,t){M.push(x=[s=n,h=n]),f>t&&(f=t),t>g&&(g=t)}function t(t,e){var r=pe([t*Da,e*Da]);if(m){var u=de(m,r),i=[u[1],-u[0],0],o=de(i,u);Me(o),o=xe(o);var c=t-p,l=c>0?1:-1,v=o[0]*Pa*l,d=ga(c)>180;if(d^(v>l*p&&l*t>v)){var y=o[1]*Pa;y>g&&(g=y)}else if(v=(v+360)%360-180,d^(v>l*p&&l*t>v)){var y=-o[1]*Pa;f>y&&(f=y)}else f>e&&(f=e),e>g&&(g=e);d?p>t?a(s,t)>a(s,h)&&(h=t):a(t,h)>a(s,h)&&(s=t):h>=s?(s>t&&(s=t),t>h&&(h=t)):t>p?a(s,t)>a(s,h)&&(h=t):a(t,h)>a(s,h)&&(s=t)}else n(t,e);m=r,p=t}function e(){b.point=t}function r(){x[0]=s,x[1]=h,b.point=n,m=null}function u(n,e){if(m){var r=n-p;y+=ga(r)>180?r+(r>0?360:-360):r}else v=n,d=e;Mc.point(n,e),t(n,e)}function i(){Mc.lineStart()}function o(){u(v,d),Mc.lineEnd(),ga(y)>Ca&&(s=-(h=180)),x[0]=s,x[1]=h,m=null}function a(n,t){return(t-=n)<0?t+360:t}function c(n,t){return n[0]-t[0]}function l(n,t){return t[0]<=t[1]?t[0]<=n&&n<=t[1]:n<t[0]||t[1]<n}var s,f,h,g,p,v,d,m,y,M,x,b={point:n,lineStart:e,lineEnd:r,polygonStart:function(){b.point=u,b.lineStart=i,b.lineEnd=o,y=0,Mc.polygonStart()},polygonEnd:function(){Mc.polygonEnd(),b.point=n,b.lineStart=e,b.lineEnd=r,0>yc?(s=-(h=180),f=-(g=90)):y>Ca?g=90:-Ca>y&&(f=-90),x[0]=s,x[1]=h}};return function(n){g=h=-(s=f=1/0),M=[],ta.geo.stream(n,b);var t=M.length;if(t){M.sort(c);for(var e,r=1,u=M[0],i=[u];t>r;++r)e=M[r],l(e[0],u)||l(e[1],u)?(a(u[0],e[1])>a(u[0],u[1])&&(u[1]=e[1]),a(e[0],u[1])>a(u[0],u[1])&&(u[0]=e[0])):i.push(u=e);for(var o,e,p=-1/0,t=i.length-1,r=0,u=i[t];t>=r;u=e,++r)e=i[r],(o=a(u[1],e[0]))>p&&(p=o,s=e[0],h=u[1])}return M=x=null,1/0===s||1/0===f?[[0/0,0/0],[0/0,0/0]]:[[s,f],[h,g]]}}(),ta.geo.centroid=function(n){xc=bc=_c=wc=Sc=kc=Ec=Ac=Nc=Cc=zc=0,ta.geo.stream(n,qc);var t=Nc,e=Cc,r=zc,u=t*t+e*e+r*r;return za>u&&(t=kc,e=Ec,r=Ac,Ca>bc&&(t=_c,e=wc,r=Sc),u=t*t+e*e+r*r,za>u)?[0/0,0/0]:[Math.atan2(e,t)*Pa,tt(r/Math.sqrt(u))*Pa]};var xc,bc,_c,wc,Sc,kc,Ec,Ac,Nc,Cc,zc,qc={sphere:b,point:_e,lineStart:Se,lineEnd:ke,polygonStart:function(){qc.lineStart=Ee},polygonEnd:function(){qc.lineStart=Se}},Lc=Le(Ne,Pe,je,[-qa,-qa/2]),Tc=1e9;ta.geo.clipExtent=function(){var n,t,e,r,u,i,o={stream:function(n){return u&&(u.valid=!1),u=i(n),u.valid=!0,u},extent:function(a){return arguments.length?(i=Ie(n=+a[0][0],t=+a[0][1],e=+a[1][0],r=+a[1][1]),u&&(u.valid=!1,u=null),o):[[n,t],[e,r]]}};return o.extent([[0,0],[960,500]])},(ta.geo.conicEqualArea=function(){return Ye(Ze)}).raw=Ze,ta.geo.albers=function(){return ta.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},ta.geo.albersUsa=function(){function n(n){var i=n[0],o=n[1];return t=null,e(i,o),t||(r(i,o),t)||u(i,o),t}var t,e,r,u,i=ta.geo.albers(),o=ta.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),a=ta.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),c={point:function(n,e){t=[n,e]}};return n.invert=function(n){var t=i.scale(),e=i.translate(),r=(n[0]-e[0])/t,u=(n[1]-e[1])/t;return(u>=.12&&.234>u&&r>=-.425&&-.214>r?o:u>=.166&&.234>u&&r>=-.214&&-.115>r?a:i).invert(n)},n.stream=function(n){var t=i.stream(n),e=o.stream(n),r=a.stream(n);return{point:function(n,u){t.point(n,u),e.point(n,u),r.point(n,u)},sphere:function(){t.sphere(),e.sphere(),r.sphere()},lineStart:function(){t.lineStart(),e.lineStart(),r.lineStart()},lineEnd:function(){t.lineEnd(),e.lineEnd(),r.lineEnd()},polygonStart:function(){t.polygonStart(),e.polygonStart(),r.polygonStart()},polygonEnd:function(){t.polygonEnd(),e.polygonEnd(),r.polygonEnd()}}},n.precision=function(t){return arguments.length?(i.precision(t),o.precision(t),a.precision(t),n):i.precision()},n.scale=function(t){return arguments.length?(i.scale(t),o.scale(.35*t),a.scale(t),n.translate(i.translate())):i.scale()},n.translate=function(t){if(!arguments.length)return i.translate();var l=i.scale(),s=+t[0],f=+t[1];return e=i.translate(t).clipExtent([[s-.455*l,f-.238*l],[s+.455*l,f+.238*l]]).stream(c).point,r=o.translate([s-.307*l,f+.201*l]).clipExtent([[s-.425*l+Ca,f+.12*l+Ca],[s-.214*l-Ca,f+.234*l-Ca]]).stream(c).point,u=a.translate([s-.205*l,f+.212*l]).clipExtent([[s-.214*l+Ca,f+.166*l+Ca],[s-.115*l-Ca,f+.234*l-Ca]]).stream(c).point,n},n.scale(1070)};var Rc,Dc,Pc,Uc,jc,Fc,Hc={point:b,lineStart:b,lineEnd:b,polygonStart:function(){Dc=0,Hc.lineStart=Ve},polygonEnd:function(){Hc.lineStart=Hc.lineEnd=Hc.point=b,Rc+=ga(Dc/2)}},Oc={point:Xe,lineStart:b,lineEnd:b,polygonStart:b,polygonEnd:b},Ic={point:We,lineStart:Je,lineEnd:Ge,polygonStart:function(){Ic.lineStart=Ke},polygonEnd:function(){Ic.point=We,Ic.lineStart=Je,Ic.lineEnd=Ge}};ta.geo.path=function(){function n(n){return n&&("function"==typeof a&&i.pointRadius(+a.apply(this,arguments)),o&&o.valid||(o=u(i)),ta.geo.stream(n,o)),i.result()}function t(){return o=null,n}var e,r,u,i,o,a=4.5;return n.area=function(n){return Rc=0,ta.geo.stream(n,u(Hc)),Rc},n.centroid=function(n){return _c=wc=Sc=kc=Ec=Ac=Nc=Cc=zc=0,ta.geo.stream(n,u(Ic)),zc?[Nc/zc,Cc/zc]:Ac?[kc/Ac,Ec/Ac]:Sc?[_c/Sc,wc/Sc]:[0/0,0/0]},n.bounds=function(n){return jc=Fc=-(Pc=Uc=1/0),ta.geo.stream(n,u(Oc)),[[Pc,Uc],[jc,Fc]]},n.projection=function(n){return arguments.length?(u=(e=n)?n.stream||tr(n):y,t()):e},n.context=function(n){return arguments.length?(i=null==(r=n)?new $e:new Qe(n),"function"!=typeof a&&i.pointRadius(a),t()):r},n.pointRadius=function(t){return arguments.length?(a="function"==typeof t?t:(i.pointRadius(+t),+t),n):a},n.projection(ta.geo.albersUsa()).context(null)},ta.geo.transform=function(n){return{stream:function(t){var e=new er(t);for(var r in n)e[r]=n[r];return e}}},er.prototype={point:function(n,t){this.stream.point(n,t)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},ta.geo.projection=ur,ta.geo.projectionMutator=ir,(ta.geo.equirectangular=function(){return ur(ar)}).raw=ar.invert=ar,ta.geo.rotation=function(n){function t(t){return t=n(t[0]*Da,t[1]*Da),t[0]*=Pa,t[1]*=Pa,t}return n=lr(n[0]%360*Da,n[1]*Da,n.length>2?n[2]*Da:0),t.invert=function(t){return t=n.invert(t[0]*Da,t[1]*Da),t[0]*=Pa,t[1]*=Pa,t},t},cr.invert=ar,ta.geo.circle=function(){function n(){var n="function"==typeof r?r.apply(this,arguments):r,t=lr(-n[0]*Da,-n[1]*Da,0).invert,u=[];return e(null,null,1,{point:function(n,e){u.push(n=t(n,e)),n[0]*=Pa,n[1]*=Pa}}),{type:"Polygon",coordinates:[u]}}var t,e,r=[0,0],u=6;return n.origin=function(t){return arguments.length?(r=t,n):r},n.angle=function(r){return arguments.length?(e=gr((t=+r)*Da,u*Da),n):t},n.precision=function(r){return arguments.length?(e=gr(t*Da,(u=+r)*Da),n):u},n.angle(90)},ta.geo.distance=function(n,t){var e,r=(t[0]-n[0])*Da,u=n[1]*Da,i=t[1]*Da,o=Math.sin(r),a=Math.cos(r),c=Math.sin(u),l=Math.cos(u),s=Math.sin(i),f=Math.cos(i);return Math.atan2(Math.sqrt((e=f*o)*e+(e=l*s-c*f*a)*e),c*s+l*f*a)},ta.geo.graticule=function(){function n(){return{type:"MultiLineString",coordinates:t()}}function t(){return ta.range(Math.ceil(i/d)*d,u,d).map(h).concat(ta.range(Math.ceil(l/m)*m,c,m).map(g)).concat(ta.range(Math.ceil(r/p)*p,e,p).filter(function(n){return ga(n%d)>Ca}).map(s)).concat(ta.range(Math.ceil(a/v)*v,o,v).filter(function(n){return ga(n%m)>Ca}).map(f))}var e,r,u,i,o,a,c,l,s,f,h,g,p=10,v=p,d=90,m=360,y=2.5;return n.lines=function(){return t().map(function(n){return{type:"LineString",coordinates:n}})},n.outline=function(){return{type:"Polygon",coordinates:[h(i).concat(g(c).slice(1),h(u).reverse().slice(1),g(l).reverse().slice(1))]}},n.extent=function(t){return arguments.length?n.majorExtent(t).minorExtent(t):n.minorExtent()},n.majorExtent=function(t){return arguments.length?(i=+t[0][0],u=+t[1][0],l=+t[0][1],c=+t[1][1],i>u&&(t=i,i=u,u=t),l>c&&(t=l,l=c,c=t),n.precision(y)):[[i,l],[u,c]]},n.minorExtent=function(t){return arguments.length?(r=+t[0][0],e=+t[1][0],a=+t[0][1],o=+t[1][1],r>e&&(t=r,r=e,e=t),a>o&&(t=a,a=o,o=t),n.precision(y)):[[r,a],[e,o]]},n.step=function(t){return arguments.length?n.majorStep(t).minorStep(t):n.minorStep()},n.majorStep=function(t){return arguments.length?(d=+t[0],m=+t[1],n):[d,m]},n.minorStep=function(t){return arguments.length?(p=+t[0],v=+t[1],n):[p,v]},n.precision=function(t){return arguments.length?(y=+t,s=vr(a,o,90),f=dr(r,e,y),h=vr(l,c,90),g=dr(i,u,y),n):y},n.majorExtent([[-180,-90+Ca],[180,90-Ca]]).minorExtent([[-180,-80-Ca],[180,80+Ca]])},ta.geo.greatArc=function(){function n(){return{type:"LineString",coordinates:[t||r.apply(this,arguments),e||u.apply(this,arguments)]}}var t,e,r=mr,u=yr;return n.distance=function(){return ta.geo.distance(t||r.apply(this,arguments),e||u.apply(this,arguments))},n.source=function(e){return arguments.length?(r=e,t="function"==typeof e?null:e,n):r},n.target=function(t){return arguments.length?(u=t,e="function"==typeof t?null:t,n):u},n.precision=function(){return arguments.length?n:0},n},ta.geo.interpolate=function(n,t){return Mr(n[0]*Da,n[1]*Da,t[0]*Da,t[1]*Da)},ta.geo.length=function(n){return Yc=0,ta.geo.stream(n,Zc),Yc};var Yc,Zc={sphere:b,point:b,lineStart:xr,lineEnd:b,polygonStart:b,polygonEnd:b},Vc=br(function(n){return Math.sqrt(2/(1+n))},function(n){return 2*Math.asin(n/2)});(ta.geo.azimuthalEqualArea=function(){return ur(Vc)}).raw=Vc;var Xc=br(function(n){var t=Math.acos(n);return t&&t/Math.sin(t)},y);(ta.geo.azimuthalEquidistant=function(){return ur(Xc)}).raw=Xc,(ta.geo.conicConformal=function(){return Ye(_r)}).raw=_r,(ta.geo.conicEquidistant=function(){return Ye(wr)}).raw=wr;var $c=br(function(n){return 1/n},Math.atan);(ta.geo.gnomonic=function(){return ur($c)}).raw=$c,Sr.invert=function(n,t){return[n,2*Math.atan(Math.exp(t))-Ra]},(ta.geo.mercator=function(){return kr(Sr)}).raw=Sr;var Bc=br(function(){return 1},Math.asin);(ta.geo.orthographic=function(){return ur(Bc)}).raw=Bc;var Wc=br(function(n){return 1/(1+n)},function(n){return 2*Math.atan(n)});(ta.geo.stereographic=function(){return ur(Wc)}).raw=Wc,Er.invert=function(n,t){return[-t,2*Math.atan(Math.exp(n))-Ra]},(ta.geo.transverseMercator=function(){var n=kr(Er),t=n.center,e=n.rotate;return n.center=function(n){return n?t([-n[1],n[0]]):(n=t(),[n[1],-n[0]])},n.rotate=function(n){return n?e([n[0],n[1],n.length>2?n[2]+90:90]):(n=e(),[n[0],n[1],n[2]-90])},e([0,0,90])}).raw=Er,ta.geom={},ta.geom.hull=function(n){function t(n){if(n.length<3)return[];var t,u=Et(e),i=Et(r),o=n.length,a=[],c=[];for(t=0;o>t;t++)a.push([+u.call(this,n[t],t),+i.call(this,n[t],t),t]);for(a.sort(zr),t=0;o>t;t++)c.push([a[t][0],-a[t][1]]);var l=Cr(a),s=Cr(c),f=s[0]===l[0],h=s[s.length-1]===l[l.length-1],g=[];for(t=l.length-1;t>=0;--t)g.push(n[a[l[t]][2]]);for(t=+f;t<s.length-h;++t)g.push(n[a[s[t]][2]]);return g}var e=Ar,r=Nr;return arguments.length?t(n):(t.x=function(n){return arguments.length?(e=n,t):e},t.y=function(n){return arguments.length?(r=n,t):r},t)},ta.geom.polygon=function(n){return ya(n,Jc),n};var Jc=ta.geom.polygon.prototype=[];Jc.area=function(){for(var n,t=-1,e=this.length,r=this[e-1],u=0;++t<e;)n=r,r=this[t],u+=n[1]*r[0]-n[0]*r[1];return.5*u},Jc.centroid=function(n){var t,e,r=-1,u=this.length,i=0,o=0,a=this[u-1];for(arguments.length||(n=-1/(6*this.area()));++r<u;)t=a,a=this[r],e=t[0]*a[1]-a[0]*t[1],i+=(t[0]+a[0])*e,o+=(t[1]+a[1])*e;return[i*n,o*n]},Jc.clip=function(n){for(var t,e,r,u,i,o,a=Tr(n),c=-1,l=this.length-Tr(this),s=this[l-1];++c<l;){for(t=n.slice(),n.length=0,u=this[c],i=t[(r=t.length-a)-1],e=-1;++e<r;)o=t[e],qr(o,s,u)?(qr(i,s,u)||n.push(Lr(i,o,s,u)),n.push(o)):qr(i,s,u)&&n.push(Lr(i,o,s,u)),i=o;a&&n.push(n[0]),s=u}return n};var Gc,Kc,Qc,nl,tl,el=[],rl=[];Or.prototype.prepare=function(){for(var n,t=this.edges,e=t.length;e--;)n=t[e].edge,n.b&&n.a||t.splice(e,1);return t.sort(Yr),t.length},Qr.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},nu.prototype={insert:function(n,t){var e,r,u;if(n){if(t.P=n,t.N=n.N,n.N&&(n.N.P=t),n.N=t,n.R){for(n=n.R;n.L;)n=n.L;n.L=t}else n.R=t;e=n}else this._?(n=uu(this._),t.P=null,t.N=n,n.P=n.L=t,e=n):(t.P=t.N=null,this._=t,e=null);for(t.L=t.R=null,t.U=e,t.C=!0,n=t;e&&e.C;)r=e.U,e===r.L?(u=r.R,u&&u.C?(e.C=u.C=!1,r.C=!0,n=r):(n===e.R&&(eu(this,e),n=e,e=n.U),e.C=!1,r.C=!0,ru(this,r))):(u=r.L,u&&u.C?(e.C=u.C=!1,r.C=!0,n=r):(n===e.L&&(ru(this,e),n=e,e=n.U),e.C=!1,r.C=!0,eu(this,r))),e=n.U;this._.C=!1},remove:function(n){n.N&&(n.N.P=n.P),n.P&&(n.P.N=n.N),n.N=n.P=null;var t,e,r,u=n.U,i=n.L,o=n.R;if(e=i?o?uu(o):i:o,u?u.L===n?u.L=e:u.R=e:this._=e,i&&o?(r=e.C,e.C=n.C,e.L=i,i.U=e,e!==o?(u=e.U,e.U=n.U,n=e.R,u.L=n,e.R=o,o.U=e):(e.U=u,u=e,n=e.R)):(r=n.C,n=e),n&&(n.U=u),!r){if(n&&n.C)return void(n.C=!1);do{if(n===this._)break;if(n===u.L){if(t=u.R,t.C&&(t.C=!1,u.C=!0,eu(this,u),t=u.R),t.L&&t.L.C||t.R&&t.R.C){t.R&&t.R.C||(t.L.C=!1,t.C=!0,ru(this,t),t=u.R),t.C=u.C,u.C=t.R.C=!1,eu(this,u),n=this._;break}}else if(t=u.L,t.C&&(t.C=!1,u.C=!0,ru(this,u),t=u.L),t.L&&t.L.C||t.R&&t.R.C){t.L&&t.L.C||(t.R.C=!1,t.C=!0,eu(this,t),t=u.L),t.C=u.C,u.C=t.L.C=!1,ru(this,u),n=this._;break}t.C=!0,n=u,u=u.U}while(!n.C);n&&(n.C=!1)}}},ta.geom.voronoi=function(n){function t(n){var t=new Array(n.length),r=a[0][0],u=a[0][1],i=a[1][0],o=a[1][1];return iu(e(n),a).cells.forEach(function(e,a){var c=e.edges,l=e.site,s=t[a]=c.length?c.map(function(n){var t=n.start();return[t.x,t.y]}):l.x>=r&&l.x<=i&&l.y>=u&&l.y<=o?[[r,o],[i,o],[i,u],[r,u]]:[];s.point=n[a]}),t}function e(n){return n.map(function(n,t){return{x:Math.round(i(n,t)/Ca)*Ca,y:Math.round(o(n,t)/Ca)*Ca,i:t}})}var r=Ar,u=Nr,i=r,o=u,a=ul;return n?t(n):(t.links=function(n){return iu(e(n)).edges.filter(function(n){return n.l&&n.r}).map(function(t){return{source:n[t.l.i],target:n[t.r.i]}})},t.triangles=function(n){var t=[];return iu(e(n)).cells.forEach(function(e,r){for(var u,i,o=e.site,a=e.edges.sort(Yr),c=-1,l=a.length,s=a[l-1].edge,f=s.l===o?s.r:s.l;++c<l;)u=s,i=f,s=a[c].edge,f=s.l===o?s.r:s.l,r<i.i&&r<f.i&&au(o,i,f)<0&&t.push([n[r],n[i.i],n[f.i]])}),t},t.x=function(n){return arguments.length?(i=Et(r=n),t):r},t.y=function(n){return arguments.length?(o=Et(u=n),t):u},t.clipExtent=function(n){return arguments.length?(a=null==n?ul:n,t):a===ul?null:a},t.size=function(n){return arguments.length?t.clipExtent(n&&[[0,0],n]):a===ul?null:a&&a[1]},t)};var ul=[[-1e6,-1e6],[1e6,1e6]];ta.geom.delaunay=function(n){return ta.geom.voronoi().triangles(n)},ta.geom.quadtree=function(n,t,e,r,u){function i(n){function i(n,t,e,r,u,i,o,a){if(!isNaN(e)&&!isNaN(r))if(n.leaf){var c=n.x,s=n.y;if(null!=c)if(ga(c-e)+ga(s-r)<.01)l(n,t,e,r,u,i,o,a);else{var f=n.point;n.x=n.y=n.point=null,l(n,f,c,s,u,i,o,a),l(n,t,e,r,u,i,o,a)}else n.x=e,n.y=r,n.point=t}else l(n,t,e,r,u,i,o,a)}function l(n,t,e,r,u,o,a,c){var l=.5*(u+a),s=.5*(o+c),f=e>=l,h=r>=s,g=h<<1|f;n.leaf=!1,n=n.nodes[g]||(n.nodes[g]=su()),f?u=l:a=l,h?o=s:c=s,i(n,t,e,r,u,o,a,c)}var s,f,h,g,p,v,d,m,y,M=Et(a),x=Et(c);if(null!=t)v=t,d=e,m=r,y=u;else if(m=y=-(v=d=1/0),f=[],h=[],p=n.length,o)for(g=0;p>g;++g)s=n[g],s.x<v&&(v=s.x),s.y<d&&(d=s.y),s.x>m&&(m=s.x),s.y>y&&(y=s.y),f.push(s.x),h.push(s.y);else for(g=0;p>g;++g){var b=+M(s=n[g],g),_=+x(s,g);v>b&&(v=b),d>_&&(d=_),b>m&&(m=b),_>y&&(y=_),f.push(b),h.push(_)}var w=m-v,S=y-d;w>S?y=d+w:m=v+S;var k=su();if(k.add=function(n){i(k,n,+M(n,++g),+x(n,g),v,d,m,y)},k.visit=function(n){fu(n,k,v,d,m,y)},k.find=function(n){return hu(k,n[0],n[1],v,d,m,y)},g=-1,null==t){for(;++g<p;)i(k,n[g],f[g],h[g],v,d,m,y);--g}else n.forEach(k.add);return f=h=n=s=null,k}var o,a=Ar,c=Nr;return(o=arguments.length)?(a=cu,c=lu,3===o&&(u=e,r=t,e=t=0),i(n)):(i.x=function(n){return arguments.length?(a=n,i):a},i.y=function(n){return arguments.length?(c=n,i):c},i.extent=function(n){return arguments.length?(null==n?t=e=r=u=null:(t=+n[0][0],e=+n[0][1],r=+n[1][0],u=+n[1][1]),i):null==t?null:[[t,e],[r,u]]},i.size=function(n){return arguments.length?(null==n?t=e=r=u=null:(t=e=0,r=+n[0],u=+n[1]),i):null==t?null:[r-t,u-e]},i)},ta.interpolateRgb=gu,ta.interpolateObject=pu,ta.interpolateNumber=vu,ta.interpolateString=du;var il=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,ol=new RegExp(il.source,"g");ta.interpolate=mu,ta.interpolators=[function(n,t){var e=typeof t;return("string"===e?Ga.has(t)||/^(#|rgb\(|hsl\()/.test(t)?gu:du:t instanceof ot?gu:Array.isArray(t)?yu:"object"===e&&isNaN(t)?pu:vu)(n,t)}],ta.interpolateArray=yu;var al=function(){return y},cl=ta.map({linear:al,poly:ku,quad:function(){return _u},cubic:function(){return wu},sin:function(){return Eu},exp:function(){return Au},circle:function(){return Nu},elastic:Cu,back:zu,bounce:function(){return qu}}),ll=ta.map({"in":y,out:xu,"in-out":bu,"out-in":function(n){return bu(xu(n))}});ta.ease=function(n){var t=n.indexOf("-"),e=t>=0?n.slice(0,t):n,r=t>=0?n.slice(t+1):"in";return e=cl.get(e)||al,r=ll.get(r)||y,Mu(r(e.apply(null,ea.call(arguments,1))))},ta.interpolateHcl=Lu,ta.interpolateHsl=Tu,ta.interpolateLab=Ru,ta.interpolateRound=Du,ta.transform=function(n){var t=ua.createElementNS(ta.ns.prefix.svg,"g");return(ta.transform=function(n){if(null!=n){t.setAttribute("transform",n);var e=t.transform.baseVal.consolidate()}return new Pu(e?e.matrix:sl)})(n)},Pu.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var sl={a:1,b:0,c:0,d:1,e:0,f:0};ta.interpolateTransform=Hu,ta.layout={},ta.layout.bundle=function(){return function(n){for(var t=[],e=-1,r=n.length;++e<r;)t.push(Yu(n[e]));return t}},ta.layout.chord=function(){function n(){var n,l,f,h,g,p={},v=[],d=ta.range(i),m=[];for(e=[],r=[],n=0,h=-1;++h<i;){for(l=0,g=-1;++g<i;)l+=u[h][g];v.push(l),m.push(ta.range(i)),n+=l}for(o&&d.sort(function(n,t){return o(v[n],v[t])}),a&&m.forEach(function(n,t){n.sort(function(n,e){return a(u[t][n],u[t][e])})}),n=(La-s*i)/n,l=0,h=-1;++h<i;){for(f=l,g=-1;++g<i;){var y=d[h],M=m[y][g],x=u[y][M],b=l,_=l+=x*n;p[y+"-"+M]={index:y,subindex:M,startAngle:b,endAngle:_,value:x}}r[y]={index:y,startAngle:f,endAngle:l,value:(l-f)/n},l+=s}for(h=-1;++h<i;)for(g=h-1;++g<i;){var w=p[h+"-"+g],S=p[g+"-"+h];(w.value||S.value)&&e.push(w.value<S.value?{source:S,target:w}:{source:w,target:S})}c&&t()}function t(){e.sort(function(n,t){return c((n.source.value+n.target.value)/2,(t.source.value+t.target.value)/2)})}var e,r,u,i,o,a,c,l={},s=0;return l.matrix=function(n){return arguments.length?(i=(u=n)&&u.length,e=r=null,l):u},l.padding=function(n){return arguments.length?(s=n,e=r=null,l):s},l.sortGroups=function(n){return arguments.length?(o=n,e=r=null,l):o},l.sortSubgroups=function(n){return arguments.length?(a=n,e=null,l):a},l.sortChords=function(n){return arguments.length?(c=n,e&&t(),l):c},l.chords=function(){return e||n(),e},l.groups=function(){return r||n(),r},l},ta.layout.force=function(){function n(n){return function(t,e,r,u){if(t.point!==n){var i=t.cx-n.x,o=t.cy-n.y,a=u-e,c=i*i+o*o;if(c>a*a/d){if(p>c){var l=t.charge/c;n.px-=i*l,n.py-=o*l}return!0}if(t.point&&c&&p>c){var l=t.pointCharge/c;n.px-=i*l,n.py-=o*l}}return!t.charge}}function t(n){n.px=ta.event.x,n.py=ta.event.y,a.resume()}var e,r,u,i,o,a={},c=ta.dispatch("start","tick","end"),l=[1,1],s=.9,f=fl,h=hl,g=-30,p=gl,v=.1,d=.64,m=[],M=[];return a.tick=function(){if((r*=.99)<.005)return c.end({type:"end",alpha:r=0}),!0;var t,e,a,f,h,p,d,y,x,b=m.length,_=M.length;for(e=0;_>e;++e)a=M[e],f=a.source,h=a.target,y=h.x-f.x,x=h.y-f.y,(p=y*y+x*x)&&(p=r*i[e]*((p=Math.sqrt(p))-u[e])/p,y*=p,x*=p,h.x-=y*(d=f.weight/(h.weight+f.weight)),h.y-=x*d,f.x+=y*(d=1-d),f.y+=x*d);if((d=r*v)&&(y=l[0]/2,x=l[1]/2,e=-1,d))for(;++e<b;)a=m[e],a.x+=(y-a.x)*d,a.y+=(x-a.y)*d;if(g)for(Ju(t=ta.geom.quadtree(m),r,o),e=-1;++e<b;)(a=m[e]).fixed||t.visit(n(a));for(e=-1;++e<b;)a=m[e],a.fixed?(a.x=a.px,a.y=a.py):(a.x-=(a.px-(a.px=a.x))*s,a.y-=(a.py-(a.py=a.y))*s);c.tick({type:"tick",alpha:r})},a.nodes=function(n){return arguments.length?(m=n,a):m},a.links=function(n){return arguments.length?(M=n,a):M},a.size=function(n){return arguments.length?(l=n,a):l},a.linkDistance=function(n){return arguments.length?(f="function"==typeof n?n:+n,a):f},a.distance=a.linkDistance,a.linkStrength=function(n){return arguments.length?(h="function"==typeof n?n:+n,a):h},a.friction=function(n){return arguments.length?(s=+n,a):s},a.charge=function(n){return arguments.length?(g="function"==typeof n?n:+n,a):g},a.chargeDistance=function(n){return arguments.length?(p=n*n,a):Math.sqrt(p)},a.gravity=function(n){return arguments.length?(v=+n,a):v},a.theta=function(n){return arguments.length?(d=n*n,a):Math.sqrt(d)},a.alpha=function(n){return arguments.length?(n=+n,r?r=n>0?n:0:n>0&&(c.start({type:"start",alpha:r=n}),ta.timer(a.tick)),a):r},a.start=function(){function n(n,r){if(!e){for(e=new Array(c),a=0;c>a;++a)e[a]=[];for(a=0;s>a;++a){var u=M[a];e[u.source.index].push(u.target),e[u.target.index].push(u.source)}}for(var i,o=e[t],a=-1,l=o.length;++a<l;)if(!isNaN(i=o[a][n]))return i;return Math.random()*r}var t,e,r,c=m.length,s=M.length,p=l[0],v=l[1];for(t=0;c>t;++t)(r=m[t]).index=t,r.weight=0;for(t=0;s>t;++t)r=M[t],"number"==typeof r.source&&(r.source=m[r.source]),"number"==typeof r.target&&(r.target=m[r.target]),++r.source.weight,++r.target.weight;for(t=0;c>t;++t)r=m[t],isNaN(r.x)&&(r.x=n("x",p)),isNaN(r.y)&&(r.y=n("y",v)),isNaN(r.px)&&(r.px=r.x),isNaN(r.py)&&(r.py=r.y);if(u=[],"function"==typeof f)for(t=0;s>t;++t)u[t]=+f.call(this,M[t],t);else for(t=0;s>t;++t)u[t]=f;if(i=[],"function"==typeof h)for(t=0;s>t;++t)i[t]=+h.call(this,M[t],t);else for(t=0;s>t;++t)i[t]=h;if(o=[],"function"==typeof g)for(t=0;c>t;++t)o[t]=+g.call(this,m[t],t);else for(t=0;c>t;++t)o[t]=g;return a.resume()},a.resume=function(){return a.alpha(.1)},a.stop=function(){return a.alpha(0)},a.drag=function(){return e||(e=ta.behavior.drag().origin(y).on("dragstart.force",Xu).on("drag.force",t).on("dragend.force",$u)),arguments.length?void this.on("mouseover.force",Bu).on("mouseout.force",Wu).call(e):e},ta.rebind(a,c,"on")};var fl=20,hl=1,gl=1/0;ta.layout.hierarchy=function(){function n(u){var i,o=[u],a=[];for(u.depth=0;null!=(i=o.pop());)if(a.push(i),(l=e.call(n,i,i.depth))&&(c=l.length)){for(var c,l,s;--c>=0;)o.push(s=l[c]),s.parent=i,s.depth=i.depth+1;r&&(i.value=0),i.children=l}else r&&(i.value=+r.call(n,i,i.depth)||0),delete i.children;return Qu(u,function(n){var e,u;t&&(e=n.children)&&e.sort(t),r&&(u=n.parent)&&(u.value+=n.value)}),a}var t=ei,e=ni,r=ti;return n.sort=function(e){return arguments.length?(t=e,n):t},n.children=function(t){return arguments.length?(e=t,n):e},n.value=function(t){return arguments.length?(r=t,n):r},n.revalue=function(t){return r&&(Ku(t,function(n){n.children&&(n.value=0)}),Qu(t,function(t){var e;t.children||(t.value=+r.call(n,t,t.depth)||0),(e=t.parent)&&(e.value+=t.value)})),t},n},ta.layout.partition=function(){function n(t,e,r,u){var i=t.children;if(t.x=e,t.y=t.depth*u,t.dx=r,t.dy=u,i&&(o=i.length)){var o,a,c,l=-1;for(r=t.value?r/t.value:0;++l<o;)n(a=i[l],e,c=a.value*r,u),e+=c}}function t(n){var e=n.children,r=0;if(e&&(u=e.length))for(var u,i=-1;++i<u;)r=Math.max(r,t(e[i]));return 1+r}function e(e,i){var o=r.call(this,e,i);return n(o[0],0,u[0],u[1]/t(o[0])),o}var r=ta.layout.hierarchy(),u=[1,1];return e.size=function(n){return arguments.length?(u=n,e):u},Gu(e,r)},ta.layout.pie=function(){function n(o){var a,c=o.length,l=o.map(function(e,r){return+t.call(n,e,r)}),s=+("function"==typeof r?r.apply(this,arguments):r),f=("function"==typeof u?u.apply(this,arguments):u)-s,h=Math.min(Math.abs(f)/c,+("function"==typeof i?i.apply(this,arguments):i)),g=h*(0>f?-1:1),p=(f-c*g)/ta.sum(l),v=ta.range(c),d=[];return null!=e&&v.sort(e===pl?function(n,t){return l[t]-l[n]}:function(n,t){return e(o[n],o[t])}),v.forEach(function(n){d[n]={data:o[n],value:a=l[n],startAngle:s,endAngle:s+=a*p+g,padAngle:h}}),d}var t=Number,e=pl,r=0,u=La,i=0;return n.value=function(e){return arguments.length?(t=e,n):t},n.sort=function(t){return arguments.length?(e=t,n):e},n.startAngle=function(t){return arguments.length?(r=t,n):r},n.endAngle=function(t){return arguments.length?(u=t,n):u},n.padAngle=function(t){return arguments.length?(i=t,n):i},n};var pl={};ta.layout.stack=function(){function n(a,c){if(!(h=a.length))return a;var l=a.map(function(e,r){return t.call(n,e,r)}),s=l.map(function(t){return t.map(function(t,e){return[i.call(n,t,e),o.call(n,t,e)]})}),f=e.call(n,s,c);l=ta.permute(l,f),s=ta.permute(s,f);var h,g,p,v,d=r.call(n,s,c),m=l[0].length;for(p=0;m>p;++p)for(u.call(n,l[0][p],v=d[p],s[0][p][1]),g=1;h>g;++g)u.call(n,l[g][p],v+=s[g-1][p][1],s[g][p][1]);return a}var t=y,e=ai,r=ci,u=oi,i=ui,o=ii;return n.values=function(e){return arguments.length?(t=e,n):t},n.order=function(t){return arguments.length?(e="function"==typeof t?t:vl.get(t)||ai,n):e},n.offset=function(t){return arguments.length?(r="function"==typeof t?t:dl.get(t)||ci,n):r},n.x=function(t){return arguments.length?(i=t,n):i},n.y=function(t){return arguments.length?(o=t,n):o},n.out=function(t){return arguments.length?(u=t,n):u},n};var vl=ta.map({"inside-out":function(n){var t,e,r=n.length,u=n.map(li),i=n.map(si),o=ta.range(r).sort(function(n,t){return u[n]-u[t]}),a=0,c=0,l=[],s=[];for(t=0;r>t;++t)e=o[t],c>a?(a+=i[e],l.push(e)):(c+=i[e],s.push(e));return s.reverse().concat(l)},reverse:function(n){return ta.range(n.length).reverse()},"default":ai}),dl=ta.map({silhouette:function(n){var t,e,r,u=n.length,i=n[0].length,o=[],a=0,c=[];for(e=0;i>e;++e){for(t=0,r=0;u>t;t++)r+=n[t][e][1];r>a&&(a=r),o.push(r)}for(e=0;i>e;++e)c[e]=(a-o[e])/2;return c},wiggle:function(n){var t,e,r,u,i,o,a,c,l,s=n.length,f=n[0],h=f.length,g=[];for(g[0]=c=l=0,e=1;h>e;++e){for(t=0,u=0;s>t;++t)u+=n[t][e][1];for(t=0,i=0,a=f[e][0]-f[e-1][0];s>t;++t){for(r=0,o=(n[t][e][1]-n[t][e-1][1])/(2*a);t>r;++r)o+=(n[r][e][1]-n[r][e-1][1])/a;i+=o*n[t][e][1]}g[e]=c-=u?i/u*a:0,l>c&&(l=c)}for(e=0;h>e;++e)g[e]-=l;return g},expand:function(n){var t,e,r,u=n.length,i=n[0].length,o=1/u,a=[];for(e=0;i>e;++e){for(t=0,r=0;u>t;t++)r+=n[t][e][1];if(r)for(t=0;u>t;t++)n[t][e][1]/=r;else for(t=0;u>t;t++)n[t][e][1]=o}for(e=0;i>e;++e)a[e]=0;return a},zero:ci});ta.layout.histogram=function(){function n(n,i){for(var o,a,c=[],l=n.map(e,this),s=r.call(this,l,i),f=u.call(this,s,l,i),i=-1,h=l.length,g=f.length-1,p=t?1:1/h;++i<g;)o=c[i]=[],o.dx=f[i+1]-(o.x=f[i]),o.y=0;if(g>0)for(i=-1;++i<h;)a=l[i],a>=s[0]&&a<=s[1]&&(o=c[ta.bisect(f,a,1,g)-1],o.y+=p,o.push(n[i]));return c}var t=!0,e=Number,r=pi,u=hi;return n.value=function(t){return arguments.length?(e=t,n):e},n.range=function(t){return arguments.length?(r=Et(t),n):r},n.bins=function(t){return arguments.length?(u="number"==typeof t?function(n){return gi(n,t)}:Et(t),n):u},n.frequency=function(e){return arguments.length?(t=!!e,n):t},n},ta.layout.pack=function(){function n(n,i){var o=e.call(this,n,i),a=o[0],c=u[0],l=u[1],s=null==t?Math.sqrt:"function"==typeof t?t:function(){return t};if(a.x=a.y=0,Qu(a,function(n){n.r=+s(n.value)}),Qu(a,Mi),r){var f=r*(t?1:Math.max(2*a.r/c,2*a.r/l))/2;Qu(a,function(n){n.r+=f}),Qu(a,Mi),Qu(a,function(n){n.r-=f})}return _i(a,c/2,l/2,t?1:1/Math.max(2*a.r/c,2*a.r/l)),o}var t,e=ta.layout.hierarchy().sort(vi),r=0,u=[1,1];return n.size=function(t){return arguments.length?(u=t,n):u},n.radius=function(e){return arguments.length?(t=null==e||"function"==typeof e?e:+e,n):t},n.padding=function(t){return arguments.length?(r=+t,n):r},Gu(n,e)},ta.layout.tree=function(){function n(n,u){var s=o.call(this,n,u),f=s[0],h=t(f);if(Qu(h,e),h.parent.m=-h.z,Ku(h,r),l)Ku(f,i);else{var g=f,p=f,v=f;Ku(f,function(n){n.x<g.x&&(g=n),n.x>p.x&&(p=n),n.depth>v.depth&&(v=n)});var d=a(g,p)/2-g.x,m=c[0]/(p.x+a(p,g)/2+d),y=c[1]/(v.depth||1);Ku(f,function(n){n.x=(n.x+d)*m,n.y=n.depth*y})}return s}function t(n){for(var t,e={A:null,children:[n]},r=[e];null!=(t=r.pop());)for(var u,i=t.children,o=0,a=i.length;a>o;++o)r.push((i[o]=u={_:i[o],parent:t,children:(u=i[o].children)&&u.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:o}).a=u);return e.children[0]}function e(n){var t=n.children,e=n.parent.children,r=n.i?e[n.i-1]:null;if(t.length){Ni(n);var i=(t[0].z+t[t.length-1].z)/2;r?(n.z=r.z+a(n._,r._),n.m=n.z-i):n.z=i}else r&&(n.z=r.z+a(n._,r._));n.parent.A=u(n,r,n.parent.A||e[0])}function r(n){n._.x=n.z+n.parent.m,n.m+=n.parent.m}function u(n,t,e){if(t){for(var r,u=n,i=n,o=t,c=u.parent.children[0],l=u.m,s=i.m,f=o.m,h=c.m;o=Ei(o),u=ki(u),o&&u;)c=ki(c),i=Ei(i),i.a=n,r=o.z+f-u.z-l+a(o._,u._),r>0&&(Ai(Ci(o,n,e),n,r),l+=r,s+=r),f+=o.m,l+=u.m,h+=c.m,s+=i.m;o&&!Ei(i)&&(i.t=o,i.m+=f-s),u&&!ki(c)&&(c.t=u,c.m+=l-h,e=n)}return e}function i(n){n.x*=c[0],n.y=n.depth*c[1]}var o=ta.layout.hierarchy().sort(null).value(null),a=Si,c=[1,1],l=null;return n.separation=function(t){return arguments.length?(a=t,n):a},n.size=function(t){return arguments.length?(l=null==(c=t)?i:null,n):l?null:c},n.nodeSize=function(t){return arguments.length?(l=null==(c=t)?null:i,n):l?c:null},Gu(n,o)},ta.layout.cluster=function(){function n(n,i){var o,a=t.call(this,n,i),c=a[0],l=0;Qu(c,function(n){var t=n.children;t&&t.length?(n.x=qi(t),n.y=zi(t)):(n.x=o?l+=e(n,o):0,n.y=0,o=n)});var s=Li(c),f=Ti(c),h=s.x-e(s,f)/2,g=f.x+e(f,s)/2;return Qu(c,u?function(n){n.x=(n.x-c.x)*r[0],n.y=(c.y-n.y)*r[1]}:function(n){n.x=(n.x-h)/(g-h)*r[0],n.y=(1-(c.y?n.y/c.y:1))*r[1]}),a}var t=ta.layout.hierarchy().sort(null).value(null),e=Si,r=[1,1],u=!1;return n.separation=function(t){return arguments.length?(e=t,n):e},n.size=function(t){return arguments.length?(u=null==(r=t),n):u?null:r},n.nodeSize=function(t){return arguments.length?(u=null!=(r=t),n):u?r:null},Gu(n,t)},ta.layout.treemap=function(){function n(n,t){for(var e,r,u=-1,i=n.length;++u<i;)r=(e=n[u]).value*(0>t?0:t),e.area=isNaN(r)||0>=r?0:r}function t(e){var i=e.children;if(i&&i.length){var o,a,c,l=f(e),s=[],h=i.slice(),p=1/0,v="slice"===g?l.dx:"dice"===g?l.dy:"slice-dice"===g?1&e.depth?l.dy:l.dx:Math.min(l.dx,l.dy);for(n(h,l.dx*l.dy/e.value),s.area=0;(c=h.length)>0;)s.push(o=h[c-1]),s.area+=o.area,"squarify"!==g||(a=r(s,v))<=p?(h.pop(),p=a):(s.area-=s.pop().area,u(s,v,l,!1),v=Math.min(l.dx,l.dy),s.length=s.area=0,p=1/0);s.length&&(u(s,v,l,!0),s.length=s.area=0),i.forEach(t)}}function e(t){var r=t.children;if(r&&r.length){var i,o=f(t),a=r.slice(),c=[];for(n(a,o.dx*o.dy/t.value),c.area=0;i=a.pop();)c.push(i),c.area+=i.area,null!=i.z&&(u(c,i.z?o.dx:o.dy,o,!a.length),c.length=c.area=0);r.forEach(e)}}function r(n,t){for(var e,r=n.area,u=0,i=1/0,o=-1,a=n.length;++o<a;)(e=n[o].area)&&(i>e&&(i=e),e>u&&(u=e));return r*=r,t*=t,r?Math.max(t*u*p/r,r/(t*i*p)):1/0}function u(n,t,e,r){var u,i=-1,o=n.length,a=e.x,l=e.y,s=t?c(n.area/t):0;if(t==e.dx){for((r||s>e.dy)&&(s=e.dy);++i<o;)u=n[i],u.x=a,u.y=l,u.dy=s,a+=u.dx=Math.min(e.x+e.dx-a,s?c(u.area/s):0);u.z=!0,u.dx+=e.x+e.dx-a,e.y+=s,e.dy-=s}else{for((r||s>e.dx)&&(s=e.dx);++i<o;)u=n[i],u.x=a,u.y=l,u.dx=s,l+=u.dy=Math.min(e.y+e.dy-l,s?c(u.area/s):0);u.z=!1,u.dy+=e.y+e.dy-l,e.x+=s,e.dx-=s}}function i(r){var u=o||a(r),i=u[0];return i.x=0,i.y=0,i.dx=l[0],i.dy=l[1],o&&a.revalue(i),n([i],i.dx*i.dy/i.value),(o?e:t)(i),h&&(o=u),u}var o,a=ta.layout.hierarchy(),c=Math.round,l=[1,1],s=null,f=Ri,h=!1,g="squarify",p=.5*(1+Math.sqrt(5));
return i.size=function(n){return arguments.length?(l=n,i):l},i.padding=function(n){function t(t){var e=n.call(i,t,t.depth);return null==e?Ri(t):Di(t,"number"==typeof e?[e,e,e,e]:e)}function e(t){return Di(t,n)}if(!arguments.length)return s;var r;return f=null==(s=n)?Ri:"function"==(r=typeof n)?t:"number"===r?(n=[n,n,n,n],e):e,i},i.round=function(n){return arguments.length?(c=n?Math.round:Number,i):c!=Number},i.sticky=function(n){return arguments.length?(h=n,o=null,i):h},i.ratio=function(n){return arguments.length?(p=n,i):p},i.mode=function(n){return arguments.length?(g=n+"",i):g},Gu(i,a)},ta.random={normal:function(n,t){var e=arguments.length;return 2>e&&(t=1),1>e&&(n=0),function(){var e,r,u;do e=2*Math.random()-1,r=2*Math.random()-1,u=e*e+r*r;while(!u||u>1);return n+t*e*Math.sqrt(-2*Math.log(u)/u)}},logNormal:function(){var n=ta.random.normal.apply(ta,arguments);return function(){return Math.exp(n())}},bates:function(n){var t=ta.random.irwinHall(n);return function(){return t()/n}},irwinHall:function(n){return function(){for(var t=0,e=0;n>e;e++)t+=Math.random();return t}}},ta.scale={};var ml={floor:y,ceil:y};ta.scale.linear=function(){return Ii([0,1],[0,1],mu,!1)};var yl={s:1,g:1,p:1,r:1,e:1};ta.scale.log=function(){return Ji(ta.scale.linear().domain([0,1]),10,!0,[1,10])};var Ml=ta.format(".0e"),xl={floor:function(n){return-Math.ceil(-n)},ceil:function(n){return-Math.floor(-n)}};ta.scale.pow=function(){return Gi(ta.scale.linear(),1,[0,1])},ta.scale.sqrt=function(){return ta.scale.pow().exponent(.5)},ta.scale.ordinal=function(){return Qi([],{t:"range",a:[[]]})},ta.scale.category10=function(){return ta.scale.ordinal().range(bl)},ta.scale.category20=function(){return ta.scale.ordinal().range(_l)},ta.scale.category20b=function(){return ta.scale.ordinal().range(wl)},ta.scale.category20c=function(){return ta.scale.ordinal().range(Sl)};var bl=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(Mt),_l=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(Mt),wl=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(Mt),Sl=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(Mt);ta.scale.quantile=function(){return no([],[])},ta.scale.quantize=function(){return to(0,1,[0,1])},ta.scale.threshold=function(){return eo([.5],[0,1])},ta.scale.identity=function(){return ro([0,1])},ta.svg={},ta.svg.arc=function(){function n(){var n=Math.max(0,+e.apply(this,arguments)),l=Math.max(0,+r.apply(this,arguments)),s=o.apply(this,arguments)-Ra,f=a.apply(this,arguments)-Ra,h=Math.abs(f-s),g=s>f?0:1;if(n>l&&(p=l,l=n,n=p),h>=Ta)return t(l,g)+(n?t(n,1-g):"")+"Z";var p,v,d,m,y,M,x,b,_,w,S,k,E=0,A=0,N=[];if((m=(+c.apply(this,arguments)||0)/2)&&(d=i===kl?Math.sqrt(n*n+l*l):+i.apply(this,arguments),g||(A*=-1),l&&(A=tt(d/l*Math.sin(m))),n&&(E=tt(d/n*Math.sin(m)))),l){y=l*Math.cos(s+A),M=l*Math.sin(s+A),x=l*Math.cos(f-A),b=l*Math.sin(f-A);var C=Math.abs(f-s-2*A)<=qa?0:1;if(A&&so(y,M,x,b)===g^C){var z=(s+f)/2;y=l*Math.cos(z),M=l*Math.sin(z),x=b=null}}else y=M=0;if(n){_=n*Math.cos(f-E),w=n*Math.sin(f-E),S=n*Math.cos(s+E),k=n*Math.sin(s+E);var q=Math.abs(s-f+2*E)<=qa?0:1;if(E&&so(_,w,S,k)===1-g^q){var L=(s+f)/2;_=n*Math.cos(L),w=n*Math.sin(L),S=k=null}}else _=w=0;if((p=Math.min(Math.abs(l-n)/2,+u.apply(this,arguments)))>.001){v=l>n^g?0:1;var T=null==S?[_,w]:null==x?[y,M]:Lr([y,M],[S,k],[x,b],[_,w]),R=y-T[0],D=M-T[1],P=x-T[0],U=b-T[1],j=1/Math.sin(Math.acos((R*P+D*U)/(Math.sqrt(R*R+D*D)*Math.sqrt(P*P+U*U)))/2),F=Math.sqrt(T[0]*T[0]+T[1]*T[1]);if(null!=x){var H=Math.min(p,(l-F)/(j+1)),O=fo(null==S?[_,w]:[S,k],[y,M],l,H,g),I=fo([x,b],[_,w],l,H,g);p===H?N.push("M",O[0],"A",H,",",H," 0 0,",v," ",O[1],"A",l,",",l," 0 ",1-g^so(O[1][0],O[1][1],I[1][0],I[1][1]),",",g," ",I[1],"A",H,",",H," 0 0,",v," ",I[0]):N.push("M",O[0],"A",H,",",H," 0 1,",v," ",I[0])}else N.push("M",y,",",M);if(null!=S){var Y=Math.min(p,(n-F)/(j-1)),Z=fo([y,M],[S,k],n,-Y,g),V=fo([_,w],null==x?[y,M]:[x,b],n,-Y,g);p===Y?N.push("L",V[0],"A",Y,",",Y," 0 0,",v," ",V[1],"A",n,",",n," 0 ",g^so(V[1][0],V[1][1],Z[1][0],Z[1][1]),",",1-g," ",Z[1],"A",Y,",",Y," 0 0,",v," ",Z[0]):N.push("L",V[0],"A",Y,",",Y," 0 0,",v," ",Z[0])}else N.push("L",_,",",w)}else N.push("M",y,",",M),null!=x&&N.push("A",l,",",l," 0 ",C,",",g," ",x,",",b),N.push("L",_,",",w),null!=S&&N.push("A",n,",",n," 0 ",q,",",1-g," ",S,",",k);return N.push("Z"),N.join("")}function t(n,t){return"M0,"+n+"A"+n+","+n+" 0 1,"+t+" 0,"+-n+"A"+n+","+n+" 0 1,"+t+" 0,"+n}var e=io,r=oo,u=uo,i=kl,o=ao,a=co,c=lo;return n.innerRadius=function(t){return arguments.length?(e=Et(t),n):e},n.outerRadius=function(t){return arguments.length?(r=Et(t),n):r},n.cornerRadius=function(t){return arguments.length?(u=Et(t),n):u},n.padRadius=function(t){return arguments.length?(i=t==kl?kl:Et(t),n):i},n.startAngle=function(t){return arguments.length?(o=Et(t),n):o},n.endAngle=function(t){return arguments.length?(a=Et(t),n):a},n.padAngle=function(t){return arguments.length?(c=Et(t),n):c},n.centroid=function(){var n=(+e.apply(this,arguments)+ +r.apply(this,arguments))/2,t=(+o.apply(this,arguments)+ +a.apply(this,arguments))/2-Ra;return[Math.cos(t)*n,Math.sin(t)*n]},n};var kl="auto";ta.svg.line=function(){return ho(y)};var El=ta.map({linear:go,"linear-closed":po,step:vo,"step-before":mo,"step-after":yo,basis:So,"basis-open":ko,"basis-closed":Eo,bundle:Ao,cardinal:bo,"cardinal-open":Mo,"cardinal-closed":xo,monotone:To});El.forEach(function(n,t){t.key=n,t.closed=/-closed$/.test(n)});var Al=[0,2/3,1/3,0],Nl=[0,1/3,2/3,0],Cl=[0,1/6,2/3,1/6];ta.svg.line.radial=function(){var n=ho(Ro);return n.radius=n.x,delete n.x,n.angle=n.y,delete n.y,n},mo.reverse=yo,yo.reverse=mo,ta.svg.area=function(){return Do(y)},ta.svg.area.radial=function(){var n=Do(Ro);return n.radius=n.x,delete n.x,n.innerRadius=n.x0,delete n.x0,n.outerRadius=n.x1,delete n.x1,n.angle=n.y,delete n.y,n.startAngle=n.y0,delete n.y0,n.endAngle=n.y1,delete n.y1,n},ta.svg.chord=function(){function n(n,a){var c=t(this,i,n,a),l=t(this,o,n,a);return"M"+c.p0+r(c.r,c.p1,c.a1-c.a0)+(e(c,l)?u(c.r,c.p1,c.r,c.p0):u(c.r,c.p1,l.r,l.p0)+r(l.r,l.p1,l.a1-l.a0)+u(l.r,l.p1,c.r,c.p0))+"Z"}function t(n,t,e,r){var u=t.call(n,e,r),i=a.call(n,u,r),o=c.call(n,u,r)-Ra,s=l.call(n,u,r)-Ra;return{r:i,a0:o,a1:s,p0:[i*Math.cos(o),i*Math.sin(o)],p1:[i*Math.cos(s),i*Math.sin(s)]}}function e(n,t){return n.a0==t.a0&&n.a1==t.a1}function r(n,t,e){return"A"+n+","+n+" 0 "+ +(e>qa)+",1 "+t}function u(n,t,e,r){return"Q 0,0 "+r}var i=mr,o=yr,a=Po,c=ao,l=co;return n.radius=function(t){return arguments.length?(a=Et(t),n):a},n.source=function(t){return arguments.length?(i=Et(t),n):i},n.target=function(t){return arguments.length?(o=Et(t),n):o},n.startAngle=function(t){return arguments.length?(c=Et(t),n):c},n.endAngle=function(t){return arguments.length?(l=Et(t),n):l},n},ta.svg.diagonal=function(){function n(n,u){var i=t.call(this,n,u),o=e.call(this,n,u),a=(i.y+o.y)/2,c=[i,{x:i.x,y:a},{x:o.x,y:a},o];return c=c.map(r),"M"+c[0]+"C"+c[1]+" "+c[2]+" "+c[3]}var t=mr,e=yr,r=Uo;return n.source=function(e){return arguments.length?(t=Et(e),n):t},n.target=function(t){return arguments.length?(e=Et(t),n):e},n.projection=function(t){return arguments.length?(r=t,n):r},n},ta.svg.diagonal.radial=function(){var n=ta.svg.diagonal(),t=Uo,e=n.projection;return n.projection=function(n){return arguments.length?e(jo(t=n)):t},n},ta.svg.symbol=function(){function n(n,r){return(zl.get(t.call(this,n,r))||Oo)(e.call(this,n,r))}var t=Ho,e=Fo;return n.type=function(e){return arguments.length?(t=Et(e),n):t},n.size=function(t){return arguments.length?(e=Et(t),n):e},n};var zl=ta.map({circle:Oo,cross:function(n){var t=Math.sqrt(n/5)/2;return"M"+-3*t+","+-t+"H"+-t+"V"+-3*t+"H"+t+"V"+-t+"H"+3*t+"V"+t+"H"+t+"V"+3*t+"H"+-t+"V"+t+"H"+-3*t+"Z"},diamond:function(n){var t=Math.sqrt(n/(2*Ll)),e=t*Ll;return"M0,"+-t+"L"+e+",0 0,"+t+" "+-e+",0Z"},square:function(n){var t=Math.sqrt(n)/2;return"M"+-t+","+-t+"L"+t+","+-t+" "+t+","+t+" "+-t+","+t+"Z"},"triangle-down":function(n){var t=Math.sqrt(n/ql),e=t*ql/2;return"M0,"+e+"L"+t+","+-e+" "+-t+","+-e+"Z"},"triangle-up":function(n){var t=Math.sqrt(n/ql),e=t*ql/2;return"M0,"+-e+"L"+t+","+e+" "+-t+","+e+"Z"}});ta.svg.symbolTypes=zl.keys();var ql=Math.sqrt(3),Ll=Math.tan(30*Da);_a.transition=function(n){for(var t,e,r=Tl||++Ul,u=Xo(n),i=[],o=Rl||{time:Date.now(),ease:Su,delay:0,duration:250},a=-1,c=this.length;++a<c;){i.push(t=[]);for(var l=this[a],s=-1,f=l.length;++s<f;)(e=l[s])&&$o(e,s,u,r,o),t.push(e)}return Yo(i,u,r)},_a.interrupt=function(n){return this.each(null==n?Dl:Io(Xo(n)))};var Tl,Rl,Dl=Io(Xo()),Pl=[],Ul=0;Pl.call=_a.call,Pl.empty=_a.empty,Pl.node=_a.node,Pl.size=_a.size,ta.transition=function(n,t){return n&&n.transition?Tl?n.transition(t):n:ta.selection().transition(n)},ta.transition.prototype=Pl,Pl.select=function(n){var t,e,r,u=this.id,i=this.namespace,o=[];n=N(n);for(var a=-1,c=this.length;++a<c;){o.push(t=[]);for(var l=this[a],s=-1,f=l.length;++s<f;)(r=l[s])&&(e=n.call(r,r.__data__,s,a))?("__data__"in r&&(e.__data__=r.__data__),$o(e,s,i,u,r[i][u]),t.push(e)):t.push(null)}return Yo(o,i,u)},Pl.selectAll=function(n){var t,e,r,u,i,o=this.id,a=this.namespace,c=[];n=C(n);for(var l=-1,s=this.length;++l<s;)for(var f=this[l],h=-1,g=f.length;++h<g;)if(r=f[h]){i=r[a][o],e=n.call(r,r.__data__,h,l),c.push(t=[]);for(var p=-1,v=e.length;++p<v;)(u=e[p])&&$o(u,p,a,o,i),t.push(u)}return Yo(c,a,o)},Pl.filter=function(n){var t,e,r,u=[];"function"!=typeof n&&(n=O(n));for(var i=0,o=this.length;o>i;i++){u.push(t=[]);for(var e=this[i],a=0,c=e.length;c>a;a++)(r=e[a])&&n.call(r,r.__data__,a,i)&&t.push(r)}return Yo(u,this.namespace,this.id)},Pl.tween=function(n,t){var e=this.id,r=this.namespace;return arguments.length<2?this.node()[r][e].tween.get(n):Y(this,null==t?function(t){t[r][e].tween.remove(n)}:function(u){u[r][e].tween.set(n,t)})},Pl.attr=function(n,t){function e(){this.removeAttribute(a)}function r(){this.removeAttributeNS(a.space,a.local)}function u(n){return null==n?e:(n+="",function(){var t,e=this.getAttribute(a);return e!==n&&(t=o(e,n),function(n){this.setAttribute(a,t(n))})})}function i(n){return null==n?r:(n+="",function(){var t,e=this.getAttributeNS(a.space,a.local);return e!==n&&(t=o(e,n),function(n){this.setAttributeNS(a.space,a.local,t(n))})})}if(arguments.length<2){for(t in n)this.attr(t,n[t]);return this}var o="transform"==n?Hu:mu,a=ta.ns.qualify(n);return Zo(this,"attr."+n,t,a.local?i:u)},Pl.attrTween=function(n,t){function e(n,e){var r=t.call(this,n,e,this.getAttribute(u));return r&&function(n){this.setAttribute(u,r(n))}}function r(n,e){var r=t.call(this,n,e,this.getAttributeNS(u.space,u.local));return r&&function(n){this.setAttributeNS(u.space,u.local,r(n))}}var u=ta.ns.qualify(n);return this.tween("attr."+n,u.local?r:e)},Pl.style=function(n,e,r){function u(){this.style.removeProperty(n)}function i(e){return null==e?u:(e+="",function(){var u,i=t(this).getComputedStyle(this,null).getPropertyValue(n);return i!==e&&(u=mu(i,e),function(t){this.style.setProperty(n,u(t),r)})})}var o=arguments.length;if(3>o){if("string"!=typeof n){2>o&&(e="");for(r in n)this.style(r,n[r],e);return this}r=""}return Zo(this,"style."+n,e,i)},Pl.styleTween=function(n,e,r){function u(u,i){var o=e.call(this,u,i,t(this).getComputedStyle(this,null).getPropertyValue(n));return o&&function(t){this.style.setProperty(n,o(t),r)}}return arguments.length<3&&(r=""),this.tween("style."+n,u)},Pl.text=function(n){return Zo(this,"text",n,Vo)},Pl.remove=function(){var n=this.namespace;return this.each("end.transition",function(){var t;this[n].count<2&&(t=this.parentNode)&&t.removeChild(this)})},Pl.ease=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].ease:("function"!=typeof n&&(n=ta.ease.apply(ta,arguments)),Y(this,function(r){r[e][t].ease=n}))},Pl.delay=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].delay:Y(this,"function"==typeof n?function(r,u,i){r[e][t].delay=+n.call(r,r.__data__,u,i)}:(n=+n,function(r){r[e][t].delay=n}))},Pl.duration=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].duration:Y(this,"function"==typeof n?function(r,u,i){r[e][t].duration=Math.max(1,n.call(r,r.__data__,u,i))}:(n=Math.max(1,n),function(r){r[e][t].duration=n}))},Pl.each=function(n,t){var e=this.id,r=this.namespace;if(arguments.length<2){var u=Rl,i=Tl;try{Tl=e,Y(this,function(t,u,i){Rl=t[r][e],n.call(t,t.__data__,u,i)})}finally{Rl=u,Tl=i}}else Y(this,function(u){var i=u[r][e];(i.event||(i.event=ta.dispatch("start","end","interrupt"))).on(n,t)});return this},Pl.transition=function(){for(var n,t,e,r,u=this.id,i=++Ul,o=this.namespace,a=[],c=0,l=this.length;l>c;c++){a.push(n=[]);for(var t=this[c],s=0,f=t.length;f>s;s++)(e=t[s])&&(r=e[o][u],$o(e,s,o,i,{time:r.time,ease:r.ease,delay:r.delay+r.duration,duration:r.duration})),n.push(e)}return Yo(a,o,i)},ta.svg.axis=function(){function n(n){n.each(function(){var n,l=ta.select(this),s=this.__chart__||e,f=this.__chart__=e.copy(),h=null==c?f.ticks?f.ticks.apply(f,a):f.domain():c,g=null==t?f.tickFormat?f.tickFormat.apply(f,a):y:t,p=l.selectAll(".tick").data(h,f),v=p.enter().insert("g",".domain").attr("class","tick").style("opacity",Ca),d=ta.transition(p.exit()).style("opacity",Ca).remove(),m=ta.transition(p.order()).style("opacity",1),M=Math.max(u,0)+o,x=Ui(f),b=l.selectAll(".domain").data([0]),_=(b.enter().append("path").attr("class","domain"),ta.transition(b));v.append("line"),v.append("text");var w,S,k,E,A=v.select("line"),N=m.select("line"),C=p.select("text").text(g),z=v.select("text"),q=m.select("text"),L="top"===r||"left"===r?-1:1;if("bottom"===r||"top"===r?(n=Bo,w="x",k="y",S="x2",E="y2",C.attr("dy",0>L?"0em":".71em").style("text-anchor","middle"),_.attr("d","M"+x[0]+","+L*i+"V0H"+x[1]+"V"+L*i)):(n=Wo,w="y",k="x",S="y2",E="x2",C.attr("dy",".32em").style("text-anchor",0>L?"end":"start"),_.attr("d","M"+L*i+","+x[0]+"H0V"+x[1]+"H"+L*i)),A.attr(E,L*u),z.attr(k,L*M),N.attr(S,0).attr(E,L*u),q.attr(w,0).attr(k,L*M),f.rangeBand){var T=f,R=T.rangeBand()/2;s=f=function(n){return T(n)+R}}else s.rangeBand?s=f:d.call(n,f,s);v.call(n,s,f),m.call(n,f,f)})}var t,e=ta.scale.linear(),r=jl,u=6,i=6,o=3,a=[10],c=null;return n.scale=function(t){return arguments.length?(e=t,n):e},n.orient=function(t){return arguments.length?(r=t in Fl?t+"":jl,n):r},n.ticks=function(){return arguments.length?(a=arguments,n):a},n.tickValues=function(t){return arguments.length?(c=t,n):c},n.tickFormat=function(e){return arguments.length?(t=e,n):t},n.tickSize=function(t){var e=arguments.length;return e?(u=+t,i=+arguments[e-1],n):u},n.innerTickSize=function(t){return arguments.length?(u=+t,n):u},n.outerTickSize=function(t){return arguments.length?(i=+t,n):i},n.tickPadding=function(t){return arguments.length?(o=+t,n):o},n.tickSubdivide=function(){return arguments.length&&n},n};var jl="bottom",Fl={top:1,right:1,bottom:1,left:1};ta.svg.brush=function(){function n(t){t.each(function(){var t=ta.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",i).on("touchstart.brush",i),o=t.selectAll(".background").data([0]);o.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),t.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var a=t.selectAll(".resize").data(v,y);a.exit().remove(),a.enter().append("g").attr("class",function(n){return"resize "+n}).style("cursor",function(n){return Hl[n]}).append("rect").attr("x",function(n){return/[ew]$/.test(n)?-3:null}).attr("y",function(n){return/^[ns]/.test(n)?-3:null}).attr("width",6).attr("height",6).style("visibility","hidden"),a.style("display",n.empty()?"none":null);var c,f=ta.transition(t),h=ta.transition(o);l&&(c=Ui(l),h.attr("x",c[0]).attr("width",c[1]-c[0]),r(f)),s&&(c=Ui(s),h.attr("y",c[0]).attr("height",c[1]-c[0]),u(f)),e(f)})}function e(n){n.selectAll(".resize").attr("transform",function(n){return"translate("+f[+/e$/.test(n)]+","+h[+/^s/.test(n)]+")"})}function r(n){n.select(".extent").attr("x",f[0]),n.selectAll(".extent,.n>rect,.s>rect").attr("width",f[1]-f[0])}function u(n){n.select(".extent").attr("y",h[0]),n.selectAll(".extent,.e>rect,.w>rect").attr("height",h[1]-h[0])}function i(){function i(){32==ta.event.keyCode&&(C||(M=null,q[0]-=f[1],q[1]-=h[1],C=2),S())}function v(){32==ta.event.keyCode&&2==C&&(q[0]+=f[1],q[1]+=h[1],C=0,S())}function d(){var n=ta.mouse(b),t=!1;x&&(n[0]+=x[0],n[1]+=x[1]),C||(ta.event.altKey?(M||(M=[(f[0]+f[1])/2,(h[0]+h[1])/2]),q[0]=f[+(n[0]<M[0])],q[1]=h[+(n[1]<M[1])]):M=null),A&&m(n,l,0)&&(r(k),t=!0),N&&m(n,s,1)&&(u(k),t=!0),t&&(e(k),w({type:"brush",mode:C?"move":"resize"}))}function m(n,t,e){var r,u,i=Ui(t),c=i[0],l=i[1],s=q[e],v=e?h:f,d=v[1]-v[0];return C&&(c-=s,l-=d+s),r=(e?p:g)?Math.max(c,Math.min(l,n[e])):n[e],C?u=(r+=s)+d:(M&&(s=Math.max(c,Math.min(l,2*M[e]-r))),r>s?(u=r,r=s):u=s),v[0]!=r||v[1]!=u?(e?a=null:o=null,v[0]=r,v[1]=u,!0):void 0}function y(){d(),k.style("pointer-events","all").selectAll(".resize").style("display",n.empty()?"none":null),ta.select("body").style("cursor",null),L.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),z(),w({type:"brushend"})}var M,x,b=this,_=ta.select(ta.event.target),w=c.of(b,arguments),k=ta.select(b),E=_.datum(),A=!/^(n|s)$/.test(E)&&l,N=!/^(e|w)$/.test(E)&&s,C=_.classed("extent"),z=W(b),q=ta.mouse(b),L=ta.select(t(b)).on("keydown.brush",i).on("keyup.brush",v);if(ta.event.changedTouches?L.on("touchmove.brush",d).on("touchend.brush",y):L.on("mousemove.brush",d).on("mouseup.brush",y),k.interrupt().selectAll("*").interrupt(),C)q[0]=f[0]-q[0],q[1]=h[0]-q[1];else if(E){var T=+/w$/.test(E),R=+/^n/.test(E);x=[f[1-T]-q[0],h[1-R]-q[1]],q[0]=f[T],q[1]=h[R]}else ta.event.altKey&&(M=q.slice());k.style("pointer-events","none").selectAll(".resize").style("display",null),ta.select("body").style("cursor",_.style("cursor")),w({type:"brushstart"}),d()}var o,a,c=E(n,"brushstart","brush","brushend"),l=null,s=null,f=[0,0],h=[0,0],g=!0,p=!0,v=Ol[0];return n.event=function(n){n.each(function(){var n=c.of(this,arguments),t={x:f,y:h,i:o,j:a},e=this.__chart__||t;this.__chart__=t,Tl?ta.select(this).transition().each("start.brush",function(){o=e.i,a=e.j,f=e.x,h=e.y,n({type:"brushstart"})}).tween("brush:brush",function(){var e=yu(f,t.x),r=yu(h,t.y);return o=a=null,function(u){f=t.x=e(u),h=t.y=r(u),n({type:"brush",mode:"resize"})}}).each("end.brush",function(){o=t.i,a=t.j,n({type:"brush",mode:"resize"}),n({type:"brushend"})}):(n({type:"brushstart"}),n({type:"brush",mode:"resize"}),n({type:"brushend"}))})},n.x=function(t){return arguments.length?(l=t,v=Ol[!l<<1|!s],n):l},n.y=function(t){return arguments.length?(s=t,v=Ol[!l<<1|!s],n):s},n.clamp=function(t){return arguments.length?(l&&s?(g=!!t[0],p=!!t[1]):l?g=!!t:s&&(p=!!t),n):l&&s?[g,p]:l?g:s?p:null},n.extent=function(t){var e,r,u,i,c;return arguments.length?(l&&(e=t[0],r=t[1],s&&(e=e[0],r=r[0]),o=[e,r],l.invert&&(e=l(e),r=l(r)),e>r&&(c=e,e=r,r=c),(e!=f[0]||r!=f[1])&&(f=[e,r])),s&&(u=t[0],i=t[1],l&&(u=u[1],i=i[1]),a=[u,i],s.invert&&(u=s(u),i=s(i)),u>i&&(c=u,u=i,i=c),(u!=h[0]||i!=h[1])&&(h=[u,i])),n):(l&&(o?(e=o[0],r=o[1]):(e=f[0],r=f[1],l.invert&&(e=l.invert(e),r=l.invert(r)),e>r&&(c=e,e=r,r=c))),s&&(a?(u=a[0],i=a[1]):(u=h[0],i=h[1],s.invert&&(u=s.invert(u),i=s.invert(i)),u>i&&(c=u,u=i,i=c))),l&&s?[[e,u],[r,i]]:l?[e,r]:s&&[u,i])},n.clear=function(){return n.empty()||(f=[0,0],h=[0,0],o=a=null),n},n.empty=function(){return!!l&&f[0]==f[1]||!!s&&h[0]==h[1]},ta.rebind(n,c,"on")};var Hl={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},Ol=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],Il=ac.format=gc.timeFormat,Yl=Il.utc,Zl=Yl("%Y-%m-%dT%H:%M:%S.%LZ");Il.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?Jo:Zl,Jo.parse=function(n){var t=new Date(n);return isNaN(t)?null:t},Jo.toString=Zl.toString,ac.second=Ft(function(n){return new cc(1e3*Math.floor(n/1e3))},function(n,t){n.setTime(n.getTime()+1e3*Math.floor(t))},function(n){return n.getSeconds()}),ac.seconds=ac.second.range,ac.seconds.utc=ac.second.utc.range,ac.minute=Ft(function(n){return new cc(6e4*Math.floor(n/6e4))},function(n,t){n.setTime(n.getTime()+6e4*Math.floor(t))},function(n){return n.getMinutes()}),ac.minutes=ac.minute.range,ac.minutes.utc=ac.minute.utc.range,ac.hour=Ft(function(n){var t=n.getTimezoneOffset()/60;return new cc(36e5*(Math.floor(n/36e5-t)+t))},function(n,t){n.setTime(n.getTime()+36e5*Math.floor(t))},function(n){return n.getHours()}),ac.hours=ac.hour.range,ac.hours.utc=ac.hour.utc.range,ac.month=Ft(function(n){return n=ac.day(n),n.setDate(1),n},function(n,t){n.setMonth(n.getMonth()+t)},function(n){return n.getMonth()}),ac.months=ac.month.range,ac.months.utc=ac.month.utc.range;var Vl=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Xl=[[ac.second,1],[ac.second,5],[ac.second,15],[ac.second,30],[ac.minute,1],[ac.minute,5],[ac.minute,15],[ac.minute,30],[ac.hour,1],[ac.hour,3],[ac.hour,6],[ac.hour,12],[ac.day,1],[ac.day,2],[ac.week,1],[ac.month,1],[ac.month,3],[ac.year,1]],$l=Il.multi([[".%L",function(n){return n.getMilliseconds()}],[":%S",function(n){return n.getSeconds()}],["%I:%M",function(n){return n.getMinutes()}],["%I %p",function(n){return n.getHours()}],["%a %d",function(n){return n.getDay()&&1!=n.getDate()}],["%b %d",function(n){return 1!=n.getDate()}],["%B",function(n){return n.getMonth()}],["%Y",Ne]]),Bl={range:function(n,t,e){return ta.range(Math.ceil(n/e)*e,+t,e).map(Ko)},floor:y,ceil:y};Xl.year=ac.year,ac.scale=function(){return Go(ta.scale.linear(),Xl,$l)};var Wl=Xl.map(function(n){return[n[0].utc,n[1]]}),Jl=Yl.multi([[".%L",function(n){return n.getUTCMilliseconds()}],[":%S",function(n){return n.getUTCSeconds()}],["%I:%M",function(n){return n.getUTCMinutes()}],["%I %p",function(n){return n.getUTCHours()}],["%a %d",function(n){return n.getUTCDay()&&1!=n.getUTCDate()}],["%b %d",function(n){return 1!=n.getUTCDate()}],["%B",function(n){return n.getUTCMonth()}],["%Y",Ne]]);Wl.year=ac.year.utc,ac.scale.utc=function(){return Go(ta.scale.linear(),Wl,Jl)},ta.text=At(function(n){return n.responseText}),ta.json=function(n,t){return Nt(n,"application/json",Qo,t)},ta.html=function(n,t){return Nt(n,"text/html",na,t)},ta.xml=At(function(n){return n.responseXML}),"function"==typeof define&&define.amd?define(ta):"object"==typeof module&&module.exports&&(module.exports=ta),this.d3=ta}();
!function(a){"use strict";function b(a){this.owner=a}function c(a,b){if(Object.create)b.prototype=Object.create(a.prototype);else{var c=function(){};c.prototype=a.prototype,b.prototype=new c}return b.prototype.constructor=b,b}function d(a){var b=this.internal=new e(this);b.loadConfig(a),b.beforeInit(a),b.init(),b.afterInit(a),function c(a,b,d){Object.keys(a).forEach(function(e){b[e]=a[e].bind(d),Object.keys(a[e]).length>0&&c(a[e],b[e],d)})}(h,this,this)}function e(b){var c=this;c.d3=a.d3?a.d3:"undefined"!=typeof require?require("d3"):void 0,c.api=b,c.config=c.getDefaultConfig(),c.data={},c.cache={},c.axes={}}function f(a){b.call(this,a)}function g(a,b){function c(a,b){a.attr("transform",function(a){return"translate("+Math.ceil(b(a)+u)+", 0)"})}function d(a,b){a.attr("transform",function(a){return"translate(0,"+Math.ceil(b(a))+")"})}function e(a){var b=a[0],c=a[a.length-1];return c>b?[b,c]:[c,b]}function f(a){var b,c,d=[];if(a.ticks)return a.ticks.apply(a,n);for(c=a.domain(),b=Math.ceil(c[0]);b<c[1];b++)d.push(b);return d.length>0&&d[0]>0&&d.unshift(d[0]-(d[1]-d[0])),d}function g(){var a,c=p.copy();return b.isCategory&&(a=p.domain(),c.domain([a[0],a[1]-1])),c}function h(a){var b=m?m(a):a;return"undefined"!=typeof b?b:""}function i(a){if(A)return A;var b={h:11.5,w:5.5};return a.select("text").text(h).each(function(a){var c=this.getBoundingClientRect(),d=h(a),e=c.height,f=d?c.width/d.length:void 0;e&&f&&(b.h=e,b.w=f)}).text(""),A=b,b}function j(c){return b.withoutTransition?c:a.transition(c)}function k(m){m.each(function(){function m(a,c){function d(a,b){f=void 0;for(var h=1;h<b.length;h++)if(" "===b.charAt(h)&&(f=h),e=b.substr(0,h+1),g=U.w*e.length,g>c)return d(a.concat(b.substr(0,f?f:h)),b.slice(f?f+1:h));return a.concat(b)}var e,f,g,i=h(a),j=[];return"[object Array]"===Object.prototype.toString.call(i)?i:((!c||0>=c)&&(c=X?95:b.isCategory?Math.ceil(F(G[1])-F(G[0]))-12:110),d(j,i+""))}function n(a,b){var c=U.h;return 0===b&&(c="left"===q||"right"===q?-((V[a.index]-1)*(U.h/2)-3):".71em"),c}function v(a){var b=p(a)+(o?0:u);return L[0]<b&&b<L[1]?r:0}function w(a){return a?a>0?"start":"end":"middle"}function x(a){return a?"rotate("+a+")":""}function y(a){return a?8*Math.sin(Math.PI*(a/180)):0}function z(a){return a?11.5-2.5*(a/15)*(a>0?1:-1):W}var A,B,C,D=k.g=a.select(this),E=this.__chart__||p,F=this.__chart__=g(),G=t?t:f(F),H=D.selectAll(".tick").data(G,F),I=H.enter().insert("g",".domain").attr("class","tick").style("opacity",1e-6),J=H.exit().remove(),K=j(H).style("opacity",1),L=p.rangeExtent?p.rangeExtent():e(p.range()),M=D.selectAll(".domain").data([0]),N=(M.enter().append("path").attr("class","domain"),j(M));I.append("line"),I.append("text");var O=I.select("line"),P=K.select("line"),Q=I.select("text"),R=K.select("text");b.isCategory?(u=Math.ceil((F(1)-F(0))/2),B=o?0:u,C=o?u:0):u=B=0;var S,T,U=i(D.select(".tick")),V=[],W=Math.max(r,0)+s,X="left"===q||"right"===q;S=H.select("text"),T=S.selectAll("tspan").data(function(a,c){var d=b.tickMultiline?m(a,b.tickWidth):[].concat(h(a));return V[c]=d.length,d.map(function(a){return{index:c,splitted:a}})}),T.enter().append("tspan"),T.exit().remove(),T.text(function(a){return a.splitted});var Y=b.tickTextRotate;switch(q){case"bottom":A=c,O.attr("y2",r),Q.attr("y",W),P.attr("x1",B).attr("x2",B).attr("y2",v),R.attr("x",0).attr("y",z(Y)).style("text-anchor",w(Y)).attr("transform",x(Y)),T.attr("x",0).attr("dy",n).attr("dx",y(Y)),N.attr("d","M"+L[0]+","+l+"V0H"+L[1]+"V"+l);break;case"top":A=c,O.attr("y2",-r),Q.attr("y",-W),P.attr("x2",0).attr("y2",-r),R.attr("x",0).attr("y",-W),S.style("text-anchor","middle"),T.attr("x",0).attr("dy","0em"),N.attr("d","M"+L[0]+","+-l+"V0H"+L[1]+"V"+-l);break;case"left":A=d,O.attr("x2",-r),Q.attr("x",-W),P.attr("x2",-r).attr("y1",C).attr("y2",C),R.attr("x",-W).attr("y",u),S.style("text-anchor","end"),T.attr("x",-W).attr("dy",n),N.attr("d","M"+-l+","+L[0]+"H0V"+L[1]+"H"+-l);break;case"right":A=d,O.attr("x2",r),Q.attr("x",W),P.attr("x2",r).attr("y2",0),R.attr("x",W).attr("y",0),S.style("text-anchor","start"),T.attr("x",W).attr("dy",n),N.attr("d","M"+l+","+L[0]+"H0V"+L[1]+"H"+l)}if(F.rangeBand){var Z=F,$=Z.rangeBand()/2;E=F=function(a){return Z(a)+$}}else E.rangeBand?E=F:J.call(A,F);I.call(A,E),K.call(A,F)})}var l,m,n,o,p=a.scale.linear(),q="bottom",r=6,s=3,t=null,u=0,v=!0;return b=b||{},l=b.withOuterTick?6:0,k.scale=function(a){return arguments.length?(p=a,k):p},k.orient=function(a){return arguments.length?(q=a in{top:1,right:1,bottom:1,left:1}?a+"":"bottom",k):q},k.tickFormat=function(a){return arguments.length?(m=a,k):m},k.tickCentered=function(a){return arguments.length?(o=a,k):o},k.tickOffset=function(){return u},k.tickInterval=function(){var a,c;return b.isCategory?a=2*u:(c=k.g.select("path.domain").node().getTotalLength()-2*l,a=c/k.g.selectAll("line").size()),a===1/0?0:a},k.ticks=function(){return arguments.length?(n=arguments,k):n},k.tickCulling=function(a){return arguments.length?(v=a,k):v},k.tickValues=function(a){if("function"==typeof a)t=function(){return a(p.domain())};else{if(!arguments.length)return t;t=a}return k},k}var h,i,j,k={version:"0.4.11"};k.generate=function(a){return new d(a)},k.chart={fn:d.prototype,internal:{fn:e.prototype,axis:{fn:f.prototype}}},h=k.chart.fn,i=k.chart.internal.fn,j=k.chart.internal.axis.fn,i.beforeInit=function(){},i.afterInit=function(){},i.init=function(){var a=this,b=a.config;if(a.initParams(),b.data_url)a.convertUrlToData(b.data_url,b.data_mimeType,b.data_headers,b.data_keys,a.initWithData);else if(b.data_json)a.initWithData(a.convertJsonToData(b.data_json,b.data_keys));else if(b.data_rows)a.initWithData(a.convertRowsToData(b.data_rows));else{if(!b.data_columns)throw Error("url or json or rows or columns is required.");a.initWithData(a.convertColumnsToData(b.data_columns))}},i.initParams=function(){var a=this,b=a.d3,c=a.config;a.clipId="c3-"+ +new Date+"-clip",a.clipIdForXAxis=a.clipId+"-xaxis",a.clipIdForYAxis=a.clipId+"-yaxis",a.clipIdForGrid=a.clipId+"-grid",a.clipIdForSubchart=a.clipId+"-subchart",a.clipPath=a.getClipPath(a.clipId),a.clipPathForXAxis=a.getClipPath(a.clipIdForXAxis),a.clipPathForYAxis=a.getClipPath(a.clipIdForYAxis),a.clipPathForGrid=a.getClipPath(a.clipIdForGrid),a.clipPathForSubchart=a.getClipPath(a.clipIdForSubchart),a.dragStart=null,a.dragging=!1,a.flowing=!1,a.cancelClick=!1,a.mouseover=!1,a.transiting=!1,a.color=a.generateColor(),a.levelColor=a.generateLevelColor(),a.dataTimeFormat=c.data_xLocaltime?b.time.format:b.time.format.utc,a.axisTimeFormat=c.axis_x_localtime?b.time.format:b.time.format.utc,a.defaultAxisTimeFormat=a.axisTimeFormat.multi([[".%L",function(a){return a.getMilliseconds()}],[":%S",function(a){return a.getSeconds()}],["%I:%M",function(a){return a.getMinutes()}],["%I %p",function(a){return a.getHours()}],["%-m/%-d",function(a){return a.getDay()&&1!==a.getDate()}],["%-m/%-d",function(a){return 1!==a.getDate()}],["%-m/%-d",function(a){return a.getMonth()}],["%Y/%-m/%-d",function(){return!0}]]),a.hiddenTargetIds=[],a.hiddenLegendIds=[],a.focusedTargetIds=[],a.defocusedTargetIds=[],a.xOrient=c.axis_rotated?"left":"bottom",a.yOrient=c.axis_rotated?c.axis_y_inner?"top":"bottom":c.axis_y_inner?"right":"left",a.y2Orient=c.axis_rotated?c.axis_y2_inner?"bottom":"top":c.axis_y2_inner?"left":"right",a.subXOrient=c.axis_rotated?"left":"bottom",a.isLegendRight="right"===c.legend_position,a.isLegendInset="inset"===c.legend_position,a.isLegendTop="top-left"===c.legend_inset_anchor||"top-right"===c.legend_inset_anchor,a.isLegendLeft="top-left"===c.legend_inset_anchor||"bottom-left"===c.legend_inset_anchor,a.legendStep=0,a.legendItemWidth=0,a.legendItemHeight=0,a.currentMaxTickWidths={x:0,y:0,y2:0},a.rotated_padding_left=30,a.rotated_padding_right=c.axis_rotated&&!c.axis_x_show?0:30,a.rotated_padding_top=5,a.withoutFadeIn={},a.intervalForObserveInserted=void 0,a.axes.subx=b.selectAll([])},i.initChartElements=function(){this.initBar&&this.initBar(),this.initLine&&this.initLine(),this.initArc&&this.initArc(),this.initGauge&&this.initGauge(),this.initText&&this.initText()},i.initWithData=function(a){var b,c,d=this,e=d.d3,g=d.config,h=!0;d.axis=new f(d),d.initPie&&d.initPie(),d.initBrush&&d.initBrush(),d.initZoom&&d.initZoom(),g.bindto?"function"==typeof g.bindto.node?d.selectChart=g.bindto:d.selectChart=e.select(g.bindto):d.selectChart=e.selectAll([]),d.selectChart.empty()&&(d.selectChart=e.select(document.createElement("div")).style("opacity",0),d.observeInserted(d.selectChart),h=!1),d.selectChart.html("").classed("c3",!0),d.data.xs={},d.data.targets=d.convertDataToTargets(a),g.data_filter&&(d.data.targets=d.data.targets.filter(g.data_filter)),g.data_hide&&d.addHiddenTargetIds(g.data_hide===!0?d.mapToIds(d.data.targets):g.data_hide),g.legend_hide&&d.addHiddenLegendIds(g.legend_hide===!0?d.mapToIds(d.data.targets):g.legend_hide),d.hasType("gauge")&&(g.legend_show=!1),d.updateSizes(),d.updateScales(),d.x.domain(e.extent(d.getXDomain(d.data.targets))),d.y.domain(d.getYDomain(d.data.targets,"y")),d.y2.domain(d.getYDomain(d.data.targets,"y2")),d.subX.domain(d.x.domain()),d.subY.domain(d.y.domain()),d.subY2.domain(d.y2.domain()),d.orgXDomain=d.x.domain(),d.brush&&d.brush.scale(d.subX),g.zoom_enabled&&d.zoom.scale(d.x),d.svg=d.selectChart.append("svg").style("overflow","hidden").on("mouseenter",function(){return g.onmouseover.call(d)}).on("mouseleave",function(){return g.onmouseout.call(d)}),d.config.svg_classname&&d.svg.attr("class",d.config.svg_classname),b=d.svg.append("defs"),d.clipChart=d.appendClip(b,d.clipId),d.clipXAxis=d.appendClip(b,d.clipIdForXAxis),d.clipYAxis=d.appendClip(b,d.clipIdForYAxis),d.clipGrid=d.appendClip(b,d.clipIdForGrid),d.clipSubchart=d.appendClip(b,d.clipIdForSubchart),d.updateSvgSize(),c=d.main=d.svg.append("g").attr("transform",d.getTranslate("main")),d.initSubchart&&d.initSubchart(),d.initTooltip&&d.initTooltip(),d.initLegend&&d.initLegend(),d.initTitle&&d.initTitle(),c.append("text").attr("class",l.text+" "+l.empty).attr("text-anchor","middle").attr("dominant-baseline","middle"),d.initRegion(),d.initGrid(),c.append("g").attr("clip-path",d.clipPath).attr("class",l.chart),g.grid_lines_front&&d.initGridLines(),d.initEventRect(),d.initChartElements(),c.insert("rect",g.zoom_privileged?null:"g."+l.regions).attr("class",l.zoomRect).attr("width",d.width).attr("height",d.height).style("opacity",0).on("dblclick.zoom",null),g.axis_x_extent&&d.brush.extent(d.getDefaultExtent()),d.axis.init(),d.updateTargets(d.data.targets),h&&(d.updateDimension(),d.config.oninit.call(d),d.redraw({withTransition:!1,withTransform:!0,withUpdateXDomain:!0,withUpdateOrgXDomain:!0,withTransitionForAxis:!1})),d.bindResize(),d.api.element=d.selectChart.node()},i.smoothLines=function(a,b){var c=this;"grid"===b&&a.each(function(){var a=c.d3.select(this),b=a.attr("x1"),d=a.attr("x2"),e=a.attr("y1"),f=a.attr("y2");a.attr({x1:Math.ceil(b),x2:Math.ceil(d),y1:Math.ceil(e),y2:Math.ceil(f)})})},i.updateSizes=function(){var a=this,b=a.config,c=a.legend?a.getLegendHeight():0,d=a.legend?a.getLegendWidth():0,e=a.isLegendRight||a.isLegendInset?0:c,f=a.hasArcType(),g=b.axis_rotated||f?0:a.getHorizontalAxisHeight("x"),h=b.subchart_show&&!f?b.subchart_size_height+g:0;a.currentWidth=a.getCurrentWidth(),a.currentHeight=a.getCurrentHeight(),a.margin=b.axis_rotated?{top:a.getHorizontalAxisHeight("y2")+a.getCurrentPaddingTop(),right:f?0:a.getCurrentPaddingRight(),bottom:a.getHorizontalAxisHeight("y")+e+a.getCurrentPaddingBottom(),left:h+(f?0:a.getCurrentPaddingLeft())}:{top:4+a.getCurrentPaddingTop(),right:f?0:a.getCurrentPaddingRight(),bottom:g+h+e+a.getCurrentPaddingBottom(),left:f?0:a.getCurrentPaddingLeft()},a.margin2=b.axis_rotated?{top:a.margin.top,right:NaN,bottom:20+e,left:a.rotated_padding_left}:{top:a.currentHeight-h-e,right:NaN,bottom:g+e,left:a.margin.left},a.margin3={top:0,right:NaN,bottom:0,left:0},a.updateSizeForLegend&&a.updateSizeForLegend(c,d),a.width=a.currentWidth-a.margin.left-a.margin.right,a.height=a.currentHeight-a.margin.top-a.margin.bottom,a.width<0&&(a.width=0),a.height<0&&(a.height=0),a.width2=b.axis_rotated?a.margin.left-a.rotated_padding_left-a.rotated_padding_right:a.width,a.height2=b.axis_rotated?a.height:a.currentHeight-a.margin2.top-a.margin2.bottom,a.width2<0&&(a.width2=0),a.height2<0&&(a.height2=0),a.arcWidth=a.width-(a.isLegendRight?d+10:0),a.arcHeight=a.height-(a.isLegendRight?0:10),a.hasType("gauge")&&!b.gauge_fullCircle&&(a.arcHeight+=a.height-a.getGaugeLabelHeight()),a.updateRadius&&a.updateRadius(),a.isLegendRight&&f&&(a.margin3.left=a.arcWidth/2+1.1*a.radiusExpanded)},i.updateTargets=function(a){var b=this;b.updateTargetsForText(a),b.updateTargetsForBar(a),b.updateTargetsForLine(a),b.hasArcType()&&b.updateTargetsForArc&&b.updateTargetsForArc(a),b.updateTargetsForSubchart&&b.updateTargetsForSubchart(a),b.showTargets()},i.showTargets=function(){var a=this;a.svg.selectAll("."+l.target).filter(function(b){return a.isTargetToShow(b.id)}).transition().duration(a.config.transition_duration).style("opacity",1)},i.redraw=function(a,b){var c,d,e,f,g,h,i,j,k,m,n,o,p,q,r,s,t,u,v,x,y,z,A,B,C,D,E,F,G,H=this,I=H.main,J=H.d3,K=H.config,L=H.getShapeIndices(H.isAreaType),M=H.getShapeIndices(H.isBarType),N=H.getShapeIndices(H.isLineType),O=H.hasArcType(),P=H.filterTargetsToShow(H.data.targets),Q=H.xv.bind(H);if(a=a||{},c=w(a,"withY",!0),d=w(a,"withSubchart",!0),e=w(a,"withTransition",!0),h=w(a,"withTransform",!1),i=w(a,"withUpdateXDomain",!1),j=w(a,"withUpdateOrgXDomain",!1),k=w(a,"withTrimXDomain",!0),p=w(a,"withUpdateXAxis",i),m=w(a,"withLegend",!1),n=w(a,"withEventRect",!0),o=w(a,"withDimension",!0),f=w(a,"withTransitionForExit",e),g=w(a,"withTransitionForAxis",e),v=e?K.transition_duration:0,x=f?v:0,y=g?v:0,b=b||H.axis.generateTransitions(y),m&&K.legend_show?H.updateLegend(H.mapToIds(H.data.targets),a,b):o&&H.updateDimension(!0),H.isCategorized()&&0===P.length&&H.x.domain([0,H.axes.x.selectAll(".tick").size()]),P.length?(H.updateXDomain(P,i,j,k),K.axis_x_tick_values||(B=H.axis.updateXAxisTickValues(P))):(H.xAxis.tickValues([]),H.subXAxis.tickValues([])),K.zoom_rescale&&!a.flow&&(E=H.x.orgDomain()),H.y.domain(H.getYDomain(P,"y",E)),H.y2.domain(H.getYDomain(P,"y2",E)),!K.axis_y_tick_values&&K.axis_y_tick_count&&H.yAxis.tickValues(H.axis.generateTickValues(H.y.domain(),K.axis_y_tick_count)),!K.axis_y2_tick_values&&K.axis_y2_tick_count&&H.y2Axis.tickValues(H.axis.generateTickValues(H.y2.domain(),K.axis_y2_tick_count)),H.axis.redraw(b,O),H.axis.updateLabels(e),(i||p)&&P.length)if(K.axis_x_tick_culling&&B){for(C=1;C<B.length;C++)if(B.length/C<K.axis_x_tick_culling_max){D=C;break}H.svg.selectAll("."+l.axisX+" .tick text").each(function(a){var b=B.indexOf(a);b>=0&&J.select(this).style("display",b%D?"none":"block")})}else H.svg.selectAll("."+l.axisX+" .tick text").style("display","block");q=H.generateDrawArea?H.generateDrawArea(L,!1):void 0,r=H.generateDrawBar?H.generateDrawBar(M):void 0,s=H.generateDrawLine?H.generateDrawLine(N,!1):void 0,t=H.generateXYForText(L,M,N,!0),u=H.generateXYForText(L,M,N,!1),c&&(H.subY.domain(H.getYDomain(P,"y")),H.subY2.domain(H.getYDomain(P,"y2"))),H.updateXgridFocus(),I.select("text."+l.text+"."+l.empty).attr("x",H.width/2).attr("y",H.height/2).text(K.data_empty_label_text).transition().style("opacity",P.length?0:1),H.updateGrid(v),H.updateRegion(v),H.updateBar(x),H.updateLine(x),H.updateArea(x),H.updateCircle(),H.hasDataLabel()&&H.updateText(x),H.redrawTitle&&H.redrawTitle(),H.redrawArc&&H.redrawArc(v,x,h),H.redrawSubchart&&H.redrawSubchart(d,b,v,x,L,M,N),I.selectAll("."+l.selectedCircles).filter(H.isBarType.bind(H)).selectAll("circle").remove(),K.interaction_enabled&&!a.flow&&n&&(H.redrawEventRect(),H.updateZoom&&H.updateZoom()),H.updateCircleY(),F=(H.config.axis_rotated?H.circleY:H.circleX).bind(H),G=(H.config.axis_rotated?H.circleX:H.circleY).bind(H),a.flow&&(A=H.generateFlow({targets:P,flow:a.flow,duration:a.flow.duration,drawBar:r,drawLine:s,drawArea:q,cx:F,cy:G,xv:Q,xForText:t,yForText:u})),(v||A)&&H.isTabVisible()?J.transition().duration(v).each(function(){var b=[];[H.redrawBar(r,!0),H.redrawLine(s,!0),H.redrawArea(q,!0),H.redrawCircle(F,G,!0),H.redrawText(t,u,a.flow,!0),H.redrawRegion(!0),H.redrawGrid(!0)].forEach(function(a){a.forEach(function(a){b.push(a)})}),z=H.generateWait(),b.forEach(function(a){z.add(a)})}).call(z,function(){A&&A(),K.onrendered&&K.onrendered.call(H)}):(H.redrawBar(r),H.redrawLine(s),H.redrawArea(q),H.redrawCircle(F,G),H.redrawText(t,u,a.flow),H.redrawRegion(),H.redrawGrid(),K.onrendered&&K.onrendered.call(H)),H.mapToIds(H.data.targets).forEach(function(a){H.withoutFadeIn[a]=!0})},i.updateAndRedraw=function(a){var b,c=this,d=c.config;a=a||{},a.withTransition=w(a,"withTransition",!0),a.withTransform=w(a,"withTransform",!1),a.withLegend=w(a,"withLegend",!1),a.withUpdateXDomain=!0,a.withUpdateOrgXDomain=!0,a.withTransitionForExit=!1,a.withTransitionForTransform=w(a,"withTransitionForTransform",a.withTransition),c.updateSizes(),a.withLegend&&d.legend_show||(b=c.axis.generateTransitions(a.withTransitionForAxis?d.transition_duration:0),c.updateScales(),c.updateSvgSize(),c.transformAll(a.withTransitionForTransform,b)),c.redraw(a,b)},i.redrawWithoutRescale=function(){this.redraw({withY:!1,withSubchart:!1,withEventRect:!1,withTransitionForAxis:!1})},i.isTimeSeries=function(){return"timeseries"===this.config.axis_x_type},i.isCategorized=function(){return this.config.axis_x_type.indexOf("categor")>=0},i.isCustomX=function(){var a=this,b=a.config;return!a.isTimeSeries()&&(b.data_x||v(b.data_xs))},i.isTimeSeriesY=function(){return"timeseries"===this.config.axis_y_type},i.getTranslate=function(a){var b,c,d=this,e=d.config;return"main"===a?(b=s(d.margin.left),c=s(d.margin.top)):"context"===a?(b=s(d.margin2.left),c=s(d.margin2.top)):"legend"===a?(b=d.margin3.left,c=d.margin3.top):"x"===a?(b=0,c=e.axis_rotated?0:d.height):"y"===a?(b=0,c=e.axis_rotated?d.height:0):"y2"===a?(b=e.axis_rotated?0:d.width,c=e.axis_rotated?1:0):"subx"===a?(b=0,c=e.axis_rotated?0:d.height2):"arc"===a&&(b=d.arcWidth/2,c=d.arcHeight/2),"translate("+b+","+c+")"},i.initialOpacity=function(a){return null!==a.value&&this.withoutFadeIn[a.id]?1:0},i.initialOpacityForCircle=function(a){return null!==a.value&&this.withoutFadeIn[a.id]?this.opacityForCircle(a):0},i.opacityForCircle=function(a){var b=this.config.point_show?1:0;return m(a.value)?this.isScatterType(a)?.5:b:0},i.opacityForText=function(){return this.hasDataLabel()?1:0},i.xx=function(a){return a?this.x(a.x):null},i.xv=function(a){var b=this,c=a.value;return b.isTimeSeries()?c=b.parseDate(a.value):b.isCategorized()&&"string"==typeof a.value&&(c=b.config.axis_x_categories.indexOf(a.value)),Math.ceil(b.x(c))},i.yv=function(a){var b=this,c=a.axis&&"y2"===a.axis?b.y2:b.y;return Math.ceil(c(a.value))},i.subxx=function(a){return a?this.subX(a.x):null},i.transformMain=function(a,b){var c,d,e,f=this;b&&b.axisX?c=b.axisX:(c=f.main.select("."+l.axisX),a&&(c=c.transition())),b&&b.axisY?d=b.axisY:(d=f.main.select("."+l.axisY),a&&(d=d.transition())),b&&b.axisY2?e=b.axisY2:(e=f.main.select("."+l.axisY2),a&&(e=e.transition())),(a?f.main.transition():f.main).attr("transform",f.getTranslate("main")),c.attr("transform",f.getTranslate("x")),d.attr("transform",f.getTranslate("y")),e.attr("transform",f.getTranslate("y2")),f.main.select("."+l.chartArcs).attr("transform",f.getTranslate("arc"))},i.transformAll=function(a,b){var c=this;c.transformMain(a,b),c.config.subchart_show&&c.transformContext(a,b),c.legend&&c.transformLegend(a)},i.updateSvgSize=function(){var a=this,b=a.svg.select(".c3-brush .background");a.svg.attr("width",a.currentWidth).attr("height",a.currentHeight),a.svg.selectAll(["#"+a.clipId,"#"+a.clipIdForGrid]).select("rect").attr("width",a.width).attr("height",a.height),a.svg.select("#"+a.clipIdForXAxis).select("rect").attr("x",a.getXAxisClipX.bind(a)).attr("y",a.getXAxisClipY.bind(a)).attr("width",a.getXAxisClipWidth.bind(a)).attr("height",a.getXAxisClipHeight.bind(a)),a.svg.select("#"+a.clipIdForYAxis).select("rect").attr("x",a.getYAxisClipX.bind(a)).attr("y",a.getYAxisClipY.bind(a)).attr("width",a.getYAxisClipWidth.bind(a)).attr("height",a.getYAxisClipHeight.bind(a)),a.svg.select("#"+a.clipIdForSubchart).select("rect").attr("width",a.width).attr("height",b.size()?b.attr("height"):0),a.svg.select("."+l.zoomRect).attr("width",a.width).attr("height",a.height),a.selectChart.style("max-height",a.currentHeight+"px")},i.updateDimension=function(a){var b=this;a||(b.config.axis_rotated?(b.axes.x.call(b.xAxis),b.axes.subx.call(b.subXAxis)):(b.axes.y.call(b.yAxis),b.axes.y2.call(b.y2Axis))),b.updateSizes(),b.updateScales(),b.updateSvgSize(),b.transformAll(!1)},i.observeInserted=function(b){var c,d=this;return"undefined"==typeof MutationObserver?void a.console.error("MutationObserver not defined."):(c=new MutationObserver(function(e){e.forEach(function(e){"childList"===e.type&&e.previousSibling&&(c.disconnect(),d.intervalForObserveInserted=a.setInterval(function(){b.node().parentNode&&(a.clearInterval(d.intervalForObserveInserted),d.updateDimension(),d.brush&&d.brush.update(),d.config.oninit.call(d),d.redraw({withTransform:!0,withUpdateXDomain:!0,withUpdateOrgXDomain:!0,withTransition:!1,withTransitionForTransform:!1,withLegend:!0}),b.transition().style("opacity",1))},10))})}),void c.observe(b.node(),{attributes:!0,childList:!0,characterData:!0}))},i.bindResize=function(){var b=this,c=b.config;if(b.resizeFunction=b.generateResize(),b.resizeFunction.add(function(){c.onresize.call(b)}),c.resize_auto&&b.resizeFunction.add(function(){void 0!==b.resizeTimeout&&a.clearTimeout(b.resizeTimeout),b.resizeTimeout=a.setTimeout(function(){delete b.resizeTimeout,b.api.flush()},100)}),b.resizeFunction.add(function(){c.onresized.call(b)}),a.attachEvent)a.attachEvent("onresize",b.resizeFunction);else if(a.addEventListener)a.addEventListener("resize",b.resizeFunction,!1);else{var d=a.onresize;d?d.add&&d.remove||(d=b.generateResize(),d.add(a.onresize)):d=b.generateResize(),d.add(b.resizeFunction),a.onresize=d}},i.generateResize=function(){function a(){b.forEach(function(a){a()})}var b=[];return a.add=function(a){b.push(a)},a.remove=function(a){for(var c=0;c<b.length;c++)if(b[c]===a){b.splice(c,1);break}},a},i.endall=function(a,b){var c=0;a.each(function(){++c}).each("end",function(){--c||b.apply(this,arguments)})},i.generateWait=function(){var a=[],b=function(b,c){var d=setInterval(function(){var b=0;a.forEach(function(a){if(a.empty())return void(b+=1);try{a.transition()}catch(c){b+=1}}),b===a.length&&(clearInterval(d),c&&c())},10)};return b.add=function(b){a.push(b)},b},i.parseDate=function(b){var c,d=this;return b instanceof Date?c=b:"string"==typeof b?c=d.dataTimeFormat(d.config.data_xFormat).parse(b):"number"!=typeof b||isNaN(b)||(c=new Date(+b)),c&&!isNaN(+c)||a.console.error("Failed to parse x '"+b+"' to Date object"),c},i.isTabVisible=function(){var a;return"undefined"!=typeof document.hidden?a="hidden":"undefined"!=typeof document.mozHidden?a="mozHidden":"undefined"!=typeof document.msHidden?a="msHidden":"undefined"!=typeof document.webkitHidden&&(a="webkitHidden"),!document[a]},i.getDefaultConfig=function(){var a={bindto:"#chart",svg_classname:void 0,size_width:void 0,size_height:void 0,padding_left:void 0,padding_right:void 0,padding_top:void 0,padding_bottom:void 0,resize_auto:!0,zoom_enabled:!1,zoom_extent:void 0,zoom_privileged:!1,zoom_rescale:!1,zoom_onzoom:function(){},zoom_onzoomstart:function(){},zoom_onzoomend:function(){},zoom_x_min:void 0,zoom_x_max:void 0,interaction_brighten:!0,interaction_enabled:!0,onmouseover:function(){},onmouseout:function(){},onresize:function(){},onresized:function(){},oninit:function(){},onrendered:function(){},transition_duration:350,data_x:void 0,data_xs:{},data_xFormat:"%Y-%m-%d",data_xLocaltime:!0,data_xSort:!0,data_idConverter:function(a){return a},data_names:{},data_classes:{},data_groups:[],data_axes:{},data_type:void 0,data_types:{},data_labels:{},data_order:"desc",data_regions:{},data_color:void 0,data_colors:{},data_hide:!1,data_filter:void 0,data_selection_enabled:!1,data_selection_grouped:!1,data_selection_isselectable:function(){return!0},data_selection_multiple:!0,data_selection_draggable:!1,data_onclick:function(){},data_onmouseover:function(){},data_onmouseout:function(){},data_onselected:function(){},data_onunselected:function(){},data_url:void 0,data_headers:void 0,data_json:void 0,data_rows:void 0,data_columns:void 0,data_mimeType:void 0,data_keys:void 0,data_empty_label_text:"",subchart_show:!1,subchart_size_height:60,subchart_axis_x_show:!0,subchart_onbrush:function(){},color_pattern:[],color_threshold:{},legend_show:!0,legend_hide:!1,legend_position:"bottom",legend_inset_anchor:"top-left",legend_inset_x:10,legend_inset_y:0,legend_inset_step:void 0,legend_item_onclick:void 0,legend_item_onmouseover:void 0,legend_item_onmouseout:void 0,legend_equally:!1,legend_padding:0,legend_item_tile_width:10,legend_item_tile_height:10,axis_rotated:!1,axis_x_show:!0,axis_x_type:"indexed",axis_x_localtime:!0,axis_x_categories:[],axis_x_tick_centered:!1,axis_x_tick_format:void 0,axis_x_tick_culling:{},axis_x_tick_culling_max:10,axis_x_tick_count:void 0,axis_x_tick_fit:!0,axis_x_tick_values:null,axis_x_tick_rotate:0,axis_x_tick_outer:!0,axis_x_tick_multiline:!0,axis_x_tick_width:null,axis_x_max:void 0,axis_x_min:void 0,axis_x_padding:{},axis_x_height:void 0,axis_x_extent:void 0,axis_x_label:{},axis_y_show:!0,axis_y_type:void 0,axis_y_max:void 0,axis_y_min:void 0,axis_y_inverted:!1,axis_y_center:void 0,axis_y_inner:void 0,axis_y_label:{},axis_y_tick_format:void 0,axis_y_tick_outer:!0,axis_y_tick_values:null,axis_y_tick_rotate:0,axis_y_tick_count:void 0,axis_y_tick_time_value:void 0,axis_y_tick_time_interval:void 0,axis_y_padding:{},axis_y_default:void 0,axis_y2_show:!1,axis_y2_max:void 0,axis_y2_min:void 0,axis_y2_inverted:!1,axis_y2_center:void 0,axis_y2_inner:void 0,axis_y2_label:{},axis_y2_tick_format:void 0,axis_y2_tick_outer:!0,axis_y2_tick_values:null,axis_y2_tick_count:void 0,axis_y2_padding:{},axis_y2_default:void 0,grid_x_show:!1,grid_x_type:"tick",grid_x_lines:[],grid_y_show:!1,grid_y_lines:[],grid_y_ticks:10,grid_focus_show:!0,grid_lines_front:!0,point_show:!0,point_r:2.5,point_sensitivity:10,point_focus_expand_enabled:!0,point_focus_expand_r:void 0,point_select_r:void 0,line_connectNull:!1,line_step_type:"step",bar_width:void 0,bar_width_ratio:.6,bar_width_max:void 0,bar_zerobased:!0,area_zerobased:!0,area_above:!1,pie_label_show:!0,pie_label_format:void 0,pie_label_threshold:.05,pie_label_ratio:void 0,pie_expand:{},pie_expand_duration:50,gauge_fullCircle:!1,gauge_label_show:!0,gauge_label_format:void 0,gauge_min:0,gauge_max:100,gauge_startingAngle:-1*Math.PI/2,gauge_units:void 0,gauge_width:void 0,gauge_expand:{},gauge_expand_duration:50,donut_label_show:!0,donut_label_format:void 0,donut_label_threshold:.05,donut_label_ratio:void 0,donut_width:void 0,donut_title:"",donut_expand:{},donut_expand_duration:50,spline_interpolation_type:"cardinal",regions:[],tooltip_show:!0,tooltip_grouped:!0,tooltip_format_title:void 0,tooltip_format_name:void 0,tooltip_format_value:void 0,tooltip_position:void 0,tooltip_contents:function(a,b,c,d){return this.getTooltipContent?this.getTooltipContent(a,b,c,d):""},tooltip_init_show:!1,tooltip_init_x:0,tooltip_init_position:{top:"0px",left:"50px"},tooltip_onshow:function(){},tooltip_onhide:function(){},title_text:void 0,title_padding:{top:0,right:0,bottom:0,left:0},title_position:"top-center"};return Object.keys(this.additionalConfig).forEach(function(b){a[b]=this.additionalConfig[b]},this),a},i.additionalConfig={},i.loadConfig=function(a){function b(){var a=d.shift();return a&&c&&"object"==typeof c&&a in c?(c=c[a],b()):a?void 0:c}var c,d,e,f=this.config;Object.keys(f).forEach(function(g){c=a,d=g.split("_"),e=b(),q(e)&&(f[g]=e)})},i.getScale=function(a,b,c){return(c?this.d3.time.scale():this.d3.scale.linear()).range([a,b])},i.getX=function(a,b,c,d){var e,f=this,g=f.getScale(a,b,f.isTimeSeries()),h=c?g.domain(c):g;f.isCategorized()?(d=d||function(){return 0},g=function(a,b){var c=h(a)+d(a);return b?c:Math.ceil(c)}):g=function(a,b){var c=h(a);return b?c:Math.ceil(c)};for(e in h)g[e]=h[e];return g.orgDomain=function(){return h.domain()},f.isCategorized()&&(g.domain=function(a){return arguments.length?(h.domain(a),g):(a=this.orgDomain(),[a[0],a[1]+1])}),g},i.getY=function(a,b,c){var d=this.getScale(a,b,this.isTimeSeriesY());return c&&d.domain(c),d},i.getYScale=function(a){return"y2"===this.axis.getId(a)?this.y2:this.y},i.getSubYScale=function(a){return"y2"===this.axis.getId(a)?this.subY2:this.subY},i.updateScales=function(){var a=this,b=a.config,c=!a.x;a.xMin=b.axis_rotated?1:0,a.xMax=b.axis_rotated?a.height:a.width,a.yMin=b.axis_rotated?0:a.height,a.yMax=b.axis_rotated?a.width:1,a.subXMin=a.xMin,a.subXMax=a.xMax,a.subYMin=b.axis_rotated?0:a.height2,a.subYMax=b.axis_rotated?a.width2:1,a.x=a.getX(a.xMin,a.xMax,c?void 0:a.x.orgDomain(),function(){return a.xAxis.tickOffset()}),a.y=a.getY(a.yMin,a.yMax,c?b.axis_y_default:a.y.domain()),a.y2=a.getY(a.yMin,a.yMax,c?b.axis_y2_default:a.y2.domain()),a.subX=a.getX(a.xMin,a.xMax,a.orgXDomain,function(b){return b%1?0:a.subXAxis.tickOffset()}),a.subY=a.getY(a.subYMin,a.subYMax,c?b.axis_y_default:a.subY.domain()),a.subY2=a.getY(a.subYMin,a.subYMax,c?b.axis_y2_default:a.subY2.domain()),a.xAxisTickFormat=a.axis.getXAxisTickFormat(),a.xAxisTickValues=a.axis.getXAxisTickValues(),a.yAxisTickValues=a.axis.getYAxisTickValues(),a.y2AxisTickValues=a.axis.getY2AxisTickValues(),a.xAxis=a.axis.getXAxis(a.x,a.xOrient,a.xAxisTickFormat,a.xAxisTickValues,b.axis_x_tick_outer),a.subXAxis=a.axis.getXAxis(a.subX,a.subXOrient,a.xAxisTickFormat,a.xAxisTickValues,b.axis_x_tick_outer),a.yAxis=a.axis.getYAxis(a.y,a.yOrient,b.axis_y_tick_format,a.yAxisTickValues,b.axis_y_tick_outer),a.y2Axis=a.axis.getYAxis(a.y2,a.y2Orient,b.axis_y2_tick_format,a.y2AxisTickValues,b.axis_y2_tick_outer),c||(a.brush&&a.brush.scale(a.subX),b.zoom_enabled&&a.zoom.scale(a.x)),a.updateArc&&a.updateArc()},i.getYDomainMin=function(a){var b,c,d,e,f,g,h=this,i=h.config,j=h.mapToIds(a),k=h.getValuesAsIdKeyed(a);if(i.data_groups.length>0)for(g=h.hasNegativeValueInTargets(a),b=0;b<i.data_groups.length;b++)if(e=i.data_groups[b].filter(function(a){return j.indexOf(a)>=0}),0!==e.length)for(d=e[0],g&&k[d]&&k[d].forEach(function(a,b){k[d][b]=0>a?a:0}),c=1;c<e.length;c++)f=e[c],k[f]&&k[f].forEach(function(a,b){h.axis.getId(f)!==h.axis.getId(d)||!k[d]||g&&+a>0||(k[d][b]+=+a)});return h.d3.min(Object.keys(k).map(function(a){return h.d3.min(k[a])}))},i.getYDomainMax=function(a){var b,c,d,e,f,g,h=this,i=h.config,j=h.mapToIds(a),k=h.getValuesAsIdKeyed(a);if(i.data_groups.length>0)for(g=h.hasPositiveValueInTargets(a),b=0;b<i.data_groups.length;b++)if(e=i.data_groups[b].filter(function(a){return j.indexOf(a)>=0}),0!==e.length)for(d=e[0],g&&k[d]&&k[d].forEach(function(a,b){k[d][b]=a>0?a:0}),c=1;c<e.length;c++)f=e[c],k[f]&&k[f].forEach(function(a,b){h.axis.getId(f)!==h.axis.getId(d)||!k[d]||g&&0>+a||(k[d][b]+=+a)});return h.d3.max(Object.keys(k).map(function(a){return h.d3.max(k[a])}))},i.getYDomain=function(a,b,c){var d,e,f,g,h,i,j,k,l,n,o,p=this,q=p.config,r=a.filter(function(a){return p.axis.getId(a.id)===b}),s=c?p.filterByXDomain(r,c):r,u="y2"===b?q.axis_y2_min:q.axis_y_min,w="y2"===b?q.axis_y2_max:q.axis_y_max,x=p.getYDomainMin(s),y=p.getYDomainMax(s),z="y2"===b?q.axis_y2_center:q.axis_y_center,A=p.hasType("bar",s)&&q.bar_zerobased||p.hasType("area",s)&&q.area_zerobased,B="y2"===b?q.axis_y2_inverted:q.axis_y_inverted,C=p.hasDataLabel()&&q.axis_rotated,D=p.hasDataLabel()&&!q.axis_rotated;return x=m(u)?u:m(w)?w>x?x:w-10:x,y=m(w)?w:m(u)?y>u?y:u+10:y,0===s.length?"y2"===b?p.y2.domain():p.y.domain():(isNaN(x)&&(x=0),isNaN(y)&&(y=x),x===y&&(0>x?y=0:x=0),n=x>=0&&y>=0,o=0>=x&&0>=y,(m(u)&&n||m(w)&&o)&&(A=!1),A&&(n&&(x=0),o&&(y=0)),e=Math.abs(y-x),f=g=h=.1*e,"undefined"!=typeof z&&(i=Math.max(Math.abs(x),Math.abs(y)),y=z+i,x=z-i),C?(j=p.getDataLabelLength(x,y,"width"),k=t(p.y.range()),l=[j[0]/k,j[1]/k],
g+=e*(l[1]/(1-l[0]-l[1])),h+=e*(l[0]/(1-l[0]-l[1]))):D&&(j=p.getDataLabelLength(x,y,"height"),g+=p.axis.convertPixelsToAxisPadding(j[1],e),h+=p.axis.convertPixelsToAxisPadding(j[0],e)),"y"===b&&v(q.axis_y_padding)&&(g=p.axis.getPadding(q.axis_y_padding,"top",g,e),h=p.axis.getPadding(q.axis_y_padding,"bottom",h,e)),"y2"===b&&v(q.axis_y2_padding)&&(g=p.axis.getPadding(q.axis_y2_padding,"top",g,e),h=p.axis.getPadding(q.axis_y2_padding,"bottom",h,e)),A&&(n&&(h=x),o&&(g=-y)),d=[x-h,y+g],B?d.reverse():d)},i.getXDomainMin=function(a){var b=this,c=b.config;return q(c.axis_x_min)?b.isTimeSeries()?this.parseDate(c.axis_x_min):c.axis_x_min:b.d3.min(a,function(a){return b.d3.min(a.values,function(a){return a.x})})},i.getXDomainMax=function(a){var b=this,c=b.config;return q(c.axis_x_max)?b.isTimeSeries()?this.parseDate(c.axis_x_max):c.axis_x_max:b.d3.max(a,function(a){return b.d3.max(a.values,function(a){return a.x})})},i.getXDomainPadding=function(a){var b,c,d,e,f=this,g=f.config,h=a[1]-a[0];return f.isCategorized()?c=0:f.hasType("bar")?(b=f.getMaxDataCount(),c=b>1?h/(b-1)/2:.5):c=.01*h,"object"==typeof g.axis_x_padding&&v(g.axis_x_padding)?(d=m(g.axis_x_padding.left)?g.axis_x_padding.left:c,e=m(g.axis_x_padding.right)?g.axis_x_padding.right:c):d=e="number"==typeof g.axis_x_padding?g.axis_x_padding:c,{left:d,right:e}},i.getXDomain=function(a){var b=this,c=[b.getXDomainMin(a),b.getXDomainMax(a)],d=c[0],e=c[1],f=b.getXDomainPadding(c),g=0,h=0;return d-e!==0||b.isCategorized()||(b.isTimeSeries()?(d=new Date(.5*d.getTime()),e=new Date(1.5*e.getTime())):(d=0===d?1:.5*d,e=0===e?-1:1.5*e)),(d||0===d)&&(g=b.isTimeSeries()?new Date(d.getTime()-f.left):d-f.left),(e||0===e)&&(h=b.isTimeSeries()?new Date(e.getTime()+f.right):e+f.right),[g,h]},i.updateXDomain=function(a,b,c,d,e){var f=this,g=f.config;return c&&(f.x.domain(e?e:f.d3.extent(f.getXDomain(a))),f.orgXDomain=f.x.domain(),g.zoom_enabled&&f.zoom.scale(f.x).updateScaleExtent(),f.subX.domain(f.x.domain()),f.brush&&f.brush.scale(f.subX)),b&&(f.x.domain(e?e:!f.brush||f.brush.empty()?f.orgXDomain:f.brush.extent()),g.zoom_enabled&&f.zoom.scale(f.x).updateScaleExtent()),d&&f.x.domain(f.trimXDomain(f.x.orgDomain())),f.x.domain()},i.trimXDomain=function(a){var b=this.getZoomDomain(),c=b[0],d=b[1];return a[0]<=c&&(a[1]=+a[1]+(c-a[0]),a[0]=c),d<=a[1]&&(a[0]=+a[0]-(a[1]-d),a[1]=d),a},i.isX=function(a){var b=this,c=b.config;return c.data_x&&a===c.data_x||v(c.data_xs)&&x(c.data_xs,a)},i.isNotX=function(a){return!this.isX(a)},i.getXKey=function(a){var b=this,c=b.config;return c.data_x?c.data_x:v(c.data_xs)?c.data_xs[a]:null},i.getXValuesOfXKey=function(a,b){var c,d=this,e=b&&v(b)?d.mapToIds(b):[];return e.forEach(function(b){d.getXKey(b)===a&&(c=d.data.xs[b])}),c},i.getIndexByX=function(a){var b=this,c=b.filterByX(b.data.targets,a);return c.length?c[0].index:null},i.getXValue=function(a,b){var c=this;return a in c.data.xs&&c.data.xs[a]&&m(c.data.xs[a][b])?c.data.xs[a][b]:b},i.getOtherTargetXs=function(){var a=this,b=Object.keys(a.data.xs);return b.length?a.data.xs[b[0]]:null},i.getOtherTargetX=function(a){var b=this.getOtherTargetXs();return b&&a<b.length?b[a]:null},i.addXs=function(a){var b=this;Object.keys(a).forEach(function(c){b.config.data_xs[c]=a[c]})},i.hasMultipleX=function(a){return this.d3.set(Object.keys(a).map(function(b){return a[b]})).size()>1},i.isMultipleX=function(){return v(this.config.data_xs)||!this.config.data_xSort||this.hasType("scatter")},i.addName=function(a){var b,c=this;return a&&(b=c.config.data_names[a.id],a.name=void 0!==b?b:a.id),a},i.getValueOnIndex=function(a,b){var c=a.filter(function(a){return a.index===b});return c.length?c[0]:null},i.updateTargetX=function(a,b){var c=this;a.forEach(function(a){a.values.forEach(function(d,e){d.x=c.generateTargetX(b[e],a.id,e)}),c.data.xs[a.id]=b})},i.updateTargetXs=function(a,b){var c=this;a.forEach(function(a){b[a.id]&&c.updateTargetX([a],b[a.id])})},i.generateTargetX=function(a,b,c){var d,e=this;return d=e.isTimeSeries()?a?e.parseDate(a):e.parseDate(e.getXValue(b,c)):e.isCustomX()&&!e.isCategorized()?m(a)?+a:e.getXValue(b,c):c},i.cloneTarget=function(a){return{id:a.id,id_org:a.id_org,values:a.values.map(function(a){return{x:a.x,value:a.value,id:a.id}})}},i.updateXs=function(){var a=this;a.data.targets.length&&(a.xs=[],a.data.targets[0].values.forEach(function(b){a.xs[b.index]=b.x}))},i.getPrevX=function(a){var b=this.xs[a-1];return"undefined"!=typeof b?b:null},i.getNextX=function(a){var b=this.xs[a+1];return"undefined"!=typeof b?b:null},i.getMaxDataCount=function(){var a=this;return a.d3.max(a.data.targets,function(a){return a.values.length})},i.getMaxDataCountTarget=function(a){var b,c=a.length,d=0;return c>1?a.forEach(function(a){a.values.length>d&&(b=a,d=a.values.length)}):b=c?a[0]:null,b},i.getEdgeX=function(a){var b=this;return a.length?[b.d3.min(a,function(a){return a.values[0].x}),b.d3.max(a,function(a){return a.values[a.values.length-1].x})]:[0,0]},i.mapToIds=function(a){return a.map(function(a){return a.id})},i.mapToTargetIds=function(a){var b=this;return a?[].concat(a):b.mapToIds(b.data.targets)},i.hasTarget=function(a,b){var c,d=this.mapToIds(a);for(c=0;c<d.length;c++)if(d[c]===b)return!0;return!1},i.isTargetToShow=function(a){return this.hiddenTargetIds.indexOf(a)<0},i.isLegendToShow=function(a){return this.hiddenLegendIds.indexOf(a)<0},i.filterTargetsToShow=function(a){var b=this;return a.filter(function(a){return b.isTargetToShow(a.id)})},i.mapTargetsToUniqueXs=function(a){var b=this,c=b.d3.set(b.d3.merge(a.map(function(a){return a.values.map(function(a){return+a.x})}))).values();return c=b.isTimeSeries()?c.map(function(a){return new Date(+a)}):c.map(function(a){return+a}),c.sort(function(a,b){return b>a?-1:a>b?1:a>=b?0:NaN})},i.addHiddenTargetIds=function(a){this.hiddenTargetIds=this.hiddenTargetIds.concat(a)},i.removeHiddenTargetIds=function(a){this.hiddenTargetIds=this.hiddenTargetIds.filter(function(b){return a.indexOf(b)<0})},i.addHiddenLegendIds=function(a){this.hiddenLegendIds=this.hiddenLegendIds.concat(a)},i.removeHiddenLegendIds=function(a){this.hiddenLegendIds=this.hiddenLegendIds.filter(function(b){return a.indexOf(b)<0})},i.getValuesAsIdKeyed=function(a){var b={};return a.forEach(function(a){b[a.id]=[],a.values.forEach(function(c){b[a.id].push(c.value)})}),b},i.checkValueInTargets=function(a,b){var c,d,e,f=Object.keys(a);for(c=0;c<f.length;c++)for(e=a[f[c]].values,d=0;d<e.length;d++)if(b(e[d].value))return!0;return!1},i.hasNegativeValueInTargets=function(a){return this.checkValueInTargets(a,function(a){return 0>a})},i.hasPositiveValueInTargets=function(a){return this.checkValueInTargets(a,function(a){return a>0})},i.isOrderDesc=function(){var a=this.config;return"string"==typeof a.data_order&&"desc"===a.data_order.toLowerCase()},i.isOrderAsc=function(){var a=this.config;return"string"==typeof a.data_order&&"asc"===a.data_order.toLowerCase()},i.orderTargets=function(a){var b=this,c=b.config,d=b.isOrderAsc(),e=b.isOrderDesc();return d||e?a.sort(function(a,b){var c=function(a,b){return a+Math.abs(b.value)},e=a.values.reduce(c,0),f=b.values.reduce(c,0);return d?f-e:e-f}):n(c.data_order)&&a.sort(c.data_order),a},i.filterByX=function(a,b){return this.d3.merge(a.map(function(a){return a.values})).filter(function(a){return a.x-b===0})},i.filterRemoveNull=function(a){return a.filter(function(a){return m(a.value)})},i.filterByXDomain=function(a,b){return a.map(function(a){return{id:a.id,id_org:a.id_org,values:a.values.filter(function(a){return b[0]<=a.x&&a.x<=b[1]})}})},i.hasDataLabel=function(){var a=this.config;return"boolean"==typeof a.data_labels&&a.data_labels?!0:!("object"!=typeof a.data_labels||!v(a.data_labels))},i.getDataLabelLength=function(a,b,c){var d=this,e=[0,0],f=1.3;return d.selectChart.select("svg").selectAll(".dummy").data([a,b]).enter().append("text").text(function(a){return d.dataLabelFormat(a.id)(a)}).each(function(a,b){e[b]=this.getBoundingClientRect()[c]*f}).remove(),e},i.isNoneArc=function(a){return this.hasTarget(this.data.targets,a.id)},i.isArc=function(a){return"data"in a&&this.hasTarget(this.data.targets,a.data.id)},i.findSameXOfValues=function(a,b){var c,d=a[b].x,e=[];for(c=b-1;c>=0&&d===a[c].x;c--)e.push(a[c]);for(c=b;c<a.length&&d===a[c].x;c++)e.push(a[c]);return e},i.findClosestFromTargets=function(a,b){var c,d=this;return c=a.map(function(a){return d.findClosest(a.values,b)}),d.findClosest(c,b)},i.findClosest=function(a,b){var c,d=this,e=d.config.point_sensitivity;return a.filter(function(a){return a&&d.isBarType(a.id)}).forEach(function(a){var b=d.main.select("."+l.bars+d.getTargetSelectorSuffix(a.id)+" ."+l.bar+"-"+a.index).node();!c&&d.isWithinBar(b)&&(c=a)}),a.filter(function(a){return a&&!d.isBarType(a.id)}).forEach(function(a){var f=d.dist(a,b);e>f&&(e=f,c=a)}),c},i.dist=function(a,b){var c=this,d=c.config,e=d.axis_rotated?1:0,f=d.axis_rotated?0:1,g=c.circleY(a,a.index),h=c.x(a.x);return Math.sqrt(Math.pow(h-b[e],2)+Math.pow(g-b[f],2))},i.convertValuesToStep=function(a){var b,c=[].concat(a);if(!this.isCategorized())return a;for(b=a.length+1;b>0;b--)c[b]=c[b-1];return c[0]={x:c[0].x-1,value:c[0].value,id:c[0].id},c[a.length+1]={x:c[a.length].x+1,value:c[a.length].value,id:c[a.length].id},c},i.updateDataAttributes=function(a,b){var c=this,d=c.config,e=d["data_"+a];return"undefined"==typeof b?e:(Object.keys(b).forEach(function(a){e[a]=b[a]}),c.redraw({withLegend:!0}),e)},i.convertUrlToData=function(a,b,c,d,e){var f=this,g=b?b:"csv",h=f.d3.xhr(a);c&&Object.keys(c).forEach(function(a){h.header(a,c[a])}),h.get(function(a,b){var c;if(!b)throw new Error(a.responseURL+" "+a.status+" ("+a.statusText+")");c="json"===g?f.convertJsonToData(JSON.parse(b.response),d):"tsv"===g?f.convertTsvToData(b.response):f.convertCsvToData(b.response),e.call(f,c)})},i.convertXsvToData=function(a,b){var c,d=b.parseRows(a);return 1===d.length?(c=[{}],d[0].forEach(function(a){c[0][a]=null})):c=b.parse(a),c},i.convertCsvToData=function(a){return this.convertXsvToData(a,this.d3.csv)},i.convertTsvToData=function(a){return this.convertXsvToData(a,this.d3.tsv)},i.convertJsonToData=function(a,b){var c,d,e=this,f=[];return b?(b.x?(c=b.value.concat(b.x),e.config.data_x=b.x):c=b.value,f.push(c),a.forEach(function(a){var b=[];c.forEach(function(c){var d=e.findValueInJson(a,c);p(d)&&(d=null),b.push(d)}),f.push(b)}),d=e.convertRowsToData(f)):(Object.keys(a).forEach(function(b){f.push([b].concat(a[b]))}),d=e.convertColumnsToData(f)),d},i.findValueInJson=function(a,b){b=b.replace(/\[(\w+)\]/g,".$1"),b=b.replace(/^\./,"");for(var c=b.split("."),d=0;d<c.length;++d){var e=c[d];if(!(e in a))return;a=a[e]}return a},i.convertRowsToData=function(a){var b,c,d=a[0],e={},f=[];for(b=1;b<a.length;b++){for(e={},c=0;c<a[b].length;c++){if(p(a[b][c]))throw new Error("Source data is missing a component at ("+b+","+c+")!");e[d[c]]=a[b][c]}f.push(e)}return f},i.convertColumnsToData=function(a){var b,c,d,e=[];for(b=0;b<a.length;b++)for(d=a[b][0],c=1;c<a[b].length;c++){if(p(e[c-1])&&(e[c-1]={}),p(a[b][c]))throw new Error("Source data is missing a component at ("+b+","+c+")!");e[c-1][d]=a[b][c]}return e},i.convertDataToTargets=function(a,b){var c,d=this,e=d.config,f=d.d3.keys(a[0]).filter(d.isNotX,d),g=d.d3.keys(a[0]).filter(d.isX,d);return f.forEach(function(c){var f=d.getXKey(c);d.isCustomX()||d.isTimeSeries()?g.indexOf(f)>=0?d.data.xs[c]=(b&&d.data.xs[c]?d.data.xs[c]:[]).concat(a.map(function(a){return a[f]}).filter(m).map(function(a,b){return d.generateTargetX(a,c,b)})):e.data_x?d.data.xs[c]=d.getOtherTargetXs():v(e.data_xs)&&(d.data.xs[c]=d.getXValuesOfXKey(f,d.data.targets)):d.data.xs[c]=a.map(function(a,b){return b})}),f.forEach(function(a){if(!d.data.xs[a])throw new Error('x is not defined for id = "'+a+'".')}),c=f.map(function(b,c){var f=e.data_idConverter(b);return{id:f,id_org:b,values:a.map(function(a,g){var h,i=d.getXKey(b),j=a[i],k=null===a[b]||isNaN(a[b])?null:+a[b];return d.isCustomX()&&d.isCategorized()&&0===c&&!p(j)?(0===c&&0===g&&(e.axis_x_categories=[]),h=e.axis_x_categories.indexOf(j),-1===h&&(h=e.axis_x_categories.length,e.axis_x_categories.push(j))):h=d.generateTargetX(j,b,g),(p(a[b])||d.data.xs[b].length<=g)&&(h=void 0),{x:h,value:k,id:f}}).filter(function(a){return q(a.x)})}}),c.forEach(function(a){var b;e.data_xSort&&(a.values=a.values.sort(function(a,b){var c=a.x||0===a.x?a.x:1/0,d=b.x||0===b.x?b.x:1/0;return c-d})),b=0,a.values.forEach(function(a){a.index=b++}),d.data.xs[a.id].sort(function(a,b){return a-b})}),d.hasNegativeValue=d.hasNegativeValueInTargets(c),d.hasPositiveValue=d.hasPositiveValueInTargets(c),e.data_type&&d.setTargetType(d.mapToIds(c).filter(function(a){return!(a in e.data_types)}),e.data_type),c.forEach(function(a){d.addCache(a.id_org,a)}),c},i.load=function(a,b){var c=this;a&&(b.filter&&(a=a.filter(b.filter)),(b.type||b.types)&&a.forEach(function(a){var d=b.types&&b.types[a.id]?b.types[a.id]:b.type;c.setTargetType(a.id,d)}),c.data.targets.forEach(function(b){for(var c=0;c<a.length;c++)if(b.id===a[c].id){b.values=a[c].values,a.splice(c,1);break}}),c.data.targets=c.data.targets.concat(a)),c.updateTargets(c.data.targets),c.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0}),b.done&&b.done()},i.loadFromArgs=function(a){var b=this;a.data?b.load(b.convertDataToTargets(a.data),a):a.url?b.convertUrlToData(a.url,a.mimeType,a.headers,a.keys,function(c){b.load(b.convertDataToTargets(c),a)}):a.json?b.load(b.convertDataToTargets(b.convertJsonToData(a.json,a.keys)),a):a.rows?b.load(b.convertDataToTargets(b.convertRowsToData(a.rows)),a):a.columns?b.load(b.convertDataToTargets(b.convertColumnsToData(a.columns)),a):b.load(null,a)},i.unload=function(a,b){var c=this;return b||(b=function(){}),a=a.filter(function(a){return c.hasTarget(c.data.targets,a)}),a&&0!==a.length?(c.svg.selectAll(a.map(function(a){return c.selectorTarget(a)})).transition().style("opacity",0).remove().call(c.endall,b),void a.forEach(function(a){c.withoutFadeIn[a]=!1,c.legend&&c.legend.selectAll("."+l.legendItem+c.getTargetSelectorSuffix(a)).remove(),c.data.targets=c.data.targets.filter(function(b){return b.id!==a})})):void b()},i.categoryName=function(a){var b=this.config;return a<b.axis_x_categories.length?b.axis_x_categories[a]:a},i.initEventRect=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.eventRects).style("fill-opacity",0)},i.redrawEventRect=function(){var a,b,c=this,d=c.config,e=c.isMultipleX(),f=c.main.select("."+l.eventRects).style("cursor",d.zoom_enabled?d.axis_rotated?"ns-resize":"ew-resize":null).classed(l.eventRectsMultiple,e).classed(l.eventRectsSingle,!e);f.selectAll("."+l.eventRect).remove(),c.eventRect=f.selectAll("."+l.eventRect),e?(a=c.eventRect.data([0]),c.generateEventRectsForMultipleXs(a.enter()),c.updateEventRect(a)):(b=c.getMaxDataCountTarget(c.data.targets),f.datum(b?b.values:[]),c.eventRect=f.selectAll("."+l.eventRect),a=c.eventRect.data(function(a){return a}),c.generateEventRectsForSingleX(a.enter()),c.updateEventRect(a),a.exit().remove())},i.updateEventRect=function(a){var b,c,d,e,f,g,h=this,i=h.config;a=a||h.eventRect.data(function(a){return a}),h.isMultipleX()?(b=0,c=0,d=h.width,e=h.height):(!h.isCustomX()&&!h.isTimeSeries()||h.isCategorized()?(f=h.getEventRectWidth(),g=function(a){return h.x(a.x)-f/2}):(h.updateXs(),f=function(a){var b=h.getPrevX(a.index),c=h.getNextX(a.index);return null===b&&null===c?i.axis_rotated?h.height:h.width:(null===b&&(b=h.x.domain()[0]),null===c&&(c=h.x.domain()[1]),Math.max(0,(h.x(c)-h.x(b))/2))},g=function(a){var b=h.getPrevX(a.index),c=h.getNextX(a.index),d=h.data.xs[a.id][a.index];return null===b&&null===c?0:(null===b&&(b=h.x.domain()[0]),(h.x(d)+h.x(b))/2)}),b=i.axis_rotated?0:g,c=i.axis_rotated?g:0,d=i.axis_rotated?h.width:f,e=i.axis_rotated?f:h.height),a.attr("class",h.classEvent.bind(h)).attr("x",b).attr("y",c).attr("width",d).attr("height",e)},i.generateEventRectsForSingleX=function(a){var b=this,c=b.d3,d=b.config;a.append("rect").attr("class",b.classEvent.bind(b)).style("cursor",d.data_selection_enabled&&d.data_selection_grouped?"pointer":null).on("mouseover",function(a){var c=a.index;b.dragging||b.flowing||b.hasArcType()||(d.point_focus_expand_enabled&&b.expandCircles(c,null,!0),b.expandBars(c,null,!0),b.main.selectAll("."+l.shape+"-"+c).each(function(a){d.data_onmouseover.call(b.api,a)}))}).on("mouseout",function(a){var c=a.index;b.config&&(b.hasArcType()||(b.hideXGridFocus(),b.hideTooltip(),b.unexpandCircles(),b.unexpandBars(),b.main.selectAll("."+l.shape+"-"+c).each(function(a){d.data_onmouseout.call(b.api,a)})))}).on("mousemove",function(a){var e,f=a.index,g=b.svg.select("."+l.eventRect+"-"+f);b.dragging||b.flowing||b.hasArcType()||(b.isStepType(a)&&"step-after"===b.config.line_step_type&&c.mouse(this)[0]<b.x(b.getXValue(a.id,f))&&(f-=1),e=b.filterTargetsToShow(b.data.targets).map(function(a){return b.addName(b.getValueOnIndex(a.values,f))}),d.tooltip_grouped&&(b.showTooltip(e,this),b.showXGridFocus(e)),(!d.tooltip_grouped||d.data_selection_enabled&&!d.data_selection_grouped)&&b.main.selectAll("."+l.shape+"-"+f).each(function(){c.select(this).classed(l.EXPANDED,!0),d.data_selection_enabled&&g.style("cursor",d.data_selection_grouped?"pointer":null),d.tooltip_grouped||(b.hideXGridFocus(),b.hideTooltip(),d.data_selection_grouped||(b.unexpandCircles(f),b.unexpandBars(f)))}).filter(function(a){return b.isWithinShape(this,a)}).each(function(a){d.data_selection_enabled&&(d.data_selection_grouped||d.data_selection_isselectable(a))&&g.style("cursor","pointer"),d.tooltip_grouped||(b.showTooltip([a],this),b.showXGridFocus([a]),d.point_focus_expand_enabled&&b.expandCircles(f,a.id,!0),b.expandBars(f,a.id,!0))}))}).on("click",function(a){var e=a.index;if(!b.hasArcType()&&b.toggleShape){if(b.cancelClick)return void(b.cancelClick=!1);b.isStepType(a)&&"step-after"===d.line_step_type&&c.mouse(this)[0]<b.x(b.getXValue(a.id,e))&&(e-=1),b.main.selectAll("."+l.shape+"-"+e).each(function(a){(d.data_selection_grouped||b.isWithinShape(this,a))&&(b.toggleShape(this,a,e),b.config.data_onclick.call(b.api,a,this))})}}).call(d.data_selection_draggable&&b.drag?c.behavior.drag().origin(Object).on("drag",function(){b.drag(c.mouse(this))}).on("dragstart",function(){b.dragstart(c.mouse(this))}).on("dragend",function(){b.dragend()}):function(){})},i.generateEventRectsForMultipleXs=function(a){function b(){c.svg.select("."+l.eventRect).style("cursor",null),c.hideXGridFocus(),c.hideTooltip(),c.unexpandCircles(),c.unexpandBars()}var c=this,d=c.d3,e=c.config;a.append("rect").attr("x",0).attr("y",0).attr("width",c.width).attr("height",c.height).attr("class",l.eventRect).on("mouseout",function(){c.config&&(c.hasArcType()||b())}).on("mousemove",function(){var a,f,g,h,i=c.filterTargetsToShow(c.data.targets);if(!c.dragging&&!c.hasArcType(i)){if(a=d.mouse(this),f=c.findClosestFromTargets(i,a),!c.mouseover||f&&f.id===c.mouseover.id||(e.data_onmouseout.call(c.api,c.mouseover),c.mouseover=void 0),!f)return void b();g=c.isScatterType(f)||!e.tooltip_grouped?[f]:c.filterByX(i,f.x),h=g.map(function(a){return c.addName(a)}),c.showTooltip(h,this),e.point_focus_expand_enabled&&c.expandCircles(f.index,f.id,!0),c.expandBars(f.index,f.id,!0),c.showXGridFocus(h),(c.isBarType(f.id)||c.dist(f,a)<e.point_sensitivity)&&(c.svg.select("."+l.eventRect).style("cursor","pointer"),c.mouseover||(e.data_onmouseover.call(c.api,f),c.mouseover=f))}}).on("click",function(){var a,b,f=c.filterTargetsToShow(c.data.targets);c.hasArcType(f)||(a=d.mouse(this),b=c.findClosestFromTargets(f,a),b&&(c.isBarType(b.id)||c.dist(b,a)<e.point_sensitivity)&&c.main.selectAll("."+l.shapes+c.getTargetSelectorSuffix(b.id)).selectAll("."+l.shape+"-"+b.index).each(function(){(e.data_selection_grouped||c.isWithinShape(this,b))&&(c.toggleShape(this,b,b.index),c.config.data_onclick.call(c.api,b,this))}))}).call(e.data_selection_draggable&&c.drag?d.behavior.drag().origin(Object).on("drag",function(){c.drag(d.mouse(this))}).on("dragstart",function(){c.dragstart(d.mouse(this))}).on("dragend",function(){c.dragend()}):function(){})},i.dispatchEvent=function(b,c,d){var e=this,f="."+l.eventRect+(e.isMultipleX()?"":"-"+c),g=e.main.select(f).node(),h=g.getBoundingClientRect(),i=h.left+(d?d[0]:0),j=h.top+(d?d[1]:0),k=document.createEvent("MouseEvents");k.initMouseEvent(b,!0,!0,a,0,i,j,i,j,!1,!1,!1,!1,0,null),g.dispatchEvent(k)},i.getCurrentWidth=function(){var a=this,b=a.config;return b.size_width?b.size_width:a.getParentWidth()},i.getCurrentHeight=function(){var a=this,b=a.config,c=b.size_height?b.size_height:a.getParentHeight();return c>0?c:320/(a.hasType("gauge")&&!b.gauge_fullCircle?2:1)},i.getCurrentPaddingTop=function(){var a=this,b=a.config,c=m(b.padding_top)?b.padding_top:0;return a.title&&a.title.node()&&(c+=a.getTitlePadding()),c},i.getCurrentPaddingBottom=function(){var a=this.config;return m(a.padding_bottom)?a.padding_bottom:0},i.getCurrentPaddingLeft=function(a){var b=this,c=b.config;return m(c.padding_left)?c.padding_left:c.axis_rotated?c.axis_x_show?Math.max(r(b.getAxisWidthByAxisId("x",a)),40):1:!c.axis_y_show||c.axis_y_inner?b.axis.getYAxisLabelPosition().isOuter?30:1:r(b.getAxisWidthByAxisId("y",a))},i.getCurrentPaddingRight=function(){var a=this,b=a.config,c=10,d=a.isLegendRight?a.getLegendWidth()+20:0;return m(b.padding_right)?b.padding_right+1:b.axis_rotated?c+d:!b.axis_y2_show||b.axis_y2_inner?2+d+(a.axis.getY2AxisLabelPosition().isOuter?20:0):r(a.getAxisWidthByAxisId("y2"))+d},i.getParentRectValue=function(a){for(var b,c=this.selectChart.node();c&&"BODY"!==c.tagName;){try{b=c.getBoundingClientRect()[a]}catch(d){"width"===a&&(b=c.offsetWidth)}if(b)break;c=c.parentNode}return b},i.getParentWidth=function(){return this.getParentRectValue("width")},i.getParentHeight=function(){var a=this.selectChart.style("height");return a.indexOf("px")>0?+a.replace("px",""):0},i.getSvgLeft=function(a){var b=this,c=b.config,d=c.axis_rotated||!c.axis_rotated&&!c.axis_y_inner,e=c.axis_rotated?l.axisX:l.axisY,f=b.main.select("."+e).node(),g=f&&d?f.getBoundingClientRect():{right:0},h=b.selectChart.node().getBoundingClientRect(),i=b.hasArcType(),j=g.right-h.left-(i?0:b.getCurrentPaddingLeft(a));return j>0?j:0},i.getAxisWidthByAxisId=function(a,b){var c=this,d=c.axis.getLabelPositionById(a);return c.axis.getMaxTickWidth(a,b)+(d.isInner?20:40)},i.getHorizontalAxisHeight=function(a){var b=this,c=b.config,d=30;return"x"!==a||c.axis_x_show?"x"===a&&c.axis_x_height?c.axis_x_height:"y"!==a||c.axis_y_show?"y2"!==a||c.axis_y2_show?("x"===a&&!c.axis_rotated&&c.axis_x_tick_rotate&&(d=30+b.axis.getMaxTickWidth(a)*Math.cos(Math.PI*(90-c.axis_x_tick_rotate)/180)),"y"===a&&c.axis_rotated&&c.axis_y_tick_rotate&&(d=30+b.axis.getMaxTickWidth(a)*Math.cos(Math.PI*(90-c.axis_y_tick_rotate)/180)),d+(b.axis.getLabelPositionById(a).isInner?0:10)+("y2"===a?-10:0)):b.rotated_padding_top:!c.legend_show||b.isLegendRight||b.isLegendInset?1:10:8},i.getEventRectWidth=function(){return Math.max(0,this.xAxis.tickInterval())},i.getShapeIndices=function(a){var b,c,d=this,e=d.config,f={},g=0;return d.filterTargetsToShow(d.data.targets.filter(a,d)).forEach(function(a){for(b=0;b<e.data_groups.length;b++)if(!(e.data_groups[b].indexOf(a.id)<0))for(c=0;c<e.data_groups[b].length;c++)if(e.data_groups[b][c]in f){f[a.id]=f[e.data_groups[b][c]];break}p(f[a.id])&&(f[a.id]=g++)}),f.__max__=g-1,f},i.getShapeX=function(a,b,c,d){var e=this,f=d?e.subX:e.x;return function(d){var e=d.id in c?c[d.id]:0;return d.x||0===d.x?f(d.x)-a*(b/2-e):0}},i.getShapeY=function(a){var b=this;return function(c){var d=a?b.getSubYScale(c.id):b.getYScale(c.id);return d(c.value)}},i.getShapeOffset=function(a,b,c){var d=this,e=d.orderTargets(d.filterTargetsToShow(d.data.targets.filter(a,d))),f=e.map(function(a){return a.id});return function(a,g){var h=c?d.getSubYScale(a.id):d.getYScale(a.id),i=h(0),j=i;return e.forEach(function(c){var e=d.isStepType(a)?d.convertValuesToStep(c.values):c.values;c.id!==a.id&&b[c.id]===b[a.id]&&f.indexOf(c.id)<f.indexOf(a.id)&&("undefined"!=typeof e[g]&&+e[g].x===+a.x||(g=-1,e.forEach(function(b,c){b.x===a.x&&(g=c)})),g in e&&e[g].value*a.value>=0&&(j+=h(e[g].value)-i))}),j}},i.isWithinShape=function(a,b){var c,d=this,e=d.d3.select(a);return d.isTargetToShow(b.id)?"circle"===a.nodeName?c=d.isStepType(b)?d.isWithinStep(a,d.getYScale(b.id)(b.value)):d.isWithinCircle(a,1.5*d.pointSelectR(b)):"path"===a.nodeName&&(c=e.classed(l.bar)?d.isWithinBar(a):!0):c=!1,c},i.getInterpolate=function(a){var b=this,c=b.isInterpolationType(b.config.spline_interpolation_type)?b.config.spline_interpolation_type:"cardinal";return b.isSplineType(a)?c:b.isStepType(a)?b.config.line_step_type:"linear"},i.initLine=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartLines)},i.updateTargetsForLine=function(a){var b,c,d=this,e=d.config,f=d.classChartLine.bind(d),g=d.classLines.bind(d),h=d.classAreas.bind(d),i=d.classCircles.bind(d),j=d.classFocus.bind(d);b=d.main.select("."+l.chartLines).selectAll("."+l.chartLine).data(a).attr("class",function(a){return f(a)+j(a)}),c=b.enter().append("g").attr("class",f).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",g),c.append("g").attr("class",h),c.append("g").attr("class",function(a){return d.generateClass(l.selectedCircles,a.id)}),c.append("g").attr("class",i).style("cursor",function(a){return e.data_selection_isselectable(a)?"pointer":null}),a.forEach(function(a){d.main.selectAll("."+l.selectedCircles+d.getTargetSelectorSuffix(a.id)).selectAll("."+l.selectedCircle).each(function(b){b.value=a.values[b.index].value})})},i.updateLine=function(a){var b=this;b.mainLine=b.main.selectAll("."+l.lines).selectAll("."+l.line).data(b.lineData.bind(b)),b.mainLine.enter().append("path").attr("class",b.classLine.bind(b)).style("stroke",b.color),b.mainLine.style("opacity",b.initialOpacity.bind(b)).style("shape-rendering",function(a){return b.isStepType(a)?"crispEdges":""}).attr("transform",null),b.mainLine.exit().transition().duration(a).style("opacity",0).remove()},i.redrawLine=function(a,b){return[(b?this.mainLine.transition(Math.random().toString()):this.mainLine).attr("d",a).style("stroke",this.color).style("opacity",1)]},i.generateDrawLine=function(a,b){var c=this,d=c.config,e=c.d3.svg.line(),f=c.generateGetLinePoints(a,b),g=b?c.getSubYScale:c.getYScale,h=function(a){return(b?c.subxx:c.xx).call(c,a)},i=function(a,b){return d.data_groups.length>0?f(a,b)[0][1]:g.call(c,a.id)(a.value)};return e=d.axis_rotated?e.x(i).y(h):e.x(h).y(i),d.line_connectNull||(e=e.defined(function(a){return null!=a.value})),function(a){var f,h=d.line_connectNull?c.filterRemoveNull(a.values):a.values,i=b?c.x:c.subX,j=g.call(c,a.id),k=0,l=0;return c.isLineType(a)?d.data_regions[a.id]?f=c.lineWithRegions(h,i,j,d.data_regions[a.id]):(c.isStepType(a)&&(h=c.convertValuesToStep(h)),f=e.interpolate(c.getInterpolate(a))(h)):(h[0]&&(k=i(h[0].x),l=j(h[0].value)),f=d.axis_rotated?"M "+l+" "+k:"M "+k+" "+l),f?f:"M 0 0"}},i.generateGetLinePoints=function(a,b){var c=this,d=c.config,e=a.__max__+1,f=c.getShapeX(0,e,a,!!b),g=c.getShapeY(!!b),h=c.getShapeOffset(c.isLineType,a,!!b),i=b?c.getSubYScale:c.getYScale;return function(a,b){var e=i.call(c,a.id)(0),j=h(a,b)||e,k=f(a),l=g(a);return d.axis_rotated&&(0<a.value&&e>l||a.value<0&&l>e)&&(l=e),[[k,l-(e-j)],[k,l-(e-j)],[k,l-(e-j)],[k,l-(e-j)]]}},i.lineWithRegions=function(a,b,c,d){function e(a,b){var c;for(c=0;c<b.length;c++)if(b[c].start<a&&a<=b[c].end)return!0;return!1}function f(a){return"M"+a[0][0]+" "+a[0][1]+" "+a[1][0]+" "+a[1][1]}var g,h,i,j,k,l,m,n,o,r,s,t,u=this,v=u.config,w=-1,x="M",y=u.isCategorized()?.5:0,z=[];if(q(d))for(g=0;g<d.length;g++)z[g]={},p(d[g].start)?z[g].start=a[0].x:z[g].start=u.isTimeSeries()?u.parseDate(d[g].start):d[g].start,p(d[g].end)?z[g].end=a[a.length-1].x:z[g].end=u.isTimeSeries()?u.parseDate(d[g].end):d[g].end;for(s=v.axis_rotated?function(a){return c(a.value)}:function(a){return b(a.x)},t=v.axis_rotated?function(a){return b(a.x)}:function(a){return c(a.value)},i=u.isTimeSeries()?function(a,d,e,g){var h,i=a.x.getTime(),j=d.x-a.x,l=new Date(i+j*e),m=new Date(i+j*(e+g));return h=v.axis_rotated?[[c(k(e)),b(l)],[c(k(e+g)),b(m)]]:[[b(l),c(k(e))],[b(m),c(k(e+g))]],f(h)}:function(a,d,e,g){var h;return h=v.axis_rotated?[[c(k(e),!0),b(j(e))],[c(k(e+g),!0),b(j(e+g))]]:[[b(j(e),!0),c(k(e))],[b(j(e+g),!0),c(k(e+g))]],f(h)},g=0;g<a.length;g++){if(p(z)||!e(a[g].x,z))x+=" "+s(a[g])+" "+t(a[g]);else for(j=u.getScale(a[g-1].x+y,a[g].x+y,u.isTimeSeries()),k=u.getScale(a[g-1].value,a[g].value),l=b(a[g].x)-b(a[g-1].x),m=c(a[g].value)-c(a[g-1].value),n=Math.sqrt(Math.pow(l,2)+Math.pow(m,2)),o=2/n,r=2*o,h=o;1>=h;h+=r)x+=i(a[g-1],a[g],h,o);w=a[g].x}return x},i.updateArea=function(a){var b=this,c=b.d3;b.mainArea=b.main.selectAll("."+l.areas).selectAll("."+l.area).data(b.lineData.bind(b)),b.mainArea.enter().append("path").attr("class",b.classArea.bind(b)).style("fill",b.color).style("opacity",function(){return b.orgAreaOpacity=+c.select(this).style("opacity"),0}),b.mainArea.style("opacity",b.orgAreaOpacity),b.mainArea.exit().transition().duration(a).style("opacity",0).remove()},i.redrawArea=function(a,b){return[(b?this.mainArea.transition(Math.random().toString()):this.mainArea).attr("d",a).style("fill",this.color).style("opacity",this.orgAreaOpacity)]},i.generateDrawArea=function(a,b){var c=this,d=c.config,e=c.d3.svg.area(),f=c.generateGetAreaPoints(a,b),g=b?c.getSubYScale:c.getYScale,h=function(a){return(b?c.subxx:c.xx).call(c,a)},i=function(a,b){return d.data_groups.length>0?f(a,b)[0][1]:g.call(c,a.id)(c.getAreaBaseValue(a.id))},j=function(a,b){return d.data_groups.length>0?f(a,b)[1][1]:g.call(c,a.id)(a.value)};return e=d.axis_rotated?e.x0(i).x1(j).y(h):e.x(h).y0(d.area_above?0:i).y1(j),d.line_connectNull||(e=e.defined(function(a){return null!==a.value})),function(a){var b,f=d.line_connectNull?c.filterRemoveNull(a.values):a.values,g=0,h=0;return c.isAreaType(a)?(c.isStepType(a)&&(f=c.convertValuesToStep(f)),b=e.interpolate(c.getInterpolate(a))(f)):(f[0]&&(g=c.x(f[0].x),h=c.getYScale(a.id)(f[0].value)),b=d.axis_rotated?"M "+h+" "+g:"M "+g+" "+h),b?b:"M 0 0"}},i.getAreaBaseValue=function(){return 0},i.generateGetAreaPoints=function(a,b){var c=this,d=c.config,e=a.__max__+1,f=c.getShapeX(0,e,a,!!b),g=c.getShapeY(!!b),h=c.getShapeOffset(c.isAreaType,a,!!b),i=b?c.getSubYScale:c.getYScale;return function(a,b){var e=i.call(c,a.id)(0),j=h(a,b)||e,k=f(a),l=g(a);return d.axis_rotated&&(0<a.value&&e>l||a.value<0&&l>e)&&(l=e),[[k,j],[k,l-(e-j)],[k,l-(e-j)],[k,j]]}},i.updateCircle=function(){var a=this;a.mainCircle=a.main.selectAll("."+l.circles).selectAll("."+l.circle).data(a.lineOrScatterData.bind(a)),a.mainCircle.enter().append("circle").attr("class",a.classCircle.bind(a)).attr("r",a.pointR.bind(a)).style("fill",a.color),a.mainCircle.style("opacity",a.initialOpacityForCircle.bind(a)),a.mainCircle.exit().remove()},i.redrawCircle=function(a,b,c){var d=this.main.selectAll("."+l.selectedCircle);return[(c?this.mainCircle.transition(Math.random().toString()):this.mainCircle).style("opacity",this.opacityForCircle.bind(this)).style("fill",this.color).attr("cx",a).attr("cy",b),(c?d.transition(Math.random().toString()):d).attr("cx",a).attr("cy",b)]},i.circleX=function(a){return a.x||0===a.x?this.x(a.x):null},i.updateCircleY=function(){var a,b,c=this;c.config.data_groups.length>0?(a=c.getShapeIndices(c.isLineType),b=c.generateGetLinePoints(a),c.circleY=function(a,c){return b(a,c)[0][1]}):c.circleY=function(a){return c.getYScale(a.id)(a.value)}},i.getCircles=function(a,b){var c=this;return(b?c.main.selectAll("."+l.circles+c.getTargetSelectorSuffix(b)):c.main).selectAll("."+l.circle+(m(a)?"-"+a:""))},i.expandCircles=function(a,b,c){var d=this,e=d.pointExpandedR.bind(d);c&&d.unexpandCircles(),d.getCircles(a,b).classed(l.EXPANDED,!0).attr("r",e)},i.unexpandCircles=function(a){var b=this,c=b.pointR.bind(b);b.getCircles(a).filter(function(){return b.d3.select(this).classed(l.EXPANDED)}).classed(l.EXPANDED,!1).attr("r",c)},i.pointR=function(a){var b=this,c=b.config;return b.isStepType(a)?0:n(c.point_r)?c.point_r(a):c.point_r;
},i.pointExpandedR=function(a){var b=this,c=b.config;return c.point_focus_expand_enabled?c.point_focus_expand_r?c.point_focus_expand_r:1.75*b.pointR(a):b.pointR(a)},i.pointSelectR=function(a){var b=this,c=b.config;return n(c.point_select_r)?c.point_select_r(a):c.point_select_r?c.point_select_r:4*b.pointR(a)},i.isWithinCircle=function(a,b){var c=this.d3,d=c.mouse(a),e=c.select(a),f=+e.attr("cx"),g=+e.attr("cy");return Math.sqrt(Math.pow(f-d[0],2)+Math.pow(g-d[1],2))<b},i.isWithinStep=function(a,b){return Math.abs(b-this.d3.mouse(a)[1])<30},i.initBar=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartBars)},i.updateTargetsForBar=function(a){var b,c,d=this,e=d.config,f=d.classChartBar.bind(d),g=d.classBars.bind(d),h=d.classFocus.bind(d);b=d.main.select("."+l.chartBars).selectAll("."+l.chartBar).data(a).attr("class",function(a){return f(a)+h(a)}),c=b.enter().append("g").attr("class",f).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",g).style("cursor",function(a){return e.data_selection_isselectable(a)?"pointer":null})},i.updateBar=function(a){var b=this,c=b.barData.bind(b),d=b.classBar.bind(b),e=b.initialOpacity.bind(b),f=function(a){return b.color(a.id)};b.mainBar=b.main.selectAll("."+l.bars).selectAll("."+l.bar).data(c),b.mainBar.enter().append("path").attr("class",d).style("stroke",f).style("fill",f),b.mainBar.style("opacity",e),b.mainBar.exit().transition().duration(a).style("opacity",0).remove()},i.redrawBar=function(a,b){return[(b?this.mainBar.transition(Math.random().toString()):this.mainBar).attr("d",a).style("fill",this.color).style("opacity",1)]},i.getBarW=function(a,b){var c=this,d=c.config,e="number"==typeof d.bar_width?d.bar_width:b?a.tickInterval()*d.bar_width_ratio/b:0;return d.bar_width_max&&e>d.bar_width_max?d.bar_width_max:e},i.getBars=function(a,b){var c=this;return(b?c.main.selectAll("."+l.bars+c.getTargetSelectorSuffix(b)):c.main).selectAll("."+l.bar+(m(a)?"-"+a:""))},i.expandBars=function(a,b,c){var d=this;c&&d.unexpandBars(),d.getBars(a,b).classed(l.EXPANDED,!0)},i.unexpandBars=function(a){var b=this;b.getBars(a).classed(l.EXPANDED,!1)},i.generateDrawBar=function(a,b){var c=this,d=c.config,e=c.generateGetBarPoints(a,b);return function(a,b){var c=e(a,b),f=d.axis_rotated?1:0,g=d.axis_rotated?0:1,h="M "+c[0][f]+","+c[0][g]+" L"+c[1][f]+","+c[1][g]+" L"+c[2][f]+","+c[2][g]+" L"+c[3][f]+","+c[3][g]+" z";return h}},i.generateGetBarPoints=function(a,b){var c=this,d=b?c.subXAxis:c.xAxis,e=a.__max__+1,f=c.getBarW(d,e),g=c.getShapeX(f,e,a,!!b),h=c.getShapeY(!!b),i=c.getShapeOffset(c.isBarType,a,!!b),j=b?c.getSubYScale:c.getYScale;return function(a,b){var d=j.call(c,a.id)(0),e=i(a,b)||d,k=g(a),l=h(a);return c.config.axis_rotated&&(0<a.value&&d>l||a.value<0&&l>d)&&(l=d),[[k,e],[k,l-(d-e)],[k+f,l-(d-e)],[k+f,e]]}},i.isWithinBar=function(a){var b=this.d3.mouse(a),c=a.getBoundingClientRect(),d=a.pathSegList.getItem(0),e=a.pathSegList.getItem(1),f=Math.min(d.x,e.x),g=Math.min(d.y,e.y),h=c.width,i=c.height,j=2,k=f-j,l=f+h+j,m=g+i+j,n=g-j;return k<b[0]&&b[0]<l&&n<b[1]&&b[1]<m},i.initText=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartTexts),a.mainText=a.d3.selectAll([])},i.updateTargetsForText=function(a){var b,c,d=this,e=d.classChartText.bind(d),f=d.classTexts.bind(d),g=d.classFocus.bind(d);b=d.main.select("."+l.chartTexts).selectAll("."+l.chartText).data(a).attr("class",function(a){return e(a)+g(a)}),c=b.enter().append("g").attr("class",e).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",f)},i.updateText=function(a){var b=this,c=b.config,d=b.barOrLineData.bind(b),e=b.classText.bind(b);b.mainText=b.main.selectAll("."+l.texts).selectAll("."+l.text).data(d),b.mainText.enter().append("text").attr("class",e).attr("text-anchor",function(a){return c.axis_rotated?a.value<0?"end":"start":"middle"}).style("stroke","none").style("fill",function(a){return b.color(a)}).style("fill-opacity",0),b.mainText.text(function(a,c,d){return b.dataLabelFormat(a.id)(a.value,a.id,c,d)}),b.mainText.exit().transition().duration(a).style("fill-opacity",0).remove()},i.redrawText=function(a,b,c,d){return[(d?this.mainText.transition():this.mainText).attr("x",a).attr("y",b).style("fill",this.color).style("fill-opacity",c?0:this.opacityForText.bind(this))]},i.getTextRect=function(a,b,c){var d,e=this.d3.select("body").append("div").classed("c3",!0),f=e.append("svg").style("visibility","hidden").style("position","fixed").style("top",0).style("left",0),g=this.d3.select(c).style("font");return f.selectAll(".dummy").data([a]).enter().append("text").classed(b?b:"",!0).style("font",g).text(a).each(function(){d=this.getBoundingClientRect()}),e.remove(),d},i.generateXYForText=function(a,b,c,d){var e=this,f=e.generateGetAreaPoints(a,!1),g=e.generateGetBarPoints(b,!1),h=e.generateGetLinePoints(c,!1),i=d?e.getXForText:e.getYForText;return function(a,b){var c=e.isAreaType(a)?f:e.isBarType(a)?g:h;return i.call(e,c(a,b),a,this)}},i.getXForText=function(a,b,c){var d,e,f=this,g=c.getBoundingClientRect();return f.config.axis_rotated?(e=f.isBarType(b)?4:6,d=a[2][1]+e*(b.value<0?-1:1)):d=f.hasType("bar")?(a[2][0]+a[0][0])/2:a[0][0],null===b.value&&(d>f.width?d=f.width-g.width:0>d&&(d=4)),d},i.getYForText=function(a,b,c){var d,e=this,f=c.getBoundingClientRect();return e.config.axis_rotated?d=(a[0][0]+a[2][0]+.6*f.height)/2:(d=a[2][1],b.value<0||0===b.value&&!e.hasPositiveValue?(d+=f.height,e.isBarType(b)&&e.isSafari()?d-=3:!e.isBarType(b)&&e.isChrome()&&(d+=3)):d+=e.isBarType(b)?-3:-6),null!==b.value||e.config.axis_rotated||(d<f.height?d=f.height:d>this.height&&(d=this.height-4)),d},i.setTargetType=function(a,b){var c=this,d=c.config;c.mapToTargetIds(a).forEach(function(a){c.withoutFadeIn[a]=b===d.data_types[a],d.data_types[a]=b}),a||(d.data_type=b)},i.hasType=function(a,b){var c=this,d=c.config.data_types,e=!1;return b=b||c.data.targets,b&&b.length?b.forEach(function(b){var c=d[b.id];(c&&c.indexOf(a)>=0||!c&&"line"===a)&&(e=!0)}):Object.keys(d).length?Object.keys(d).forEach(function(b){d[b]===a&&(e=!0)}):e=c.config.data_type===a,e},i.hasArcType=function(a){return this.hasType("pie",a)||this.hasType("donut",a)||this.hasType("gauge",a)},i.isLineType=function(a){var b=this.config,c=o(a)?a:a.id;return!b.data_types[c]||["line","spline","area","area-spline","step","area-step"].indexOf(b.data_types[c])>=0},i.isStepType=function(a){var b=o(a)?a:a.id;return["step","area-step"].indexOf(this.config.data_types[b])>=0},i.isSplineType=function(a){var b=o(a)?a:a.id;return["spline","area-spline"].indexOf(this.config.data_types[b])>=0},i.isAreaType=function(a){var b=o(a)?a:a.id;return["area","area-spline","area-step"].indexOf(this.config.data_types[b])>=0},i.isBarType=function(a){var b=o(a)?a:a.id;return"bar"===this.config.data_types[b]},i.isScatterType=function(a){var b=o(a)?a:a.id;return"scatter"===this.config.data_types[b]},i.isPieType=function(a){var b=o(a)?a:a.id;return"pie"===this.config.data_types[b]},i.isGaugeType=function(a){var b=o(a)?a:a.id;return"gauge"===this.config.data_types[b]},i.isDonutType=function(a){var b=o(a)?a:a.id;return"donut"===this.config.data_types[b]},i.isArcType=function(a){return this.isPieType(a)||this.isDonutType(a)||this.isGaugeType(a)},i.lineData=function(a){return this.isLineType(a)?[a]:[]},i.arcData=function(a){return this.isArcType(a.data)?[a]:[]},i.barData=function(a){return this.isBarType(a)?a.values:[]},i.lineOrScatterData=function(a){return this.isLineType(a)||this.isScatterType(a)?a.values:[]},i.barOrLineData=function(a){return this.isBarType(a)||this.isLineType(a)?a.values:[]},i.isInterpolationType=function(a){return["linear","linear-closed","basis","basis-open","basis-closed","bundle","cardinal","cardinal-open","cardinal-closed","monotone"].indexOf(a)>=0},i.initGrid=function(){var a=this,b=a.config,c=a.d3;a.grid=a.main.append("g").attr("clip-path",a.clipPathForGrid).attr("class",l.grid),b.grid_x_show&&a.grid.append("g").attr("class",l.xgrids),b.grid_y_show&&a.grid.append("g").attr("class",l.ygrids),b.grid_focus_show&&a.grid.append("g").attr("class",l.xgridFocus).append("line").attr("class",l.xgridFocus),a.xgrid=c.selectAll([]),b.grid_lines_front||a.initGridLines()},i.initGridLines=function(){var a=this,b=a.d3;a.gridLines=a.main.append("g").attr("clip-path",a.clipPathForGrid).attr("class",l.grid+" "+l.gridLines),a.gridLines.append("g").attr("class",l.xgridLines),a.gridLines.append("g").attr("class",l.ygridLines),a.xgridLines=b.selectAll([])},i.updateXGrid=function(a){var b=this,c=b.config,d=b.d3,e=b.generateGridData(c.grid_x_type,b.x),f=b.isCategorized()?b.xAxis.tickOffset():0;b.xgridAttr=c.axis_rotated?{x1:0,x2:b.width,y1:function(a){return b.x(a)-f},y2:function(a){return b.x(a)-f}}:{x1:function(a){return b.x(a)+f},x2:function(a){return b.x(a)+f},y1:0,y2:b.height},b.xgrid=b.main.select("."+l.xgrids).selectAll("."+l.xgrid).data(e),b.xgrid.enter().append("line").attr("class",l.xgrid),a||b.xgrid.attr(b.xgridAttr).style("opacity",function(){return+d.select(this).attr(c.axis_rotated?"y1":"x1")===(c.axis_rotated?b.height:0)?0:1}),b.xgrid.exit().remove()},i.updateYGrid=function(){var a=this,b=a.config,c=a.yAxis.tickValues()||a.y.ticks(b.grid_y_ticks);a.ygrid=a.main.select("."+l.ygrids).selectAll("."+l.ygrid).data(c),a.ygrid.enter().append("line").attr("class",l.ygrid),a.ygrid.attr("x1",b.axis_rotated?a.y:0).attr("x2",b.axis_rotated?a.y:a.width).attr("y1",b.axis_rotated?0:a.y).attr("y2",b.axis_rotated?a.height:a.y),a.ygrid.exit().remove(),a.smoothLines(a.ygrid,"grid")},i.gridTextAnchor=function(a){return a.position?a.position:"end"},i.gridTextDx=function(a){return"start"===a.position?4:"middle"===a.position?0:-4},i.xGridTextX=function(a){return"start"===a.position?-this.height:"middle"===a.position?-this.height/2:0},i.yGridTextX=function(a){return"start"===a.position?0:"middle"===a.position?this.width/2:this.width},i.updateGrid=function(a){var b,c,d,e=this,f=e.main,g=e.config;e.grid.style("visibility",e.hasArcType()?"hidden":"visible"),f.select("line."+l.xgridFocus).style("visibility","hidden"),g.grid_x_show&&e.updateXGrid(),e.xgridLines=f.select("."+l.xgridLines).selectAll("."+l.xgridLine).data(g.grid_x_lines),b=e.xgridLines.enter().append("g").attr("class",function(a){return l.xgridLine+(a["class"]?" "+a["class"]:"")}),b.append("line").style("opacity",0),b.append("text").attr("text-anchor",e.gridTextAnchor).attr("transform",g.axis_rotated?"":"rotate(-90)").attr("dx",e.gridTextDx).attr("dy",-5).style("opacity",0),e.xgridLines.exit().transition().duration(a).style("opacity",0).remove(),g.grid_y_show&&e.updateYGrid(),e.ygridLines=f.select("."+l.ygridLines).selectAll("."+l.ygridLine).data(g.grid_y_lines),c=e.ygridLines.enter().append("g").attr("class",function(a){return l.ygridLine+(a["class"]?" "+a["class"]:"")}),c.append("line").style("opacity",0),c.append("text").attr("text-anchor",e.gridTextAnchor).attr("transform",g.axis_rotated?"rotate(-90)":"").attr("dx",e.gridTextDx).attr("dy",-5).style("opacity",0),d=e.yv.bind(e),e.ygridLines.select("line").transition().duration(a).attr("x1",g.axis_rotated?d:0).attr("x2",g.axis_rotated?d:e.width).attr("y1",g.axis_rotated?0:d).attr("y2",g.axis_rotated?e.height:d).style("opacity",1),e.ygridLines.select("text").transition().duration(a).attr("x",g.axis_rotated?e.xGridTextX.bind(e):e.yGridTextX.bind(e)).attr("y",d).text(function(a){return a.text}).style("opacity",1),e.ygridLines.exit().transition().duration(a).style("opacity",0).remove()},i.redrawGrid=function(a){var b=this,c=b.config,d=b.xv.bind(b),e=b.xgridLines.select("line"),f=b.xgridLines.select("text");return[(a?e.transition():e).attr("x1",c.axis_rotated?0:d).attr("x2",c.axis_rotated?b.width:d).attr("y1",c.axis_rotated?d:0).attr("y2",c.axis_rotated?d:b.height).style("opacity",1),(a?f.transition():f).attr("x",c.axis_rotated?b.yGridTextX.bind(b):b.xGridTextX.bind(b)).attr("y",d).text(function(a){return a.text}).style("opacity",1)]},i.showXGridFocus=function(a){var b=this,c=b.config,d=a.filter(function(a){return a&&m(a.value)}),e=b.main.selectAll("line."+l.xgridFocus),f=b.xx.bind(b);c.tooltip_show&&(b.hasType("scatter")||b.hasArcType()||(e.style("visibility","visible").data([d[0]]).attr(c.axis_rotated?"y1":"x1",f).attr(c.axis_rotated?"y2":"x2",f),b.smoothLines(e,"grid")))},i.hideXGridFocus=function(){this.main.select("line."+l.xgridFocus).style("visibility","hidden")},i.updateXgridFocus=function(){var a=this,b=a.config;a.main.select("line."+l.xgridFocus).attr("x1",b.axis_rotated?0:-10).attr("x2",b.axis_rotated?a.width:-10).attr("y1",b.axis_rotated?-10:0).attr("y2",b.axis_rotated?-10:a.height)},i.generateGridData=function(a,b){var c,d,e,f,g=this,h=[],i=g.main.select("."+l.axisX).selectAll(".tick").size();if("year"===a)for(c=g.getXDomain(),d=c[0].getFullYear(),e=c[1].getFullYear(),f=d;e>=f;f++)h.push(new Date(f+"-01-01 00:00:00"));else h=b.ticks(10),h.length>i&&(h=h.filter(function(a){return(""+a).indexOf(".")<0}));return h},i.getGridFilterToRemove=function(a){return a?function(b){var c=!1;return[].concat(a).forEach(function(a){("value"in a&&b.value===a.value||"class"in a&&b["class"]===a["class"])&&(c=!0)}),c}:function(){return!0}},i.removeGridLines=function(a,b){var c=this,d=c.config,e=c.getGridFilterToRemove(a),f=function(a){return!e(a)},g=b?l.xgridLines:l.ygridLines,h=b?l.xgridLine:l.ygridLine;c.main.select("."+g).selectAll("."+h).filter(e).transition().duration(d.transition_duration).style("opacity",0).remove(),b?d.grid_x_lines=d.grid_x_lines.filter(f):d.grid_y_lines=d.grid_y_lines.filter(f)},i.initTooltip=function(){var a,b=this,c=b.config;if(b.tooltip=b.selectChart.style("position","relative").append("div").attr("class",l.tooltipContainer).style("position","absolute").style("pointer-events","none").style("display","none"),c.tooltip_init_show){if(b.isTimeSeries()&&o(c.tooltip_init_x)){for(c.tooltip_init_x=b.parseDate(c.tooltip_init_x),a=0;a<b.data.targets[0].values.length&&b.data.targets[0].values[a].x-c.tooltip_init_x!==0;a++);c.tooltip_init_x=a}b.tooltip.html(c.tooltip_contents.call(b,b.data.targets.map(function(a){return b.addName(a.values[c.tooltip_init_x])}),b.axis.getXAxisTickFormat(),b.getYFormat(b.hasArcType()),b.color)),b.tooltip.style("top",c.tooltip_init_position.top).style("left",c.tooltip_init_position.left).style("display","block")}},i.getTooltipContent=function(a,b,c,d){var e,f,g,h,i,j,k=this,l=k.config,m=l.tooltip_format_title||b,n=l.tooltip_format_name||function(a){return a},o=l.tooltip_format_value||c,p=k.isOrderAsc();if(0===l.data_groups.length)a.sort(function(a,b){var c=a?a.value:null,d=b?b.value:null;return p?c-d:d-c});else{var q=k.orderTargets(k.data.targets).map(function(a){return a.id});a.sort(function(a,b){var c=a?a.value:null,d=b?b.value:null;return c>0&&d>0&&(c=a?q.indexOf(a.id):null,d=b?q.indexOf(b.id):null),p?c-d:d-c})}for(f=0;f<a.length;f++)if(a[f]&&(a[f].value||0===a[f].value)&&(e||(g=y(m?m(a[f].x):a[f].x),e="<table class='"+k.CLASS.tooltip+"'>"+(g||0===g?"<tr><th colspan='2'>"+g+"</th></tr>":"")),h=y(o(a[f].value,a[f].ratio,a[f].id,a[f].index,a)),void 0!==h)){if(null===a[f].name)continue;i=y(n(a[f].name,a[f].ratio,a[f].id,a[f].index)),j=k.levelColor?k.levelColor(a[f].value):d(a[f].id),e+="<tr class='"+k.CLASS.tooltipName+"-"+k.getTargetSelectorSuffix(a[f].id)+"'>",e+="<td class='name'><span style='background-color:"+j+"'></span>"+i+"</td>",e+="<td class='value'>"+h+"</td>",e+="</tr>"}return e+"</table>"},i.tooltipPosition=function(a,b,c,d){var e,f,g,h,i,j=this,k=j.config,l=j.d3,m=j.hasArcType(),n=l.mouse(d);return m?(f=(j.width-(j.isLegendRight?j.getLegendWidth():0))/2+n[0],h=j.height/2+n[1]+20):(e=j.getSvgLeft(!0),k.axis_rotated?(f=e+n[0]+100,g=f+b,i=j.currentWidth-j.getCurrentPaddingRight(),h=j.x(a[0].x)+20):(f=e+j.getCurrentPaddingLeft(!0)+j.x(a[0].x)+20,g=f+b,i=e+j.currentWidth-j.getCurrentPaddingRight(),h=n[1]+15),g>i&&(f-=g-i+20),h+c>j.currentHeight&&(h-=c+30)),0>h&&(h=0),{top:h,left:f}},i.showTooltip=function(a,b){var c,d,e,f=this,g=f.config,h=f.hasArcType(),j=a.filter(function(a){return a&&m(a.value)}),k=g.tooltip_position||i.tooltipPosition;0!==j.length&&g.tooltip_show&&(f.tooltip.html(g.tooltip_contents.call(f,a,f.axis.getXAxisTickFormat(),f.getYFormat(h),f.color)).style("display","block"),c=f.tooltip.property("offsetWidth"),d=f.tooltip.property("offsetHeight"),e=k.call(this,j,c,d,b),f.tooltip.style("top",e.top+"px").style("left",e.left+"px"))},i.hideTooltip=function(){this.tooltip.style("display","none")},i.initLegend=function(){var a=this;return a.legendItemTextBox={},a.legendHasRendered=!1,a.legend=a.svg.append("g").attr("transform",a.getTranslate("legend")),a.config.legend_show?void a.updateLegendWithDefaults():(a.legend.style("visibility","hidden"),void(a.hiddenLegendIds=a.mapToIds(a.data.targets)))},i.updateLegendWithDefaults=function(){var a=this;a.updateLegend(a.mapToIds(a.data.targets),{withTransform:!1,withTransitionForTransform:!1,withTransition:!1})},i.updateSizeForLegend=function(a,b){var c=this,d=c.config,e={top:c.isLegendTop?c.getCurrentPaddingTop()+d.legend_inset_y+5.5:c.currentHeight-a-c.getCurrentPaddingBottom()-d.legend_inset_y,left:c.isLegendLeft?c.getCurrentPaddingLeft()+d.legend_inset_x+.5:c.currentWidth-b-c.getCurrentPaddingRight()-d.legend_inset_x+.5};c.margin3={top:c.isLegendRight?0:c.isLegendInset?e.top:c.currentHeight-a,right:NaN,bottom:0,left:c.isLegendRight?c.currentWidth-b:c.isLegendInset?e.left:0}},i.transformLegend=function(a){var b=this;(a?b.legend.transition():b.legend).attr("transform",b.getTranslate("legend"))},i.updateLegendStep=function(a){this.legendStep=a},i.updateLegendItemWidth=function(a){this.legendItemWidth=a},i.updateLegendItemHeight=function(a){this.legendItemHeight=a},i.getLegendWidth=function(){var a=this;return a.config.legend_show?a.isLegendRight||a.isLegendInset?a.legendItemWidth*(a.legendStep+1):a.currentWidth:0},i.getLegendHeight=function(){var a=this,b=0;return a.config.legend_show&&(b=a.isLegendRight?a.currentHeight:Math.max(20,a.legendItemHeight)*(a.legendStep+1)),b},i.opacityForLegend=function(a){return a.classed(l.legendItemHidden)?null:1},i.opacityForUnfocusedLegend=function(a){return a.classed(l.legendItemHidden)?null:.3},i.toggleFocusLegend=function(a,b){var c=this;a=c.mapToTargetIds(a),c.legend.selectAll("."+l.legendItem).filter(function(b){return a.indexOf(b)>=0}).classed(l.legendItemFocused,b).transition().duration(100).style("opacity",function(){var a=b?c.opacityForLegend:c.opacityForUnfocusedLegend;return a.call(c,c.d3.select(this))})},i.revertLegend=function(){var a=this,b=a.d3;a.legend.selectAll("."+l.legendItem).classed(l.legendItemFocused,!1).transition().duration(100).style("opacity",function(){return a.opacityForLegend(b.select(this))})},i.showLegend=function(a){var b=this,c=b.config;c.legend_show||(c.legend_show=!0,b.legend.style("visibility","visible"),b.legendHasRendered||b.updateLegendWithDefaults()),b.removeHiddenLegendIds(a),b.legend.selectAll(b.selectorLegends(a)).style("visibility","visible").transition().style("opacity",function(){return b.opacityForLegend(b.d3.select(this))})},i.hideLegend=function(a){var b=this,c=b.config;c.legend_show&&u(a)&&(c.legend_show=!1,b.legend.style("visibility","hidden")),b.addHiddenLegendIds(a),b.legend.selectAll(b.selectorLegends(a)).style("opacity",0).style("visibility","hidden")},i.clearLegendItemTextBoxCache=function(){this.legendItemTextBox={}},i.updateLegend=function(a,b,c){function d(a,b){return y.legendItemTextBox[b]||(y.legendItemTextBox[b]=y.getTextRect(a.textContent,l.legendItem,a)),y.legendItemTextBox[b]}function e(b,c,e){function f(a,b){b||(g=(o-G-n)/2,E>g&&(g=(o-n)/2,G=0,M++)),L[a]=M,K[M]=y.isLegendInset?10:g,H[a]=G,G+=n}var g,h,i=0===e,j=e===a.length-1,k=d(b,c),l=k.width+F+(!j||y.isLegendRight||y.isLegendInset?B:0)+z.legend_padding,m=k.height+A,n=y.isLegendRight||y.isLegendInset?m:l,o=y.isLegendRight||y.isLegendInset?y.getLegendHeight():y.getLegendWidth();return i&&(G=0,M=0,C=0,D=0),z.legend_show&&!y.isLegendToShow(c)?void(I[c]=J[c]=L[c]=H[c]=0):(I[c]=l,J[c]=m,(!C||l>=C)&&(C=l),(!D||m>=D)&&(D=m),h=y.isLegendRight||y.isLegendInset?D:C,void(z.legend_equally?(Object.keys(I).forEach(function(a){I[a]=C}),Object.keys(J).forEach(function(a){J[a]=D}),g=(o-h*a.length)/2,E>g?(G=0,M=0,a.forEach(function(a){f(a)})):f(c,!0)):f(c)))}var f,g,h,i,j,k,m,n,o,p,r,s,t,u,v,x,y=this,z=y.config,A=4,B=10,C=0,D=0,E=10,F=z.legend_item_tile_width+5,G=0,H={},I={},J={},K=[0],L={},M=0;a=a.filter(function(a){return!q(z.data_names[a])||null!==z.data_names[a]}),b=b||{},r=w(b,"withTransition",!0),s=w(b,"withTransitionForTransform",!0),y.isLegendInset&&(M=z.legend_inset_step?z.legend_inset_step:a.length,y.updateLegendStep(M)),y.isLegendRight?(f=function(a){return C*L[a]},i=function(a){return K[L[a]]+H[a]}):y.isLegendInset?(f=function(a){return C*L[a]+10},i=function(a){return K[L[a]]+H[a]}):(f=function(a){return K[L[a]]+H[a]},i=function(a){return D*L[a]}),g=function(a,b){return f(a,b)+4+z.legend_item_tile_width},j=function(a,b){return i(a,b)+9},h=function(a,b){return f(a,b)},k=function(a,b){return i(a,b)-5},m=function(a,b){return f(a,b)-2},n=function(a,b){return f(a,b)-2+z.legend_item_tile_width},o=function(a,b){return i(a,b)+4},p=y.legend.selectAll("."+l.legendItem).data(a).enter().append("g").attr("class",function(a){return y.generateClass(l.legendItem,a)}).style("visibility",function(a){return y.isLegendToShow(a)?"visible":"hidden"}).style("cursor","pointer").on("click",function(a){z.legend_item_onclick?z.legend_item_onclick.call(y,a):y.d3.event.altKey?(y.api.hide(),y.api.show(a)):(y.api.toggle(a),y.isTargetToShow(a)?y.api.focus(a):y.api.revert())}).on("mouseover",function(a){z.legend_item_onmouseover?z.legend_item_onmouseover.call(y,a):(y.d3.select(this).classed(l.legendItemFocused,!0),!y.transiting&&y.isTargetToShow(a)&&y.api.focus(a))}).on("mouseout",function(a){z.legend_item_onmouseout?z.legend_item_onmouseout.call(y,a):(y.d3.select(this).classed(l.legendItemFocused,!1),y.api.revert())}),p.append("text").text(function(a){return q(z.data_names[a])?z.data_names[a]:a}).each(function(a,b){e(this,a,b)}).style("pointer-events","none").attr("x",y.isLegendRight||y.isLegendInset?g:-200).attr("y",y.isLegendRight||y.isLegendInset?-200:j),p.append("rect").attr("class",l.legendItemEvent).style("fill-opacity",0).attr("x",y.isLegendRight||y.isLegendInset?h:-200).attr("y",y.isLegendRight||y.isLegendInset?-200:k),p.append("line").attr("class",l.legendItemTile).style("stroke",y.color).style("pointer-events","none").attr("x1",y.isLegendRight||y.isLegendInset?m:-200).attr("y1",y.isLegendRight||y.isLegendInset?-200:o).attr("x2",y.isLegendRight||y.isLegendInset?n:-200).attr("y2",y.isLegendRight||y.isLegendInset?-200:o).attr("stroke-width",z.legend_item_tile_height),x=y.legend.select("."+l.legendBackground+" rect"),y.isLegendInset&&C>0&&0===x.size()&&(x=y.legend.insert("g","."+l.legendItem).attr("class",l.legendBackground).append("rect")),t=y.legend.selectAll("text").data(a).text(function(a){return q(z.data_names[a])?z.data_names[a]:a}).each(function(a,b){e(this,a,b)}),(r?t.transition():t).attr("x",g).attr("y",j),u=y.legend.selectAll("rect."+l.legendItemEvent).data(a),(r?u.transition():u).attr("width",function(a){return I[a]}).attr("height",function(a){return J[a]}).attr("x",h).attr("y",k),v=y.legend.selectAll("line."+l.legendItemTile).data(a),(r?v.transition():v).style("stroke",y.color).attr("x1",m).attr("y1",o).attr("x2",n).attr("y2",o),x&&(r?x.transition():x).attr("height",y.getLegendHeight()-12).attr("width",C*(M+1)+10),y.legend.selectAll("."+l.legendItem).classed(l.legendItemHidden,function(a){return!y.isTargetToShow(a)}),y.updateLegendItemWidth(C),y.updateLegendItemHeight(D),y.updateLegendStep(M),y.updateSizes(),y.updateScales(),y.updateSvgSize(),y.transformAll(s,c),y.legendHasRendered=!0},i.initTitle=function(){var a=this;a.title=a.svg.append("text").text(a.config.title_text).attr("class",a.CLASS.title)},i.redrawTitle=function(){var a=this;a.title.attr("x",a.xForTitle.bind(a)).attr("y",a.yForTitle.bind(a))},i.xForTitle=function(){var a,b=this,c=b.config,d=c.title_position||"left";return a=d.indexOf("right")>=0?b.currentWidth-b.getTextRect(b.title.node().textContent,b.CLASS.title,b.title.node()).width-c.title_padding.right:d.indexOf("center")>=0?(b.currentWidth-b.getTextRect(b.title.node().textContent,b.CLASS.title,b.title.node()).width)/2:c.title_padding.left},i.yForTitle=function(){var a=this;return a.config.title_padding.top+a.getTextRect(a.title.node().textContent,a.CLASS.title,a.title.node()).height},i.getTitlePadding=function(){var a=this;return a.yForTitle()+a.config.title_padding.bottom},c(b,f),f.prototype.init=function(){var a=this.owner,b=a.config,c=a.main;a.axes.x=c.append("g").attr("class",l.axis+" "+l.axisX).attr("clip-path",a.clipPathForXAxis).attr("transform",a.getTranslate("x")).style("visibility",b.axis_x_show?"visible":"hidden"),a.axes.x.append("text").attr("class",l.axisXLabel).attr("transform",b.axis_rotated?"rotate(-90)":"").style("text-anchor",this.textAnchorForXAxisLabel.bind(this)),a.axes.y=c.append("g").attr("class",l.axis+" "+l.axisY).attr("clip-path",b.axis_y_inner?"":a.clipPathForYAxis).attr("transform",a.getTranslate("y")).style("visibility",b.axis_y_show?"visible":"hidden"),a.axes.y.append("text").attr("class",l.axisYLabel).attr("transform",b.axis_rotated?"":"rotate(-90)").style("text-anchor",this.textAnchorForYAxisLabel.bind(this)),a.axes.y2=c.append("g").attr("class",l.axis+" "+l.axisY2).attr("transform",a.getTranslate("y2")).style("visibility",b.axis_y2_show?"visible":"hidden"),a.axes.y2.append("text").attr("class",l.axisY2Label).attr("transform",b.axis_rotated?"":"rotate(-90)").style("text-anchor",this.textAnchorForY2AxisLabel.bind(this))},f.prototype.getXAxis=function(a,b,c,d,e,f,h){var i=this.owner,j=i.config,k={isCategory:i.isCategorized(),withOuterTick:e,tickMultiline:j.axis_x_tick_multiline,tickWidth:j.axis_x_tick_width,tickTextRotate:h?0:j.axis_x_tick_rotate,withoutTransition:f},l=g(i.d3,k).scale(a).orient(b);return i.isTimeSeries()&&d&&"function"!=typeof d&&(d=d.map(function(a){return i.parseDate(a)})),l.tickFormat(c).tickValues(d),i.isCategorized()&&(l.tickCentered(j.axis_x_tick_centered),u(j.axis_x_tick_culling)&&(j.axis_x_tick_culling=!1)),l},f.prototype.updateXAxisTickValues=function(a,b){var c,d=this.owner,e=d.config;return(e.axis_x_tick_fit||e.axis_x_tick_count)&&(c=this.generateTickValues(d.mapTargetsToUniqueXs(a),e.axis_x_tick_count,d.isTimeSeries())),b?b.tickValues(c):(d.xAxis.tickValues(c),d.subXAxis.tickValues(c)),c},f.prototype.getYAxis=function(a,b,c,d,e,f,h){var i=this.owner,j=i.config,k={withOuterTick:e,withoutTransition:f,tickTextRotate:h?0:j.axis_y_tick_rotate},l=g(i.d3,k).scale(a).orient(b).tickFormat(c);return i.isTimeSeriesY()?l.ticks(i.d3.time[j.axis_y_tick_time_value],j.axis_y_tick_time_interval):l.tickValues(d),l},f.prototype.getId=function(a){var b=this.owner.config;return a in b.data_axes?b.data_axes[a]:"y"},f.prototype.getXAxisTickFormat=function(){var a=this.owner,b=a.config,c=a.isTimeSeries()?a.defaultAxisTimeFormat:a.isCategorized()?a.categoryName:function(a){return 0>a?a.toFixed(0):a};return b.axis_x_tick_format&&(n(b.axis_x_tick_format)?c=b.axis_x_tick_format:a.isTimeSeries()&&(c=function(c){return c?a.axisTimeFormat(b.axis_x_tick_format)(c):""})),n(c)?function(b){return c.call(a,b)}:c},f.prototype.getTickValues=function(a,b){return a?a:b?b.tickValues():void 0},f.prototype.getXAxisTickValues=function(){return this.getTickValues(this.owner.config.axis_x_tick_values,this.owner.xAxis)},f.prototype.getYAxisTickValues=function(){return this.getTickValues(this.owner.config.axis_y_tick_values,this.owner.yAxis)},f.prototype.getY2AxisTickValues=function(){return this.getTickValues(this.owner.config.axis_y2_tick_values,this.owner.y2Axis)},f.prototype.getLabelOptionByAxisId=function(a){var b,c=this.owner,d=c.config;return"y"===a?b=d.axis_y_label:"y2"===a?b=d.axis_y2_label:"x"===a&&(b=d.axis_x_label),b},f.prototype.getLabelText=function(a){var b=this.getLabelOptionByAxisId(a);return o(b)?b:b?b.text:null},f.prototype.setLabelText=function(a,b){var c=this.owner,d=c.config,e=this.getLabelOptionByAxisId(a);o(e)?"y"===a?d.axis_y_label=b:"y2"===a?d.axis_y2_label=b:"x"===a&&(d.axis_x_label=b):e&&(e.text=b)},f.prototype.getLabelPosition=function(a,b){var c=this.getLabelOptionByAxisId(a),d=c&&"object"==typeof c&&c.position?c.position:b;return{isInner:d.indexOf("inner")>=0,isOuter:d.indexOf("outer")>=0,isLeft:d.indexOf("left")>=0,isCenter:d.indexOf("center")>=0,isRight:d.indexOf("right")>=0,isTop:d.indexOf("top")>=0,isMiddle:d.indexOf("middle")>=0,isBottom:d.indexOf("bottom")>=0}},f.prototype.getXAxisLabelPosition=function(){return this.getLabelPosition("x",this.owner.config.axis_rotated?"inner-top":"inner-right")},f.prototype.getYAxisLabelPosition=function(){return this.getLabelPosition("y",this.owner.config.axis_rotated?"inner-right":"inner-top")},f.prototype.getY2AxisLabelPosition=function(){return this.getLabelPosition("y2",this.owner.config.axis_rotated?"inner-right":"inner-top")},f.prototype.getLabelPositionById=function(a){return"y2"===a?this.getY2AxisLabelPosition():"y"===a?this.getYAxisLabelPosition():this.getXAxisLabelPosition()},f.prototype.textForXAxisLabel=function(){return this.getLabelText("x")},f.prototype.textForYAxisLabel=function(){return this.getLabelText("y")},f.prototype.textForY2AxisLabel=function(){return this.getLabelText("y2")},f.prototype.xForAxisLabel=function(a,b){var c=this.owner;return a?b.isLeft?0:b.isCenter?c.width/2:c.width:b.isBottom?-c.height:b.isMiddle?-c.height/2:0},f.prototype.dxForAxisLabel=function(a,b){return a?b.isLeft?"0.5em":b.isRight?"-0.5em":"0":b.isTop?"-0.5em":b.isBottom?"0.5em":"0"},f.prototype.textAnchorForAxisLabel=function(a,b){return a?b.isLeft?"start":b.isCenter?"middle":"end":b.isBottom?"start":b.isMiddle?"middle":"end"},f.prototype.xForXAxisLabel=function(){return this.xForAxisLabel(!this.owner.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.xForYAxisLabel=function(){return this.xForAxisLabel(this.owner.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.xForY2AxisLabel=function(){return this.xForAxisLabel(this.owner.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.dxForXAxisLabel=function(){return this.dxForAxisLabel(!this.owner.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.dxForYAxisLabel=function(){return this.dxForAxisLabel(this.owner.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.dxForY2AxisLabel=function(){return this.dxForAxisLabel(this.owner.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.dyForXAxisLabel=function(){var a=this.owner,b=a.config,c=this.getXAxisLabelPosition();return b.axis_rotated?c.isInner?"1.2em":-25-this.getMaxTickWidth("x"):c.isInner?"-0.5em":b.axis_x_height?b.axis_x_height-10:"3em"},f.prototype.dyForYAxisLabel=function(){var a=this.owner,b=this.getYAxisLabelPosition();return a.config.axis_rotated?b.isInner?"-0.5em":"3em":b.isInner?"1.2em":-10-(a.config.axis_y_inner?0:this.getMaxTickWidth("y")+10)},f.prototype.dyForY2AxisLabel=function(){var a=this.owner,b=this.getY2AxisLabelPosition();return a.config.axis_rotated?b.isInner?"1.2em":"-2.2em":b.isInner?"-0.5em":15+(a.config.axis_y2_inner?0:this.getMaxTickWidth("y2")+15)},f.prototype.textAnchorForXAxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(!a.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.textAnchorForYAxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(a.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.textAnchorForY2AxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(a.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.getMaxTickWidth=function(a,b){var c,d,e,f,g,h=this.owner,i=h.config,j=0;return b&&h.currentMaxTickWidths[a]?h.currentMaxTickWidths[a]:(h.svg&&(c=h.filterTargetsToShow(h.data.targets),"y"===a?(d=h.y.copy().domain(h.getYDomain(c,"y")),e=this.getYAxis(d,h.yOrient,i.axis_y_tick_format,h.yAxisTickValues,!1,!0,!0)):"y2"===a?(d=h.y2.copy().domain(h.getYDomain(c,"y2")),
e=this.getYAxis(d,h.y2Orient,i.axis_y2_tick_format,h.y2AxisTickValues,!1,!0,!0)):(d=h.x.copy().domain(h.getXDomain(c)),e=this.getXAxis(d,h.xOrient,h.xAxisTickFormat,h.xAxisTickValues,!1,!0,!0),this.updateXAxisTickValues(c,e)),f=h.d3.select("body").append("div").classed("c3",!0),g=f.append("svg").style("visibility","hidden").style("position","fixed").style("top",0).style("left",0),g.append("g").call(e).each(function(){h.d3.select(this).selectAll("text").each(function(){var a=this.getBoundingClientRect();j<a.width&&(j=a.width)}),f.remove()})),h.currentMaxTickWidths[a]=0>=j?h.currentMaxTickWidths[a]:j,h.currentMaxTickWidths[a])},f.prototype.updateLabels=function(a){var b=this.owner,c=b.main.select("."+l.axisX+" ."+l.axisXLabel),d=b.main.select("."+l.axisY+" ."+l.axisYLabel),e=b.main.select("."+l.axisY2+" ."+l.axisY2Label);(a?c.transition():c).attr("x",this.xForXAxisLabel.bind(this)).attr("dx",this.dxForXAxisLabel.bind(this)).attr("dy",this.dyForXAxisLabel.bind(this)).text(this.textForXAxisLabel.bind(this)),(a?d.transition():d).attr("x",this.xForYAxisLabel.bind(this)).attr("dx",this.dxForYAxisLabel.bind(this)).attr("dy",this.dyForYAxisLabel.bind(this)).text(this.textForYAxisLabel.bind(this)),(a?e.transition():e).attr("x",this.xForY2AxisLabel.bind(this)).attr("dx",this.dxForY2AxisLabel.bind(this)).attr("dy",this.dyForY2AxisLabel.bind(this)).text(this.textForY2AxisLabel.bind(this))},f.prototype.getPadding=function(a,b,c,d){var e="number"==typeof a?a:a[b];return m(e)?"ratio"===a.unit?a[b]*d:this.convertPixelsToAxisPadding(e,d):c},f.prototype.convertPixelsToAxisPadding=function(a,b){var c=this.owner,d=c.config.axis_rotated?c.width:c.height;return b*(a/d)},f.prototype.generateTickValues=function(a,b,c){var d,e,f,g,h,i,j,k=a;if(b)if(d=n(b)?b():b,1===d)k=[a[0]];else if(2===d)k=[a[0],a[a.length-1]];else if(d>2){for(g=d-2,e=a[0],f=a[a.length-1],h=(f-e)/(g+1),k=[e],i=0;g>i;i++)j=+e+h*(i+1),k.push(c?new Date(j):j);k.push(f)}return c||(k=k.sort(function(a,b){return a-b})),k},f.prototype.generateTransitions=function(a){var b=this.owner,c=b.axes;return{axisX:a?c.x.transition().duration(a):c.x,axisY:a?c.y.transition().duration(a):c.y,axisY2:a?c.y2.transition().duration(a):c.y2,axisSubX:a?c.subx.transition().duration(a):c.subx}},f.prototype.redraw=function(a,b){var c=this.owner;c.axes.x.style("opacity",b?0:1),c.axes.y.style("opacity",b?0:1),c.axes.y2.style("opacity",b?0:1),c.axes.subx.style("opacity",b?0:1),a.axisX.call(c.xAxis),a.axisY.call(c.yAxis),a.axisY2.call(c.y2Axis),a.axisSubX.call(c.subXAxis)},i.getClipPath=function(b){var c=a.navigator.appVersion.toLowerCase().indexOf("msie 9.")>=0;return"url("+(c?"":document.URL.split("#")[0])+"#"+b+")"},i.appendClip=function(a,b){return a.append("clipPath").attr("id",b).append("rect")},i.getAxisClipX=function(a){var b=Math.max(30,this.margin.left);return a?-(1+b):-(b-1)},i.getAxisClipY=function(a){return a?-20:-this.margin.top},i.getXAxisClipX=function(){var a=this;return a.getAxisClipX(!a.config.axis_rotated)},i.getXAxisClipY=function(){var a=this;return a.getAxisClipY(!a.config.axis_rotated)},i.getYAxisClipX=function(){var a=this;return a.config.axis_y_inner?-1:a.getAxisClipX(a.config.axis_rotated)},i.getYAxisClipY=function(){var a=this;return a.getAxisClipY(a.config.axis_rotated)},i.getAxisClipWidth=function(a){var b=this,c=Math.max(30,b.margin.left),d=Math.max(30,b.margin.right);return a?b.width+2+c+d:b.margin.left+20},i.getAxisClipHeight=function(a){return(a?this.margin.bottom:this.margin.top+this.height)+20},i.getXAxisClipWidth=function(){var a=this;return a.getAxisClipWidth(!a.config.axis_rotated)},i.getXAxisClipHeight=function(){var a=this;return a.getAxisClipHeight(!a.config.axis_rotated)},i.getYAxisClipWidth=function(){var a=this;return a.getAxisClipWidth(a.config.axis_rotated)+(a.config.axis_y_inner?20:0)},i.getYAxisClipHeight=function(){var a=this;return a.getAxisClipHeight(a.config.axis_rotated)},i.initPie=function(){var a=this,b=a.d3,c=a.config;a.pie=b.layout.pie().value(function(a){return a.values.reduce(function(a,b){return a+b.value},0)}),c.data_order||a.pie.sort(null)},i.updateRadius=function(){var a=this,b=a.config,c=b.gauge_width||b.donut_width;a.radiusExpanded=Math.min(a.arcWidth,a.arcHeight)/2,a.radius=.95*a.radiusExpanded,a.innerRadiusRatio=c?(a.radius-c)/a.radius:.6,a.innerRadius=a.hasType("donut")||a.hasType("gauge")?a.radius*a.innerRadiusRatio:0},i.updateArc=function(){var a=this;a.svgArc=a.getSvgArc(),a.svgArcExpanded=a.getSvgArcExpanded(),a.svgArcExpandedSub=a.getSvgArcExpanded(.98)},i.updateAngle=function(a){var b,c,d,e,f=this,g=f.config,h=!1,i=0;return g?(f.pie(f.filterTargetsToShow(f.data.targets)).forEach(function(b){h||b.data.id!==a.data.id||(h=!0,a=b,a.index=i),i++}),isNaN(a.startAngle)&&(a.startAngle=0),isNaN(a.endAngle)&&(a.endAngle=a.startAngle),f.isGaugeType(a.data)&&(b=g.gauge_min,c=g.gauge_max,d=Math.PI*(g.gauge_fullCircle?2:1)/(c-b),e=a.value<b?0:a.value<c?a.value-b:c-b,a.startAngle=g.gauge_startingAngle,a.endAngle=a.startAngle+d*e),h?a:null):null},i.getSvgArc=function(){var a=this,b=a.d3.svg.arc().outerRadius(a.radius).innerRadius(a.innerRadius),c=function(c,d){var e;return d?b(c):(e=a.updateAngle(c),e?b(e):"M 0 0")};return c.centroid=b.centroid,c},i.getSvgArcExpanded=function(a){var b=this,c=b.d3.svg.arc().outerRadius(b.radiusExpanded*(a?a:1)).innerRadius(b.innerRadius);return function(a){var d=b.updateAngle(a);return d?c(d):"M 0 0"}},i.getArc=function(a,b,c){return c||this.isArcType(a.data)?this.svgArc(a,b):"M 0 0"},i.transformForArcLabel=function(a){var b,c,d,e,f,g=this,h=g.config,i=g.updateAngle(a),j="";return i&&!g.hasType("gauge")&&(b=this.svgArc.centroid(i),c=isNaN(b[0])?0:b[0],d=isNaN(b[1])?0:b[1],e=Math.sqrt(c*c+d*d),f=g.hasType("donut")&&h.donut_label_ratio?n(h.donut_label_ratio)?h.donut_label_ratio(a,g.radius,e):h.donut_label_ratio:g.hasType("pie")&&h.pie_label_ratio?n(h.pie_label_ratio)?h.pie_label_ratio(a,g.radius,e):h.pie_label_ratio:g.radius&&e?(36/g.radius>.375?1.175-36/g.radius:.8)*g.radius/e:0,j="translate("+c*f+","+d*f+")"),j},i.getArcRatio=function(a){var b=this,c=b.config,d=Math.PI*(b.hasType("gauge")&&!c.gauge_fullCircle?1:2);return a?(a.endAngle-a.startAngle)/d:null},i.convertToArcData=function(a){return this.addName({id:a.data.id,value:a.value,ratio:this.getArcRatio(a),index:a.index})},i.textForArcLabel=function(a){var b,c,d,e,f,g=this;return g.shouldShowArcLabel()?(b=g.updateAngle(a),c=b?b.value:null,d=g.getArcRatio(b),e=a.data.id,g.hasType("gauge")||g.meetsArcLabelThreshold(d)?(f=g.getArcLabelFormat(),f?f(c,d,e):g.defaultArcValueFormat(c,d)):""):""},i.expandArc=function(b){var c,d=this;return d.transiting?void(c=a.setInterval(function(){d.transiting||(a.clearInterval(c),d.legend.selectAll(".c3-legend-item-focused").size()>0&&d.expandArc(b))},10)):(b=d.mapToTargetIds(b),void d.svg.selectAll(d.selectorTargets(b,"."+l.chartArc)).each(function(a){d.shouldExpand(a.data.id)&&d.d3.select(this).selectAll("path").transition().duration(d.expandDuration(a.data.id)).attr("d",d.svgArcExpanded).transition().duration(2*d.expandDuration(a.data.id)).attr("d",d.svgArcExpandedSub).each(function(a){d.isDonutType(a.data)})}))},i.unexpandArc=function(a){var b=this;b.transiting||(a=b.mapToTargetIds(a),b.svg.selectAll(b.selectorTargets(a,"."+l.chartArc)).selectAll("path").transition().duration(function(a){return b.expandDuration(a.data.id)}).attr("d",b.svgArc),b.svg.selectAll("."+l.arc).style("opacity",1))},i.expandDuration=function(a){var b=this,c=b.config;return b.isDonutType(a)?c.donut_expand_duration:b.isGaugeType(a)?c.gauge_expand_duration:b.isPieType(a)?c.pie_expand_duration:50},i.shouldExpand=function(a){var b=this,c=b.config;return b.isDonutType(a)&&c.donut_expand||b.isGaugeType(a)&&c.gauge_expand||b.isPieType(a)&&c.pie_expand},i.shouldShowArcLabel=function(){var a=this,b=a.config,c=!0;return a.hasType("donut")?c=b.donut_label_show:a.hasType("pie")&&(c=b.pie_label_show),c},i.meetsArcLabelThreshold=function(a){var b=this,c=b.config,d=b.hasType("donut")?c.donut_label_threshold:c.pie_label_threshold;return a>=d},i.getArcLabelFormat=function(){var a=this,b=a.config,c=b.pie_label_format;return a.hasType("gauge")?c=b.gauge_label_format:a.hasType("donut")&&(c=b.donut_label_format),c},i.getArcTitle=function(){var a=this;return a.hasType("donut")?a.config.donut_title:""},i.updateTargetsForArc=function(a){var b,c,d=this,e=d.main,f=d.classChartArc.bind(d),g=d.classArcs.bind(d),h=d.classFocus.bind(d);b=e.select("."+l.chartArcs).selectAll("."+l.chartArc).data(d.pie(a)).attr("class",function(a){return f(a)+h(a.data)}),c=b.enter().append("g").attr("class",f),c.append("g").attr("class",g),c.append("text").attr("dy",d.hasType("gauge")?"-.1em":".35em").style("opacity",0).style("text-anchor","middle").style("pointer-events","none")},i.initArc=function(){var a=this;a.arcs=a.main.select("."+l.chart).append("g").attr("class",l.chartArcs).attr("transform",a.getTranslate("arc")),a.arcs.append("text").attr("class",l.chartArcsTitle).style("text-anchor","middle").text(a.getArcTitle())},i.redrawArc=function(a,b,c){var d,e=this,f=e.d3,g=e.config,h=e.main;d=h.selectAll("."+l.arcs).selectAll("."+l.arc).data(e.arcData.bind(e)),d.enter().append("path").attr("class",e.classArc.bind(e)).style("fill",function(a){return e.color(a.data)}).style("cursor",function(a){return g.interaction_enabled&&g.data_selection_isselectable(a)?"pointer":null}).style("opacity",0).each(function(a){e.isGaugeType(a.data)&&(a.startAngle=a.endAngle=g.gauge_startingAngle),this._current=a}),d.attr("transform",function(a){return!e.isGaugeType(a.data)&&c?"scale(0)":""}).style("opacity",function(a){return a===this._current?0:1}).on("mouseover",g.interaction_enabled?function(a){var b,c;e.transiting||(b=e.updateAngle(a),b&&(c=e.convertToArcData(b),e.expandArc(b.data.id),e.api.focus(b.data.id),e.toggleFocusLegend(b.data.id,!0),e.config.data_onmouseover(c,this)))}:null).on("mousemove",g.interaction_enabled?function(a){var b,c,d=e.updateAngle(a);d&&(b=e.convertToArcData(d),c=[b],e.showTooltip(c,this))}:null).on("mouseout",g.interaction_enabled?function(a){var b,c;e.transiting||(b=e.updateAngle(a),b&&(c=e.convertToArcData(b),e.unexpandArc(b.data.id),e.api.revert(),e.revertLegend(),e.hideTooltip(),e.config.data_onmouseout(c,this)))}:null).on("click",g.interaction_enabled?function(a,b){var c,d=e.updateAngle(a);d&&(c=e.convertToArcData(d),e.toggleShape&&e.toggleShape(this,c,b),e.config.data_onclick.call(e.api,c,this))}:null).each(function(){e.transiting=!0}).transition().duration(a).attrTween("d",function(a){var b,c=e.updateAngle(a);return c?(isNaN(this._current.startAngle)&&(this._current.startAngle=0),isNaN(this._current.endAngle)&&(this._current.endAngle=this._current.startAngle),b=f.interpolate(this._current,c),this._current=b(0),function(c){var d=b(c);return d.data=a.data,e.getArc(d,!0)}):function(){return"M 0 0"}}).attr("transform",c?"scale(1)":"").style("fill",function(a){return e.levelColor?e.levelColor(a.data.values[0].value):e.color(a.data.id)}).style("opacity",1).call(e.endall,function(){e.transiting=!1}),d.exit().transition().duration(b).style("opacity",0).remove(),h.selectAll("."+l.chartArc).select("text").style("opacity",0).attr("class",function(a){return e.isGaugeType(a.data)?l.gaugeValue:""}).text(e.textForArcLabel.bind(e)).attr("transform",e.transformForArcLabel.bind(e)).style("font-size",function(a){return e.isGaugeType(a.data)?Math.round(e.radius/5)+"px":""}).transition().duration(a).style("opacity",function(a){return e.isTargetToShow(a.data.id)&&e.isArcType(a.data)?1:0}),h.select("."+l.chartArcsTitle).style("opacity",e.hasType("donut")||e.hasType("gauge")?1:0),e.hasType("gauge")&&(e.arcs.select("."+l.chartArcsBackground).attr("d",function(){var a={data:[{value:g.gauge_max}],startAngle:g.gauge_startingAngle,endAngle:-1*g.gauge_startingAngle};return e.getArc(a,!0,!0)}),e.arcs.select("."+l.chartArcsGaugeUnit).attr("dy",".75em").text(g.gauge_label_show?g.gauge_units:""),e.arcs.select("."+l.chartArcsGaugeMin).attr("dx",-1*(e.innerRadius+(e.radius-e.innerRadius)/(g.gauge_fullCircle?1:2))+"px").attr("dy","1.2em").text(g.gauge_label_show?g.gauge_min:""),e.arcs.select("."+l.chartArcsGaugeMax).attr("dx",e.innerRadius+(e.radius-e.innerRadius)/(g.gauge_fullCircle?1:2)+"px").attr("dy","1.2em").text(g.gauge_label_show?g.gauge_max:""))},i.initGauge=function(){var a=this.arcs;this.hasType("gauge")&&(a.append("path").attr("class",l.chartArcsBackground),a.append("text").attr("class",l.chartArcsGaugeUnit).style("text-anchor","middle").style("pointer-events","none"),a.append("text").attr("class",l.chartArcsGaugeMin).style("text-anchor","middle").style("pointer-events","none"),a.append("text").attr("class",l.chartArcsGaugeMax).style("text-anchor","middle").style("pointer-events","none"))},i.getGaugeLabelHeight=function(){return this.config.gauge_label_show?20:0},i.initRegion=function(){var a=this;a.region=a.main.append("g").attr("clip-path",a.clipPath).attr("class",l.regions)},i.updateRegion=function(a){var b=this,c=b.config;b.region.style("visibility",b.hasArcType()?"hidden":"visible"),b.mainRegion=b.main.select("."+l.regions).selectAll("."+l.region).data(c.regions),b.mainRegion.enter().append("g").append("rect").style("fill-opacity",0),b.mainRegion.attr("class",b.classRegion.bind(b)),b.mainRegion.exit().transition().duration(a).style("opacity",0).remove()},i.redrawRegion=function(a){var b=this,c=b.mainRegion.selectAll("rect").each(function(){var a=b.d3.select(this.parentNode).datum();b.d3.select(this).datum(a)}),d=b.regionX.bind(b),e=b.regionY.bind(b),f=b.regionWidth.bind(b),g=b.regionHeight.bind(b);return[(a?c.transition():c).attr("x",d).attr("y",e).attr("width",f).attr("height",g).style("fill-opacity",function(a){return m(a.opacity)?a.opacity:.1})]},i.regionX=function(a){var b,c=this,d=c.config,e="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated&&"start"in a?e(a.start):0:d.axis_rotated?0:"start"in a?c.x(c.isTimeSeries()?c.parseDate(a.start):a.start):0},i.regionY=function(a){var b,c=this,d=c.config,e="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated?0:"end"in a?e(a.end):0:d.axis_rotated&&"start"in a?c.x(c.isTimeSeries()?c.parseDate(a.start):a.start):0},i.regionWidth=function(a){var b,c=this,d=c.config,e=c.regionX(a),f="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated&&"end"in a?f(a.end):c.width:d.axis_rotated?c.width:"end"in a?c.x(c.isTimeSeries()?c.parseDate(a.end):a.end):c.width,e>b?0:b-e},i.regionHeight=function(a){var b,c=this,d=c.config,e=this.regionY(a),f="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated?c.height:"start"in a?f(a.start):c.height:d.axis_rotated&&"end"in a?c.x(c.isTimeSeries()?c.parseDate(a.end):a.end):c.height,e>b?0:b-e},i.isRegionOnX=function(a){return!a.axis||"x"===a.axis},i.drag=function(a){var b,c,d,e,f,g,h,i,j=this,k=j.config,m=j.main,n=j.d3;j.hasArcType()||k.data_selection_enabled&&(k.zoom_enabled&&!j.zoom.altDomain||k.data_selection_multiple&&(b=j.dragStart[0],c=j.dragStart[1],d=a[0],e=a[1],f=Math.min(b,d),g=Math.max(b,d),h=k.data_selection_grouped?j.margin.top:Math.min(c,e),i=k.data_selection_grouped?j.height:Math.max(c,e),m.select("."+l.dragarea).attr("x",f).attr("y",h).attr("width",g-f).attr("height",i-h),m.selectAll("."+l.shapes).selectAll("."+l.shape).filter(function(a){return k.data_selection_isselectable(a)}).each(function(a,b){var c,d,e,k,m,o,p=n.select(this),q=p.classed(l.SELECTED),r=p.classed(l.INCLUDED),s=!1;if(p.classed(l.circle))c=1*p.attr("cx"),d=1*p.attr("cy"),m=j.togglePoint,s=c>f&&g>c&&d>h&&i>d;else{if(!p.classed(l.bar))return;o=z(this),c=o.x,d=o.y,e=o.width,k=o.height,m=j.togglePath,s=!(c>g||f>c+e||d>i||h>d+k)}s^r&&(p.classed(l.INCLUDED,!r),p.classed(l.SELECTED,!q),m.call(j,!q,p,a,b))})))},i.dragstart=function(a){var b=this,c=b.config;b.hasArcType()||c.data_selection_enabled&&(b.dragStart=a,b.main.select("."+l.chart).append("rect").attr("class",l.dragarea).style("opacity",.1),b.dragging=!0)},i.dragend=function(){var a=this,b=a.config;a.hasArcType()||b.data_selection_enabled&&(a.main.select("."+l.dragarea).transition().duration(100).style("opacity",0).remove(),a.main.selectAll("."+l.shape).classed(l.INCLUDED,!1),a.dragging=!1)},i.selectPoint=function(a,b,c){var d=this,e=d.config,f=(e.axis_rotated?d.circleY:d.circleX).bind(d),g=(e.axis_rotated?d.circleX:d.circleY).bind(d),h=d.pointSelectR.bind(d);e.data_onselected.call(d.api,b,a.node()),d.main.select("."+l.selectedCircles+d.getTargetSelectorSuffix(b.id)).selectAll("."+l.selectedCircle+"-"+c).data([b]).enter().append("circle").attr("class",function(){return d.generateClass(l.selectedCircle,c)}).attr("cx",f).attr("cy",g).attr("stroke",function(){return d.color(b)}).attr("r",function(a){return 1.4*d.pointSelectR(a)}).transition().duration(100).attr("r",h)},i.unselectPoint=function(a,b,c){var d=this;d.config.data_onunselected.call(d.api,b,a.node()),d.main.select("."+l.selectedCircles+d.getTargetSelectorSuffix(b.id)).selectAll("."+l.selectedCircle+"-"+c).transition().duration(100).attr("r",0).remove()},i.togglePoint=function(a,b,c,d){a?this.selectPoint(b,c,d):this.unselectPoint(b,c,d)},i.selectPath=function(a,b){var c=this;c.config.data_onselected.call(c,b,a.node()),c.config.interaction_brighten&&a.transition().duration(100).style("fill",function(){return c.d3.rgb(c.color(b)).brighter(.75)})},i.unselectPath=function(a,b){var c=this;c.config.data_onunselected.call(c,b,a.node()),c.config.interaction_brighten&&a.transition().duration(100).style("fill",function(){return c.color(b)})},i.togglePath=function(a,b,c,d){a?this.selectPath(b,c,d):this.unselectPath(b,c,d)},i.getToggle=function(a,b){var c,d=this;return"circle"===a.nodeName?c=d.isStepType(b)?function(){}:d.togglePoint:"path"===a.nodeName&&(c=d.togglePath),c},i.toggleShape=function(a,b,c){var d=this,e=d.d3,f=d.config,g=e.select(a),h=g.classed(l.SELECTED),i=d.getToggle(a,b).bind(d);f.data_selection_enabled&&f.data_selection_isselectable(b)&&(f.data_selection_multiple||d.main.selectAll("."+l.shapes+(f.data_selection_grouped?d.getTargetSelectorSuffix(b.id):"")).selectAll("."+l.shape).each(function(a,b){var c=e.select(this);c.classed(l.SELECTED)&&i(!1,c.classed(l.SELECTED,!1),a,b)}),g.classed(l.SELECTED,!h),i(!h,g,b,c))},i.initBrush=function(){var a=this,b=a.d3;a.brush=b.svg.brush().on("brush",function(){a.redrawForBrush()}),a.brush.update=function(){return a.context&&a.context.select("."+l.brush).call(this),this},a.brush.scale=function(b){return a.config.axis_rotated?this.y(b):this.x(b)}},i.initSubchart=function(){var a=this,b=a.config,c=a.context=a.svg.append("g").attr("transform",a.getTranslate("context")),d=b.subchart_show?"visible":"hidden";c.style("visibility",d),c.append("g").attr("clip-path",a.clipPathForSubchart).attr("class",l.chart),c.select("."+l.chart).append("g").attr("class",l.chartBars),c.select("."+l.chart).append("g").attr("class",l.chartLines),c.append("g").attr("clip-path",a.clipPath).attr("class",l.brush).call(a.brush),a.axes.subx=c.append("g").attr("class",l.axisX).attr("transform",a.getTranslate("subx")).attr("clip-path",b.axis_rotated?"":a.clipPathForXAxis).style("visibility",b.subchart_axis_x_show?d:"hidden")},i.updateTargetsForSubchart=function(a){var b,c,d,e,f=this,g=f.context,h=f.config,i=f.classChartBar.bind(f),j=f.classBars.bind(f),k=f.classChartLine.bind(f),m=f.classLines.bind(f),n=f.classAreas.bind(f);h.subchart_show&&(e=g.select("."+l.chartBars).selectAll("."+l.chartBar).data(a).attr("class",i),d=e.enter().append("g").style("opacity",0).attr("class",i),d.append("g").attr("class",j),c=g.select("."+l.chartLines).selectAll("."+l.chartLine).data(a).attr("class",k),b=c.enter().append("g").style("opacity",0).attr("class",k),b.append("g").attr("class",m),b.append("g").attr("class",n),g.selectAll("."+l.brush+" rect").attr(h.axis_rotated?"width":"height",h.axis_rotated?f.width2:f.height2))},i.updateBarForSubchart=function(a){var b=this;b.contextBar=b.context.selectAll("."+l.bars).selectAll("."+l.bar).data(b.barData.bind(b)),b.contextBar.enter().append("path").attr("class",b.classBar.bind(b)).style("stroke","none").style("fill",b.color),b.contextBar.style("opacity",b.initialOpacity.bind(b)),b.contextBar.exit().transition().duration(a).style("opacity",0).remove()},i.redrawBarForSubchart=function(a,b,c){(b?this.contextBar.transition(Math.random().toString()).duration(c):this.contextBar).attr("d",a).style("opacity",1)},i.updateLineForSubchart=function(a){var b=this;b.contextLine=b.context.selectAll("."+l.lines).selectAll("."+l.line).data(b.lineData.bind(b)),b.contextLine.enter().append("path").attr("class",b.classLine.bind(b)).style("stroke",b.color),b.contextLine.style("opacity",b.initialOpacity.bind(b)),b.contextLine.exit().transition().duration(a).style("opacity",0).remove()},i.redrawLineForSubchart=function(a,b,c){(b?this.contextLine.transition(Math.random().toString()).duration(c):this.contextLine).attr("d",a).style("opacity",1)},i.updateAreaForSubchart=function(a){var b=this,c=b.d3;b.contextArea=b.context.selectAll("."+l.areas).selectAll("."+l.area).data(b.lineData.bind(b)),b.contextArea.enter().append("path").attr("class",b.classArea.bind(b)).style("fill",b.color).style("opacity",function(){return b.orgAreaOpacity=+c.select(this).style("opacity"),0}),b.contextArea.style("opacity",0),b.contextArea.exit().transition().duration(a).style("opacity",0).remove()},i.redrawAreaForSubchart=function(a,b,c){(b?this.contextArea.transition(Math.random().toString()).duration(c):this.contextArea).attr("d",a).style("fill",this.color).style("opacity",this.orgAreaOpacity)},i.redrawSubchart=function(a,b,c,d,e,f,g){var h,i,j,k=this,l=k.d3,m=k.config;k.context.style("visibility",m.subchart_show?"visible":"hidden"),m.subchart_show&&(l.event&&"zoom"===l.event.type&&k.brush.extent(k.x.orgDomain()).update(),a&&(k.brush.empty()||k.brush.extent(k.x.orgDomain()).update(),h=k.generateDrawArea(e,!0),i=k.generateDrawBar(f,!0),j=k.generateDrawLine(g,!0),k.updateBarForSubchart(c),k.updateLineForSubchart(c),k.updateAreaForSubchart(c),k.redrawBarForSubchart(i,c,c),k.redrawLineForSubchart(j,c,c),k.redrawAreaForSubchart(h,c,c)))},i.redrawForBrush=function(){var a=this,b=a.x;a.redraw({withTransition:!1,withY:a.config.zoom_rescale,withSubchart:!1,withUpdateXDomain:!0,withDimension:!1}),a.config.subchart_onbrush.call(a.api,b.orgDomain())},i.transformContext=function(a,b){var c,d=this;b&&b.axisSubX?c=b.axisSubX:(c=d.context.select("."+l.axisX),a&&(c=c.transition())),d.context.attr("transform",d.getTranslate("context")),c.attr("transform",d.getTranslate("subx"))},i.getDefaultExtent=function(){var a=this,b=a.config,c=n(b.axis_x_extent)?b.axis_x_extent(a.getXDomain(a.data.targets)):b.axis_x_extent;return a.isTimeSeries()&&(c=[a.parseDate(c[0]),a.parseDate(c[1])]),c},i.initZoom=function(){var a,b=this,c=b.d3,d=b.config;b.zoom=c.behavior.zoom().on("zoomstart",function(){a=c.event.sourceEvent,b.zoom.altDomain=c.event.sourceEvent.altKey?b.x.orgDomain():null,d.zoom_onzoomstart.call(b.api,c.event.sourceEvent)}).on("zoom",function(){b.redrawForZoom.call(b)}).on("zoomend",function(){var e=c.event.sourceEvent;e&&a.clientX===e.clientX&&a.clientY===e.clientY||(b.redrawEventRect(),b.updateZoom(),d.zoom_onzoomend.call(b.api,b.x.orgDomain()))}),b.zoom.scale=function(a){return d.axis_rotated?this.y(a):this.x(a)},b.zoom.orgScaleExtent=function(){var a=d.zoom_extent?d.zoom_extent:[1,10];return[a[0],Math.max(b.getMaxDataCount()/a[1],a[1])]},b.zoom.updateScaleExtent=function(){var a=t(b.x.orgDomain())/t(b.getZoomDomain()),c=this.orgScaleExtent();return this.scaleExtent([c[0]*a,c[1]*a]),this}},i.getZoomDomain=function(){var a=this,b=a.config,c=a.d3,d=c.min([a.orgXDomain[0],b.zoom_x_min]),e=c.max([a.orgXDomain[1],b.zoom_x_max]);return[d,e]},i.updateZoom=function(){var a=this,b=a.config.zoom_enabled?a.zoom:function(){};a.main.select("."+l.zoomRect).call(b).on("dblclick.zoom",null),a.main.selectAll("."+l.eventRect).call(b).on("dblclick.zoom",null)},i.redrawForZoom=function(){var a=this,b=a.d3,c=a.config,d=a.zoom,e=a.x;if(c.zoom_enabled&&0!==a.filterTargetsToShow(a.data.targets).length){if("mousemove"===b.event.sourceEvent.type&&d.altDomain)return e.domain(d.altDomain),void d.scale(e).updateScaleExtent();a.isCategorized()&&e.orgDomain()[0]===a.orgXDomain[0]&&e.domain([a.orgXDomain[0]-1e-10,e.orgDomain()[1]]),a.redraw({withTransition:!1,withY:c.zoom_rescale,withSubchart:!1,withEventRect:!1,withDimension:!1}),"mousemove"===b.event.sourceEvent.type&&(a.cancelClick=!0),c.zoom_onzoom.call(a.api,e.orgDomain())}},i.generateColor=function(){var a=this,b=a.config,c=a.d3,d=b.data_colors,e=v(b.color_pattern)?b.color_pattern:c.scale.category10().range(),f=b.data_color,g=[];return function(a){var b,c=a.id||a.data&&a.data.id||a;return d[c]instanceof Function?b=d[c](a):d[c]?b=d[c]:(g.indexOf(c)<0&&g.push(c),b=e[g.indexOf(c)%e.length],d[c]=b),f instanceof Function?f(b,a):b}},i.generateLevelColor=function(){var a=this,b=a.config,c=b.color_pattern,d=b.color_threshold,e="value"===d.unit,f=d.values&&d.values.length?d.values:[],g=d.max||100;return v(b.color_threshold)?function(a){var b,d,h=c[c.length-1];for(b=0;b<f.length;b++)if(d=e?a:100*a/g,d<f[b]){h=c[b];break}return h}:null},i.getYFormat=function(a){var b=this,c=a&&!b.hasType("gauge")?b.defaultArcValueFormat:b.yFormat,d=a&&!b.hasType("gauge")?b.defaultArcValueFormat:b.y2Format;return function(a,e,f){var g="y2"===b.axis.getId(f)?d:c;return g.call(b,a,e)}},i.yFormat=function(a){var b=this,c=b.config,d=c.axis_y_tick_format?c.axis_y_tick_format:b.defaultValueFormat;return d(a)},i.y2Format=function(a){var b=this,c=b.config,d=c.axis_y2_tick_format?c.axis_y2_tick_format:b.defaultValueFormat;return d(a)},i.defaultValueFormat=function(a){return m(a)?+a:""},i.defaultArcValueFormat=function(a,b){return(100*b).toFixed(1)+"%"},i.dataLabelFormat=function(a){var b,c=this,d=c.config.data_labels,e=function(a){return m(a)?+a:""};return b="function"==typeof d.format?d.format:"object"==typeof d.format?d.format[a]?d.format[a]===!0?e:d.format[a]:function(){return""}:e},i.hasCaches=function(a){for(var b=0;b<a.length;b++)if(!(a[b]in this.cache))return!1;return!0},i.addCache=function(a,b){this.cache[a]=this.cloneTarget(b)},i.getCaches=function(a){var b,c=[];for(b=0;b<a.length;b++)a[b]in this.cache&&c.push(this.cloneTarget(this.cache[a[b]]));return c};var l=i.CLASS={target:"c3-target",chart:"c3-chart",chartLine:"c3-chart-line",chartLines:"c3-chart-lines",chartBar:"c3-chart-bar",chartBars:"c3-chart-bars",chartText:"c3-chart-text",chartTexts:"c3-chart-texts",chartArc:"c3-chart-arc",chartArcs:"c3-chart-arcs",chartArcsTitle:"c3-chart-arcs-title",chartArcsBackground:"c3-chart-arcs-background",chartArcsGaugeUnit:"c3-chart-arcs-gauge-unit",chartArcsGaugeMax:"c3-chart-arcs-gauge-max",chartArcsGaugeMin:"c3-chart-arcs-gauge-min",selectedCircle:"c3-selected-circle",selectedCircles:"c3-selected-circles",eventRect:"c3-event-rect",eventRects:"c3-event-rects",eventRectsSingle:"c3-event-rects-single",eventRectsMultiple:"c3-event-rects-multiple",zoomRect:"c3-zoom-rect",brush:"c3-brush",focused:"c3-focused",defocused:"c3-defocused",region:"c3-region",regions:"c3-regions",title:"c3-title",tooltipContainer:"c3-tooltip-container",tooltip:"c3-tooltip",tooltipName:"c3-tooltip-name",shape:"c3-shape",shapes:"c3-shapes",line:"c3-line",lines:"c3-lines",bar:"c3-bar",bars:"c3-bars",circle:"c3-circle",circles:"c3-circles",arc:"c3-arc",arcs:"c3-arcs",area:"c3-area",areas:"c3-areas",empty:"c3-empty",text:"c3-text",texts:"c3-texts",gaugeValue:"c3-gauge-value",grid:"c3-grid",gridLines:"c3-grid-lines",xgrid:"c3-xgrid",xgrids:"c3-xgrids",xgridLine:"c3-xgrid-line",xgridLines:"c3-xgrid-lines",xgridFocus:"c3-xgrid-focus",ygrid:"c3-ygrid",ygrids:"c3-ygrids",ygridLine:"c3-ygrid-line",ygridLines:"c3-ygrid-lines",axis:"c3-axis",axisX:"c3-axis-x",axisXLabel:"c3-axis-x-label",axisY:"c3-axis-y",axisYLabel:"c3-axis-y-label",axisY2:"c3-axis-y2",axisY2Label:"c3-axis-y2-label",legendBackground:"c3-legend-background",legendItem:"c3-legend-item",legendItemEvent:"c3-legend-item-event",legendItemTile:"c3-legend-item-tile",legendItemHidden:"c3-legend-item-hidden",legendItemFocused:"c3-legend-item-focused",dragarea:"c3-dragarea",EXPANDED:"_expanded_",SELECTED:"_selected_",INCLUDED:"_included_"};i.generateClass=function(a,b){return" "+a+" "+a+this.getTargetSelectorSuffix(b)},i.classText=function(a){return this.generateClass(l.text,a.index)},i.classTexts=function(a){return this.generateClass(l.texts,a.id)},i.classShape=function(a){return this.generateClass(l.shape,a.index)},i.classShapes=function(a){return this.generateClass(l.shapes,a.id)},i.classLine=function(a){return this.classShape(a)+this.generateClass(l.line,a.id)},i.classLines=function(a){return this.classShapes(a)+this.generateClass(l.lines,a.id)},i.classCircle=function(a){return this.classShape(a)+this.generateClass(l.circle,a.index)},i.classCircles=function(a){return this.classShapes(a)+this.generateClass(l.circles,a.id)},i.classBar=function(a){return this.classShape(a)+this.generateClass(l.bar,a.index)},i.classBars=function(a){return this.classShapes(a)+this.generateClass(l.bars,a.id)},i.classArc=function(a){return this.classShape(a.data)+this.generateClass(l.arc,a.data.id)},i.classArcs=function(a){return this.classShapes(a.data)+this.generateClass(l.arcs,a.data.id)},i.classArea=function(a){return this.classShape(a)+this.generateClass(l.area,a.id)},i.classAreas=function(a){return this.classShapes(a)+this.generateClass(l.areas,a.id)},i.classRegion=function(a,b){return this.generateClass(l.region,b)+" "+("class"in a?a["class"]:"")},i.classEvent=function(a){return this.generateClass(l.eventRect,a.index)},i.classTarget=function(a){var b=this,c=b.config.data_classes[a],d="";return c&&(d=" "+l.target+"-"+c),b.generateClass(l.target,a)+d},i.classFocus=function(a){return this.classFocused(a)+this.classDefocused(a)},i.classFocused=function(a){return" "+(this.focusedTargetIds.indexOf(a.id)>=0?l.focused:"")},i.classDefocused=function(a){return" "+(this.defocusedTargetIds.indexOf(a.id)>=0?l.defocused:"")},i.classChartText=function(a){return l.chartText+this.classTarget(a.id)},i.classChartLine=function(a){return l.chartLine+this.classTarget(a.id)},i.classChartBar=function(a){return l.chartBar+this.classTarget(a.id)},i.classChartArc=function(a){return l.chartArc+this.classTarget(a.data.id)},i.getTargetSelectorSuffix=function(a){return a||0===a?("-"+a).replace(/[\s?!@#$%^&*()_=+,.<>'":;\[\]\/|~`{}\\]/g,"-"):""},i.selectorTarget=function(a,b){return(b||"")+"."+l.target+this.getTargetSelectorSuffix(a)},i.selectorTargets=function(a,b){var c=this;return a=a||[],a.length?a.map(function(a){return c.selectorTarget(a,b)}):null},i.selectorLegend=function(a){return"."+l.legendItem+this.getTargetSelectorSuffix(a)},i.selectorLegends=function(a){var b=this;return a&&a.length?a.map(function(a){return b.selectorLegend(a)}):null};var m=i.isValue=function(a){return a||0===a},n=i.isFunction=function(a){return"function"==typeof a},o=i.isString=function(a){return"string"==typeof a},p=i.isUndefined=function(a){return"undefined"==typeof a},q=i.isDefined=function(a){return"undefined"!=typeof a},r=i.ceil10=function(a){return 10*Math.ceil(a/10)},s=i.asHalfPixel=function(a){return Math.ceil(a)+.5},t=i.diffDomain=function(a){return a[1]-a[0]},u=i.isEmpty=function(a){return"undefined"==typeof a||null===a||o(a)&&0===a.length||"object"==typeof a&&0===Object.keys(a).length},v=i.notEmpty=function(a){return!i.isEmpty(a)},w=i.getOption=function(a,b,c){return q(a[b])?a[b]:c},x=i.hasValue=function(a,b){var c=!1;return Object.keys(a).forEach(function(d){a[d]===b&&(c=!0)}),c},y=i.sanitise=function(a){return"string"==typeof a?a.replace(/</g,"&lt;").replace(/>/g,"&gt;"):a},z=i.getPathBox=function(a){var b=a.getBoundingClientRect(),c=[a.pathSegList.getItem(0),a.pathSegList.getItem(1)],d=c[0].x,e=Math.min(c[0].y,c[1].y);return{x:d,y:e,width:b.width,height:b.height}};h.focus=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a.filter(c.isTargetToShow,c))),this.revert(),this.defocus(),b.classed(l.focused,!0).classed(l.defocused,!1),
c.hasArcType()&&c.expandArc(a),c.toggleFocusLegend(a,!0),c.focusedTargetIds=a,c.defocusedTargetIds=c.defocusedTargetIds.filter(function(b){return a.indexOf(b)<0})},h.defocus=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a.filter(c.isTargetToShow,c))),b.classed(l.focused,!1).classed(l.defocused,!0),c.hasArcType()&&c.unexpandArc(a),c.toggleFocusLegend(a,!1),c.focusedTargetIds=c.focusedTargetIds.filter(function(b){return a.indexOf(b)<0}),c.defocusedTargetIds=a},h.revert=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a)),b.classed(l.focused,!1).classed(l.defocused,!1),c.hasArcType()&&c.unexpandArc(a),c.config.legend_show&&(c.showLegend(a.filter(c.isLegendToShow.bind(c))),c.legend.selectAll(c.selectorLegends(a)).filter(function(){return c.d3.select(this).classed(l.legendItemFocused)}).classed(l.legendItemFocused,!1)),c.focusedTargetIds=[],c.defocusedTargetIds=[]},h.show=function(a,b){var c,d=this.internal;a=d.mapToTargetIds(a),b=b||{},d.removeHiddenTargetIds(a),c=d.svg.selectAll(d.selectorTargets(a)),c.transition().style("opacity",1,"important").call(d.endall,function(){c.style("opacity",null).style("opacity",1)}),b.withLegend&&d.showLegend(a),d.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0})},h.hide=function(a,b){var c,d=this.internal;a=d.mapToTargetIds(a),b=b||{},d.addHiddenTargetIds(a),c=d.svg.selectAll(d.selectorTargets(a)),c.transition().style("opacity",0,"important").call(d.endall,function(){c.style("opacity",null).style("opacity",0)}),b.withLegend&&d.hideLegend(a),d.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0})},h.toggle=function(a,b){var c=this,d=this.internal;d.mapToTargetIds(a).forEach(function(a){d.isTargetToShow(a)?c.hide(a,b):c.show(a,b)})},h.zoom=function(a){var b=this.internal;return a&&(b.isTimeSeries()&&(a=a.map(function(a){return b.parseDate(a)})),b.brush.extent(a),b.redraw({withUpdateXDomain:!0,withY:b.config.zoom_rescale}),b.config.zoom_onzoom.call(this,b.x.orgDomain())),b.brush.extent()},h.zoom.enable=function(a){var b=this.internal;b.config.zoom_enabled=a,b.updateAndRedraw()},h.unzoom=function(){var a=this.internal;a.brush.clear().update(),a.redraw({withUpdateXDomain:!0})},h.zoom.max=function(a){var b=this.internal,c=b.config,d=b.d3;return 0===a||a?void(c.zoom_x_max=d.max([b.orgXDomain[1],a])):c.zoom_x_max},h.zoom.min=function(a){var b=this.internal,c=b.config,d=b.d3;return 0===a||a?void(c.zoom_x_min=d.min([b.orgXDomain[0],a])):c.zoom_x_min},h.zoom.range=function(a){return arguments.length?(q(a.max)&&this.domain.max(a.max),void(q(a.min)&&this.domain.min(a.min))):{max:this.domain.max(),min:this.domain.min()}},h.load=function(a){var b=this.internal,c=b.config;return a.xs&&b.addXs(a.xs),"names"in a&&h.data.names.bind(this)(a.names),"classes"in a&&Object.keys(a.classes).forEach(function(b){c.data_classes[b]=a.classes[b]}),"categories"in a&&b.isCategorized()&&(c.axis_x_categories=a.categories),"axes"in a&&Object.keys(a.axes).forEach(function(b){c.data_axes[b]=a.axes[b]}),"colors"in a&&Object.keys(a.colors).forEach(function(b){c.data_colors[b]=a.colors[b]}),"cacheIds"in a&&b.hasCaches(a.cacheIds)?void b.load(b.getCaches(a.cacheIds),a.done):void("unload"in a?b.unload(b.mapToTargetIds("boolean"==typeof a.unload&&a.unload?null:a.unload),function(){b.loadFromArgs(a)}):b.loadFromArgs(a))},h.unload=function(a){var b=this.internal;a=a||{},a instanceof Array?a={ids:a}:"string"==typeof a&&(a={ids:[a]}),b.unload(b.mapToTargetIds(a.ids),function(){b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0}),a.done&&a.done()})},h.flow=function(a){var b,c,d,e,f,g,h,i,j=this.internal,k=[],l=j.getMaxDataCount(),n=0,o=0;if(a.json)c=j.convertJsonToData(a.json,a.keys);else if(a.rows)c=j.convertRowsToData(a.rows);else{if(!a.columns)return;c=j.convertColumnsToData(a.columns)}b=j.convertDataToTargets(c,!0),j.data.targets.forEach(function(a){var c,d,e=!1;for(c=0;c<b.length;c++)if(a.id===b[c].id){for(e=!0,a.values[a.values.length-1]&&(o=a.values[a.values.length-1].index+1),n=b[c].values.length,d=0;n>d;d++)b[c].values[d].index=o+d,j.isTimeSeries()||(b[c].values[d].x=o+d);a.values=a.values.concat(b[c].values),b.splice(c,1);break}e||k.push(a.id)}),j.data.targets.forEach(function(a){var b,c;for(b=0;b<k.length;b++)if(a.id===k[b])for(o=a.values[a.values.length-1].index+1,c=0;n>c;c++)a.values.push({id:a.id,index:o+c,x:j.isTimeSeries()?j.getOtherTargetX(o+c):o+c,value:null})}),j.data.targets.length&&b.forEach(function(a){var b,c=[];for(b=j.data.targets[0].values[0].index;o>b;b++)c.push({id:a.id,index:b,x:j.isTimeSeries()?j.getOtherTargetX(b):b,value:null});a.values.forEach(function(a){a.index+=o,j.isTimeSeries()||(a.x+=o)}),a.values=c.concat(a.values)}),j.data.targets=j.data.targets.concat(b),d=j.getMaxDataCount(),f=j.data.targets[0],g=f.values[0],q(a.to)?(n=0,i=j.isTimeSeries()?j.parseDate(a.to):a.to,f.values.forEach(function(a){a.x<i&&n++})):q(a.length)&&(n=a.length),l?1===l&&j.isTimeSeries()&&(h=(f.values[f.values.length-1].x-g.x)/2,e=[new Date(+g.x-h),new Date(+g.x+h)],j.updateXDomain(null,!0,!0,!1,e)):(h=j.isTimeSeries()?f.values.length>1?f.values[f.values.length-1].x-g.x:g.x-j.getXDomain(j.data.targets)[0]:1,e=[g.x-h,g.x],j.updateXDomain(null,!0,!0,!1,e)),j.updateTargets(j.data.targets),j.redraw({flow:{index:g.index,length:n,duration:m(a.duration)?a.duration:j.config.transition_duration,done:a.done,orgDataCount:l},withLegend:!0,withTransition:l>1,withTrimXDomain:!1,withUpdateXAxis:!0})},i.generateFlow=function(a){var b=this,c=b.config,d=b.d3;return function(){var e,f,g,h=a.targets,i=a.flow,j=a.drawBar,k=a.drawLine,m=a.drawArea,n=a.cx,o=a.cy,p=a.xv,q=a.xForText,r=a.yForText,s=a.duration,u=1,v=i.index,w=i.length,x=b.getValueOnIndex(b.data.targets[0].values,v),y=b.getValueOnIndex(b.data.targets[0].values,v+w),z=b.x.domain(),A=i.duration||s,B=i.done||function(){},C=b.generateWait(),D=b.xgrid||d.selectAll([]),E=b.xgridLines||d.selectAll([]),F=b.mainRegion||d.selectAll([]),G=b.mainText||d.selectAll([]),H=b.mainBar||d.selectAll([]),I=b.mainLine||d.selectAll([]),J=b.mainArea||d.selectAll([]),K=b.mainCircle||d.selectAll([]);b.flowing=!0,b.data.targets.forEach(function(a){a.values.splice(0,w)}),g=b.updateXDomain(h,!0,!0),b.updateXGrid&&b.updateXGrid(!0),i.orgDataCount?e=1===i.orgDataCount||(x&&x.x)===(y&&y.x)?b.x(z[0])-b.x(g[0]):b.isTimeSeries()?b.x(z[0])-b.x(g[0]):b.x(x.x)-b.x(y.x):1!==b.data.targets[0].values.length?e=b.x(z[0])-b.x(g[0]):b.isTimeSeries()?(x=b.getValueOnIndex(b.data.targets[0].values,0),y=b.getValueOnIndex(b.data.targets[0].values,b.data.targets[0].values.length-1),e=b.x(x.x)-b.x(y.x)):e=t(g)/2,u=t(z)/t(g),f="translate("+e+",0) scale("+u+",1)",b.hideXGridFocus(),d.transition().ease("linear").duration(A).each(function(){C.add(b.axes.x.transition().call(b.xAxis)),C.add(H.transition().attr("transform",f)),C.add(I.transition().attr("transform",f)),C.add(J.transition().attr("transform",f)),C.add(K.transition().attr("transform",f)),C.add(G.transition().attr("transform",f)),C.add(F.filter(b.isRegionOnX).transition().attr("transform",f)),C.add(D.transition().attr("transform",f)),C.add(E.transition().attr("transform",f))}).call(C,function(){var a,d=[],e=[],f=[];if(w){for(a=0;w>a;a++)d.push("."+l.shape+"-"+(v+a)),e.push("."+l.text+"-"+(v+a)),f.push("."+l.eventRect+"-"+(v+a));b.svg.selectAll("."+l.shapes).selectAll(d).remove(),b.svg.selectAll("."+l.texts).selectAll(e).remove(),b.svg.selectAll("."+l.eventRects).selectAll(f).remove(),b.svg.select("."+l.xgrid).remove()}D.attr("transform",null).attr(b.xgridAttr),E.attr("transform",null),E.select("line").attr("x1",c.axis_rotated?0:p).attr("x2",c.axis_rotated?b.width:p),E.select("text").attr("x",c.axis_rotated?b.width:0).attr("y",p),H.attr("transform",null).attr("d",j),I.attr("transform",null).attr("d",k),J.attr("transform",null).attr("d",m),K.attr("transform",null).attr("cx",n).attr("cy",o),G.attr("transform",null).attr("x",q).attr("y",r).style("fill-opacity",b.opacityForText.bind(b)),F.attr("transform",null),F.select("rect").filter(b.isRegionOnX).attr("x",b.regionX.bind(b)).attr("width",b.regionWidth.bind(b)),c.interaction_enabled&&b.redrawEventRect(),B(),b.flowing=!1})}},h.selected=function(a){var b=this.internal,c=b.d3;return c.merge(b.main.selectAll("."+l.shapes+b.getTargetSelectorSuffix(a)).selectAll("."+l.shape).filter(function(){return c.select(this).classed(l.SELECTED)}).map(function(a){return a.map(function(a){var b=a.__data__;return b.data?b.data:b})}))},h.select=function(a,b,c){var d=this.internal,e=d.d3,f=d.config;f.data_selection_enabled&&d.main.selectAll("."+l.shapes).selectAll("."+l.shape).each(function(g,h){var i=e.select(this),j=g.data?g.data.id:g.id,k=d.getToggle(this,g).bind(d),m=f.data_selection_grouped||!a||a.indexOf(j)>=0,n=!b||b.indexOf(h)>=0,o=i.classed(l.SELECTED);i.classed(l.line)||i.classed(l.area)||(m&&n?f.data_selection_isselectable(g)&&!o&&k(!0,i.classed(l.SELECTED,!0),g,h):q(c)&&c&&o&&k(!1,i.classed(l.SELECTED,!1),g,h))})},h.unselect=function(a,b){var c=this.internal,d=c.d3,e=c.config;e.data_selection_enabled&&c.main.selectAll("."+l.shapes).selectAll("."+l.shape).each(function(f,g){var h=d.select(this),i=f.data?f.data.id:f.id,j=c.getToggle(this,f).bind(c),k=e.data_selection_grouped||!a||a.indexOf(i)>=0,m=!b||b.indexOf(g)>=0,n=h.classed(l.SELECTED);h.classed(l.line)||h.classed(l.area)||k&&m&&e.data_selection_isselectable(f)&&n&&j(!1,h.classed(l.SELECTED,!1),f,g)})},h.transform=function(a,b){var c=this.internal,d=["pie","donut"].indexOf(a)>=0?{withTransform:!0}:null;c.transformTo(b,a,d)},i.transformTo=function(a,b,c){var d=this,e=!d.hasArcType(),f=c||{withTransitionForAxis:e};f.withTransitionForTransform=!1,d.transiting=!1,d.setTargetType(a,b),d.updateTargets(d.data.targets),d.updateAndRedraw(f)},h.groups=function(a){var b=this.internal,c=b.config;return p(a)?c.data_groups:(c.data_groups=a,b.redraw(),c.data_groups)},h.xgrids=function(a){var b=this.internal,c=b.config;return a?(c.grid_x_lines=a,b.redrawWithoutRescale(),c.grid_x_lines):c.grid_x_lines},h.xgrids.add=function(a){var b=this.internal;return this.xgrids(b.config.grid_x_lines.concat(a?a:[]))},h.xgrids.remove=function(a){var b=this.internal;b.removeGridLines(a,!0)},h.ygrids=function(a){var b=this.internal,c=b.config;return a?(c.grid_y_lines=a,b.redrawWithoutRescale(),c.grid_y_lines):c.grid_y_lines},h.ygrids.add=function(a){var b=this.internal;return this.ygrids(b.config.grid_y_lines.concat(a?a:[]))},h.ygrids.remove=function(a){var b=this.internal;b.removeGridLines(a,!1)},h.regions=function(a){var b=this.internal,c=b.config;return a?(c.regions=a,b.redrawWithoutRescale(),c.regions):c.regions},h.regions.add=function(a){var b=this.internal,c=b.config;return a?(c.regions=c.regions.concat(a),b.redrawWithoutRescale(),c.regions):c.regions},h.regions.remove=function(a){var b,c,d,e=this.internal,f=e.config;return a=a||{},b=e.getOption(a,"duration",f.transition_duration),c=e.getOption(a,"classes",[l.region]),d=e.main.select("."+l.regions).selectAll(c.map(function(a){return"."+a})),(b?d.transition().duration(b):d).style("opacity",0).remove(),f.regions=f.regions.filter(function(a){var b=!1;return a["class"]?(a["class"].split(" ").forEach(function(a){c.indexOf(a)>=0&&(b=!0)}),!b):!0}),f.regions},h.data=function(a){var b=this.internal.data.targets;return"undefined"==typeof a?b:b.filter(function(b){return[].concat(a).indexOf(b.id)>=0})},h.data.shown=function(a){return this.internal.filterTargetsToShow(this.data(a))},h.data.values=function(a){var b,c=null;return a&&(b=this.data(a),c=b[0]?b[0].values.map(function(a){return a.value}):null),c},h.data.names=function(a){return this.internal.clearLegendItemTextBoxCache(),this.internal.updateDataAttributes("names",a)},h.data.colors=function(a){return this.internal.updateDataAttributes("colors",a)},h.data.axes=function(a){return this.internal.updateDataAttributes("axes",a)},h.category=function(a,b){var c=this.internal,d=c.config;return arguments.length>1&&(d.axis_x_categories[a]=b,c.redraw()),d.axis_x_categories[a]},h.categories=function(a){var b=this.internal,c=b.config;return arguments.length?(c.axis_x_categories=a,b.redraw(),c.axis_x_categories):c.axis_x_categories},h.color=function(a){var b=this.internal;return b.color(a)},h.x=function(a){var b=this.internal;return arguments.length&&(b.updateTargetX(b.data.targets,a),b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})),b.data.xs},h.xs=function(a){var b=this.internal;return arguments.length&&(b.updateTargetXs(b.data.targets,a),b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})),b.data.xs},h.axis=function(){},h.axis.labels=function(a){var b=this.internal;arguments.length&&(Object.keys(a).forEach(function(c){b.axis.setLabelText(c,a[c])}),b.axis.updateLabels())},h.axis.max=function(a){var b=this.internal,c=b.config;return arguments.length?("object"==typeof a?(m(a.x)&&(c.axis_x_max=a.x),m(a.y)&&(c.axis_y_max=a.y),m(a.y2)&&(c.axis_y2_max=a.y2)):c.axis_y_max=c.axis_y2_max=a,void b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})):{x:c.axis_x_max,y:c.axis_y_max,y2:c.axis_y2_max}},h.axis.min=function(a){var b=this.internal,c=b.config;return arguments.length?("object"==typeof a?(m(a.x)&&(c.axis_x_min=a.x),m(a.y)&&(c.axis_y_min=a.y),m(a.y2)&&(c.axis_y2_min=a.y2)):c.axis_y_min=c.axis_y2_min=a,void b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})):{x:c.axis_x_min,y:c.axis_y_min,y2:c.axis_y2_min}},h.axis.range=function(a){return arguments.length?(q(a.max)&&this.axis.max(a.max),void(q(a.min)&&this.axis.min(a.min))):{max:this.axis.max(),min:this.axis.min()}},h.legend=function(){},h.legend.show=function(a){var b=this.internal;b.showLegend(b.mapToTargetIds(a)),b.updateAndRedraw({withLegend:!0})},h.legend.hide=function(a){var b=this.internal;b.hideLegend(b.mapToTargetIds(a)),b.updateAndRedraw({withLegend:!0})},h.resize=function(a){var b=this.internal,c=b.config;c.size_width=a?a.width:null,c.size_height=a?a.height:null,this.flush()},h.flush=function(){var a=this.internal;a.updateAndRedraw({withLegend:!0,withTransition:!1,withTransitionForTransform:!1})},h.destroy=function(){var b=this.internal;if(a.clearInterval(b.intervalForObserveInserted),void 0!==b.resizeTimeout&&a.clearTimeout(b.resizeTimeout),a.detachEvent)a.detachEvent("onresize",b.resizeFunction);else if(a.removeEventListener)a.removeEventListener("resize",b.resizeFunction);else{var c=a.onresize;c&&c.add&&c.remove&&c.remove(b.resizeFunction)}return b.selectChart.classed("c3",!1).html(""),Object.keys(b).forEach(function(a){b[a]=null}),null},h.tooltip=function(){},h.tooltip.show=function(a){var b,c,d=this.internal;a.mouse&&(c=a.mouse),a.data?d.isMultipleX()?(c=[d.x(a.data.x),d.getYScale(a.data.id)(a.data.value)],b=null):b=m(a.data.index)?a.data.index:d.getIndexByX(a.data.x):"undefined"!=typeof a.x?b=d.getIndexByX(a.x):"undefined"!=typeof a.index&&(b=a.index),d.dispatchEvent("mouseover",b,c),d.dispatchEvent("mousemove",b,c),d.config.tooltip_onshow.call(d,a.data)},h.tooltip.hide=function(){this.internal.dispatchEvent("mouseout",0),this.internal.config.tooltip_onhide.call(this)};var A;i.isSafari=function(){var b=a.navigator.userAgent;return b.indexOf("Safari")>=0&&b.indexOf("Chrome")<0},i.isChrome=function(){var b=a.navigator.userAgent;return b.indexOf("Chrome")>=0},Function.prototype.bind||(Function.prototype.bind=function(a){if("function"!=typeof this)throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");var b=Array.prototype.slice.call(arguments,1),c=this,d=function(){},e=function(){return c.apply(this instanceof d?this:a,b.concat(Array.prototype.slice.call(arguments)))};return d.prototype=this.prototype,e.prototype=new d,e}),function(){"SVGPathSeg"in a||(a.SVGPathSeg=function(a,b,c){this.pathSegType=a,this.pathSegTypeAsLetter=b,this._owningPathSegList=c},SVGPathSeg.PATHSEG_UNKNOWN=0,SVGPathSeg.PATHSEG_CLOSEPATH=1,SVGPathSeg.PATHSEG_MOVETO_ABS=2,SVGPathSeg.PATHSEG_MOVETO_REL=3,SVGPathSeg.PATHSEG_LINETO_ABS=4,SVGPathSeg.PATHSEG_LINETO_REL=5,SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS=6,SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL=7,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS=8,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL=9,SVGPathSeg.PATHSEG_ARC_ABS=10,SVGPathSeg.PATHSEG_ARC_REL=11,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS=12,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL=13,SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS=14,SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL=15,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS=16,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL=17,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS=18,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL=19,SVGPathSeg.prototype._segmentChanged=function(){this._owningPathSegList&&this._owningPathSegList.segmentChanged(this)},a.SVGPathSegClosePath=function(a){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CLOSEPATH,"z",a)},SVGPathSegClosePath.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegClosePath.prototype.toString=function(){return"[object SVGPathSegClosePath]"},SVGPathSegClosePath.prototype._asPathString=function(){return this.pathSegTypeAsLetter},SVGPathSegClosePath.prototype.clone=function(){return new SVGPathSegClosePath(void 0)},a.SVGPathSegMovetoAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_MOVETO_ABS,"M",a),this._x=b,this._y=c},SVGPathSegMovetoAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegMovetoAbs.prototype.toString=function(){return"[object SVGPathSegMovetoAbs]"},SVGPathSegMovetoAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegMovetoAbs.prototype.clone=function(){return new SVGPathSegMovetoAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegMovetoAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegMovetoAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegMovetoRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_MOVETO_REL,"m",a),this._x=b,this._y=c},SVGPathSegMovetoRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegMovetoRel.prototype.toString=function(){return"[object SVGPathSegMovetoRel]"},SVGPathSegMovetoRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegMovetoRel.prototype.clone=function(){return new SVGPathSegMovetoRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegMovetoRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegMovetoRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_ABS,"L",a),this._x=b,this._y=c},SVGPathSegLinetoAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoAbs.prototype.toString=function(){return"[object SVGPathSegLinetoAbs]"},SVGPathSegLinetoAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegLinetoAbs.prototype.clone=function(){return new SVGPathSegLinetoAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegLinetoAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegLinetoAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_REL,"l",a),this._x=b,this._y=c},SVGPathSegLinetoRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoRel.prototype.toString=function(){return"[object SVGPathSegLinetoRel]"},SVGPathSegLinetoRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegLinetoRel.prototype.clone=function(){return new SVGPathSegLinetoRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegLinetoRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegLinetoRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicAbs=function(a,b,c,d,e,f,g){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS,"C",a),this._x=b,this._y=c,this._x1=d,this._y1=e,this._x2=f,this._y2=g},SVGPathSegCurvetoCubicAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicAbs]"},SVGPathSegCurvetoCubicAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicAbs.prototype.clone=function(){return new SVGPathSegCurvetoCubicAbs(void 0,this._x,this._y,this._x1,this._y1,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicRel=function(a,b,c,d,e,f,g){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL,"c",a),this._x=b,this._y=c,this._x1=d,this._y1=e,this._x2=f,this._y2=g},SVGPathSegCurvetoCubicRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicRel.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicRel]"},SVGPathSegCurvetoCubicRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicRel.prototype.clone=function(){return new SVGPathSegCurvetoCubicRel(void 0,this._x,this._y,this._x1,this._y1,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticAbs=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS,"Q",a),this._x=b,this._y=c,this._x1=d,this._y1=e},SVGPathSegCurvetoQuadraticAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticAbs]"},SVGPathSegCurvetoQuadraticAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticAbs.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticAbs(void 0,this._x,this._y,this._x1,this._y1)},Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticRel=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL,"q",a),this._x=b,this._y=c,this._x1=d,this._y1=e},SVGPathSegCurvetoQuadraticRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticRel.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticRel]"},SVGPathSegCurvetoQuadraticRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticRel.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticRel(void 0,this._x,this._y,this._x1,this._y1)},Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegArcAbs=function(a,b,c,d,e,f,g,h){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_ARC_ABS,"A",a),this._x=b,this._y=c,this._r1=d,this._r2=e,this._angle=f,this._largeArcFlag=g,this._sweepFlag=h},SVGPathSegArcAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegArcAbs.prototype.toString=function(){return"[object SVGPathSegArcAbs]"},SVGPathSegArcAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._r1+" "+this._r2+" "+this._angle+" "+(this._largeArcFlag?"1":"0")+" "+(this._sweepFlag?"1":"0")+" "+this._x+" "+this._y},SVGPathSegArcAbs.prototype.clone=function(){return new SVGPathSegArcAbs(void 0,this._x,this._y,this._r1,this._r2,this._angle,this._largeArcFlag,this._sweepFlag)},Object.defineProperty(SVGPathSegArcAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"r1",{get:function(){return this._r1},set:function(a){this._r1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"r2",{get:function(){return this._r2},set:function(a){this._r2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"angle",{get:function(){return this._angle},set:function(a){this._angle=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"largeArcFlag",{get:function(){return this._largeArcFlag},set:function(a){this._largeArcFlag=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"sweepFlag",{get:function(){return this._sweepFlag},set:function(a){this._sweepFlag=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegArcRel=function(a,b,c,d,e,f,g,h){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_ARC_REL,"a",a),this._x=b,this._y=c,this._r1=d,this._r2=e,this._angle=f,this._largeArcFlag=g,this._sweepFlag=h},SVGPathSegArcRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegArcRel.prototype.toString=function(){return"[object SVGPathSegArcRel]"},SVGPathSegArcRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._r1+" "+this._r2+" "+this._angle+" "+(this._largeArcFlag?"1":"0")+" "+(this._sweepFlag?"1":"0")+" "+this._x+" "+this._y},SVGPathSegArcRel.prototype.clone=function(){return new SVGPathSegArcRel(void 0,this._x,this._y,this._r1,this._r2,this._angle,this._largeArcFlag,this._sweepFlag)},Object.defineProperty(SVGPathSegArcRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"r1",{get:function(){return this._r1},set:function(a){this._r1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"r2",{get:function(){return this._r2},set:function(a){this._r2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"angle",{get:function(){return this._angle},set:function(a){this._angle=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"largeArcFlag",{get:function(){return this._largeArcFlag},set:function(a){this._largeArcFlag=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"sweepFlag",{get:function(){return this._sweepFlag},set:function(a){this._sweepFlag=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoHorizontalAbs=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS,"H",a),this._x=b},SVGPathSegLinetoHorizontalAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoHorizontalAbs.prototype.toString=function(){return"[object SVGPathSegLinetoHorizontalAbs]"},SVGPathSegLinetoHorizontalAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x},SVGPathSegLinetoHorizontalAbs.prototype.clone=function(){return new SVGPathSegLinetoHorizontalAbs(void 0,this._x)},Object.defineProperty(SVGPathSegLinetoHorizontalAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoHorizontalRel=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL,"h",a),this._x=b},SVGPathSegLinetoHorizontalRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoHorizontalRel.prototype.toString=function(){return"[object SVGPathSegLinetoHorizontalRel]"},SVGPathSegLinetoHorizontalRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x},SVGPathSegLinetoHorizontalRel.prototype.clone=function(){return new SVGPathSegLinetoHorizontalRel(void 0,this._x)},Object.defineProperty(SVGPathSegLinetoHorizontalRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoVerticalAbs=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS,"V",a),this._y=b},SVGPathSegLinetoVerticalAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoVerticalAbs.prototype.toString=function(){return"[object SVGPathSegLinetoVerticalAbs]"},SVGPathSegLinetoVerticalAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._y},SVGPathSegLinetoVerticalAbs.prototype.clone=function(){return new SVGPathSegLinetoVerticalAbs(void 0,this._y)},Object.defineProperty(SVGPathSegLinetoVerticalAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoVerticalRel=function(a,b){
SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL,"v",a),this._y=b},SVGPathSegLinetoVerticalRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoVerticalRel.prototype.toString=function(){return"[object SVGPathSegLinetoVerticalRel]"},SVGPathSegLinetoVerticalRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._y},SVGPathSegLinetoVerticalRel.prototype.clone=function(){return new SVGPathSegLinetoVerticalRel(void 0,this._y)},Object.defineProperty(SVGPathSegLinetoVerticalRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicSmoothAbs=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS,"S",a),this._x=b,this._y=c,this._x2=d,this._y2=e},SVGPathSegCurvetoCubicSmoothAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicSmoothAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicSmoothAbs]"},SVGPathSegCurvetoCubicSmoothAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicSmoothAbs.prototype.clone=function(){return new SVGPathSegCurvetoCubicSmoothAbs(void 0,this._x,this._y,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicSmoothRel=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL,"s",a),this._x=b,this._y=c,this._x2=d,this._y2=e},SVGPathSegCurvetoCubicSmoothRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicSmoothRel.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicSmoothRel]"},SVGPathSegCurvetoCubicSmoothRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicSmoothRel.prototype.clone=function(){return new SVGPathSegCurvetoCubicSmoothRel(void 0,this._x,this._y,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticSmoothAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS,"T",a),this._x=b,this._y=c},SVGPathSegCurvetoQuadraticSmoothAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticSmoothAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticSmoothAbs]"},SVGPathSegCurvetoQuadraticSmoothAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticSmoothAbs.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticSmoothAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticSmoothRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL,"t",a),this._x=b,this._y=c},SVGPathSegCurvetoQuadraticSmoothRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticSmoothRel.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticSmoothRel]"},SVGPathSegCurvetoQuadraticSmoothRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticSmoothRel.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticSmoothRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),SVGPathElement.prototype.createSVGPathSegClosePath=function(){return new SVGPathSegClosePath(void 0)},SVGPathElement.prototype.createSVGPathSegMovetoAbs=function(a,b){return new SVGPathSegMovetoAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegMovetoRel=function(a,b){return new SVGPathSegMovetoRel(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegLinetoAbs=function(a,b){return new SVGPathSegLinetoAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegLinetoRel=function(a,b){return new SVGPathSegLinetoRel(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicAbs=function(a,b,c,d,e,f){return new SVGPathSegCurvetoCubicAbs(void 0,a,b,c,d,e,f)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicRel=function(a,b,c,d,e,f){return new SVGPathSegCurvetoCubicRel(void 0,a,b,c,d,e,f)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticAbs=function(a,b,c,d){return new SVGPathSegCurvetoQuadraticAbs(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticRel=function(a,b,c,d){return new SVGPathSegCurvetoQuadraticRel(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegArcAbs=function(a,b,c,d,e,f,g){return new SVGPathSegArcAbs(void 0,a,b,c,d,e,f,g)},SVGPathElement.prototype.createSVGPathSegArcRel=function(a,b,c,d,e,f,g){return new SVGPathSegArcRel(void 0,a,b,c,d,e,f,g)},SVGPathElement.prototype.createSVGPathSegLinetoHorizontalAbs=function(a){return new SVGPathSegLinetoHorizontalAbs(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoHorizontalRel=function(a){return new SVGPathSegLinetoHorizontalRel(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoVerticalAbs=function(a){return new SVGPathSegLinetoVerticalAbs(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoVerticalRel=function(a){return new SVGPathSegLinetoVerticalRel(void 0,a)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicSmoothAbs=function(a,b,c,d){return new SVGPathSegCurvetoCubicSmoothAbs(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicSmoothRel=function(a,b,c,d){return new SVGPathSegCurvetoCubicSmoothRel(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticSmoothAbs=function(a,b){return new SVGPathSegCurvetoQuadraticSmoothAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticSmoothRel=function(a,b){return new SVGPathSegCurvetoQuadraticSmoothRel(void 0,a,b)}),"SVGPathSegList"in a||(a.SVGPathSegList=function(a){this._pathElement=a,this._list=this._parsePath(this._pathElement.getAttribute("d")),this._mutationObserverConfig={attributes:!0,attributeFilter:["d"]},this._pathElementMutationObserver=new MutationObserver(this._updateListFromPathMutations.bind(this)),this._pathElementMutationObserver.observe(this._pathElement,this._mutationObserverConfig)},Object.defineProperty(SVGPathSegList.prototype,"numberOfItems",{get:function(){return this._checkPathSynchronizedToList(),this._list.length},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"pathSegList",{get:function(){return this._pathSegList||(this._pathSegList=new SVGPathSegList(this)),this._pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"normalizedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"animatedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"animatedNormalizedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),SVGPathSegList.prototype._checkPathSynchronizedToList=function(){this._updateListFromPathMutations(this._pathElementMutationObserver.takeRecords())},SVGPathSegList.prototype._updateListFromPathMutations=function(a){if(this._pathElement){var b=!1;a.forEach(function(a){"d"==a.attributeName&&(b=!0)}),b&&(this._list=this._parsePath(this._pathElement.getAttribute("d")))}},SVGPathSegList.prototype._writeListToPath=function(){this._pathElementMutationObserver.disconnect(),this._pathElement.setAttribute("d",SVGPathSegList._pathSegArrayAsString(this._list)),this._pathElementMutationObserver.observe(this._pathElement,this._mutationObserverConfig)},SVGPathSegList.prototype.segmentChanged=function(a){this._writeListToPath()},SVGPathSegList.prototype.clear=function(){this._checkPathSynchronizedToList(),this._list.forEach(function(a){a._owningPathSegList=null}),this._list=[],this._writeListToPath()},SVGPathSegList.prototype.initialize=function(a){return this._checkPathSynchronizedToList(),this._list=[a],a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype._checkValidIndex=function(a){if(isNaN(a)||0>a||a>=this.numberOfItems)throw"INDEX_SIZE_ERR"},SVGPathSegList.prototype.getItem=function(a){return this._checkPathSynchronizedToList(),this._checkValidIndex(a),this._list[a]},SVGPathSegList.prototype.insertItemBefore=function(a,b){return this._checkPathSynchronizedToList(),b>this.numberOfItems&&(b=this.numberOfItems),a._owningPathSegList&&(a=a.clone()),this._list.splice(b,0,a),a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype.replaceItem=function(a,b){return this._checkPathSynchronizedToList(),a._owningPathSegList&&(a=a.clone()),this._checkValidIndex(b),this._list[b]=a,a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype.removeItem=function(a){this._checkPathSynchronizedToList(),this._checkValidIndex(a);var b=this._list[a];return this._list.splice(a,1),this._writeListToPath(),b},SVGPathSegList.prototype.appendItem=function(a){return this._checkPathSynchronizedToList(),a._owningPathSegList&&(a=a.clone()),this._list.push(a),a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList._pathSegArrayAsString=function(a){var b="",c=!0;return a.forEach(function(a){c?(c=!1,b+=a._asPathString()):b+=" "+a._asPathString()}),b},SVGPathSegList.prototype._parsePath=function(a){if(!a||0==a.length)return[];var b=this,c=function(){this.pathSegList=[]};c.prototype.appendSegment=function(a){this.pathSegList.push(a)};var d=function(a){this._string=a,this._currentIndex=0,this._endIndex=this._string.length,this._previousCommand=SVGPathSeg.PATHSEG_UNKNOWN,this._skipOptionalSpaces()};d.prototype._isCurrentSpace=function(){var a=this._string[this._currentIndex];return" ">=a&&(" "==a||"\n"==a||"	"==a||"\r"==a||"\f"==a)},d.prototype._skipOptionalSpaces=function(){for(;this._currentIndex<this._endIndex&&this._isCurrentSpace();)this._currentIndex++;return this._currentIndex<this._endIndex},d.prototype._skipOptionalSpacesOrDelimiter=function(){return this._currentIndex<this._endIndex&&!this._isCurrentSpace()&&","!=this._string.charAt(this._currentIndex)?!1:(this._skipOptionalSpaces()&&this._currentIndex<this._endIndex&&","==this._string.charAt(this._currentIndex)&&(this._currentIndex++,this._skipOptionalSpaces()),this._currentIndex<this._endIndex)},d.prototype.hasMoreData=function(){return this._currentIndex<this._endIndex},d.prototype.peekSegmentType=function(){var a=this._string[this._currentIndex];return this._pathSegTypeFromChar(a)},d.prototype._pathSegTypeFromChar=function(a){switch(a){case"Z":case"z":return SVGPathSeg.PATHSEG_CLOSEPATH;case"M":return SVGPathSeg.PATHSEG_MOVETO_ABS;case"m":return SVGPathSeg.PATHSEG_MOVETO_REL;case"L":return SVGPathSeg.PATHSEG_LINETO_ABS;case"l":return SVGPathSeg.PATHSEG_LINETO_REL;case"C":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS;case"c":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL;case"Q":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS;case"q":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL;case"A":return SVGPathSeg.PATHSEG_ARC_ABS;case"a":return SVGPathSeg.PATHSEG_ARC_REL;case"H":return SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS;case"h":return SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL;case"V":return SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS;case"v":return SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL;case"S":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS;case"s":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL;case"T":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS;case"t":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL;default:return SVGPathSeg.PATHSEG_UNKNOWN}},d.prototype._nextCommandHelper=function(a,b){return("+"==a||"-"==a||"."==a||a>="0"&&"9">=a)&&b!=SVGPathSeg.PATHSEG_CLOSEPATH?b==SVGPathSeg.PATHSEG_MOVETO_ABS?SVGPathSeg.PATHSEG_LINETO_ABS:b==SVGPathSeg.PATHSEG_MOVETO_REL?SVGPathSeg.PATHSEG_LINETO_REL:b:SVGPathSeg.PATHSEG_UNKNOWN},d.prototype.initialCommandIsMoveTo=function(){if(!this.hasMoreData())return!0;var a=this.peekSegmentType();return a==SVGPathSeg.PATHSEG_MOVETO_ABS||a==SVGPathSeg.PATHSEG_MOVETO_REL},d.prototype._parseNumber=function(){var a=0,b=0,c=1,d=0,e=1,f=1,g=this._currentIndex;if(this._skipOptionalSpaces(),this._currentIndex<this._endIndex&&"+"==this._string.charAt(this._currentIndex)?this._currentIndex++:this._currentIndex<this._endIndex&&"-"==this._string.charAt(this._currentIndex)&&(this._currentIndex++,e=-1),!(this._currentIndex==this._endIndex||(this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")&&"."!=this._string.charAt(this._currentIndex))){for(var h=this._currentIndex;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)this._currentIndex++;if(this._currentIndex!=h)for(var i=this._currentIndex-1,j=1;i>=h;)b+=j*(this._string.charAt(i--)-"0"),j*=10;if(this._currentIndex<this._endIndex&&"."==this._string.charAt(this._currentIndex)){if(this._currentIndex++,this._currentIndex>=this._endIndex||this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")return;for(;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)d+=(this._string.charAt(this._currentIndex++)-"0")*(c*=.1)}if(this._currentIndex!=g&&this._currentIndex+1<this._endIndex&&("e"==this._string.charAt(this._currentIndex)||"E"==this._string.charAt(this._currentIndex))&&"x"!=this._string.charAt(this._currentIndex+1)&&"m"!=this._string.charAt(this._currentIndex+1)){if(this._currentIndex++,"+"==this._string.charAt(this._currentIndex)?this._currentIndex++:"-"==this._string.charAt(this._currentIndex)&&(this._currentIndex++,f=-1),this._currentIndex>=this._endIndex||this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")return;for(;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)a*=10,a+=this._string.charAt(this._currentIndex)-"0",this._currentIndex++}var k=b+d;if(k*=e,a&&(k*=Math.pow(10,f*a)),g!=this._currentIndex)return this._skipOptionalSpacesOrDelimiter(),k}},d.prototype._parseArcFlag=function(){if(!(this._currentIndex>=this._endIndex)){var a=!1,b=this._string.charAt(this._currentIndex++);if("0"==b)a=!1;else{if("1"!=b)return;a=!0}return this._skipOptionalSpacesOrDelimiter(),a}},d.prototype.parseSegment=function(){var a=this._string[this._currentIndex],c=this._pathSegTypeFromChar(a);if(c==SVGPathSeg.PATHSEG_UNKNOWN){if(this._previousCommand==SVGPathSeg.PATHSEG_UNKNOWN)return null;if(c=this._nextCommandHelper(a,this._previousCommand),c==SVGPathSeg.PATHSEG_UNKNOWN)return null}else this._currentIndex++;switch(this._previousCommand=c,c){case SVGPathSeg.PATHSEG_MOVETO_REL:return new SVGPathSegMovetoRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_MOVETO_ABS:return new SVGPathSegMovetoAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_REL:return new SVGPathSegLinetoRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_ABS:return new SVGPathSegLinetoAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL:return new SVGPathSegLinetoHorizontalRel(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS:return new SVGPathSegLinetoHorizontalAbs(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL:return new SVGPathSegLinetoVerticalRel(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS:return new SVGPathSegLinetoVerticalAbs(b,this._parseNumber());case SVGPathSeg.PATHSEG_CLOSEPATH:return this._skipOptionalSpaces(),new SVGPathSegClosePath(b);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicRel(b,d.x,d.y,d.x1,d.y1,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicAbs(b,d.x,d.y,d.x1,d.y1,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL:var d={x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicSmoothRel(b,d.x,d.y,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:var d={x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicSmoothAbs(b,d.x,d.y,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoQuadraticRel(b,d.x,d.y,d.x1,d.y1);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoQuadraticAbs(b,d.x,d.y,d.x1,d.y1);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:return new SVGPathSegCurvetoQuadraticSmoothRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:return new SVGPathSegCurvetoQuadraticSmoothAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_ARC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),arcAngle:this._parseNumber(),arcLarge:this._parseArcFlag(),arcSweep:this._parseArcFlag(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegArcRel(b,d.x,d.y,d.x1,d.y1,d.arcAngle,d.arcLarge,d.arcSweep);case SVGPathSeg.PATHSEG_ARC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),arcAngle:this._parseNumber(),arcLarge:this._parseArcFlag(),arcSweep:this._parseArcFlag(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegArcAbs(b,d.x,d.y,d.x1,d.y1,d.arcAngle,d.arcLarge,d.arcSweep);default:throw"Unknown path seg type."}};var e=new c,f=new d(a);if(!f.initialCommandIsMoveTo())return[];for(;f.hasMoreData();){var g=f.parseSegment();if(!g)return[];e.appendSegment(g)}return e.pathSegList})}(),"function"==typeof define&&define.amd?define("c3",["d3"],function(){return k}):"undefined"!=typeof exports&&"undefined"!=typeof module?module.exports=k:a.c3=k}(window);
;(function(){
var f,aa=aa||{},ba=this;function ca(a){return void 0!==a}function da(a){return"string"==typeof a}function ea(a){return"number"==typeof a}function fa(a,b){var c=a.split("."),d=ba;c[0]in d||!d.execScript||d.execScript("var "+c[0]);for(var e;c.length&&(e=c.shift());)!c.length&&ca(b)?d[e]=b:d=d[e]&&d[e]!==Object.prototype[e]?d[e]:d[e]={}}function ha(a){a=a.split(".");for(var b=ba,c;c=a.shift();)if(null!=b[c])b=b[c];else return null;return b}function ja(){}
function ka(a){a.rn=void 0;a.va=function(){return a.rn?a.rn:a.rn=new a}}
function n(a){var b=typeof a;if("object"==b)if(a){if(a instanceof Array)return"array";if(a instanceof Object)return b;var c=Object.prototype.toString.call(a);if("[object Window]"==c)return"object";if("[object Array]"==c||"number"==typeof a.length&&"undefined"!=typeof a.splice&&"undefined"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable("splice"))return"array";if("[object Function]"==c||"undefined"!=typeof a.call&&"undefined"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable("call"))return"function"}else return"null";
else if("function"==b&&"undefined"==typeof a.call)return"object";return b}function la(a){return"array"==n(a)}function ma(a){var b=n(a);return"array"==b||"object"==b&&"number"==typeof a.length}function oa(a){return"function"==n(a)}function pa(a){var b=typeof a;return"object"==b&&null!=a||"function"==b}function qa(a){return a[ra]||(a[ra]=++sa)}var ra="closure_uid_"+(1E9*Math.random()>>>0),sa=0;function ta(a,b,c){return a.call.apply(a.bind,arguments)}
function ua(a,b,c){if(!a)throw Error();if(2<arguments.length){var d=Array.prototype.slice.call(arguments,2);return function(){var c=Array.prototype.slice.call(arguments);Array.prototype.unshift.apply(c,d);return a.apply(b,c)}}return function(){return a.apply(b,arguments)}}function q(a,b,c){q=Function.prototype.bind&&-1!=Function.prototype.bind.toString().indexOf("native code")?ta:ua;return q.apply(null,arguments)}
function va(a,b){var c=Array.prototype.slice.call(arguments,1);return function(){var b=c.slice();b.push.apply(b,arguments);return a.apply(this,b)}}var wa=Date.now||function(){return+new Date};function r(a,b){function c(){}c.prototype=b.prototype;a.m=b.prototype;a.prototype=new c;a.prototype.constructor=a;a.base=function(a,c,g){for(var d=Array(arguments.length-2),e=2;e<arguments.length;e++)d[e-2]=arguments[e];return b.prototype[c].apply(a,d)}};function xa(a,b){this.jg=a;this.ki=b;this.tc=q(this.tc,this);this.filters={}}xa.prototype.register=function(a,b){return this.filters[a]=b};xa.prototype.tc=function(a){var b,c,d;var e=this.filters;var g=[];for(d in e){var h=e[d];g.push(function(){var e;var g=a.querySelectorAll(d);var m=[];var p=0;for(e=g.length;p<e;p++){b=g[p];try{m.push(this.jg(h,b))}catch(u){c=u,m.push(this.ki.log(c.message,String(c.stack)))}}return m}.call(this))}return g};var ya;function za(a,b){return 0==a.lastIndexOf(b,0)}function Aa(a,b){var c=a.length-b.length;return 0<=c&&a.indexOf(b,c)==c}function Ba(a){return/^[\s\xa0]*$/.test(a)}function Ca(a){return Ba(null==a?"":String(a))}function Da(a){return a.replace(/[\t\r\n ]+/g," ").replace(/^[\t\r\n ]+|[\t\r\n ]+$/g,"")}var Ea=String.prototype.trim?function(a){return a.trim()}:function(a){return a.replace(/^[\s\xa0]+|[\s\xa0]+$/g,"")};function Fa(a){return decodeURIComponent(a.replace(/\+/g," "))}
function Ga(a,b){return a.replace(/(\r\n|\r|\n)/g,b?"\x3cbr /\x3e":"\x3cbr\x3e")}function Ha(a){if(!Ia.test(a))return a;-1!=a.indexOf("\x26")&&(a=a.replace(Ja,"\x26amp;"));-1!=a.indexOf("\x3c")&&(a=a.replace(Ka,"\x26lt;"));-1!=a.indexOf("\x3e")&&(a=a.replace(La,"\x26gt;"));-1!=a.indexOf('"')&&(a=a.replace(Ma,"\x26quot;"));-1!=a.indexOf("'")&&(a=a.replace(Na,"\x26#39;"));-1!=a.indexOf("\x00")&&(a=a.replace(Oa,"\x26#0;"));return a}var Ja=/&/g,Ka=/</g,La=/>/g,Ma=/"/g,Na=/'/g,Oa=/\x00/g,Ia=/[\x00&<>"']/;
function Pa(a){return Qa(a,"\x26")?"document"in ba?Ra(a):Sa(a):a}function Ra(a){var b={"\x26amp;":"\x26","\x26lt;":"\x3c","\x26gt;":"\x3e","\x26quot;":'"'};var c=ba.document.createElement("div");return a.replace(Ta,function(a,e){var d=b[a];if(d)return d;if("#"==e.charAt(0)){var h=Number("0"+e.substr(1));isNaN(h)||(d=String.fromCharCode(h))}d||(c.innerHTML=a+" ",d=c.firstChild.nodeValue.slice(0,-1));return b[a]=d})}
function Sa(a){return a.replace(/&([^;]+);/g,function(a,c){switch(c){case "amp":return"\x26";case "lt":return"\x3c";case "gt":return"\x3e";case "quot":return'"';default:if("#"==c.charAt(0)){var b=Number("0"+c.substr(1));if(!isNaN(b))return String.fromCharCode(b)}return a}})}var Ta=/&([^;\s<&]+);?/g;function Qa(a,b){return-1!=a.indexOf(b)}function Ua(a){return String(a).replace(/([-()\[\]{}+?*.$\^|,:#<!\\])/g,"\\$1").replace(/\x08/g,"\\x08")}
var Va=String.prototype.repeat?function(a,b){return a.repeat(b)}:function(a,b){return Array(b+1).join(a)};function Wa(a,b){var c=ca(void 0)?a.toFixed(void 0):String(a),d=c.indexOf(".");-1==d&&(d=c.length);return Va("0",Math.max(0,b-d))+c}function Xa(a,b){return a<b?-1:a>b?1:0}function Za(a){return String(a).replace(/\-([a-z])/g,function(a,c){return c.toUpperCase()})}function $a(a){return String(a).replace(/([A-Z])/g,"-$1").toLowerCase()}
function ab(a){var b=da(void 0)?Ua(void 0):"\\s";return a.replace(new RegExp("(^"+(b?"|["+b+"]+":"")+")([a-z])","g"),function(a,b,e){return b+e.toUpperCase()})};function bb(a){return a}function cb(a,b){var c=arguments,d=c.length;return function(){var a;d&&(a=c[d-1].apply(this,arguments));for(var b=d-2;0<=b;b--)a=c[b].call(this,a);return a}};var db=Array.prototype.indexOf?function(a,b,c){return Array.prototype.indexOf.call(a,b,c)}:function(a,b,c){c=null==c?0:0>c?Math.max(0,a.length+c):c;if(da(a))return da(b)&&1==b.length?a.indexOf(b,c):-1;for(;c<a.length;c++)if(c in a&&a[c]===b)return c;return-1},eb=Array.prototype.forEach?function(a,b,c){Array.prototype.forEach.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=da(a)?a.split(""):a,g=0;g<d;g++)g in e&&b.call(c,e[g],g,a)},gb=Array.prototype.filter?function(a,b,c){return Array.prototype.filter.call(a,
b,c)}:function(a,b,c){for(var d=a.length,e=[],g=0,h=da(a)?a.split(""):a,k=0;k<d;k++)if(k in h){var l=h[k];b.call(c,l,k,a)&&(e[g++]=l)}return e},hb=Array.prototype.map?function(a,b,c){return Array.prototype.map.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=Array(d),g=da(a)?a.split(""):a,h=0;h<d;h++)h in g&&(e[h]=b.call(c,g[h],h,a));return e},ib=Array.prototype.reduce?function(a,b,c,d){d&&(b=q(b,d));return Array.prototype.reduce.call(a,b,c)}:function(a,b,c,d){var e=c;eb(a,function(c,h){e=b.call(d,
e,c,h,a)});return e},jb=Array.prototype.some?function(a,b,c){return Array.prototype.some.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=da(a)?a.split(""):a,g=0;g<d;g++)if(g in e&&b.call(c,e[g],g,a))return!0;return!1},kb=Array.prototype.every?function(a,b,c){return Array.prototype.every.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=da(a)?a.split(""):a,g=0;g<d;g++)if(g in e&&!b.call(c,e[g],g,a))return!1;return!0};function lb(a,b){var c=mb(a,b);return 0>c?null:da(a)?a.charAt(c):a[c]}
function mb(a,b){for(var c=a.length,d=da(a)?a.split(""):a,e=0;e<c;e++)if(e in d&&b.call(void 0,d[e],e,a))return e;return-1}function nb(a,b){return 0<=db(a,b)}function ob(a,b){var c=db(a,b),d;(d=0<=c)&&Array.prototype.splice.call(a,c,1);return d}function qb(a,b){var c=mb(a,b);return 0<=c?(Array.prototype.splice.call(a,c,1),!0):!1}function rb(a){return Array.prototype.concat.apply([],arguments)}function sb(a){var b=a.length;if(0<b){for(var c=Array(b),d=0;d<b;d++)c[d]=a[d];return c}return[]}
function tb(a,b,c,d){Array.prototype.splice.apply(a,ub(arguments,1))}function ub(a,b,c){return 2>=arguments.length?Array.prototype.slice.call(a,b):Array.prototype.slice.call(a,b,c)}function xb(a){for(var b={},c=0,d=0;d<a.length;){var e=a[d++];var g=e;g=pa(g)?"o"+qa(g):(typeof g).charAt(0)+g;Object.prototype.hasOwnProperty.call(b,g)||(b[g]=!0,a[c++]=e)}a.length=c}function yb(a,b){var c=[],d=0,e=a;void 0!==b&&(d=a,e=b);if(0>1*(e-d))return[];for(;d<e;d+=1)c.push(d);return c};var zb;a:{var Ab=ba.navigator;if(Ab){var Bb=Ab.userAgent;if(Bb){zb=Bb;break a}}zb=""}function Cb(a){return Qa(zb,a)};function Db(a,b,c){for(var d in a)b.call(c,a[d],d,a)}function Fb(a,b){for(var c in a)if(b.call(void 0,a[c],c,a))return!0;return!1}function Gb(a){var b=0,c;for(c in a)b++;return b}function Hb(a){var b=[],c=0,d;for(d in a)b[c++]=a[d];return b}function Ib(a){var b=[],c=0,d;for(d in a)b[c++]=d;return b}function Jb(a,b){return null!==a&&b in a}function Kb(){var a=Lb,b;for(b in a)return!1;return!0}function Mb(a,b){b in a&&delete a[b]}
function Nb(a,b,c){if(null!==a&&b in a)throw Error('The object already contains the key "'+b+'"');a[b]=c}function Ob(a,b,c){return null!==a&&b in a?a[b]:c}var Pb="constructor hasOwnProperty isPrototypeOf propertyIsEnumerable toLocaleString toString valueOf".split(" ");function Qb(a,b){for(var c,d,e=1;e<arguments.length;e++){d=arguments[e];for(c in d)a[c]=d[c];for(var g=0;g<Pb.length;g++)c=Pb[g],Object.prototype.hasOwnProperty.call(d,c)&&(a[c]=d[c])}}
function Rb(a){var b=arguments.length;if(1==b&&la(arguments[0]))return Rb.apply(null,arguments[0]);for(var c={},d=0;d<b;d++)c[arguments[d]]=!0;return c};function Sb(){return Cb("Safari")&&!(Vb()||Cb("Coast")||Cb("Opera")||Cb("Edge")||Cb("Silk")||Cb("Android"))}function Vb(){return(Cb("Chrome")||Cb("CriOS"))&&!Cb("Edge")};function Wb(a,b,c){var d=a;b&&(d=q(a,b));d=Wb.qw(d);oa(ba.setImmediate)&&(c||Wb.lw())?ba.setImmediate(d):(Wb.Zr||(Wb.Zr=Wb.Hu()),Wb.Zr(d))}Wb.lw=function(){return ba.Window&&ba.Window.prototype&&!Cb("Edge")&&ba.Window.prototype.setImmediate==ba.setImmediate?!1:!0};
Wb.Hu=function(){var a=ba.MessageChannel;"undefined"===typeof a&&"undefined"!==typeof window&&window.postMessage&&window.addEventListener&&!Cb("Presto")&&(a=function(){var a=document.createElement("IFRAME");a.style.display="none";a.src="";document.documentElement.appendChild(a);var b=a.contentWindow,a=b.document;a.open();a.write("");a.close();var c="callImmediate"+Math.random(),d="file:"==b.location.protocol?"*":b.location.protocol+"//"+b.location.host,a=q(function(a){if(("*"==d||a.origin==d)&&a.data==
c)this.port1.onmessage()},this);b.addEventListener("message",a,!1);this.port1={};this.port2={postMessage:function(){b.postMessage(c,d)}}});if("undefined"!==typeof a&&!Cb("Trident")&&!Cb("MSIE")){var b=new a,c={},d=c;b.port1.onmessage=function(){if(ca(c.next)){c=c.next;var a=c.Mp;c.Mp=null;a()}};return function(a){d.next={Mp:a};d=d.next;b.port2.postMessage(0)}}return"undefined"!==typeof document&&"onreadystatechange"in document.createElement("SCRIPT")?function(a){var b=document.createElement("SCRIPT");
b.onreadystatechange=function(){b.onreadystatechange=null;b.parentNode.removeChild(b);b=null;a();a=null};document.documentElement.appendChild(b)}:function(a){ba.setTimeout(a,0)}};Wb.qw=bb;function Xb(){0!=Yb&&(Zb[qa(this)]=this);this.Ee=this.Ee;this.Tf=this.Tf}var Yb=0,Zb={};Xb.prototype.Ee=!1;Xb.prototype.eb=function(){if(!this.Ee&&(this.Ee=!0,this.V(),0!=Yb)){var a=qa(this);delete Zb[a]}};function $b(a,b){a.Ee?ca(void 0)?b.call(void 0):b():(a.Tf||(a.Tf=[]),a.Tf.push(ca(void 0)?q(b,void 0):b))}Xb.prototype.V=function(){if(this.Tf)for(;this.Tf.length;)this.Tf.shift()()};function ac(a){a&&"function"==typeof a.eb&&a.eb()};function bc(){return Cb("iPhone")&&!Cb("iPod")&&!Cb("iPad")}function cc(){bc()||Cb("iPad")||Cb("iPod")};function dc(a){dc[" "](a);return a}dc[" "]=ja;function ec(a,b){try{return dc(a[b]),!0}catch(c){}return!1}function fc(a,b,c){return Object.prototype.hasOwnProperty.call(a,b)?a[b]:a[b]=c(b)};var gc=Cb("Opera"),t=Cb("Trident")||Cb("MSIE"),hc=Cb("Edge"),ic=hc||t,jc=Cb("Gecko")&&!(Qa(zb.toLowerCase(),"webkit")&&!Cb("Edge"))&&!(Cb("Trident")||Cb("MSIE"))&&!Cb("Edge"),kc=Qa(zb.toLowerCase(),"webkit")&&!Cb("Edge");kc&&Cb("Mobile");var mc=Cb("Macintosh");Cb("Windows");Cb("Linux")||Cb("CrOS");var nc=ba.navigator||null;nc&&Qa(nc.appVersion||"","X11");Cb("Android");bc();Cb("iPad");Cb("iPod");cc();function oc(){var a=ba.document;return a?a.documentMode:void 0}var pc;
a:{var qc="",rc=function(){var a=zb;if(jc)return/rv\:([^\);]+)(\)|;)/.exec(a);if(hc)return/Edge\/([\d\.]+)/.exec(a);if(t)return/\b(?:MSIE|rv)[: ]([^\);]+)(\)|;)/.exec(a);if(kc)return/WebKit\/(\S+)/.exec(a);if(gc)return/(?:Version)[ \/]?(\S+)/.exec(a)}();rc&&(qc=rc?rc[1]:"");if(t){var sc=oc();if(null!=sc&&sc>parseFloat(qc)){pc=String(sc);break a}}pc=qc}var tc={};
function uc(a){return fc(tc,a,function(){for(var b=0,c=Ea(String(pc)).split("."),d=Ea(String(a)).split("."),e=Math.max(c.length,d.length),g=0;0==b&&g<e;g++){var h=c[g]||"",k=d[g]||"";do{h=/(\d*)(\D*)(.*)/.exec(h)||["","","",""];k=/(\d*)(\D*)(.*)/.exec(k)||["","","",""];if(0==h[0].length&&0==k[0].length)break;b=Xa(0==h[1].length?0:parseInt(h[1],10),0==k[1].length?0:parseInt(k[1],10))||Xa(0==h[2].length,0==k[2].length)||Xa(h[2],k[2]);h=h[3];k=k[3]}while(0==b)}return 0<=b})}
function vc(a){return Number(wc)>=a}var xc;var yc=ba.document;xc=yc&&t?oc()||("CSS1Compat"==yc.compatMode?parseInt(pc,10):5):void 0;var wc=xc;var zc=!t||vc(9),Ac=!t||vc(9),Bc=t&&!uc("9");!kc||uc("528");jc&&uc("1.9b")||t&&uc("8")||gc&&uc("9.5")||kc&&uc("528");jc&&!uc("8")||t&&uc("9");function Cc(a,b){this.type=a;this.currentTarget=this.target=b;this.defaultPrevented=this.jd=!1;this.mi=!0}Cc.prototype.stopPropagation=function(){this.jd=!0};Cc.prototype.preventDefault=function(){this.defaultPrevented=!0;this.mi=!1};function Ec(a){a.preventDefault()};function Fc(a){return kc?"webkit"+a:gc?"o"+a.toLowerCase():a.toLowerCase()}
var Gc={fp:"click",Ny:"rightclick",Sw:"dblclick",Qx:"mousedown",Wx:"mouseup",Vx:"mouseover",Ux:"mouseout",Tx:"mousemove",Rx:"mouseenter",Sx:"mouseleave",Sy:"selectionchange",Ty:"selectstart",yz:"wheel",Dx:"keypress",Cx:"keydown",jp:"keyup",Ew:"blur",rx:"focus",Tw:"deactivate",sx:t?"focusin":"DOMFocusIn",tx:t?"focusout":"DOMFocusOut",Iw:"change",Ky:"reset",Ry:"select",ez:"submit",Ax:"input",Gy:"propertychange",ix:"dragstart",dx:"drag",fx:"dragenter",hx:"dragover",gx:"dragleave",jx:"drop",ex:"dragend",
mz:"touchstart",lz:"touchmove",kz:"touchend",jz:"touchcancel",Dw:"beforeunload",Nw:"consolemessage",Ow:"contextmenu",Uw:"devicemotion",Vw:"deviceorientation",Yw:"DOMContentLoaded",nx:"error",yx:"help",Ex:"load",Mx:"losecapture",py:"orientationchange",Iy:"readystatechange",Ly:"resize",Oy:"scroll",pz:"unload",Gw:"canplay",Hw:"canplaythrough",kx:"durationchange",lx:"emptied",mx:"ended",Hx:"loadeddata",Ix:"loadedmetadata",uy:"pause",vy:"play",wy:"playing",Hy:"ratechange",Py:"seeked",Qy:"seeking",az:"stalled",
fz:"suspend",iz:"timeupdate",wz:"volumechange",xz:"waiting",$y:"sourceopen",Zy:"sourceended",Yy:"sourceclosed",rw:"abort",rz:"update",uz:"updatestart",sz:"updateend",wx:"hashchange",qy:"pagehide",ry:"pageshow",Fy:"popstate",Qw:"copy",ty:"paste",Rw:"cut",yw:"beforecopy",zw:"beforecut",Bw:"beforepaste",oy:"online",ny:"offline",Ox:"message",Mw:"connect",Bx:"install",sw:"activate",qx:"fetch",ux:"foreignfetch",Px:"messageerror",bz:"statechange",tz:"updatefound",Pw:"controllerchange",xw:Fc("AnimationStart"),
vw:Fc("AnimationEnd"),ww:Fc("AnimationIteration"),nz:Fc("TransitionEnd"),yy:"pointerdown",Ey:"pointerup",xy:"pointercancel",By:"pointermove",Dy:"pointerover",Cy:"pointerout",zy:"pointerenter",Ay:"pointerleave",vx:"gotpointercapture",Nx:"lostpointercapture",Xx:"MSGestureChange",Yx:"MSGestureEnd",Zx:"MSGestureHold",$x:"MSGestureStart",ay:"MSGestureTap",by:"MSGotPointerCapture",cy:"MSInertiaStart",dy:"MSLostPointerCapture",ey:"MSPointerCancel",fy:"MSPointerDown",gy:"MSPointerEnter",hy:"MSPointerHover",
iy:"MSPointerLeave",jy:"MSPointerMove",ky:"MSPointerOut",ly:"MSPointerOver",my:"MSPointerUp",gz:"text",hz:t?"textinput":"textInput",Kw:"compositionstart",Lw:"compositionupdate",Jw:"compositionend",Aw:"beforeinput",ox:"exit",Fx:"loadabort",Gx:"loadcommit",Jx:"loadredirect",Kx:"loadstart",Lx:"loadstop",My:"responsive",Uy:"sizechanged",qz:"unresponsive",vz:"visibilitychange",dz:"storage",cx:"DOMSubtreeModified",Zw:"DOMNodeInserted",ax:"DOMNodeRemoved",bx:"DOMNodeRemovedFromDocument",$w:"DOMNodeInsertedIntoDocument",
Ww:"DOMAttrModified",Xw:"DOMCharacterDataModified",Cw:"beforeprint",tw:"afterprint"};function Hc(a,b){Cc.call(this,a?a.type:"");this.relatedTarget=this.currentTarget=this.target=null;this.button=this.screenY=this.screenX=this.clientY=this.clientX=this.offsetY=this.offsetX=0;this.key="";this.charCode=this.keyCode=0;this.metaKey=this.shiftKey=this.altKey=this.ctrlKey=!1;this.state=null;this.Yn=!1;this.Wc=null;a&&this.init(a,b)}r(Hc,Cc);var Ic=[1,4,2];
Hc.prototype.init=function(a,b){var c=this.type=a.type,d=a.changedTouches?a.changedTouches[0]:null;this.target=a.target||a.srcElement;this.currentTarget=b;var e=a.relatedTarget;e?jc&&(ec(e,"nodeName")||(e=null)):"mouseover"==c?e=a.fromElement:"mouseout"==c&&(e=a.toElement);this.relatedTarget=e;null===d?(this.offsetX=kc||void 0!==a.offsetX?a.offsetX:a.layerX,this.offsetY=kc||void 0!==a.offsetY?a.offsetY:a.layerY,this.clientX=void 0!==a.clientX?a.clientX:a.pageX,this.clientY=void 0!==a.clientY?a.clientY:
a.pageY,this.screenX=a.screenX||0,this.screenY=a.screenY||0):(this.clientX=void 0!==d.clientX?d.clientX:d.pageX,this.clientY=void 0!==d.clientY?d.clientY:d.pageY,this.screenX=d.screenX||0,this.screenY=d.screenY||0);this.button=a.button;this.keyCode=a.keyCode||0;this.key=a.key||"";this.charCode=a.charCode||("keypress"==c?a.keyCode:0);this.ctrlKey=a.ctrlKey;this.altKey=a.altKey;this.shiftKey=a.shiftKey;this.metaKey=a.metaKey;this.Yn=mc?a.metaKey:a.ctrlKey;this.state=a.state;this.Wc=a;a.defaultPrevented&&
this.preventDefault()};function Jc(a){return(zc?0==a.Wc.button:"click"==a.type?!0:!!(a.Wc.button&Ic[0]))&&!(kc&&mc&&a.ctrlKey)}Hc.prototype.stopPropagation=function(){Hc.m.stopPropagation.call(this);this.Wc.stopPropagation?this.Wc.stopPropagation():this.Wc.cancelBubble=!0};Hc.prototype.preventDefault=function(){Hc.m.preventDefault.call(this);var a=this.Wc;if(a.preventDefault)a.preventDefault();else if(a.returnValue=!1,Bc)try{if(a.ctrlKey||112<=a.keyCode&&123>=a.keyCode)a.keyCode=-1}catch(b){}};var Kc="closure_listenable_"+(1E6*Math.random()|0);function Lc(a){return!(!a||!a[Kc])}var Mc=0;function Nc(a,b,c,d,e){this.listener=a;this.rk=null;this.src=b;this.type=c;this.capture=!!d;this.lc=e;this.key=++Mc;this.Rc=this.Ui=!1}function Qc(a){a.Rc=!0;a.listener=null;a.rk=null;a.src=null;a.lc=null};function Rc(a){this.src=a;this.bb={};this.Fi=0}f=Rc.prototype;f.add=function(a,b,c,d,e){var g=a.toString();a=this.bb[g];a||(a=this.bb[g]=[],this.Fi++);var h=Sc(a,b,d,e);-1<h?(b=a[h],c||(b.Ui=!1)):(b=new Nc(b,this.src,g,!!d,e),b.Ui=c,a.push(b));return b};f.remove=function(a,b,c,d){a=a.toString();if(!(a in this.bb))return!1;var e=this.bb[a];b=Sc(e,b,c,d);return-1<b?(Qc(e[b]),Array.prototype.splice.call(e,b,1),0==e.length&&(delete this.bb[a],this.Fi--),!0):!1};
function Tc(a,b){var c=b.type;if(!(c in a.bb))return!1;var d=ob(a.bb[c],b);d&&(Qc(b),0==a.bb[c].length&&(delete a.bb[c],a.Fi--));return d}f.ld=function(a){a=a&&a.toString();var b=0,c;for(c in this.bb)if(!a||c==a){for(var d=this.bb[c],e=0;e<d.length;e++)++b,Qc(d[e]);delete this.bb[c];this.Fi--}return b};f.Lh=function(a,b,c,d){a=this.bb[a.toString()];var e=-1;a&&(e=Sc(a,b,c,d));return-1<e?a[e]:null};
f.hasListener=function(a,b){var c=ca(a),d=c?a.toString():"",e=ca(b);return Fb(this.bb,function(a){for(var g=0;g<a.length;++g)if(!(c&&a[g].type!=d||e&&a[g].capture!=b))return!0;return!1})};function Sc(a,b,c,d){for(var e=0;e<a.length;++e){var g=a[e];if(!g.Rc&&g.listener==b&&g.capture==!!c&&g.lc==d)return e}return-1};var Uc="closure_lm_"+(1E6*Math.random()|0),Vc={},Wc=0;function v(a,b,c,d,e){if(la(b)){for(var g=0;g<b.length;g++)v(a,b[g],c,d,e);return null}c=Xc(c);return Lc(a)?a.A(b,c,d,e):Yc(a,b,c,!1,d,e)}
function Yc(a,b,c,d,e,g){if(!b)throw Error("Invalid event type");var h=!!e,k=Zc(a);k||(a[Uc]=k=new Rc(a));c=k.add(b,c,d,e,g);if(c.rk)return c;d=$c();c.rk=d;d.src=a;d.listener=c;if(a.addEventListener)a.addEventListener(b.toString(),d,h);else if(a.attachEvent)a.attachEvent(ad(b.toString()),d);else throw Error("addEventListener and attachEvent are unavailable.");Wc++;return c}
function $c(){var a=bd,b=Ac?function(c){return a.call(b.src,b.listener,c)}:function(c){c=a.call(b.src,b.listener,c);if(!c)return c};return b}function cd(a,b,c,d,e){if(la(b)){for(var g=0;g<b.length;g++)cd(a,b[g],c,d,e);return null}c=Xc(c);return Lc(a)?a.Hn(b,c,d,e):Yc(a,b,c,!0,d,e)}function dd(a,b,c,d,e){if(la(b))for(var g=0;g<b.length;g++)dd(a,b[g],c,d,e);else c=Xc(c),Lc(a)?a.Aa(b,c,d,e):a&&(a=Zc(a))&&(b=a.Lh(b,c,!!d,e))&&ed(b)}
function ed(a){if(ea(a)||!a||a.Rc)return!1;var b=a.src;if(Lc(b))return Tc(b.Mc,a);var c=a.type,d=a.rk;b.removeEventListener?b.removeEventListener(c,d,a.capture):b.detachEvent&&b.detachEvent(ad(c),d);Wc--;(c=Zc(b))?(Tc(c,a),0==c.Fi&&(c.src=null,b[Uc]=null)):Qc(a);return!0}function fd(a){if(a)if(Lc(a))a.Mc&&a.Mc.ld(void 0);else if(a=Zc(a)){var b=0,c;for(c in a.bb)for(var d=a.bb[c].concat(),e=0;e<d.length;++e)ed(d[e])&&++b}}function ad(a){return a in Vc?Vc[a]:Vc[a]="on"+a}
function hd(a,b,c,d){return Lc(a)?id(a,b,c,d):jd(a,b,c,d)}function jd(a,b,c,d){var e=!0;if(a=Zc(a))if(b=a.bb[b.toString()])for(b=b.concat(),a=0;a<b.length;a++){var g=b[a];g&&g.capture==c&&!g.Rc&&(g=kd(g,d),e=e&&!1!==g)}return e}function kd(a,b){var c=a.listener,d=a.lc||a.src;a.Ui&&ed(a);return c.call(d,b)}
function bd(a,b){if(a.Rc)return!0;if(!Ac){var c=b||ha("window.event"),d=new Hc(c,this),e=!0;if(!(0>c.keyCode||void 0!=c.returnValue)){a:{var g=!1;if(0==c.keyCode)try{c.keyCode=-1;break a}catch(l){g=!0}if(g||void 0==c.returnValue)c.returnValue=!0}c=[];for(g=d.currentTarget;g;g=g.parentNode)c.push(g);for(var h=a.type,k=c.length-1;!d.jd&&0<=k;k--)d.currentTarget=c[k],g=jd(c[k],h,!0,d),e=e&&g;for(k=0;!d.jd&&k<c.length;k++)d.currentTarget=c[k],g=jd(c[k],h,!1,d),e=e&&g}return e}return kd(a,new Hc(b,this))}
function Zc(a){a=a[Uc];return a instanceof Rc?a:null}var ld="__closure_events_fn_"+(1E9*Math.random()>>>0);function Xc(a){if(oa(a))return a;a[ld]||(a[ld]=function(b){return a.handleEvent(b)});return a[ld]};function md(){Xb.call(this);this.Mc=new Rc(this);this.Lt=this;this.Xn=null}r(md,Xb);md.prototype[Kc]=!0;f=md.prototype;f.Hj=function(){return this.Xn};f.Gk=function(a){this.Xn=a};f.addEventListener=function(a,b,c,d){v(this,a,b,c,d)};f.removeEventListener=function(a,b,c,d){dd(this,a,b,c,d)};
f.dispatchEvent=function(a){var b,c=this.Hj();if(c)for(b=[];c;c=c.Hj())b.push(c);var c=this.Lt,d=a.type||a;if(da(a))a=new Cc(a,c);else if(a instanceof Cc)a.target=a.target||c;else{var e=a;a=new Cc(d,c);Qb(a,e)}var e=!0;if(b)for(var g=b.length-1;!a.jd&&0<=g;g--){var h=a.currentTarget=b[g];e=id(h,d,!0,a)&&e}a.jd||(h=a.currentTarget=c,e=id(h,d,!0,a)&&e,a.jd||(e=id(h,d,!1,a)&&e));if(b)for(g=0;!a.jd&&g<b.length;g++)h=a.currentTarget=b[g],e=id(h,d,!1,a)&&e;return e};
f.V=function(){md.m.V.call(this);this.Mc&&this.Mc.ld(void 0);this.Xn=null};f.A=function(a,b,c,d){return this.Mc.add(String(a),b,!1,c,d)};f.Hn=function(a,b,c,d){return this.Mc.add(String(a),b,!0,c,d)};f.Aa=function(a,b,c,d){return this.Mc.remove(String(a),b,c,d)};function id(a,b,c,d){b=a.Mc.bb[String(b)];if(!b)return!0;b=b.concat();for(var e=!0,g=0;g<b.length;++g){var h=b[g];if(h&&!h.Rc&&h.capture==c){var k=h.listener,l=h.lc||h.src;h.Ui&&Tc(a.Mc,h);e=!1!==k.call(l,d)&&e}}return e&&0!=d.mi}
f.Lh=function(a,b,c,d){return this.Mc.Lh(String(a),b,c,d)};f.hasListener=function(a,b){return this.Mc.hasListener(ca(a)?String(a):void 0,b)};function nd(a,b){md.call(this);this.Mf=a||1;this.ah=b||ba;this.Jl=q(this.hw,this);this.Fn=wa()}r(nd,md);f=nd.prototype;f.enabled=!1;f.ua=null;f.setInterval=function(a){this.Mf=a;this.ua&&this.enabled?(this.stop(),this.start()):this.ua&&this.stop()};
f.hw=function(){if(this.enabled){var a=wa()-this.Fn;0<a&&a<.8*this.Mf?this.ua=this.ah.setTimeout(this.Jl,this.Mf-a):(this.ua&&(this.ah.clearTimeout(this.ua),this.ua=null),this.dispatchEvent("tick"),this.enabled&&(this.ua=this.ah.setTimeout(this.Jl,this.Mf),this.Fn=wa()))}};f.start=function(){this.enabled=!0;this.ua||(this.ua=this.ah.setTimeout(this.Jl,this.Mf),this.Fn=wa())};f.stop=function(){this.enabled=!1;this.ua&&(this.ah.clearTimeout(this.ua),this.ua=null)};
f.V=function(){nd.m.V.call(this);this.stop();delete this.ah};function od(a,b,c){if(oa(a))c&&(a=q(a,c));else if(a&&"function"==typeof a.handleEvent)a=q(a.handleEvent,a);else throw Error("Invalid listener argument");return 2147483647<Number(b)?-1:ba.setTimeout(a,b||0)};function pd(a){return/^\s*$/.test(a)?!1:/^[\],:{}\s\u2028\u2029]*$/.test(a.replace(/\\["\\\/bfnrtu]/g,"@").replace(/(?:"[^"\\\n\r\u2028\u2029\x00-\x08\x0a-\x1f]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)[\s\u2028\u2029]*(?=:|,|]|}|$)/g,"]").replace(/(?:^|:|,)(?:[\s\u2028\u2029]*\[)+/g,""))}function qd(a){a=String(a);if(pd(a))try{return eval("("+a+")")}catch(b){}throw Error("Invalid JSON string: "+a);}function rd(a){var b=[];sd(new td,a,b);return b.join("")}function td(){this.xk=void 0}
function sd(a,b,c){if(null==b)c.push("null");else{if("object"==typeof b){if(la(b)){var d=b;b=d.length;c.push("[");for(var e="",g=0;g<b;g++)c.push(e),e=d[g],sd(a,a.xk?a.xk.call(d,String(g),e):e,c),e=",";c.push("]");return}if(b instanceof String||b instanceof Number||b instanceof Boolean)b=b.valueOf();else{c.push("{");g="";for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&(e=b[d],"function"!=typeof e&&(c.push(g),ud(d,c),c.push(":"),sd(a,a.xk?a.xk.call(b,d,e):e,c),g=","));c.push("}");return}}switch(typeof b){case "string":ud(b,
c);break;case "number":c.push(isFinite(b)&&!isNaN(b)?String(b):"null");break;case "boolean":c.push(String(b));break;case "function":c.push("null");break;default:throw Error("Unknown type: "+typeof b);}}}var vd={'"':'\\"',"\\":"\\\\","/":"\\/","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t","\x0B":"\\u000b"},wd=/\uffff/.test("")?/[\\\"\x00-\x1f\x7f-\uffff]/g:/[\\\"\x00-\x1f\x7f-\xff]/g;
function ud(a,b){b.push('"',a.replace(wd,function(a){var b=vd[a];b||(b="\\u"+(a.charCodeAt(0)|65536).toString(16).substr(1),vd[a]=b);return b}),'"')};function xd(a){var b=Error();if(Error.captureStackTrace)Error.captureStackTrace(b,a||xd),b=String(b.stack);else{try{throw b;}catch(c){b=c}b=(b=b.stack)?String(b):null}b||(b=yd(a||arguments.callee.caller,[]));return b}
function yd(a,b){var c=[];if(nb(b,a))c.push("[...circular reference...]");else if(a&&50>b.length){c.push(zd(a)+"(");for(var d=a.arguments,e=0;d&&e<d.length;e++){0<e&&c.push(", ");var g=d[e];switch(typeof g){case "object":g=g?"object":"null";break;case "string":break;case "number":g=String(g);break;case "boolean":g=g?"true":"false";break;case "function":g=(g=zd(g))?g:"[fn]";break;default:g=typeof g}40<g.length&&(g=g.substr(0,40)+"...");c.push(g)}b.push(a);c.push(")\n");try{c.push(yd(a.caller,b))}catch(h){c.push("[exception trying to get caller]\n")}}else a?
c.push("[...long stack...]"):c.push("[end]");return c.join("")}function zd(a){if(Ad[a])return Ad[a];a=String(a);if(!Ad[a]){var b=/function ([^\(]+)/.exec(a);Ad[a]=b?b[1]:"[Anonymous]"}return Ad[a]}var Ad={};function Bd(){}Bd.prototype.Jp=null;function Cd(a){var b;(b=a.Jp)||(b={},Dd(a)&&(b[0]=!0,b[1]=!0),b=a.Jp=b);return b};var Ed;function Fd(){}r(Fd,Bd);function Hd(a){return(a=Dd(a))?new ActiveXObject(a):new XMLHttpRequest}function Dd(a){if(!a.Kq&&"undefined"==typeof XMLHttpRequest&&"undefined"!=typeof ActiveXObject){for(var b=["MSXML2.XMLHTTP.6.0","MSXML2.XMLHTTP.3.0","MSXML2.XMLHTTP","Microsoft.XMLHTTP"],c=0;c<b.length;c++){var d=b[c];try{return new ActiveXObject(d),a.Kq=d}catch(e){}}throw Error("Could not create ActiveXObject. ActiveX might be disabled, or MSXML might not be installed");}return a.Kq}Ed=new Fd;function Id(a){return a.xd&&"function"==typeof a.xd?a.xd():ma(a)||da(a)?a.length:Gb(a)}function Jd(a){if(a.Gb&&"function"==typeof a.Gb)return a.Gb();if(da(a))return a.split("");if(ma(a)){for(var b=[],c=a.length,d=0;d<c;d++)b.push(a[d]);return b}return Hb(a)}function Kd(a){if(a.rc&&"function"==typeof a.rc)return a.rc();if(!a.Gb||"function"!=typeof a.Gb){if(ma(a)||da(a)){var b=[];a=a.length;for(var c=0;c<a;c++)b.push(c);return b}return Ib(a)}}
function Ld(a,b,c){if(a.forEach&&"function"==typeof a.forEach)a.forEach(b,c);else if(ma(a)||da(a))eb(a,b,c);else for(var d=Kd(a),e=Jd(a),g=e.length,h=0;h<g;h++)b.call(c,e[h],d&&d[h],a)}function Md(a,b){if("function"==typeof a.every)return a.every(b,void 0);if(ma(a)||da(a))return kb(a,b,void 0);for(var c=Kd(a),d=Jd(a),e=d.length,g=0;g<e;g++)if(!b.call(void 0,d[g],c&&c[g],a))return!1;return!0};var Nd="StopIteration"in ba?ba.StopIteration:{message:"StopIteration",stack:""};function Od(){}Od.prototype.next=function(){throw Nd;};function Pd(a,b){this.jb={};this.Fa=[];this.Ka=0;var c=arguments.length;if(1<c){if(c%2)throw Error("Uneven number of arguments");for(var d=0;d<c;d+=2)this.set(arguments[d],arguments[d+1])}else a&&this.addAll(a)}f=Pd.prototype;f.xd=function(){return this.Ka};f.Gb=function(){Qd(this);for(var a=[],b=0;b<this.Fa.length;b++)a.push(this.jb[this.Fa[b]]);return a};f.rc=function(){Qd(this);return this.Fa.concat()};f.ae=function(a){return Rd(this.jb,a)};
f.Ch=function(a){for(var b=0;b<this.Fa.length;b++){var c=this.Fa[b];if(Rd(this.jb,c)&&this.jb[c]==a)return!0}return!1};f.pb=function(a,b){if(this===a)return!0;if(this.Ka!=a.xd())return!1;var c=b||Sd;Qd(this);for(var d,e=0;d=this.Fa[e];e++)if(!c(this.get(d),a.get(d)))return!1;return!0};function Sd(a,b){return a===b}f.isEmpty=function(){return 0==this.Ka};f.clear=function(){this.jb={};this.Ka=this.Fa.length=0};
f.remove=function(a){return Rd(this.jb,a)?(delete this.jb[a],this.Ka--,this.Fa.length>2*this.Ka&&Qd(this),!0):!1};function Qd(a){var b,c;if(a.Ka!=a.Fa.length){for(b=c=0;c<a.Fa.length;){var d=a.Fa[c];Rd(a.jb,d)&&(a.Fa[b++]=d);c++}a.Fa.length=b}if(a.Ka!=a.Fa.length){var e={};for(b=c=0;c<a.Fa.length;)d=a.Fa[c],Rd(e,d)||(a.Fa[b++]=d,e[d]=1),c++;a.Fa.length=b}}f.get=function(a,b){return Rd(this.jb,a)?this.jb[a]:b};f.set=function(a,b){Rd(this.jb,a)||(this.Ka++,this.Fa.push(a));this.jb[a]=b};
f.addAll=function(a){if(a instanceof Pd){var b=a.rc();a=a.Gb()}else b=Ib(a),a=Hb(a);for(var c=0;c<b.length;c++)this.set(b[c],a[c])};f.forEach=function(a,b){for(var c=this.rc(),d=0;d<c.length;d++){var e=c[d],g=this.get(e);a.call(b,g,e,this)}};f.clone=function(){return new Pd(this)};f.transpose=function(){for(var a=new Pd,b=0;b<this.Fa.length;b++){var c=this.Fa[b];a.set(this.jb[c],c)}return a};function Rd(a,b){return Object.prototype.hasOwnProperty.call(a,b)};var Td=/^(?:([^:/?#.]+):)?(?:\/\/(?:([^/?#]*)@)?([^/#?]*?)(?::([0-9]+))?(?=[/#?]|$))?([^?#]+)?(?:\?([^#]*))?(?:#([\s\S]*))?$/;function Ud(a,b){if(a)for(var c=a.split("\x26"),d=0;d<c.length;d++){var e=c[d].indexOf("\x3d"),g=null;if(0<=e){var h=c[d].substring(0,e);g=c[d].substring(e+1)}else h=c[d];b(h,g?Fa(g):"")}};function Vd(a){md.call(this);this.headers=new Pd;this.cl=a||null;this.tb=!1;this.bl=this.Z=null;this.Nf="";this.Kf=this.nn=this.Uj=this.jm=!1;this.No=0;this.Tk=null;this.Qr=Wd;this.To=this.Pv=this.Is=!1}r(Vd,md);var Wd="",Xd=/^https?$/i,Yd=["POST","PUT"];f=Vd.prototype;
f.send=function(a,b,c,d){if(this.Z)throw Error("[goog.net.XhrIo] Object is active with another request\x3d"+this.Nf+"; newUri\x3d"+a);b=b?b.toUpperCase():"GET";this.Nf=a;this.jm=!1;this.tb=!0;this.Z=this.cl?Hd(this.cl):Hd(Ed);this.bl=this.cl?Cd(this.cl):Cd(Ed);this.Z.onreadystatechange=q(this.ur,this);this.Pv&&"onprogress"in this.Z&&(this.Z.onprogress=q(function(a){this.sr(a,!0)},this),this.Z.upload&&(this.Z.upload.onprogress=q(this.sr,this)));try{this.nn=!0,this.Z.open(b,String(a),!0),this.nn=!1}catch(g){Zd(this);
return}a=c||"";var e=this.headers.clone();d&&Ld(d,function(a,b){e.set(b,a)});d=lb(e.rc(),$d);c=ba.FormData&&a instanceof ba.FormData;!nb(Yd,b)||d||c||e.set("Content-Type","application/x-www-form-urlencoded;charset\x3dutf-8");e.forEach(function(a,b){this.Z.setRequestHeader(b,a)},this);this.Qr&&(this.Z.responseType=this.Qr);"withCredentials"in this.Z&&this.Z.withCredentials!==this.Is&&(this.Z.withCredentials=this.Is);try{ae(this),0<this.No&&((this.To=ce(this.Z))?(this.Z.timeout=this.No,this.Z.ontimeout=
q(this.zs,this)):this.Tk=od(this.zs,this.No,this)),this.Uj=!0,this.Z.send(a),this.Uj=!1}catch(g){Zd(this)}};function ce(a){return t&&uc(9)&&ea(a.timeout)&&ca(a.ontimeout)}function $d(a){return"content-type"==a.toLowerCase()}f.zs=function(){"undefined"!=typeof aa&&this.Z&&(this.dispatchEvent("timeout"),this.abort(8))};function Zd(a){a.tb=!1;a.Z&&(a.Kf=!0,a.Z.abort(),a.Kf=!1);de(a);ee(a)}function de(a){a.jm||(a.jm=!0,a.dispatchEvent("complete"),a.dispatchEvent("error"))}
f.abort=function(){this.Z&&this.tb&&(this.tb=!1,this.Kf=!0,this.Z.abort(),this.Kf=!1,this.dispatchEvent("complete"),this.dispatchEvent("abort"),ee(this))};f.V=function(){this.Z&&(this.tb&&(this.tb=!1,this.Kf=!0,this.Z.abort(),this.Kf=!1),ee(this,!0));Vd.m.V.call(this)};f.ur=function(){this.Ee||(this.nn||this.Uj||this.Kf?fe(this):this.Av())};f.Av=function(){fe(this)};
function fe(a){if(a.tb&&"undefined"!=typeof aa&&(!a.bl[1]||4!=(a.Z?a.Z.readyState:0)||2!=ge(a)))if(a.Uj&&4==(a.Z?a.Z.readyState:0))od(a.ur,0,a);else if(a.dispatchEvent("readystatechange"),a.Xj()){a.tb=!1;try{a.Qq()?(a.dispatchEvent("complete"),a.dispatchEvent("success")):de(a)}finally{ee(a)}}}f.sr=function(a,b){this.dispatchEvent(he(a,"progress"));this.dispatchEvent(he(a,b?"downloadprogress":"uploadprogress"))};
function he(a,b){return{type:b,lengthComputable:a.lengthComputable,loaded:a.loaded,total:a.total}}function ee(a,b){if(a.Z){ae(a);var c=a.Z,d=a.bl[0]?ja:null;a.Z=null;a.bl=null;b||a.dispatchEvent("ready");try{c.onreadystatechange=d}catch(e){}}}function ae(a){a.Z&&a.To&&(a.Z.ontimeout=null);ea(a.Tk)&&(ba.clearTimeout(a.Tk),a.Tk=null)}f.Eg=function(){return!!this.Z};f.Xj=function(){return 4==(this.Z?this.Z.readyState:0)};
f.Qq=function(){var a=ge(this);a:switch(a){case 200:case 201:case 202:case 204:case 206:case 304:case 1223:var b=!0;break a;default:b=!1}if(!b){if(a=0===a)a=String(this.Nf).match(Td)[1]||null,!a&&ba.self&&ba.self.location&&(a=ba.self.location.protocol,a=a.substr(0,a.length-1)),a=!Xd.test(a?a.toLowerCase():"");b=a}return b};function ge(a){try{return 2<(a.Z?a.Z.readyState:0)?a.Z.status:-1}catch(b){return-1}}f.Ij=function(){try{return this.Z?this.Z.responseText:""}catch(a){return""}};
f.Pc=function(a){if(this.Z){var b=this.Z.responseText;a&&0==b.indexOf(a)&&(b=b.substring(a.length));return qd(b)}};f.getResponseHeader=function(a){if(this.Z&&this.Xj())return a=this.Z.getResponseHeader(a),null===a?void 0:a};f.getAllResponseHeaders=function(){return this.Z&&this.Xj()?this.Z.getAllResponseHeaders():""};
function ie(a){var b={};a=a.getAllResponseHeaders().split("\r\n");for(var c=0;c<a.length;c++)if(!Ba(a[c])){var d=2;for(var e=a[c].split(": "),g=[];0<d&&e.length;)g.push(e.shift()),d--;e.length&&g.push(e.join(": "));d=g;b[d[0]]=b[d[0]]?b[d[0]]+(", "+d[1]):d[1]}return b};function je(a){null==a&&(a={});this.ka=a.ka;this.Pf=a.Pf;this.bf=a.bf;null==this.ka&&(this.ka=!0);null==this.Pf&&(this.Pf=!0);null==this.bf&&(this.bf=!0)};function ke(a){var b=this;ke.m.constructor.call(this,a);this.ar=[];this.nc=new je;this.A("complete",function(){return b.dispatchEvent(le)})}r(ke,Vd);var le="done";ke.prototype.send=function(a,b,c,d){var e;this.dispatchEvent("send");var g=this.ar;var h=0;for(e=g.length;h<e;h++){var k=g[h];k.apply(d)}return ke.m.send.call(this,a,b,c,d)};function me(a,b,c,d){this.ka=a;this.Rt=b;this.ew=c;this.l=d;this.Bf=q(this.Bf,this);this.handleError=q(this.handleError,this);this.Fm=q(this.Fm,this);this.Mi=this.wp=0;this.zl=null;this.Ks=[];this.navigator=ne(this.l).navigator;this.ir=oe(this);this.Pr=[];t||v(ne(this.l),pe,this.Fm)}var pe="beforeunload";f=me.prototype;f.Fm=function(){var a;var b=this.Ks;var c=0;for(a=b.length;c<a;c++){var d=b[c];d.abort()}};
f.fc=function(a){null==a&&(a=new je);a=this.Ol(a);a.A("error",this.handleError);a.A("complete",this.Bf);return a};f.handleError=function(a){qe(this,a);var b=a.target;if(this.sn(b)){var c=b.Pc();a=a.target.nc;a.ka&&this.ka.$i();return re(this,c,a,ge(b))}a.target.nc.ka&&(c=oe(this),0===ge(b)&&!1===c&&!1!==this.ir?this.ka.sv():500<=ge(b)&&this.ka.error(ge(b)),this.ir=c)};function oe(a){var b=null;null!=a.navigator&&null!=a.navigator.onLine&&(b=a.navigator.onLine?!0:!1);return b}
f.Bf=function(a){qe(this,a);var b=a.target;if(204!==ge(b)){a=a.target.nc;try{var c=b.Pc();a.ka&&this.ka.$i();return re(this,c,a,ge(b))}catch(d){}}};function re(a,b,c,d){c.ka&&a.ka.apply(b,d);if(c.bf)return a.ew.apply(b)}
f.Ol=function(a){var b=this;var c=new ke;c.nc=a;this.Ks.push(c);c.Ic=this.wp;this.wp++;c.ar.push(this.Rt);c.A("send",function(c){a=c.target.nc;0===b.Mi&&a.ka&&a.Pf&&(c=b.ka.Pf(),b.zl=c.pop());return b.Mi++});c.A(le,function(){b.Mi--;if(0===b.Mi&&null!=b.zl)return b.zl.destroy()});return c};f.sn=function(a){a=a.getResponseHeader("Content-Type");return null!=a&&-1!==a.indexOf("json")};function se(a,b){a.Pr.push(b)}
function qe(a,b){var c;var d=a.Pr;var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];e.push(h.apply(b))}};var te=!t||vc(9),ue=!jc&&!t||t&&vc(9)||jc&&uc("1.9.1"),ve=t&&!uc("9"),we=t||gc||kc;var xe={area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0};function ye(){this.Pk="";this.Dt=ze}ye.prototype.Oe=!0;ye.prototype.ke=function(){return this.Pk};ye.prototype.toString=function(){return"Const{"+this.Pk+"}"};function Ae(a){return a instanceof ye&&a.constructor===ye&&a.Dt===ze?a.Pk:"type_error:Const"}var ze={};function Be(a){var b=new ye;b.Pk=a;return b}Be("");function Ce(){this.bo="";this.zt=Ee}Ce.prototype.Oe=!0;var Ee={};Ce.prototype.ke=function(){return this.bo};Ce.prototype.Dg=function(a){this.bo=a;return this};var Fe=(new Ce).Dg(""),Ge=/^([-,."'%_!# a-zA-Z0-9]+|(?:rgb|hsl)a?\([0-9.%, ]+\))$/;function He(){this.co="";this.Gt=Ie}He.prototype.Oe=!0;He.prototype.ke=function(){return this.co};He.prototype.ln=!0;He.prototype.ie=function(){return 1};function Je(a){if(a instanceof He&&a.constructor===He&&a.Gt===Ie)return a.co;n(a);return"type_error:TrustedResourceUrl"}var Ie={};function Ke(a){var b=new He;b.co=a;return b};function Le(){this.We="";this.At=Me}Le.prototype.Oe=!0;Le.prototype.ke=function(){return this.We};Le.prototype.ln=!0;Le.prototype.ie=function(){return 1};function Ne(a){if(a instanceof Le&&a.constructor===Le&&a.At===Me)return a.We;n(a);return"type_error:SafeUrl"}var Oe=/^(?:(?:https?|mailto|ftp):|[^:/?#]*(?:[/?#]|$))/i;function Pe(a){if(a instanceof Le)return a;a=a.Oe?a.ke():String(a);Oe.test(a)||(a="about:invalid#zClosurez");return Qe(a)}var Me={};function Qe(a){var b=new Le;b.We=a;return b}Qe("about:blank");function Re(){this.We="";this.yt=Se;this.gm=null}Re.prototype.ln=!0;Re.prototype.ie=function(){return this.gm};Re.prototype.Oe=!0;Re.prototype.ke=function(){return this.We};function Te(a){if(a instanceof Re&&a.constructor===Re&&a.yt===Se)return a.We;n(a);return"type_error:SafeHtml"}function Ue(a){if(a instanceof Re)return a;var b=null;a.ln&&(b=a.ie());return Ve(Ha(a.Oe?a.ke():String(a)),b)}
function We(a){if(a instanceof Re)return a;a=Ue(a);var b=Te(a);b=Ga(b.replace(/  /g," \x26#160;"),void 0);return Ve(b,a.ie())}var Xe=/^[a-zA-Z0-9-]+$/,Ye={action:!0,cite:!0,data:!0,formaction:!0,href:!0,manifest:!0,poster:!0,src:!0},Ze={APPLET:!0,BASE:!0,EMBED:!0,IFRAME:!0,LINK:!0,MATH:!0,META:!0,OBJECT:!0,SCRIPT:!0,STYLE:!0,SVG:!0,TEMPLATE:!0};
function $e(a,b,c){var d=String(a);if(!Xe.test(d))throw Error("Invalid tag name \x3c"+d+"\x3e.");if(d.toUpperCase()in Ze)throw Error("Tag name \x3c"+d+"\x3e is not allowed for SafeHtml.");return af(String(a),b,c)}function bf(a){function b(a){la(a)?eb(a,b):(a=Ue(a),d+=Te(a),a=a.ie(),0==c?c=a:0!=a&&c!=a&&(c=null))}var c=0,d="";eb(arguments,b);return Ve(d,c)}var Se={};function Ve(a,b){return(new Re).Dg(a,b)}Re.prototype.Dg=function(a,b){this.We=a;this.gm=b;return this};
function af(a,b,c){var d=null,e="";if(b)for(E in b){if(!Xe.test(E))throw Error('Invalid attribute name "'+E+'".');var g=b[E];if(null!=g){var h=a;var k=E;var l=g;if(l instanceof ye)l=Ae(l);else if("style"==k.toLowerCase()){g=void 0;h=l;if(!pa(h))throw Error('The "style" attribute requires goog.html.SafeStyle or map of style properties, '+typeof h+" given: "+h);if(!(h instanceof Ce)){l="";for(g in h){if(!/^[-_a-zA-Z0-9]+$/.test(g))throw Error("Name allows only [-_a-zA-Z0-9], got: "+g);var m=h[g];if(null!=
m){if(m instanceof ye)m=Ae(m);else if(Ge.test(m)){for(var p=!0,u=!0,A=0;A<m.length;A++){var B=m.charAt(A);"'"==B&&u?p=!p:'"'==B&&p&&(u=!u)}p&&u||(m="zClosurez")}else m="zClosurez";l+=g+":"+m+";"}}h=l?(new Ce).Dg(l):Fe}h instanceof Ce&&h.constructor===Ce&&h.zt===Ee?g=h.bo:(n(h),g="type_error:SafeStyle");l=g}else{if(/^on/i.test(k))throw Error('Attribute "'+k+'" requires goog.string.Const value, "'+l+'" given.');if(k.toLowerCase()in Ye)if(l instanceof He)l=Je(l);else if(l instanceof Le)l=Ne(l);else if(da(l))l=
Pe(l).ke();else throw Error('Attribute "'+k+'" on tag "'+h+'" requires goog.html.SafeUrl, goog.string.Const, or string, value "'+l+'" given.');}l.Oe&&(l=l.ke());k=k+'\x3d"'+Ha(String(l))+'"';e+=" "+k}}var E="\x3c"+a+e;null!=c?la(c)||(c=[c]):c=[];!0===xe[a.toLowerCase()]?E+="\x3e":(d=bf(c),E+="\x3e"+Te(d)+"\x3c/"+a+"\x3e",d=d.ie());(a=b&&b.dir)&&(d=/^(ltr|rtl|auto)$/i.test(a)?0:null);return Ve(E,d)}Ve("\x3c!DOCTYPE html\x3e",0);var cf=Ve("",0);Ve("\x3cbr\x3e",0);function df(a,b){this.x=ca(a)?a:0;this.y=ca(b)?b:0}f=df.prototype;f.clone=function(){return new df(this.x,this.y)};f.pb=function(a){return a instanceof df&&ef(this,a)};function ef(a,b){return a==b?!0:a&&b?a.x==b.x&&a.y==b.y:!1}function ff(a,b){return new df(a.x-b.x,a.y-b.y)}f.ceil=function(){this.x=Math.ceil(this.x);this.y=Math.ceil(this.y);return this};f.floor=function(){this.x=Math.floor(this.x);this.y=Math.floor(this.y);return this};
f.round=function(){this.x=Math.round(this.x);this.y=Math.round(this.y);return this};f.translate=function(a,b){a instanceof df?(this.x+=a.x,this.y+=a.y):(this.x+=Number(a),ea(b)&&(this.y+=b));return this};f.scale=function(a,b){var c=ea(b)?b:a;this.x*=a;this.y*=c;return this};function gf(a,b){this.width=a;this.height=b}f=gf.prototype;f.clone=function(){return new gf(this.width,this.height)};f.area=function(){return this.width*this.height};f.isEmpty=function(){return!this.area()};f.ceil=function(){this.width=Math.ceil(this.width);this.height=Math.ceil(this.height);return this};f.floor=function(){this.width=Math.floor(this.width);this.height=Math.floor(this.height);return this};f.round=function(){this.width=Math.round(this.width);this.height=Math.round(this.height);return this};
f.scale=function(a,b){var c=ea(b)?b:a;this.width*=a;this.height*=c;return this};function jf(a){return a?new kf(lf(a)):ya||(ya=new kf)}function mf(a,b){return da(b)?a.getElementById(b):b}function nf(a,b){return(b||document).getElementsByTagName(String(a))}function of(a,b){var c=b||document;return(c.getElementsByClassName?c.getElementsByClassName(a)[0]:c.querySelectorAll&&c.querySelector?c.querySelector("."+a):pf(document,"*",a,b)[0])||null}
function pf(a,b,c,d){a=d||a;var e=b&&"*"!=b?String(b).toUpperCase():"";if(a.querySelectorAll&&a.querySelector&&(e||c))return a.querySelectorAll(e+(c?"."+c:""));if(c&&a.getElementsByClassName){d=a.getElementsByClassName(c);if(e){a={};for(var g=b=0,h;h=d[g];g++)e==h.nodeName&&(a[b++]=h);a.length=b;return a}return d}d=a.getElementsByTagName(e||"*");if(c){a={};for(g=b=0;h=d[g];g++)e=h.className,"function"==typeof e.split&&nb(e.split(/\s+/),c)&&(a[b++]=h);a.length=b;return a}return d}
function qf(a,b){Db(b,function(b,d){b&&b.Oe&&(b=b.ke());"style"==d?a.style.cssText=b:"class"==d?a.className=b:"for"==d?a.htmlFor=b:rf.hasOwnProperty(d)?a.setAttribute(rf[d],b):za(d,"aria-")||za(d,"data-")?a.setAttribute(d,b):a[d]=b})}var rf={cellpadding:"cellPadding",cellspacing:"cellSpacing",colspan:"colSpan",frameborder:"frameBorder",height:"height",maxlength:"maxLength",nonce:"nonce",role:"role",rowspan:"rowSpan",type:"type",usemap:"useMap",valign:"vAlign",width:"width"};
function sf(a){a=a.document;a=tf(a)?a.documentElement:a.body;return new gf(a.clientWidth,a.clientHeight)}function uf(a){var b=vf(a);a=a.parentWindow||a.defaultView;return t&&uc("10")&&a.pageYOffset!=b.scrollTop?new df(b.scrollLeft,b.scrollTop):new df(a.pageXOffset||b.scrollLeft,a.pageYOffset||b.scrollTop)}function vf(a){return a.scrollingElement?a.scrollingElement:!kc&&tf(a)?a.documentElement:a.body||a.documentElement}function wf(a){return a?a.parentWindow||a.defaultView:window}
function xf(a,b,c){return yf(document,arguments)}function yf(a,b){var c=String(b[0]),d=b[1];if(!te&&d&&(d.name||d.type)){c=["\x3c",c];d.name&&c.push(' name\x3d"',Ha(d.name),'"');if(d.type){c.push(' type\x3d"',Ha(d.type),'"');var e={};Qb(e,d);delete e.type;d=e}c.push("\x3e");c=c.join("")}c=a.createElement(c);d&&(da(d)?c.className=d:la(d)?c.className=d.join(" "):qf(c,d));2<b.length&&zf(a,c,b,2);return c}
function zf(a,b,c,d){function e(c){c&&b.appendChild(da(c)?a.createTextNode(c):c)}for(;d<c.length;d++){var g=c[d];!ma(g)||pa(g)&&0<g.nodeType?e(g):eb(Af(g)?sb(g):g,e)}}function tf(a){return"CSS1Compat"==a.compatMode}function Bf(a,b){zf(lf(a),a,arguments,1)}function Cf(a){for(var b;b=a.firstChild;)a.removeChild(b)}function Df(a,b){b.parentNode&&b.parentNode.insertBefore(a,b)}function Ef(a,b){b.parentNode&&b.parentNode.insertBefore(a,b.nextSibling)}
function Ff(a){return a&&a.parentNode?a.parentNode.removeChild(a):null}function Gf(a,b){for(;a&&1!=a.nodeType;)a=b?a.nextSibling:a.previousSibling;return a}function Hf(a){return pa(a)&&1==a.nodeType}function If(a,b){if(!a||!b)return!1;if(a.contains&&1==b.nodeType)return a==b||a.contains(b);if("undefined"!=typeof a.compareDocumentPosition)return a==b||!!(a.compareDocumentPosition(b)&16);for(;b&&a!=b;)b=b.parentNode;return b==a}function lf(a){return 9==a.nodeType?a:a.ownerDocument||a.document}
function Jf(a){return a.contentDocument||a.contentWindow.document}function Kf(a,b){if("textContent"in a)a.textContent=b;else if(3==a.nodeType)a.data=b;else if(a.firstChild&&3==a.firstChild.nodeType){for(;a.lastChild!=a.firstChild;)a.removeChild(a.lastChild);a.firstChild.data=b}else Cf(a),a.appendChild(lf(a).createTextNode(String(b)))}var Lf={SCRIPT:1,STYLE:1,HEAD:1,IFRAME:1,OBJECT:1},Mf={IMG:" ",BR:"\n"};function Nf(a){return Of(a)&&Pf(a)}
function Qf(a,b){b?a.tabIndex=0:(a.tabIndex=-1,a.removeAttribute("tabIndex"))}function Of(a){return t&&!uc("9")?(a=a.getAttributeNode("tabindex"),null!=a&&a.specified):a.hasAttribute("tabindex")}function Pf(a){a=a.tabIndex;return ea(a)&&0<=a&&32768>a}
function Rf(a){if(ve&&null!==a&&"innerText"in a)a=a.innerText.replace(/(\r\n|\r|\n)/g,"\n");else{var b=[];Sf(a,b,!0);a=b.join("")}a=a.replace(/ \xAD /g," ").replace(/\xAD/g,"");a=a.replace(/\u200B/g,"");ve||(a=a.replace(/ +/g," "));" "!=a&&(a=a.replace(/^\s*/,""));return a}function Tf(a){var b=[];Sf(a,b,!1);return b.join("")}
function Sf(a,b,c){if(!(a.nodeName in Lf))if(3==a.nodeType)c?b.push(String(a.nodeValue).replace(/(\r\n|\r|\n)/g,"")):b.push(a.nodeValue);else if(a.nodeName in Mf)b.push(Mf[a.nodeName]);else for(a=a.firstChild;a;)Sf(a,b,c),a=a.nextSibling}function Af(a){if(a&&"number"==typeof a.length){if(pa(a))return"function"==typeof a.item||"string"==typeof a.item;if(oa(a))return"function"==typeof a.item}return!1}function Uf(a){try{return a&&a.activeElement}catch(b){}return null}
function kf(a){this.P=a||ba.document||document}f=kf.prototype;f.J=jf;f.o=function(a){return mf(this.P,a)};f.getElementsByTagName=function(a,b){return(b||this.P).getElementsByTagName(String(a))};f.xm=function(a,b){var c=b||this.P,d=c||document;return d.querySelectorAll&&d.querySelector?d.querySelectorAll("."+a):pf(document,"*",a,c)};f.Ej=function(a,b){return of(a,b||this.P)};f.D=function(a,b,c){return yf(this.P,arguments)};f.createElement=function(a){return this.P.createElement(String(a))};
f.createTextNode=function(a){return this.P.createTextNode(String(a))};function ne(a){a=a.P;return a.parentWindow||a.defaultView}f.sm=function(a){return Uf(a||this.P)};f.appendChild=function(a,b){a.appendChild(b)};f.append=Bf;f.canHaveChildren=function(a){if(1!=a.nodeType)return!1;switch(a.tagName){case "APPLET":case "AREA":case "BASE":case "BR":case "COL":case "COMMAND":case "EMBED":case "FRAME":case "HR":case "IMG":case "INPUT":case "IFRAME":case "ISINDEX":case "KEYGEN":case "LINK":case "NOFRAMES":case "NOSCRIPT":case "META":case "OBJECT":case "PARAM":case "SCRIPT":case "SOURCE":case "STYLE":case "TRACK":case "WBR":return!1}return!0};
f.Ug=Cf;f.Mq=Df;f.Vj=Ef;function Vf(a,b){a.insertBefore(b,a.childNodes[0]||null)}f.removeNode=Ff;function Wf(a,b){var c=b.parentNode;c&&c.replaceChild(a,b)}f.he=function(a){return ue&&void 0!=a.children?a.children:gb(a.childNodes,function(a){return 1==a.nodeType})};function Xf(a){return ca(a.nextElementSibling)?a.nextElementSibling:Gf(a.nextSibling,!0)}
function Yf(a){var b;if(we&&!(t&&uc("9")&&!uc("10")&&ba.SVGElement&&a instanceof ba.SVGElement)&&(b=a.parentElement))return b;b=a.parentNode;return Hf(b)?b:null}f.contains=If;f.Sc=Kf;
f.ne=function(a){var b;(b="A"==a.tagName||"INPUT"==a.tagName||"TEXTAREA"==a.tagName||"SELECT"==a.tagName||"BUTTON"==a.tagName?!a.disabled&&(!Of(a)||Pf(a)):Nf(a))&&t?(a=!oa(a.getBoundingClientRect)||t&&null==a.parentElement?{height:a.offsetHeight,width:a.offsetWidth}:a.getBoundingClientRect(),a=null!=a&&0<a.height&&0<a.width):a=b;return a};f.wg=Rf;function Zf(a,b,c,d){this.top=a;this.right=b;this.bottom=c;this.left=d}f=Zf.prototype;f.zq=function(){return this.bottom-this.top};f.clone=function(){return new Zf(this.top,this.right,this.bottom,this.left)};f.contains=function(a){return this&&a?a instanceof Zf?a.left>=this.left&&a.right<=this.right&&a.top>=this.top&&a.bottom<=this.bottom:a.x>=this.left&&a.x<=this.right&&a.y>=this.top&&a.y<=this.bottom:!1};
f.expand=function(a,b,c,d){pa(a)?(this.top-=a.top,this.right+=a.right,this.bottom+=a.bottom,this.left-=a.left):(this.top-=a,this.right+=Number(b),this.bottom+=Number(c),this.left-=Number(d));return this};f.ceil=function(){this.top=Math.ceil(this.top);this.right=Math.ceil(this.right);this.bottom=Math.ceil(this.bottom);this.left=Math.ceil(this.left);return this};
f.floor=function(){this.top=Math.floor(this.top);this.right=Math.floor(this.right);this.bottom=Math.floor(this.bottom);this.left=Math.floor(this.left);return this};f.round=function(){this.top=Math.round(this.top);this.right=Math.round(this.right);this.bottom=Math.round(this.bottom);this.left=Math.round(this.left);return this};f.translate=function(a,b){a instanceof df?(this.left+=a.x,this.right+=a.x,this.top+=a.y,this.bottom+=a.y):(this.left+=a,this.right+=a,ea(b)&&(this.top+=b,this.bottom+=b));return this};
f.scale=function(a,b){var c=ea(b)?b:a;this.left*=a;this.right*=a;this.top*=c;this.bottom*=c;return this};function $f(a,b,c,d){this.left=a;this.top=b;this.width=c;this.height=d}f=$f.prototype;f.clone=function(){return new $f(this.left,this.top,this.width,this.height)};f.Nq=function(a){var b=Math.max(this.left,a.left),c=Math.min(this.left+this.width,a.left+a.width);if(b<=c){var d=Math.max(this.top,a.top);a=Math.min(this.top+this.height,a.top+a.height);if(d<=a)return this.left=b,this.top=d,this.width=c-b,this.height=a-d,!0}return!1};
f.contains=function(a){return a instanceof df?a.x>=this.left&&a.x<=this.left+this.width&&a.y>=this.top&&a.y<=this.top+this.height:this.left<=a.left&&this.left+this.width>=a.left+a.width&&this.top<=a.top&&this.top+this.height>=a.top+a.height};f.ceil=function(){this.left=Math.ceil(this.left);this.top=Math.ceil(this.top);this.width=Math.ceil(this.width);this.height=Math.ceil(this.height);return this};
f.floor=function(){this.left=Math.floor(this.left);this.top=Math.floor(this.top);this.width=Math.floor(this.width);this.height=Math.floor(this.height);return this};f.round=function(){this.left=Math.round(this.left);this.top=Math.round(this.top);this.width=Math.round(this.width);this.height=Math.round(this.height);return this};f.translate=function(a,b){a instanceof df?(this.left+=a.x,this.top+=a.y):(this.left+=a,ea(b)&&(this.top+=b));return this};
f.scale=function(a,b){var c=ea(b)?b:a;this.left*=a;this.width*=a;this.top*=c;this.height*=c;return this};function ag(a,b,c){if(da(b))(b=bg(a,b))&&(a.style[b]=c);else for(var d in b){c=a;var e=b[d],g=bg(c,d);g&&(c.style[g]=e)}}var cg={};function bg(a,b){var c=cg[b];if(!c){var d=Za(b),c=d;void 0===a.style[d]&&(d=(kc?"Webkit":jc?"Moz":t?"ms":gc?"O":null)+ab(d),void 0!==a.style[d]&&(c=d));cg[b]=c}return c}function dg(a,b){var c=lf(a);return c.defaultView&&c.defaultView.getComputedStyle&&(c=c.defaultView.getComputedStyle(a,null))?c[b]||c.getPropertyValue(b)||"":""}
function eg(a,b){return dg(a,b)||(a.currentStyle?a.currentStyle[b]:null)||a.style&&a.style[b]}function fg(a){return eg(a,"position")}function ig(a,b,c){if(b instanceof df){var d=b.x;b=b.y}else d=b,b=c;a.style.left=jg(d,!1);a.style.top=jg(b,!1)}function kg(a){a=a?lf(a):document;return!t||vc(9)||tf(jf(a).P)?a.documentElement:a.body}
function lg(a){try{var b=a.getBoundingClientRect()}catch(c){return{left:0,top:0,right:0,bottom:0}}t&&a.ownerDocument.body&&(a=a.ownerDocument,b.left-=a.documentElement.clientLeft+a.body.clientLeft,b.top-=a.documentElement.clientTop+a.body.clientTop);return b}
function mg(a){if(t&&!vc(8))return a.offsetParent;var b=lf(a),c=eg(a,"position"),d="fixed"==c||"absolute"==c;for(a=a.parentNode;a&&a!=b;a=a.parentNode)if(11==a.nodeType&&a.host&&(a=a.host),c=eg(a,"position"),d=d&&"static"==c&&a!=b.documentElement&&a!=b.body,!d&&(a.scrollWidth>a.clientWidth||a.scrollHeight>a.clientHeight||"fixed"==c||"absolute"==c||"relative"==c))return a;return null}
function ng(a){for(var b=new Zf(0,Infinity,Infinity,0),c=jf(a),d=c.P.body,e=c.P.documentElement,g=vf(c.P);a=mg(a);)if(!(t&&0==a.clientWidth||kc&&0==a.clientHeight&&a==d)&&a!=d&&a!=e&&"visible"!=eg(a,"overflow")){var h=og(a),k=new df(a.clientLeft,a.clientTop);h.x+=k.x;h.y+=k.y;b.top=Math.max(b.top,h.y);b.right=Math.min(b.right,h.x+a.clientWidth);b.bottom=Math.min(b.bottom,h.y+a.clientHeight);b.left=Math.max(b.left,h.x)}d=g.scrollLeft;g=g.scrollTop;b.left=Math.max(b.left,d);b.top=Math.max(b.top,g);
c=sf(ne(c)||window);b.right=Math.min(b.right,d+c.width);b.bottom=Math.min(b.bottom,g+c.height);return 0<=b.top&&0<=b.left&&b.bottom>b.top&&b.right>b.left?b:null}
function pg(a,b){var c=b||vf(document),d=c||vf(document);var e=og(a);var g=og(d),h=qg(d);if(d==vf(document)){var k=e.x-d.scrollLeft;e=e.y-d.scrollTop;t&&!vc(10)&&(k+=h.left,e+=h.top)}else k=e.x-g.x-h.left,e=e.y-g.y-h.top;var h=rg(a),g=d.clientHeight-h.height,l=d.scrollLeft,m=d.scrollTop,l=l+Math.min(k,Math.max(k-(d.clientWidth-h.width),0)),m=m+Math.min(e,Math.max(e-g,0));k=new df(l,m);c.scrollLeft=k.x;c.scrollTop=k.y}
function og(a){var b=lf(a),c=new df(0,0),d=kg(b);if(a==d)return c;a=lg(a);b=uf(jf(b).P);c.x=a.left+b.x;c.y=a.top+b.y;return c}function sg(a,b,c){if(b instanceof gf)c=b.height,b=b.width;else if(void 0==c)throw Error("missing height argument");a.style.width=jg(b,!0);a.style.height=jg(c,!0)}function jg(a,b){"number"==typeof a&&(a=(b?Math.round(a):a)+"px");return a}
function tg(a){var b=rg;if("none"!=eg(a,"display"))return b(a);var c=a.style,d=c.display,e=c.visibility,g=c.position;c.visibility="hidden";c.position="absolute";c.display="inline";a=b(a);c.display=d;c.position=g;c.visibility=e;return a}function rg(a){var b=a.offsetWidth,c=a.offsetHeight,d=kc&&!b&&!c;return ca(b)&&!d||!a.getBoundingClientRect?new gf(b,c):(a=lg(a),new gf(a.right-a.left,a.bottom-a.top))}
function ug(a,b){var c=a.style;"opacity"in c?c.opacity=b:"MozOpacity"in c?c.MozOpacity=b:"filter"in c&&(c.filter=""===b?"":"alpha(opacity\x3d"+100*Number(b)+")")}function w(a,b){a.style.display=b?"":"none"}function vg(a){return"rtl"==eg(a,"direction")}var wg=jc?"MozUserSelect":kc||hc?"WebkitUserSelect":null;
function xg(a,b,c){c=c?null:a.getElementsByTagName("*");if(wg){if(b=b?"none":"",a.style&&(a.style[wg]=b),c){a=0;for(var d;d=c[a];a++)d.style&&(d.style[wg]=b)}}else if(t||gc)if(b=b?"on":"",a.setAttribute("unselectable",b),c)for(a=0;d=c[a];a++)d.setAttribute("unselectable",b)}
function yg(a,b){if(/^\d+px?$/.test(b))return parseInt(b,10);var c=a.style.left,d=a.runtimeStyle.left;a.runtimeStyle.left=a.currentStyle.left;a.style.left=b;var e=a.style.pixelLeft;a.style.left=c;a.runtimeStyle.left=d;return+e}function zg(a,b){var c=a.currentStyle?a.currentStyle[b]:null;return c?yg(a,c):0}var Ag={thin:2,medium:4,thick:6};
function Bg(a,b){if("none"==(a.currentStyle?a.currentStyle[b+"Style"]:null))return 0;var c=a.currentStyle?a.currentStyle[b+"Width"]:null;return c in Ag?Ag[c]:yg(a,c)}function qg(a){if(t&&!vc(9)){var b=Bg(a,"borderLeft");var c=Bg(a,"borderRight");var d=Bg(a,"borderTop");a=Bg(a,"borderBottom");return new Zf(d,c,a,b)}b=dg(a,"borderLeftWidth");c=dg(a,"borderRightWidth");d=dg(a,"borderTopWidth");a=dg(a,"borderBottomWidth");return new Zf(parseFloat(d),parseFloat(c),parseFloat(a),parseFloat(b))};function Cg(a){Cg.m.constructor.call(this,a)}r(Cg,kf);f=Cg.prototype;f.H=function(a,b,c){var d;null==b&&(b=null);null==c&&(c=null);b=this.D(a,b);if(da(c))this.Sc(b,c);else if(null!=c&&null!=c.nodeType)c.appendChild(b);else if(la(c)){var e=0;for(d=c.length;e<d;e++)a=c[e],b.appendChild(a)}return b};f.show=function(a){return w(a,!0)};f.hide=function(a){return w(a,!1)};f.en=function(a){return null!=a.children&&0<a.children.length};
function Dg(a,b,c){null==c&&(c=null);null==c&&(c=a.P);return c.querySelector(b)}f.all=function(a,b){null==b&&(b=null);null==b&&(b=this.P);return b.querySelectorAll(a)};function Eg(a,b,c){null==c&&(c=null);return a.Ej(b,c)}function Fg(a,b,c){null==c&&(c=null);return a.xm(b,c)}function Gg(a,b,c,d){var e;c=String(c);var g=a.all("option",b);var h=[];b=0;for(e=g.length;b<e;b++)if(a=g[b],a.value===c)if(d(a))break;else h.push(void 0);else h.push(void 0);return h}
f.select=function(a,b){return Gg(this,a,b,function(a){return a.selected=!0})};f.unselect=function(a,b){return Gg(this,a,b,function(a){return a.selected=!1})};function Hg(a){var b;if(t)for(b=[];a.hasChildNodes();)b.push(a.removeChild(a.lastChild));else a.innerHTML=""};function Ig(){md.call(this);this.ma=Jg;this.endTime=this.startTime=null}r(Ig,md);var Jg=0;Ig.prototype.oe=function(){this.ud("begin")};Ig.prototype.Ue=function(){this.ud("end")};Ig.prototype.ud=function(a){this.dispatchEvent(a)};function Kg(a,b,c){Xb.call(this);this.In=a;this.Mf=b||0;this.qb=c;this.Wt=q(this.wu,this)}r(Kg,Xb);f=Kg.prototype;f.Ic=0;f.V=function(){Kg.m.V.call(this);this.stop();delete this.In;delete this.qb};f.start=function(a){this.stop();this.Ic=od(this.Wt,ca(a)?a:this.Mf)};f.stop=function(){this.Eg()&&ba.clearTimeout(this.Ic);this.Ic=0};f.Eg=function(){return 0!=this.Ic};f.wu=function(){this.Ic=0;this.In&&this.In.call(this.qb)};var Lb={},Lg=null;function Mg(a){a=qa(a);delete Lb[a];Kb()&&Lg&&Lg.stop()}function Ng(){Lg||(Lg=new Kg(function(){Og()},20));var a=Lg;a.Eg()||a.start()}function Og(){var a=wa();Db(Lb,function(b){b.oj(a)});Kb()||Ng()};function Pg(a,b,c,d){Ig.call(this);if(!la(a)||!la(b))throw Error("Start and end parameters must be arrays");if(a.length!=b.length)throw Error("Start and end points must be the same length");this.zi=a;this.zu=b;this.duration=c;this.xp=d;this.coords=[];this.$k=!1;this.progress=0}r(Pg,Ig);f=Pg.prototype;
f.play=function(a){if(a||this.ma==Jg)this.progress=0,this.coords=this.zi;else if(1==this.ma)return!1;Mg(this);this.startTime=a=wa();-1==this.ma&&(this.startTime-=this.duration*this.progress);this.endTime=this.startTime+this.duration;this.progress||this.oe();this.ud("play");-1==this.ma&&this.ud("resume");this.ma=1;var b=qa(this);b in Lb||(Lb[b]=this);Ng();this.oj(a);return!0};f.stop=function(a){Mg(this);this.ma=Jg;a&&(this.progress=1);Qg(this,this.progress);this.ud("stop");this.Ue()};
f.pause=function(){1==this.ma&&(Mg(this),this.ma=-1,this.ud("pause"))};f.V=function(){this.ma==Jg||this.stop(!1);this.ud("destroy");Pg.m.V.call(this)};f.destroy=function(){this.eb()};f.oj=function(a){a<this.startTime&&(this.endTime=a+this.endTime-this.startTime,this.startTime=a);this.progress=(a-this.startTime)/(this.endTime-this.startTime);1<this.progress&&(this.progress=1);Qg(this,this.progress);1==this.progress?(this.ma=Jg,Mg(this),this.ud("finish"),this.Ue()):1==this.ma&&this.Sn()};
function Qg(a,b){oa(a.xp)&&(b=a.xp(b));a.coords=Array(a.zi.length);for(var c=0;c<a.zi.length;c++)a.coords[c]=(a.zu[c]-a.zi[c])*b+a.zi[c]}f.Sn=function(){this.ud("animate")};f.ud=function(a){this.dispatchEvent(new Rg(a,this))};function Rg(a,b){Cc.call(this,a);this.coords=b.coords;this.x=b.coords[0];this.y=b.coords[1];this.z=b.coords[2];this.duration=b.duration;this.progress=b.progress;this.state=b.ma}r(Rg,Cc);function Sg(a,b,c,d,e){Pg.call(this,b,c,d,e);this.element=a}r(Sg,Pg);f=Sg.prototype;f.Zk=ja;f.Zj=function(){ca(this.Cd)||(this.Cd=vg(this.element));return this.Cd};f.Sn=function(){this.Zk();Sg.m.Sn.call(this)};f.Ue=function(){this.Zk();Sg.m.Ue.call(this)};f.oe=function(){this.Zk();Sg.m.oe.call(this)};function Tg(a,b,c,d,e){ea(b)&&(b=[b]);ea(c)&&(c=[c]);Sg.call(this,a,b,c,d,e);if(1!=b.length||1!=c.length)throw Error("Start and end points must be 1D");this.dk=Ug}r(Tg,Sg);var Vg=1/1024,Ug=-1;f=Tg.prototype;
f.Zk=function(){var a=this.coords[0];Math.abs(a-this.dk)>=Vg&&(ug(this.element,a),this.dk=a)};f.oe=function(){this.dk=Ug;Tg.m.oe.call(this)};f.Ue=function(){this.dk=Ug;Tg.m.Ue.call(this)};f.show=function(){this.element.style.display=""};f.hide=function(){this.element.style.display="none"};function Wg(a,b,c){Tg.call(this,a,1,0,b,c)}r(Wg,Tg);Wg.prototype.oe=function(){this.show();Wg.m.oe.call(this)};Wg.prototype.Ue=function(){this.hide();Wg.m.Ue.call(this)};
function Xg(a,b,c){Tg.call(this,a,0,1,b,c)}r(Xg,Tg);Xg.prototype.oe=function(){this.show();Xg.m.oe.call(this)};function Yg(a){a=a.className;return da(a)&&a.match(/\S+/g)||[]}function Zg(a,b){var c=Yg(a),d=ub(arguments,1),e=c.length+d.length;$g(c,d);a.className=c.join(" ");return c.length==e}function ah(a,b){var c=Yg(a),d=ub(arguments,1),e=bh(c,d);a.className=e.join(" ");return e.length==c.length-d.length}function $g(a,b){for(var c=0;c<b.length;c++)nb(a,b[c])||a.push(b[c])}function bh(a,b){return gb(a,function(a){return!nb(b,a)})}
function ch(a,b,c){var d=Yg(a);da(b)?ob(d,b):la(b)&&(d=bh(d,b));da(c)&&!nb(d,c)?d.push(c):la(c)&&$g(d,c);a.className=d.join(" ")}function dh(a,b){return nb(Yg(a),b)}function eh(a,b){var c=!dh(a,b);c?Zg(a,b):ah(a,b);return c};function fh(a){Xb.call(this);this.qb=a;this.Fa={}}r(fh,Xb);var gh=[];f=fh.prototype;f.A=function(a,b,c,d){la(b)||(b&&(gh[0]=b.toString()),b=gh);for(var e=0;e<b.length;e++){var g=v(a,b[e],c||this.handleEvent,d||!1,this.qb||this);if(!g)break;this.Fa[g.key]=g}return this};f.Hn=function(a,b,c,d){return hh(this,a,b,c,d)};function hh(a,b,c,d,e,g){if(la(c))for(var h=0;h<c.length;h++)hh(a,b,c[h],d,e,g);else{b=cd(b,c,d||a.handleEvent,e,g||a.qb||a);if(!b)return a;a.Fa[b.key]=b}return a}
f.Aa=function(a,b,c,d,e){if(la(b))for(var g=0;g<b.length;g++)this.Aa(a,b[g],c,d,e);else c=c||this.handleEvent,e=e||this.qb||this,c=Xc(c),d=!!d,b=Lc(a)?a.Lh(b,c,d,e):a?(a=Zc(a))?a.Lh(b,c,d,e):null:null,b&&(ed(b),delete this.Fa[b.key]);return this};f.ld=function(){Db(this.Fa,function(a,b){this.Fa.hasOwnProperty(b)&&ed(a)},this);this.Fa={}};f.V=function(){fh.m.V.call(this);this.ld()};f.handleEvent=function(){throw Error("EventHandler.handleEvent not implemented");};function ih(){}ka(ih);ih.prototype.pv=0;function jh(a){return":"+(a.pv++).toString(36)};function kh(a){md.call(this);this.R=a||jf();this.Cd=lh;this.Ic=null;this.la=!1;this.F=null;this.Ke=void 0;this.Vc=this.gc=this.Wb=this.Ua=null;this.Fs=!1}r(kh,md);kh.prototype.dv=ih.va();var lh=null;
function mh(a,b){switch(a){case 1:return b?"disable":"enable";case 2:return b?"highlight":"unhighlight";case 4:return b?"activate":"deactivate";case 8:return b?"select":"unselect";case 16:return b?"check":"uncheck";case 32:return b?"focus":"blur";case 64:return b?"open":"close"}throw Error("Invalid component state");}function nh(a){return a.Ic||(a.Ic=jh(a.dv))}function oh(a,b){a.Wb&&a.Wb.Vc&&(Mb(a.Wb.Vc,a.Ic),Nb(a.Wb.Vc,b,a));a.Ic=b}f=kh.prototype;f.o=function(){return this.F};
f.xm=function(a){return this.F?this.R.xm(a,this.F):[]};f.Ej=function(a){return this.F?this.R.Ej(a,this.F):null};f.Sa=function(){this.Ke||(this.Ke=new fh(this));return this.Ke};function ph(a,b){if(a==b)throw Error("Unable to set parent component");if(b&&a.Wb&&a.Ic&&qh(a.Wb,a.Ic)&&a.Wb!=b)throw Error("Unable to set parent component");a.Wb=b;kh.m.Gk.call(a,b)}f.getParent=function(){return this.Wb};f.Gk=function(a){if(this.Wb&&this.Wb!=a)throw Error("Method not supported");kh.m.Gk.call(this,a)};f.J=function(){return this.R};
f.D=function(){this.F=this.R.createElement("DIV")};f.render=function(a){this.ji(a)};f.Wf=function(a){this.ji(a.parentNode,a)};f.ji=function(a,b){if(this.la)throw Error("Component already rendered");this.F||this.D();a?a.insertBefore(this.F,b||null):this.R.P.body.appendChild(this.F);this.Wb&&!this.Wb.la||this.Ea()};
f.M=function(a){if(this.la)throw Error("Component already rendered");if(a&&this.wb(a)){this.Fs=!0;var b=lf(a);this.R&&this.R.P==b||(this.R=jf(a));this.Ga(a);this.Ea()}else throw Error("Invalid element to decorate");};f.wb=function(){return!0};f.Ga=function(a){this.F=a};f.Ea=function(){this.la=!0;rh(this,function(a){!a.la&&a.o()&&a.Ea()})};f.fb=function(){rh(this,function(a){a.la&&a.fb()});this.Ke&&this.Ke.ld();this.la=!1};
f.V=function(){this.la&&this.fb();this.Ke&&(this.Ke.eb(),delete this.Ke);rh(this,function(a){a.eb()});!this.Fs&&this.F&&Ff(this.F);this.Wb=this.Ua=this.F=this.Vc=this.gc=null;kh.m.V.call(this)};f.$a=function(a,b){this.Cl(a,sh(this),b)};
f.Cl=function(a,b,c){if(a.la&&(c||!this.la))throw Error("Component already rendered");if(0>b||b>sh(this))throw Error("Child component index out of bounds");this.Vc&&this.gc||(this.Vc={},this.gc=[]);if(a.getParent()==this){var d=nh(a);this.Vc[d]=a;ob(this.gc,a)}else Nb(this.Vc,nh(a),a);ph(a,this);tb(this.gc,b,0,a);a.la&&this.la&&a.getParent()==this?(c=this.Eb(),b=c.childNodes[b]||null,b!=a.o()&&c.insertBefore(a.o(),b)):c?(this.F||this.D(),b=th(this,b+1),a.ji(this.Eb(),b?b.F:null)):this.la&&!a.la&&
a.F&&a.F.parentNode&&1==a.F.parentNode.nodeType&&a.Ea()};f.Eb=function(){return this.F};f.Zj=function(){null==this.Cd&&(this.Cd=vg(this.la?this.F:this.R.P.body));return this.Cd};f.Zg=function(a){if(this.la)throw Error("Component already rendered");this.Cd=a};f.en=function(){return!!this.gc&&0!=this.gc.length};function sh(a){return a.gc?a.gc.length:0}function uh(a){var b=[];rh(a,function(a){b.push(nh(a))});return b}function qh(a,b){return a.Vc&&b?Ob(a.Vc,b)||null:null}
function th(a,b){return a.gc?a.gc[b]||null:null}function rh(a,b,c){a.gc&&eb(a.gc,b,c)}function vh(a,b){return a.gc&&b?db(a.gc,b):-1}f.removeChild=function(a,b){if(a){var c=da(a)?a:nh(a);a=qh(this,c);c&&a&&(Mb(this.Vc,c),ob(this.gc,a),b&&(a.fb(),a.F&&Ff(a.F)),ph(a,null))}if(!a)throw Error("Child is not in parent component");return a};f.Ug=function(a){for(var b=[];this.en();)b.push(this.removeChild(th(this,0),a));return b};function wh(){this.tag="div";this.Jd=[]}wh.prototype.D=function(a){return a.J().H(this.tag,this.pd(a),a.ge())};wh.prototype.Zc=function(a){return this.Jd.concat(a.gq)};wh.prototype.pd=function(a){return{id:nh(a),"class":this.Zc(a).join(" ")}};ka(wh);function xh(a){null==a&&(a={});null==a.l&&(a.l=new Cg);this.l=a.l;xh.m.constructor.call(this,a.l);this.S=a.S;this.caption=a.caption;null==this.S&&(this.S=wh.va());this.gq=[];this.Xf=!0;null==this.caption&&(this.caption="")}r(xh,kh);f=xh.prototype;f.addClass=function(a){return this.gq.push(a)};f.ge=function(){return this.caption};f.Yr=function(a){this.caption=a};f.D=function(){var a;if(!this.la&&(null!=this.S?this.F=a=this.S.D(this):xh.m.D.call(this),this.Xf))return rh(this,function(b){b.D();return a.appendChild(b.o())})};
f.render=function(a){var b=this;null==a&&(a=null);return this.Xe(function(c){return null!=a?a.insertBefore(c,null):b.l.P.body.appendChild(c)})};f.Wf=function(a){var b=this;return this.Xe(function(c){return null!=(null!=a?a.parentNode:void 0)?a.parentNode.insertBefore(c,a):b.l.P.body.appendChild(c)})};f.io=function(a){var b=this;return this.Xe(function(c){return b.l.Vj(c,a)})};f.ji=function(a,b){var c=this;null==b&&(b=null);return this.Xe(function(d){return null!=a?a.insertBefore(d,b):c.l.P.body.appendChild(d)})};
f.Xe=function(a){if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());return this.Ea()};f.Ea=function(){xh.m.Ea.call(this);return this.jh()};f.jh=function(){};f.destroy=function(){this.fb();return this.l.removeNode(this.o())};function yh(a,b){this.Jd=a;this.Yt=null!=b?b:"×"}r(yh,wh);yh.prototype.D=function(a){return a.J().H("div",{"class":[this.Jd,"btn-icon"].join(" ")},this.Yt)};function zh(a){null==a&&(a={});null==a.S&&(a.S=new yh(a.kf));zh.m.constructor.call(this,a);this.addClass(a.kf)}r(zh,xh);zh.prototype.jh=function(){var a=this;v(this.o(),"click",function(b){b.preventDefault();return a.dispatchEvent("action")})};zh.prototype.wb=function(a){return null!=a&&dh(a,this.kf)};zh.prototype.Ga=function(a){this.F=a};function Ah(a){null==a&&(a={});null==a.kf&&(a.kf=Bh);Ah.m.constructor.call(this,a);null!=a.Rc&&(this.Rc=a.Rc)}r(Ah,zh);var Bh="goog-icon-remove close";function Ch(){Ch.m.constructor.call(this);this.Jd.push(Dh);this.Jd.push(Eh)}r(Ch,wh);var Fh={info:"alert-info",success:"alert-success",warn:"alert-warning",error:"alert-danger"},Dh="flash-msg",Eh="alert";Ch.prototype.M=function(a){var b;Zg(a,Eh);var c=[];for(b in Fh){var d=Fh[b];dh(a,b)?c.push(Zg(a,d)):c.push(void 0)}return c};Ch.prototype.D=function(a){var b=a.J();var c=Ch.m.D.call(this,a);var d=b.H("span","msg-text",c);b.Sc(d,a.kk);return c};Ch.prototype.mark=function(a,b){return Zg(a,Fh[b])};ka(Ch);function Gh(a){null==a.S&&(a.S=Ch.va());Gh.m.constructor.call(this,a);this.kk=a.kk;this.severity=a.severity;this.yj=a.yj;this.pg=a.pg;this.timeout=a.timeout;null==this.Cu&&(this.Cu=!0);null==this.pg&&(this.pg=!0);null==this.timeout&&(this.timeout=1E4)}r(Gh,xh);Gh.prototype.D=function(){Gh.m.D.call(this);this.pg&&Hh(this,this.o());this.yj&&Ih(this,this.o());var a=this.o();a=null!=this.severity?this.S.mark(a,this.severity):void 0;return a};Gh.prototype.wb=function(a){return null!=a&&dh(a,Dh)};
function Hh(a,b){var c=new Ah({l:a.l});c.A("action",function(){return a.destroy()});c.D();b.appendChild(c.o());c.Ea()}Gh.prototype.Ga=function(a){Gh.m.Ga.call(this,a);this.S.M(a);dh(a,"error")||dh(a,Fh["alert-danger"])||Hh(this,a);if(!dh(a,"error")&&!dh(a,"ho-hide"))return Ih(this,a)};function Ih(a,b){return setTimeout(function(){var c=new Wg(b,1E3);c.A("end",function(){return a.destroy()});return c.play()},a.timeout)};function Jh(){Jh.m.constructor.call(this);this.Jd.push(Kh)}r(Jh,wh);var Kh="flash";ka(Jh);function Lh(a){null==a.S&&(a.S=Jh.va());this.Eo=[];Lh.m.constructor.call(this,a)}r(Lh,xh);f=Lh.prototype;f.fw=function(a,b,c){null==b&&(b=void 0);null==c&&(c=!0);return this.ih(a,"success",b,c)};f.ih=function(a,b,c,d,e){var g;null==b&&(b="info");null==c&&(c=void 0);null==d&&(d=!0);null==e&&(e=1E4);void 0===c&&(c="error"!==b);var h=la(a)?a:[a];a=[];var k=0;for(g=h.length;k<g;k++){var l=h[k];l=new Gh({l:this.l,kk:l,severity:b,yj:c,pg:d,timeout:e});this.Eo.push(l);a.push(l);this.$a(l)}return a};
f.$a=function(a){return a.render(this.o())};function Mh(a,b){return a.ih(b,"error")}f.wb=function(a){return null!=a&&dh(a,Kh)};f.Ga=function(a){var b;Lh.m.Ga.call(this,a);var c=this.l.he(a);var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a=new Gh({l:this.l});d.push(a.M(g))}return d};f.D=function(){Lh.m.D.call(this);return this.Ga(this.o())};f.$i=function(){var a;var b=this.Eo;var c=0;for(a=b.length;c<a;c++){var d=b[c];d.destroy()}return this.Eo=[]};Cb("Firefox");var Nh=bc()||Cb("iPod"),Oh=Cb("iPad");!Cb("Android")||Vb()||Cb("Firefox")||Cb("Opera")||Cb("Silk");Vb();Sb()&&cc();var Ph=!t&&!Sb();function Rh(a,b,c){Ph&&a.dataset?a.dataset[b]=c:a.setAttribute("data-"+$a(b),c)}function Sh(a,b){return Ph&&a.dataset?b in a.dataset?a.dataset[b]:null:a.getAttribute("data-"+$a(b))}function Th(a,b){return Ph&&a.dataset?b in a.dataset:a.hasAttribute?a.hasAttribute("data-"+$a(b)):!!a.getAttribute("data-"+$a(b))};function Uh(a,b,c,d,e,g){if(!(t||hc||kc&&uc("525")))return!0;if(mc&&e)return Vh(a);if(e&&!d)return!1;ea(b)&&(b=Wh(b));e=17==b||18==b||mc&&91==b;if((!c||mc)&&e||mc&&16==b&&(d||g))return!1;if((kc||hc)&&d&&c)switch(a){case 220:case 219:case 221:case 192:case 186:case 189:case 187:case 188:case 190:case 191:case 192:case 222:return!1}if(t&&d&&b==a)return!1;switch(a){case 13:return!0;case 27:return!(kc||hc)}return Vh(a)}
function Vh(a){if(48<=a&&57>=a||96<=a&&106>=a||65<=a&&90>=a||(kc||hc)&&0==a)return!0;switch(a){case 32:case 43:case 63:case 64:case 107:case 109:case 110:case 111:case 186:case 59:case 189:case 187:case 61:case 188:case 190:case 191:case 192:case 222:case 219:case 220:case 221:return!0;default:return!1}}function Wh(a){if(jc)a=Xh(a);else if(mc&&kc)switch(a){case 93:a=91}return a}
function Xh(a){switch(a){case 61:return 187;case 59:return 186;case 173:return 189;case 224:return 91;case 0:return 224;default:return a}};function Yh(a,b){md.call(this);a&&this.jf(a,b)}r(Yh,md);f=Yh.prototype;f.F=null;f.ak=null;f.zn=null;f.bk=null;f.Jc=-1;f.Re=-1;f.El=!1;
var Zh={3:13,12:144,63232:38,63233:40,63234:37,63235:39,63236:112,63237:113,63238:114,63239:115,63240:116,63241:117,63242:118,63243:119,63244:120,63245:121,63246:122,63247:123,63248:44,63272:46,63273:36,63275:35,63276:33,63277:34,63289:144,63302:45},$h={Up:38,Down:40,Left:37,Right:39,Enter:13,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,"U+007F":46,Home:36,End:35,PageUp:33,PageDown:34,Insert:45},ai=t||hc||kc&&uc("525"),bi=mc&&jc;f=Yh.prototype;
f.Ou=function(a){if(kc||hc)if(17==this.Jc&&!a.ctrlKey||18==this.Jc&&!a.altKey||mc&&91==this.Jc&&!a.metaKey)this.Re=this.Jc=-1;-1==this.Jc&&(a.ctrlKey&&17!=a.keyCode?this.Jc=17:a.altKey&&18!=a.keyCode?this.Jc=18:a.metaKey&&91!=a.keyCode&&(this.Jc=91));ai&&!Uh(a.keyCode,this.Jc,a.shiftKey,a.ctrlKey,a.altKey,a.metaKey)?this.handleEvent(a):(this.Re=Wh(a.keyCode),bi&&(this.El=a.altKey))};f.Qu=function(a){this.Re=this.Jc=-1;this.El=a.altKey};
f.handleEvent=function(a){var b=a.Wc,c=b.altKey;if(t&&"keypress"==a.type){var d=this.Re;var e=13!=d&&27!=d?b.keyCode:0}else(kc||hc)&&"keypress"==a.type?(d=this.Re,e=0<=b.charCode&&63232>b.charCode&&Vh(d)?b.charCode:0):gc&&!kc?(d=this.Re,e=Vh(d)?b.keyCode:0):(d=b.keyCode||this.Re,e=b.charCode||0,bi&&(c=this.El),mc&&63==e&&224==d&&(d=191));var g=d=Wh(d);d?63232<=d&&d in Zh?g=Zh[d]:25==d&&a.shiftKey&&(g=9):b.keyIdentifier&&b.keyIdentifier in $h&&(g=$h[b.keyIdentifier]);a=g==this.Jc;this.Jc=g;b=new ci(g,
e,a,b);b.altKey=c;this.dispatchEvent(b)};f.o=function(){return this.F};f.jf=function(a,b){this.bk&&this.detach();this.F=a;this.ak=v(this.F,"keypress",this,b);this.zn=v(this.F,"keydown",this.Ou,b,this);this.bk=v(this.F,"keyup",this.Qu,b,this)};f.detach=function(){this.ak&&(ed(this.ak),ed(this.zn),ed(this.bk),this.bk=this.zn=this.ak=null);this.F=null;this.Re=this.Jc=-1};f.V=function(){Yh.m.V.call(this);this.detach()};
function ci(a,b,c,d){Hc.call(this,d);this.type="key";this.keyCode=a;this.charCode=b;this.repeat=c}r(ci,Hc);var di;Rb("A AREA BUTTON HEAD INPUT LINK MENU META OPTGROUP OPTION PROGRESS STYLE SELECT SOURCE TEXTAREA TITLE TRACK".split(" "));function ei(a,b){b?a.setAttribute("role",b):a.removeAttribute("role")}
function fi(a,b,c){la(c)&&(c=c.join(" "));var d="aria-"+b;""===c||void 0==c?(di||(di={atomic:!1,autocomplete:"none",dropeffect:"none",haspopup:!1,live:"off",multiline:!1,multiselectable:!1,orientation:"vertical",readonly:!1,relevant:"additions text",required:!1,sort:"none",busy:!1,disabled:!1,hidden:!1,invalid:"false"}),c=di,b in c?a.setAttribute(d,c[b]):a.removeAttribute(d)):a.setAttribute(d,c)}function gi(a,b){var c="";b&&(c=b.id);fi(a,"activedescendant",c)};function hi(a){if(a.classList)return a.classList;a=a.className;return da(a)&&a.match(/\S+/g)||[]}function ii(a,b){return a.classList?a.classList.contains(b):nb(hi(a),b)}function ji(a,b){a.classList?a.classList.add(b):ii(a,b)||(a.className+=0<a.className.length?" "+b:b)}function ki(a,b){if(a.classList)eb(b,function(b){ji(a,b)});else{var c={};eb(hi(a),function(a){c[a]=!0});eb(b,function(a){c[a]=!0});a.className="";for(var d in c)a.className+=0<a.className.length?" "+d:d}}
function li(a,b){a.classList?a.classList.remove(b):ii(a,b)&&(a.className=gb(hi(a),function(a){return a!=b}).join(" "))}function mi(a,b){a.classList?eb(b,function(b){li(a,b)}):a.className=gb(hi(a),function(a){return!nb(b,a)}).join(" ")};function ni(){}var oi;ka(ni);var pi={button:"pressed",checkbox:"checked",menuitem:"selected",menuitemcheckbox:"checked",menuitemradio:"checked",radio:"checked",tab:"selected",treeitem:"selected"};f=ni.prototype;f.yf=function(){};f.D=function(a){return a.J().D("DIV",this.Zc(a).join(" "),a.wd())};f.Eb=function(a){return a};f.Fh=function(a,b,c){if(a=a.o?a.o():a){var d=[b];t&&!uc("7")&&(d=qi(hi(a),b),d.push(b));(c?ki:mi)(a,d)}};f.wb=function(){return!0};
f.M=function(a,b){b.id&&oh(a,b.id);var c=this.Eb(b);c&&c.firstChild?ri(a,c.firstChild.nextSibling?sb(c.childNodes):c.firstChild):a.Ab=null;var d=0,e=this.ta(),g=this.ta(),h=!1,k=!1,l=!1,m=sb(hi(b));eb(m,function(a){h||a!=e?k||a!=g?d|=this.Jj(a):k=!0:(h=!0,g==e&&(k=!0));1==this.Jj(a)&&Nf(c)&&Qf(c,!1)},this);a.ma=d;h||(m.push(e),g==e&&(k=!0));k||m.push(g);var p=a.Xc;p&&m.push.apply(m,p);if(t&&!uc("7")){var u=qi(m);0<u.length&&(m.push.apply(m,u),l=!0)}if(!h||!k||p||l)b.className=m.join(" ");return b};
f.Lf=function(a){a.Zj()&&this.Zg(a.o(),!0);a.isEnabled()&&this.ve(a,a.za())};function si(a,b,c){if(a=c||a.yf())c=b.getAttribute("role")||null,a!=c&&ei(b,a)}function ti(a,b,c){var d=b.Dp;null!=d&&a.ro(c,d);b.za()||fi(c,"hidden",!b.za());b.isEnabled()||a.Fd(c,1,!b.isEnabled());ui(b,8)&&a.Fd(c,8,b.Fg());ui(b,16)&&a.Fd(c,16,!!(b.ma&16));ui(b,64)&&a.Fd(c,64,b.Qe())}f.ro=function(a,b){fi(a,"label",b)};f.Wg=function(a,b){xg(a,!b,!t&&!gc)};f.Zg=function(a,b){this.Fh(a,this.ta()+"-rtl",b)};
f.ne=function(a){var b;return ui(a,32)&&(b=a.Fb())?Nf(b):!1};f.ve=function(a,b){var c;if(ui(a,32)&&(c=a.Fb())){if(!b&&a.ma&32){try{c.blur()}catch(d){}a.ma&32&&a.handleBlur(null)}Nf(c)!=b&&Qf(c,b)}};f.da=function(a,b){w(a,b);a&&fi(a,"hidden",!b)};f.setState=function(a,b,c){var d=a.o();if(d){var e=this.Kh(b);e&&this.Fh(a,e,c);this.Fd(d,b,c)}};
f.Fd=function(a,b,c){oi||(oi={1:"disabled",8:"selected",16:"checked",64:"expanded"});b=oi[b];var d=a.getAttribute("role")||null;d&&(d=pi[d]||b,b="checked"==b||"selected"==b?d:b);b&&fi(a,b,c)};f.vc=function(a,b){var c=this.Eb(a);if(c&&(Cf(c),b))if(da(b))Kf(c,b);else{var d=function(a){if(a){var b=lf(c);c.appendChild(da(a)?b.createTextNode(a):a)}};la(b)?eb(b,d):!ma(b)||"nodeType"in b?d(b):eb(sb(b),d)}};f.Fb=function(a){return a.o()};f.ta=function(){return"goog-control"};
f.Zc=function(a){var b=this.ta(),c=[b],d=this.ta();d!=b&&c.push(d);b=a.getState();for(d=[];b;){var e=b&-b;d.push(this.Kh(e));b&=~e}c.push.apply(c,d);(a=a.Xc)&&c.push.apply(c,a);t&&!uc("7")&&c.push.apply(c,qi(c));return c};function qi(a,b){var c=[];b&&(a=rb(a,[b]));eb([],function(d){!kb(d,va(nb,a))||b&&!nb(d,b)||c.push(d.join("_"))});return c}f.Kh=function(a){this.Yi||vi(this);return this.Yi[a]};
f.Jj=function(a){if(!this.ts){this.Yi||vi(this);var b=this.Yi,c={},d;for(d in b)c[b[d]]=d;this.ts=c}a=parseInt(this.ts[a],10);return isNaN(a)?0:a};function vi(a){var b=a.ta();Qa(b.replace(/\xa0|\s/g," ")," ");a.Yi={1:b+"-disabled",2:b+"-hover",4:b+"-active",8:b+"-selected",16:b+"-checked",32:b+"-focused",64:b+"-open"}};function wi(a,b){if(!a)throw Error("Invalid class name "+a);if(!oa(b))throw Error("Invalid decorator function "+b);xi[a]=b}var yi={},xi={};function zi(a,b,c){kh.call(this,c);if(!b){b=this.constructor;for(var d;b;){d=qa(b);if(d=yi[d])break;b=b.m?b.m.constructor:null}b=d?oa(d.va)?d.va():new d:null}this.U=b;this.Ab=ca(a)?a:null;this.Dp=null}r(zi,kh);f=zi.prototype;f.Ab=null;f.ma=0;f.Bi=39;f.mh=255;f.Ai=0;f.La=!0;f.Xc=null;f.Tm=!0;f.Ap=!1;f.ao=null;function Ai(a,b){a.la&&b!=a.Tm&&Bi(a,b);a.Tm=b}f.Fb=function(){return this.U.Fb(this)};f.Fj=function(){return this.Vb||(this.Vb=new Yh)};
function Ci(a,b){b&&(a.Xc?nb(a.Xc,b)||a.Xc.push(b):a.Xc=[b],a.U.Fh(a,b,!0))}f.Fh=function(a,b){b?Ci(this,a):a&&this.Xc&&ob(this.Xc,a)&&(0==this.Xc.length&&(this.Xc=null),this.U.Fh(this,a,!1))};f.D=function(){var a=this.U.D(this);this.F=a;si(this.U,a,this.je());this.Wj()||this.U.Wg(a,!1);this.za()||this.U.da(a,!1)};f.je=function(){return this.ao};f.ro=function(a){this.Dp=a;var b=this.o();b&&this.U.ro(b,a)};f.Eb=function(){return this.U.Eb(this.o())};f.wb=function(a){return this.U.wb(a)};
f.Ga=function(a){this.F=a=this.U.M(this,a);si(this.U,a,this.je());this.Wj()||this.U.Wg(a,!1);this.La="none"!=a.style.display};f.Ea=function(){zi.m.Ea.call(this);ti(this.U,this,this.F);this.U.Lf(this);if(this.Bi&-2&&(this.Tm&&Bi(this,!0),ui(this,32))){var a=this.Fb();if(a){var b=this.Fj();b.jf(a);this.Sa().A(b,"key",this.$c).A(a,"focus",this.handleFocus).A(a,"blur",this.handleBlur)}}};
function Bi(a,b){var c=a.Sa(),d=a.o();b?(c.A(d,"mouseover",a.xg).A(d,"mousedown",a.me).A(d,"mouseup",a.yg).A(d,"mouseout",a.Um),a.Oh!=ja&&c.A(d,"contextmenu",a.Oh),t&&(uc(9)||c.A(d,"dblclick",a.Bq),a.Vh||(a.Vh=new Di(a),$b(a,va(ac,a.Vh))))):(c.Aa(d,"mouseover",a.xg).Aa(d,"mousedown",a.me).Aa(d,"mouseup",a.yg).Aa(d,"mouseout",a.Um),a.Oh!=ja&&c.Aa(d,"contextmenu",a.Oh),t&&(uc(9)||c.Aa(d,"dblclick",a.Bq),ac(a.Vh),a.Vh=null))}
f.fb=function(){zi.m.fb.call(this);this.Vb&&this.Vb.detach();this.za()&&this.isEnabled()&&this.U.ve(this,!1)};f.V=function(){zi.m.V.call(this);this.Vb&&(this.Vb.eb(),delete this.Vb);delete this.U;this.Vh=this.Xc=this.Ab=null};f.wd=function(){return this.Ab};f.vc=function(a){this.U.vc(this.o(),a);this.Ab=a};function ri(a,b){a.Ab=b}f.ge=function(){var a=this.wd();if(!a)return"";a=da(a)?a:la(a)?hb(a,Tf).join(""):Rf(a);return Da(a)};f.Yr=function(a){this.vc(a)};
f.Zg=function(a){zi.m.Zg.call(this,a);var b=this.o();b&&this.U.Zg(b,a)};f.Wj=function(){return this.Ap};f.Wg=function(a){this.Ap=a;var b=this.o();b&&this.U.Wg(b,a)};f.za=function(){return this.La};f.da=function(a,b){if(b||this.La!=a&&this.dispatchEvent(a?"show":"hide")){var c=this.o();c&&this.U.da(c,a);this.isEnabled()&&this.U.ve(this,a);this.La=a;return!0}return!1};f.isEnabled=function(){return!(this.ma&1)};
f.pa=function(a){var b=this.getParent();b&&"function"==typeof b.isEnabled&&!b.isEnabled()||!Ei(this,1,!a)||(a||(this.setActive(!1),this.md(!1)),this.za()&&this.U.ve(this,a),this.setState(1,!a,!0))};f.md=function(a){Ei(this,2,a)&&this.setState(2,a)};f.Eg=function(){return!!(this.ma&4)};f.setActive=function(a){Ei(this,4,a)&&this.setState(4,a)};f.Fg=function(){return!!(this.ma&8)};f.setSelected=function(a){Ei(this,8,a)&&this.setState(8,a)};f.Qe=function(){return!!(this.ma&64)};
f.setOpen=function(a){Ei(this,64,a)&&this.setState(64,a)};f.getState=function(){return this.ma};f.setState=function(a,b,c){c||1!=a?ui(this,a)&&b!=!!(this.ma&a)&&(this.U.setState(this,a,b),this.ma=b?this.ma|a:this.ma&~a):this.pa(!b)};function ui(a,b){return!!(a.Bi&b)}f.mc=function(a,b){if(this.la&&this.ma&a&&!b)throw Error("Component already rendered");!b&&this.ma&a&&this.setState(a,!1);this.Bi=b?this.Bi|a:this.Bi&~a};function Fi(a,b){return!!(a.mh&b)&&ui(a,b)}
function Ei(a,b,c){return ui(a,b)&&!!(a.ma&b)!=c&&(!(a.Ai&b)||a.dispatchEvent(mh(b,c)))&&!a.Ee}f.xg=function(a){!Gi(a,this.o())&&this.dispatchEvent("enter")&&this.isEnabled()&&Fi(this,2)&&this.md(!0)};f.Um=function(a){!Gi(a,this.o())&&this.dispatchEvent("leave")&&(Fi(this,4)&&this.setActive(!1),Fi(this,2)&&this.md(!1))};f.Oh=ja;function Gi(a,b){return!!a.relatedTarget&&If(b,a.relatedTarget)}
f.me=function(a){this.isEnabled()&&(Fi(this,2)&&this.md(!0),Jc(a)&&(Fi(this,4)&&this.setActive(!0),this.U&&this.U.ne(this)&&this.Fb().focus()));!this.Wj()&&Jc(a)&&a.preventDefault()};f.yg=function(a){this.isEnabled()&&(Fi(this,2)&&this.md(!0),this.Eg()&&this.Pg(a)&&Fi(this,4)&&this.setActive(!1))};f.Bq=function(a){this.isEnabled()&&this.Pg(a)};
f.Pg=function(a){if(Fi(this,16)){var b=!(this.ma&16);Ei(this,16,b)&&this.setState(16,b)}Fi(this,8)&&this.setSelected(!0);Fi(this,64)&&this.setOpen(!this.Qe());b=new Cc("action",this);a&&(b.altKey=a.altKey,b.ctrlKey=a.ctrlKey,b.metaKey=a.metaKey,b.shiftKey=a.shiftKey,b.Yn=a.Yn);return this.dispatchEvent(b)};f.handleFocus=function(){Fi(this,32)&&Ei(this,32,!0)&&this.setState(32,!0)};f.handleBlur=function(){Fi(this,4)&&this.setActive(!1);Fi(this,32)&&Ei(this,32,!1)&&this.setState(32,!1)};
f.$c=function(a){return this.za()&&this.isEnabled()&&this.le(a)?(a.preventDefault(),a.stopPropagation(),!0):!1};f.le=function(a){return 13==a.keyCode&&this.Pg(a)};if(!oa(zi))throw Error("Invalid component class "+zi);if(!oa(ni))throw Error("Invalid renderer class "+ni);var Hi=qa(zi);yi[Hi]=ni;wi("goog-control",function(){return new zi(null)});
function Di(a){Xb.call(this);this.mj=a;this.aj=!1;this.qb=new fh(this);$b(this,va(ac,this.qb));a=this.mj.F;this.qb.A(a,"mousedown",this.Ph).A(a,"mouseup",this.Su).A(a,"click",this.Nh)}r(Di,Xb);var Ii=!t||vc(9);Di.prototype.Ph=function(){this.aj=!1};Di.prototype.Su=function(){this.aj=!0};
function Ji(a,b){if(!Ii)return a.button=0,a.type=b,a;var c=document.createEvent("MouseEvents");c.initMouseEvent(b,a.bubbles,a.cancelable,a.view||null,a.detail,a.screenX,a.screenY,a.clientX,a.clientY,a.ctrlKey,a.altKey,a.shiftKey,a.metaKey,0,a.relatedTarget||null);return c}Di.prototype.Nh=function(a){if(this.aj)this.aj=!1;else{var b=a.Wc,c=b.button,d=b.type,e=Ji(b,"mousedown");this.mj.me(new Hc(e,a.currentTarget));e=Ji(b,"mouseup");this.mj.yg(new Hc(e,a.currentTarget));Ii||(b.button=c,b.type=d)}};
Di.prototype.V=function(){this.mj=null;Di.m.V.call(this)};function Ki(){Ki.m.constructor.call(this)}r(Ki,ni);Ki.prototype.M=function(a,b){Ki.m.M.call(this,a,b);a.vc(b.value);return b};Ki.prototype.wb=function(a){return"INPUT"===a.tagName};Ki.prototype.D=function(a){return a.J().H("input",{type:"text","class":this.Zc(a).join(" "),placeholder:a.placeholder})};ka(Ki);function Li(a,b,c){null==b&&(b=Ki.va());null==c&&(c=null);this.Cf=q(this.Cf,this);Li.m.constructor.call(this,a,b,c);Ai(this,!1);this.Wg(!0);null==a&&this.vc("");this.Gi=null;this.placeholder=""}r(Li,zi);function Mi(a,b){a.placeholder=b;null==a.placeholder&&(a.placeholder="");a.o()&&a.o().setAttribute("placeholder",String(a.placeholder))}f=Li.prototype;f.da=function(a,b){null==b&&(b=!1);this.Gi&&w(this.Gi,a);return Li.m.da.call(this,a,b)};
f.ra=function(a){a=String(a);this.vc(a);return this.o().value=a};f.vc=function(a){this.Ab=a;this.dispatchEvent(new Cc("VALUE_CHANGE",this))};f.aa=function(){return this.o().value};f.clear=function(){return this.ra("")};f.Wf=function(a){Li.m.Wf.call(this,a);return this.Ih()};f.render=function(a){Li.m.render.call(this,a);this.Ih();v(this.o(),"keyup",this.Cf)};f.Cf=function(a){var b;return null!=(null!=a?null!=(b=a.target)?b.value:void 0:void 0)?this.vc(a.target.value):this.vc(this.o().value)};
f.Ih=function(){this.Yj(this.o())||(this.Gi=this.o(),this.F=this.Yj(this.Gi)?this.Gi:this.R.he(this.o())[0])};f.Yj=function(a){return"INPUT"===a.tagName};f.hk=function(a){null==a&&(a=!0);return this.o().required=a?"required":void 0};f.destroy=function(){this.fb();return this.l.removeNode(this.o())};function Ni(a,b){this.type=a;return this.currentTarget=this.target=b||null}f=Ni.prototype;f.jd=!1;f.defaultPrevented=!1;f.mi=!0;f.stopPropagation=function(){this.jd=!0};f.preventDefault=function(){this.defaultPrevented=!0;this.mi=!1};function Oi(a){this.l=a;this.Me=q(this.Me,this);Oi.m.constructor.call(this,null,null,this.l);this.Yd=new Ah({l:this.l});this.Yd.A("action",this.Me)}r(Oi,Li);Oi.prototype.render=function(a){this.$d=a;Oi.m.render.call(this,this.$d);this.Yd.render(this.$d);w(this.Yd.o(),!1);this.F=this.o()};
Oi.prototype.Me=function(){this.ra("");w(this.Yd.o(),!1);var a=new Ni("change",this.o()),b;a.mi=!0;var c=[];for(b=a.target;b;)c.push(b),b=b.parentNode;for(b=c.length-1;0<=b&&!a.jd;)hd(c[b],a.type,!0,new Hc(a,c[b])),b--;for(b=0;b<c.length&&!a.jd;)hd(c[b],a.type,!1,new Hc(a,c[b])),b++;return this.dispatchEvent(new Cc("clear",this))};function Pi(a){if(!a.o().getAttribute("readonly"))return w(a.Yd.o(),!0)}
Oi.prototype.Cf=function(a){Oi.m.Cf.call(this,a);if(this.aa())return Pi(this);this.dispatchEvent(new Cc("clear",this));return w(this.Yd.o(),!1)};var Qi={il:["př. n. l.","n. l."],hp:["př. n. l.","n. l."],lp:"1 2 3 4 5 6 7 8 9 10 11 12".split(" "),op:"1 2 3 4 5 6 7 8 9 10 11 12".split(" "),ml:"ledna února března dubna května června července srpna září října listopadu prosince".split(" "),eg:"leden únor březen duben květen červen červenec srpen září říjen listopad prosinec".split(" "),ql:"led úno bře dub kvě čvn čvc srp zář říj lis pro".split(" "),tl:"led úno bře dub kvě čvn čvc srp zář říj lis pro".split(" "),wl:"neděle pondělí úterý středa čtvrtek pátek sobota".split(" "),
qp:"neděle pondělí úterý středa čtvrtek pátek sobota".split(" "),sl:"ne po út st čt pá so".split(" "),vl:"ne po út st čt pá so".split(" "),it:"NPÚSČPS".split(""),pp:"NPÚSČPS".split(""),rl:["Q1","Q2","Q3","Q4"],pl:["1. čtvrtletí","2. čtvrtletí","3. čtvrtletí","4. čtvrtletí"],fl:["dop.","odp."],dg:["EEEE d. MMMM y","d. MMMM y","d. M. y","dd.MM.yy"],gh:["H:mm:ss zzzz","H:mm:ss z","H:mm:ss","H:mm"],hl:["{1} {0}","{1} {0}","{1} {0}","{1} {0}"],Ii:0,xl:[5,6],Ji:3},Ri={il:["BC","AD"],hp:["Before Christ",
"Anno Domini"],lp:"JFMAMJJASOND".split(""),op:"JFMAMJJASOND".split(""),ml:"January February March April May June July August September October November December".split(" "),eg:"January February March April May June July August September October November December".split(" "),ql:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" "),tl:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" "),wl:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),qp:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),
sl:"Sun Mon Tue Wed Thu Fri Sat".split(" "),vl:"Sun Mon Tue Wed Thu Fri Sat".split(" "),it:"SMTWTFS".split(""),pp:"SMTWTFS".split(""),rl:["Q1","Q2","Q3","Q4"],pl:["1st quarter","2nd quarter","3rd quarter","4th quarter"],fl:["AM","PM"],dg:["EEEE, MMMM d, y","MMMM d, y","MMM d, y","M/d/yy"],gh:["h:mm:ss a zzzz","h:mm:ss a z","h:mm:ss a","h:mm a"],hl:["{1} 'at' {0}","{1} 'at' {0}","{1}, {0}","{1}, {0}"],Ii:6,xl:[5,6],Ji:5},Si=Ri,Si=Ri;function Ti(a,b){switch(b){case 1:return 0!=a%4||0==a%100&&0!=a%400?28:29;case 5:case 8:case 10:case 3:return 30}return 31}function Ui(a,b,c,d,e){a=new Date(a,b,c);d=ca(d)?d:3;e=e||0;b=((a.getDay()+6)%7-e+7)%7;e=a.valueOf()+864E5*((d-e+7)%7-b);d=(new Date((new Date(e)).getFullYear(),0,1)).valueOf();return Math.floor(Math.round((e-d)/864E5)/7)+1}
function Wi(a,b,c,d,e,g){da(a)?(this.years=a==Xi?b:0,this.months=a==Yi?b:0,this.days=a==Zi?b:0,this.hours=a==$i?b:0,this.minutes=a==aj?b:0,this.seconds=a==bj?b:0):(this.years=a||0,this.months=b||0,this.days=c||0,this.hours=d||0,this.minutes=e||0,this.seconds=g||0)}
Wi.prototype.toIsoString=function(a){var b=Math.min(this.years,this.months,this.days,this.hours,this.minutes,this.seconds),c=Math.max(this.years,this.months,this.days,this.hours,this.minutes,this.seconds);if(0>b&&0<c)return null;if(!a&&0==b&&0==c)return"PT0S";c=[];0>b&&c.push("-");c.push("P");(this.years||a)&&c.push(Math.abs(this.years)+"Y");(this.months||a)&&c.push(Math.abs(this.months)+"M");(this.days||a)&&c.push(Math.abs(this.days)+"D");if(this.hours||this.minutes||this.seconds||a)c.push("T"),
(this.hours||a)&&c.push(Math.abs(this.hours)+"H"),(this.minutes||a)&&c.push(Math.abs(this.minutes)+"M"),(this.seconds||a)&&c.push(Math.abs(this.seconds)+"S");return c.join("")};Wi.prototype.pb=function(a){return a.years==this.years&&a.months==this.months&&a.days==this.days&&a.hours==this.hours&&a.minutes==this.minutes&&a.seconds==this.seconds};Wi.prototype.clone=function(){return new Wi(this.years,this.months,this.days,this.hours,this.minutes,this.seconds)};
var Xi="y",Yi="m",Zi="d",$i="h",aj="n",bj="s";Wi.prototype.Ub=function(){return 0==this.years&&0==this.months&&0==this.days&&0==this.hours&&0==this.minutes&&0==this.seconds};Wi.prototype.add=function(a){this.years+=a.years;this.months+=a.months;this.days+=a.days;this.hours+=a.hours;this.minutes+=a.minutes;this.seconds+=a.seconds};
function cj(a,b,c){ea(a)?(this.date=dj(a,b||0,c||1),ej(this,c||1)):pa(a)?(this.date=dj(a.getFullYear(),a.getMonth(),a.getDate()),ej(this,a.getDate())):(this.date=new Date(wa()),a=this.date.getDate(),this.date.setHours(0),this.date.setMinutes(0),this.date.setSeconds(0),this.date.setMilliseconds(0),ej(this,a))}function dj(a,b,c){b=new Date(a,b,c);0<=a&&100>a&&b.setFullYear(b.getFullYear()-1900);return b}f=cj.prototype;f.Yc=Si.Ii;f.xf=Si.Ji;
f.clone=function(){var a=new cj(this.date);a.Yc=this.Yc;a.xf=this.xf;return a};f.getFullYear=function(){return this.date.getFullYear()};f.getYear=function(){return this.getFullYear()};f.getMonth=function(){return this.date.getMonth()};f.getDate=function(){return this.date.getDate()};f.getTime=function(){return this.date.getTime()};f.getDay=function(){return this.date.getDay()};f.getUTCFullYear=function(){return this.date.getUTCFullYear()};f.getUTCMonth=function(){return this.date.getUTCMonth()};
f.getUTCDate=function(){return this.date.getUTCDate()};f.getUTCDay=function(){return this.date.getDay()};f.getUTCHours=function(){return this.date.getUTCHours()};f.getUTCMinutes=function(){return this.date.getUTCMinutes()};function fj(a){return Ti(a.getFullYear(),a.getMonth())}f.getTimezoneOffset=function(){return this.date.getTimezoneOffset()};function gj(a){a=a.getTimezoneOffset();if(0==a)a="Z";else{var b=Math.abs(a)/60,c=Math.floor(b),b=60*(b-c);a=(0<a?"-":"+")+Wa(c,2)+":"+Wa(b,2)}return a}
f.set=function(a){this.date=new Date(a.getFullYear(),a.getMonth(),a.getDate())};f.setFullYear=function(a){this.date.setFullYear(a)};f.setYear=function(a){this.setFullYear(a)};f.setMonth=function(a){this.date.setMonth(a)};f.setDate=function(a){this.date.setDate(a)};f.setTime=function(a){this.date.setTime(a)};f.setUTCFullYear=function(a){this.date.setUTCFullYear(a)};f.setUTCMonth=function(a){this.date.setUTCMonth(a)};f.setUTCDate=function(a){this.date.setUTCDate(a)};
f.add=function(a){if(a.years||a.months){var b=this.getMonth()+a.months+12*a.years,c=this.getYear()+Math.floor(b/12),b=b%12;0>b&&(b+=12);var d=Math.min(Ti(c,b),this.getDate());this.setDate(1);this.setFullYear(c);this.setMonth(b);this.setDate(d)}a.days&&(b=new Date(this.getYear(),this.getMonth(),this.getDate(),12),a=new Date(b.getTime()+864E5*a.days),this.setDate(1),this.setFullYear(a.getFullYear()),this.setMonth(a.getMonth()),this.setDate(a.getDate()),ej(this,a.getDate()))};
f.toIsoString=function(a,b){return[this.getFullYear(),Wa(this.getMonth()+1,2),Wa(this.getDate(),2)].join(a?"-":"")+(b?gj(this):"")};f.pb=function(a){return!(!a||this.getYear()!=a.getYear()||this.getMonth()!=a.getMonth()||this.getDate()!=a.getDate())};f.toString=function(){return this.toIsoString()};function ej(a,b){if(a.getDate()!=b){var c=a.getDate()<b?1:-1;a.date.setUTCHours(a.date.getUTCHours()+c)}}f.valueOf=function(){return this.date.valueOf()};
function hj(a,b,c,d,e,g,h){this.date=ea(a)?new Date(a,b||0,c||1,d||0,e||0,g||0,h||0):new Date(a&&a.getTime?a.getTime():wa())}r(hj,cj);f=hj.prototype;f.getHours=function(){return this.date.getHours()};f.getMinutes=function(){return this.date.getMinutes()};f.getSeconds=function(){return this.date.getSeconds()};f.getMilliseconds=function(){return this.date.getMilliseconds()};f.getUTCDay=function(){return this.date.getUTCDay()};f.getUTCHours=function(){return this.date.getUTCHours()};
f.getUTCMinutes=function(){return this.date.getUTCMinutes()};f.getUTCSeconds=function(){return this.date.getUTCSeconds()};f.getUTCMilliseconds=function(){return this.date.getUTCMilliseconds()};f.setHours=function(a){this.date.setHours(a)};f.setMinutes=function(a){this.date.setMinutes(a)};f.setSeconds=function(a){this.date.setSeconds(a)};f.setMilliseconds=function(a){this.date.setMilliseconds(a)};f.setUTCHours=function(a){this.date.setUTCHours(a)};f.setUTCMinutes=function(a){this.date.setUTCMinutes(a)};
f.setUTCSeconds=function(a){this.date.setUTCSeconds(a)};f.setUTCMilliseconds=function(a){this.date.setUTCMilliseconds(a)};f.add=function(a){cj.prototype.add.call(this,a);a.hours&&this.setUTCHours(this.date.getUTCHours()+a.hours);a.minutes&&this.setUTCMinutes(this.date.getUTCMinutes()+a.minutes);a.seconds&&this.setUTCSeconds(this.date.getUTCSeconds()+a.seconds)};
f.toIsoString=function(a,b){var c=cj.prototype.toIsoString.call(this,a);return a?c+" "+Wa(this.getHours(),2)+":"+Wa(this.getMinutes(),2)+":"+Wa(this.getSeconds(),2)+(b?gj(this):""):c+"T"+Wa(this.getHours(),2)+Wa(this.getMinutes(),2)+Wa(this.getSeconds(),2)+(b?gj(this):"")};f.pb=function(a){return this.getTime()==a.getTime()};f.toString=function(){return this.toIsoString()};f.clone=function(){var a=new hj(this.date);a.Yc=this.Yc;a.xf=this.xf;return a};function ij(){}
function jj(a){if("number"==typeof a){var b=new ij;b.Mk=a;var c=a;if(0==c)c="Etc/GMT";else{var d=["Etc/GMT",0>c?"-":"+"];c=Math.abs(c);d.push(Math.floor(c/60)%100);c%=60;0!=c&&d.push(":",Wa(c,2));c=d.join("")}b.Mo=c;c=a;0==c?c="UTC":(d=["UTC",0>c?"+":"-"],c=Math.abs(c),d.push(Math.floor(c/60)%100),c%=60,0!=c&&d.push(":",c),c=d.join(""));a=kj(a);b.Po=[c,c];b.$f={cz:a,sp:a};b.Wk=[];return b}b=new ij;b.Mo=a.id;b.Mk=-a.std_offset;b.Po=a.names;b.$f=a.names_ext;b.Wk=a.transitions;return b}
function kj(a){var b=["GMT"];b.push(0>=a?"+":"-");a=Math.abs(a);b.push(Wa(Math.floor(a/60)%100,2),":",Wa(a%60,2));return b.join("")}function lj(a,b){for(var c=Date.UTC(b.getUTCFullYear(),b.getUTCMonth(),b.getUTCDate(),b.getUTCHours(),b.getUTCMinutes())/36E5,d=0;d<a.Wk.length&&c>=a.Wk[d];)d+=2;return 0==d?0:a.Wk[d-1]};function mj(a,b){this.Va=[];this.ca=b||Si;"number"==typeof a?this.Qi(a):this.lh(a)}var nj=[/^\'(?:[^\']|\'\')*(\'|$)/,/^(?:G+|y+|M+|k+|S+|E+|a+|h+|K+|H+|c+|L+|Q+|d+|m+|s+|v+|V+|w+|z+|Z+)/,/^[^\'GyMkSEahKHcLQdmsvVwzZ]+/];function oj(a){return a.getHours?a.getHours():0}
mj.prototype.lh=function(a){for(pj&&(a=a.replace(/\u200f/g,""));a;){for(var b=a,c=0;c<nj.length;++c){var d=a.match(nj[c]);if(d){var e=d[0];a=a.substring(e.length);0==c&&("''"==e?e="'":(e=e.substring(1,"'"==d[1]?e.length-1:e.length),e=e.replace(/\'\'/g,"'")));this.Va.push({text:e,type:c});break}}if(b===a)throw Error("Malformed pattern part: "+a);}};
mj.prototype.format=function(a,b){if(!a)throw Error("The date to format must be non-null.");var c=b?6E4*(a.getTimezoneOffset()-(b.Mk-lj(b,a))):0,d=c?new Date(a.getTime()+c):a,e=d;b&&d.getTimezoneOffset()!=a.getTimezoneOffset()&&(d=new Date(d.getTime()+6E4*(d.getTimezoneOffset()-a.getTimezoneOffset())),e=new Date(a.getTime()+(c+(0<c?-864E5:864E5))));for(var c=[],g=0;g<this.Va.length;++g){var h=this.Va[g].text;1==this.Va[g].type?c.push(qj(this,h,a,d,e,b)):c.push(h)}return c.join("")};
mj.prototype.Qi=function(a){if(4>a)var b=this.ca.dg[a];else if(8>a)b=this.ca.gh[a-4];else if(12>a)b=this.ca.hl[a-8],b=b.replace("{1}",this.ca.dg[a-8]),b=b.replace("{0}",this.ca.gh[a-8]);else{this.Qi(10);return}this.lh(b)};function rj(a,b){var c=String(b);var d=a.ca||Si;if(void 0!==d.yl){for(var e=[],g=0;g<c.length;g++){var h=c.charCodeAt(g);e.push(48<=h&&57>=h?String.fromCharCode(d.yl+h-48):c.charAt(g))}c=e.join("")}return c}var pj=!1;
function sj(a){if(!(a.getHours&&a.getSeconds&&a.getMinutes))throw Error("The date to format has no time (probably a goog.date.Date). Use Date or goog.date.DateTime, or use a pattern without time fields.");}
function qj(a,b,c,d,e,g){var h=b.length;switch(b.charAt(0)){case "G":return c=0<d.getFullYear()?1:0,4<=h?a.ca.hp[c]:a.ca.il[c];case "y":return c=d.getFullYear(),0>c&&(c=-c),2==h&&(c%=100),rj(a,Wa(c,h));case "M":a:switch(c=d.getMonth(),h){case 5:h=a.ca.lp[c];break a;case 4:h=a.ca.ml[c];break a;case 3:h=a.ca.ql[c];break a;default:h=rj(a,Wa(c+1,h))}return h;case "k":return sj(e),c=oj(e)||24,rj(a,Wa(c,h));case "S":return rj(a,(e.getTime()%1E3/1E3).toFixed(Math.min(3,h)).substr(2)+(3<h?Wa(0,h-3):""));
case "E":return c=d.getDay(),4<=h?a.ca.wl[c]:a.ca.sl[c];case "a":return sj(e),h=oj(e),a.ca.fl[12<=h&&24>h?1:0];case "h":return sj(e),c=oj(e)%12||12,rj(a,Wa(c,h));case "K":return sj(e),c=oj(e)%12,rj(a,Wa(c,h));case "H":return sj(e),c=oj(e),rj(a,Wa(c,h));case "c":a:switch(c=d.getDay(),h){case 5:h=a.ca.pp[c];break a;case 4:h=a.ca.qp[c];break a;case 3:h=a.ca.vl[c];break a;default:h=rj(a,Wa(c,1))}return h;case "L":a:switch(c=d.getMonth(),h){case 5:h=a.ca.op[c];break a;case 4:h=a.ca.eg[c];break a;case 3:h=
a.ca.tl[c];break a;default:h=rj(a,Wa(c+1,h))}return h;case "Q":return c=Math.floor(d.getMonth()/3),4>h?a.ca.rl[c]:a.ca.pl[c];case "d":return rj(a,Wa(d.getDate(),h));case "m":return sj(e),rj(a,Wa(e.getMinutes(),h));case "s":return sj(e),rj(a,Wa(e.getSeconds(),h));case "v":return h=g||jj(c.getTimezoneOffset()),h.Mo;case "V":return a=g||jj(c.getTimezoneOffset()),2>=h?a.Mo:0<lj(a,c)?ca(a.$f.Ws)?a.$f.Ws:a.$f.DST_GENERIC_LOCATION:ca(a.$f.sp)?a.$f.sp:a.$f.STD_GENERIC_LOCATION;case "w":return c=Ui(e.getFullYear(),
e.getMonth(),e.getDate(),a.ca.Ji,a.ca.Ii),rj(a,Wa(c,h));case "z":return a=g||jj(c.getTimezoneOffset()),4>h?a.Po[0<lj(a,c)?2:0]:a.Po[0<lj(a,c)?3:1];case "Z":return b=g||jj(c.getTimezoneOffset()),4>h?(h=-(b.Mk-lj(b,c)),a=[0>h?"-":"+"],h=Math.abs(h),a.push(Wa(Math.floor(h/60)%100,2),Wa(h%60,2)),h=a.join("")):h=rj(a,kj(b.Mk-lj(b,c))),h;default:return""}};function tj(a,b){this.Va=[];this.ca=b||Si;"number"==typeof a?this.Qi(a):this.lh(a)}
tj.prototype.lh=function(a){for(var b=!1,c="",d=0;d<a.length;d++){var e=a.charAt(d);if(" "==e)for(0<c.length&&(this.Va.push({text:c,count:0,gg:!1}),c=""),this.Va.push({text:" ",count:0,gg:!1});d<a.length-1&&" "==a.charAt(d+1);)d++;else if(b)"'"==e?d+1<a.length&&"'"==a.charAt(d+1)?(c+="'",d++):b=!1:c+=e;else if(0<=uj.indexOf(e)){0<c.length&&(this.Va.push({text:c,count:0,gg:!1}),c="");var g=a.charAt(d);for(var h=d+1;h<a.length&&a.charAt(h)==g;)h++;g=h-d;this.Va.push({text:e,count:g,gg:!1});d+=g-1}else"'"==
e?d+1<a.length&&"'"==a.charAt(d+1)?(c+="'",d++):b=!0:c+=e}0<c.length&&this.Va.push({text:c,count:0,gg:!1});a=!1;for(b=0;b<this.Va.length;b++)vj(this.Va[b])?!a&&b+1<this.Va.length&&vj(this.Va[b+1])&&(a=!0,this.Va[b].gg=!0):a=!1};tj.prototype.Qi=function(a){11<a&&(a=10);if(4>a)var b=this.ca.dg[a];else 8>a?b=this.ca.gh[a-4]:(b=this.ca.hl[a-8],b=b.replace("{1}",this.ca.dg[a-8]),b=b.replace("{0}",this.ca.gh[a-8]));this.lh(b)};tj.prototype.parse=function(a,b,c){return wj(this,a,b,c||0,!1)};
function wj(a,b,c,d,e){for(var g=new xj,h=[d],k=-1,l=0,m=0,p=0;p<a.Va.length;p++)if(0<a.Va[p].count)if(0>k&&a.Va[p].gg&&(k=p,l=d,m=0),0<=k){var u=a.Va[p].count;if(p==k&&(u-=m,m++,0==u))return 0;yj(a,b,h,a.Va[p],u,g)||(p=k-1,h[0]=l)}else{if(k=-1,!yj(a,b,h,a.Va[p],0,g))return 0}else{k=-1;if(" "==a.Va[p].text.charAt(0)){if(u=h[0],zj(b,h),h[0]>u)continue}else if(b.indexOf(a.Va[p].text,h[0])==h[0]){h[0]+=a.Va[p].text.length;continue}return 0}a:if(void 0!=g.pq&&void 0!=g.year&&0==g.pq&&0<g.year&&(g.year=
-(g.year-1)),void 0!=g.year&&c.setFullYear(g.year),a=c.getDate(),c.setDate(1),void 0!=g.month&&c.setMonth(g.month),void 0!=g.day?c.setDate(g.day):(b=Ti(c.getFullYear(),c.getMonth()),c.setDate(a>b?b:a)),oa(c.setHours)&&(void 0==g.hours&&(g.hours=c.getHours()),void 0!=g.Bp&&0<g.Bp&&12>g.hours&&(g.hours+=12),c.setHours(g.hours)),oa(c.setMinutes)&&void 0!=g.minutes&&c.setMinutes(g.minutes),oa(c.setSeconds)&&void 0!=g.seconds&&c.setSeconds(g.seconds),oa(c.setMilliseconds)&&void 0!=g.milliseconds&&c.setMilliseconds(g.milliseconds),
e&&(void 0!=g.year&&g.year!=c.getFullYear()||void 0!=g.month&&g.month!=c.getMonth()||void 0!=g.day&&g.day!=c.getDate()||24<=g.hours||60<=g.minutes||60<=g.seconds||1E3<=g.milliseconds))c=!1;else{void 0!=g.Qo&&c.setTime(c.getTime()+6E4*(g.Qo-c.getTimezoneOffset()));g.Ot&&(e=new Date,e.setFullYear(e.getFullYear()-80),c.getTime()<e.getTime()&&c.setFullYear(e.getFullYear()+100));if(void 0!=g.cm)if(void 0==g.day)g=(7+g.cm-c.getDay())%7,3<g&&(g-=7),e=c.getMonth(),c.setDate(c.getDate()+g),c.getMonth()!=e&&
c.setDate(c.getDate()+(0<g?-7:7));else if(g.cm!=c.getDay()){c=!1;break a}c=!0}return c?h[0]-d:0}var uj="GyMdkHmsSEDahKzZvQL";function vj(a){if(0>=a.count)return!1;var b="MydhHmsSDkK".indexOf(a.text.charAt(0));return 0<b||0==b&&3>a.count}function zj(a,b){var c=a.substring(b[0]).match(/^\s+/);c&&(b[0]+=c[0].length)}
function yj(a,b,c,d,e,g){zj(b,c);var h=c[0],k=d.text.charAt(0),l=-1;if(vj(d))if(0<e){if(h+e>b.length)return!1;l=Aj(a,b.substring(0,h+e),c)}else l=Aj(a,b,c);switch(k){case "G":return l=Bj(b,c,a.ca.il),0<=l&&(g.pq=l),!0;case "M":case "L":a:{d=l;if(0>d){d=Bj(b,c,a.ca.ml.concat(a.ca.eg).concat(a.ca.ql).concat(a.ca.tl));if(0>d){g=!1;break a}g.month=d%12}else g.month=d-1;g=!0}return g;case "E":return d=Bj(b,c,a.ca.wl),0>d&&(d=Bj(b,c,a.ca.sl)),0>d?g=!1:(g.cm=d,g=!0),g;case "a":return l=Bj(b,c,a.ca.fl),0<=
l&&(g.Bp=l),!0;case "y":a:{if(0>l){var m=b.charAt(c[0]);if("+"!=m&&"-"!=m){g=!1;break a}c[0]++;l=Aj(a,b,c);if(0>l){g=!1;break a}"-"==m&&(l=-l)}m||2!=c[0]-h||2!=d.count?g.year=l:(a=l,b=(new Date).getFullYear()-80,c=b%100,g.Ot=a==c,a+=100*Math.floor(b/100)+(a<c?100:0),g.year=a);g=!0}return g;case "Q":return 0>l?(d=Bj(b,c,a.ca.pl),0>d&&(d=Bj(b,c,a.ca.rl)),0>d?g=!1:(g.month=3*d,g.day=1,g=!0)):g=!1,g;case "d":return 0<=l&&(g.day=l),!0;case "S":return a=c[0]-h,g.milliseconds=3>a?l*Math.pow(10,3-a):Math.round(l/
Math.pow(10,a-3)),!0;case "h":12==l&&(l=0);case "K":case "H":case "k":return 0<=l&&(g.hours=l),!0;case "m":return 0<=l&&(g.minutes=l),!0;case "s":return 0<=l&&(g.seconds=l),!0;case "z":case "Z":case "v":b.indexOf("GMT",c[0])==c[0]&&(c[0]+=3);a:if(c[0]>=b.length)g.Qo=0,g=!0;else{d=1;switch(b.charAt(c[0])){case "-":d=-1;case "+":c[0]++}h=c[0];l=Aj(a,b,c);if(0>l)g=!1;else{if(c[0]<b.length&&":"==b.charAt(c[0])){m=60*l;c[0]++;l=Aj(a,b,c);if(0>l){g=!1;break a}m+=l}else m=l,m=24>m&&2>=c[0]-h?60*m:m%100+
m/100*60;g.Qo=-(m*d);g=!0}}return g;default:return!1}}function Aj(a,b,c){if(a.ca.yl){for(var d=[],e=c[0];e<b.length;e++){var g=b.charCodeAt(e)-a.ca.yl;d.push(0<=g&&9>=g?String.fromCharCode(g+48):b.charAt(e))}b=d.join("")}else b=b.substring(c[0]);a=b.match(/^\d+/);if(!a)return-1;c[0]+=a[0].length;return parseInt(a[0],10)}
function Bj(a,b,c){var d=0,e=-1;a=a.substring(b[0]).toLowerCase();for(var g=0;g<c.length;g++){var h=c[g].length;h>d&&0==a.indexOf(c[g].toLowerCase())&&(e=g,d=h)}0<=e&&(b[0]+=d);return e}function xj(){};function Cj(a,b){this.Ho=a;this.Ge=b}var Dj=new cj(0,0,1),Ej=new cj(9999,11,31);Cj.prototype.getStartDate=function(){return this.Ho};Cj.prototype.contains=function(a){return a.valueOf()>=this.Ho.valueOf()&&a.valueOf()<=this.Ge.valueOf()};Cj.prototype.iterator=function(){return new Fj(this)};function Gj(a,b){var c=a.clone();c.add(new Wi(Zi,b));return c}function Hj(a,b){var c=a.clone();c.setDate(1);c.add(new Wi(Yi,b));return c}function Ij(a){a=Jj(a);a=Gj(a,-1);return new Cj(a,a.clone())}
function Kj(a){a=Jj(a);return new Cj(a,a.clone())}function Lj(a){a=Jj(a);var b=Gj(a,-1);return new Cj(Gj(a,-7),b)}function Mj(a){a=Jj(a);return new Cj(Hj(a,-1),Gj(Hj(a,0),-1))}function Nj(a){a=Jj(a);var b=(a.getDay()+6)%7;var c=a.Yc;a=Gj(a,-(b>=c?b-c:b+(7-c)));b=Gj(a,6);b=new Cj(a,b);a=Gj(b.getStartDate(),-7);b=Gj(b.Ge,-7);return new Cj(a,b)}function Jj(a){return a?a.clone():new cj}function Fj(a){this.Pn=a.getStartDate().clone();this.Ge=Number(a.Ge.toIsoString())}r(Fj,Od);
Fj.prototype.next=function(){if(Number(this.Pn.toIsoString())>this.Ge)throw Nd;var a=this.Pn.clone();this.Pn.add(new Wi(Zi,1));return a};var Oj={tp:"y",Jt:"y G",Kt:"LLLL y",up:"LLLL y",bt:"d. M.",ct:"dd. MMMM",ft:"d. M.",et:"d. MMMM",ht:"d. M. y",Ht:"EEE d. M.",It:"EEE d. M. y",Us:"d."},Pj={tp:"y",Jt:"y G",Kt:"MMM y",up:"MMMM y",bt:"MMM d",ct:"MMMM dd",ft:"M/d",et:"MMMM d",ht:"MMM d, y",Ht:"EEE, MMM d",It:"EEE, MMM d, y",Us:"d"},Qj=Pj,Qj=Pj;function Rj(a,b){this.Tt=a;this.R=b||jf()}Rj.prototype.J=function(){return this.R};Rj.prototype.W=function(){return this.Tt};Rj.prototype.uk=function(a,b){var c=this.J().createElement("TD");c.colSpan=b?2:3;c.className=this.W()+"-today-cont";Sj(this,c,"Today",this.W()+"-today-btn");a.appendChild(c);c=this.J().createElement("TD");c.colSpan=b?4:3;a.appendChild(c);c=this.J().createElement("TD");c.colSpan=2;c.className=this.W()+"-none-cont";Sj(this,c,"None",this.W()+"-none-btn");a.appendChild(c)};
function Sj(a,b,c,d){var e=[a.W()+"-btn"];d&&e.push(d);d=a.J().createElement("BUTTON");d.className=e.join(" ");d.appendChild(a.J().createTextNode(c));b.appendChild(d)};function Tj(a,b,c,d){kh.call(this,c);this.wc=b||Si;this.ow=this.wc.vl;this.av=new mj("d",this.wc);this.Zu=new mj("dd",this.wc);this.cv=new mj("w",this.wc);this.$u=new mj("M d",this.wc);this.Uh=new mj(Qj.tp||"y",this.wc);this.bv=new mj(Qj.up||"MMMM y",this.wc);this.U=d||new Rj(this.W(),this.J());this.Wa=new cj(a);this.Wa.xf=this.wc.Ji;this.Wa.Yc=this.wc.Ii;this.Oa=this.Wa.clone();this.Oa.setDate(1);this.al="      ".split(" ");this.al[this.wc.xl[0]]=this.W()+"-wkend-start";this.al[this.wc.xl[1]]=this.W()+
"-wkend-end";this.Hg={};this.Af=[]}r(Tj,kh);f=Tj.prototype;f.gs=!0;f.dw=!0;f.Uo=new Cj(Dj,Ej);f.Bu=!0;f.yi=!0;f.ls=!0;f.kh=!0;f.Jk=!0;f.Fo=!1;f.kq=null;f.hv=!1;f.uj=null;f.tj=null;f.sj=null;f.Xt=ih.va();f.W=function(){return"goog-date-picker"};function Uj(a){w(a.wj,a.Jk);w(a.vj,a.kh);w(a.vs,a.Jk||a.kh)}f.Gr=function(){this.Oa.add(new Wi(Yi,-1));Vj(this);Wj(this)};f.jr=function(){this.Oa.add(new Wi(Yi,1));Vj(this);Wj(this)};f.Ov=function(){this.Oa.add(new Wi(Xi,-1));Vj(this);Wj(this)};
f.rv=function(){this.Oa.add(new Wi(Xi,1));Vj(this);Wj(this)};f.Wr=function(){this.setDate(new cj)};f.Vr=function(){this.kh&&this.setDate(null)};f.getDate=function(){return this.Wa&&this.Wa.clone()};f.setDate=function(a){Xj(this,a,!0)};
function Xj(a,b,c){var d=b==a.Wa||b&&a.Wa&&b.getFullYear()==a.Wa.getFullYear()&&b.getMonth()==a.Wa.getMonth(),e=b==a.Wa||d&&b.getDate()==a.Wa.getDate();a.Wa=b&&new cj(b);b&&(a.Oa.set(a.Wa),a.Oa.setFullYear(a.Wa.getFullYear()),a.Oa.setDate(1));Vj(a);c&&a.dispatchEvent(new Yj("select",a,a.Wa));e||a.dispatchEvent(new Yj("change",a,a.Wa));d||Wj(a)}
function Zj(a){if(a.uj){for(var b=a.uj;b.firstChild;)b.removeChild(b.firstChild);var c=a.U,d=a.yi,e=a.wc.dg[0].toLowerCase();if(a.Fo)e=c.J().createElement("TD"),e.colSpan=d?1:2,Sj(c,e,"«",c.W()+"-previousMonth"),b.appendChild(e),e=c.J().createElement("TD"),e.colSpan=d?6:5,e.className=c.W()+"-monthyear",b.appendChild(e),e=c.J().createElement("TD"),Sj(c,e,"»",c.W()+"-nextMonth"),b.appendChild(e);else{d=c.J().createElement("TD");d.colSpan=5;Sj(c,d,"«",c.W()+"-previousMonth");Sj(c,d,"",c.W()+"-month");
Sj(c,d,"»",c.W()+"-nextMonth");var g=c.J().createElement("TD");g.colSpan=3;Sj(c,g,"«",c.W()+"-previousYear");Sj(c,g,"",c.W()+"-year");Sj(c,g,"»",c.W()+"-nextYear");e.indexOf("y")<e.indexOf("m")?(b.appendChild(g),b.appendChild(d)):(b.appendChild(d),b.appendChild(g))}if(a.Fo){ak(a,b,a.W()+"-previousMonth",a.Gr);if(c=of(a.W()+"-previousMonth",b))fi(c,"hidden",!0),c.tabIndex=-1;ak(a,b,a.W()+"-nextMonth",a.jr);if(c=of(a.W()+"-nextMonth",b))fi(c,"hidden",!0),c.tabIndex=-1;a.tj=of(a.W()+"-monthyear",b)}else ak(a,
b,a.W()+"-previousMonth",a.Gr),ak(a,b,a.W()+"-nextMonth",a.jr),ak(a,b,a.W()+"-month",a.cw),ak(a,b,a.W()+"-previousYear",a.Ov),ak(a,b,a.W()+"-nextYear",a.rv),ak(a,b,a.W()+"-year",a.Co),a.sf=of(a.W()+"-month",b),a.tf=jf().Ej(a.W()+"-year",b)}}function ak(a,b,c,d){b=of(c,b);a.Sa().A(b,"click",function(a){a.preventDefault();d.call(this,a)})}
function bk(a){if(a.sj){var b=a.sj;Cf(b);a.U.uk(b,a.yi);ak(a,b,a.W()+"-today-btn",a.Wr);ak(a,b,a.W()+"-none-btn",a.Vr);a.wj=of(a.W()+"-today-btn",b);a.vj=of(a.W()+"-none-btn",b);Uj(a)}}
f.Ga=function(a){Tj.m.Ga.call(this,a);ji(a,this.W());var b=this.R.createElement("TABLE"),c=this.R.createElement("THEAD"),d=this.R.createElement("TBODY"),e=this.R.createElement("TFOOT");ei(d,"grid");d.tabIndex=0;this.Lo=d;this.vs=e;var g=this.R.createElement("TR");g.className=this.W()+"-head";this.uj=g;Zj(this);c.appendChild(g);this.Gc=[];for(var h=0;7>h;h++){g=this.R.createElement("TR");this.Gc[h]=[];for(var k=0;8>k;k++){var l=this.R.createElement(0==k||0==h?"th":"td");0!=k&&0!=h||k==h||(l.className=
0==k?this.W()+"-week":this.W()+"-wday",ei(l,0==k?"rowheader":"columnheader"));g.appendChild(l);this.Gc[h][k]=l}d.appendChild(g)}g=this.R.createElement("TR");g.className=this.W()+"-foot";this.sj=g;bk(this);e.appendChild(g);b.cellSpacing="0";b.cellPadding="0";b.appendChild(c);b.appendChild(d);b.appendChild(e);a.appendChild(b);if(this.o()){if(this.ls)for(b=0;7>b;b++)Kf(this.Gc[0][b+1],this.ow[((b+this.Oa.Yc+7)%7+1)%7]);w(this.Gc[0][0].parentElement||this.Gc[0][0].parentNode,this.ls)}Vj(this);a.tabIndex=
0};f.D=function(){Tj.m.D.call(this);this.Ga(this.o())};f.Ea=function(){Tj.m.Ea.call(this);var a=this.Sa();a.A(this.Lo,"click",this.Lu);a.A(ck(this,this.o()),"key",this.Mu)};f.fb=function(){Tj.m.fb.call(this);this.rg();for(var a in this.Hg)this.Hg[a].eb();this.Hg={}};f.create=Tj.prototype.M;f.V=function(){Tj.m.V.call(this);this.vj=this.wj=this.tf=this.tj=this.sf=this.sj=this.uj=this.vs=this.Lo=this.Gc=null};
f.Lu=function(a){if("TD"==a.target.tagName){var b,c=-2,d=-2;for(b=a.target;b;b=b.previousSibling,c++);for(b=a.target.parentNode;b;b=b.previousSibling,d++);a=this.Af[d][c];this.Uo.contains(a)&&this.setDate(a.clone())}};
f.Mu=function(a){switch(a.keyCode){case 33:a.preventDefault();var b=-1;break;case 34:a.preventDefault();b=1;break;case 37:a.preventDefault();var c=-1;break;case 39:a.preventDefault();c=1;break;case 38:a.preventDefault();c=-7;break;case 40:a.preventDefault();c=7;break;case 36:a.preventDefault(),this.Wr();case 46:a.preventDefault();this.Vr();break;case 13:case 32:a.preventDefault(),Xj(this,this.Wa,!0);default:return}this.Wa?(a=this.Wa.clone(),a.add(new Wi(0,b,c))):(a=this.Oa.clone(),a.setDate(1));this.Uo.contains(a)&&
Xj(this,a,!1)};f.cw=function(a){a.stopPropagation();a=[];for(var b=0;12>b;b++)a.push(this.wc.eg[b]);dk(this,this.sf,a,this.Ru,this.wc.eg[this.Oa.getMonth()])};f.Co=function(a){a.stopPropagation();a=[];for(var b=this.Oa.getFullYear(),c=this.Oa.clone(),d=-5;5>=d;d++)c.setFullYear(b+d),a.push(this.Uh.format(c));dk(this,this.tf,a,this.dn,this.Uh.format(this.Oa))};f.Ru=function(a){a=Number(a.getAttribute("itemIndex"));this.Oa.setMonth(a);Vj(this);this.sf.focus&&this.sf.focus()};
f.dn=function(a){if(3==a.firstChild.nodeType){a=Number(a.getAttribute("itemIndex"));var b=this.Oa.getFullYear();this.Oa.setFullYear(b+a-5);Vj(this)}this.tf.focus()};
function dk(a,b,c,d,e){a.rg();var g=a.R.createElement("DIV");g.className=a.W()+"-menu";a.Ng=null;for(var h=a.R.createElement("UL"),k=0;k<c.length;k++){var l=a.R.D("LI",null,c[k]);l.setAttribute("itemIndex",k);c[k]==e&&(a.Ng=l);h.appendChild(l)}g.appendChild(h);g.style.left=b.offsetLeft+b.parentNode.offsetLeft+"px";g.style.top=b.offsetTop+"px";g.style.width=b.clientWidth+"px";a.sf.parentNode.appendChild(g);a.Rf=g;a.Ng||(a.Ng=h.firstChild);a.Ng.className=a.W()+"-menu-selected";a.Mn=d;b=a.Sa();b.A(a.Rf,
"click",a.Cq);b.A(ck(a,a.Rf),"key",a.Dq);b.A(a.R.P,"click",a.rg);g.tabIndex=0;g.focus()}f.Cq=function(a){a.stopPropagation();this.rg();this.Mn&&this.Mn(a.target)};
f.Dq=function(a){a.stopPropagation();var b=this.Ng;switch(a.keyCode){case 35:a.preventDefault();var c=b.parentNode.lastChild;break;case 36:a.preventDefault();c=b.parentNode.firstChild;break;case 38:a.preventDefault();c=b.previousSibling;break;case 40:a.preventDefault();c=b.nextSibling;break;case 13:case 9:case 0:a.preventDefault(),this.rg(),this.Mn(b)}c&&c!=b&&(b.className="",c.className=this.W()+"-menu-selected",this.Ng=c)};
f.rg=function(){if(this.Rf){var a=this.Sa();a.Aa(this.Rf,"click",this.Cq);a.Aa(ck(this,this.Rf),"key",this.Dq);a.Aa(this.R.P,"click",this.rg);Ff(this.Rf);delete this.Rf}};
function Vj(a){if(a.o()){var b=a.Oa.clone();b.setDate(1);a.tj&&Kf(a.tj,a.bv.format(b));a.sf&&Kf(a.sf,a.wc.eg[b.getMonth()]);a.tf&&Kf(a.tf,a.Uh.format(b));var c=((b.getDay()+6)%7-b.Yc+7)%7;var d=fj(b);b.add(new Wi(Yi,-1));b.setDate(fj(b)-(c-1));a.gs&&!a.Bu&&33>d+c&&b.add(new Wi(Zi,-7));c=new Wi(Zi,1);a.Af=[];for(d=0;6>d;d++){a.Af[d]=[];for(var e=0;7>e;e++){a.Af[d][e]=b.clone();var g=b.getFullYear();b.add(c);0==b.getMonth()&&1==b.getDate()&&g++;b.setFullYear(g)}}if(a.o())for(b=a.Oa.getMonth(),e=new cj,
c=e.getFullYear(),d=e.getMonth(),e=e.getDate(),g=0;6>g;g++){a.yi?(Kf(a.Gc[g+1][0],a.cv.format(a.Af[g][0])),a.Gc[g+1][0].className=a.W()+"-week"):(Kf(a.Gc[g+1][0],""),a.Gc[g+1][0].className="");for(var h=0;7>h;h++){var k=a.Af[g][h],l=a.Gc[g+1][h+1];l.id||(l.id=jh(a.Xt));ei(l,"gridcell");var m=a.$u.format(k);fi(l,"label",m);m=[a.W()+"-date"];a.Uo.contains(k)||m.push(a.W()+"-unavailable-date");if(a.dw||k.getMonth()==b){k.getMonth()!=b&&m.push(a.W()+"-other-month");var p=(h+a.Oa.Yc+7)%7;a.al[p]&&m.push(a.al[p]);
k.getDate()==e&&k.getMonth()==d&&k.getFullYear()==c&&m.push(a.W()+"-today");a.Wa&&k.getDate()==a.Wa.getDate()&&k.getMonth()==a.Wa.getMonth()&&k.getFullYear()==a.Wa.getFullYear()&&(m.push(a.W()+"-selected"),fi(a.Lo,"activedescendant",l.id));a.kq&&(p=a.kq(k))&&m.push(p);k=a.hv?a.Zu.format(k):a.av.format(k);Kf(l,k)}else Kf(l,"");l.className=m.join(" ")}4<=g&&w(a.Gc[g+1][0].parentElement||a.Gc[g+1][0].parentNode,a.Af[g][0].getMonth()==b||a.gs)}}}
function Wj(a){var b=new Yj("changeActiveMonth",a,a.Oa.clone());a.dispatchEvent(b)}function ck(a,b){var c=qa(b);c in a.Hg||(a.Hg[c]=new Yh(b));return a.Hg[c]}function Yj(a,b,c){Cc.call(this,a,b);this.date=c}r(Yj,Cc);function ek(a,b){md.call(this);this.qb=new fh(this);this.uo(a||null);b&&(this.dh=b)}r(ek,md);f=ek.prototype;f.F=null;f.St=!0;f.Fp=null;f.Gp=null;f.Gg=!1;f.aw=!1;f.Dn=-1;f.Xu=!1;f.yu=!0;f.dh="toggle_display";f.o=function(){return this.F};f.uo=function(a){if(this.Gg)throw Error("Can not change this state of the popup while showing.");this.F=a};f.Sa=function(){return this.qb};f.za=function(){return this.Gg};f.da=function(a){this.xi&&this.xi.stop();this.Th&&this.Th.stop();a?this.Do():this.Bg()};
f.uc=ja;
f.Do=function(){if(!this.Gg&&this.Tn()){if(!this.F)throw Error("Caller must call setElement before trying to show the popup");this.uc();var a=lf(this.F);this.Xu&&this.qb.A(a,"keydown",this.uv,!0);if(this.St)if(this.qb.A(a,"mousedown",this.mr,!0),t){try{var b=a.activeElement}catch(d){}for(;b&&"IFRAME"==b.nodeName;){try{var c=Jf(b)}catch(d){break}a=c;b=a.activeElement}this.qb.A(a,"mousedown",this.mr,!0);this.qb.A(a,"deactivate",this.lr)}else this.qb.A(a,"blur",this.lr);"toggle_display"==this.dh?(this.F.style.visibility=
"visible",w(this.F,!0)):"move_offscreen"==this.dh&&this.uc();this.Gg=!0;this.Dn=wa();this.xi?(cd(this.xi,"end",this.Uf,!1,this),this.xi.play()):this.Uf()}};f.Bg=function(a){if(!this.Gg||!this.dispatchEvent({type:"beforehide",target:a}))return!1;this.qb&&this.qb.ld();this.Gg=!1;wa();this.Th?(cd(this.Th,"end",va(this.bq,a),!1,this),this.Th.play()):this.bq(a);return!0};
f.bq=function(a){"toggle_display"==this.dh?this.aw?od(this.Hq,0,this):this.Hq():"move_offscreen"==this.dh&&(this.F.style.top="-10000px");this.Ve(a)};f.Hq=function(){this.F.style.visibility="hidden";w(this.F,!1)};f.Tn=function(){return this.dispatchEvent("beforeshow")};f.Uf=function(){this.dispatchEvent("show")};f.Ve=function(a){this.dispatchEvent({type:"hide",target:a})};f.mr=function(a){a=a.target;If(this.F,a)||fk(this,a)||this.Gp&&!If(this.Gp,a)||150>wa()-this.Dn||this.Bg(a)};
f.uv=function(a){27==a.keyCode&&this.Bg(a.target)&&(a.preventDefault(),a.stopPropagation())};f.lr=function(a){if(this.yu){var b=lf(this.F);if("undefined"!=typeof document.activeElement){if(a=b.activeElement,!a||If(this.F,a)||"BODY"==a.tagName)return}else if(a.target!=b)return;150>wa()-this.Dn||this.Bg()}};function fk(a,b){return jb(a.Fp||[],function(a){return b===a||If(a,b)})}f.V=function(){ek.m.V.call(this);this.qb.eb();ac(this.xi);ac(this.Th);delete this.F;delete this.qb;delete this.Fp};function gk(a,b,c,d,e,g,h,k){var l;if(l=c.offsetParent){var m="HTML"==l.tagName||"BODY"==l.tagName;if(!m||"static"!=fg(l)){var p=og(l);m||(m=(m=vg(l))&&jc?-l.scrollLeft:!m||ic&&uc("8")||"visible"==eg(l,"overflowX")?l.scrollLeft:l.scrollWidth-l.clientWidth-l.scrollLeft,p=ff(p,new df(m,l.scrollTop)))}}l=p||new df;p=og(a);m=tg(a);p=new $f(p.x,p.y,m.width,m.height);(m=ng(a))&&p.Nq(new $f(m.left,m.top,m.right-m.left,m.bottom-m.top));var m=jf(a),u=jf(c);if(m.P!=u.P){var A=m.P.body;var u=ne(u),B=new df(0,
0),E=wf(lf(A));if(ec(E,"parent")){var J=A;do{if(E==u)var L=og(J);else L=lg(J),L=new df(L.left,L.top);B.x+=L.x;B.y+=L.y}while(E&&E!=u&&E!=E.parent&&(J=E.frameElement)&&(E=E.parent))}A=ff(B,og(A));!t||vc(9)||tf(m.P)||(A=ff(A,uf(m.P)));p.left+=A.x;p.top+=A.y}a=hk(a,b);b=p.left;a&4?b+=p.width:a&2&&(b+=p.width/2);b=new df(b,p.top+(a&1?p.height:0));b=ff(b,l);e&&(b.x+=(a&4?-1:1)*e.x,b.y+=(a&1?-1:1)*e.y);var Q;h&&(Q=ng(c))&&(Q.top-=l.y,Q.right-=l.x,Q.bottom-=l.y,Q.left-=l.x);return ik(b,c,d,g,Q,h,k)}
function ik(a,b,c,d,e,g,h){a=a.clone();var k=hk(b,c);c=tg(b);h=h?h.clone():c.clone();a=a.clone();h=h.clone();var l=0;if(d||0!=k)k&4?a.x-=h.width+(d?d.right:0):k&2?a.x-=h.width/2:d&&(a.x+=d.left),k&1?a.y-=h.height+(d?d.bottom:0):d&&(a.y+=d.top);if(g){if(e){d=a;k=h;l=0;65==(g&65)&&(d.x<e.left||d.x>=e.right)&&(g&=-2);132==(g&132)&&(d.y<e.top||d.y>=e.bottom)&&(g&=-5);d.x<e.left&&g&1&&(d.x=e.left,l|=1);if(g&16){var m=d.x;d.x<e.left&&(d.x=e.left,l|=4);d.x+k.width>e.right&&(k.width=Math.min(e.right-d.x,
m+k.width-e.left),k.width=Math.max(k.width,0),l|=4)}d.x+k.width>e.right&&g&1&&(d.x=Math.max(e.right-k.width,e.left),l|=1);g&2&&(l|=(d.x<e.left?16:0)|(d.x+k.width>e.right?32:0));d.y<e.top&&g&4&&(d.y=e.top,l|=2);g&32&&(m=d.y,d.y<e.top&&(d.y=e.top,l|=8),d.y+k.height>e.bottom&&(k.height=Math.min(e.bottom-d.y,m+k.height-e.top),k.height=Math.max(k.height,0),l|=8));d.y+k.height>e.bottom&&g&4&&(d.y=Math.max(e.bottom-k.height,e.top),l|=2);g&8&&(l|=(d.y<e.top?64:0)|(d.y+k.height>e.bottom?128:0));e=l}else e=
256;l=e}g=new $f(0,0,0,0);g.left=a.x;g.top=a.y;g.width=h.width;g.height=h.height;e=l;if(e&496)return e;ig(b,new df(g.left,g.top));h=new gf(g.width,g.height);c==h||c&&h&&c.width==h.width&&c.height==h.height||(c=h,a=tf(jf(lf(b)).P),!t||uc("10")||a&&uc("8")?(b=b.style,jc?b.MozBoxSizing="border-box":kc?b.WebkitBoxSizing="border-box":b.boxSizing="border-box",b.width=Math.max(c.width,0)+"px",b.height=Math.max(c.height,0)+"px"):(h=b.style,a?(t?(k=zg(b,"paddingLeft"),d=zg(b,"paddingRight"),g=zg(b,"paddingTop"),
a=zg(b,"paddingBottom"),a=new Zf(g,d,a,k)):(k=dg(b,"paddingLeft"),d=dg(b,"paddingRight"),g=dg(b,"paddingTop"),a=dg(b,"paddingBottom"),a=new Zf(parseFloat(g),parseFloat(d),parseFloat(a),parseFloat(k))),b=qg(b),h.pixelWidth=c.width-b.left-a.left-a.right-b.right,h.pixelHeight=c.height-b.top-a.top-a.bottom-b.bottom):(h.pixelWidth=c.width,h.pixelHeight=c.height)));return e}function hk(a,b){return(b&8&&vg(a)?b^4:b)&-9};function jk(){}jk.prototype.uc=function(){};function kk(a,b,c){this.element=a;this.fq=b;this.Gv=c}r(kk,jk);kk.prototype.uc=function(a,b,c){gk(this.element,this.fq,a,b,void 0,c,this.Gv)};function lk(a,b){this.Jv=8;this.Zn=b||void 0;ek.call(this,a)}r(lk,ek);lk.prototype.setPosition=function(a){this.Zn=a||void 0;this.za()&&this.uc()};lk.prototype.uc=function(){if(this.Zn){var a=!this.za()&&"move_offscreen"!=this.dh,b=this.o();a&&(b.style.visibility="hidden",w(b,!0));this.Zn.uc(b,this.Jv,this.Lz);a&&w(b,!1)}};function mk(a,b){kh.call(this,b);this.pc=a||new Tj}r(mk,kh);f=mk.prototype;f.pc=null;f.Bd=null;f.ek=null;f.hf=!0;f.D=function(){mk.m.D.call(this);this.o().className="goog-popupdatepicker";this.Bd=new lk(this.o());this.Bd.Gk(this)};f.za=function(){return this.Bd?this.Bd.za():!1};f.Ea=function(){mk.m.Ea.call(this);if(!this.pc.la){var a=this.o();a.style.visibility="hidden";w(a,!1);this.pc.M(a)}this.Sa().A(this.pc,"change",this.ci).A(this.pc,"select",this.Un)};
f.V=function(){mk.m.V.call(this);this.Bd&&(this.Bd.eb(),this.Bd=null);this.pc.eb();this.ek=this.pc=null};f.wb=function(){return!1};f.vm=function(){return this.pc};f.getDate=function(){return this.pc.getDate()};f.setDate=function(a){this.pc.setDate(a)};f.jf=function(a){this.Sa().A(a,"mousedown",this.hs)};f.detach=function(a){this.Sa().Aa(a,"mousedown",this.hs)};f.Xr=function(a){this.hf=a};
f.hs=function(a){this.ek=a=a.currentTarget;this.Bd.setPosition(new kk(a,9,197));this.Sa().Aa(this.pc,"change",this.ci).Aa(this.pc,"select",this.Un);this.pc.setDate(null);this.dispatchEvent("show");this.Bd.da(!0);this.hf&&this.o().focus();this.Sa().A(this.pc,"change",this.ci).A(this.pc,"select",this.Un)};f.gn=function(){this.Bd.da(!1);this.hf&&this.ek&&this.ek.focus()};f.Un=function(a){this.gn();this.dispatchEvent(a)};f.ci=function(a){this.dispatchEvent(a)};function nk(a,b,c,d){kh.call(this,d);this.bm=a;this.jq=b;this.lb=new mk(c,d);this.$a(this.lb);this.lb.Xr(!1)}r(nk,kh);f=nk.prototype;f.bm=null;f.jq=null;f.lb=null;f.Dr=null;f.vm=function(){return this.lb.vm()};f.getDate=function(){var a=ok(this),b=this.lb.getDate();a&&b?a.pb(b)||this.lb.setDate(a):this.lb.setDate(null);return a};f.setDate=function(a){this.lb.setDate(a)};f.Fk=function(a){var b=this.o();b.Bn?b.Bn.ra(a):b.value=a};
function ok(a){var b=a.o();b=b.Bn?b.Bn.aa():b.value;if(b=Ea(b)){var c=new hj;if(0<wj(a.jq,b,c,0,!0))return c}return null}f.D=function(){this.F=this.J().D("INPUT",{type:"text"});this.lb.D()};f.Ea=function(){(this.Dr||this.J().P.body).appendChild(this.lb.o());nk.m.Ea.call(this);var a=this.o();this.lb.jf(a);this.lb.setDate(ok(this));a=this.Sa();a.A(this.lb,"change",this.ci);a.A(this.lb,"show",this.zv)};f.fb=function(){nk.m.fb.call(this);var a=this.o();this.lb.detach(a);this.lb.fb();Ff(this.lb.o())};
f.Ga=function(a){nk.m.Ga.call(this,a);this.lb.D()};f.V=function(){nk.m.V.call(this);this.lb.eb();this.Dr=this.lb=null};f.gn=function(){this.lb.gn()};f.zv=function(){var a=ok(this);this.setDate(a);a&&(a=this.vm().getDate(),this.Fk(a?this.bm.format(a):""))};f.ci=function(a){a=a.date;this.Fk(a?this.bm.format(a):"")};function pk(a){if(oa(document.createEvent)){var b=document.createEvent("HTMLEvents");b.initEvent("change",!1,!0);a.dispatchEvent(b)}else a.fireEvent("onchange")};function qk(a,b,c,d){qk.m.constructor.call(this,a,b,c,d);this.eh=null}r(qk,Tj);qk.prototype.Co=function(a){var b,c=this;if(null==this.eh)qk.m.Co.call(this,a);else{a.stopPropagation();var d=this.Oa.clone();var e=function(a){d.setFullYear(a);return c.Uh.format(d)};return dk(this,this.tf,function(){var a;var c=yb(this.eh[0],this.eh[1]);var d=[];var l=0;for(a=c.length;l<a;l++)b=c[l],d.push(e(b));return d}.call(this),this.dn,this.Uh.format(this.Oa))}};
function rk(a,b,c){return null!=a.eh?a.eh[0]+c:b+c-5}qk.prototype.dn=function(a){3===a.firstChild.nodeType&&(this.Oa.setFullYear(rk(this,this.Oa.getFullYear(),Number(a.getAttribute("itemIndex")))),Vj(this));return this.tf.focus()};function sk(a,b,c,d){sk.m.constructor.call(this,a,b,c,d)}r(sk,nk);var tk={5:/[0-9][0-9]:[0-9][0-9]/,8:/[0-9][0-9]:[0-9][0-9]:[0-9][0-9]/};sk.prototype.Fk=function(a){return sk.m.Fk.call(this,uk(this,a))};function uk(a,b){var c;if(c=a.o().value){var d=c.split(" ").pop();c=tk[d.length];if(null!=c&&c.test(d))return[b,d].join(" ")}return b};function vk(a,b,c){this.l=a;this.pattern=null!=b?b:"yyyy'-'MM'-'dd";this.Ad=c;this.o=q(this.o,this);this.Gm=q(this.Gm,this);vk.m.constructor.call(this,this.l);null==this.Ad&&(this.Ad={});null==(a=this.Ad).js&&(a.js=!1);null==(a=this.Ad).zp&&(a.zp=!1);null==(a=this.Ad).ks&&(a.ks=!1);null==(a=this.Ad).Ds&&(a.Ds=!1)}r(vk,Oi);
vk.prototype.M=function(a){this.H=a;this.H.type="text";var b=this.pattern;var c=Sh(this.H,"pattern");null!=c&&(b=String(c));c=b;b=new mj(c);var d=new tj(c);c=new qk;if(a=Sh(a,"yearRange")){if(a=qd(a),!la(a))throw Error("data-year-range must be an Array");}else a=null;if(null!=a&&2>a.length)throw Error("Given unexpected range values, expected two given: "+a.length);c.eh=a;c.Jk=this.Ad.js;c.wj&&Uj(c);c.kh=this.Ad.zp;c.vj&&Uj(c);c.yi=this.Ad.ks;Zj(c);bk(c);Vj(c);c.Fo=this.Ad.Ds;Zj(c);Vj(c);b=new sk(b,
d,c);b.A("change",this.Gm);this.Yd=new Ah({l:this.l});this.Yd.A("action",this.Me);this.Yd.render(Yf(this.H));b.M(this.H);return b};vk.prototype.Gm=function(a){pk(a.currentTarget.o());return this.Cf(a)};vk.prototype.o=function(){return this.H};function wk(a,b){return v(a,"click",function(){eh(a,"collapsed");var c=Sh(a,"target");c=Eg(b,String(c));return eh(c,"in")})};function xk(a,b){this.ee=this.ag=this.Ye="";this.Sg=null;this.Jh=this.hd="";this.Qc=this.vn=!1;if(a instanceof xk){this.Qc=ca(b)?b:a.Qc;yk(this,a.Ye);var c=a.ag;zk(this);this.ag=c;c=a.ee;zk(this);this.ee=c;Ak(this,a.Sg);Bk(this,a.hd);Ck(this,a.mb.clone());this.Xg(a.Jh)}else if(a&&(c=String(a).match(Td))){this.Qc=!!b;yk(this,c[1]||"",!0);var d=c[2]||"";zk(this);this.ag=Dk(d);d=c[3]||"";zk(this);this.ee=Dk(d,!0);Ak(this,c[4]);Bk(this,c[5]||"",!0);Ck(this,c[6]||"",!0);this.Xg(c[7]||"",!0)}else this.Qc=
!!b,this.mb=new Ek(null,0,this.Qc)}f=xk.prototype;f.toString=function(){var a=[],b=this.Ye;b&&a.push(Fk(b,Gk,!0),":");var c=this.ee;if(c||"file"==b)a.push("//"),(b=this.ag)&&a.push(Fk(b,Gk,!0),"@"),a.push(encodeURIComponent(String(c)).replace(/%25([0-9a-fA-F]{2})/g,"%$1")),c=this.Sg,null!=c&&a.push(":",String(c));if(c=this.hd)this.ee&&"/"!=c.charAt(0)&&a.push("/"),a.push(Fk(c,"/"==c.charAt(0)?Hk:Ik,!0));(c=this.mb.toString())&&a.push("?",c);(c=this.Jh)&&a.push("#",Fk(c,Jk));return a.join("")};
f.resolve=function(a){var b=this.clone(),c=!!a.Ye;c?yk(b,a.Ye):c=!!a.ag;if(c){var d=a.ag;zk(b);b.ag=d}else c=!!a.ee;c?(d=a.ee,zk(b),b.ee=d):c=null!=a.Sg;d=a.hd;if(c)Ak(b,a.Sg);else if(c=!!a.hd){if("/"!=d.charAt(0))if(this.ee&&!this.hd)d="/"+d;else{var e=b.hd.lastIndexOf("/");-1!=e&&(d=b.hd.substr(0,e+1)+d)}e=d;if(".."==e||"."==e)d="";else if(Qa(e,"./")||Qa(e,"/.")){for(var d=za(e,"/"),e=e.split("/"),g=[],h=0;h<e.length;){var k=e[h++];"."==k?d&&h==e.length&&g.push(""):".."==k?((1<g.length||1==g.length&&
""!=g[0])&&g.pop(),d&&h==e.length&&g.push("")):(g.push(k),d=!0)}d=g.join("/")}else d=e}c?Bk(b,d):c=""!==a.mb.toString();c?Ck(b,a.mb.clone()):c=!!a.Jh;c&&b.Xg(a.Jh);return b};f.clone=function(){return new xk(this)};function yk(a,b,c){zk(a);a.Ye=c?Dk(b,!0):b;a.Ye&&(a.Ye=a.Ye.replace(/:$/,""))}function Ak(a,b){zk(a);if(b){b=Number(b);if(isNaN(b)||0>b)throw Error("Bad port number "+b);a.Sg=b}else a.Sg=null}function Bk(a,b,c){zk(a);a.hd=c?Dk(b,!0):b}
function Ck(a,b,c){zk(a);b instanceof Ek?(a.mb=b,a.mb.wo(a.Qc)):(c||(b=Fk(b,Kk)),a.mb=new Ek(b,0,a.Qc));return a}f.bs=function(a,b){return Ck(this,a,b)};f.Oc=function(){return this.mb.toString()};function Lk(a,b,c){zk(a);a.mb.set(b,c)}f.Xg=function(a,b){zk(this);this.Jh=b?Dk(a):a;return this};f.removeParameter=function(a){zk(this);this.mb.remove(a);return this};f.cs=function(a){this.vn=a;return this};f.Pq=function(){return this.vn};
function zk(a){if(a.vn)throw Error("Tried to modify a read-only Uri");}f.wo=function(a){this.Qc=a;this.mb&&this.mb.wo(a);return this};function Dk(a,b){return a?b?decodeURI(a.replace(/%25/g,"%2525")):decodeURIComponent(a):""}function Fk(a,b,c){return da(a)?(a=encodeURI(a).replace(b,Mk),c&&(a=a.replace(/%25([0-9a-fA-F]{2})/g,"%$1")),a):null}function Mk(a){a=a.charCodeAt(0);return"%"+(a>>4&15).toString(16)+(a&15).toString(16)}var Gk=/[#\/\?@]/g,Ik=/[\#\?:]/g,Hk=/[\#\?]/g,Kk=/[\#\?@]/g,Jk=/#/g;
function Ek(a,b,c){this.Ka=this.Ya=null;this.Hc=a||null;this.Qc=!!c}function Nk(a){a.Ya||(a.Ya=new Pd,a.Ka=0,a.Hc&&Ud(a.Hc,function(b,c){a.add(Fa(b),c)}))}f=Ek.prototype;f.xd=function(){Nk(this);return this.Ka};f.add=function(a,b){Nk(this);this.Hc=null;a=Ok(this,a);var c=this.Ya.get(a);c||this.Ya.set(a,c=[]);c.push(b);this.Ka+=1;return this};f.remove=function(a){Nk(this);a=Ok(this,a);return this.Ya.ae(a)?(this.Hc=null,this.Ka-=this.Ya.get(a).length,this.Ya.remove(a)):!1};
f.clear=function(){this.Ya=this.Hc=null;this.Ka=0};f.isEmpty=function(){Nk(this);return 0==this.Ka};f.ae=function(a){Nk(this);a=Ok(this,a);return this.Ya.ae(a)};f.Ch=function(a){var b=this.Gb();return nb(b,a)};f.forEach=function(a,b){Nk(this);this.Ya.forEach(function(c,d){eb(c,function(c){a.call(b,c,d,this)},this)},this)};f.rc=function(){Nk(this);for(var a=this.Ya.Gb(),b=this.Ya.rc(),c=[],d=0;d<b.length;d++)for(var e=a[d],g=0;g<e.length;g++)c.push(b[d]);return c};
f.Gb=function(a){Nk(this);var b=[];if(da(a))this.ae(a)&&(b=rb(b,this.Ya.get(Ok(this,a))));else{a=this.Ya.Gb();for(var c=0;c<a.length;c++)b=rb(b,a[c])}return b};f.set=function(a,b){Nk(this);this.Hc=null;a=Ok(this,a);this.ae(a)&&(this.Ka-=this.Ya.get(a).length);this.Ya.set(a,[b]);this.Ka+=1;return this};f.get=function(a,b){var c=a?this.Gb(a):[];return 0<c.length?String(c[0]):b};
f.toString=function(){if(this.Hc)return this.Hc;if(!this.Ya)return"";for(var a=[],b=this.Ya.rc(),c=0;c<b.length;c++)for(var d=b[c],e=encodeURIComponent(String(d)),d=this.Gb(d),g=0;g<d.length;g++){var h=e;""!==d[g]&&(h+="\x3d"+encodeURIComponent(String(d[g])));a.push(h)}return this.Hc=a.join("\x26")};f.clone=function(){var a=new Ek;a.Hc=this.Hc;this.Ya&&(a.Ya=this.Ya.clone(),a.Ka=this.Ka);return a};function Ok(a,b){var c=String(b);a.Qc&&(c=c.toLowerCase());return c}
f.wo=function(a){a&&!this.Qc&&(Nk(this),this.Hc=null,this.Ya.forEach(function(a,c){var b=c.toLowerCase();c!=b&&(this.remove(c),this.remove(b),0<a.length&&(this.Hc=null,this.Ya.set(Ok(this,b),sb(a)),this.Ka+=a.length))},this));this.Qc=a};f.extend=function(a){for(var b=0;b<arguments.length;b++)Ld(arguments[b],function(a,b){this.add(b,a)},this)};function Pk(){this.Lr=wa()}new Pk;Pk.prototype.set=function(a){this.Lr=a};Pk.prototype.reset=function(){this.set(wa())};Pk.prototype.get=function(){return this.Lr};function Qk(){md.call(this);this.On="closure_frame"+Rk++;this.kn=[];Sk[this.On]=this}var Tk;r(Qk,md);var Sk={},Rk=0;function Uk(a,b){var c=jf(a);Ld(b,function(b,e){la(b)||(b=[b]);eb(b,function(b){b=c.D("INPUT",{type:"hidden",name:e,value:b});a.appendChild(b)})})}f=Qk.prototype;f.Ra=null;f.Ta=null;f.If=null;f.qv=0;f.tb=!1;f.Bh=!1;f.Rk=!1;f.Nf=null;f.ck=null;f.Hf=null;f.Wh=!1;
f.send=function(a,b,c,d){if(this.tb)throw Error("[goog.net.IframeIo] Unable to send, already active.");this.Nf=a=new xk(a);b=b?b.toUpperCase():"GET";c&&(zk(a),Lk(a,"zx",Math.floor(2147483648*Math.random()).toString(36)+Math.abs(Math.floor(2147483648*Math.random())^wa()).toString(36)));Tk||(Tk=xf("FORM"),Tk.acceptCharset="utf-8",c=Tk.style,c.position="absolute",c.visibility="hidden",c.top=c.left="-10px",c.width=c.height="10px",c.overflow="hidden",document.body.appendChild(Tk));this.Ra=Tk;"GET"==b&&
Uk(this.Ra,a.mb);d&&Uk(this.Ra,d);this.Ra.action=a.toString();this.Ra.method=b;Vk(this);Wk(this)};function Xk(a,b){if(a.tb)throw Error("[goog.net.IframeIo] Unable to send, already active.");var c=new xk(b.action);a.Nf=c;a.Ra=b;a.Ra.action=c.toString();Vk(a)}f.abort=function(){if(this.tb){var a=Yk(this);fd(a);this.Rk=this.tb=this.Bh=!1;this.dispatchEvent("abort");Zk(this)}};
f.V=function(){this.tb&&this.abort();Qk.m.V.call(this);this.Ta&&$k(this);Wk(this);delete this.qq;this.Nf=this.ck=this.Ra=null;delete Sk[this.On]};f.Xj=function(){return this.Bh};f.Qq=function(){return this.Rk};f.Eg=function(){return this.tb};f.Ij=function(){return this.ck};f.Pc=function(){return qd(this.ck)};
function Vk(a){a.tb=!0;a.Bh=!1;a.If=a.On+"_"+(a.qv++).toString(36);var b={name:a.If,id:a.If};t&&7>Number(pc)&&(b.src='javascript:""');a.Ta=jf(a.Ra).D("IFRAME",b);b=a.Ta.style;b.visibility="hidden";b.width=b.height="10px";b.display="none";kc?b.marginTop=b.marginLeft="-10px":(b.position="absolute",b.top=b.left="-10px");if(t&&!uc("11")){a.Ra.target=a.If||"";jf(a.Ra).P.body.appendChild(a.Ta);a.Wh||v(a.Ta,"readystatechange",a.Vn,!1,a);try{a.km=!1,a.Ra.submit()}catch(W){a.Wh||dd(a.Ta,"readystatechange",
a.Vn,!1,a),al(a)}}else{jf(a.Ra).P.body.appendChild(a.Ta);var b=a.If+"_inner",c=Jf(a.Ta);if(document.baseURI){var d=Ha(b);Be("Short HTML snippet, input escaped, safe URL, for performance");d='\x3chead\x3e\x3cbase href\x3d"'+Ha(document.baseURI)+'"\x3e\x3c/head\x3e\x3cbody\x3e\x3ciframe id\x3d"'+d+'" name\x3d"'+d+'"\x3e\x3c/iframe\x3e';d=Ve(d,null)}else d=Ha(b),Be("Short HTML snippet, input escaped, for performance"),d=Ve('\x3cbody\x3e\x3ciframe id\x3d"'+d+'" name\x3d"'+d+'"\x3e\x3c/iframe\x3e',null);
gc&&!kc?c.documentElement.innerHTML=Te(d):c.write(Te(d));a.Wh||v(c.getElementById(b),"load",a.mk,!1,a);var e=nf("TEXTAREA",a.Ra);d=0;for(var g=e.length;d<g;d++){var h=e[d].value;Tf(e[d])!=h&&(Kf(e[d],h),e[d].value=h)}e=c.importNode(a.Ra,!0);e.target=b;e.action=a.Ra.action;c.body.appendChild(e);var h=nf("SELECT",a.Ra),k=nf("SELECT",e);d=0;for(g=h.length;d<g;d++)for(var l=nf("OPTION",h[d]),m=nf("OPTION",k[d]),p=0,u=l.length;p<u;p++)m[p].selected=l[p].selected;h=nf("INPUT",a.Ra);k=nf("INPUT",e);d=0;
for(g=h.length;d<g;d++)if("file"==h[d].type&&h[d].value!=k[d].value){a.Ra.target=b;e=a.Ra;break}try{a.km=!1,e.submit(),c.close(),jc&&od(a.ys,250,a)}catch(W){d=W;try{var A=ha("window.location.href");if(da(d))var B={message:d,name:"Unknown error",lineNumber:"Not available",fileName:A,stack:"Not available"};else{var g=!1;try{var E=d.lineNumber||d.line||"Not available"}catch(Vi){E="Not available",g=!0}try{var J=d.fileName||d.filename||d.sourceURL||ba.$googDebugFname||A}catch(Vi){J="Not available",g=!0}B=
!g&&d.lineNumber&&d.fileName&&d.stack&&d.message&&d.name?d:{message:d.message||"Not available",name:d.name||"UnknownError",lineNumber:E,fileName:J,stack:d.stack||"Not available"}}var L=B.fileName;null!=L||(L="");if(/^https?:\/\//i.test(L)){var Q=Pe(L);Be("view-source scheme plus HTTP/HTTPS URL");var ia="view-source:"+Ne(Q);var na=Qe(ia)}else{var Ya=Be("sanitizedviewsrc");na=Qe(Ae(Ya))}var Oc=bf(We("Message: "+B.message+"\nUrl: "),$e("a",{href:na,target:"_new"},B.fileName),We("\nLine: "+B.lineNumber+
"\n\nBrowser stack:\n"+B.stack+"-\x3e [end]\n\nJS stack traversal:\n"+xd(void 0)+"-\x3e "))}catch(Vi){Oc=We("Exception trying to expose exception! You win, we lose. "+Vi)}Te(Oc);a.Wh||dd(c.getElementById(b),"load",a.mk,!1,a);c.close();al(a)}}}f.Vn=function(){if("complete"==this.Ta.readyState){dd(this.Ta,"readystatechange",this.Vn,!1,this);try{var a=Jf(this.Ta);if(t&&"about:blank"==a.location&&!navigator.onLine){al(this);return}}catch(b){al(this);return}bl(this,a)}};
f.mk=function(){if(!gc||kc||"about:blank"!=(this.Ta?Jf(Yk(this)):null).location){dd(Yk(this),"load",this.mk,!1,this);try{bl(this,this.Ta?Jf(Yk(this)):null)}catch(a){al(this)}}};function bl(a,b){a.Bh=!0;a.tb=!1;try{var c=b.body;a.ck=c.textContent||c.innerText}catch(e){var d=1}d||"function"!=typeof a.qq||(c=a.qq(b))&&(d=4);d?al(a):(a.Rk=!0,a.dispatchEvent("complete"),a.dispatchEvent("success"),Zk(a))}
function al(a){a.km||(a.Rk=!1,a.tb=!1,a.Bh=!0,a.dispatchEvent("complete"),a.dispatchEvent("error"),Zk(a),a.km=!0)}function Zk(a){$k(a);Wk(a);a.Ra=null;a.dispatchEvent("ready")}function $k(a){var b=a.Ta;b&&(b.onreadystatechange=null,b.onload=null,b.onerror=null,a.kn.push(b));a.Hf&&(ba.clearTimeout(a.Hf),a.Hf=null);jc||gc&&!kc?a.Hf=od(a.nq,2E3,a):a.nq();a.Ta=null;a.If=null}f.nq=function(){this.Hf&&(ba.clearTimeout(this.Hf),this.Hf=null);for(;0!=this.kn.length;){var a=this.kn.pop();Ff(a)}};
function Wk(a){a.Ra&&a.Ra==Tk&&Cf(a.Ra)}function Yk(a){return a.Ta?t&&!uc("11")?a.Ta:Jf(a.Ta).getElementById(a.If+"_inner"):null}f.ys=function(){if(this.tb){var a=this.Ta?Jf(Yk(this)):null;a&&!ec(a,"documentUri")?(this.Wh||dd(Yk(this),"load",this.mk,!1,this),al(this)):od(this.ys,250,this)}};function cl(){}function dl(a,b,c){var d,e;var g=null!=(d=a._rest_links)?null!=(e=d[b])?e.url:void 0:void 0;if(!g)throw Error("Given object is not a model");var h;if(h=!c,0<=db(a._rest_links[b].methods,h))throw Error("Given model does not support "+c+" method");return g};function el(a){var b;var c=this.vp=a;for(b in c)a=c[b],this[b]=a}el.prototype.toString=function(){if(null!=this._autocomplete_value)var a=this._autocomplete_value;else if(this._obj_name)a=this._obj_name;else a:{a=this.vp;for(var b in a){a=a[b];break a}a=void 0}return a};function fl(){}fl.prototype.fc=function(a){if(la(a)){var b;var c=[];var d=0;for(b=a.length;d<b;d++){var e=a[d];c.push(this.qh(e))}a=c}else a=this.qh(a);return a};fl.prototype.qh=function(a){return new el(a)};function gl(a,b){this.name=a;this.descending=null!=b?b:!1}gl.prototype.getName=function(){return this.name};gl.prototype.Ri=function(){return this.descending=!1};gl.prototype.fm=function(){return this.descending=!0};gl.prototype.toString=function(){return(this.descending?"-":"")+this.name};function hl(){this.gb=[]}f=hl.prototype;f.add=function(a){return this.gb.push(a)};f.remove=function(a){return qb(this.gb,function(b){return b.getName()===a})};f.Ri=function(a){var b;(b=il(this,a))&&b.Ri();if(b)return b.Ri();b=new gl(a,!1);return this.add(b)};f.fm=function(a){var b;if(b=il(this,a))return b.fm();b=new gl(a,!0);return this.add(b)};function il(a,b){return lb(a.gb,function(a){return a.getName()===b})}f.toString=function(){return this.gb.join(",")};function jl(a,b,c,d){this.name=a;this.qe=b;this.value=c;this.tn=d;null==this.tn&&(this.tn=!1)}f=jl.prototype;f.getName=function(){return this.name};f.Je=function(){return this.qe};f.aa=function(){return this.value};f.Gj=function(){return Aa(this.name,this.qe)?this.name:[this.name,this.qe].join("__")};f.ra=function(a){this.value=a};function kl(a){return null==a||0===a.length};function ll(a){null==a&&(a="");this.Lk=new hl;null==this.base&&(this.base=10);null==this.offset&&(this.offset=0);this.uri=new xk(a);this.wf=["id"];this.qo=ml;this.accept="application/json";this.filters=new Ek;var b;a={};var c=this.uri.mb.rc();var d=0;for(b=c.length;d<b;d++){var e=c[d];a[e]=this.uri.mb.get(e);this.uri.removeParameter(e)}this.Dh=a}var ml="RAW";f=ll.prototype;f.nw=function(){return this.qo="VERBOSE"};f.raw=function(){return this.qo=ml};f.Yl=function(){return this.uri.toString()};
function nl(a,b){var c;a.filters.remove(b.getName());var d=a.uri.mb.rc();var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];za(h,b.getName())?e.push(a.uri.removeParameter(h)):e.push(void 0)}}f.filter=function(a){a.aa()?(nl(this,a),this.filters.add(a.getName(),a),Lk(this.uri,a.Gj(),a.aa())):nl(this,a);return this};f.Ie=function(a){return this.filters.Gb(a)};
function ol(a,b){var c=a.Ie(b.getName());c=b.tn?lb(c,function(a){return a.Je()===b.Je()}):0==c.length?null:c[0];if(!c&&(null==b.aa()||""===b.aa()))return!1;if(!c)return!0;var d=b.aa();return c.aa()===d&&c.Je()===b.Je()||void 0===c.aa()&&""===d?!1:!0}f.each=function(a){var b;var c=this.uri.mb;var d=c.rc();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(a(h,c.Gb(h)))}return e};
f.sort=function(a,b){switch(b){case 1:return this.Lk.Ri(a);case 2:return this.Lk.fm(a);case 3:return this.Lk.remove(a);default:throw Error("Given invalid sort direction "+b);}};function pl(a,b,c){this.Yb=a;this.context=null!=b?b:"";this.nc=null!=c?c:new je;this.headers={};this.headers["Content-Type"]=ql;this.headers.Accept=ql;this.Gh=new cl;this.qd=new fl}var rl={Ft:"X-Total",Hi:"X-Base",kt:"X-Offset",mt:"X-Order",tt:"X-Referer",px:"X-Extra-Fields",Xs:"X-Fields",Bt:"X-Serialization-Format",Jy:"X-Requested-With"},ql="application/json";function sl(a,b,c){null==c&&(c=null);null!=c&&cd(b,"error",function(){return a.sn(b)?c(b.Pc()):c(void 0)})}
pl.prototype.find=function(a,b,c,d){var e=this;null==c&&(c=null);null==d&&(d=null);var g=this.Yb.fc(this.nc);cd(g,"success",function(){if(null!=b){var a={total:parseInt(g.getResponseHeader(rl.Ft),10)};return b(e.qd.fc(g.Pc()),a)}});sl(this,g,c);c=null;null!=d&&(c={},Qb(c,this.headers),null!=d.base&&(c[rl.Hi]=d.base),null!=d.offset&&(c[rl.kt]=d.offset),c[rl.mt]=d.Lk.toString(),null!=d.Rv&&(c[rl.tt]=d.Rv),0<d.wf.length&&(c[rl.Xs]=d.wf.join(",")),c[rl.Bt]=d.qo,c.Accept=d.accept);return this.send(tl(this,
a,"api-"),"GET",g,null,c)};function tl(a,b,c){pa(b)?(a=[dl(b,c+a.context,"GET")],a.push(b.id),b=a.join("/")):b=String(b);return b}pl.prototype.get=function(a,b,c){var d=this;null==c&&(c=null);var e=this.Yb.fc(this.nc);cd(e,"success",function(){return b(d.qd.fc(e.Pc()))});sl(this,e,c);return this.send(tl(this,a,"api-resource-"),"GET",e)};
pl.prototype["delete"]=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Yb.fc(this.nc);cd(d,"success",function(){if(null!=b)return b()});sl(this,d,c);a=dl(a,"api-resource-"+this.context,"DELETE");return this.send(a,"DELETE",d)};f=pl.prototype;f.create=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Yb.fc(this.nc);cd(d,"success",function(){if(null!=b)return b(d.Pc())});sl(this,d,c);c=dl(a,"api-"+this.context,"POST");return this.send(c,"POST",d,rd(a))};
f.update=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Yb.fc(this.nc);cd(d,"success",function(){if(null!=b)return b(d.Pc())});sl(this,d,c);c=dl(a,"api-resource-"+this.context,"PUT");return this.send(""+c+"/"+a.id,"PUT",d,rd(a))};
f.request=function(a,b,c,d,e,g,h){var k=this;null==c&&(c={});null==d&&(d=null);null==e&&(e=null);null==g&&(g=!1);null==h&&(h={});var l=this.Yb.fc(this.nc);cd(l,"success",function(){if(d)return d(g?k.qd.fc(l.Pc()):ul(l))});sl(this,l,e);Qb(h,this.headers);return this.send(a,b,l,c,h)};function ul(a){var b=null;try{b=a.Pc()}catch(c){b=a.Ij()}return b}
function vl(a,b,c){var d;null==d&&(d=null);var e=a.Yb.fc(a.nc);cd(e,"success",function(){var a=ul(e);return c(a)});sl(a,e,d);return a.send(b,"GET",e,null,{Accept:"text/html,application/json"})}f.send=function(a,b,c,d,e){null==c&&(c=null);null==d&&(d=null);null==e&&(e=null);pa(d)&&(d=rd(d),lb(Ib(e),this.su)||(e["Content-Type"]=ql));null==c&&(c=this.Yb.fc(this.nc));return c.send(a,b,d,null!=e?e:this.headers)};f.su=function(a){return"content-type"==a.toLowerCase()};
function wl(a,b,c,d,e){var g=a.Yb.fc(a.nc);g.Hn("success",function(){var b;if(null!=g.getResponseHeader("Location"))return a.reload(g.getResponseHeader("Location"));if(xl(g,"text/html"))return e(g.Ij());try{return d(null!=(b=g.Pc())?b:{})}catch(k){return d({})}});sl(a,g,e);return a.send(b,"POST",g,c,{Accept:"text/html,application/json","Content-Type":"application/x-www-form-urlencoded","X-Requested-With":"XMLHttpRequest"})}f.reload=function(a){ne(this.Yb.l).location.assign(a);return ne(this.Yb.l).location.reload(!0)};
function yl(a,b,c,d){var e=new Qk;Xk(e,b);e.A("success",function(b){b=b.target.Pc();if(null!=b.location)return a.reload(b.location);c(b);return re(a.Yb,b,a.nc,200)});return e.A("error",function(a){return d(a.target.Ij())})}f.sn=function(a){return xl(a,"application/json")};function xl(a,b){return a.getResponseHeader("Content-Type")?-1!==a.getResponseHeader("Content-Type").indexOf(b):!1};function zl(a,b,c){md.call(this);this.target=a;this.handle=b||a;this.fk=c||new $f(NaN,NaN,NaN,NaN);this.P=lf(a);this.Rb=new fh(this);$b(this,va(ac,this.Rb));this.deltaY=this.deltaX=this.ss=this.rs=this.screenY=this.screenX=this.clientY=this.clientX=0;this.Qb=!0;this.rf=!1;this.Fr=!0;this.Jq=0;this.$k=this.ev=!1;v(this.handle,["touchstart","mousedown"],this.ps,!1,this)}r(zl,md);var Al=ba.document&&ba.document.documentElement&&!!ba.document.documentElement.setCapture&&!!ba.document.releaseCapture;
f=zl.prototype;f.Sa=function(){return this.Rb};f.pa=function(a){this.Qb=a};f.V=function(){zl.m.V.call(this);dd(this.handle,["touchstart","mousedown"],this.ps,!1,this);this.Rb.ld();Al&&this.P.releaseCapture();this.handle=this.target=null};function Bl(a){ca(a.Cd)||(a.Cd=vg(a.target));return a.Cd}
f.ps=function(a){var b="mousedown"==a.type;if(!this.Qb||this.rf||b&&!Jc(a))this.dispatchEvent("earlycancel");else{if(0==this.Jq)if(this.dispatchEvent(new Cl("start",this,a.clientX,a.clientY)))this.rf=!0,this.Fr&&b&&a.preventDefault();else return;else this.Fr&&b&&a.preventDefault();var b=this.P,c=b.documentElement,d=!Al;this.Rb.A(b,["touchmove","mousemove"],this.Tu,d);this.Rb.A(b,["touchend","mouseup"],this.xj,d);Al?(c.setCapture(!1),this.Rb.A(c,"losecapture",this.xj)):this.Rb.A(wf(b),"blur",this.xj);
t&&this.ev&&this.Rb.A(b,"dragstart",Ec);this.Wv&&this.Rb.A(this.Wv,"scroll",this.Bv,d);this.clientX=this.rs=a.clientX;this.clientY=this.ss=a.clientY;this.screenX=a.screenX;this.screenY=a.screenY;this.$k?(a=this.target,c=a.offsetLeft,d=a.offsetParent,d||"fixed"!=fg(a)||(d=lf(a).documentElement),d?(jc?(b=qg(d),c+=b.left):vc(8)&&!vc(9)&&(b=qg(d),c-=b.left),a=vg(d)?d.clientWidth-(c+a.offsetWidth):c):a=c):a=this.target.offsetLeft;this.deltaX=a;this.deltaY=this.target.offsetTop;this.Wn=uf(jf(this.P).P)}};
f.xj=function(a){this.Rb.ld();Al&&this.P.releaseCapture();this.rf?(this.rf=!1,this.dispatchEvent(new Cl("end",this,a.clientX,a.clientY,0,Dl(this,this.deltaX),El(this,this.deltaY)))):this.dispatchEvent("earlycancel")};
f.Tu=function(a){if(this.Qb){var b=(this.$k&&Bl(this)?-1:1)*(a.clientX-this.clientX),c=a.clientY-this.clientY;this.clientX=a.clientX;this.clientY=a.clientY;this.screenX=a.screenX;this.screenY=a.screenY;if(!this.rf){var d=this.rs-this.clientX,e=this.ss-this.clientY;if(d*d+e*e>this.Jq)if(this.dispatchEvent(new Cl("start",this,a.clientX,a.clientY)))this.rf=!0;else{this.Ee||this.xj(a);return}}c=Fl(this,b,c);b=c.x;c=c.y;this.rf&&this.dispatchEvent(new Cl("beforedrag",this,a.clientX,a.clientY,0,b,c))&&
(Gl(this,a,b,c),a.preventDefault())}};function Fl(a,b,c){var d=uf(jf(a.P).P);b+=d.x-a.Wn.x;c+=d.y-a.Wn.y;a.Wn=d;a.deltaX+=b;a.deltaY+=c;return new df(Dl(a,a.deltaX),El(a,a.deltaY))}f.Bv=function(a){var b=Fl(this,0,0);a.clientX=this.clientX;a.clientY=this.clientY;Gl(this,a,b.x,b.y)};function Gl(a,b,c,d){a.$k&&Bl(a)?a.target.style.right=c+"px":a.target.style.left=c+"px";a.target.style.top=d+"px";a.dispatchEvent(new Cl("drag",a,b.clientX,b.clientY,0,c,d))}
function Dl(a,b){var c=a.fk,d=isNaN(c.left)?null:c.left,c=isNaN(c.width)?0:c.width;return Math.min(null!=d?d+c:Infinity,Math.max(null!=d?d:-Infinity,b))}function El(a,b){var c=a.fk,d=isNaN(c.top)?null:c.top,c=isNaN(c.height)?0:c.height;return Math.min(null!=d?d+c:Infinity,Math.max(null!=d?d:-Infinity,b))}function Cl(a,b,c,d,e,g,h){Cc.call(this,a);this.clientX=c;this.clientY=d;this.left=ca(g)?g:b.deltaX;this.top=ca(h)?h:b.deltaY}r(Cl,Cc);(function(){for(var a=["ms","moz","webkit","o"],b=0,c;c=a[b]&&!ba.requestAnimationFrame;++b)ba.requestAnimationFrame=ba[c+"RequestAnimationFrame"],ba.cancelAnimationFrame=ba[c+"CancelAnimationFrame"]||ba[c+"CancelRequestAnimationFrame"];if(!ba.requestAnimationFrame){var d=0;ba.requestAnimationFrame=function(a){var b=(new Date).getTime(),c=Math.max(0,16-(b-d));d=b+c;return ba.setTimeout(function(){a(b+c)},c)};ba.cancelAnimationFrame||(ba.cancelAnimationFrame=function(a){clearTimeout(a)})}})();
var Hl=[[],[]],Il=0,Jl=!1,Nl=0;function Ol(a,b){var c=Nl++,d={lv:{id:c,fn:a.measure,context:b},ov:{id:c,fn:a.nv,context:b},state:{},ub:void 0,$j:!1};return function(){0<arguments.length?(d.ub||(d.ub=[]),d.ub.length=0,d.ub.push.apply(d.ub,arguments),d.ub.push(d.state)):d.ub&&0!=d.ub.length?(d.ub[0]=d.state,d.ub.length=1):d.ub=[d.state];d.$j||(d.$j=!0,Hl[Il].push(d));Jl||(Jl=!0,window.requestAnimationFrame(Pl))}}
function Pl(){Jl=!1;var a=Hl[Il],b=a.length;Il=(Il+1)%2;for(var c,d=0;d<b;++d){c=a[d];var e=c.lv;c.$j=!1;e.fn&&e.fn.apply(e.context,c.ub)}for(d=0;d<b;++d)c=a[d],e=c.ov,c.$j=!1,e.fn&&e.fn.apply(e.context,c.ub),c.state={};a.length=0};var Ql=t?Ke(Ae(Be('javascript:""'))):Ke(Ae(Be("about:blank"))),Rl=Je(Ql),Sl=t?Ke(Ae(Be('javascript:""'))):Ke(Ae(Be("javascript:undefined")));Je(Sl);function Tl(a){md.call(this);this.F=a;a=t?"focusout":"blur";this.fv=v(this.F,t?"focusin":"focus",this,!t);this.gv=v(this.F,a,this,!t)}r(Tl,md);Tl.prototype.handleEvent=function(a){var b=new Hc(a.Wc);b.type="focusin"==a.type||"focus"==a.type?"focusin":"focusout";this.dispatchEvent(b)};Tl.prototype.V=function(){Tl.m.V.call(this);ed(this.fv);ed(this.gv);delete this.F};function Ul(a,b){this.F=a;this.R=b};function Vl(a,b){kh.call(this,b);this.kw=!!a;this.Ig=null;this.Or=Ol({nv:this.zk},this)}r(Vl,kh);f=Vl.prototype;f.qm=null;f.La=!1;f.yc=null;f.ec=null;f.nd=null;f.Gl=!1;f.ta=function(){return"goog-modalpopup"};f.Cj=function(){return this.yc};f.D=function(){Vl.m.D.call(this);var a=this.o(),b=Ea(this.ta()).split(" ");ki(a,b);Qf(a,!0);w(a,!1);Wl(this);Xl(this)};
function Wl(a){if(a.kw&&!a.ec){var b=a.J().D("IFRAME",{frameborder:0,style:"border:0;vertical-align:bottom;",src:Rl});a.ec=b;a.ec.className=a.ta()+"-bg";w(a.ec,!1);ug(a.ec,0)}a.yc||(a.yc=a.J().D("DIV",a.ta()+"-bg"),w(a.yc,!1))}function Xl(a){a.nd||(a.nd=a.J().createElement("SPAN"),w(a.nd,!1),Qf(a.nd,!0),a.nd.style.position="absolute")}f.Nr=function(){this.Gl=!1};f.wb=function(a){return!!a&&"DIV"==a.tagName};
f.Ga=function(a){Vl.m.Ga.call(this,a);a=Ea(this.ta()).split(" ");ki(this.o(),a);Wl(this);Xl(this);Qf(this.o(),!0);w(this.o(),!1)};f.Ea=function(){this.ec&&Df(this.ec,this.o());Df(this.yc,this.o());Vl.m.Ea.call(this);Ef(this.nd,this.o());this.qm=new Tl(this.J().P);this.Sa().A(this.qm,"focusin",this.onFocus);Yl(this,!1)};f.fb=function(){this.za()&&this.da(!1);ac(this.qm);Vl.m.fb.call(this);Ff(this.ec);Ff(this.yc);Ff(this.nd)};
f.da=function(a){a!=this.La&&(this.Rg&&this.Rg.stop(),this.ph&&this.ph.stop(),this.Qg&&this.Qg.stop(),this.oh&&this.oh.stop(),this.la&&Yl(this,a),a?this.Do():this.Bg())};
function Yl(a,b){a.dr||(a.dr=new Ul(a.F,a.R));var c=a.dr;if(b){c.Ag||(c.Ag=[]);for(var d=c.R.he(c.R.P.body),e=0;e<d.length;e++){var g=d[e],h;if(h=g!=c.F)h=g.getAttribute("aria-hidden"),h=!(null==h||void 0==h?0:String(h));h&&(fi(g,"hidden",!0),c.Ag.push(g))}}else if(c.Ag){for(e=0;e<c.Ag.length;e++)c.Ag[e].removeAttribute("aria-hidden");c.Ag=null}}
f.Do=function(){if(this.dispatchEvent("beforeshow")){try{this.Ig=this.J().P.activeElement}catch(a){}this.zk();this.uc();this.Sa().A(ne(this.J()),"resize",this.zk).A(ne(this.J()),"orientationchange",this.Or);Zl(this,!0);this.focus();this.La=!0;this.Rg&&this.ph?(cd(this.Rg,"end",this.Uf,!1,this),this.ph.play(),this.Rg.play()):this.Uf()}};
f.Bg=function(){if(this.dispatchEvent("beforehide")){this.Sa().Aa(ne(this.J()),"resize",this.zk).Aa(ne(this.J()),"orientationchange",this.Or);this.La=!1;this.Qg&&this.oh?(cd(this.Qg,"end",this.Ve,!1,this),this.oh.play(),this.Qg.play()):this.Ve();a:{try{var a=this.J(),b=a.P.body,c=a.P.activeElement||b;if(!this.Ig||this.Ig==b){this.Ig=null;break a}(c==b||a.contains(this.o(),c))&&this.Ig.focus()}catch(d){}this.Ig=null}}};function Zl(a,b){a.ec&&w(a.ec,b);a.yc&&w(a.yc,b);w(a.o(),b);w(a.nd,b)}f.Uf=function(){this.dispatchEvent("show")};
f.Ve=function(){Zl(this,!1);this.dispatchEvent("hide")};f.za=function(){return this.La};f.focus=function(){this.uq()};f.zk=function(){this.ec&&w(this.ec,!1);this.yc&&w(this.yc,!1);var a=this.J().P,b=sf(wf(a)||window||window),c=Math.max(b.width,Math.max(a.body.scrollWidth,a.documentElement.scrollWidth)),a=Math.max(b.height,Math.max(a.body.scrollHeight,a.documentElement.scrollHeight));this.ec&&(w(this.ec,!0),sg(this.ec,c,a));this.yc&&(w(this.yc,!0),sg(this.yc,c,a))};
f.uc=function(){var a;var b=this.J().P;var c=wf(b)||window;"fixed"==fg(this.o())?b=a=0:(b=uf(this.J().P),a=b.x,b=b.y);var d=tg(this.o()),c=sf(c||window);a=Math.max(a+c.width/2-d.width/2,0);b=Math.max(b+c.height/2-d.height/2,0);ig(this.o(),a,b);ig(this.nd,a,b)};f.onFocus=function(a){this.Gl?this.Nr():a.target==this.nd&&od(this.uq,0,this)};f.uq=function(){try{t&&this.J().P.body.focus(),this.o().focus()}catch(a){}};
f.V=function(){ac(this.Rg);this.Rg=null;ac(this.Qg);this.Qg=null;ac(this.ph);this.ph=null;ac(this.oh);this.oh=null;Vl.m.V.call(this)};function $l(a,b,c){Vl.call(this,b,c);this.hc=a||"modal-dialog";this.vb=am(am(new bm,cm,!0),dm,!1,!0)}r($l,Vl);f=$l.prototype;f.Au=!0;f.Rj=!0;f.er=!0;f.xu=!0;f.nh=.5;f.Di="";f.Ab=null;f.Fe=null;f.vu=!1;f.Nb=null;f.cc=null;f.Uk=null;f.xc=null;f.ac=null;f.cb=null;f.ao="dialog";f.ta=function(){return this.hc};f.wi=function(a){this.Di=a;this.cc&&Kf(this.cc,a)};f.Aq=function(){return this.Di};f.Sc=function(a){a instanceof Re||(a=Ue(a),a=Ve(Ga(Te(a)),a.ie()));this.Ab=a;this.ac&&(this.ac.innerHTML=Te(a))};
f.wd=function(){return null!=this.Ab?Te(this.Ab):""};f.je=function(){return this.ao};f.Eb=function(){this.o()||this.render();return this.ac};function em(a){a.o()||a.render();return a.o()}f.Cj=function(){this.o()||this.render();return $l.m.Cj.call(this)};function fm(a,b){a.nh=b;if(a.o()){var c=a.Cj();c&&ug(c,a.nh)}}f.um=function(){return this.ta()};
function gm(a,b){var c=Ea(a.hc+"-title-draggable").split(" ");a.o()&&(b?ki(a.Nb,c):mi(a.Nb,c));if(b&&!a.Fe){var d=new zl(a.o(),a.Nb);a.Fe=d;ki(a.Nb,c);v(a.Fe,"start",a.$v,!1,a)}else!b&&a.Fe&&(a.Fe.eb(),a.Fe=null)}
f.D=function(){$l.m.D.call(this);var a=this.o(),b=this.J();this.Nb=b.D("DIV",this.hc+"-title",this.cc=b.D("SPAN",{className:this.hc+"-title-text",id:nh(this)},this.Di),this.xc=b.D("SPAN",this.hc+"-title-close"));Bf(a,this.Nb,this.ac=b.D("DIV",this.hc+"-content"),this.cb=b.D("DIV",this.hc+"-buttons"));ei(this.cc,"heading");ei(this.xc,"button");Qf(this.xc,!0);fi(this.xc,"label",hm);this.Uk=this.cc.id;ei(a,this.je());fi(a,"labelledby",this.Uk||"");this.Ab&&(this.ac.innerHTML=Te(this.Ab));w(this.xc,this.Rj);
this.vb&&(a=this.vb,a.F=this.cb,a.render());w(this.cb,!!this.vb);fm(this,this.nh)};
f.Ga=function(a){$l.m.Ga.call(this,a);a=this.o();var b=this.hc+"-content";this.ac=pf(document,null,b,a)[0];this.ac||(this.ac=this.J().D("DIV",b),this.Ab&&(this.ac.innerHTML=Te(this.Ab)),a.appendChild(this.ac));var b=this.hc+"-title",c=this.hc+"-title-text",d=this.hc+"-title-close";(this.Nb=pf(document,null,b,a)[0])?(this.cc=pf(document,null,c,this.Nb)[0],this.xc=pf(document,null,d,this.Nb)[0]):(this.Nb=this.J().D("DIV",b),a.insertBefore(this.Nb,this.ac));this.cc?(this.Di=Rf(this.cc),this.cc.id||(this.cc.id=
nh(this))):(this.cc=xf("SPAN",{className:c,id:nh(this)}),this.Nb.appendChild(this.cc));this.Uk=this.cc.id;fi(a,"labelledby",this.Uk||"");this.xc||(this.xc=this.J().D("SPAN",d),this.Nb.appendChild(this.xc));w(this.xc,this.Rj);b=this.hc+"-buttons";(this.cb=pf(document,null,b,a)[0])?(this.vb=new bm(this.J()),this.vb.M(this.cb)):(this.cb=this.J().D("DIV",b),a.appendChild(this.cb),this.vb&&(a=this.vb,a.F=this.cb,a.render()),w(this.cb,!!this.vb));fm(this,this.nh)};
f.Ea=function(){$l.m.Ea.call(this);this.Sa().A(this.o(),"keydown",this.ei).A(this.o(),"keypress",this.ei);this.Sa().A(this.cb,"click",this.tv);gm(this,this.xu);this.Sa().A(this.xc,"click",this.Dv);var a=this.o();ei(a,this.je());""!==this.cc.id&&fi(a,"labelledby",this.cc.id);if(!this.er){this.er=!1;if(this.la){var a=this.J(),b=this.Cj();a.removeNode(this.ec);a.removeNode(b)}this.za()&&Yl(this,!1)}};f.fb=function(){this.za()&&this.da(!1);gm(this,!1);$l.m.fb.call(this)};
f.da=function(a){a!=this.za()&&(this.la||this.render(),$l.m.da.call(this,a))};f.Uf=function(){$l.m.Uf.call(this);this.dispatchEvent(im)};f.Ve=function(){$l.m.Ve.call(this);this.dispatchEvent(jm);this.vu&&this.eb()};
f.$v=function(){var a=this.J().P,b=sf(wf(a)||window||window),c=Math.max(a.body.scrollWidth,b.width),a=Math.max(a.body.scrollHeight,b.height),d=tg(this.o());"fixed"==fg(this.o())?this.Fe.fk=new $f(0,0,Math.max(0,b.width-d.width),Math.max(0,b.height-d.height)):this.Fe.fk=new $f(0,0,c-d.width,a-d.height)};f.Dv=function(){km(this)};function km(a){if(a.Rj){var b=a.vb,c=b&&b.Pl;c?(b=b.get(c),a.dispatchEvent(new lm(c,b))&&a.da(!1)):a.da(!1)}}f.V=function(){this.cb=this.xc=null;$l.m.V.call(this)};
function mm(a,b){a.vb=b;if(a.cb){if(a.vb){var c=a.vb;c.F=a.cb;c.render()}else a.cb.innerHTML=Te(cf);w(a.cb,!!a.vb)}}f.tv=function(a){a:{for(a=a.target;null!=a&&a!=this.cb;){if("BUTTON"==a.tagName)break a;a=a.parentNode}a=null}if(a&&!a.disabled){a=a.name;var b=this.vb.get(a);this.dispatchEvent(new lm(a,b))&&this.da(!1)}};
f.ei=function(a){var b=!1,c=!1,d=this.vb;var e=a.target;if("keydown"==a.type)if(this.Au&&27==a.keyCode){var g=d&&d.Pl;e="SELECT"==e.tagName&&!e.disabled;g&&!e?(c=!0,b=d.get(g),b=this.dispatchEvent(new lm(g,b))):e||(b=!0)}else{if(9==a.keyCode&&a.shiftKey&&e==this.o()){this.Gl=!0;try{this.nd.focus()}catch(p){}od(this.Nr,0,this)}}else if(13==a.keyCode){if("BUTTON"==e.tagName&&!e.disabled)g=e.name;else if(e==this.xc)km(this);else if(d){var h=d.pj,k;if(k=h)a:{k=nf("BUTTON",d.F);for(var l=0,m;m=k[l];l++)if(m.name==
h||m.id==h){k=m;break a}k=null}e=("TEXTAREA"==e.tagName||"SELECT"==e.tagName||"A"==e.tagName)&&!e.disabled;!k||k.disabled||e||(g=h)}g&&d&&(c=!0,b=this.dispatchEvent(new lm(g,String(d.get(g)))))}else e==this.xc&&32==a.keyCode&&km(this);if(b||c)a.stopPropagation(),a.preventDefault();b&&this.da(!1)};function lm(a,b){this.type=nm;this.key=a;this.caption=b}r(lm,Cc);var nm="dialogselect",jm="afterhide",im="aftershow";function bm(a){this.R=a||jf();Pd.call(this)}r(bm,Pd);f=bm.prototype;f.hc="goog-buttonset";
f.pj=null;f.F=null;f.Pl=null;f.set=function(a,b,c,d){Pd.prototype.set.call(this,a,b);c&&(this.pj=a);d&&(this.Pl=a);return this};function am(a,b,c,d){return a.set(b.key,b.caption,c,d)}f.render=function(){if(this.F){this.F.innerHTML=Te(cf);var a=jf(this.F);this.forEach(function(b,c){var d=a.D("BUTTON",{name:c},b);c==this.pj&&(d.className=this.hc+"-default");this.F.appendChild(d)},this)}};
f.M=function(a){if(a&&1==a.nodeType){this.F=a;a=nf("BUTTON",this.F);for(var b=0,c,d,e;c=a[b];b++)if(d=c.name||c.id,e=Rf(c)||c.value,d){var g=0==b;this.set(d,e,g,c.name==om);g&&ji(c,this.hc+"-default")}}};f.o=function(){return this.F};f.J=function(){return this.R};var om="cancel",hm="Close",cm={key:"ok",caption:"OK"},dm={key:om,caption:"Cancel"},pm={key:"yes",caption:"Yes"},qm={key:"no",caption:"No"},rm={key:"save",caption:"Save"},sm={key:"continue",caption:"Continue"};
"undefined"!=typeof document&&(am(new bm,cm,!0,!0),am(am(new bm,cm,!0),dm,!1,!0),am(am(new bm,pm,!0),qm,!1,!0),am(am(am(new bm,pm),qm,!0),dm,!1,!0),am(am(am(new bm,sm),rm),dm,!0,!0));function tm(){}r(tm,ni);ka(tm);f=tm.prototype;f.yf=function(){return"button"};f.Fd=function(a,b,c){switch(b){case 8:case 16:fi(a,"pressed",c);break;default:case 64:case 1:tm.m.Fd.call(this,a,b,c)}};f.D=function(a){var b=tm.m.D.call(this,a);this.Ik(b,a.Lj());var c=a.aa();c&&this.ra(b,c);ui(a,16)&&this.Fd(b,16,!!(a.ma&16));return b};f.M=function(a,b){b=tm.m.M.call(this,a,b);var c=this.aa(b);a.Vo=c;a.Oo=this.Lj(b);ui(a,16)&&this.Fd(b,16,!!(a.ma&16));return b};f.aa=ja;f.ra=ja;f.Lj=function(a){return a.title};
f.Ik=function(a,b){a&&(b?a.title=b:a.removeAttribute("title"))};f.ta=function(){return"goog-button"};function um(){}r(um,tm);ka(um);f=um.prototype;f.yf=function(){};f.D=function(a){Ai(a,!1);a.mh&=-256;a.mc(32,!1);return a.J().D("BUTTON",{"class":this.Zc(a).join(" "),disabled:!a.isEnabled(),title:a.Lj()||"",value:a.aa()||""},a.ge()||"")};f.wb=function(a){return"BUTTON"==a.tagName||"INPUT"==a.tagName&&("button"==a.type||"submit"==a.type||"reset"==a.type)};f.M=function(a,b){Ai(a,!1);a.mh&=-256;a.mc(32,!1);if(b.disabled){var c=this.Kh(1);ji(b,c)}return um.m.M.call(this,a,b)};
f.Lf=function(a){a.Sa().A(a.o(),"click",a.Pg)};f.Wg=ja;f.Zg=ja;f.ne=function(a){return a.isEnabled()};f.ve=ja;f.setState=function(a,b,c){um.m.setState.call(this,a,b,c);(a=a.o())&&1==b&&(a.disabled=c)};f.aa=function(a){return a.value};f.ra=function(a,b){a&&(a.value=b)};f.Fd=ja;function vm(a,b,c){zi.call(this,a,b||um.va(),c)}r(vm,zi);f=vm.prototype;f.aa=function(){return this.Vo};f.ra=function(a){this.Vo=a;this.U.ra(this.o(),a)};f.Lj=function(){return this.Oo};f.Ik=function(a){this.Oo=a;this.U.Ik(this.o(),a)};f.V=function(){vm.m.V.call(this);delete this.Vo;delete this.Oo};f.Ea=function(){vm.m.Ea.call(this);if(ui(this,32)){var a=this.Fb();a&&this.Sa().A(a,"keyup",this.le)}};
f.le=function(a){return 13==a.keyCode&&"key"==a.type||32==a.keyCode&&"keyup"==a.type?this.Pg(a):32==a.keyCode};wi("goog-button",function(){return new vm(null)});function wm(a,b){var c=Array.prototype.slice.call(arguments),d=c.shift();if("undefined"==typeof d)throw Error("[goog.string.format] Template required");return d.replace(/%([0\-\ \+]*)(\d+)?(\.(\d+))?([%sfdiu])/g,function(a,b,d,k,l,m,p,u){if("%"==m)return"%";var e=c.shift();if("undefined"==typeof e)throw Error("[goog.string.format] Not enough arguments");arguments[0]=e;return wm.de[m].apply(null,arguments)})}wm.de={};
wm.de.s=function(a,b,c){return isNaN(c)||""==c||a.length>=Number(c)?a:a=-1<b.indexOf("-",0)?a+Va(" ",Number(c)-a.length):Va(" ",Number(c)-a.length)+a};
wm.de.f=function(a,b,c,d,e){d=a.toString();isNaN(e)||""==e||(d=parseFloat(a).toFixed(e));var g=0>Number(a)?"-":0<=b.indexOf("+")?"+":0<=b.indexOf(" ")?" ":"";0<=Number(a)&&(d=g+d);if(isNaN(c)||d.length>=Number(c))return d;d=isNaN(e)?Math.abs(Number(a)).toString():Math.abs(Number(a)).toFixed(e);a=Number(c)-d.length-g.length;0<=b.indexOf("-",0)?d=g+d+Va(" ",a):(b=0<=b.indexOf("0",0)?"0":" ",d=g+Va(b,a)+d);return d};wm.de.d=function(a,b,c,d,e,g,h,k){return wm.de.f(parseInt(a,10),b,c,d,0,g,h,k)};
wm.de.i=wm.de.d;wm.de.u=wm.de.d;function xm(a){var b=["date","datetime"],c;var d=0;for(c=b.length;d<c;d++){var e=b[d];if(dh(a,e))return!0}return!1};function ym(a,b){var c,d;null==c&&(c=500);if(xm(a)||"date"===(d=a.type)||"datetime"===d){var e=!1;v(a,"change",function(a){e||b(a);return e=!1});v(a,"keyup",function(a){b(a);return e=!0})}else{var g=null;var h=v(a,"change",function(a){g&&g.stop();return b(a)});v(a,"keyup",function(a){g&&g.stop();g=new Kg(function(){b(a);return ed(h)},c);return g.start()})}}function zm(a,b){v(a,"keypress",function(a){if(13===a.keyCode)return b(a)})}function Am(a,b){v(a,"click",b);zm(a,b)};function Bm(a,b,c){a.push(encodeURIComponent(b)+"\x3d"+encodeURIComponent(c))}function Cm(a){var b=a.type;if(!ca(b))return null;switch(b.toLowerCase()){case "checkbox":case "radio":return a.checked?a.value:null;case "select-one":return b=a.selectedIndex,0<=b?a.options[b].value:null;case "select-multiple":for(var b=[],c,d=0;c=a.options[d];d++)c.selected&&b.push(c.value);return b.length?b:null;default:return ca(a.value)?a.value:null}}
function Dm(a,b){var c=a.type;if(ca(c))switch(c.toLowerCase()){case "checkbox":case "radio":a.checked=b;break;case "select-one":a.selectedIndex=-1;if(da(b))for(var d=0;c=a.options[d];d++)if(c.value==b){c.selected=!0;break}break;case "select-multiple":c=b;da(c)&&(c=[c]);for(var e=0;d=a.options[e];e++)if(d.selected=!1,c)for(var g,h=0;g=c[h];h++)d.value==g&&(d.selected=!0);break;default:a.value=null!=b?b:""}};function Em(a){return 2>a.length?a:Array.prototype.slice.call(a,0,a.length-1)}function Fm(a,b){null==b&&(b=null);return 0==a.length?b:a[a.length-1]}function Gm(a,b,c){null==c&&(c=null);return 0==a.length?c:b(a[0])?a[0]:Gm(ub(a,1),b,c)}function Hm(a,b){return la(a)?hb(a,b,void 0):[]}function Im(a,b){var c={};return la(a)?ib(a,b,c,void 0):[]}function Jm(a){return la(a)?0==a.length:!0};function Km(a,b,c,d){null==c&&(c=bb);null==d&&(d=null);return Th(a,b)?c(Sh(a,b)):d};function Lm(a,b){null==b&&(b=null);var c=parseInt(a,10);isFinite(c)||null==b||(c=b);return c}function Mm(a){return 0<a};function Nm(a,b){try{return qd(a)}catch(c){return oa(b)?b(a):null}};function Om(a){this.Ep=a}ka(Om);f=Om.prototype;f.yf=function(){return this.Ep};function Pm(a,b){a&&(a.tabIndex=b?0:-1)}f.D=function(a){return a.J().D("DIV",this.Zc(a).join(" "))};f.Eb=function(a){return a};f.wb=function(a){return"DIV"==a.tagName};f.M=function(a,b){b.id&&oh(a,b.id);var c=this.ta(),d=!1,e=hi(b);e&&eb(e,function(b){b==c?d=!0:b&&(b==c+"-disabled"?a.pa(!1):b==c+"-horizontal"?a.setOrientation(Qm):b==c+"-vertical"&&a.setOrientation(Rm))},this);d||ji(b,c);Sm(this,a,this.Eb(b));return b};
function Sm(a,b,c){if(c)for(var d=c.firstChild,e;d&&d.parentNode==c;){e=d.nextSibling;if(1==d.nodeType){var g=a.wm(d);g&&(g.F=d,b.isEnabled()||g.pa(!1),b.$a(g),g.M(d))}else d.nodeValue&&""!=Ea(d.nodeValue)||c.removeChild(d);d=e}}f.wm=function(a){a:{a=hi(a);for(var b=0,c=a.length;b<c;b++){var d=a[b];if(d=d in xi?xi[d]():null){a=d;break a}}a=null}return a};f.Lf=function(a){a=a.o();xg(a,!0,jc);t&&(a.hideFocus=!0);var b=this.yf();b&&ei(a,b)};f.Fb=function(a){return a.o()};f.ta=function(){return"goog-container"};
f.Zc=function(a){var b=this.ta(),c=[b,a.Vf==Qm?b+"-horizontal":b+"-vertical"];a.isEnabled()||c.push(b+"-disabled");return c};function Tm(a,b,c){kh.call(this,c);this.U=b||Om.va();this.Vf=a||Rm}r(Tm,kh);var Qm="horizontal",Rm="vertical";f=Tm.prototype;f.An=null;f.Vb=null;f.U=null;f.Vf=null;f.La=!0;f.Qb=!0;f.rm=!0;f.Xa=-1;f.Kb=null;f.Sf=!1;f.Mt=!1;f.Ev=!0;f.Id=null;f.Fb=function(){return this.An||this.U.Fb(this)};f.Fj=function(){return this.Vb||(this.Vb=new Yh(this.Fb()))};f.D=function(){this.F=this.U.D(this)};f.Eb=function(){return this.U.Eb(this.o())};f.wb=function(a){return this.U.wb(a)};
f.Ga=function(a){this.F=this.U.M(this,a);"none"==a.style.display&&(this.La=!1)};f.Ea=function(){Tm.m.Ea.call(this);rh(this,function(a){a.la&&Um(this,a)},this);var a=this.o();this.U.Lf(this);this.da(this.La,!0);this.Sa().A(this,"enter",this.Lm).A(this,"highlight",this.Nu).A(this,"unhighlight",this.Vu).A(this,"open",this.Uu).A(this,"close",this.Ju).A(a,"mousedown",this.me).A(lf(a),"mouseup",this.Ku).A(a,["mousedown","mouseup","mouseover","mouseout","contextmenu"],this.Iu);this.ne()&&Vm(this,!0)};
function Vm(a,b){var c=a.Sa(),d=a.Fb();b?c.A(d,"focus",a.handleFocus).A(d,"blur",a.handleBlur).A(a.Fj(),"key",a.$c):c.Aa(d,"focus",a.handleFocus).Aa(d,"blur",a.handleBlur).Aa(a.Fj(),"key",a.$c)}f.fb=function(){this.Yg(-1);this.Kb&&this.Kb.setOpen(!1);this.Sf=!1;Tm.m.fb.call(this)};f.V=function(){Tm.m.V.call(this);this.Vb&&(this.Vb.eb(),this.Vb=null);this.U=this.Kb=this.Id=this.An=null};f.Lm=function(){return!0};
f.Nu=function(a){var b=vh(this,a.target);if(-1<b&&b!=this.Xa){var c=th(this,this.Xa);c&&c.md(!1);this.Xa=b;c=th(this,this.Xa);this.Sf&&c.setActive(!0);this.Ev&&this.Kb&&c!=this.Kb&&(ui(c,64)?c.setOpen(!0):this.Kb.setOpen(!1))}b=this.o();null!=a.target.o()&&fi(b,"activedescendant",a.target.o().id)};f.Vu=function(a){a.target==th(this,this.Xa)&&(this.Xa=-1);this.o().removeAttribute("aria-activedescendant")};
f.Uu=function(a){(a=a.target)&&a!=this.Kb&&a.getParent()==this&&(this.Kb&&this.Kb.setOpen(!1),this.Kb=a)};f.Ju=function(a){a.target==this.Kb&&(this.Kb=null);var b=this.o(),c=a.target.o();b&&a.target.ma&2&&c&&gi(b,c)};f.me=function(a){this.Qb&&(this.Sf=!0);var b=this.Fb();b&&Nf(b)?b.focus():a.preventDefault()};f.Ku=function(){this.Sf=!1};
f.Iu=function(a){a:{var b=a.target;if(this.Id)for(var c=this.o();b&&b!==c;){var d=b.id;if(d in this.Id){b=this.Id[d];break a}b=b.parentNode}b=null}if(b)switch(a.type){case "mousedown":b.me(a);break;case "mouseup":b.yg(a);break;case "mouseover":b.xg(a);break;case "mouseout":b.Um(a);break;case "contextmenu":b.Oh(a)}};f.handleFocus=function(){};f.handleBlur=function(){this.Yg(-1);this.Sf=!1;this.Kb&&this.Kb.setOpen(!1)};
f.$c=function(a){return this.isEnabled()&&this.za()&&(0!=sh(this)||this.An)&&this.le(a)?(a.preventDefault(),a.stopPropagation(),!0):!1};
f.le=function(a){var b=th(this,this.Xa);if(b&&"function"==typeof b.$c&&b.$c(a)||this.Kb&&this.Kb!=b&&"function"==typeof this.Kb.$c&&this.Kb.$c(a))return!0;if(a.shiftKey||a.ctrlKey||a.metaKey||a.altKey)return!1;switch(a.keyCode){case 27:if(this.ne())this.Fb().blur();else return!1;break;case 36:Wm(this);break;case 35:Xm(this);break;case 38:if(this.Vf==Rm)Ym(this);else return!1;break;case 37:if(this.Vf==Qm)this.Zj()?Zm(this):Ym(this);else return!1;break;case 40:if(this.Vf==Rm)Zm(this);else return!1;
break;case 39:if(this.Vf==Qm)this.Zj()?Ym(this):Zm(this);else return!1;break;default:return!1}return!0};function Um(a,b){var c=b.o(),c=c.id||(c.id=nh(b));a.Id||(a.Id={});a.Id[c]=b}f.$a=function(a,b){Tm.m.$a.call(this,a,b)};
f.Cl=function(a,b,c){a.Ai|=2;a.Ai|=64;!this.ne()&&this.Mt||a.mc(32,!1);Ai(a,!1);var d=a.getParent()==this?vh(this,a):-1;Tm.m.Cl.call(this,a,b,c);a.la&&this.la&&Um(this,a);a=d;-1==a&&(a=sh(this));a==this.Xa?this.Xa=Math.min(sh(this)-1,b):a>this.Xa&&b<=this.Xa?this.Xa++:a<this.Xa&&b>this.Xa&&this.Xa--};
f.removeChild=function(a,b){if(a=da(a)?qh(this,a):a){var c=vh(this,a);-1!=c&&(c==this.Xa?(a.md(!1),this.Xa=-1):c<this.Xa&&this.Xa--);(c=a.o())&&c.id&&this.Id&&Mb(this.Id,c.id)}a=Tm.m.removeChild.call(this,a,b);Ai(a,!0);return a};f.setOrientation=function(a){if(this.o())throw Error("Component already rendered");this.Vf=a};f.za=function(){return this.La};
f.da=function(a,b){if(b||this.La!=a&&this.dispatchEvent(a?"show":"hide")){this.La=a;var c=this.o();c&&(w(c,a),this.ne()&&Pm(this.Fb(),this.Qb&&this.La),b||this.dispatchEvent(this.La?"aftershow":"afterhide"));return!0}return!1};f.isEnabled=function(){return this.Qb};
f.pa=function(a){this.Qb!=a&&this.dispatchEvent(a?"enable":"disable")&&(a?(this.Qb=!0,rh(this,function(a){a.Gs?delete a.Gs:a.pa(!0)})):(rh(this,function(a){a.isEnabled()?a.pa(!1):a.Gs=!0}),this.Sf=this.Qb=!1),this.ne()&&Pm(this.Fb(),a&&this.La))};f.ne=function(){return this.rm};f.ve=function(a){a!=this.rm&&this.la&&Vm(this,a);this.rm=a;this.Qb&&this.La&&Pm(this.Fb(),a)};f.Yg=function(a){(a=th(this,a))?a.md(!0):-1<this.Xa&&th(this,this.Xa).md(!1)};f.md=function(a){this.Yg(vh(this,a))};
function Wm(a){$m(a,function(a,c){return(a+1)%c},sh(a)-1)}function Xm(a){$m(a,function(a,c){a--;return 0>a?c-1:a},0)}function Zm(a){$m(a,function(a,c){return(a+1)%c},a.Xa)}function Ym(a){$m(a,function(a,c){a--;return 0>a?c-1:a},a.Xa)}function $m(a,b,c){c=0>c?vh(a,a.Kb):c;var d=sh(a);c=b.call(a,c,d);for(var e=0;e<=d;){var g=th(a,c);if(g&&a.Kp(g)){a.Yg(c);break}e++;c=b.call(a,c,d)}}f.Kp=function(a){return a.za()&&a.isEnabled()&&ui(a,2)};function an(){}r(an,ni);ka(an);an.prototype.ta=function(){return"goog-menuheader"};function bn(a,b,c){zi.call(this,a,c||an.va(),b);this.mc(1,!1);this.mc(2,!1);this.mc(4,!1);this.mc(32,!1);this.ma=1}r(bn,zi);wi("goog-menuheader",function(){return new bn(null)});function cn(){this.Qp=[]}r(cn,ni);ka(cn);function dn(a,b){var c=a.Qp[b];if(!c){switch(b){case 0:c=a.ta()+"-highlight";break;case 1:c=a.ta()+"-checkbox";break;case 2:c=a.ta()+"-content"}a.Qp[b]=c}return c}f=cn.prototype;f.yf=function(){return"menuitem"};f.D=function(a){var b=a.J().D("DIV",this.Zc(a).join(" "),this.nj(a.wd(),a.J()));en(this,a,b,ui(a,8)||ui(a,16));return b};f.Eb=function(a){return a&&a.firstChild};
f.M=function(a,b){var c=ca(b.firstElementChild)?b.firstElementChild:Gf(b.firstChild,!0),d=dn(this,2);c&&ii(c,d)||b.appendChild(this.nj(b.childNodes,a.J()));ii(b,"goog-option")&&(a.Ek(!0),this.Ek(a,b,!0));return cn.m.M.call(this,a,b)};f.vc=function(a,b){var c=this.Eb(a),d=fn(this,a)?c.firstChild:null;cn.m.vc.call(this,a,b);d&&!fn(this,a)&&c.insertBefore(d,c.firstChild||null)};f.nj=function(a,b){var c=dn(this,2);return b.D("DIV",c,a)};f.Hk=function(a,b,c){a&&b&&en(this,a,b,c)};
f.Ek=function(a,b,c){a&&b&&en(this,a,b,c)};function fn(a,b){var c=a.Eb(b);if(c){var c=c.firstChild,d=dn(a,1);return!!c&&Hf(c)&&ii(c,d)}return!1}function en(a,b,c,d){si(a,c,b.je());ti(a,b,c);d!=fn(a,c)&&(d?ji(c,"goog-option"):li(c,"goog-option"),c=a.Eb(c),d?(a=dn(a,1),c.insertBefore(b.J().D("DIV",a),c.firstChild||null)):c.removeChild(c.firstChild))}f.Kh=function(a){switch(a){case 2:return dn(this,0);case 16:case 8:return"goog-option-selected";default:return cn.m.Kh.call(this,a)}};
f.Jj=function(a){var b=dn(this,0);switch(a){case "goog-option-selected":return 16;case b:return 2;default:return cn.m.Jj.call(this,a)}};f.ta=function(){return"goog-menuitem"};function gn(a,b,c,d){zi.call(this,a,d||cn.va(),c);this.ra(b)}r(gn,zi);f=gn.prototype;f.aa=function(){var a=this.Ua;return null!=a?a:this.ge()};f.ra=function(a){this.Ua=a};f.mc=function(a,b){gn.m.mc.call(this,a,b);switch(a){case 8:this.ma&16&&!b&&Ei(this,16,!1)&&this.setState(16,!1);var c=this.o();c&&this.U.Hk(this,c,b);break;case 16:(c=this.o())&&this.U.Ek(this,c,b)}};f.Hk=function(a){this.mc(8,a)};f.Ek=function(a){this.mc(16,a)};
f.ge=function(){var a=this.wd();return la(a)?(a=hb(a,function(a){return Hf(a)&&(ii(a,"goog-menuitem-accel")||ii(a,"goog-menuitem-mnemonic-separator"))?"":Tf(a)}).join(""),Da(a)):gn.m.ge.call(this)};f.yg=function(a){var b=this.getParent();if(b){var c=b.zr;b.zr=null;if(c&&ea(a.clientX)&&ef(c,new df(a.clientX,a.clientY)))return}gn.m.yg.call(this,a)};f.le=function(a){return a.keyCode==this.cr&&this.Pg(a)?!0:gn.m.le.call(this,a)};f.Fu=function(){return this.cr};wi("goog-menuitem",function(){return new gn(null)});
gn.prototype.je=function(){return ui(this,16)?"menuitemcheckbox":ui(this,8)?"menuitemradio":gn.m.je.call(this)};gn.prototype.getParent=function(){return zi.prototype.getParent.call(this)};gn.prototype.Hj=function(){return zi.prototype.Hj.call(this)};function hn(){}r(hn,ni);ka(hn);hn.prototype.D=function(a){return a.J().D("DIV",this.ta())};hn.prototype.M=function(a,b){b.id&&oh(a,b.id);if("HR"==b.tagName){var c=b;b=this.D(a);Df(b,c);Ff(c)}else ji(b,this.ta());return b};hn.prototype.vc=function(){};hn.prototype.ta=function(){return"goog-menuseparator"};function jn(a,b){zi.call(this,null,a||hn.va(),b);this.mc(1,!1);this.mc(2,!1);this.mc(4,!1);this.mc(32,!1);this.ma=1}r(jn,zi);jn.prototype.Ea=function(){jn.m.Ea.call(this);var a=this.o();ei(a,"separator")};wi("goog-menuseparator",function(){return new jn});function kn(a){this.Ep=a||"menu"}r(kn,Om);ka(kn);kn.prototype.wb=function(a){return"UL"==a.tagName||kn.m.wb.call(this,a)};kn.prototype.wm=function(a){return"HR"==a.tagName?new jn:kn.m.wm.call(this,a)};kn.prototype.ta=function(){return"goog-menu"};kn.prototype.Lf=function(a){kn.m.Lf.call(this,a);a=a.o();fi(a,"haspopup","true")};wi("goog-menuseparator",function(){return new jn});function ln(a,b){Tm.call(this,Rm,b||kn.va(),a);this.ve(!1)}r(ln,Tm);f=ln.prototype;f.hf=!0;f.Nt=!1;f.ta=function(){return this.U.ta()};f.removeItem=function(a){(a=this.removeChild(a,!0))&&a.eb()};f.setPosition=function(a,b){var c=this.za();c||w(this.o(),!0);var d=this.o(),e=a,g=b,h=og(d);e instanceof df&&(g=e.y,e=e.x);ig(d,d.offsetLeft+(e-h.x),d.offsetTop+(Number(g)-h.y));c||w(this.o(),!1)};f.Xr=function(a){(this.hf=a)&&this.ve(!0)};
f.da=function(a,b,c){(b=ln.m.da.call(this,a,b))&&a&&this.la&&this.hf&&this.Fb().focus();this.zr=a&&c&&ea(c.clientX)?new df(c.clientX,c.clientY):null;return b};f.Lm=function(a){this.hf&&this.Fb().focus();return ln.m.Lm.call(this,a)};f.Kp=function(a){return(this.Nt||a.isEnabled())&&a.za()&&ui(a,2)};f.Ga=function(a){var b=this.U;var c=this.J();c=pf(c.P,"DIV",b.ta()+"-content",a);for(var d=c.length,e=0;e<d;e++)Sm(b,this,c[e]);ln.m.Ga.call(this,a)};
f.le=function(a){var b=ln.m.le.call(this,a);b||rh(this,function(c){!b&&c.Fu&&c.cr==a.keyCode&&(this.isEnabled()&&this.md(c),b=c.$c(a))},this);return b};f.Yg=function(a){ln.m.Yg.call(this,a);(a=th(this,a))&&pg(a.o(),this.o())};function mn(){mn.m.constructor.call(this)}r(mn,kn);mn.prototype.D=function(a){a=a.J().D("ul");a.setAttribute("class","dropdown-menu");return a};ka(mn);function nn(a,b){null==b&&(b=mn.va());nn.m.constructor.call(this,a,b)}r(nn,ln);nn.prototype.da=function(a){nn.m.da.call(this,a);null!=this.o()&&(this.o().style.display=a?"block":"none");return nn.m.da.call(this,a)};function on(){on.m.constructor.call(this)}r(on,cn);on.prototype.D=function(a){var b=a.J();var c=this.nj(a.wd(),b);c=b.D("li",{},c);en(this,a,c,ui(a,8)||ui(a,16));ti(this,a,c);return c};on.prototype.nj=function(a,b){return b.D("a","menuitem",a)};ka(on);function pn(){this.Xl=q(this.Xl,this);pn.m.constructor.call(this);this.Jd.push("paginator");this.Oq=[[!1,!1],[!1,!1]];this.Ko=this.Ci=null;this.Jo="%d per page";this.xn="LI";this.Sq="SPAN";this.Br=qn;this.Rr=[this.ru,this.Xl];this.Si=rn;this.iq=sn;this.hq=tn}r(pn,wh);
var un={el:"active",jl:"disabled",st:"previous",jt:"next",ut:"result",np:"result-number",vt:"result-caption",xt:"result-total",wt:"result-displayed",nl:"paging",ot:"pagination",nt:"page-item",Os:"base-switcher",sy:"paginator"},vn={Ki:"p",nl:"paging",Ps:"baseType",Ns:"baseRange",Ss:"customBaseLabel",Rs:"customBaseErrorMessage"},qn="simple",rn=[1,1E3],sn="Display number of rows",tn="Row number has to be between %s and %s",wn=[["Items total:","%d"],["Displayed","%d to %d"]];f=pn.prototype;
f.D=function(a){var b,c;var d=pn.m.D.call(this,a);var e=a.J();for(b=c=0;1>=c;b=++c){var g=wm.apply(wm,(0,this.Rr[b])(a));b=d;var h=a,k=g,l=g=h.J().D(this.Sq,{});h.J().Sc(l,k);b.appendChild(g)}xn(this,a,d,e);a=yn(this,a,e);d.appendChild(a);this.Ko=a;return this.Ci=d};
function zn(a,b){var c,d,e;var g=[];if(2>b.fi)g=[b.page];else{var h=(b.fi-1)/4;var k=Math.max(b.pm,b.page-3);var l=Math.min(b.Cn,b.page+3);g=function(){e=[];for(var a=k;k<=l?a<=l:a>=l;k<=l?a++:a--)e.push(a);return e}.apply(a);for(c=d=0;4>=d;c=++d)g.push(Math.round(h*c)+b.pm);xb(g);g.sort(function(a,b){return a<b?-1:a>b?1:0})}return g}function An(a,b,c,d,e,g){a.Oq[e][g]||(c=Eg(b,c,d),d="",null!=c&&(d=b.wg(c)),null!=d&&""!==d&&(wn[e][g]=d),a.Oq[e][g]=!0)}
function Bn(a,b,c){zm(c,function(){var d=b.J();var e=Eg(d,"paginator__custom-base-input",c);if(null!=e){var g=Yf(c);e=Lm(Cm(e));var h=a.Si;var k=h[0];h=h[1];if(k<=e&&e<=h)return g=Eg(d,"paginator__error-message",g),null!=g&&d.removeNode(g),b.ti(e);e=a.Si;e=d.H("div","paginator__error-message",[d.H("div","alert alert-danger",wm(a.hq,e[0],e[1]))]);return d.appendChild(g,e)}})}
f.Ut=function(a){if(la(a)&&2===a.length&&kb(a,Mm))return a;ba.console.warn('Paginator "data-base-rage" contains invalid value. A tuple with two positive number was expected.');return rn};
f.M=function(a,b){var c,d,e,g;var h=a.J();var k=[Eg(h,un.xt,b),Eg(h,un.wt,b)];var l=c=0;for(g=k.length;c<g;l=++c){var m=k[l];null!=m&&(An(this,h,un.vt,m,l,0),An(this,h,un.np,m,l,1),m=Eg(h,un.np,m),null!=m&&(l=wm.apply(wm,(0,this.Rr[l])(a)),a.J().Sc(m,l)))}this.Br=Km(b,vn.nl,String,qn);c=null!=(d=Eg(h,un.ot,b))?d:b;g=Eg(h,un.st,c);a.page===a.pm||0===a.total?Cn(g,un.jl,un.el):this.vi(g,a.page-1);d=h.P.createDocumentFragment();"full"===this.Br&&xn(this,a,d,h);h.Vj(d,g);d=Eg(h,un.jt,c);a.page===a.Cn||
0===a.total?Cn(d,un.jl,un.el):this.vi(d,a.page+1);this.iq=Km(b,vn.Ss,String,sn);this.hq=Km(b,vn.Rs,String,tn);this.Si=Km(b,vn.Ns,cb(this.Ut,Nm));"custom"===Km(b,vn.Ps)&&(c=a.J(),d=Eg(c,"paginator__custom-base-wrapper",b),null!=d&&c.removeNode(d),d=a.J(),c=Eg(d,"result-displayed",b),null!=c&&(g=Yf(c),c=d.H("span","paginator__custom-base-wrapper"),Vf(g,c),d.appendChild(c,d.H("span","paginator__custom-base-label",this.iq)),k=this.Si,g=k[0],k=k[1],g=d.H("input",{type:"number",value:a.Dj(),"class":"paginator__custom-base-input",
min:g,max:k}),d.appendChild(c,g),Bn(this,a,g)));d=null!=(e=Eg(h,un.nl,b))?e:b;this.Ci?Vf(d,this.Ci):(e=Eg(h,un.Os,b),null!=e&&(d=Km(e,"pattern"),null!=d&&(this.Jo=d),h=yn(this,a,h),e.appendChild(h),this.Ko=h,this.Ci=e,Dm(this.Ko,String(a.base))));return ag(b,"visibility","inherit")};f.Xl=function(a){return[wn[1][1],0===(0===a.total?0:a.offset+a.count)||0===a.total?0:a.offset+1,0===a.total?0:a.offset+a.count]};f.ru=function(a){return[wn[0][1],a.total]};
function xn(a,b,c,d){var e,g;var h=zn(a,b);var k=[];var l=e=0;for(g=h.length;e<g;l=++e){var m=h[l];var p=d.H(a.Sq,{},String(m));var u=d.H(a.xn,"page-item");m===b.page?Cn(u,un.el,un.jl):a.vi(u,m);u.appendChild(p);c.appendChild(u);h[l+1]>m+1?k.push(c.appendChild(d.D(a.xn,{},"..."))):k.push(void 0)}}f.vi=function(a,b){ah(a,"disabled");return Rh(a,"p",String(b))};function Cn(a,b,c){Zg(a,b);ah(a,c);Ph&&a.dataset?Th(a,"p")&&delete a.dataset.p:a.removeAttribute("data-"+$a("p"))}
function yn(a,b,c){var d;var e=c.D("div","dropdown");a.ig=c.D("button","btn btn-default dropdown-toggle");Dn(a,a.ig,b.base);c.appendChild(e,a.ig);var g=new nn(a.l);g.da(!1);var h=b.Il;var k=0;for(d=h.length;k<d;k++){var l=h[k];var m=new gn(wm(a.Jo,l),l,a.l,on.va());g.$a(m,!0)}v(g,"action",function(d){l=d.target.Ua;g.da(!1);c.appendChild(e,a.ig);Dn(a,a.ig,l);return b.ti(l)});g.render(e);v(a.ig,"click",function(){return g.da(!g.za())});v(c.P.body,"click",function(){if(g.za())return g.da(!1)},!0);return e}
function Dn(a,b,c){if(null!=c&&null!=b)return b.innerHTML=[wm(a.Jo,c),'\x3cspan class\x3d"caret"\x3e\x3c/span\x3e'].join("")}f.ti=function(a){return Dn(this,this.ig,a)};f.Am=function(a){for(;;){if(!a||dh(a,"pagination"))return null;if(a.tagName===this.xn&&Th(a,"p"))return Km(a,"p");a=Yf(a)}};function En(a){this.Hl=q(this.Hl,this);null==a.S&&(a.S=new pn);En.m.constructor.call(this,a);this.base=a.base;this.page=a.page;this.base=0<=this.base?this.base:Fn;this.page=0<=this.page?this.page:1;this.pm=1;this.lj=[];this.bb=[];this.Ci=null;this.Il=this.lq=[10,25,50,100,500,1E3];this.vq=!1}r(En,xh);var Fn=10;f=En.prototype;f.Dj=function(){return this.base};f.Hl=function(a){return Gm([this.base,a],Mm,Fn)};
f.init=function(a,b){this.total=a;this.count=b;Gn(this);this.Cn=this.fi;return null!=this.count?this.count:this.count=this.base};function Gn(a){a.fi=Math.ceil(a.total/a.base);return a.Cn=a.fi}f.wb=function(a){return null!=a&&dh(a,"paginator")};f.jh=function(){return Hn(this,Eg(this.J(),"pagination",this.o()))};
f.refresh=function(a){var b;this.total=a.total;this.count=a.count;null!=a.offset&&(this.offset=a.offset);Gn(this);a=this.J();var c=this.o();if(null!=c){var d=Xf(c);a.removeNode(c);var e=Fg(a,un.nt,c);var g=0;for(b=e.length;g<b;g++){var h=e[g];a.removeNode(h)}e=Fg(a,un.ut,c);g=0;for(b=e.length;g<b;g++)h=e[g],h.innerHTML="";a.Mq(c,d)}this.Ga(this.o());this.jh();h=[];b=this.lj;c=0;for(g=b.length;c<g;c++)d=b[c],a=this.clone(),Wf(a,d),h.push(a);this.lj=h;return this.show(!0)};
f.ti=function(a){this.base=a;this.offset=0;this.page=1;Gn(this);return In(this)};f.Ga=function(a){this.vq="true"===Km(a,"forceDisplay");if(!this.Il){var b=a.querySelector(".base-switcher");(null!=b||this.Il)&&Km(b,"bases",qd,this.lq)}this.S.M(this,a);this.F=a;return Jn(this,!1)};f.vi=function(a){this.page=a;this.offset=(this.page-1)*this.base;return Gn(this)};
function Hn(a,b){var c=v(b,"click",function(b){b=a.S.Am(b.target,a.l);if(null!=b)return a.page=Lm(b),a.offset=(a.page-1)*a.base,In(a)});return a.bb.push(c)}function In(a){var b;var c=a.bb;var d=0;for(b=c.length;d<b;d++){var e=c[d];ed(e)}a.bb=[];return a.dispatchEvent(new Cc("go-to",{offset:a.offset,base:a.base,page:a.page}))}f.clone=function(){var a=this.o().cloneNode(!0);Hn(this,a);Zg(this.o(),"top");Zg(a,"bottom");ah(a,"top");Bn(this.S,this,a);return a};
f.show=function(a,b){null==b&&(b=!1);return!b&&2>this.fi&&!this.vq?Jn(this,!1):Jn(this,a)};function Jn(a,b){var c;var d=b?ah:Zg;var e=[a.o()].concat(a.lj);var g=[];var h=0;for(c=e.length;h<c;h++){var k=e[h];g.push(d(k,"empty"))}return g};function Kn(a,b){this.l=a;this.H=b;this.sc=q(this.sc,this);this.name="";if(!Th(this.H,"col")){var c=this.l.wg(this.H);this.name=c?String(c):""}v(this.H,"click",this.sc);Kn.m.constructor.call(this);this.direction=Ln}r(Kn,md);var Mn={Ms:"goog-tablesorter-sorted",Vs:"goog-tablesorter-sorted-reverse"},Ln=3,Nn={Vy:"sort-asc",Wy:"sort-desc",Xy:"sort-no"};Kn.prototype.o=function(){return this.H};Kn.prototype.getName=function(){if(!this.name){var a=this.name=Sh(this.H,"col");this.name=a?String(a):""}return String(this.name)};
Kn.prototype.ie=function(){return this.direction};
Kn.prototype.sc=function(){if(dh(this.H,Mn.Ms)){this.direction=2;ah(this.H,"goog-tablesorter-sorted");Zg(this.H,"goog-tablesorter-sorted-reverse");var a=this.dispatchEvent(new Cc("sort-desc"))}else dh(this.H,Mn.Vs)?(this.direction=Ln,ah(this.H,"goog-tablesorter-sorted"),ah(this.H,"goog-tablesorter-sorted-reverse"),a=this.dispatchEvent(new Cc("sort-no"))):(this.direction=1,Zg(this.H,"goog-tablesorter-sorted"),ah(this.H,"goog-tablesorter-sorted-reverse"),a=this.dispatchEvent(new Cc("sort-asc")));return a};function On(a){this.l=a;this.Gf=q(this.Gf,this);On.m.constructor.call(this);this.table=null;this.headers=[]}r(On,md);On.prototype.M=function(a){var b;this.table=a;var c=Fg(this.l,"sortable",a);var d=[];a=0;for(b=c.length;a<b;a++){var e=c[a];e=new Kn(this.l,e);v(e,Hb(Nn),this.Gf);d.push(this.headers.push(e))}return d};On.prototype.Gf=function(a){return this.dispatchEvent(new Cc("sort",a.target))};function Pn(){Pn.m.constructor.call(this)}r(Pn,um);Pn.prototype.D=function(a){var b=Pn.m.D.call(this,a);var c=a.J();c.Sc(b,"");var d=c.H("span",{},b);c.Sc(d,a.ge());Zg(b,"btn");return b};Pn.prototype.wb=function(a){return"DIV"===a.tagName||"SPAN"===a.tagName?!0:Pn.m.wb.call(this,a)};ka(Pn);function Qn(){Qn.m.constructor.call(this);this.tag="a";this.Jd=["btn","btn-link"]}r(Qn,wh);Qn.prototype.D=function(a){var b=Qn.m.D.call(this,a);b.innerHTML="";b.href=a.href;b.title=a.Aq();null!=a.Kj()&&(b.target=a.Kj());var c=a.J().H("span",{});a.Cg?c.innerHTML=a.Cg:a.J().Sc(c,a.ge());b.appendChild(c);return b};ka(Qn);function Rn(a){null==a.S&&(a.S=Qn.va());Rn.m.constructor.call(this,a);this.href=a.href;this.caption=a.caption;this.Cg=a.Cg;this.title=a.title;this.target=a.target;null==this.href&&(this.href="");null==this.caption&&(this.caption="");null==this.Cg&&(this.Cg="");null==this.title&&(this.title=this.caption)}r(Rn,xh);Rn.prototype.Aq=function(){return this.title};Rn.prototype.Kj=function(){return this.target};Rn.prototype.wd=function(){return this.o()?this.l.wg(this.o()):""};function Sn(){}Sn.prototype.format=function(a,b){null==b&&(b=null);var c="";if(null!=b){var d=b.split("__");for(c=a;0<d.length;){var e=d.shift();c=null!=c[e]?c[e]:""}}if(la(c)){var g;var h=[];e=0;for(g=c.length;e<g;e++)d=c[e],h.push(Tn(d));c=h.join(", ")}else c=Tn(c);return c};function Tn(a){pa(a)&&(a=new el(a));return null!=a._verbose?a._verbose:String(a)};function Un(a,b,c){Un.m.constructor.call(this,a,b,c)}r(Un,Om);Un.prototype.Lf=function(a){var b=a.o();if(a=this.yf())return ei(b,a)};function Vn(){Vn.m.constructor.call(this);this.tag="tbody"}r(Vn,Un);Vn.prototype.D=function(a){return a.J().H(this.tag,this.Zc(a).join(" "))};ka(Vn);function Wn(a){null==a&&(a={});this.ad=q(this.ad,this);null==a.S&&(a.S=Vn.va());var b=a.orientation;var c=a.S;this.l=a.l;Wn.m.constructor.call(this,b,c,this.l);this.Xf=!0;this.ve(!1);this.selected=null}r(Wn,Tm);f=Wn.prototype;f.add=function(a){a=new Xn({l:this.l,caption:a});return this.$a(a)};f.$a=function(a,b){var c;null==b&&(b=!1);Wn.m.$a.call(this,a,b);var d=[32,8];var e=0;for(c=d.length;e<c;e++){var g=d[e];a.mc(g,!0)}};f.he=function(){var a=[];rh(this,function(b){return a.push(b)});return a};
f.D=function(){var a;if(!this.la&&(null!=this.S?this.F=a=this.S.D(this):Wn.m.D.call(this),this.Xf))return rh(this,function(a){return a.D()})};f.render=function(a){var b=this;null==a&&(a=null);return this.Xe(function(c){return null!=a?a.insertBefore(c,null):b.l.P.body.appendChild(c)})};f.Xe=function(a){var b=this;if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());this.Xf&&rh(this,function(a){return a.render(b.o())});this.Ea();return this.Qj()};
f.Qj=function(){return this.A("action",this.ad)};f.ad=function(a){a=a.target;this.selected&&this.setSelected(!1);this.selected=a;return this.setSelected(!0)};f.setSelected=function(a){return this.selected.setState(8,a)};f.destroy=function(){this.fb();return this.l.removeNode(this.o())};f.me=function(){this.Qb&&(this.Sf=!0)};f.po=function(a){var b=this;return rh(this,function(c){if(c.Ua.id===a.id)return c.setSelected(!0),b.A("action",function(){return c.setSelected(!1)})})};function Yn(a){null==a&&(a={});Yn.m.constructor.call(this,a.content,a.S,a.l);this.Xf=!0}r(Yn,zi);f=Yn.prototype;f.addClass=function(a){return Ci(this,a)};f.D=function(){var a;if(!this.la&&(null!=this.S?this.F=a=this.S.D(this):Yn.m.D.call(this),this.Xf))return rh(this,function(a){return a.D()})};f.render=function(a){var b=this;null==a&&(a=null);return this.Xe(function(c){return null!=a?a.insertBefore(c,null):b.l.P.body.appendChild(c)})};
f.Xe=function(a){var b=this;if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());this.Xf&&rh(this,function(a){return a.render(b.o())});return this.Ea()};f.destroy=function(){this.fb();return this.l.removeNode(this.o())};function Zn(){Zn.m.constructor.call(this);this.tag="div"}r(Zn,ni);Zn.prototype.D=function(a){var b=a.J().H(this.tag,this.Zc(a).join(" "),a.wd());ti(this,a,b);return b};ka(Zn);function $n(){$n.m.constructor.call(this);this.tag="tr"}r($n,Zn);ka($n);function ao(){ao.m.constructor.call(this);this.tag="td"}r(ao,wh);ao.prototype.D=function(a){var b=a.J();var c=b.H(this.tag,this.pd(a));b=b.H("span","cell-content");b.innerHTML=a.ge();c.appendChild(b);return c};ka(ao);function Xn(a){null==a.S&&(a.S=ao.va());Xn.m.constructor.call(this,a)}r(Xn,xh);function bo(a){null==a&&(a={});this.Ef=q(this.Ef,this);this.ad=q(this.ad,this);null==a.S&&(a.S=$n.va());bo.m.constructor.call(this,a);this.confirm=a.confirm;this.cells=[];this.Ze=null;Ci(this,co)}r(bo,Yn);var co="row-unselected",eo={SELECTED:"row-selected",hh:co};function fo(a){var b=new Xn({l:a.l,caption:""});a.$a(b);return b}f=bo.prototype;f.Wj=function(){return!0};f.Hk=function(a){this.Ze=a;return this.Ze.A(go,this.ad)};
f.ad=function(a){var b,c;var d=this.Fg()?[eo.SELECTED,eo.hh]:[eo.hh,eo.SELECTED];var e=[Zg,ah];var g=b=0;for(c=e.length;b<c;g=++b){var h=e[g];h(this.o(),d[g])}return this.dispatchEvent(new Cc("selection-change",a))};f.Fg=function(){return null!=this.Ze&&!!this.Ze.aa()};
f.Ef=function(a){var b=this;var c=a.target.Ua.action;var d=c.confirm;var e=a.target;return d?(this.confirm.vc(d.text),this.confirm.wi(d.title),ho(this.confirm,d.true_label,d.false_label),this.confirm.open(),this.confirm.focus(),cd(this.confirm,nm,function(a){if("yes"===a.key)return"DELETE"===c.method?b.dispatchEvent(new Cc("DeleteButton",e)):b.dispatchEvent(new Cc("RemoteButton",e))})):this.dispatchEvent(new Cc("RemoteButton",a.target))};
f.Nj=function(a){return this.dispatchEvent(new Cc("DeleteButton",a.target))};function io(a){null==a&&(a={});null==a.S&&(a.S=Pn.va());io.m.constructor.call(this,a.content,a.S,a.l);this.l=a.l}r(io,vm);function jo(a){this.handleError=q(this.handleError,this);jo.m.constructor.call(this,a);this.gb={}}r(jo,io);f=jo.prototype;f.call=function(a,b,c,d,e,g){var h=this;null==e&&(e=null);null==g&&(g=!1);this.pa(!1);b=this.Yl(b);return a.request(b,c,d,function(a){h.pa(!0);null!=e&&e(a);return h.dispatchEvent(new Cc("remote-button-success",a))},this.handleError,g)};f.handleError=function(a){this.pa(!0);return this.dispatchEvent(new Cc("remote-button-error",a))};f.D=function(){Ci(this,"remote-button");return jo.m.D.call(this)};
f.M=function(a){a.setAttribute("type","button");jo.m.M.call(this,a);var b;if(a=Sh(a,"fields")){var c=String(a).split(",");var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a=Eg(this.l,"remote-button-param-"+g);null!=a?d.push(this.gb[g]=a):d.push(void 0)}a=d}else a=void 0;return a};f.Yl=function(a){a=new xk(a);var b=this.gb;for(d in b){var c=b[d];Lk(a,d,Cm(c))}var d=[a.hd];a.Oc()&&d.push(a.Oc());return d.join("?")};function ko(a){ko.m.constructor.call(this,a)}r(ko,jo);ko.prototype.call=function(a,b,c,d,e){null==e&&(e=null);return ko.m.call.call(this,a,b,c,d,e,!0)};function lo(){};function mo(){mo.m.constructor.call(this);this.tag="ul";this.Jd.push("error-list")}r(mo,wh);ka(mo);function no(a){null==a&&(a={});null==a.S&&(a.S=mo.va());no.m.constructor.call(this,a);this.Vq=null}r(no,xh);no.prototype.ih=function(a){this.l.Ug(this.o());this.Vq=this.l.H("li",{},a);return this.o().appendChild(this.Vq)};function oo(a){null==a&&(a={});this.kc=q(this.kc,this);oo.m.constructor.call(this,a);this.name=a.name;this.required=a.required;this.size=a.size;this.caption=a.caption;this.Hd=a.Hd;this.lm=a.lm;this.placeholder=a.placeholder;pa(a.value)&&(a.value=rd(a.value));this.ra(a.value);null==this.required&&(this.required=!1);null==this.size&&(this.size=50);null==this.caption&&(this.caption="");null==this.Hd&&(this.Hd=new lo);null==this.lm&&(this.lm=new no({l:this.l}))}r(oo,xh);var go="change";f=oo.prototype;
f.$a=function(){};f.jh=function(){this.ra(this.value);return this.Qj()};f.ra=function(a){this.value=a;if(null!=this.o())return Dm(this.o(),a)};f.aa=function(){return this.la?Cm(this.o()):this.value};f.focus=function(){var a;return null!=(a=this.o())?a.focus():void 0};f.M=function(a){oo.m.M.call(this,a);return this.Qj()};f.Qj=function(){return v(this.o(),"change",this.kc)};f.kc=function(a){return this.dispatchEvent(new Cc(go,a))};function po(){po.m.constructor.call(this);this.tag="input"}r(po,wh);po.prototype.D=function(a){return a.J().D(this.tag,this.pd(a))};po.prototype.pd=function(a){var b={"class":this.Zc(a).join(" "),size:a.size,id:nh(a)};a.required&&(b.required="required");null!=a.placeholder&&(b.placeholder=a.placeholder);return b};ka(po);function qo(){qo.m.constructor.call(this)}r(qo,po);qo.prototype.pd=function(a){var b=qo.m.pd.call(this,a);b.type=a.type;return b};ka(qo);function ro(a){null==a&&(a={});null==(null!=a?a.S:void 0)&&(a.S=qo.va());ro.m.constructor.call(this,a);this.type=a.type;null==this.type&&(this.type="text")}r(ro,oo);f=ro.prototype;f.clear=function(){return this.o().value=""};f.render=function(a){ro.m.render.call(this,a);return this.Ih()};f.Wf=function(a){ro.m.Wf.call(this,a);return this.Ih()};f.Ih=function(){this.Yj(this.o())||(this.F=this.R.he(this.o())[0])};f.Yj=function(a){return"INPUT"===a.tagName};
f.hk=function(a){null==a&&(a=!0);return this.o().required=a?"required":void 0};function so(){so.m.constructor.call(this)}r(so,qo);so.prototype.pd=function(a){var b=so.m.pd.call(this,a);a.aa()&&(b.checked="checked");return b};ka(so);function to(a){null==a.S&&(a.S=so.va());a.type="checkbox";null==a.value&&(a.value=!!a.value);to.m.constructor.call(this,a)}r(to,ro);to.prototype.aa=function(){return!!to.m.aa.call(this)};to.prototype.ra=function(a){to.m.ra.call(this,!!a);return this.dispatchEvent(new Cc(go,this))};function uo(a,b,c,d,e,g){this.l=a;this.rows=b;this.cols=c;this.yq=d;this.confirm=e;this.Ze=null!=g?g:!1;this.Nl=q(this.Nl,this);this.Ml=q(this.Ml,this);uo.m.constructor.call(this);this.cp={rest:this.Ml,web:this.Nl}}r(uo,xh);f=uo.prototype;f.so=function(a){this.cols=a};
f.fc=function(a,b){var c;null==b&&(b=!0);var d=new bo({l:this.l,confirm:this.confirm});d.Ua=a;this.rows.$a(d,!1);if(this.Ze){var e=new Xn({l:this.l});var g=new to({l:this.l});e.$a(g);e.addClass("select-row");d.$a(e);d.Hk(g)}var h=this.cols;g=0;for(c=h.length;g<c;g++)e=h[g],vo(this,a,e,d);if(null!=a._class_names)for(h=a._class_names,g=0,c=h.length;g<c;g++)e=h[g],Ci(d,e);la(a._actions)&&0!=a._actions.length&&b&&wo(this,d,a);return d};
function vo(a,b,c,d){var e=new Xn({l:a.l});b._default_action?(a=new Rn({l:a.l,href:b._web_links[b._default_action],Cg:a.yq.format(b,c)}),e.$a(a)):e.Yr(a.yq.format(b,c));e.addClass(c);d.$a(e);return e}function wo(a,b,c){var d;var e=fo(b);e.addClass("actions");var g=c._actions;var h=0;for(d=g.length;h<d;h++){var k=g[h];var l=a;null!=l.cp[k.type]?l.cp[k.type](k,c,e,b):ne(l.l).console.warn("Non-existent action type: "+k.type)}b.$a(e);return e}
f.Ml=function(a,b,c,d){var e=new ko({content:a.verbose_name,l:this.l});b={fr:b,action:a};var g=a.verbose_name;Ci(e,this.um(a));e.Ua=b;e.Ik(g);c.$a(e);e.A("action",d.Ef);return e};f.Nl=function(a,b,c){b=new Rn({l:this.l,href:b._web_links[a.name],caption:a.verbose_name,target:a.target});b.addClass(this.um(a));return c.$a(b)};f.um=function(a){return a.class_name||a.name||""};function xo(a,b,c,d,e,g,h,k,l){this.l=a;this.ko=b;this.cols=c;this.us=d;this.confirm=e;this.query=g;this.kb=h;this.ka=k;this.Tr=null!=l?l:!1;this.Ym=q(this.Ym,this);this.Nj=q(this.Nj,this);this.Gf=q(this.Gf,this);this.$m=q(this.$m,this);this.Ef=q(this.Ef,this);this.Zm=q(this.Zm,this);this.Jm=q(this.Jm,this);this.refresh=q(this.refresh,this);this.Km=q(this.Km,this);xo.m.constructor.call(this);this.Go=this.tbody=this.table=null;this.rows=new Wn({l:this.l});this.Ak=null;this.es=!0;this.data=[]}
r(xo,xh);function yo(a){var b;return null!=(b=Yf(a.table))?b.querySelector(".paginator"):void 0}function zo(a,b){var c=null;null!=b?(c=a.kb,c.base=Km(b,"base",cb(c.Hl,Lm)),c.offset=(c.page-1)*c.base,c={offset:(a.kb.page-1)*a.kb.base}):a.kb=null;return c}f=xo.prototype;
f.M=function(a){var b,c=this;this.table=a;this.Ak=new uo(this.l,this.rows,this.cols,new Sn,this.confirm,this.Tr);this.us.A(Ao,this.refresh);Eg(this.l,"actions",this.table)||(this.es=!1);Bo(this);var d=yo(this);a=zo(this,d);Co(this,Do(this,a),function(a){c.Go=new On(c.l);c.Go.M(c.table);c.Go.A("sort",c.Gf);null!=c.kb&&(c.kb.init(a.total,a.count),Eo(c,d));c.dispatchEvent(new Cc("loaded",a));return c.A("delete-item",c.Km)});var e=[this.table,d];var g=0;for(b=e.length;g<b;g++)a=e[g],Am(a,function(a){return c.dispatchEvent(new Cc("action",
a))});g=Dg(this.l,"input.select-all",this.table);null!=g&&(a=new to({l:this.l}),a.M(g),a.A(go,this.Ym))};f.Oc=function(){return this.us.Oc()};f.Km=function(a){var b=this;return this.ko["delete"](a.target,function(){return Co(b,Do(b),function(a){if(null!=b.kb)return b.kb.refresh(a)})})};function Do(a,b){var c,d,e;null==b&&(b={});a.query.base=null!=(c=b.base)?c:null!=(d=a.kb)?d.base:void 0;a.query.offset=null!=b.offset?b.offset:null!=(e=a.kb)?e.offset:void 0;return a.query}
function Eo(a,b){null!=b?a.kb.M(b):a.kb.Wf(a.table);a.kb.A("go-to",function(b){return Co(a,Do(a,b.target),function(b){return a.kb.refresh(b)})});Fo(a)}f.refresh=function(){var a=this;this.query=this.Oc();return Co(this,Do(this,{offset:this.query.offset}),function(b){var c;b.offset=a.query.offset;return null!=(c=a.kb)?c.refresh(b):void 0})};function Fo(a){var b=a.kb;if(null==b.o())b=null;else{var c=b.clone();b.lj.push(c);b=c}a.l.Vj(b,a.table)}
function Co(a,b,c){null==c&&(c=null);null!=c&&(a.oq=c);return a.ko.load(b,a.Jm)}function Go(a){var b;if(a.rows.la){var c=a.rows;c.fb();Hg(c.o());c.Ug()}var d=a.data;c=0;for(b=d.length;c<b;c++){var e=d[c];e=a.Ak.fc(e,a.es);e.A("DeleteButton",a.Nj);e.A("RemoteButton",a.Ef);e.A("selection-change",a.$m)}a.rows.la||a.rows.render(a.table);a.rows.A("action",a.Zm)}f.Jm=function(a,b){this.data=a;Go(this);b.count=this.data.length;null!=this.oq&&this.oq(b);return this.data};
f.Zm=function(a){a.target.setState(8,!0);return this.dispatchEvent(new Cc("selected-item",a.target.Ua))};function Bo(a){var b=a.table.querySelector("tbody");null!=b&&a.l.removeNode(b)}
f.Ef=function(a){var b=this;a=a.target;var c=a.Ua.fr;var d=a.Ua.action;return a.call(this.ko.client,c._rest_links[d.name].url,d.method,d.data,function(a){b.ka.fw(d.success_text);var c=b.Ak,e;var k=uh(c.rows);var l=[];var m=0;for(e=k.length;m<e;m++){var p=k[m];p=qh(c.rows,p);if(p.Ua.id===a.id){var u=p;u.Ua=a;Hg(u.o());u.Ug();k=u.o();p=c.cols;e=0;for(m=p.length;e<m;e++){var A=p[e];vo(c,a,A,u).render(k)}la(a._actions)&&0!=a._actions.length?wo(c,u,a).render(k):fo(u).render();break}else l.push(void 0)}return l})};
f.$m=function(a){return this.dispatchEvent(a)};f.Gf=function(a){var b=this;a=a.target;this.query.sort(a.getName(),a.ie());return Co(this,Do(this),function(a){return b.kb.refresh(a)})};f.Nj=function(a){a=a.target;this.dispatchEvent(new Cc("delete-item",a.Ua.fr));return this.l.removeNode(a.Ua.bc)};f.bs=function(a){this.query=a};f.po=function(a){return this.rows.po(a)};f.so=function(a){this.cols=a};
f.vg=function(){var a;if(this.Tr){var b=this.rows.he();var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];e.Fg()&&c.push(e)}return c}return[]};f.Ym=function(a){var b;var c=this.rows.he();var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];null!=g.Ze?d.push(g.Ze.ra(a.currentTarget.aa())):d.push(void 0)}return d};function Ho(a){this.table=a;this.lo=this.cols=null}Ho.prototype.ug=function(a){var b;return String(null!=(b=Sh(this.table,a))?b:"")};function Io(a,b){var c=a.ug(b).split(",");1===c.length&&""===c[0]&&(c=[]);return c};function Jo(a){this.client=a}Jo.prototype.load=function(a,b){return this.client.find(a.Yl(),b,null,a)};Jo.prototype["delete"]=function(a,b){return this.client["delete"](a,b)};Re.prototype.Dg=function(a,b){this.We=a;this.gm=b;return this};function Ko(){Ko.m.constructor.call(this)}r(Ko,bm);var Lo={yes:"btn btn-primary",no:"btn btn-default",ok:"btn btn-primary",cancel:"btn btn-default"};Ko.prototype.render=function(){if(this.F){this.F.innerHTML="";var a=jf(this.F);var b=function(b,d){var c=a.D("button",{name:d,className:Lo[d]},b);d===this.pj&&Zg(c,this.hc+"-default");return this.F.appendChild(c)};return Ld(this,b,this)}};function Mo(a,b,c){Mo.m.constructor.call(this,a,b,c);a=new Ko;am(a,pm,!0);am(a,qm,!1,!0);mm(this,a)}r(Mo,$l);function ho(a,b,c){a=a.vb;null!=b&&a.set("yes",b);null!=c&&a.set("no",c)}f=Mo.prototype;f.vc=function(a){this.Ab=(new Re).Dg(a,null);null!=this.ac&&(this.ac.innerHTML=Te(this.Ab))};f.open=function(){return this.da(!0)};f.hide=function(){return this.da(!1)};f.da=function(a){Mo.m.da.call(this,a);ag(this.o(),"display",a?"block":"none");return w(this.pf,a)};
f.D=function(){this.F=this.R.H("div","modal fade in");var a=this.R.H("div","modal-dialog",this.F);this.pf=this.R.H("div","modal-content",a);Zg(this.pf,this.ta());Qf(this.pf,!0);w(this.pf,!1);Wl(this);Xl(this);this.yc.className="modal-backdrop fade in";this.cc=this.R.D("h4","modal-title",this.Di);a={className:"modal-header",id:nh(this)};this.Nb=this.R.D("div",a);this.Rj&&(a=new Ah({l:this.R}),a.render(this.Nb),this.xc=a.o());this.Nb.appendChild(this.cc);this.iw=this.Nb.id;this.ac=this.R.D("div","modal-body");
this.cb=this.R.D("div","modal-footer");this.vb&&(a=this.vb,a.F=this.cb,a.render());w(this.cb,!!this.vb);Bf(this.pf,this.Nb,this.ac,this.cb);ei(this.pf,this.je());fi(this.pf,"labelledby",this.iw||"");this.Ab&&this.ac&&(this.ac.innerHTML=Te(this.Ab));return fm(this,this.nh)};function No(a,b,c){No.m.constructor.call(this,a,b,c);this.confirm="Do you really want to delete %s?"}r(No,Mo);function Oo(){}Oo.prototype.ug=function(a,b){var c;return null!=(c=Sh(a,b))?c:""};function Po(a){this.grid=a;this.Rm=q(this.Rm,this);this.grid.A("loaded",this.Rm)}Po.prototype.M=function(a){return this.gr=a.querySelector(".table-empty")};Po.prototype.Rm=function(a){var b;a=0===a.target.count;null!=(b=this.grid.kb)&&b.show(!a);if(this.gr)return w(this.gr,a)};function Qo(a,b){this.l=a;this.H=b;this.Bm=q(this.Bm,this);this.kc=q(this.kc,this);Qo.m.constructor.call(this);ym(this.H,this.kc);this.$r()}r(Qo,md);f=Qo.prototype;f.kc=function(a){return this.dispatchEvent(new Cc("change",a))};function Ro(a,b,c){a.ra(c);return a.ui(b)}f.o=function(){return this.H};f.aa=function(){return Cm(this.H)};f.ra=function(a){return Dm(this.H,a)};f.ui=function(a){this.qe=a};f.reset=function(){return this.ra("")};f.getName=function(){return this.name};f.Je=function(){return String(this.qe)};
f.$r=function(){var a=So(this).split("__");this.ui(Fm(a,""));return this.name=this.Ie()};function So(a){return String(Sh(a.H,"filter"))}f.Ie=function(){this.filter||(this.filter=So(this));return this.filter};f.Gj=function(){return this.Bm().Gj()};f.Bm=function(){return new jl(this.getName(),this.Je(),this.aa(),!0)};f.apply=function(a){var b;var c=new jl(this.getName(),this.Je(),this.aa(),!0);(b=ol(a,c))&&a.filter(c);return b};f.Rq=function(a){return a===this.Ie()};function To(){To.m.constructor.call(this);this.tag="select"}r(To,po);To.prototype.D=function(a){var b;var c=To.m.D.call(this,a);var d=a.options;for(b in d){var e=d[b];c.appendChild(Uo(a.J(),b,e))}return c};To.prototype.pd=function(a){var b=To.m.pd.call(this,a);Mb(b,"type");a.multiple&&(b.multiple="multiple");return b};function Uo(a,b,c){b=a.D("option",{value:b});a.Sc(b,c);return b}ka(To);function Vo(a){null==a&&(a={});null==(null!=a?a.S:void 0)&&(a.S=To.va());null==a.size&&(a.size=1);Vo.m.constructor.call(this,a);this.multiple=a.multiple;this.options=a.options;null==this.multiple&&(this.multiple=!1);null==this.options&&(this.options={});a=this.options;if(null==this.value){a:{for(b in a)break a;var b=void 0}this.ra(b)}this.options=a}r(Vo,oo);Vo.prototype.ra=function(a){if(void 0!==a)return Vo.m.ra.call(this,String(a))};function Wo(a,b){Wo.m.constructor.call(this,a,b);this.options={gt:"\x3e",gte:"\x3e\x3d",eq:"\x3d",lte:"\x3c\x3d",lt:"\x3c"};this.select=new Vo({l:a,options:this.options});this.select.addClass(Xo);this.select.Wf(b);this.select.ra(String(this.qe));this.select.A(go,this.kc)}r(Wo,Qo);var Xo="filter-extended";f=Wo.prototype;f.mq="eq";f.Ie=function(){return this.name};f.Je=function(){return String(this.select.aa())};f.reset=function(){this.ra(String(this.qe));return this.ui(this.mq)};
f.ui=function(a){this.qe=kl(a)?this.mq:a;if(null!=this.select)return this.select.ra(String(this.qe))};f.$r=function(){var a=So(this).split("__");this.ui(Fm(a,""));return this.name=Em(a).join("__")};f.apply=function(a){var b;var c=new jl(this.getName(),String(this.select.aa()),this.aa(),!1);(b=ol(a,c))&&a.filter(c);return b};f.Rq=function(a){return Em(a.split("__")).join("__")===this.name};function Yo(a,b,c,d){this.grid=a;this.query=b;this.Kc=c;this.l=d;this.kc=q(this.kc,this);Yo.m.constructor.call(this);this.gb={};this.os={};this.Dh={}}r(Yo,md);f=Yo.prototype;
f.Xo=function(a){var b;var c=dh(a,"extended-filters");var d=this.l.all("thead *[data-filter]",a);var e=0;for(b=d.length;e<b;e++){var g=d[e];var h;if(h=c){var k=void 0;h=g;h=xm(h)||"number"===(k=h.type)||"date"===k||"datetime"===k}g=h?new Wo(this.l,g):new Qo(this.l,g);Jb(this.gb,g.Ie())?ne(this.l).console.warn("Table #"+a.id+" contains duplicated columns with name '"+g.Ie()+"'."):(this.gb[g.Ie()]=g,v(g,"change",this.kc))}Zo(this);return this.query};
function Zo(a){var b=$o(a.Kc),c;var d={};var e=0;for(c=b.length;e<c;e++){var g=b[e];var h=g.split("__");if(null!=h[h.length-2]){var k,l,m=h;h=[];var p=l=0;for(k=m.length-2;0<=k?l<=k:l>=k;p=0<=k?++l:--l)h.push(m[p]);h=h.join("__")}else h=h[0];d[g]={operator:g.replace(h,"").replace("__",""),value:a.Kc.get(g),name:g}}a.os=d;a.Dh=a.query.Dh;d=a.Dh;g=[];for(A in d)b=d[A],null!=a.gb[A]?g.push(a.gb[A].ra(b)):g.push(a.query.filter(new jl(A,"",b)));ap(a);var u,b=a.gb;d=[];for(u in b){var A=b[u];d.push(A.apply(a.query))}}
function ap(a){var b,c,d;var e=a.gb;var g=[];for(b in e){var h=e[b];g.push(function(){var a=this.os;var b=[];for(c in a)d=a[c],h.Rq(c)?b.push(Ro(h,d.operator,d.value)):b.push(void 0);return b}.call(a))}}f.kc=function(a){return this.filter(a.currentTarget)};f.filter=function(a){var b;if(a.apply(this.query)){var c=this.gb;for(b in c){var d=c[b];za(b,a.getName())&&d.apply(this.query)}this.query.offset=0;return this.qj()}};f.qj=function(){return this.dispatchEvent(new Cc("filter-changed",{}))};f.Oc=function(){return this.query};function bp(a,b,c,d,e){this.client=a;this.l=b;this.Xb=c;this.Kc=d;this.query=e;this.gk=q(this.gk,this);this.ad=q(this.ad,this);this.Sm=q(this.Sm,this)}var cp={$s:"webLink"};bp.prototype.Xo=function(a){a.A("selected-item",this.ad);a.A("loaded",this.Sm);a=(a=this.Kc.get("i"))?this.client.get([this.query.uri.hd,a].join("/"),this.gk):void 0;return a};bp.prototype.Sm=function(a){if(this.jk)return a.currentTarget.po(this.jk)};bp.prototype.ad=function(a){return this.gk(a.target)};
bp.prototype.gk=function(a){var b;this.jk=a;this.Kc.set("i",this.jk.id);var c=this.l.all(".snippet[data-web-link]");var d=[];a=0;for(b=c.length;a<b;a++){var e=c[a];var g=String(Sh(e,cp.$s)).split(".");e=g[0];g=null!=g[1]?g[1]:null;e=this.jk._web_links[e];null!=g&&(e=e[g]);null!=e?d.push(this.client.request(e,"GET")):d.push(void 0)}return d};function dp(){Cc.call(this,"navigate")}r(dp,Cc);function ep(){return!(Cb("iPad")||Cb("Android")&&!Cb("Mobile")||Cb("Silk"))&&(Cb("iPod")||Cb("iPhone")||Cb("Android")||Cb("IEMobile"))};function fp(a,b){for(var c=[a],d=b.length-1;0<=d;--d)c.push(typeof b[d],b[d]);return c.join("\x0B")};function gp(a,b,c,d){md.call(this);if(a&&!b)throw Error("Can't use invisible history without providing a blank page.");if(c)var e=c;else{e="history_state"+hp;var g=$e("input",{type:"text",name:e,id:e,style:Be("display:none")});document.write(Te(g));e=mf(document,e)}this.Rh=e;this.ze=c?wf(lf(c)):window;this.jn=b;t&&!b&&(this.jn="https"==window.location.protocol?Ke(Ae(Be("https:///"))):Ke(Ae(Be('javascript:""'))));this.ua=new nd(ip);$b(this,va(ac,this.ua));this.bg=!a;this.Rb=new fh(this);if(a||jp){if(d)var h=
d;else{a="history_iframe"+hp;d=this.jn;b={id:a,style:Be("display:none"),sandbox:void 0};d&&Je(d);c={};c.src=d||null;c.srcdoc=null;d={sandbox:""};e={};for(h in c)e[h]=c[h];for(h in d)e[h]=d[h];for(h in b){g=h.toLowerCase();if(g in c)throw Error('Cannot override "'+g+'" attribute, got "'+h+'" with value "'+b[h]+'"');g in d&&delete e[g];e[h]=b[h]}h=af("iframe",e,void 0);document.write(Te(h));h=mf(document,a)}this.Ta=h;this.Bs=!0}jp&&(this.Rb.A(this.ze,"load",this.vv),this.ds=this.im=!1);this.bg?kp(this,
this.zf(),!0):lp(this,this.Rh.value);hp++}r(gp,md);gp.prototype.Qb=!1;gp.prototype.Lg=!1;gp.prototype.Jg=null;var mp=function(a,b){var c=b||fp;return function(){var b=this||ba,b=b.closure_memoize_cache_||(b.closure_memoize_cache_={}),e=c(qa(a),arguments);return b.hasOwnProperty(e)?b[e]:b[e]=a.apply(this,arguments)}}(function(){return t?vc(8):"onhashchange"in ba}),jp=t&&!vc(8);f=gp.prototype;f.Kg=null;f.V=function(){gp.m.V.call(this);this.Rb.eb();this.pa(!1)};
f.pa=function(a){if(a!=this.Qb)if(jp&&!this.im)this.ds=a;else if(a)if(gc?this.Rb.A(this.ze.document,np,this.Fv):jc&&this.Rb.A(this.ze,"pageshow",this.Cv),mp()&&this.bg)this.Rb.A(this.ze,"hashchange",this.wv),this.Qb=!0,this.dispatchEvent(new dp(this.zf()));else{if(!t||ep()||this.im)this.Rb.A(this.ua,"tick",q(this.Np,this,!0)),this.Qb=!0,jp||(this.Jg=this.zf(),this.dispatchEvent(new dp(this.zf()))),this.ua.start()}else this.Qb=!1,this.Rb.ld(),this.ua.stop()};
f.vv=function(){this.im=!0;this.Rh.value&&lp(this,this.Rh.value,!0);this.pa(this.ds)};f.Cv=function(a){a.Wc.persisted&&(this.pa(!1),this.pa(!0))};f.wv=function(){var a=op(this.ze);a!=this.Jg&&pp(this,a)};f.zf=function(){return null!=this.Kg?this.Kg:this.bg?op(this.ze):qp(this)||""};f.zo=function(a,b){this.zf()!=a&&(this.bg?(kp(this,a,!1),mp()||t&&!ep()&&lp(this,a,!1,b),this.Qb&&this.Np()):(lp(this,a,!1),this.Kg=this.Jg=this.Rh.value=a,this.dispatchEvent(new dp)))};
function op(a){a=a.location.href;var b=a.indexOf("#");return 0>b?"":a.substring(b+1)}function kp(a,b,c){a=a.ze.location;var d=a.href.split("#")[0],e=Qa(a.href,"#");if(jp||e||b)d+="#"+b;d!=a.href&&(c?a.replace(d):a.href=d)}
function lp(a,b,c,d){if(a.Bs||b!=qp(a))if(a.Bs=!1,b=encodeURIComponent(String(b)),t){var e=Jf(a.Ta);e.open("text/html",c?"replace":void 0);c=bf($e("title",{},d||a.ze.document.title),$e("body",{},b));e.write(Te(c));e.close()}else if(e=Je(a.jn)+"#"+b,a=a.Ta.contentWindow)c?a.location.replace(e):a.location.href=e}
function qp(a){if(t)return a=Jf(a.Ta),a.body?Fa(a.body.innerHTML):null;var b=a.Ta.contentWindow;if(b){try{var c=Fa(op(b))}catch(d){return a.Lg||(1!=a.Lg&&a.ua.setInterval(rp),a.Lg=!0),null}a.Lg&&(0!=a.Lg&&a.ua.setInterval(ip),a.Lg=!1);return c||null}return null}f.Np=function(){if(this.bg){var a=op(this.ze);a!=this.Jg&&pp(this,a)}if(!this.bg||jp)if(a=qp(this)||"",null==this.Kg||a==this.Kg)this.Kg=null,a!=this.Jg&&pp(this,a)};
function pp(a,b){a.Jg=a.Rh.value=b;a.bg?(jp&&lp(a,b),kp(a,b)):lp(a,b);a.dispatchEvent(new dp(a.zf()))}f.Fv=function(){this.ua.stop();this.ua.start()};var np=["mousedown","keydown","mousemove"],hp=0,ip=150,rp=1E4;function sp(a,b){var c;for(c in b){var d=b[c];Jb(a,c)||(a[c]=d)}return a}function tp(a,b){var c,d;var e={};if(null!=a.length){var g=c=0;for(d=a.length;c<d;g=++c){var h=a[g];g=b(h,g);h=g[0];var k=g[1];e[h]=k}}else for(g in a)h=a[g],c=b(h,g),h=c[0],k=c[1],e[h]=k;return e}function up(a,b,c){a=a||{};if(0==b.length)b=a;else{if(1===b.length)var d=a||{};else d=a,c=up(Ob(a,b[0]),ub(b,1),c),d=d||{};d[b[0]]=c;b=d}return b}
function vp(a,b,c){null==c&&(c=null);a=a||{};return 0==b.length?a:1===b.length?Ob(a,b[0],c):vp(Ob(a,b[0],c),ub(b,1),c)};function wp(a,b){this.Kc=a;this.Tq=null!=b?b:"";this.Yk=q(this.Yk,this);this.Nm=q(this.Nm,this);this.Mm=q(this.Mm,this);this.So=q(this.So,this);wp.m.constructor.call(this);this.history=new gp;this.history.pa(!0);this.base=xp;this.ll="grid"}r(wp,md);var yp={Hi:"b",Ki:"p"},xp=0,zp={Hi:xp,Ki:1},Ao="state-changed";f=wp.prototype;f.ti=function(a){this.base=a};f.Am=function(){return Lm(this.Kc.get("p"),1)};
f.Dj=function(){var a=Nm(ba.localStorage.getItem(this.ll))||{};a=Lm(vp(a,[this.Tq,"b"]));return 0<a?a:this.base};f.handle=function(a,b){this.kb=a;this.ye=b;this.kb.A("go-to",this.Nm);this.history.A("navigate",this.So);return this.ye.A("filter-changed",this.Mm)};f.So=function(){Zo(this.ye);return this.qj()};f.qj=function(){return this.dispatchEvent(new Cc(Ao,{}))};f.Mm=function(){this.kb.vi(1);return this.Yk({page:1})};f.Nm=function(a){return this.Yk(a.target)};
f.Yk=function(a){if(null!=a){if(null!=a.base){var b={};b[yp.Hi]=a.base;var c=Nm(ba.localStorage.getItem(this.ll))||{};for(g in b){var d=b[g];up(c,[this.Tq,g],d)}ba.localStorage.setItem(this.ll,rd(c))}b={};b[yp.Ki]=a.page!==zp.Ki?a.page:null;c=this.ye;a={};var e=c.gb;for(k in e){var g=e[k];var h=d=g.aa();(null!=c.Dh[g.getName()]&&""===h||""!==h)&&(a[g.Gj()]=d)}Qb(a,b);this.Kc.ii();var k=this.Kc;b=!0;var l;null==b&&(b=!1);for(l in a)c=a[l],Ap(k.Db,k.key(l),c,b);return k.Yq.hash=k.Db.toString()}};
f.Oc=function(){return this.ye.Oc()};function Bp(a,b,c,d){this.grid=a;this.client=b;this.url=c;this.interval=null!=d?d:3E3;this.update=q(this.update,this);this.fetch=q(this.fetch,this);this.timer=new nd(this.interval);this.timer.A("tick",this.fetch)}Bp.prototype.start=function(){if(1<this.interval)return this.timer.start()};Bp.prototype.fetch=function(){this.timer.stop();return this.client.find(this.url,this.update)};Bp.prototype.update=function(a){0<a["count-changed"]&&this.grid.refresh();return this.timer.start()};function Cp(a){this.Le=q(this.Le,this);Cp.m.constructor.call(this,a);this.ye=a.ye;this.client=a.client;this.ext=this.type=null}r(Cp,Rn);Cp.prototype.Ga=function(a){var b;Cp.m.Ga.call(this,a);v(a,"click",this.Le);this.Ar=new xk(this.o().href);this.mn=this.Ar.mb;var c=this.ye.grid.cols;var d=0;for(b=c.length;d<b;d++)a=c[d],this.mn.remove(a)};Cp.prototype.getExtension=function(){var a;this.ext||(this.ext=null!=(a=Sh(this.o(),"type"))?a:"",this.ext=String(this.ext));return this.ext};
Cp.prototype.Le=function(){var a=this.o(),b;var c=this.ye.Oc().uri.clone();Bk(c,this.Ar.hd);var d=this.mn.rc();var e=0;for(b=d.length;e<b;e++){var g=d[e];null==c.mb.get(g)&&Lk(c,g,this.mn.Gb(g))}return a.href=c.toString()};function Dp(a){this.Kn=a;this.get=q(this.get,this);this.set=q(this.set,this)}Dp.prototype.set=function(a,b){return/string|number|boolean/.test(typeof b)?this.Kn.setItem(a,b):this.Kn.setItem(a,rd(b))};Dp.prototype.get=function(a){var b;a=this.Kn.getItem(a);if(null===a)return a;if("string"===typeof a||a instanceof String)if("["===(b=a.trim().substring(0,1))||"{"===b)try{a=qd(a)}catch(c){}return a};function Ep(a,b,c){this.l=a;this.Hh=b;this.grid=c;this.toArray=q(this.toArray,this);this.show=q(this.show,this);this.dm=q(this.dm,this);this.Ao=q(this.Ao,this);this.Ro=q(this.Ro,this);this.Mh=q(this.Mh,this);this.Bk=q(this.Bk,this);this.vo=q(this.vo,this);this.om=q(this.om,this);this.kc=q(this.kc,this);this.Cm=q(this.Cm,this);this.ym=q(this.ym,this);this.zm=q(this.zm,this);this.ii=q(this.ii,this);this.render=q(this.render,this);this.Om=q(this.Om,this);this.M=q(this.M,this);Ep.m.constructor.call(this);
this.headers=[];this.cols={};this.zj={};this.Op=[];this.Wo=[];this.Oi=this.grid.cols;this.Io=["columns_storage",this.grid.table.id].join("-");this.tq="";this.He=null;this.cf=new Dp(ne(this.l).localStorage)}r(Ep,xh);f=Ep.prototype;
f.M=function(a){this.H=a;if(this.grid.table.id||""===this.grid.table.id){Ep.m.M.call(this,this.H);this.headers=this.l.all("thead \x3e tr \x3e th",this.grid.table);this.Wo=this.Cm();this.cols=this.zm();this.zj=this.ym();if(this.He=this.l.o(["columns-filtering-text",this.grid.table.id].join("-")))return this.tq=this.He.innerHTML,this.l.hide(this.He),v(this.He,"click",this.dm),v(this.grid,"loaded",this.Om),v(this.Hh,"filter-changed",this.om),this.render();ne(this.l).console.warn("Missing filtering text element for #"+
this.H.id)}else ne(this.l).console.warn("Missing table id attribute for "+this.grid.table.className)};f.Om=function(){return this.show(this.Mh())};function Fp(a,b,c){var d=a.l.H("li","managed-col");var e=[a.grid.table.id,"manage-",b].join("");var g=a.l.H("input",{type:"checkbox",name:b,checked:null!=a.cols[b]?a.cols[b]:!0,id:e},d);b=a.l.H("label",{"class":a.zj[b]?"is-filtered":"","for":e},a.Wo[b]);a.l.append(d,b);v(g,"click",a.kc);a.l.append(c,d);return g}
f.render=function(){var a=this.l.H("ul","managed-columns");var b,c;var d=this.Oi;var e=[];var g=b=0;for(c=d.length;b<c;g=++b)g=d[g],e.push(Fp(this,g,a));this.Op=e;this.Bk(this.Mh());this.ii();return this.l.append(this.H,a)};f.ii=function(){return this.l.removeNode(Eg(this.l,"managed-columns",this.H))};f.zm=function(){var a;var b=this.cf.get(this.Io);if(!b){b={};var c=this.Oi;var d=0;for(a=c.length;d<a;d++){var e=c[d];b[e]=!0}this.cf.set(this.Io,b)}return b};
f.ym=function(){return tp(this.Hh.gb,function(a){return[a.getName(),""!==a.aa()]})};f.Cm=function(){var a=this;return tp(this.l.all("th \x3e span",this.grid.table),function(b,c){return[a.Oi[c],a.l.wg(b)]})};f.kc=function(){this.cols=this.Mh();this.show(this.cols);return this.Bk(this.cols)};f.om=function(){this.vo(this.Hh.Oc());return this.render()};f.vo=function(a){var b;var c=this.zj;var d=[];for(b in c)d.push(this.zj[b]=null!=a.filters[b]);return d};
f.Bk=function(a){this.Ro(a);return this.cf.set(this.Io,a)};f.Mh=function(){return tp(this.Op,function(a){return[a.name,null!=a.checked&&a.checked]})};f.Ro=function(a){var b;var c=[];for(b in a){var d=a[b];!d&&this.Hh.Oc().filters.ae(b)&&c.push(this.Wo[b].toLowerCase())}return 0<c.length?(this.He.innerHTML=this.tq.replace("%(columns)s",c.join(", ")),this.l.show(this.He)):this.l.hide(this.He)};
f.Ao=function(){var a;var b=this.headers;var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];null!=this.cols[e.className]?c.push(w(e,this.cols[e.className])):c.push(void 0)}return c};f.dm=function(){var a=this.Hh,b;var c=a.gb;var d=[];for(b in c){var e=c[b];e.reset();d.push(e.apply(a.query))}return a.qj()};f.show=function(a){var b=this.grid;a=this.toArray(a);b.so(a);b.Ak.so(a);Go(b);return this.Ao()};
f.toArray=function(a){var b;var c=this.Oi;var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a[g]&&d.push(g)}return d};function Gp(a,b,c,d,e){var g=this;this.l=a;this.client=b;this.url=c;this.bf=d;this.register=e;this.Qh=q(this.Qh,this);Gp.m.constructor.call(this);this.sd=new Mo(void 0,void 0,this.l);mm(this.sd,null);Am(em(this.sd),function(a){return g.dispatchEvent(new Cc("action",a))})}r(Gp,md);Gp.prototype.open=function(){return vl(this.client,this.url,this.Qh)};Gp.prototype.Qh=function(a){if(a.snippets[this.bf])return this.sd.vc(a.snippets[this.bf]),this.eo(a)};
Gp.prototype.eo=function(){var a=this.sd.Eb();this.register.tc(a);this.sd.da(!0);Zg(this.sd.o(),this.bf);return this.dispatchEvent(new Cc("open",this.sd))};Gp.prototype.wi=function(a){return this.sd.wi(a)};var Hp={};function Ip(a,b){a.removeAttribute("disabled");var c=Eg(b,"btn-spinner",a);if(null!=c)return b.removeNode(c)}function Jp(a,b){var c;var d=b.all(".form-btns button.btn.disable-on-submit",a);var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];e.push(Ip(h,b))}return e}
function Kp(a,b){var c;if(!dh(a,Hp.uw)){var d=b.all(".form-btns button.btn.disable-on-submit",a);var e=function(a){var c=a.currentTarget;Eg(b,"btn-spinner",c)||b.appendChild(c,b.H("i","fa fa-refresh fa-spin btn-spinner"));if(null!=a.target.name)return b.appendChild(a.target.parentNode,b.H("input",{type:"hidden",name:a.target.name,value:a.target.value}))};var g=0;for(c=d.length;g<c;g++){var h=d[g];Am(h,e)}v(a,"submit",function(){var a;var b=[];var c=0;for(a=d.length;c<a;c++)h=d[c],b.push(h.setAttribute("disabled",
!0));return b})}};function Lp(a,b,c,d,e){this.l=a;this.client=b;this.grid=c;this.Xb=d;this.ka=e;this.Xm=q(this.Xm,this);this.handleSelect=q(this.handleSelect,this);this.Hb=q(this.Hb,this);this.handleError=q(this.handleError,this);this.hm=q(this.hm,this);this.tk=q(this.tk,this);this.Wl=q(this.Wl,this);this.Bf=q(this.Bf,this);this.bn=q(this.bn,this);this.sc=q(this.sc,this);this.zc=new io({l:this.l});this.nm={};this.pi={};this.ri=[];this.gb=this.form=null}
var Mp={SELECTED:"field-selected",hh:"field-unselected"},Np={Li:"formSnippet",Ls:"apiUrl",Ys:"formUrl"};f=Lp.prototype;f.M=function(a){this.H=a;a=[Np.Li,Np.Ls,Np.Ys];var b,c;var d=[];var e=0;for(c=a.length;e<c;e++){var g=a[e];if(b=Km(this.H,g))d.push(String(b));else throw Error("Missing `data-"+g+"` for BulkChange");}this.xq=d[0];this.Qt=d[1];this.Du=d[2];this.zc.M(this.H);this.zc.A("action",this.sc);this.zc.pa(!1);return v(this.grid,"selection-change",this.Xm)};
f.sc=function(){if(this.zc.isEnabled()){var a=new Gp(this.l,this.client,[this.Du,"?snippet\x3d",this.xq].join(""),this.xq,this.Xb);a.A("open",this.Hb);return a.open()}};f.bn=function(){var a,b;var c={};var d=this.pi;for(a in d){var e=d[a];c[a]=Cm(e)}return this.client.request([this.Qt,"?id__in\x3d",rd(function(){var a;var c=this.ri;var d=[];var e=0;for(a=c.length;e<a;e++)b=c[e],d.push(b.Ua.id);return d}.call(this))].join(""),"PUT",c,this.Bf,this.handleError)};f.Bf=function(){return ne(this.l).location.reload(!0)};
f.Wl=function(a){var b;a=Hm(null!=(b=a.messages)?b.errors:void 0,function(a){return a.id});return Jm(a)?null:wm(Km(this.H,"bulkErrorMessage",String,"Please fix errors for following objects: %s"),a.join(", "))};f.tk=function(a){a=Eg(this.l,"errorlist",a);if(null!=a)return this.l.removeNode(a)};f.hm=function(a){var b;var c=this.nm;var d=[];for(b in c){var e=c[b];Jb(a,b)?(ji(e,"error"),this.tk(e),d.push(this.l.appendChild(e,this.l.H("ul","errorlist",[this.l.H("li",null,a[b])])))):(li(e,"error"),d.push(this.tk(e)))}return d};
f.handleError=function(a){var b=this.Wl(a);null!=b&&Mh(this.ka,b);this.hm(Im(a.messages.errors,function(a,b){return sp(a,b.errors)}));if(null!=this.form)return Jp(this.form,this.l)};
f.Hb=function(a){var b;this.pi={};this.ri=this.grid.vg();a=a.target.Eb();var c=Eg(this.l,"btn-save",a);null!=c&&(this.Qk=new io({l:this.l}),this.Qk.M(c),this.Qk.pa(!1),this.Qk.A("action",this.bn));this.form=Dg(this.l,"form",a);null!=this.form&&v(this.form,"submit",function(a){return a.preventDefault()});c=Eg(this.l,"bulk-change-object-summary",a);null!=c&&this.l.Sc(c,wm(this.l.wg(c),this.ri.length));var d=Fg(this.l,"field",a);var e=[];var g=0;for(b=d.length;g<b;g++)a=d[g],c=Op(Pp(this,a)),Qp(this,
a,Op(Pp(this,a))),e.push(this.nm[c]=a);return e};function Qp(a,b,c){Zg(b,"field-unselected");var d=new to({l:a.l});d.render(b);Rh(d.o(),"target",c);d.A(go,a.handleSelect)}f.Sk=function(a,b){var c,d;var e=b?[Mp.SELECTED,Mp.hh]:[Mp.hh,Mp.SELECTED];var g=[Zg,ah];var h=[];var k=c=0;for(d=g.length;c<d;k=++c){var l=g[k];h.push(l(a,e[k]))}return h};
f.handleSelect=function(a){var b=this.nm[Km(a.target.target,"target")];if(null==b)ne(this.l).console.warn("Given an invalid checkbox ID "+nh(a.target)+" for BulkChange");else{a=a.target.target.checked;this.Sk(b,a);var c=Pp(this,b);if(null==c)ne(this.l).console.warn("Missing an input in field ."+b.className);else return a?this.pi[Op(c)]=c:Mb(this.pi,Op(c)),this.Qk.pa(a||Gb(this.pi))}};function Pp(a,b){return Dg(a.l,"input, select, textarea",b)}
function Op(a){return Fm(a.name.split("-").slice(-1))}f.Xm=function(){this.ri=this.grid.vg();return this.zc.pa(0<this.ri.length)};function Rp(){var a;null==a&&(a={});return this.rq=a.rq,a}function Sp(a){var b=Lm(String(Sh(a.table,"interval")),0);1<b&&(b=new nd(b),b.A("tick",function(){return a.refresh()}),b.start())};function Tp(a,b){this.client=a;this.l=b;this.gd=q(this.gd,this);Tp.m.constructor.call(this);this.Ti=[];this.Zi=!1}r(Tp,md);f=Tp.prototype;f.M=function(a){var b,c;if(null!=a&&(b=this.l.all(".btn-save",a))){this.Ti=[];var d=0;for(c=b.length;d<c;d++){var e=b[d];var g=new vm;g.M(e);this.Ti.push(g)}this.zg(a);return Up(a)}};function Up(a){return v(a,"submit",function(a){return a.preventDefault()})}
f.zg=function(a){var b,c=this;var d=this.Ti;var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(h.A("action",function(b){h=b.target;h.pa(!1);b=h.o().getAttribute("name");var d=h.o().getAttribute("value");null!=b&&(b=c.l.D("input",{type:"hidden",name:b,value:d}),c.l.appendChild(a,b));return c.send(a)}))}return e};f.send=function(a){this.form=a};
f.gd=function(){var a;if(this.Zi){var b=this.l.all("input[type\x3dtext], textarea",this.form);var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];c.push(Dm(e,""))}return c}};f.di=function(){};function Vp(a){var b;var c=a.Ti;var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.pa(!0))};function Wp(a,b){this.gd=q(this.gd,this);this.di=q(this.di,this);Wp.m.constructor.call(this,a,b)}r(Wp,Tp);Wp.prototype.M=function(a){Wp.m.M.call(this,a);return this.url=a.action};
Wp.prototype.send=function(a){Wp.m.send.call(this,a);var b=this.client;var c=this.gd,d=this.di;b:{var e;var g=a.querySelectorAll("input[type\x3dfile]");var h=0;for(e=g.length;h<e;h++){var k=g[h];if(Cm(k)){k=!0;break b}}k=!1}if(k)b=yl(b,a,c,d);else{k=a.action;h=[];var l=a.elements,m;for(g=0;m=l[g];g++)if(m.form==a&&!m.disabled&&"FIELDSET"!=m.tagName)switch(e=m.name,m.type.toLowerCase()){case "file":case "submit":case "reset":case "button":break;case "select-multiple":m=Cm(m);if(null!=m)for(var p,u=
0;p=m[u];u++)Bm(h,e,p);break;default:p=Cm(m),null!=p&&Bm(h,e,p)}l=a.getElementsByTagName("INPUT");for(g=0;m=l[g];g++)m.form==a&&"image"==m.type.toLowerCase()&&(e=m.name,Bm(h,e,m.value),Bm(h,e+".x","0"),Bm(h,e+".y","0"));a=h.join("\x26");b=wl(b,k,a,c,d)}return b};Wp.prototype.di=function(a){Wp.m.di.call(this,a);if(null!=a)return Vp(this),a=this.l.Iq(a).querySelector("form"),this.M(a),this.dispatchEvent(new Cc("error",a))};
Wp.prototype.gd=function(a){Wp.m.gd.call(this,a);Vp(this);return this.dispatchEvent(new Cc("saved",a))};function Xp(a,b,c,d,e){this.Ff=q(this.Ff,this);Xp.m.constructor.call(this,a,b,c,d,e);this.Dl=new Wp(this.client,this.l);this.Dl.A("saved",this.Ff);this.Dl.A("error",this.Qh)}r(Xp,Gp);Xp.prototype.eo=function(a){var b=this.sd.Eb();b=Dg(this.l,"form",b);dh(b,"ajax")&&ah(b,"ajax");this.Dl.M(b);return Xp.m.eo.call(this,a)};Xp.prototype.Ff=function(a){if(a.target&&a.target.snippets)return this.Qh(a.target);this.sd.da(!1);return this.dispatchEvent(new Cc("success-close",a.target))};function Yp(a,b){this.l=a;this.Yb=b;this.mm=q(this.mm,this);this.gd=q(this.gd,this);this.Le=q(this.Le,this);this.zc=new jo({l:a});this.client=new pl(this.Yb);this.method=this.url="";this.Vk=this.Ae=this.Er=this.$n=this.data=null}var Zp={pt:"postAction",qt:"postElement",at:"method",URL:"url",Ts:"data",Ct:"state"},$p={reload:function(a){return ne(a.l).location.reload(!0)},remove:function(a){a.Vk=a.l.o(a.Er);a=null!=a.Vk?ne(a.l).setTimeout(a.mm(),1E3):void 0;return a}};
Yp.prototype.M=function(a){this.Ae=a;this.zc.M(this.Ae);var b;this.url=String(Sh(this.Ae,Zp.URL));this.method=String(Sh(this.Ae,Zp.at));this.data=null!=(b=Sh(this.Ae,Zp.Ts))?b:{};this.$n=String(Sh(this.Ae,Zp.pt));this.Er=String(Sh(this.Ae,Zp.qt));this.state=Sh(this.Ae,Zp.Ct);this.zc.A("action",this.Le);a=this.state&&ne(this.l).location.hash==="#"+this.state?this.Le():void 0;return a};
Yp.prototype.Le=function(){this.state&&(ne(this.l).location.hash=this.state);return this.zc.call(this.client,this.url,this.method,this.data,this.gd)};Yp.prototype.gd=function(){if(Jb($p,this.$n))return $p[this.$n](this)};Yp.prototype.mm=function(){var a=this;var b=new Wg(this.Vk,1E3);b.A("end",function(){a.l.removeNode(a.Vk)});return b.play()};function aq(a){var b=0,c=0;if(bq(a))b=a.selectionStart,c=-1;else if(t&&!uc("9")){var d=cq(a),e=d[0],d=d[1];if(e.inRange(d)){e.setEndPoint("EndToStart",d);if("textarea"==a.type){d.duplicate();b=a=e.text;for(c=!1;!c;)0==e.compareEndPoints("StartToEnd",e)?c=!0:(e.moveEnd("character",-1),e.text==a?b+="\r\n":c=!0);e=[b.length,-1];return e}b=e.text.length;c=-1}}return[b,c]}
function cq(a){var b=a.ownerDocument||a.document,c=b.selection.createRange();"textarea"==a.type?(b=b.body.createTextRange(),b.moveToElementText(a)):b=a.createTextRange();return[b,c]}function dq(a,b){"textarea"==a.type&&(b=a.value.substring(0,b).replace(/(\r\n|\r|\n)/g,"\n").length);return b}function bq(a){try{return"number"==typeof a.selectionStart}catch(b){return!1}};function eq(a,b,c,d){Xb.call(this);d=d||150;this.Og=null!=c?c:!0;this.si=a||",;";this.uu=this.si.substring(0,1);a=this.Og?"[\\s"+this.si+"]+":"[\\s]+";this.As=new RegExp("^"+a+"|"+a+"$","g");this.Xv=new RegExp("\\s*["+this.si+"]$");this.Xq=b||"";this.Nv=!1;this.Mv=this.Og;this.ua=0<d?new nd(d):null;this.Bb=new fh(this);this.Ni=new fh(this);this.Vb=new Yh;this.Uq=-1}r(eq,Xb);var fq=(Nh||Oh)&&!uc("533.17.9");f=eq.prototype;f.pw=!0;f.Eu=!0;f.Cs=!1;f.Zv=!0;f.Yv=!0;f.Bl=null;f.Ca=null;f.En="";f.ff=!1;
f.mo=!1;f.jw=!0;f.sm=function(){return this.Ca};f.aa=function(){return this.Ca.value};f.ra=function(a){this.Ca.value=a};function gq(a,b){Hf(b)&&(ei(b,"combobox"),fi(b,"autocomplete","list"));a.Bb.A(b,"focus",a.handleFocus);a.Bb.A(b,"blur",a.handleBlur);a.Ca||(a.Ni.A(b,"keydown",a.or),Hf(b)&&Uf(lf(b))==b&&hq(a,b))}
function iq(a,b){if(a.Ca){var c=b.toString();if(ca(void 0)?0:a.Og){var d=jq(a,a.aa(),aq(a.Ca)[0]),e=kq(a,a.aa());a.Xv.test(c)||(c=c.replace(/[\s\xa0]+$/,"")+a.uu);a.pw&&(0==d||Ba(e[d-1])||(c=" "+c),d==e.length-1&&(c+=" "));if(c!=e[d]){e[d]=c;c=a.Ca;(jc||t&&uc("9"))&&c.blur();c.value=e.join("");for(var g=0,h=0;h<=d;h++)g+=e[h].length;c.focus();d=g;e=a.Ca;c=d;bq(e)?e.selectionStart=c:t&&!uc("9")&&(g=cq(e),h=g[0],h.inRange(g[1])&&(c=dq(e,c),h.collapse(!0),h.move("character",c),h.select()));e=a.Ca;bq(e)?
e.selectionEnd=d:t&&!uc("9")&&(g=cq(e),c=g[1],g[0].inRange(c)&&(d=dq(e,d),e=dq(e,aq(e)[0]),c.collapse(!0),c.moveEnd("character",d-e),c.select()))}}else a.ra(c);a.mo=!0}return!1}f.V=function(){eq.m.V.call(this);null!=this.Bl&&window.clearTimeout(this.Bl);this.Bb.eb();delete this.Bb;this.Ni.eb();this.Vb.eb();ac(this.ua)};
f.$c=function(a){switch(a.keyCode){case 40:if(this.Na.Qe())return this.Cs?lq(this.Na):mq(this.Na),a.preventDefault(),!0;if(!this.Og)return this.update(!0),a.preventDefault(),!0;break;case 38:if(this.Na.Qe())return this.Cs?mq(this.Na):lq(this.Na),a.preventDefault(),!0;break;case 9:if(!this.Na.Qe()||a.shiftKey||this.Nv)this.Na.Lc();else if(this.update(),this.Na.Yf()&&this.Mv)return a.preventDefault(),!0;break;case 13:if(this.Na.Qe()){if(this.update(),this.Na.Yf())return a.preventDefault(),a.stopPropagation(),
!0}else this.Na.Lc();break;case 27:if(this.Na.Qe())return this.Na.Lc(),a.preventDefault(),a.stopPropagation(),!0;break;case 229:if(!this.ff)return this.ff||(this.Bb.A(this.Ca,"keyup",this.qr),this.Bb.A(this.Ca,"keypress",this.pr),this.ff=!0),!0;break;default:this.ua&&!this.jw&&(this.ua.stop(),this.ua.start())}return nq(this,a)};function nq(a,b){var c=a.Og&&b.charCode&&-1!=a.si.indexOf(String.fromCharCode(b.charCode));a.Zv&&c&&a.update();return a.Yv&&c&&a.Na.Yf()?(b.preventDefault(),!0):!1}f.Pu=function(){return!1};
f.handleFocus=function(a){hq(this,a.target||null)};function hq(a,b){a.Ni.ld();a.Na&&oq(a.Na);b!=a.Ca&&(a.Ca=b,a.ua&&(a.ua.start(),a.Bb.A(a.ua,"tick",a.vr)),a.En=a.aa(),a.Vb.jf(a.Ca),a.Bb.A(a.Vb,"key",a.ei),a.Bb.A(a.Ca,"mousedown",a.rr),t&&a.Bb.A(a.Ca,"keypress",a.nr))}f.handleBlur=function(){fq?this.Bl=window.setTimeout(q(this.Hr,this),0):this.Hr()};
f.Hr=function(){this.Ca&&(this.Bb.Aa(this.Vb,"key",this.ei),this.Vb.detach(),this.Bb.Aa(this.Ca,"keyup",this.Pu),this.Bb.Aa(this.Ca,"mousedown",this.rr),t&&this.Bb.Aa(this.Ca,"keypress",this.nr),this.ff&&pq(this),this.Ca=null,this.ua&&(this.ua.stop(),this.Bb.Aa(this.ua,"tick",this.vr)),this.Na&&qq(this.Na))};f.vr=function(){this.update()};f.or=function(a){this.handleFocus(a)};f.ei=function(a){this.Uq=a.keyCode;this.Na&&this.$c(a)};f.pr=function(){this.ff&&229!=this.Uq&&pq(this)};
f.qr=function(a){this.ff&&(13==a.keyCode||77==a.keyCode&&a.ctrlKey)&&pq(this)};f.rr=function(a){this.Na&&this.me(a)};f.me=function(){};function pq(a){a.ff&&(a.ff=!1,a.Bb.Aa(a.Ca,"keypress",a.pr),a.Bb.Aa(a.Ca,"keyup",a.qr))}f.nr=function(a){nq(this,a)};
f.update=function(a){if(this.Ca&&(a||this.aa()!=this.En)){if(a||!this.mo){a=aq(this.Ca)[0];var b=this.aa();a=kq(this,b)[jq(this,b,a)];a=this.As?String(a).replace(this.As,""):a;this.Na&&(this.Na.Za=this.Ca,this.Na.zo(a,this.aa()))}this.En=this.aa()}this.mo=!1};function jq(a,b,c){a=kq(a,b);if(c==b.length)return a.length-1;for(var d=b=0,e=0;d<a.length&&e<=c;d++)e+=a[d].length,b=d;return b}
function kq(a,b){if(!a.Og)return[b];for(var c=String(b).split(""),d=[],e=[],g=0,h=!1;g<c.length;g++)a.Xq&&-1!=a.Xq.indexOf(c[g])?(a.Eu&&!h&&(d.push(e.join("")),e.length=0),e.push(c[g]),h=!h):h||-1==a.si.indexOf(c[g])?e.push(c[g]):(e.push(c[g]),d.push(e.join("")),e.length=0);d.push(e.join(""));return d};function rq(a,b,c,d){null==a&&(a=null);null==b&&(b=null);null==c&&(c=null);null==d&&(d=null);rq.m.constructor.call(this,a,b,c,d)}r(rq,eq);rq.prototype.handleFocus=function(a){return rq.m.handleFocus.call(this,a)};rq.prototype.aa=function(){var a;return null!=(a=rq.m.aa.call(this))?a:""};function sq(){sq.m.constructor.call(this)}r(sq,ni);sq.prototype.ta=function(){return"tag"};sq.prototype.D=function(a){var b=sq.m.D.call(this,a);a.Pq()||(a=a.J().D("span",{"class":"goog-icon-remove"}),b.appendChild(a));return b};ka(sq);function tq(a,b){this.l=a;this.Iv=b;tq.m.constructor.call(this)}r(tq,sq);tq.prototype.D=function(a){var b=tq.m.D.call(this,a);a=uq(this.Iv,a.Ua);Vf(b,a);return b};function vq(a){this.l=a}var wq={Zs:"ac-image ac-image-list",zx:"ac-image-placeholder"};vq.prototype.vk=function(a,b,c){a instanceof el||(a=a.data);c.appendChild(uq(this,a));return c.appendChild(this.l.createTextNode(a.toString()))};function xq(a){return a.l.H("div","ac-image-placeholder")}function uq(a,b){null==b&&(b=null);return null!=b&&null!=b.photo?a.l.D("img",{src:b.photo,"class":wq.Zs,alt:b.toString()}):xq(a)};function yq(a,b){this.fa=a||[];this.mw=!b}yq.prototype.yk=function(a,b,c){if(this.mw){var d=this.fa,e=zq(a,b,d);0==e.length&&(e=Aq(a,b,d));b=e}else b=zq(a,b,this.fa);c(a,b)};function zq(a,b,c){var d=[];if(""!=a){a=Ua(a);a=new RegExp("(^|\\W+)"+a,"i");for(var e=0;e<c.length&&d.length<b;e++){var g=c[e];String(g).match(a)&&d.push(g)}}return d}
function Aq(a,b,c){for(var d=[],e=0;e<c.length;e++){var g=c[e],h=a.toLowerCase(),k=String(g).toLowerCase(),l=0;if(-1!=k.indexOf(h))l=parseInt((k.indexOf(h)/4).toString(),10);else for(var m=h.split(""),p=-1,u=10,A=0,B;B=m[A];A++)B=k.indexOf(B),B>p?(p=B-p-1,p>u-5&&(p=u-5),l+=p,p=B):(l+=u,u+=5);l<6*h.length&&d.push({Dd:g,Ur:l,index:e})}d.sort(function(a,b){var c=a.Ur-b.Ur;return 0!=c?c:a.index-b.index});a=[];for(A=0;A<b&&A<d.length;A++)a.push(d[A].Dd);return a};function Bq(a,b,c){null==b&&(b=!1);this.bw=null!=c?c:!0;Bq.m.constructor.call(this,a,b)}r(Bq,yq);Bq.prototype.yk=function(a,b,c,d){null==d&&(d=void 0);return this.bw&&""===a?c(a,this.fa.length<=b?this.fa:this.fa.slice(0,+(b-1)+1||9E9)):Bq.m.yk.call(this,a,b,c,d)};function Cq(a,b,c){md.call(this);this.Qf=a;this.Vg=c;this.U=b;v(b,[Dq,Eq,Fq,Gq],this.handleEvent,!1,this);this.nb=null;this.fa=[];this.bd=-1;this.Cb=0;this.qf=this.Za=null;this.qn={}}r(Cq,md);f=Cq.prototype;f.jv=10;f.Hp=!0;f.yp=!1;f.Js=!1;f.Xk=!1;var Dq="hilite",Eq="select",Gq="dismiss",Fq="canceldismiss";f=Cq.prototype;f.zf=function(){return this.nb};
f.handleEvent=function(a){var b=this.Qf;if(a.target==this.U)switch(a.type){case Dq:this.yd(a.bc);break;case Eq:var c=!1;if(ea(a.bc)){a=a.bc;var d=this.fa[Hq(this,a)],c=!!d&&b.wn&&b.wn(d);d&&!c&&this.bd!=a&&this.yd(a)}c||this.Yf();break;case Fq:oq(this);break;case Gq:qq(this)}};f.zo=function(a,b){this.nb!=a&&(this.nb=a,this.Qf.yk(this.nb,this.jv,q(this.Zq,this),b),oq(this))};f.Kj=function(){return this.Za};f.Qe=function(){return this.U.za()};
function mq(a){for(var b=a.Cb+a.fa.length-1,c=a.bd,d=0;d<a.fa.length;d++){if(c>=a.Cb&&c<b)c++;else if(-1==c)c=a.Cb;else if(a.yp&&c==b){a.yd(-1);break}else if(a.Js&&c==b)c=a.Cb;else break;if(a.yd(c))break}}function lq(a){for(var b=a.Cb+a.fa.length-1,c=a.bd,d=0;d<a.fa.length;d++){if(c>a.Cb)c--;else if(a.yp&&c==a.Cb){a.yd(-1);break}else if(!a.Js||-1!=c&&c!=a.Cb)break;else c=b;if(a.yd(c))break}}
f.yd=function(a){var b=Hq(this,a),c=this.fa[b];return c&&this.Qf.wn&&this.Qf.wn(c)?!1:(this.bd=a,this.U.yd(a),-1!=b)};f.Yf=function(){var a=Hq(this,this.bd);if(-1!=a){var b=this.fa[a],c=iq(this.Vg,b);this.Xk?(this.nb=null,qq(this)):this.Lc();c||(this.dispatchEvent({type:"update",bc:b,index:a}),this.Xk&&this.Vg.update(!0));return!0}this.Lc();this.dispatchEvent({type:"update",bc:null,index:null});return!1};
f.Lc=function(){this.bd=-1;this.nb=null;this.Cb+=this.fa.length;this.fa=[];window.clearTimeout(this.qf);this.qf=null;this.U.Lc();this.dispatchEvent("suggestionsupdate");this.dispatchEvent(Gq)};function qq(a){a.qf||(a.qf=window.setTimeout(q(a.Lc,a),100))}f.Lq=function(){return this.qf?(window.clearTimeout(this.qf),this.qf=null,!0):!1};function oq(a){a.Lq()||window.setTimeout(q(a.Lq,a),10)}f.V=function(){Cq.m.V.call(this);delete this.qn;this.U.eb();this.Vg.eb();this.Qf=null};
f.Zq=function(a,b,c){this.nb==a&&this.se(b,c)};f.se=function(a,b){var c="object"==n(b)&&b,d=(c?c.Gu():b)?Hq(this,this.bd):-1;this.Cb+=this.fa.length;this.fa=a;for(var e=[],g=0;g<a.length;++g)e.push({id:this.Cb+g,data:a[g]});g=null;this.Za&&(g=this.qn[qa(this.Za)]||this.Za);this.U.Cp=g;this.U.se(e,this.nb,this.Za);g=this.Hp;c&&void 0!==c.tm()&&(g=c.tm());this.bd=-1;(g||0<=d)&&0!=e.length&&this.nb&&(0<=d?this.yd(this.Cb+d):mq(this));this.dispatchEvent("suggestionsupdate")};
function Hq(a,b){var c=b-a.Cb;return 0>c||c>=a.fa.length?-1:c}f.update=function(a){this.Vg.update(a)};function Iq(a,b,c){this.Yf=q(this.Yf,this);this.Rn=q(this.Rn,this);this.se=q(this.se,this);Iq.m.constructor.call(this,a,b,c)}r(Iq,Cq);f=Iq.prototype;f.zo=function(a,b){null==b&&(b=void 0);this.nb=a;this.Qf.yk(this.nb,this.Qf.fa.length,q(this.Zq,this),b);return oq(this)};
f.se=function(a,b){var c;var d=((c="object"===n(b)&&b)?c.Gu():b)?Hq(this,this.bd):-1;this.Cb+=this.fa.length;this.fa=this.Rn(a);var e=[];var g=this.fa;for(l in g){var h=g[l];var k={id:this.Cb+Lm(l),data:h};null!=h.group&&(k.group=h.group);e.push(k);k=null}this.Za&&(k=this.qn[qa(this.Za)]||this.Za);this.U.Cp=k;this.U.se(e,this.nb,this.Za);var l=this.Hp;c&&null!=c.tm()&&(l=c.tm());this.bd=-1;if(l||0<=d&&0!==e.length&&this.nb)0<=d?this.yd(this.Cb+d):mq(this);this.dispatchEvent("suggestionsupdate")};
f.Rn=function(a){var b;var c=[];var d=0;for(b=a.length;d<b;d++){var e=a[d];this.Fg(e,this.U.vg())||c.push(e)}return c};f.Fg=function(a,b){return null!=b[a.toString()]};f.Yf=function(){var a=Hq(this,this.bd);if(-1!==a){var b=this.fa[a];var c=iq(this.Vg,b);this.Xk?(this.nb=null,qq(this)):this.Lc();c||(this.dispatchEvent({type:"update",bc:b,index:a}),this.Xk&&this.Vg.update(!0));return!0}this.Lc();this.dispatchEvent({type:"update",bc:null,index:null});return!1};function Jq(a,b){this.l=a;this.select=b}Jq.prototype.load=function(a){var b;var c=Cm(this.select);if(null==c)return null;var d=String(c);var e=0;for(b=a.length;e<b;e++)if(c=a[e],String(c.id)===d)return c;return null};Jq.prototype.df=function(a){null==a&&(a=null);return Dm(this.select,null!=a?String(a.id):"")};Jq.prototype.Zi=function(){var a=Cm(this.select);if(null!=a)return this.l.unselect(this.select,a)};function Kq(a,b){this.l=a;this.S=b;this.cn=q(this.cn,this);this.Hb=q(this.Hb,this);this.Me=q(this.Me,this);this.Hm=q(this.Hm,this);Kq.m.constructor.call(this);this.input=this.Ok=this.lc=this.select=null}r(Kq,md);function Lq(a,b){var c=a.Ok.load(b);if(null!=c)return Mq(a,c)}
function Mq(a,b){var c=a.lc,d=a.input.o();Hf(d)&&(d.removeAttribute("role"),d.removeAttribute("aria-autocomplete"));d==c.Ca&&c.handleBlur();c.Bb.Aa(d,"focus",c.handleFocus);c.Bb.Aa(d,"blur",c.handleBlur);c.Ca||c.Ni.Aa(d,"keydown",c.or);a.input.ra(b.toString());gq(a.lc,a.input.o());a.input.aa()&&Pi(a.input);a.l.select(a.select,b.pk);return Nq(a.S,b)}f=Kq.prototype;
f.M=function(a){this.select=a;if(a.hasAttribute("multiple"))throw Error("For select-multiple use wzk.ui.ac.ExtSelectbox");this.S.M(this.select);this.Ok=new Jq(this.l,this.select);this.S.A(Oq,this.Me);this.S.A(Pq,this.Hb);this.input=this.S.input;this.select.hasAttribute("placeholder")&&Mi(this.input,this.select.getAttribute("placeholder"));a=Sh(this.select,"chooseLabel");this.Zt=Sh(this.select,"chooseValue");if(a)return a=this.l.H("span","ac-choose-value",a),this.l.Vj(a,this.input.o()),v(a,"click",
this.Hm)};f.Hm=function(){this.l.select(this.select,this.Zt);return Lq(this,this.data)};f.Me=function(){this.Ok.Zi();return hd(this.select,"change",!1,{type:"change",target:this.select})};f.Hb=function(){this.S.input.o().focus();return this.gf.se(this.data)};
f.load=function(a){this.data=a;this.Mg=new Bq(this.data,!1);this.lc=new rq(null,null,!1);this.gf=new Iq(this.Mg,this.S,this.lc);a=this.S.input.o();this.gf.Za=a;this.lc.Na=this.gf;gq(this.lc,this.S.input.o());this.gf.A("update",this.cn);Lq(this,this.data);return this.dispatchEvent(new Cc("loaded"))};f.cn=function(a){this.Ok.df(a.bc);Nq(this.S,a.bc);this.S.input.Cf();return hd(this.select,"change",!1,{type:"change",target:this.select})};f.destroy=function(){return this.input.destroy()};function Qq(a,b,c,d){md.call(this);this.Wb=a||document.body;this.R=jf(this.Wb);this.Sv=!a;this.F=null;this.nb="";this.fa=[];this.ue=[];this.qs=this.Tj=-1;this.La=!1;this.className="ac-renderer";this.oi="ac-row";this.Wq="active";this.Al="ac-active";this.Yu="ac-highlighted";this.be=b||null;this.Es=null!=d?d:!0;this.iv=!0;this.Sj=!1;this.Tv=!!c;this.bh=!1;this.ik=0;this.Bo=!1}r(Qq,md);f=Qq.prototype;f.Vt=0;f.o=function(){return this.F};
f.se=function(a,b,c){this.nb=b;this.fa=a;this.Tj=-1;this.qs=wa();this.Za=c;this.ue=[];this.redraw()};f.Lc=function(){this.La&&(this.La=!1,Rq(this,!1),0<this.ik?(ac(this.hg),this.hg=new Wg(this.F,this.ik),this.hg.play()):w(this.F,!1))};f.show=function(){this.La||(this.La=!0,Rq(this,!0),0<this.ik?(ac(this.hg),this.hg=new Xg(this.F,this.ik),this.hg.play()):w(this.F,!0))};
function Rq(a,b){a.Za&&(fi(a.Za,"haspopup",b),fi(a.F,"expanded",b),fi(a.Za,"expanded",b),b?fi(a.Za,"owns",a.F.id):(a.Za.removeAttribute("aria-owns"),gi(a.Za,null)))}f.za=function(){return this.La};function Sq(a,b){var c=0<=b&&b<a.fa.length?a.fa[b]:void 0,d=0<=b&&b<a.ue.length?a.ue[b]:void 0;a.dispatchEvent({type:"rowhilite",Oz:d,bc:c?c.data:null})&&(0<=a.Tj&&mi(a.ue[a.Tj],[a.Al,a.Wq]),a.Tj=b,d&&(ki(d,[a.Al,a.Wq]),a.Za&&gi(a.Za,d),pg(d,a.F)))}
f.yd=function(a){if(-1==a)Sq(this,-1);else for(var b=0;b<this.fa.length;b++)if(this.fa[b].id==a){Sq(this,b);break}};f.Ln=function(){if(!this.F){var a=this.R.D("DIV",{style:"display:none"});this.Bo&&(a.style.overflowY="auto");this.F=a;ki(a,Ea(this.className).split(" "));ei(a,"listbox");a.id=jh(ih.va());this.R.appendChild(this.Wb,a);v(a,"click",this.Nh,!1,this);v(a,"mousedown",this.Ph,!1,this);v(a,"mouseover",this.Vm,!1,this)}};
f.redraw=function(){this.Ln();this.bh&&(this.F.style.visibility="hidden");this.Zo&&(this.F.style.minWidth=this.Zo.clientWidth-this.Vt+"px");this.ue.length=0;this.R.Ug(this.F);if(this.be&&this.be.render)this.be.render(this,this.F,this.fa,this.nb);else{var a=null;eb(this.fa,function(b){b=this.wk(b,this.nb);this.bh?this.F.insertBefore(b,a):this.R.appendChild(this.F,b);a=b},this)}0==this.fa.length?this.Lc():(this.show(),this.uc(),xg(this.F,!0))};
f.uc=function(){if(this.Za&&this.Sv){var a=this.Cp||this.Za;var b=this.Tv?5:1;this.bh&&(b^=1);var c=65;this.Bo&&(this.F.style.height="",c|=32);gk(a,b,this.F,b^1,null,null,c);this.bh&&(this.F.style.visibility="visible")}};f.Kj=function(){return this.Za||null};f.V=function(){this.F&&(dd(this.F,"click",this.Nh,!1,this),dd(this.F,"mousedown",this.Ph,!1,this),dd(this.F,"mouseover",this.Vm,!1,this),this.R.removeNode(this.F),this.F=null,this.La=!1);ac(this.hg);this.Wb=null;Qq.m.V.call(this)};
function Tq(a,b,c){if(a.Sj||!a.Hs)if(3==b.nodeType){var d=null;la(c)&&1<c.length&&!a.Sj&&(d=ub(c,1));c=Uq(a,c);if(0!=c.length){var e=b.nodeValue,g=a.iv?new RegExp("\\b(?:"+c+")","gi"):new RegExp(c,"gi");c=[];for(var h=0,k=g.exec(e),l=0;k;)l++,c.push(e.substring(h,k.index)),c.push(e.substring(k.index,g.lastIndex)),h=g.lastIndex,k=g.exec(e);c.push(e.substring(h));if(1<c.length){d=a.Sj?l:1;for(e=0;e<d;e++)g=2*e,b.nodeValue=c[g],h=a.R.createElement("B"),h.className=a.Yu,a.R.appendChild(h,a.R.createTextNode(c[g+
1])),h=b.parentNode.insertBefore(h,b.nextSibling),b.parentNode.insertBefore(a.R.createTextNode(""),h.nextSibling),b=h.nextSibling;b.nodeValue=ub(c,2*d).join("");a.Hs=!0}else d&&Tq(a,b,d)}}else for(b=b.firstChild;b;)d=b.nextSibling,Tq(a,b,c),b=d}
function Uq(a,b){var c="";if(!b)return c;la(b)&&(b=gb(b,function(a){return!Ba(null==a?"":String(a))}));a.Sj?la(b)?c=hb(b,Ua).join("|"):(c=b.replace(/[\s\xa0]+/g," ").replace(/^\s+|\s+$/g,""),c=Ua(c),c=c.replace(/ /g,"|")):la(b)?c=0<b.length?Ua(b[0]):"":/^\W/.test(b)||(c=Ua(b));return c}
f.wk=function(a,b){var c=this.R.D("DIV",{className:this.oi,id:jh(ih.va())});ei(c,"option");this.be&&this.be.vk?this.be.vk(a,0,c):Kf(c,a.data.toString());b&&this.Es&&(this.Hs=!1,Tq(this,c,b));ji(c,this.oi);this.ue.push(c);return c};function Vq(a,b){for(;b&&b!=a.F&&!ii(b,a.oi);)b=b.parentNode;return b?db(a.ue,b):-1}f.Nh=function(a){var b=Vq(this,a.target);0<=b&&this.dispatchEvent({type:Eq,bc:this.fa[b].id});a.stopPropagation()};f.Ph=function(a){a.stopPropagation();a.preventDefault()};
f.Vm=function(a){a=Vq(this,a.target);0<=a&&!(300>wa()-this.qs)&&this.dispatchEvent({type:Dq,bc:this.fa[a].id})};function Wq(a){null==a&&(a={});null==a.kf&&(a.kf=Xq);a.S=new yh(a.kf,"");Wq.m.constructor.call(this,a)}r(Wq,zh);var Xq="open-list";function Yq(a,b,c,d,e){this.l=a;this.Fc=c;this.Ll=q(this.Ll,this);this.Pi=q(this.Pi,this);this.jo=q(this.jo,this);this.redraw=q(this.redraw,this);this.vg=q(this.vg,this);this.Df=q(this.Df,this);this.Hb=q(this.Hb,this);this.xo=q(this.xo,this);this.yo=q(this.yo,this);Yq.m.constructor.call(this,null,c,d,e);this.className=Zq;this.oi=$q;this.Al=ar;this.input=this.Jf=null}r(Yq,Qq);var Zq="ac dropdown-menu",$q="ac-row",ar="active",Oq="clean",Pq="open";f=Yq.prototype;
f.M=function(a){this.select=a;w(this.select,!1);this.Tg=this.select.hasAttribute("readonly");this.$d=this.l.H("span","ac-item ac-buttons");null!=this.Fc&&(this.Jf=uq(this.Fc),this.l.appendChild(this.$d,this.Jf));a=Yf(this.select);this.l.appendChild(a,this.$d);this.input=new Oi(this.l);this.input.A("VALUE_CHANGE",this.Df);this.input.render(this.$d);this.select.hasAttribute(!this.Tg)&&this.input.o().setAttribute("placeholder",this.select.getAttribute("placeholder"));this.items=this.l.D("div");null!=
this.Fc&&Zg(this.input.o(),"with-image");if(this.Tg)return this.input.o().setAttribute("readonly","true");this.pe=new Wq({l:this.l});this.pe.A("action",this.Hb);return this.pe.render(this.$d)};f.yo=function(a){this.ws=a};f.xo=function(){};f.Hb=function(){return this.dispatchEvent(new Cc(Pq))};f.Df=function(a){if(!this.Tg){if(a.target.wd())return w(this.pe.o(),!1);w(this.pe.o(),!0);null!=this.Fc&&(a=xq(this.Fc),Wf(a,this.Jf),this.Jf=a);return this.dispatchEvent(new Cc(Oq))}};
function Nq(a,b){null==b&&(b=null);if(null!=a.Jf){var c=uq(a.Fc,b);Wf(c,a.Jf);return a.Jf=c}}f.Ln=function(){if(!this.F){var a=this.l.D("ul",{style:"display:none"});this.Bo&&ag(a,"overflow-y","auto");this.F=a;ki(a,Ea(this.className).split(" "));ei(a,"listbox");a.id=jh(ih.va());this.l.appendChild(this.$d,a);v(a,"click",this.Nh,!1,this);v(a,"mousedown",this.Ph,!1,this);v(a,"mouseover",this.Vm,!1,this)}};f.vg=function(){return null==this.ws?{}:this.ws.getTags()};
f.redraw=function(){this.Ln();this.bh&&(this.F.style.visibility="hidden");if(this.Zo){var a=this.Zo.clientWidth+"px";this.F.style.minWidth=a}this.ue.length=0;this.R.Ug(this.F);if(this.be&&this.be.render)this.be.render(this,this.F,this.fa,this.nb);else{a=this.fa;var b,c;var d={};var e=[];var g=b=0;for(c=a.length;g<c;g++){var h=a[g];null!=h.group?null!=d[h.group]?e[d[h.group]].push(h):(d[h.group]=b,e[d[h.group]]=[h],b++):(e.push(h),b++)}this.jo(e,null)}if(0===this.fa.length)this.Lc();else return this.show(),
this.uc(),xg(this.F,!0)};f.jo=function(a,b){var c,d;var e=[];var g=0;for(c=a.length;g<c;g++){var h=a[g];if(h instanceof Array){var k=this.Ll(h[0]?h[0].group:"");var l=this.l.H("ul");k.appendChild(l);var m=0;for(d=h.length;m<d;m++){var p=h[m];l.appendChild(this.wk(p,this.nb))}this.Pi(k,b);e.push(b=k)}else h=this.wk(h,this.nb),this.Pi(h,b),e.push(b=h)}return e};f.Pi=function(a,b){return this.bh?this.F.insertBefore(a,b):this.R.appendChild(this.F,a)};f.Ll=function(a){return this.l.H("li","",a)};
f.wk=function(a,b){var c=jh(ih.va());c=this.l.D("li",{className:$q,id:c});ei(c,"option");Rh(c,"value",a.data.id);this.Fc&&this.Fc.vk?this.Fc.vk(a,0,c):Kf(c,a.data.toString());b&&this.Es&&Tq(this,c,b);Zg(c,this.oi);this.ue.push(c);return c};function br(){br.m.constructor.call(this)}r(br,ni);br.prototype.ta=function(){return"tag-container"};ka(br);function cr(a,b,c,d){this.Tg=null!=d?d:!1;this.sc=q(this.sc,this);null==b&&(b=sq.va());cr.m.constructor.call(this,a,b,c);this.icon=this.listener=null}r(cr,zi);f=cr.prototype;f.D=function(){cr.m.D.call(this);this.icon=Eg(this.R,"goog-icon-remove",this.o());return this.zg(this.o())};f.pa=function(a){cr.m.pa.call(this,a);this.icon&&w(this.icon,a);a?this.zg(this.o()):fd(this.o())};f.zg=function(a){if(null!=a)return v(a,[Gc.fp,Gc.jp],this.sc)};
f.sc=function(a){if(a.target===this.icon&&(a.type===Gc.jp&&13===a.keyCode||a.type===Gc.fp))return this.dispatchEvent("remove")};f.Pq=function(){return this.Tg};f.cs=function(a){return this.Tg=a};function dr(a,b,c){null==b&&(b=br.va());dr.m.constructor.call(this,a,b,c);this.$g={}}r(dr,zi);function er(a,b,c,d,e){null==e&&(e=!1);b=new cr(b,d,a.R);b.cs(e);b.Ua=c;return a.add(b)}f=dr.prototype;f.add=function(a){if(null==this.$g[this.getKey(a.Ua)])return this.zg(a),this.$a(a,!0),this.$g[this.getKey(a.Ua)]=a,this.dispatchEvent("add"),this.dispatchEvent(new Cc("add-tag",a))};f.getTags=function(){return this.$g};f.isEmpty=function(){return 0===Gb(this.$g)};f.getKey=function(a){return a.toString()};
f.zg=function(a){var b=this;return cd(a,"remove",function(c){c.stopPropagation();Mb(b.$g,b.getKey(c.target.Ua));b.removeChild(c.target,!0);b.dispatchEvent("remove");return b.dispatchEvent(new Cc("remove-tag",a))})};f.pa=function(a){var b;dr.m.pa.call(this,a);var c=this.$g;for(b in c){var d=c[b];d.pa(a)}};f.io=function(a){return this.ji(Yf(a),Xf(a))};function fr(a,b,c,d){var e=this;this.l=a;this.S=b;this.Fc=c;this.lc=null!=d?d:null;this.Hb=q(this.Hb,this);this.Df=q(this.Df,this);fr.m.constructor.call(this);this.Ec=new dr(null,null,this.l);this.input=new Oi(a);this.input.A("VALUE_CHANGE",this.Df);this.pe=new Wq({l:this.l});this.pe.A("action",this.Hb);this.Ec.A("add-tag",function(a){return e.lc.add(a.target)});this.Ec.A("remove-tag",function(a){return e.lc.remove(a.target)});this.pn=new rq(null,null,!1);this.Mg=this.autoComplete=null;this.enabled=
!0}r(fr,md);f=fr.prototype;f.M=function(a,b){if(!a.hasAttribute("multiple"))throw Error("For select-one use wzk.ui.ac.SelectAutoComplete");this.render(a,b);if(a.disabled)return this.pa(!1)};
f.render=function(a,b){this.data=b;this.Mg=new Bq(this.data,!1);this.autoComplete=new Iq(this.Mg,this.S,this.pn);w(a,!1);this.Ec.io(a);this.S.yo(this.Ec);this.S.xo(this.data);var c=a.hasAttribute("readonly");if(!c){var d=this.l.D("div","ac-buttons");this.input.render(d);this.pe.render(d);this.S.$d=d;this.l.Mq(d,a);a.hasAttribute("placeholder")&&Mi(this.input,a.getAttribute("placeholder"));this.pn.Na=this.autoComplete;gq(this.pn,this.input.o());gr(this,this.autoComplete);hr(this,a)}if(this.lc)return this.lc.load(b,
this.Ec,this.Fc,c)};f.Df=function(a){return a.target.wd()?w(this.pe.o(),!1):w(this.pe.o(),!0)};f.clear=function(){return this.input.ra("")};f.pa=function(a){this.enabled=a;this.input.pa(this.enabled);this.input.da(!1);return this.Ec.pa(!1)};f.refresh=function(a){this.Mg.fa=a||[]};function gr(a,b){v(b,"update",function(b){if(null!=a.Fc){var c=a.Fc;c=new tq(c.l,c)}else c=null;er(a.Ec,b.bc.toString(),b.bc,c);return a.input.clear()})}f.Hb=function(){this.input.o().focus();return this.autoComplete.se(this.data)};
function hr(a,b){var c=b.required;a.input.hk(c);b.required=void 0;v(a.Ec,"add-tag",function(){return a.input.hk(!1)});v(a.Ec,"remove-tag",function(){if(a.Ec.isEmpty()&&c)return a.input.hk(!0)})};function ir(a,b){this.l=a;this.select=b;this.table={}}f=ir.prototype;f.load=function(a,b,c,d){var e;null==d&&(d=!1);var g=[];var h=0;for(e=a.length;h<e;h++){var k=a[h];g.push(this.table[String(this.getKey(k))]=k)}a=Cm(this.select);if(null!=a){e=[];k=0;for(h=a.length;k<h;k++){g=a[k];g=this.table[g];var l=null!=c?new tq(c.l,c):null;e.push(er(b,g.toString(),g,l,d))}return e}};
f.df=function(a){var b={};a=a.getTags();for(e in a){var c=a[e];c=c.Ua;b[this.getKey(c)]=c}var d=this.l.he(this.select);a=0;for(c=d.length;a<c;a++){var e=d[a];e.selected=Jb(b,e.value)}return hd(this.select,"change",!1,{type:"change",target:this.select})};f.add=function(a){return this.l.select(this.select,this.getKey(a))};f.remove=function(a){return this.l.unselect(this.select,this.getKey(a))};f.getKey=function(a){return a.id};function jr(a,b){this.select=a;this.cf=b;this.an=q(this.an,this);this.Ec=null}f=jr.prototype;f.load=function(a,b,c,d){this.Ec=b;this.cf.load(a,b,c,d);if(null!=this.select.form)return v(this.select.form,"submit",this.an)};f.df=function(a){return this.cf.df(a)};f.add=function(a){return this.cf.add(a.Ua)};f.remove=function(a){return this.cf.remove(a.Ua)};f.an=function(){return this.df(this.Ec)};function kr(){this.Jn=q(this.Jn,this)}kr.prototype.load=function(a,b,c){var d;this.l=b;b=[];var e=a.children;a=0;for(d=e.length;a<d;a++){var g=e[a];"OPTGROUP"===g.nodeName?(g=this.Jn(g),null!=g&&(b=b.concat(g))):(g=this.qh(g),null!=g&&b.push(g))}return c(b)};kr.prototype.Jn=function(a){var b;var c=a.getAttribute("label");var d=[];var e=this.l.all("option",a);a=0;for(b=e.length;a<b;a++){var g=e[a];g=this.qh(g,c);null!=g&&d.push(g)}return 0<d.length?d:null};
kr.prototype.qh=function(a,b){null==b&&(b=void 0);var c=a.getAttribute("value");if(null==c||""===c||"__all__"===c)return null;c={zz:this.l.wg(a),id:c};null!=b&&(c.group=b);var d=Sh(a,"image");null!=d&&0<d.length&&(c.photo=d);return new el(c)};function lr(a,b){var c=mr(a,b);(new kr).load(a,b,function(a){return c.load(a)});return c}function nr(a,b){var c=or(a,b);return(new kr).load(a,b,function(b){return c.M(a,b)})}function mr(a,b){var c=new Yq(b,0,pr(a,b));c=new Kq(b,c);c.M(a);return c}function or(a,b){if(null!=a){var c=pr(a,b);var d=new Yq(b,0,c);return new fr(b,d,c,new jr(a,new ir(b,a)))}}function pr(a,b){var c=null;null!=a.querySelector("option[data-image]")&&(c=new vq(b));return c};function qr(a,b,c){this.l=a;this.client=b;this.register=c;this.Hb=q(this.Hb,this);this.Ff=q(this.Ff,this)}qr.prototype.data=function(a){return String(Sh(this.H,a))};
qr.prototype.M=function(a){this.H=a;this.Nn=new Xp(this.l,this.client,this.data("url"),this.data("form"),this.register);this.H.title&&this.Nn.wi(this.H.title);(this.hi=this.l.o(this.data("related")))&&dh(this.hi,"related-fulltext-search")&&(this.hi=this.hi,this.gf=lr(this.hi,this.l),this.Nn.A("success-close",this.Ff));return v(this.H,"click",this.Hb)};
qr.prototype.Ff=function(a){a=new el(a.target.obj);var b=this.gf;var c=b.l.H("option",{value:a.pk});b.l.Sc(c,a.toString());b.select.appendChild(c);b.data.push(a);b.Mg.fa=b.data||[];return Mq(this.gf,a)};qr.prototype.Hb=function(){return this.Nn.open()};function rr(a,b,c,d){var e=String(Sh(a,"url"));var g=String(Sh(a,"snippetName"));var h=new Xp(b,new pl(c),e,g,d);(b=Sh(a,"modalTitle"))&&h.wi(String(b));return v(a,"click",function(a){a.preventDefault();return h.open()})};function sr(a){this.start=null!=a?a:0}sr.prototype.next=function(){return this.start++};sr.prototype.tc=function(a){return a.replace("__prefix__",this.start.toString())};function tr(a){null==a&&(a={});tr.m.constructor.call(this,a)}r(tr,xh);tr.prototype.Ga=function(a){var b;var c=Fg(this.l,"inline-line",a);var d=0;for(b=c.length;d<b;d++)a=c[d],ur(this,a)};function ur(a,b){var c=vr(a.l,b);if(null==c)return null;w(c,!1);var d=new Ah({l:a.l,Rc:b});d.io(c);return d};function wr(a,b,c,d){this.bc=a;this.Gh=b;this.Xb=c;this.l=d;this.Qm=q(this.Qm,this);wr.m.constructor.call(this);this.parent=Yf(this.bc);a=this.bc.cloneNode(!0);ah(a,this.qk);this.clone=a;this.Uv=new tr({l:this.l});a=xr;this.qk=a.mp;this.lk=a.ip;ah(this.clone,a.ip,a.mp)}r(wr,md);var xr={ip:"even",mp:"odd"};function yr(a,b,c){var d=ur(a.Uv,b);null!=d&&d.A("action",a.Qm);c&&a.Xb.tc(b)}
wr.prototype.Qm=function(a){var b=a.target;a=vr(this.l,b.Rc);if(null!=a)return a.checked=!0,this.dispatchEvent(new Cc("delete",b.Rc))};wr.prototype.oj=function(a){Zg(a,this.lk);return zr(this)};function zr(a){var b;return b=[a.qk,a.lk],a.lk=b[0],a.qk=b[1],b}wr.prototype.pa=function(a){var b;var c=this.l.he(this.parent);var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];d.push(w(ca(g.lastElementChild)?g.lastElementChild:Gf(g.lastChild,!1),a))}return d};function Ar(a){this.fieldset=a;this.vd=0;this.fh=1E3;this.wq=null}Ar.prototype.load=function(){var a;var b=this.fieldset.querySelectorAll("input[type\x3dhidden]");var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];Aa(e.name,"TOTAL_FORMS")?(this.vd=parseInt(e.value,10),c.push(this.wq=e)):Aa(e.name,"MAX_NUM_FORMS")?(this.fh=parseInt(e.value,10),c.push(e)):Aa(e.name,"INITIAL_FORMS")?c.push(e):c.push(void 0)}return c};Ar.prototype.df=function(a){var b;return null!=(b=this.wq)?b.value=a:void 0};function Br(a,b){this.l=a;this.Xb=b;this.sc=q(this.sc,this);this.Wm=q(this.Wm,this);this.vd=0;this.fh=1E3;this.listener=this.qd=this.qg=null}
Br.prototype.Wm=function(a){var b;a=a.target;if(ii(a,"empty")){this.vd--;this.qg.df(this.vd);var c=this.qd;var d,e;c.Gh.start--;for(b=a;b=Xf(b);){eh(b,c.lk);eh(b,c.qk);var g=c.l.all("input, select, textarea",b);var h=0;for(d=g.length;h<d;h++){var k=g[h];var l=["name","id","data-for"];var m=0;for(e=l.length;m<e;m++){var p=l[m];if(null!=k[p]){var u=k[p].split("-");u[u.length-2]=parseInt(u[u.length-2],10)-1;k[p]=u.join("-")}}}}c.l.removeNode(a);return zr(c)}b=this.l.all("input[type\x3dfile]",a);p=0;
for(k=b.length;p<k;p++)c=b[p],this.l.removeNode(c);w(a,!1);k=this.l.all("input, select",a);b=[];c=0;for(p=k.length;c<p;c++)a=k[c],b.push(a.removeAttribute("required"));return b};
Br.prototype.sc=function(){var a=this.qd;var b=a.clone.cloneNode(!0);yr(a,b,!0);a.oj(b);var c,d;var e=a.l.all("input, select, textarea",b);var g=0;for(c=e.length;g<c;g++){var h=e[g];var k=["name","id","data-for"];var l=0;for(d=k.length;l<d;l++){var m=k[l];null!=h[m]&&(h[m]=a.Gh.tc(h[m]))}}c=a.l.all("div[data-for]",b);d=[];g=0;for(l=c.length;g<l;g++)m=c[g],h=Sh(m,"for"),null!=h?d.push(Rh(m,"for",a.Gh.tc(h))):d.push(void 0);a.parent.appendChild(b);a.Gh.next();this.vd++;this.qg.df(this.vd);this.vd===
this.fh&&(Zg(this.zc,"disabled"),ed(this.listener));if(1===this.l.all("input[type\x3dfile]",b).length)return a=(a=Dg(this.l,"input[type\x3dfile]",b))?a.click():void 0,a};function vr(a,b){var c;var d=a.all("input[type\x3dcheckbox]",b);var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];Aa(h.id,"DELETE")&&e.push(h)}return e.length?e[0]:null};function Cr(a,b){this.Zl=a instanceof df?a:new df(a,b)}r(Cr,jk);Cr.prototype.uc=function(a,b,c,d){gk(kg(a),0,a,b,this.Zl,c,null,d)};function Dr(a){this.jb=new Pd;a&&this.addAll(a)}function Er(a){var b=typeof a;return"object"==b&&a||"function"==b?"o"+qa(a):b.substr(0,1)+a}f=Dr.prototype;f.xd=function(){return this.jb.xd()};f.add=function(a){this.jb.set(Er(a),a)};f.addAll=function(a){a=Jd(a);for(var b=a.length,c=0;c<b;c++)this.add(a[c])};f.ld=function(a){a=Jd(a);for(var b=a.length,c=0;c<b;c++)this.remove(a[c])};f.remove=function(a){return this.jb.remove(Er(a))};f.clear=function(){this.jb.clear()};f.isEmpty=function(){return this.jb.isEmpty()};
f.contains=function(a){return this.jb.ae(Er(a))};f.Nq=function(a){var b=new Dr;a=Jd(a);for(var c=0;c<a.length;c++){var d=a[c];this.contains(d)&&b.add(d)}return b};f.Gb=function(){return this.jb.Gb()};f.clone=function(){return new Dr(this)};f.pb=function(a){return this.xd()==Id(a)&&Fr(this,a)};
function Fr(a,b){var c=Id(b);if(a.xd()>c)return!1;!(b instanceof Dr)&&5<c&&(b=new Dr(b));return Md(a,function(a){var c=b;if(c.contains&&"function"==typeof c.contains)a=c.contains(a);else if(c.Ch&&"function"==typeof c.Ch)a=c.Ch(a);else if(ma(c)||da(c))a=nb(c,a);else a:{for(var d in c)if(c[d]==a){a=!0;break a}a=!1}return a})};function Gr(a,b,c){this.R=c||(a?jf(mf(document,a)):jf());lk.call(this,this.R.D("DIV",{style:"position:absolute;display:none;"}));this.am=new df(1,1);this.uf=new Dr;this.Zf=null;a&&this.jf(a);null!=b&&Kf(this.o(),b)}r(Gr,lk);var Hr=[];f=Gr.prototype;f.oc=null;f.className="goog-tooltip";f.fs=500;f.Wu=0;f.J=function(){return this.R};
f.jf=function(a){a=mf(document,a);this.uf.add(a);v(a,"mouseover",this.xg,!1,this);v(a,"mouseout",this.Oj,!1,this);v(a,"mousemove",this.Eq,!1,this);v(a,"focus",this.handleFocus,!1,this);v(a,"blur",this.Oj,!1,this)};f.detach=function(a){if(a)a=mf(document,a),Ir(this,a),this.uf.remove(a);else{for(var b=this.uf.Gb(),c=0;a=b[c];c++)Ir(this,a);this.uf.clear()}};
function Ir(a,b){dd(b,"mouseover",a.xg,!1,a);dd(b,"mouseout",a.Oj,!1,a);dd(b,"mousemove",a.Eq,!1,a);dd(b,"focus",a.handleFocus,!1,a);dd(b,"blur",a.Oj,!1,a)}f.uo=function(a){var b=this.o();b&&Ff(b);Gr.m.uo.call(this,a);a?(b=this.R.P.body,b.insertBefore(a,b.lastChild),ac(this.Zf),this.Zf=new Tl(this.o()),$b(this,va(ac,this.Zf)),v(this.Zf,"focusin",this.rh,void 0,this),v(this.Zf,"focusout",this.Nk,void 0,this)):(ac(this.Zf),this.Zf=null)};
f.getState=function(){return this.af?this.za()?4:1:this.Sh?3:this.za()?2:0};f.Tn=function(){if(!ek.prototype.Tn.call(this))return!1;if(this.anchor)for(var a,b=0;a=Hr[b];b++)If(a.o(),this.anchor)||a.da(!1);nb(Hr,this)||Hr.push(this);a=this.o();a.className=this.className;this.rh();v(a,"mouseover",this.Gq,!1,this);v(a,"mouseout",this.Fq,!1,this);Jr(this);return!0};
f.Ve=function(){ob(Hr,this);for(var a=this.o(),b,c=0;b=Hr[c];c++)b.anchor&&If(a,b.anchor)&&b.da(!1);this.Cr&&this.Cr.Nk();dd(a,"mouseover",this.Gq,!1,this);dd(a,"mouseout",this.Fq,!1,this);this.anchor=void 0;0==this.getState()&&(this.Dk=!1);ek.prototype.Ve.call(this)};f.$q=function(a,b){this.anchor==a&&this.uf.contains(this.anchor)&&(this.Dk||!this.Nz?(this.da(!1),this.za()||(this.anchor=a,this.setPosition(b||Kr(this,0)),this.da(!0))):this.anchor=void 0);this.af=void 0};f.sm=function(){return this.oc};
f.kv=function(a){this.Sh=void 0;if(a==this.anchor){a=this.J();var b=a.sm();a=b&&this.o()&&a.contains(this.o(),b);null!=this.oc&&(this.oc==this.o()||this.uf.contains(this.oc))||a||this.Pp&&this.Pp.oc||this.da(!1)}};function Lr(a,b){var c=uf(a.R.P);a.am.x=b.clientX+c.x;a.am.y=b.clientY+c.y}f.xg=function(a){var b=Mr(this,a.target);this.oc=b;this.rh();b!=this.anchor&&(this.anchor=b,this.af||(this.af=od(q(this.$q,this,b,void 0),this.fs)),Nr(this),Lr(this,a))};
function Mr(a,b){try{for(;b&&!a.uf.contains(b);)b=b.parentNode;return b}catch(c){return null}}f.Eq=function(a){Lr(this,a);this.Dk=!0};f.handleFocus=function(a){this.oc=a=Mr(this,a.target);this.Dk=!0;if(this.anchor!=a){this.anchor=a;var b=Kr(this,1);this.rh();this.af||(this.af=od(q(this.$q,this,a,b),this.fs));Nr(this)}};function Kr(a,b){if(0==b){var c=a.am.clone();return new Or(c)}return new Pr(a.oc)}function Nr(a){if(a.anchor)for(var b,c=0;b=Hr[c];c++)If(b.o(),a.anchor)&&(b.Pp=a,a.Cr=b)}
f.Oj=function(a){var b=Mr(this,a.target),c=Mr(this,a.relatedTarget);b!=c&&(b==this.oc&&(this.oc=null),Jr(this),this.Dk=!1,!this.za()||a.relatedTarget&&If(this.o(),a.relatedTarget)?this.anchor=void 0:this.Nk())};f.Gq=function(){var a=this.o();this.oc!=a&&(this.rh(),this.oc=a)};f.Fq=function(a){var b=this.o();this.oc!=b||a.relatedTarget&&If(b,a.relatedTarget)||(this.oc=null,this.Nk())};function Jr(a){a.af&&(ba.clearTimeout(a.af),a.af=void 0)}
f.Nk=function(){2==this.getState()&&(this.Sh=od(q(this.kv,this,this.anchor),this.Wu))};f.rh=function(){this.Sh&&(ba.clearTimeout(this.Sh),this.Sh=void 0)};f.V=function(){this.da(!1);Jr(this);this.detach();this.o()&&Ff(this.o());this.oc=null;delete this.R;Gr.m.V.call(this)};function Or(a,b){Cr.call(this,a,b)}r(Or,Cr);Or.prototype.uc=function(a,b,c){b=kg(a);b=ng(b);c=c?new Zf(c.top+10,c.right,c.bottom,c.left+10):new Zf(10,0,0,10);ik(this.Zl,a,8,c,b,9)&496&&ik(this.Zl,a,8,c,b,5)};
function Pr(a){kk.call(this,a,5)}r(Pr,kk);Pr.prototype.uc=function(a,b,c){var d=new df(10,0);gk(this.element,this.fq,a,b,d,c,9)&496&&gk(this.element,4,a,1,d,c,5)};function Qr(a){null==a&&(a={});this.Mj=q(this.Mj,this);this.Pj=q(this.Pj,this);this.l=a.l;this.Kk=!1;Qr.m.constructor.call(this,a)}r(Qr,Yn);Qr.prototype.Ga=function(a){this.element=a;v(a,"click",this.Pj);v(Dg(this.l,"body"),"click",this.Mj);v(a,"touchstart",this.Pj);v(Dg(this.l,"body"),"touchstart",this.Mj);return Qr.m.Ga.call(this,this.element)};Qr.prototype.Pj=function(a){a.stopPropagation();this.Kk=!this.Kk;return this.toggle()};Qr.prototype.Mj=function(){this.Kk=!1;return this.toggle()};
Qr.prototype.toggle=function(){return this.Kk?Zg(this.element,"open"):ah(this.element,"open")};function Rr(a){this.Db=null!=a?a:"?";this.query={};this.Xg(this.Db)}Rr.prototype.Xg=function(a){this.Db=a;this.Db=this.Db.replace("#","?");return this.uri=new xk(this.Db)};function Ap(a,b,c,d){null==d&&(d=!1);null==c||""===c&&!d?a.uri.removeParameter(b):Lk(a.uri,b,c)}Rr.prototype.toString=function(){return String(this.uri.Oc())};function Sr(a,b,c){this.prefix=a;this.Db=b;this.Yq=c}f=Sr.prototype;f.get=function(a){a=this.key(a);(a=this.Db.uri.mb.get(a))||(a="");return a};function $o(a){var b;var c=a.Db.uri.mb.rc();var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];var h=a.prefix;g.substring(0,h.length)===h&&d.push(g.replace(a.prefix+"-",""))}return d}f.ii=function(){var a;var b=$o(this);var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];c.push(this.remove(e))}return c};
f.set=function(a,b){Ap(this.Db,this.key(a),b);return this.Yq.hash=this.Db.toString()};f.remove=function(a){a=this.key(a);return this.Db.uri.removeParameter(a)};f.key=function(a){return[this.prefix,a].join("-")};function Tr(){}Tr.prototype.apply=function(){};function Ur(a){this.P=a||{cookie:""}}f=Ur.prototype;f.isEnabled=function(){return navigator.cookieEnabled};f.set=function(a,b,c,d,e,g){if(/[;=\s]/.test(a))throw Error('Invalid cookie name "'+a+'"');if(/[;\r\n]/.test(b))throw Error('Invalid cookie value "'+b+'"');ca(c)||(c=-1);e=e?";domain\x3d"+e:"";d=d?";path\x3d"+d:"";g=g?";secure":"";c=0>c?"":0==c?";expires\x3d"+(new Date(1970,1,1)).toUTCString():";expires\x3d"+(new Date(wa()+1E3*c)).toUTCString();this.P.cookie=a+"\x3d"+b+e+d+c+g};
f.get=function(a,b){for(var c=a+"\x3d",d=(this.P.cookie||"").split(";"),e=0,g;e<d.length;e++){g=Ea(d[e]);if(0==g.lastIndexOf(c,0))return g.substr(c.length);if(g==a)return""}return b};f.remove=function(a,b,c){var d=this.ae(a);this.set(a,"",0,b,c);return d};f.rc=function(){return Vr(this).keys};f.Gb=function(){return Vr(this).values};f.isEmpty=function(){return!this.P.cookie};f.xd=function(){return this.P.cookie?(this.P.cookie||"").split(";").length:0};f.ae=function(a){return ca(this.get(a))};
f.Ch=function(a){for(var b=Vr(this).values,c=0;c<b.length;c++)if(b[c]==a)return!0;return!1};f.clear=function(){for(var a=Vr(this).keys,b=a.length-1;0<=b;b--)this.remove(a[b])};function Vr(a){a=(a.P.cookie||"").split(";");for(var b=[],c=[],d,e,g=0;g<a.length;g++)e=Ea(a[g]),d=e.indexOf("\x3d"),-1==d?(b.push(""),c.push(e)):(b.push(e.substring(0,d)),c.push(e.substring(d+1)));return{keys:b,values:c}};function Wr(a){Wr.m.constructor.call(this);this.cq=new Ur(a);var b,c,d;var e=[document.body,document.getElementById("wrapper")];var g=[];var h=0;for(d=e.length;h<d;h++){a=e[h];var k;if(k=null!=a)k=Sh(a,"initial"),k=!(null==k||""===k);k&&g.push(a)}this.tu=0==g.length?null:null!=(b=qd(Sh(g[0],"initial")))?null!=(c=b.net)?c["csrf-token"]:void 0:void 0}r(Wr,Tr);
Wr.prototype.apply=function(a){var b;var c=[this.tu,this.cq.get("csrftoken")];var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];null!=g&&d.push(g)}0==d.length?(g=this.cq.get("Authorization"),null!=g&&(a.Authorization=g)):a["X-CsrfToken"]=d[0]};function Xr(a,b,c){this.Xb=a;this.l=b;this.nk=c}var Yr={Li:"snippet",oz:"snippetType",kl:"snippetLoader"},Zr={Li:"data-snippet",kl:"data-snippet-loader"};f=Xr.prototype;
f.apply=function(a){var b,c;if(a.snippets){var d={};var e=this.l.all("*["+Zr.kl+"]");var g=0;for(b=e.length;g<b;g++){var h=e[g];var k=String(Sh(h,Yr.kl)).split(" ");var l=0;for(c=k.length;l<c;l++){var m=k[l];d[m]=h}}k=a.snippets;g=[];for(m in k)h=k[m],null!=d[m]&&this.tc(d[m],h),a=Dg(this.l,"*["+Zr.Li+"\x3d"+m+"]"),null!=a?g.push(this.tc(a,h)):g.push(void 0);return g}};
f.tc=function(a,b){var c;var d=null!=(c=Sh(a,"snippetType"))?c:"replace";return function(){switch(d){case "replace":return this.replace;case "append":return this.append;case "prepend":return this.Lv}}.call(this).call(this,b,a)};f.replace=function(a,b){b.innerHTML=a;return this.Xb.tc(b)};f.append=function(a,b){var c=this.l.Iq(a);b.appendChild(c);return this.Xb.tc(c)};f.Lv=function(a,b){var c=this.l.Iq(a);Vf(b,c);return this.Xb.tc(c)};function $r(){}$r.prototype.apply=function(){};function as(a){this.Uc=a}r(as,$r);as.prototype.apply=function(a){if(401===ge(a.target))return this.Uc.location.reload(!0)};function bs(a,b){this.ka=a;this.Te=b;sp(this.Te,cs)}var cs={error:"Internal error occured. Service is unavailable, sorry.",loading:"Loading...",offline:"Offline..."};f=bs.prototype;
f.apply=function(a,b){var c;if(!Jb(this.Te,String(b))){null!=a.errors?(la(a.errors)||da(a.errors))&&Mh(this.ka,a.errors):null!=a.error&&Mh(this.ka,a.error);var d=null!=(c=a.messages)?c:a.message;if(null!=d){for(h in d)c=d[h],da(c)&&this.ka.ih(c,h);var e;if(d=null!=(e=d.errors)?e["non-field-errors"]:void 0){var g=[];var h=0;for(c=d.length;h<c;h++)e=d[h],g.push(Mh(this.ka,e));d=g}else d=void 0;return d}}};
f.error=function(a){if(Jb(this.Te,String(a))){if(a=String(a),null!=this.Te[a])return Mh(this.ka,this.Te[a])}else return Mh(this.ka,this.Te.error)};f.Pf=function(){return[ds(this,this.Te.loading,"loading",!1,!1)]};f.sv=function(){return[ds(this,this.Te.offline,"info",!0,!0)]};function ds(a,b,c,d,e){b=new Gh({l:a.ka.J(),kk:b,severity:"info",yj:d,pg:e});null!=c&&b.addClass(c);a.ka.$a(b);return b}f.$i=function(){return this.ka.$i()};function es(){this.log=function(){}};function fs(a){this.$o=null!=a?a:null;this.Pm=q(this.Pm,this);this.jg=q(this.jg,this);this.ki=new es;this.Xb=new xa(this.jg,this.ki);this.Kr=new xa(this.jg,this.ki);null==this.$o&&(this.$o=me);this.Db=this.Eh=this.Yb=null;this.nk={};this.Kc={};this.Uc=null}
function gs(a,b,c){var d=hs,e={Mr:!0};d.Uc=a;null==c&&(c={});null==e&&(e={});d.dateFormat=null;a=null!=d.Uc.console?q(d.Uc.console.error,d.Uc.console):function(){};d.ki.log=a;d.Eh=d.Uc.document;d.Db=new Rr(d.Uc.location.hash);d.nk={app:d,Db:d.Db,ka:b};null==d.dateFormat&&(d.dateFormat={gw:Qi,Hv:Oj,format:"dd'.'MM'.'yyyy"});Si=d.dateFormat.gw;Qj=d.dateFormat.Hv;var g=new Cg(d.Eh);var h=new Xr(d.Xb,g,d.nk);a=new Wr(d.Uc.document);b=new bs(b,c);d.Yb=new d.$o(b,a,h,g);null!=e.Mr&&e.Mr&&se(d.Yb,new as(d.Uc));
e=new gp;e.pa(!0);e.A("navigate",d.Pm);d.Kr.tc(d.Eh);return d.Xb.tc(d.Eh)}fs.prototype.jg=function(a,b){return a(b,new Cg(this.Eh),this.Yb,this.nk)};fs.prototype.Pm=function(){return this.Db.Xg(this.Uc.location.hash)};fs.prototype.on=function(a,b){return this.Xb.register(a,b)};fs.prototype.once=function(a,b){return this.Kr.register(a,b)};
function is(a){var b=hs;b.once(".flash",function(b){return a.l.en(b)?a.M(b):a.render(b)});b.on("table.js-grid",function(a,b,e,g){var c;a:{var d;var l=b.all("table.js-grid");var m=c=0;for(d=l.length;c<d;m=++c){var p=l[m];if(p===a){c=m;break a}}c=-1}p=g.app.Xb;m=g.app;c="g-"+c;Jb(m.Kc,c)||(m.Kc[c]=new Sr(c,m.Db,m.Uc.location));m=m.Kc[c];var u;null==u&&(u=new Rp);g=g.ka;var A=xo,B,E,J;null==u&&(u=new Rp);l=new Oo;c=l.ug(a,"context");c=new pl(e,c);var L=new Ho(a);var Q=new Jo(c);d=new ll(l.ug(a,"resource"));
null==B&&(B=!1);var ia=L.lo&&!B?L.lo:L.lo=Io(L,"restFields");var na=0;for(J=ia.length;na<J;na++)B=ia[na],d.wf.push(B);0<d.wf.length&&(c.headers["X-Fields"]=d.wf.join(","));B=d.nw();c.headers["X-Serialization-Format"]=B;c.headers["X-Fields"]=d.wf.join(",");J=new wp(m,a.id);na=new En({base:J.Dj(),page:J.Am()});B=new No(void 0,void 0,b);null==E&&(E=!1);E=L.cols&&!E?L.cols:L.cols=Io(L,"cols");E=new A(b,Q,E,J,B,d,na,g,"true"===L.ug("rowSelectable"));B=new Yo(E,d,m,b);E.bs(B.Xo(a));E.M(a);if(l=l.ug(a,"columnsManager"))if(l=
b.o(l))A=new Ep(b,B,E),A.M(l);J.handle(na,B);J.ti(na.Dj());(new Po(E)).M(Yf(a));l=Sh(a,"mode");null!=l&&"pane"===l&&(ob(d.wf,"_default_action"),m=new bp(c,b,p,m,d),m.Xo(E));Th(a,"updateUrl")?(c=String(Sh(a,"updateUrl")),m=Lm(String(Sh(a,"updateInterval")),3E3),d=new je({Pf:!1}),m=new Bp(E,new pl(e,"",d),c,m),m.start()):Th(a,"interval")&&Sp(E);u=u.rq;null==u&&(u=Dg(b,".export-buttons"));null==u&&(u=Yf(a));c=Fg(b,"export",u);d=0;for(l=c.length;d<l;d++)u=c[d],m=new Cp({l:b,ye:B,client:new pl(e)}),m.M(u);
Th(a,"bulkChange")&&(a=b.o(String(Sh(a,"bulkChange"))),null!=a&&(b=new Lp(b,new pl(e),E,p,g),b.M(a)));return E});b.on(".remote-button",function(a,b,e){b=new Yp(b,e);b.M(a);return b});b.on("form.ajax",function(a,b,e){e=new pl(e);b=new Wp(e,b);b.Zi=!0;return b.M(a)});b.on("fieldset.inline-js",function(a,b,e,g){b=new Br(b,g.app.Xb);e=!dh(a,"disabled");var c,d;null==e&&(e=!0);b.zc=Eg(b.l,"dynamic",a);if(null!=b.zc){b.qg=new Ar(a);b.qg.load();b.vd=b.qg.vd;b.fh=b.qg.fh;var l=new sr(parseInt(b.vd,10));g=
Fg(b.l,"inline-line",a);a=g[g.length-1];b.qd=new wr(a,l,b.Xb,b.l);if(1<g.length)for(l=c=0,d=g.length-2;0<=d?c<=d:c>=d;l=0<=d?++c:--c)yr(b.qd,g[l],!1);e?(b.listener=v(b.zc,"click",b.sc),b.qd.A("delete",b.Wm)):b.qd.pa(!1);b.l.removeNode(a);b=w(b.zc,e)}else b=void 0;return b});b.on("select.fulltext-search",function(a,b){return lr(a,b)});b.on("select.fulltext-search-multiple",function(a,b){return nr(a,b)});b.on("*[data-title]",function(a,b){var c=Sh(a,"title");return new Gr(a,c,b)});b.on(".navbar-toggle",
function(a,b){return wk(a,b)});b.on("*[data-snippet-onload]",function(a,b,e){e=(a=String(Sh(a,"snippetOnload")))?e.fc(new je).send(a,"GET","",{}):void 0;return e});b.on(".related-button",function(a,b,e,g){g=g.app.Xb;e=new pl(e);return(new qr(b,e,g)).M(a)});b.on(".dropdown",function(a,b){return(new Qr({l:b})).M(a)});b.on("input[type\x3ddatetime], input.datetime, input.date",function(a,b,e,g){return(new vk(b,g.app.dateFormat.format)).M(a)})};var js;function ks(){return js=ks.m.constructor.apply(this,arguments)}r(ks,me);ks.prototype.Ol=function(a){a=ks.m.Ol.call(this,a);a.A(le,function(a){return window.__dispatch(ie(a.target)["X-Authorization-Expiration"])});return a};var hs=new fs(ks);function ls(a,b){this.l=a;this.duration=null!=b?b:ms;this.Bj=q(this.Bj,this);this.toggle=q(this.toggle,this)}var ns={Fw:"dropup-button",Qs:"caret-marker",ep:"fa-angle-up",gl:"fa-angle-left",gp:"collapse",OPENED:"opened",CLOSED:"closed"},ms=500;f=ls.prototype;
f.M=function(a){this.rj=a;this.Wi=Eg(this.l,ns.Qs,this.rj);a=Sh(this.rj,"target");if(null==a)throw Error("Dropup button element has no 'target' attribute!");this.Bc=Dg(this.l,"#"+a+", ."+a,this.l.P);if(null==this.Bc)console.warn("Dropup element with specified id or class  '"+a+"', does not exist!");else return w(this.Bc,!1),ag(this.Bc,"overflow","hidden"),this.Fl=!1,v(this.rj,"click",this.toggle)};f.toggle=function(a){a.preventDefault();return"none"==this.Bc.style.display||this.Fl?this.open():this.close()};
f.open=function(){var a=this;null==this.height&&(this.height=this.zq());w(this.Bc,!0);var b=os(this,0,this.height);b.A("begin",function(){var b;var d=a.Bj();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(ch(h,ns.CLOSED,ns.OPENED))}return e});b.A("finish",function(){ah(a.Bc,ns.gp);return a.Sk()});return b.play()};
f.close=function(){var a=this;var b=os(this,this.height,0);b.A("finish",function(){var b;var d=a.Bj();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(ch(h,ns.OPENED,ns.CLOSED))}return e});b.A("finish",function(){w(a.Bc,!1);Zg(a.Bc,ns.gp);return a.Sk()});return b.play()};f.Bj=function(){return[this.Bc,this.rj]};f.Sk=function(){if(null!=this.Wi)return dh(this.Wi,ns.gl)?ch(this.Wi,ns.gl,ns.ep):ch(this.Wi,ns.ep,ns.gl)};
function os(a,b,c){b=new Pg([b],[c],a.duration);setTimeout(function(){a.Bc.style.height=jg(c,!0)},a.duration);b.A("animate",function(b){a.Bc.style.height=jg(b.coords[0],!0)});b.A("begin",function(){return a.Fl=!0});b.A("finish",function(){return a.Fl=!1});return b}f.zq=function(){w(this.Bc,!0);var a=tg(this.Bc).height;w(this.Bc,!1);return a};function ps(a){this.Im=q(this.Im,this);ps.m.constructor.call(this,a);this.ap=a.ap;this.ka=a.ka;this.$l=this.bp=null}r(ps,xh);ps.prototype.aa=function(a){return"select"===a.tagName.toLowerCase()?Rf(a.options[a.selectedIndex]):Cm(a)||Rf(a)};ps.prototype.M=function(a){var b=this;this.H=a;this.$l=Sh(this.H,"copiedMessage");this.bp=new this.ap(this.H);this.ms=this.l.o(String(Sh(this.H,"copyTarget")));if(null!=this.ms)return this.bp.on("ready",function(){return b.bp.on("copy",b.Im)})};
ps.prototype.Im=function(a){a.clipboardData.setData("text/plain",this.aa(this.ms));if(this.$l)return this.ka.ih(this.$l,"success",void 0,!0,1E3)};hs.on("input[type\x3ddatetime], input.datetime, input.date",function(a,b){Si=Qi;Qj=Oj;return(new vk(b,"dd'.'MM'.'yyyy")).M(a)});hs.on(".hint",function(a){return v(a,"click",function(a){return a.preventDefault()})});hs.on(".copy-button-js",function(a,b,c,d){return(new ps({ap:ne(b).ZeroClipboard,l:b,ka:d.ka})).M(a)});hs.on(".dropup",function(a,b){var c;null==c&&(c=void 0);null==c&&(c=Lm(String(Sh(a,"duration")),250));return(new ls(b,c)).M(a)});hs.on(".dropdown",function(a,b){return(new Qr({l:b})).M(a)});function qs(a,b){return v(a,"submit",function(){return b.onbeforeunload=null})}hs.on("form",function(a,b){Kp(a,b);return qs(a,ne(b))});hs.on(".modal-form",function(a,b,c,d){return rr(a,b,c,d.app.Xb)});function rs(a,b){this.uk=q(this.uk,this);rs.m.constructor.call(this,a,b)}r(rs,Rj);rs.prototype.kp={Et:"Dnes",NONE:"Smazat"};
rs.prototype.uk=function(a,b){var c=this.J().createElement("TD");c.colSpan=null!=b?b:{2:3};c.className=this.W()+"-today-cont";Sj(this,c,this.kp.Et,this.W()+"-today-btn");a.appendChild(c);c=this.J().createElement("TD");c.colSpan=null!=b?b:{4:3};a.appendChild(c);c=this.J().createElement("TD");c.colSpan=2;c.className=this.W()+"-none-cont";Sj(this,c,this.kp.NONE,this.W()+"-none-btn");return a.appendChild(c)};function ss(a,b,c){var d,e,g,h;var k=d=null;var l=h=g=e=0;var m=c?".datetime-range-to":".date-range-to";c=c?"dd'.'MM'.'yyyy' 'HH':'mm":"dd'.'MM'.'yyyy";var p=new mj(c);var u=new tj(c);var A=Yf(a).querySelector(m);if(null==A)ne(b).console.warn("Missing "+m+" for input "+a.name);else{var B=function(a){return kl(Cm(a))?null:Eb(Cm(a))};var E=function(a){Zg(a,"open")};var J=function(a){ah(a,"open")};var L=function(a){ah(a,"hidden")};var Q=function(a){return Zg(a,"hidden")};var ia=function(a,b){J(a);return Q(b)};
var na=function(a){a=new qk(a,null,b,new rs("goog-date-picker",b));a.Jk=!0;a.wj&&Uj(a);a.kh=!0;a.vj&&Uj(a);a.yi=!0;Zj(a);bk(a);Vj(a);return a};var Ya=function(a){return null!=a?p.format(a):""};var Oc=function(){return null!=d||null!=k?""+Ya(d)+" - "+Ya(k):""};var W=function(a){return b.Sc(a,Oc())};var Vi=function(a){var b;ah(a,"disabled");var c=a.querySelectorAll("select");var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.removeAttribute("disabled"))};var fb=function(a){var b;Zg(a,"disabled");
var c=a.querySelectorAll("select");var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.setAttribute("disabled","disabled"))};var pb=function(a,b,c){null!=c.date?(null==d&&(d=new hj,d.setHours(e),d.setMinutes(g)),d.setYear(c.date.getYear()),d.setMonth(c.date.getMonth()),d.setDate(c.date.getDate()),Vi(b)):(d=null,fb(b));return W(a)};var vb=function(a,b,c){null!=c.date?(null==k&&(k=new hj,k.setHours(h),k.setMinutes(l)),k.setYear(c.date.getYear()),k.setMonth(c.date.getMonth()),k.setDate(c.date.getDate()),
Vi(b)):(k=null,fb(b));return W(a)};var wb=function(a,b,c){b(Lm(c.target.value));return W(a)};var Eb=function(a){var b=new hj;wj(u,a,b,0,!0);kl(a)&&(b.setHours(0),b.setMinutes(0));return b};var Tb=function(a,c,d){var e=new Vo({options:hb(yb(0,24),qz)});e.D();e.ra(d.getHours());v(e.o(),"change",a);a=new Vo({options:hb(yb(0,60),qz)});a.D();a.ra(d.getMinutes());v(a.o(),"change",c);return b.H("div","time-picker",[e.o(),b.H("div","separator",":"),a.o()])};var Ub=function(a,b,c,d,e,g,h,k){var l=Eb(Cm(c));
c=na(l);c.render(a.children[d]);e(l);e=Tb(va(wb,b,g),va(wb,b,h),l);a.children[d].appendChild(e);v(c,"select",va(k,b,e));return c};var lc=function(b,c,m,p,u){u.preventDefault();E(b);L(m);if(null==b.querySelector(".goog-date-picker"))return u=Ub(c,p,a,0,function(a){return d=a},function(a){d.setHours(a);return e=a},function(a){d.setMinutes(a);return g=a},pb),p=Ub(c,p,A,1,function(a){return k=a},function(a){k.setHours(a);return h=a},function(a){k.setMinutes(a);return l=a},vb),c.appendChild(JG(u,p)),c.appendChild(KG(b,
m))};var Dc=function(a,b,c){c.preventDefault();return ia(a,b)};var Pc=function(a,b,c,d){d.preventDefault();c=c();a.setDate(c.getStartDate());return b.setDate(c.Ge)};var gd=function(b,c,e){e.preventDefault();ia(b,c);b=Cm(a)!==Ya(d);Dm(a,Ya(d));Dm(A,Ya(k));return pk(b?a:A)};var Gd=function(a,b,c){c.preventDefault();return ia(a,b)};var be=function(a,b,c){c.preventDefault();a.setDate(null);return b.setDate(null)};var De=function(){return b.H("button","btn btn-primary btn-xs",[b.H("i","glyphicon glyphicon-calendar fa fa-calendar"),
b.H("span","label",Oc()),b.H("span","caret")])};var hf=function(){return b.H("div","popup dropdown-menu",[b.H("div","date-picker-wrapper date-picker-wrapper__from"),b.H("div","date-picker-wrapper date-picker-wrapper__to")])};var gg=function(){return b.H("div","overlay")};var Qh=function(a,c){var d=b.H("button","btn btn-default",a);v(d,"click",c);return d};var hg=function(a,b){var c=a.clone();c.add(new Wi(Zi,b));return c};var Kl=function(a){var b=Kj().Ge;var c=hg(b,-1);b.add(new Wi(Zi,-1));return new Cj(hg(b,
-1*a),c)};m=function(a){a=a();var b=a.Ho;var c=a.Ge;return function(){return new Cj(b,hg(c,1))}};var Ll=[["Dnes",m(Kj)],["Včera",m(Ij)],["Posledních 7 dní",m(Lj)],["Posledních 30 dní",m(function(){return Kl(30)})],["Minulý týden",m(Nj)],["Minulý měsíc",m(Mj)]];var Ml=function(a,c,d){a=b.H("div",c,a);v(a,"click",d);return a};var JG=function(a,c){var d,e;return b.H("div","filter-buttons",[b.H("div","btn-group-vertical",rb(function(){var b;var g=[];var h=0;for(b=Ll.length;h<b;h++){var k=Ll[h];d=k[0];
e=k[1];g.push(Qh(d,va(Pc,a,c,e)))}return g}(),[Ml("Odstranit","btn btn-default",va(be,a,c))]))])};var KG=function(a,c){return b.H("div","action-buttons",[b.H("div","btn-group",[Ml("Filtrovat","btn btn-primary",va(gd,a,c)),Ml("Zavřít","btn btn-default",va(Gd,a,c))])])};var qz=function(a){return 10>a?wm("0%s",a):String(a)};return function(){d=B(a);k=B(A);var c=De();var e=hf();var g=gg();Q(g);var h=b.H("div",{"class":"date-range-wrapper"},[c,e,g]);Yf(a).appendChild(h);v(c,"click",va(lc,h,e,g,c.querySelector(".label")));
v(g,"click",va(Dc,h,g));Q(a);return Q(A)}()}};fa("app.start",function(a,b){var c=new Lh({l:new Cg(a.document)});is(c);hs.on("input.date-range-from",function(a,b){return ss(a,b,!1)});hs.on("input.datetime-range-from",function(a,b){return ss(a,b,!0)});return gs(a,c,b)});function ts(a,b){this.Ia=[];this.Tc=b;for(var c=!0,d=a.length-1;0<=d;d--){var e=a[d]|0;c&&e==b||(this.Ia[d]=e,c=!1)}}var us={};function vs(a){if(-128<=a&&128>a){var b=us[a];if(b)return b}b=new ts([a|0],0>a?-1:0);-128<=a&&128>a&&(us[a]=b);return b}function ws(a){if(isNaN(a)||!isFinite(a))return xs;if(0>a)return ws(-a).qa();for(var b=[],c=1,d=0;a>=c;d++)b[d]=a/c|0,c*=ys;return new ts(b,0)}var ys=4294967296,xs=vs(0),zs=vs(1),As=vs(16777216);f=ts.prototype;
f.Ei=function(){return 0<this.Ia.length?this.Ia[0]:this.Tc};f.Ed=function(){if(this.Pa())return-this.qa().Ed();for(var a=0,b=1,c=0;c<this.Ia.length;c++)var d=Bs(this,c),a=a+(0<=d?d:ys+d)*b,b=b*ys;return a};
f.toString=function(a){a=a||10;if(2>a||36<a)throw Error("radix out of range: "+a);if(this.Ub())return"0";if(this.Pa())return"-"+this.qa().toString(a);for(var b=ws(Math.pow(a,6)),c=this,d="";;){var e=Cs(c,b),g=(c.subtract(e.multiply(b)).Ei()>>>0).toString(a),c=e;if(c.Ub())return g+d;for(;6>g.length;)g="0"+g;d=""+g+d}};function Bs(a,b){return 0>b?0:b<a.Ia.length?a.Ia[b]:a.Tc}f.Ub=function(){if(0!=this.Tc)return!1;for(var a=0;a<this.Ia.length;a++)if(0!=this.Ia[a])return!1;return!0};
f.Pa=function(){return-1==this.Tc};f.un=function(){return 0==this.Ia.length&&-1==this.Tc||0<this.Ia.length&&0!=(this.Ia[0]&1)};f.pb=function(a){if(this.Tc!=a.Tc)return!1;for(var b=Math.max(this.Ia.length,a.Ia.length),c=0;c<b;c++)if(Bs(this,c)!=Bs(a,c))return!1;return!0};f.Dm=function(a){return 0<this.compare(a)};f.Em=function(a){return 0<=this.compare(a)};f.ai=function(a){return 0>this.compare(a)};f.Gn=function(a){return 0>=this.compare(a)};
f.compare=function(a){a=this.subtract(a);return a.Pa()?-1:a.Ub()?0:1};f.qa=function(){return this.Qn().add(zs)};f.add=function(a){for(var b=Math.max(this.Ia.length,a.Ia.length),c=[],d=0,e=0;e<=b;e++){var g=d+(Bs(this,e)&65535)+(Bs(a,e)&65535),h=(g>>>16)+(Bs(this,e)>>>16)+(Bs(a,e)>>>16),d=h>>>16,g=g&65535,h=h&65535;c[e]=h<<16|g}return new ts(c,c[c.length-1]&-2147483648?-1:0)};f.subtract=function(a){return this.add(a.qa())};
f.multiply=function(a){if(this.Ub()||a.Ub())return xs;if(this.Pa())return a.Pa()?this.qa().multiply(a.qa()):this.qa().multiply(a).qa();if(a.Pa())return this.multiply(a.qa()).qa();if(this.ai(As)&&a.ai(As))return ws(this.Ed()*a.Ed());for(var b=this.Ia.length+a.Ia.length,c=[],d=0;d<2*b;d++)c[d]=0;for(d=0;d<this.Ia.length;d++)for(var e=0;e<a.Ia.length;e++){var g=Bs(this,d)>>>16,h=Bs(this,d)&65535,k=Bs(a,e)>>>16,l=Bs(a,e)&65535;c[2*d+2*e]+=h*l;Ds(c,2*d+2*e);c[2*d+2*e+1]+=g*l;Ds(c,2*d+2*e+1);c[2*d+2*e+
1]+=h*k;Ds(c,2*d+2*e+1);c[2*d+2*e+2]+=g*k;Ds(c,2*d+2*e+2)}for(d=0;d<b;d++)c[d]=c[2*d+1]<<16|c[2*d];for(d=b;d<2*b;d++)c[d]=0;return new ts(c,0)};function Ds(a,b){for(;(a[b]&65535)!=a[b];)a[b+1]+=a[b]>>>16,a[b]&=65535,b++}
function Cs(a,b){if(b.Ub())throw Error("division by zero");if(a.Ub())return xs;if(a.Pa())return b.Pa()?Cs(a.qa(),b.qa()):Cs(a.qa(),b).qa();if(b.Pa())return Cs(a,b.qa()).qa();if(30<a.Ia.length){if(a.Pa()||b.Pa())throw Error("slowDivide_ only works with positive integers.");for(var c=zs,d=b;d.Gn(a);)c=c.shiftLeft(1),d=d.shiftLeft(1);for(var e=c.$e(1),g=d.$e(1),h,d=d.$e(2),c=c.$e(2);!d.Ub();)h=g.add(d),h.Gn(a)&&(e=e.add(c),g=h),d=d.$e(1),c=c.$e(1);return e}c=xs;for(d=a;d.Em(b);){e=Math.max(1,Math.floor(d.Ed()/
b.Ed()));g=Math.ceil(Math.log(e)/Math.LN2);g=48>=g?1:Math.pow(2,g-48);h=ws(e);for(var k=h.multiply(b);k.Pa()||k.Dm(d);)e-=g,h=ws(e),k=h.multiply(b);h.Ub()&&(h=zs);c=c.add(h);d=d.subtract(k)}return c}f.Qn=function(){for(var a=this.Ia.length,b=[],c=0;c<a;c++)b[c]=~this.Ia[c];return new ts(b,~this.Tc)};f.Pt=function(a){for(var b=Math.max(this.Ia.length,a.Ia.length),c=[],d=0;d<b;d++)c[d]=Bs(this,d)&Bs(a,d);return new ts(c,this.Tc&a.Tc)};
f.shiftLeft=function(a){var b=a>>5;a%=32;for(var c=this.Ia.length+b+(0<a?1:0),d=[],e=0;e<c;e++)d[e]=0<a?Bs(this,e-b)<<a|Bs(this,e-b-1)>>>32-a:Bs(this,e-b);return new ts(d,this.Tc)};f.$e=function(a){var b=a>>5;a%=32;for(var c=this.Ia.length-b,d=[],e=0;e<c;e++)d[e]=0<a?Bs(this,e+b)>>>a|Bs(this,e+b+1)<<32-a:Bs(this,e+b);return new ts(d,this.Tc)};function Es(a,b){null!=a&&this.append.apply(this,arguments)}f=Es.prototype;f.Be="";f.set=function(a){this.Be=""+a};f.append=function(a,b,c){this.Be+=String(a);if(null!=b)for(var d=1;d<arguments.length;d++)this.Be+=arguments[d];return this};f.clear=function(){this.Be=""};f.toString=function(){return this.Be};function Fs(a,b){this.ib=a|0;this.Tb=b|0}var Gs={},Hs={};function Is(a){return fc(Gs,a,function(a){return new Fs(a,0>a?-1:0)})}function Js(a){a|=0;return-128<=a&&128>a?Is(a):new Fs(a,0>a?-1:0)}function Ks(a){return isNaN(a)?Is(0):a<=-Ls?Ms():a+1>=Ls?Ns():0>a?Ks(-a).qa():new Fs(a%Os|0,a/Os|0)}function Ps(a,b){return new Fs(a,b)}
function Qs(a,b){if(0==a.length)throw Error("number format error: empty string");var c=b||10;if(2>c||36<c)throw Error("radix out of range: "+c);if("-"==a.charAt(0))return Qs(a.substring(1),c).qa();if(0<=a.indexOf("-"))throw Error('number format error: interior "-" character: '+a);for(var d=Ks(Math.pow(c,8)),e=Is(0),g=0;g<a.length;g+=8){var h=Math.min(8,a.length-g),k=parseInt(a.substring(g,g+h),c);8>h?(h=Ks(Math.pow(c,h)),e=e.multiply(h).add(Ks(k))):(e=e.multiply(d),e=e.add(Ks(k)))}return e}
var Os=4294967296,Ls=Os*Os/2;function Ns(){return fc(Hs,Rs,function(){return Ps(-1,2147483647)})}function Ms(){return fc(Hs,Ss,function(){return Ps(0,-2147483648)})}function Ts(){return fc(Hs,Us,function(){return Js(16777216)})}f=Fs.prototype;f.Ei=function(){return this.ib};f.Ed=function(){return this.Tb*Os+(0<=this.ib?this.ib:Os+this.ib)};
f.toString=function(a){a=a||10;if(2>a||36<a)throw Error("radix out of range: "+a);if(this.Ub())return"0";if(this.Pa()){if(this.pb(Ms())){var b=Ks(a);var c=this.div(b);b=c.multiply(b).subtract(this);return c.toString(a)+b.Ei().toString(a)}return"-"+this.qa().toString(a)}c=Ks(Math.pow(a,6));b=this;for(var d="";;){var e=b.div(c),g=(b.subtract(e.multiply(c)).Ei()>>>0).toString(a);b=e;if(b.Ub())return g+d;for(;6>g.length;)g="0"+g;d=""+g+d}};f.Ub=function(){return 0==this.Tb&&0==this.ib};
f.Pa=function(){return 0>this.Tb};f.un=function(){return 1==(this.ib&1)};f.pb=function(a){return this.Tb==a.Tb&&this.ib==a.ib};f.ai=function(a){return 0>this.compare(a)};f.Gn=function(a){return 0>=this.compare(a)};f.Dm=function(a){return 0<this.compare(a)};f.Em=function(a){return 0<=this.compare(a)};f.compare=function(a){if(this.pb(a))return 0;var b=this.Pa(),c=a.Pa();return b&&!c?-1:!b&&c?1:this.subtract(a).Pa()?-1:1};f.qa=function(){return this.pb(Ms())?Ms():this.Qn().add(Is(1))};
f.add=function(a){var b=this.Tb>>>16,c=this.Tb&65535,d=this.ib>>>16,e=a.Tb>>>16,g=a.Tb&65535,h=a.ib>>>16;a=0+((this.ib&65535)+(a.ib&65535));h=0+(a>>>16)+(d+h);d=0+(h>>>16);d+=c+g;b=0+(d>>>16)+(b+e)&65535;return Ps((h&65535)<<16|a&65535,b<<16|d&65535)};f.subtract=function(a){return this.add(a.qa())};
f.multiply=function(a){if(this.Ub()||a.Ub())return Is(0);if(this.pb(Ms()))return a.un()?Ms():Is(0);if(a.pb(Ms()))return this.un()?Ms():Is(0);if(this.Pa())return a.Pa()?this.qa().multiply(a.qa()):this.qa().multiply(a).qa();if(a.Pa())return this.multiply(a.qa()).qa();if(this.ai(Ts())&&a.ai(Ts()))return Ks(this.Ed()*a.Ed());var b=this.Tb>>>16,c=this.Tb&65535,d=this.ib>>>16,e=this.ib&65535,g=a.Tb>>>16,h=a.Tb&65535,k=a.ib>>>16;a=a.ib&65535;var l=0+e*a;var m=0+(l>>>16)+d*a;var p=0+(m>>>16);m=(m&65535)+
e*k;p+=m>>>16;p+=c*a;var u=0+(p>>>16);p=(p&65535)+d*k;u+=p>>>16;p=(p&65535)+e*h;u=u+(p>>>16)+(b*a+c*k+d*h+e*g)&65535;return Ps((m&65535)<<16|l&65535,u<<16|p&65535)};
f.div=function(a){if(a.Ub())throw Error("division by zero");if(this.Ub())return Is(0);if(this.pb(Ms())){if(a.pb(Is(1))||a.pb(Is(-1)))return Ms();if(a.pb(Ms()))return Is(1);var b=this.$e(1).div(a).shiftLeft(1);if(b.pb(Is(0)))return a.Pa()?Is(1):Is(-1);var c=this.subtract(a.multiply(b));return b.add(c.div(a))}if(a.pb(Ms()))return Is(0);if(this.Pa())return a.Pa()?this.qa().div(a.qa()):this.qa().div(a).qa();if(a.Pa())return this.div(a.qa()).qa();var d=Is(0);for(c=this;c.Em(a);){b=Math.max(1,Math.floor(c.Ed()/
a.Ed()));for(var e=Math.ceil(Math.log(b)/Math.LN2),e=48>=e?1:Math.pow(2,e-48),g=Ks(b),h=g.multiply(a);h.Pa()||h.Dm(c);)b-=e,g=Ks(b),h=g.multiply(a);g.Ub()&&(g=Is(1));d=d.add(g);c=c.subtract(h)}return d};f.Qn=function(){return Ps(~this.ib,~this.Tb)};f.Pt=function(a){return Ps(this.ib&a.ib,this.Tb&a.Tb)};f.shiftLeft=function(a){a&=63;if(0==a)return this;var b=this.ib;return 32>a?Ps(b<<a,this.Tb<<a|b>>>32-a):Ps(0,b<<a-32)};
f.$e=function(a){a&=63;if(0==a)return this;var b=this.Tb;return 32>a?Ps(this.ib>>>a|b<<32-a,b>>a):Ps(b>>a-32,0<=b?0:-1)};var Rs=1,Ss=2,Us=6;var Vs;if("undefined"===typeof x)var x={};if("undefined"===typeof Ws)var Ws=function(){throw Error("No *print-fn* fn set for evaluation environment");};if("undefined"===typeof Xs)var Xs=function(){throw Error("No *print-err-fn* fn set for evaluation environment");};var Ys=null;if("undefined"===typeof Zs)var Zs=null;function $s(){return new y(null,5,[at,!0,bt,!0,ct,!1,dt,!1,et,null],null)}function z(a){return null!=a&&!1!==a}function ft(a){return null==a}function gt(a){return a instanceof Array}
function ht(a){return null==a?!0:!1===a?!0:!1}function C(a,b){return a[n(null==b?null:b)]?!0:a._?!0:!1}function it(a,b){var c=null==b?null:b.constructor,c=z(z(c)?c.Zp:c)?c.jj:n(b);return Error(["No protocol method ",a," defined for type ",c,": ",b].join(""))}function jt(a){var b=a.jj;return z(b)?b:""+D.c(a)}var kt="undefined"!==typeof Symbol&&"function"===n(Symbol)?Symbol.iterator:"@@iterator";function lt(a){for(var b=a.length,c=Array(b),d=0;;)if(d<b)c[d]=a[d],d+=1;else break;return c}
function mt(a){function b(a,b){a.push(b);return a}var c=[];return nt?nt(b,c,a):ot.call(null,b,c,a)}function pt(){}function qt(){}function rt(){}
var st=function st(b){if(null!=b&&null!=b.xa)return b.xa(b);var c=st[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=st._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("ICounted.-count",b);},tt=function tt(b){if(null!=b&&null!=b.Ja)return b.Ja(b);var c=tt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=tt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IEmptyableCollection.-empty",b);};function ut(){}
var vt=function vt(b,c){if(null!=b&&null!=b.wa)return b.wa(b,c);var d=vt[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=vt._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("ICollection.-conj",b);};function wt(){}
var xt=function xt(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return xt.h(arguments[0],arguments[1]);case 3:return xt.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
xt.h=function(a,b){if(null!=a&&null!=a.$)return a.$(a,b);var c=xt[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=xt._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw it("IIndexed.-nth",a);};xt.j=function(a,b,c){if(null!=a&&null!=a.Zb)return a.Zb(a,b,c);var d=xt[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=xt._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw it("IIndexed.-nth",a);};xt.I=3;function yt(){}
var zt=function zt(b){if(null!=b&&null!=b.ab)return b.ab(b);var c=zt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=zt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("ISeq.-first",b);},At=function At(b){if(null!=b&&null!=b.zb)return b.zb(b);var c=At[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=At._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("ISeq.-rest",b);};function Bt(){}function Ct(){}
var Dt=function Dt(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return Dt.h(arguments[0],arguments[1]);case 3:return Dt.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
Dt.h=function(a,b){if(null!=a&&null!=a.na)return a.na(a,b);var c=Dt[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=Dt._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw it("ILookup.-lookup",a);};Dt.j=function(a,b,c){if(null!=a&&null!=a.X)return a.X(a,b,c);var d=Dt[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=Dt._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw it("ILookup.-lookup",a);};Dt.I=3;function Et(){}
var Ft=function Ft(b,c){if(null!=b&&null!=b.sh)return b.sh(b,c);var d=Ft[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Ft._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IAssociative.-contains-key?",b);},Gt=function Gt(b,c,d){if(null!=b&&null!=b.Ob)return b.Ob(b,c,d);var e=Gt[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Gt._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("IAssociative.-assoc",b);};function Ht(){}
var It=function It(b,c){if(null!=b&&null!=b.uh)return b.uh(b,c);var d=It[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=It._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IFind.-find",b);};function Jt(){}var Kt=function Kt(b,c){if(null!=b&&null!=b.lf)return b.lf(b,c);var d=Kt[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Kt._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IMap.-dissoc",b);};function Lt(){}
var Mt=function Mt(b){if(null!=b&&null!=b.Sl)return b.Sl();var c=Mt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Mt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IMapEntry.-key",b);},Nt=function Nt(b){if(null!=b&&null!=b.Tl)return b.Tl();var c=Nt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Nt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IMapEntry.-val",b);};function Ot(){}
var Pt=function Pt(b,c){if(null!=b&&null!=b.Ul)return b.Ul(b,c);var d=Pt[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Pt._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("ISet.-disjoin",b);},Qt=function Qt(b){if(null!=b&&null!=b.ng)return b.ng(b);var c=Qt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Qt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IStack.-peek",b);},Rt=function Rt(b){if(null!=b&&null!=b.og)return b.og(b);var c=Rt[n(null==
b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Rt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IStack.-pop",b);};function St(){}var Tt=function Tt(b,c,d){if(null!=b&&null!=b.wh)return b.wh(b,c,d);var e=Tt[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Tt._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("IVector.-assoc-n",b);};function Ut(){}
var Vt=function Vt(b){if(null!=b&&null!=b.Ce)return b.Ce(b);var c=Vt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Vt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IDeref.-deref",b);};function Wt(){}var Xt=function Xt(b){if(null!=b&&null!=b.sa)return b.sa(b);var c=Xt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Xt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IMeta.-meta",b);};function Yt(){}
var Zt=function Zt(b,c){if(null!=b&&null!=b.ya)return b.ya(b,c);var d=Zt[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Zt._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IWithMeta.-with-meta",b);};function $t(){}
var au=function au(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return au.h(arguments[0],arguments[1]);case 3:return au.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
au.h=function(a,b){if(null!=a&&null!=a.xb)return a.xb(a,b);var c=au[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=au._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw it("IReduce.-reduce",a);};au.j=function(a,b,c){if(null!=a&&null!=a.yb)return a.yb(a,b,c);var d=au[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=au._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw it("IReduce.-reduce",a);};au.I=3;
var bu=function bu(b,c,d){if(null!=b&&null!=b.lg)return b.lg(b,c,d);var e=bu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=bu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("IKVReduce.-kv-reduce",b);},cu=function cu(b,c){if(null!=b&&null!=b.O)return b.O(b,c);var d=cu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=cu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IEquiv.-equiv",b);},du=function du(b){if(null!=b&&null!=b.ha)return b.ha(b);
var c=du[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=du._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IHash.-hash",b);};function eu(){}var fu=function fu(b){if(null!=b&&null!=b.oa)return b.oa(b);var c=fu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=fu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("ISeqable.-seq",b);};function gu(){}function hu(){}function iu(){}function ju(){}
var ku=function ku(b){if(null!=b&&null!=b.mg)return b.mg(b);var c=ku[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=ku._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IReversible.-rseq",b);},lu=function lu(b,c){if(null!=b&&null!=b.Yp)return b.Yp(0,c);var d=lu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=lu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IWriter.-write",b);};function mu(){}
var nu=function nu(b,c,d){if(null!=b&&null!=b.yh)return b.yh(b,c,d);var e=nu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=nu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("IWatchable.-notify-watches",b);},ou=function ou(b,c,d){if(null!=b&&null!=b.xh)return b.xh(b,c,d);var e=ou[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=ou._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("IWatchable.-add-watch",b);},pu=function pu(b,
c){if(null!=b&&null!=b.zh)return b.zh(b,c);var d=pu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=pu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IWatchable.-remove-watch",b);},qu=function qu(b){if(null!=b&&null!=b.kg)return b.kg(b);var c=qu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=qu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IEditableCollection.-as-transient",b);},ru=function ru(b,c){if(null!=b&&null!=b.nf)return b.nf(b,
c);var d=ru[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=ru._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("ITransientCollection.-conj!",b);},su=function su(b){if(null!=b&&null!=b.vh)return b.vh(b);var c=su[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=su._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("ITransientCollection.-persistent!",b);},tu=function tu(b,c,d){if(null!=b&&null!=b.mf)return b.mf(b,c,d);var e=tu[n(null==b?null:b)];if(null!=
e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=tu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw it("ITransientAssociative.-assoc!",b);},uu=function uu(b){if(null!=b&&null!=b.Rp)return b.Rp();var c=uu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=uu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IChunk.-drop-first",b);},vu=function vu(b){if(null!=b&&null!=b.Rl)return b.Rl(b);var c=vu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=vu._;if(null!=
c)return c.c?c.c(b):c.call(null,b);throw it("IChunkedSeq.-chunked-first",b);},wu=function wu(b){if(null!=b&&null!=b.bj)return b.bj(b);var c=wu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=wu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IChunkedSeq.-chunked-rest",b);},xu=function xu(b,c){if(null!=b&&null!=b.jc)return b.jc(b,c);var d=xu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=xu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IReset.-reset!",
b);},yu=function yu(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return yu.h(arguments[0],arguments[1]);case 3:return yu.j(arguments[0],arguments[1],arguments[2]);case 4:return yu.L(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return yu.ba(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
yu.h=function(a,b){if(null!=a&&null!=a.fj)return a.fj(a,b);var c=yu[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=yu._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw it("ISwap.-swap!",a);};yu.j=function(a,b,c){if(null!=a&&null!=a.gj)return a.gj(a,b,c);var d=yu[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=yu._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw it("ISwap.-swap!",a);};
yu.L=function(a,b,c,d){if(null!=a&&null!=a.hj)return a.hj(a,b,c,d);var e=yu[n(null==a?null:a)];if(null!=e)return e.L?e.L(a,b,c,d):e.call(null,a,b,c,d);e=yu._;if(null!=e)return e.L?e.L(a,b,c,d):e.call(null,a,b,c,d);throw it("ISwap.-swap!",a);};
yu.ba=function(a,b,c,d,e){if(null!=a&&null!=a.ij)return a.ij(a,b,c,d,e);var g=yu[n(null==a?null:a)];if(null!=g)return g.ba?g.ba(a,b,c,d,e):g.call(null,a,b,c,d,e);g=yu._;if(null!=g)return g.ba?g.ba(a,b,c,d,e):g.call(null,a,b,c,d,e);throw it("ISwap.-swap!",a);};yu.I=5;var zu=function zu(b){if(null!=b&&null!=b.ic)return b.ic(b);var c=zu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=zu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IIterable.-iterator",b);};
function Au(a){this.Vv=a;this.C=1073741824;this.Y=0}Au.prototype.Yp=function(a,b){return this.Vv.append(b)};function Bu(a){var b=new Es;a.ja(null,new Au(b),$s());return""+D.c(b)}var Cu="undefined"!==typeof Math.imul&&0!==Math.imul(4294967295,5)?function(a,b){return Math.imul(a,b)}:function(a,b){var c=a&65535,d=b&65535;return c*d+((a>>>16&65535)*d+c*(b>>>16&65535)<<16>>>0)|0};function Du(a){a=Cu(a|0,-862048943);return Cu(a<<15|a>>>-15,461845907)}
function Eu(a,b){var c=(a|0)^(b|0);return Cu(c<<13|c>>>-13,5)+-430675100|0}function Fu(a,b){var c=(a|0)^b,c=Cu(c^c>>>16,-2048144789),c=Cu(c^c>>>13,-1028477387);return c^c>>>16}function Gu(a){a:{var b=1;for(var c=0;;)if(b<a.length){var d=b+2,c=Eu(c,Du(a.charCodeAt(b-1)|a.charCodeAt(b)<<16));b=d}else{b=c;break a}}b=1===(a.length&1)?b^Du(a.charCodeAt(a.length-1)):b;return Fu(b,Cu(2,a.length))}var Hu={},Iu=0;
function Ju(a){255<Iu&&(Hu={},Iu=0);if(null==a)return 0;var b=Hu[a];if("number"!==typeof b){a:if(null!=a)if(b=a.length,0<b)for(var c=0,d=0;;)if(c<b)var e=c+1,d=Cu(31,d)+a.charCodeAt(c),c=e;else{b=d;break a}else b=0;else b=0;Hu[a]=b;Iu+=1}return a=b}
function Ku(a){if(null!=a&&(a.C&4194304||x===a.gu))return a.ha(null)^0;if("number"===typeof a){if(z(isFinite(a)))return Math.floor(a)%2147483647;switch(a){case Infinity:return 2146435072;case -Infinity:return-1048576;default:return 2146959360}}else return!0===a?a=1231:!1===a?a=1237:"string"===typeof a?(a=Ju(a),0!==a&&(a=Du(a),a=Eu(0,a),a=Fu(a,4))):a=a instanceof Date?a.valueOf()^0:null==a?0:du(a)^0,a}function Lu(a,b){return a^b+2654435769+(a<<6)+(a>>2)}function Mu(a,b){return b instanceof a}
function Nu(a,b,c,d,e){this.ns=a;this.name=b;this.Dd=c;this.fg=d;this.dc=e;this.C=2154168321;this.Y=4096}f=Nu.prototype;f.toString=function(){return this.Dd};f.equiv=function(a){return this.O(null,a)};f.O=function(a,b){return b instanceof Nu?this.Dd===b.Dd:!1};
f.call=function(){function a(a,b,c){return F.j?F.j(b,this,c):F.call(null,b,this,c)}function b(a,b){return F.h?F.h(b,this):F.call(null,b,this)}var c=null,c=function(c,e,g){switch(arguments.length){case 2:return b.call(this,0,e);case 3:return a.call(this,0,e,g)}throw Error("Invalid arity: "+(arguments.length-1));};c.h=b;c.j=a;return c}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return F.h?F.h(a,this):F.call(null,a,this)};
f.h=function(a,b){return F.j?F.j(a,this,b):F.call(null,a,this,b)};f.sa=function(){return this.dc};f.ya=function(a,b){return new Nu(this.ns,this.name,this.Dd,this.fg,b)};f.ha=function(){var a=this.fg;return null!=a?a:this.fg=a=Lu(Gu(this.name),Ju(this.ns))};f.ja=function(a,b){return lu(b,this.Dd)};
var Ou=function Ou(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Ou.c(arguments[0]);case 2:return Ou.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};Ou.c=function(a){if(a instanceof Nu)return a;var b=a.indexOf("/");return 1>b?Ou.h(null,a):Ou.h(a.substring(0,b),a.substring(b+1,a.length))};
Ou.h=function(a,b){var c=null!=a?[D.c(a),D.c("/"),D.c(b)].join(""):b;return new Nu(a,b,c,null,null)};Ou.I=2;function G(a){if(null==a)return null;if(null!=a&&(a.C&8388608||x===a.lu))return a.oa(null);if(gt(a)||"string"===typeof a)return 0===a.length?null:new H(a,0,null);if(C(eu,a))return fu(a);throw Error([D.c(a),D.c(" is not ISeqable")].join(""));}function I(a){if(null==a)return null;if(null!=a&&(a.C&64||x===a.Pb))return a.ab(null);a=G(a);return null==a?null:zt(a)}
function Pu(a){return null!=a?null!=a&&(a.C&64||x===a.Pb)?a.zb(null):(a=G(a))?At(a):Qu:Qu}function K(a){return null==a?null:null!=a&&(a.C&128||x===a.ej)?a.$b(null):G(Pu(a))}var M=function M(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return M.c(arguments[0]);case 2:return M.h(arguments[0],arguments[1]);default:return M.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};M.c=function(){return!0};
M.h=function(a,b){return null==a?null==b:a===b||cu(a,b)};M.w=function(a,b,c){for(;;)if(M.h(a,b))if(K(c))a=b,b=I(c),c=K(c);else return M.h(b,I(c));else return!1};M.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return M.w(b,a,c)};M.I=2;function Ru(a){this.s=a}Ru.prototype.next=function(){if(null!=this.s){var a=I(this.s);this.s=K(this.s);return{value:a,done:!1}}return{value:null,done:!0}};function Su(a){return new Ru(G(a))}function Tu(a,b){var c=Du(a),c=Eu(0,c);return Fu(c,b)}
function Uu(a){var b=0,c=1;for(a=G(a);;)if(null!=a)b+=1,c=Cu(31,c)+Ku(I(a))|0,a=K(a);else return Tu(c,b)}var Vu=Tu(1,0);function Wu(a){var b=0,c=0;for(a=G(a);;)if(null!=a)b+=1,c=c+Ku(I(a))|0,a=K(a);else return Tu(c,b)}var Xu=Tu(0,0);rt["null"]=!0;st["null"]=function(){return 0};Date.prototype.O=function(a,b){return b instanceof Date&&this.valueOf()===b.valueOf()};cu.number=function(a,b){return a===b};pt["function"]=!0;Wt["function"]=!0;Xt["function"]=function(){return null};du._=function(a){return qa(a)};
function Yu(a){return a+1}function N(a){return Vt(a)}function Zu(a,b){var c=st(a);if(0===c)return b.T?b.T():b.call(null);for(var d=xt.h(a,0),e=1;;)if(e<c)var g=xt.h(a,e),d=b.h?b.h(d,g):b.call(null,d,g),e=e+1;else return d}function $u(a,b,c){var d=st(a),e=c;for(c=0;;)if(c<d){var g=xt.h(a,c),e=b.h?b.h(e,g):b.call(null,e,g);c+=1}else return e}
function av(a,b){var c=a.length;if(0===a.length)return b.T?b.T():b.call(null);for(var d=a[0],e=1;;)if(e<c)var g=a[e],d=b.h?b.h(d,g):b.call(null,d,g),e=e+1;else return d}function bv(a,b,c){var d=a.length,e=c;for(c=0;;)if(c<d){var g=a[c],e=b.h?b.h(e,g):b.call(null,e,g);c+=1}else return e}function cv(a,b,c,d){for(var e=a.length;;)if(d<e){var g=a[d];c=b.h?b.h(c,g):b.call(null,c,g);d+=1}else return c}function dv(a){return null!=a?a.C&2||x===a.au?!0:a.C?!1:C(rt,a):C(rt,a)}
function ev(a){return null!=a?a.C&16||x===a.Vp?!0:a.C?!1:C(wt,a):C(wt,a)}function O(a,b,c){var d=P.c?P.c(a):P.call(null,a);if(c>=d)return-1;!(0<c)&&0>c&&(c+=d,c=0>c?0:c);for(;;)if(c<d){if(M.h(fv?fv(a,c):gv.call(null,a,c),b))return c;c+=1}else return-1}function hv(a,b,c){var d=P.c?P.c(a):P.call(null,a);if(0===d)return-1;0<c?(--d,c=d<c?d:c):c=0>c?d+c:c;for(;;)if(0<=c){if(M.h(fv?fv(a,c):gv.call(null,a,c),b))return c;--c}else return-1}function iv(a,b){this.v=a;this.i=b}
iv.prototype.hb=function(){return this.i<this.v.length};iv.prototype.next=function(){var a=this.v[this.i];this.i+=1;return a};function H(a,b,c){this.v=a;this.i=b;this.meta=c;this.C=166592766;this.Y=8192}f=H.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P.c?P.c(this):P.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.$=function(a,b){var c=b+this.i;if(0<=c&&c<this.v.length)return this.v[c];throw Error("Index out of bounds");};f.Zb=function(a,b,c){a=b+this.i;return 0<=a&&a<this.v.length?this.v[a]:c};
f.ic=function(){return new iv(this.v,this.i)};f.sa=function(){return this.meta};f.$b=function(){return this.i+1<this.v.length?new H(this.v,this.i+1,null):null};f.xa=function(){var a=this.v.length-this.i;return 0>a?0:a};f.mg=function(){var a=this.xa(null);return 0<a?new jv(this,a-1,null):null};f.ha=function(){return Uu(this)};f.O=function(a,b){return kv.h?kv.h(this,b):kv.call(null,this,b)};f.Ja=function(){return Qu};f.xb=function(a,b){return cv(this.v,b,this.v[this.i],this.i+1)};
f.yb=function(a,b,c){return cv(this.v,b,c,this.i)};f.ab=function(){return this.v[this.i]};f.zb=function(){return this.i+1<this.v.length?new H(this.v,this.i+1,null):Qu};f.oa=function(){return this.i<this.v.length?this:null};f.ya=function(a,b){return new H(this.v,this.i,b)};f.wa=function(a,b){return lv.h?lv.h(b,this):lv.call(null,b,this)};H.prototype[kt]=function(){return Su(this)};function mv(a,b){return b<a.length?new H(a,b,null):null}
function R(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return mv(arguments[0],0);case 2:return mv(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}function jv(a,b,c){this.Xi=a;this.i=b;this.meta=c;this.C=32374990;this.Y=8192}f=jv.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P.c?P.c(this):P.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){return 0<this.i?new jv(this.Xi,this.i-1,null):null};f.xa=function(){return this.i+1};f.ha=function(){return Uu(this)};
f.O=function(a,b){return kv.h?kv.h(this,b):kv.call(null,this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv?nv(b,this):ov.call(null,b,this)};f.yb=function(a,b,c){return pv?pv(b,c,this):ov.call(null,b,c,this)};f.ab=function(){return xt.h(this.Xi,this.i)};f.zb=function(){return 0<this.i?new jv(this.Xi,this.i-1,null):Qu};f.oa=function(){return this};f.ya=function(a,b){return new jv(this.Xi,this.i,b)};f.wa=function(a,b){return lv.h?lv.h(b,this):lv.call(null,b,this)};
jv.prototype[kt]=function(){return Su(this)};function qv(a){return I(K(a))}function rv(a){for(;;){var b=K(a);if(null!=b)a=b;else return I(a)}}cu._=function(a,b){return a===b};var sv=function sv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return sv.T();case 1:return sv.c(arguments[0]);case 2:return sv.h(arguments[0],arguments[1]);default:return sv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};sv.T=function(){return tv};
sv.c=function(a){return a};sv.h=function(a,b){return null!=a?vt(a,b):vt(Qu,b)};sv.w=function(a,b,c){for(;;)if(z(c))a=sv.h(a,b),b=I(c),c=K(c);else return sv.h(a,b)};sv.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return sv.w(b,a,c)};sv.I=2;
function P(a){if(null!=a)if(null!=a&&(a.C&2||x===a.au))a=a.xa(null);else if(gt(a))a=a.length;else if("string"===typeof a)a=a.length;else if(null!=a&&(a.C&8388608||x===a.lu))a:{a=G(a);for(var b=0;;){if(dv(a)){a=b+st(a);break a}a=K(a);b+=1}}else a=st(a);else a=0;return a}function uv(a,b,c){for(;;){if(null==a)return c;if(0===b)return G(a)?I(a):c;if(ev(a))return xt.j(a,b,c);if(G(a))a=K(a),--b;else return c}}
function gv(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return fv(arguments[0],arguments[1]);case 3:return S(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}
function fv(a,b){if("number"!==typeof b)throw Error("Index argument to nth must be a number");if(null==a)return a;if(null!=a&&(a.C&16||x===a.Vp))return a.$(null,b);if(gt(a)){if(0<=b&&b<a.length)return a[b];throw Error("Index out of bounds");}if("string"===typeof a){if(0<=b&&b<a.length)return a.charAt(b);throw Error("Index out of bounds");}if(null!=a&&(a.C&64||x===a.Pb)){a:{var c=a;for(var d=b;;){if(null==c)throw Error("Index out of bounds");if(0===d){if(G(c)){c=I(c);break a}throw Error("Index out of bounds");
}if(ev(c)){c=xt.h(c,d);break a}if(G(c))c=K(c),--d;else throw Error("Index out of bounds");}}return c}if(C(wt,a))return xt.h(a,b);throw Error([D.c("nth not supported on this type "),D.c(jt(null==a?null:a.constructor))].join(""));}
function S(a,b,c){if("number"!==typeof b)throw Error("Index argument to nth must be a number.");if(null==a)return c;if(null!=a&&(a.C&16||x===a.Vp))return a.Zb(null,b,c);if(gt(a))return 0<=b&&b<a.length?a[b]:c;if("string"===typeof a)return 0<=b&&b<a.length?a.charAt(b):c;if(null!=a&&(a.C&64||x===a.Pb))return uv(a,b,c);if(C(wt,a))return xt.h(a,b);throw Error([D.c("nth not supported on this type "),D.c(jt(null==a?null:a.constructor))].join(""));}
var F=function F(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return F.h(arguments[0],arguments[1]);case 3:return F.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};F.h=function(a,b){return null==a?null:null!=a&&(a.C&256||x===a.hu)?a.na(null,b):gt(a)?null!=b&&b<a.length?a[b|0]:null:"string"===typeof a?null!=b&&b<a.length?a.charAt(b|0):null:C(Ct,a)?Dt.h(a,b):null};
F.j=function(a,b,c){return null!=a?null!=a&&(a.C&256||x===a.hu)?a.X(null,b,c):gt(a)?null!=b&&0<=b&&b<a.length?a[b|0]:c:"string"===typeof a?null!=b&&0<=b&&b<a.length?a.charAt(b|0):c:C(Ct,a)?Dt.j(a,b,c):c:c};F.I=3;var T=function T(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 3:return T.j(arguments[0],arguments[1],arguments[2]);default:return T.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};
T.j=function(a,b,c){return null!=a?Gt(a,b,c):vv([b,c])};T.w=function(a,b,c,d){for(;;)if(a=T.j(a,b,c),z(d))b=I(d),c=qv(d),d=K(K(d));else return a};T.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return T.w(b,a,c,d)};T.I=3;
var wv=function wv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return wv.c(arguments[0]);case 2:return wv.h(arguments[0],arguments[1]);default:return wv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};wv.c=function(a){return a};wv.h=function(a,b){return null==a?null:Kt(a,b)};wv.w=function(a,b,c){for(;;){if(null==a)return null;a=wv.h(a,b);if(z(c))b=I(c),c=K(c);else return a}};
wv.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return wv.w(b,a,c)};wv.I=2;function xv(a){var b=oa(a);return b?b:null!=a?x===a.$t?!0:a.kj?!1:C(pt,a):C(pt,a)}function yv(a,b){this.B=a;this.meta=b;this.C=393217;this.Y=0}f=yv.prototype;f.sa=function(){return this.meta};f.ya=function(a,b){return new yv(this.B,b)};f.$t=x;
f.call=function(){function a(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na,Ya){a=this;return U.dj?U.dj(a.B,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na,Ya):U.call(null,a.B,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na,Ya)}function b(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na){a=this;return a.B.Ud?a.B.Ud(b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W,na)}function c(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W){a=this;return a.B.Td?a.B.Td(b,c,d,e,g,h,k,l,m,p,u,A,
B,E,L,J,Q,ia,W):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia,W)}function d(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia){a=this;return a.B.Sd?a.B.Sd(b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q,ia)}function e(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q){a=this;return a.B.Rd?a.B.Rd(b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J,Q)}function g(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J){a=this;return a.B.Qd?a.B.Qd(b,c,d,e,g,h,k,l,m,p,u,A,B,E,
L,J):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L,J)}function h(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L){a=this;return a.B.Pd?a.B.Pd(b,c,d,e,g,h,k,l,m,p,u,A,B,E,L):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E,L)}function k(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E){a=this;return a.B.Od?a.B.Od(b,c,d,e,g,h,k,l,m,p,u,A,B,E):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B,E)}function l(a,b,c,d,e,g,h,k,l,m,p,u,A,B){a=this;return a.B.Nd?a.B.Nd(b,c,d,e,g,h,k,l,m,p,u,A,B):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,B)}function m(a,b,c,d,e,g,
h,k,l,m,p,u,A){a=this;return a.B.Md?a.B.Md(b,c,d,e,g,h,k,l,m,p,u,A):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A)}function p(a,b,c,d,e,g,h,k,l,m,p,u){a=this;return a.B.Ld?a.B.Ld(b,c,d,e,g,h,k,l,m,p,u):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u)}function u(a,b,c,d,e,g,h,k,l,m,p){a=this;return a.B.Kd?a.B.Kd(b,c,d,e,g,h,k,l,m,p):a.B.call(null,b,c,d,e,g,h,k,l,m,p)}function A(a,b,c,d,e,g,h,k,l,m){a=this;return a.B.Xd?a.B.Xd(b,c,d,e,g,h,k,l,m):a.B.call(null,b,c,d,e,g,h,k,l,m)}function B(a,b,c,d,e,g,h,k,l){a=this;return a.B.Wd?
a.B.Wd(b,c,d,e,g,h,k,l):a.B.call(null,b,c,d,e,g,h,k,l)}function E(a,b,c,d,e,g,h,k){a=this;return a.B.Vd?a.B.Vd(b,c,d,e,g,h,k):a.B.call(null,b,c,d,e,g,h,k)}function J(a,b,c,d,e,g,h){a=this;return a.B.Ac?a.B.Ac(b,c,d,e,g,h):a.B.call(null,b,c,d,e,g,h)}function L(a,b,c,d,e,g){a=this;return a.B.ba?a.B.ba(b,c,d,e,g):a.B.call(null,b,c,d,e,g)}function Q(a,b,c,d,e){a=this;return a.B.L?a.B.L(b,c,d,e):a.B.call(null,b,c,d,e)}function ia(a,b,c,d){a=this;return a.B.j?a.B.j(b,c,d):a.B.call(null,b,c,d)}function na(a,
b,c){a=this;return a.B.h?a.B.h(b,c):a.B.call(null,b,c)}function Ya(a,b){a=this;return a.B.c?a.B.c(b):a.B.call(null,b)}function Oc(a){a=this;return a.B.T?a.B.T():a.B.call(null)}var W=null,W=function(W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg,Qh,hg,Kl,Ll,Ml){switch(arguments.length){case 1:return Oc.call(this,W);case 2:return Ya.call(this,W,fb);case 3:return na.call(this,W,fb,pb);case 4:return ia.call(this,W,fb,pb,vb);case 5:return Q.call(this,W,fb,pb,vb,wb);case 6:return L.call(this,W,fb,pb,
vb,wb,Eb);case 7:return J.call(this,W,fb,pb,vb,wb,Eb,Tb);case 8:return E.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub);case 9:return B.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc);case 10:return A.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc);case 11:return u.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc);case 12:return p.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd);case 13:return m.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd);case 14:return l.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be);case 15:return k.call(this,
W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De);case 16:return h.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf);case 17:return g.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg);case 18:return e.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg,Qh);case 19:return d.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg,Qh,hg);case 20:return c.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg,Qh,hg,Kl);case 21:return b.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,
Dc,Pc,gd,Gd,be,De,hf,gg,Qh,hg,Kl,Ll);case 22:return a.call(this,W,fb,pb,vb,wb,Eb,Tb,Ub,lc,Dc,Pc,gd,Gd,be,De,hf,gg,Qh,hg,Kl,Ll,Ml)}throw Error("Invalid arity: "+(arguments.length-1));};W.c=Oc;W.h=Ya;W.j=na;W.L=ia;W.ba=Q;W.Ac=L;W.Vd=J;W.Wd=E;W.Xd=B;W.Kd=A;W.Ld=u;W.Md=p;W.Nd=m;W.Od=l;W.Pd=k;W.Qd=h;W.Rd=g;W.Sd=e;W.Td=d;W.Ud=c;W.fu=b;W.dj=a;return W}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.T=function(){return this.B.T?this.B.T():this.B.call(null)};
f.c=function(a){return this.B.c?this.B.c(a):this.B.call(null,a)};f.h=function(a,b){return this.B.h?this.B.h(a,b):this.B.call(null,a,b)};f.j=function(a,b,c){return this.B.j?this.B.j(a,b,c):this.B.call(null,a,b,c)};f.L=function(a,b,c,d){return this.B.L?this.B.L(a,b,c,d):this.B.call(null,a,b,c,d)};f.ba=function(a,b,c,d,e){return this.B.ba?this.B.ba(a,b,c,d,e):this.B.call(null,a,b,c,d,e)};f.Ac=function(a,b,c,d,e,g){return this.B.Ac?this.B.Ac(a,b,c,d,e,g):this.B.call(null,a,b,c,d,e,g)};
f.Vd=function(a,b,c,d,e,g,h){return this.B.Vd?this.B.Vd(a,b,c,d,e,g,h):this.B.call(null,a,b,c,d,e,g,h)};f.Wd=function(a,b,c,d,e,g,h,k){return this.B.Wd?this.B.Wd(a,b,c,d,e,g,h,k):this.B.call(null,a,b,c,d,e,g,h,k)};f.Xd=function(a,b,c,d,e,g,h,k,l){return this.B.Xd?this.B.Xd(a,b,c,d,e,g,h,k,l):this.B.call(null,a,b,c,d,e,g,h,k,l)};f.Kd=function(a,b,c,d,e,g,h,k,l,m){return this.B.Kd?this.B.Kd(a,b,c,d,e,g,h,k,l,m):this.B.call(null,a,b,c,d,e,g,h,k,l,m)};
f.Ld=function(a,b,c,d,e,g,h,k,l,m,p){return this.B.Ld?this.B.Ld(a,b,c,d,e,g,h,k,l,m,p):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p)};f.Md=function(a,b,c,d,e,g,h,k,l,m,p,u){return this.B.Md?this.B.Md(a,b,c,d,e,g,h,k,l,m,p,u):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u)};f.Nd=function(a,b,c,d,e,g,h,k,l,m,p,u,A){return this.B.Nd?this.B.Nd(a,b,c,d,e,g,h,k,l,m,p,u,A):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A)};
f.Od=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B){return this.B.Od?this.B.Od(a,b,c,d,e,g,h,k,l,m,p,u,A,B):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B)};f.Pd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E){return this.B.Pd?this.B.Pd(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E)};f.Qd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J){return this.B.Qd?this.B.Qd(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J)};
f.Rd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L){return this.B.Rd?this.B.Rd(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L)};f.Sd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q){return this.B.Sd?this.B.Sd(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q)};
f.Td=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia){return this.B.Td?this.B.Td(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia)};f.Ud=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na){return this.B.Ud?this.B.Ud(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na)};
f.fu=function(a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya){return U.dj?U.dj(this.B,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya):U.call(null,this.B,a,b,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya)};function zv(a,b){return oa(a)?new yv(a,b):null==a?null:Zt(a,b)}function Av(a){var b=null!=a;return(b?null!=a?a.C&131072||x===a.ju||(a.C?0:C(Wt,a)):C(Wt,a):b)?Xt(a):null}function Bv(a){return null==a?null:Qt(a)}
var Cv=function Cv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Cv.c(arguments[0]);case 2:return Cv.h(arguments[0],arguments[1]);default:return Cv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Cv.c=function(a){return a};Cv.h=function(a,b){return null==a?null:Pt(a,b)};Cv.w=function(a,b,c){for(;;){if(null==a)return null;a=Cv.h(a,b);if(z(c))b=I(c),c=K(c);else return a}};
Cv.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Cv.w(b,a,c)};Cv.I=2;function Dv(a){return null==a||ht(G(a))}function Ev(a){return null==a?!1:null!=a?a.C&8||x===a.Cz?!0:a.C?!1:C(ut,a):C(ut,a)}function Fv(a){return null==a?!1:null!=a?a.C&4096||x===a.nu?!0:a.C?!1:C(Ot,a):C(Ot,a)}function Gv(a){return null!=a?a.C&16777216||x===a.mu?!0:a.C?!1:C(gu,a):C(gu,a)}function Hv(a){return null==a?!1:null!=a?a.C&1024||x===a.Wp?!0:a.C?!1:C(Jt,a):C(Jt,a)}
function Iv(a){return null!=a?a.C&67108864||x===a.Gz?!0:a.C?!1:C(iu,a):C(iu,a)}function Jv(a){return null!=a?a.C&16384||x===a.Iz?!0:a.C?!1:C(St,a):C(St,a)}function Kv(a){return null!=a?a.Y&512||x===a.Bz?!0:!1:!1}function Lv(a){var b=[];Db(a,function(a,b){return function(a,c){return b.push(c)}}(a,b));return b}function Mv(a,b,c,d,e){for(;0!==e;)c[d]=a[b],d+=1,--e,b+=1}var Nv={};function Ov(a){return!0===a}function Pv(a){return null==a?!1:null!=a?a.C&64||x===a.Pb?!0:a.C?!1:C(yt,a):C(yt,a)}
function Qv(a){return null==a?!1:!1===a?!1:!0}function Rv(a){var b=xv(a);return b?b:null!=a?a.C&1||x===a.Ez?!0:a.C?!1:C(qt,a):C(qt,a)}function Sv(a){return"number"===typeof a&&!isNaN(a)&&Infinity!==a&&parseFloat(a)===parseInt(a,10)}function Tv(a,b){return F.j(a,b,Nv)===Nv?!1:!0}function Uv(a,b){var c;if(c=null!=a)c=null!=a?a.C&512||x===a.Az?!0:a.C?!1:C(Et,a):C(Et,a);return c&&Tv(a,b)?(null!=a?x===a.cj||(a.kj?0:C(Ht,a)):C(Ht,a))?It(a,b):new V(null,2,5,X,[b,F.h(a,b)],null):null}
function ov(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return nv(arguments[0],arguments[1]);case 3:return pv(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}function nv(a,b){var c=G(b);if(c){var d=I(c),c=K(c);return nt?nt(a,d,c):ot.call(null,a,d,c)}return a.T?a.T():a.call(null)}
function pv(a,b,c){for(c=G(c);;)if(c){var d=I(c);b=a.h?a.h(b,d):a.call(null,b,d);c=K(c)}else return b}function ot(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return Vv(arguments[0],arguments[1]);case 3:return nt(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}
function Vv(a,b){return null!=b&&(b.C&524288||x===b.ku)?b.xb(null,a):gt(b)?av(b,a):"string"===typeof b?av(b,a):C($t,b)?au.h(b,a):nv(a,b)}function nt(a,b,c){return null!=c&&(c.C&524288||x===c.ku)?c.yb(null,a,b):gt(c)?bv(c,a,b):"string"===typeof c?bv(c,a,b):C($t,c)?au.j(c,a,b):pv(a,b,c)}function Wv(a,b,c){return null!=c?bu(c,a,b):b}function Xv(a){return a}function Yv(a,b,c,d){a=a.c?a.c(b):a.call(null,b);c=nt(a,c,d);return a.c?a.c(c):a.call(null,c)}
var Zv=function Zv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Zv.T();case 1:return Zv.c(arguments[0]);case 2:return Zv.h(arguments[0],arguments[1]);default:return Zv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Zv.T=function(){return 0};Zv.c=function(a){return a};Zv.h=function(a,b){return a+b};Zv.w=function(a,b,c){return nt(Zv,a+b,c)};Zv.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Zv.w(b,a,c)};Zv.I=2;
var $v=function $v(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return $v.c(arguments[0]);case 2:return $v.h(arguments[0],arguments[1]);default:return $v.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};$v.c=function(a){return-a};$v.h=function(a,b){return a-b};$v.w=function(a,b,c){return nt($v,a-b,c)};$v.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return $v.w(b,a,c)};$v.I=2;function aw(a){return a-1}
var bw=function bw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return bw.c(arguments[0]);case 2:return bw.h(arguments[0],arguments[1]);default:return bw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};bw.c=function(a){return a};bw.h=function(a,b){return a>b?a:b};bw.w=function(a,b,c){return nt(bw,a>b?a:b,c)};bw.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return bw.w(b,a,c)};bw.I=2;function cw(a,b){return(a%b+b)%b}
function dw(a){a=(a-a%2)/2;return 0<=a?Math.floor(a):Math.ceil(a)}function ew(a){a-=a>>1&1431655765;a=(a&858993459)+(a>>2&858993459);return 16843009*(a+(a>>4)&252645135)>>24}var D=function D(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return D.T();case 1:return D.c(arguments[0]);default:return D.w(arguments[0],new H(c.slice(1),0,null))}};D.T=function(){return""};D.c=function(a){return null==a?"":""+a};
D.w=function(a,b){for(var c=new Es(""+D.c(a)),d=b;;)if(z(d))c=c.append(""+D.c(I(d))),d=K(d);else return c.toString()};D.N=function(a){var b=I(a);a=K(a);return D.w(b,a)};D.I=1;function fw(a,b,c){return a.substring(b,c)}function kv(a,b){if(Gv(b))if(dv(a)&&dv(b)&&P(a)!==P(b))var c=!1;else a:{c=G(a);for(var d=G(b);;){if(null==c){c=null==d;break a}if(null!=d&&M.h(I(c),I(d)))c=K(c),d=K(d);else{c=!1;break a}}}else c=null;return Qv(c)}
function gw(a){var b=0;for(a=G(a);;)if(a){var c=I(a),b=(b+(Ku(hw.c?hw.c(c):hw.call(null,c))^Ku(iw.c?iw.c(c):iw.call(null,c))))%4503599627370496;a=K(a)}else return b}function jw(a,b,c,d,e){this.meta=a;this.first=b;this.te=c;this.count=d;this.K=e;this.C=65937646;this.Y=8192}f=jw.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,this.count)}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){return 1===this.count?null:this.te};f.xa=function(){return this.count};f.ng=function(){return this.first};f.og=function(){return this.zb(null)};
f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return this.first};f.zb=function(){return 1===this.count?Qu:this.te};f.oa=function(){return this};f.ya=function(a,b){return new jw(b,this.first,this.te,this.count,this.K)};f.wa=function(a,b){return new jw(this.meta,b,this,this.count+1,null)};
function kw(a){return null!=a?a.C&33554432||x===a.Fz?!0:a.C?!1:C(hu,a):C(hu,a)}jw.prototype[kt]=function(){return Su(this)};function lw(a){this.meta=a;this.C=65937614;this.Y=8192}f=lw.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){return null};f.xa=function(){return 0};f.ng=function(){return null};f.og=function(){throw Error("Can't pop empty list");};f.ha=function(){return Vu};
f.O=function(a,b){return kw(b)||Gv(b)?null==G(b):!1};f.Ja=function(){return this};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return null};f.zb=function(){return Qu};f.oa=function(){return null};f.ya=function(a,b){return new lw(b)};f.wa=function(a,b){return new jw(this.meta,b,null,1,null)};var Qu=new lw(null);lw.prototype[kt]=function(){return Su(this)};
function mw(a){return(null!=a?a.C&134217728||x===a.Hz||(a.C?0:C(ju,a)):C(ju,a))?ku(a):nt(sv,Qu,a)}var nw=function nw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return nw.w(0<c.length?new H(c.slice(0),0,null):null)};nw.w=function(a){if(a instanceof H&&0===a.i)var b=a.v;else a:for(b=[];;)if(null!=a)b.push(a.ab(null)),a=a.$b(null);else break a;a=b.length;for(var c=Qu;;)if(0<a){var d=a-1,c=c.wa(null,b[a-1]);a=d}else return c};nw.I=0;nw.N=function(a){return nw.w(G(a))};
function ow(a,b,c,d){this.meta=a;this.first=b;this.te=c;this.K=d;this.C=65929452;this.Y=8192}f=ow.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){return null==this.te?null:G(this.te)};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};
f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return this.first};f.zb=function(){return null==this.te?Qu:this.te};f.oa=function(){return this};f.ya=function(a,b){return new ow(b,this.first,this.te,this.K)};f.wa=function(a,b){return new ow(null,b,this,null)};ow.prototype[kt]=function(){return Su(this)};
function lv(a,b){return null==b||null!=b&&(b.C&64||x===b.Pb)?new ow(null,a,b,null):new ow(null,a,G(b),null)}function Y(a,b,c,d){this.ns=a;this.name=b;this.Sb=c;this.fg=d;this.C=2153775105;this.Y=4096}f=Y.prototype;f.toString=function(){return[D.c(":"),D.c(this.Sb)].join("")};f.equiv=function(a){return this.O(null,a)};f.O=function(a,b){return b instanceof Y?this.Sb===b.Sb:!1};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return F.h(c,this);case 3:return F.j(c,this,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return F.h(c,this)};a.j=function(a,c,d){return F.j(c,this,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return F.h(a,this)};f.h=function(a,b){return F.j(a,this,b)};
f.ha=function(){var a=this.fg;return null!=a?a:this.fg=a=Lu(Gu(this.name),Ju(this.ns))+2654435769|0};f.ja=function(a,b){return lu(b,[D.c(":"),D.c(this.Sb)].join(""))};function pw(a,b){return a===b?!0:a instanceof Y&&b instanceof Y?a.Sb===b.Sb:!1}
var qw=function qw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return qw.c(arguments[0]);case 2:return qw.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
qw.c=function(a){if(a instanceof Y)return a;if(a instanceof Nu){if(null!=a&&(a.Y&4096||x===a.Xp))var b=a.ns;else throw Error([D.c("Doesn't support namespace: "),D.c(a)].join(""));return new Y(b,rw.c?rw.c(a):rw.call(null,a),a.Dd,null)}return"string"===typeof a?(b=a.split("/"),2===b.length?new Y(b[0],b[1],a,null):new Y(null,b[0],a,null)):null};
qw.h=function(a,b){var c=a instanceof Y?rw.c?rw.c(a):rw.call(null,a):a instanceof Nu?rw.c?rw.c(a):rw.call(null,a):a,d=b instanceof Y?rw.c?rw.c(b):rw.call(null,b):b instanceof Nu?rw.c?rw.c(b):rw.call(null,b):b;return new Y(c,d,[D.c(z(c)?[D.c(c),D.c("/")].join(""):null),D.c(d)].join(""),null)};qw.I=2;function sw(a,b,c,d){this.meta=a;this.fn=b;this.s=c;this.K=d;this.C=32374988;this.Y=1}f=sw.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
function tw(a){null!=a.fn&&(a.s=a.fn.T?a.fn.T():a.fn.call(null),a.fn=null);return a.s}f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){this.oa(null);return null==this.s?null:K(this.s)};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};
f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){this.oa(null);return null==this.s?null:I(this.s)};f.zb=function(){this.oa(null);return null!=this.s?Pu(this.s):Qu};f.oa=function(){tw(this);if(null==this.s)return null;for(var a=this.s;;)if(a instanceof sw)a=tw(a);else return this.s=a,G(this.s)};f.ya=function(a,b){return new sw(b,this.fn,this.s,this.K)};f.wa=function(a,b){return lv(b,this)};sw.prototype[kt]=function(){return Su(this)};
function uw(a,b){this.Kl=a;this.end=b;this.C=2;this.Y=0}uw.prototype.add=function(a){this.Kl[this.end]=a;return this.end+=1};uw.prototype.ob=function(){var a=new vw(this.Kl,0,this.end);this.Kl=null;return a};uw.prototype.xa=function(){return this.end};function ww(a){return new uw(Array(a),0)}function vw(a,b,c){this.v=a;this.rb=b;this.end=c;this.C=524306;this.Y=0}f=vw.prototype;f.xa=function(){return this.end-this.rb};f.$=function(a,b){return this.v[this.rb+b]};
f.Zb=function(a,b,c){return 0<=b&&b<this.end-this.rb?this.v[this.rb+b]:c};f.Rp=function(){if(this.rb===this.end)throw Error("-drop-first of empty chunk");return new vw(this.v,this.rb+1,this.end)};f.xb=function(a,b){return cv(this.v,b,this.v[this.rb],this.rb+1)};f.yb=function(a,b,c){return cv(this.v,b,c,this.rb)};function xw(a,b,c,d){this.ob=a;this.zd=b;this.meta=c;this.K=d;this.C=31850732;this.Y=1536}f=xw.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){if(1<st(this.ob))return new xw(uu(this.ob),this.zd,this.meta,null);var a=fu(this.zd);return null==a?null:a};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};
f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.ab=function(){return xt.h(this.ob,0)};f.zb=function(){return 1<st(this.ob)?new xw(uu(this.ob),this.zd,this.meta,null):null==this.zd?Qu:this.zd};f.oa=function(){return this};f.Rl=function(){return this.ob};f.bj=function(){return null==this.zd?Qu:this.zd};f.ya=function(a,b){return new xw(this.ob,this.zd,b,this.K)};f.wa=function(a,b){return lv(b,this)};f.Sp=function(){return null==this.zd?null:this.zd};xw.prototype[kt]=function(){return Su(this)};
function yw(a,b){return 0===st(a)?b:new xw(a,b,null,null)}function zw(a,b){a.add(b)}function Aw(a){for(var b=[];;)if(G(a))b.push(I(a)),a=K(a);else return b}function Bw(a,b){if(dv(b))return P(b);for(var c=0,d=G(b);;)if(null!=d&&c<a)c+=1,d=K(d);else return c}
var Cw=function Cw(b){if(null==b)var c=null;else if(null==K(b))c=G(I(b));else{c=lv;var d=I(b);b=K(b);b=Cw.c?Cw.c(b):Cw.call(null,b);c=c(d,b)}return c},Dw=function Dw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Dw.T();case 1:return Dw.c(arguments[0]);case 2:return Dw.h(arguments[0],arguments[1]);default:return Dw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};
Dw.T=function(){return new sw(null,function(){return null},null,null)};Dw.c=function(a){return new sw(null,function(){return a},null,null)};Dw.h=function(a,b){return new sw(null,function(){var c=G(a);return c?Kv(c)?yw(vu(c),Dw.h(wu(c),b)):lv(I(c),Dw.h(Pu(c),b)):b},null,null)};Dw.w=function(a,b,c){return function e(a,b){return new sw(null,function(){var c=G(a);return c?Kv(c)?yw(vu(c),e(wu(c),b)):lv(I(c),e(Pu(c),b)):z(b)?e(I(b),K(b)):null},null,null)}(Dw.h(a,b),c)};
Dw.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Dw.w(b,a,c)};Dw.I=2;var Ew=function Ew(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Ew.T();case 1:return Ew.c(arguments[0]);case 2:return Ew.h(arguments[0],arguments[1]);default:return Ew.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Ew.T=function(){return qu(tv)};Ew.c=function(a){return a};Ew.h=function(a,b){return ru(a,b)};
Ew.w=function(a,b,c){for(;;)if(a=ru(a,b),z(c))b=I(c),c=K(c);else return a};Ew.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Ew.w(b,a,c)};Ew.I=2;
function Fw(a,b,c){var d=G(c);if(0===b)return a.T?a.T():a.call(null);c=zt(d);var e=At(d);if(1===b)return a.c?a.c(c):a.c?a.c(c):a.call(null,c);var d=zt(e),g=At(e);if(2===b)return a.h?a.h(c,d):a.h?a.h(c,d):a.call(null,c,d);var e=zt(g),h=At(g);if(3===b)return a.j?a.j(c,d,e):a.j?a.j(c,d,e):a.call(null,c,d,e);var g=zt(h),k=At(h);if(4===b)return a.L?a.L(c,d,e,g):a.L?a.L(c,d,e,g):a.call(null,c,d,e,g);var h=zt(k),l=At(k);if(5===b)return a.ba?a.ba(c,d,e,g,h):a.ba?a.ba(c,d,e,g,h):a.call(null,c,d,e,g,h);var k=
zt(l),m=At(l);if(6===b)return a.Ac?a.Ac(c,d,e,g,h,k):a.Ac?a.Ac(c,d,e,g,h,k):a.call(null,c,d,e,g,h,k);var l=zt(m),p=At(m);if(7===b)return a.Vd?a.Vd(c,d,e,g,h,k,l):a.Vd?a.Vd(c,d,e,g,h,k,l):a.call(null,c,d,e,g,h,k,l);var m=zt(p),u=At(p);if(8===b)return a.Wd?a.Wd(c,d,e,g,h,k,l,m):a.Wd?a.Wd(c,d,e,g,h,k,l,m):a.call(null,c,d,e,g,h,k,l,m);var p=zt(u),A=At(u);if(9===b)return a.Xd?a.Xd(c,d,e,g,h,k,l,m,p):a.Xd?a.Xd(c,d,e,g,h,k,l,m,p):a.call(null,c,d,e,g,h,k,l,m,p);var u=zt(A),B=At(A);if(10===b)return a.Kd?a.Kd(c,
d,e,g,h,k,l,m,p,u):a.Kd?a.Kd(c,d,e,g,h,k,l,m,p,u):a.call(null,c,d,e,g,h,k,l,m,p,u);var A=zt(B),E=At(B);if(11===b)return a.Ld?a.Ld(c,d,e,g,h,k,l,m,p,u,A):a.Ld?a.Ld(c,d,e,g,h,k,l,m,p,u,A):a.call(null,c,d,e,g,h,k,l,m,p,u,A);var B=zt(E),J=At(E);if(12===b)return a.Md?a.Md(c,d,e,g,h,k,l,m,p,u,A,B):a.Md?a.Md(c,d,e,g,h,k,l,m,p,u,A,B):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B);var E=zt(J),L=At(J);if(13===b)return a.Nd?a.Nd(c,d,e,g,h,k,l,m,p,u,A,B,E):a.Nd?a.Nd(c,d,e,g,h,k,l,m,p,u,A,B,E):a.call(null,c,d,e,g,h,k,l,
m,p,u,A,B,E);var J=zt(L),Q=At(L);if(14===b)return a.Od?a.Od(c,d,e,g,h,k,l,m,p,u,A,B,E,J):a.Od?a.Od(c,d,e,g,h,k,l,m,p,u,A,B,E,J):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J);var L=zt(Q),ia=At(Q);if(15===b)return a.Pd?a.Pd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L):a.Pd?a.Pd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L);var Q=zt(ia),na=At(ia);if(16===b)return a.Qd?a.Qd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q):a.Qd?a.Qd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q);
var ia=zt(na),Ya=At(na);if(17===b)return a.Rd?a.Rd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia):a.Rd?a.Rd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia);var na=zt(Ya),Oc=At(Ya);if(18===b)return a.Sd?a.Sd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na):a.Sd?a.Sd(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na);Ya=zt(Oc);Oc=At(Oc);if(19===b)return a.Td?a.Td(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya):a.Td?a.Td(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,
ia,na,Ya):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya);var W=zt(Oc);At(Oc);if(20===b)return a.Ud?a.Ud(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya,W):a.Ud?a.Ud(c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya,W):a.call(null,c,d,e,g,h,k,l,m,p,u,A,B,E,J,L,Q,ia,na,Ya,W);throw Error("Only up to 20 arguments supported on functions");}
var U=function U(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return U.h(arguments[0],arguments[1]);case 3:return U.j(arguments[0],arguments[1],arguments[2]);case 4:return U.L(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return U.ba(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:return U.w(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],new H(c.slice(5),0,null))}};
U.h=function(a,b){var c=a.I;if(a.N){var d=Bw(c+1,b);return d<=c?Fw(a,d,b):a.N(b)}return a.apply(a,Aw(b))};U.j=function(a,b,c){b=lv(b,c);c=a.I;if(a.N){var d=Bw(c+1,b);return d<=c?Fw(a,d,b):a.N(b)}return a.apply(a,Aw(b))};U.L=function(a,b,c,d){b=lv(b,lv(c,d));c=a.I;return a.N?(d=Bw(c+1,b),d<=c?Fw(a,d,b):a.N(b)):a.apply(a,Aw(b))};U.ba=function(a,b,c,d,e){b=lv(b,lv(c,lv(d,e)));c=a.I;return a.N?(d=Bw(c+1,b),d<=c?Fw(a,d,b):a.N(b)):a.apply(a,Aw(b))};
U.w=function(a,b,c,d,e,g){b=lv(b,lv(c,lv(d,lv(e,Cw(g)))));c=a.I;return a.N?(d=Bw(c+1,b),d<=c?Fw(a,d,b):a.N(b)):a.apply(a,Aw(b))};U.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),g=K(e),e=I(g),g=K(g);return U.w(b,a,c,d,e,g)};U.I=5;
var Gw=function Gw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Gw.c(arguments[0]);case 2:return Gw.h(arguments[0],arguments[1]);default:return Gw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Gw.c=function(){return!1};Gw.h=function(a,b){return!M.h(a,b)};Gw.w=function(a,b,c){return ht(U.L(M,a,b,c))};Gw.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Gw.w(b,a,c)};Gw.I=2;function Hw(a){return G(a)?a:null}
function Iw(){"undefined"===typeof Vs&&(Vs=function(a){this.mv=a;this.C=393216;this.Y=0},Vs.prototype.ya=function(a,b){return new Vs(b)},Vs.prototype.sa=function(){return this.mv},Vs.prototype.hb=function(){return!1},Vs.prototype.next=function(){return Error("No such element")},Vs.prototype.remove=function(){return Error("Unsupported operation")},Vs.Kz=function(){return new V(null,1,5,X,[Jw],null)},Vs.Zp=!0,Vs.jj="cljs.core/t_cljs$core14185",Vs.ou=function(a){return lu(a,"cljs.core/t_cljs$core14185")});
return new Vs(Kw)}function Lw(a,b){for(;;){if(null==G(b))return!0;var c=I(b);c=a.c?a.c(c):a.call(null,c);if(z(c)){c=a;var d=K(b);a=c;b=d}else return!1}}function Mw(a,b){for(;;)if(G(b)){var c=I(b);c=a.c?a.c(c):a.call(null,c);if(z(c))return c;c=a;var d=K(b);a=c;b=d}else return null}
function Nw(a){return function(){function b(b,c){return ht(a.h?a.h(b,c):a.call(null,b,c))}function c(b){return ht(a.c?a.c(b):a.call(null,b))}function d(){return ht(a.T?a.T():a.call(null))}var e=null,g=function(){function b(a,b,d){var e=null;if(2<arguments.length){for(var e=0,g=Array(arguments.length-2);e<g.length;)g[e]=arguments[e+2],++e;e=new H(g,0,null)}return c.call(this,a,b,e)}function c(b,c,d){return ht(U.L(a,b,c,d))}b.I=2;b.N=function(a){var b=I(a);a=K(a);var d=I(a);a=Pu(a);return c(b,d,a)};
b.w=c;return b}(),e=function(a,e,l){switch(arguments.length){case 0:return d.call(this);case 1:return c.call(this,a);case 2:return b.call(this,a,e);default:var h=null;if(2<arguments.length){for(var h=0,k=Array(arguments.length-2);h<k.length;)k[h]=arguments[h+2],++h;h=new H(k,0,null)}return g.w(a,e,h)}throw Error("Invalid arity: "+(arguments.length-1));};e.I=2;e.N=g.N;e.T=d;e.c=c;e.h=b;e.w=g.w;return e}()}
var Ow=function Ow(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Ow.T();case 1:return Ow.c(arguments[0]);case 2:return Ow.h(arguments[0],arguments[1]);case 3:return Ow.j(arguments[0],arguments[1],arguments[2]);default:return Ow.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};Ow.T=function(){return Xv};Ow.c=function(a){return a};
Ow.h=function(a,b){return function(){function c(c,d,e){c=b.j?b.j(c,d,e):b.call(null,c,d,e);return a.c?a.c(c):a.call(null,c)}function d(c,d){var e=b.h?b.h(c,d):b.call(null,c,d);return a.c?a.c(e):a.call(null,e)}function e(c){c=b.c?b.c(c):b.call(null,c);return a.c?a.c(c):a.call(null,c)}function g(){var c=b.T?b.T():b.call(null);return a.c?a.c(c):a.call(null,c)}var h=null,k=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+
3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){c=U.ba(b,c,d,e,g);return a.c?a.c(c):a.call(null,c)}c.I=3;c.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=Pu(a);return d(b,c,e,a)};c.w=d;return c}(),h=function(a,b,h,u){switch(arguments.length){case 0:return g.call(this);case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,h);default:var l=null;if(3<arguments.length){for(var l=0,m=Array(arguments.length-3);l<m.length;)m[l]=
arguments[l+3],++l;l=new H(m,0,null)}return k.w(a,b,h,l)}throw Error("Invalid arity: "+(arguments.length-1));};h.I=3;h.N=k.N;h.T=g;h.c=e;h.h=d;h.j=c;h.w=k.w;return h}()};
Ow.j=function(a,b,c){return function(){function d(d,e,g){d=c.j?c.j(d,e,g):c.call(null,d,e,g);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}function e(d,e){var g=c.h?c.h(d,e):c.call(null,d,e);g=b.c?b.c(g):b.call(null,g);return a.c?a.c(g):a.call(null,g)}function g(d){d=c.c?c.c(d):c.call(null,d);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}function h(){var d=c.T?c.T():c.call(null);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}var k=null,l=function(){function d(a,
b,c,d){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return e.call(this,a,b,c,g)}function e(d,e,g,h){d=U.ba(c,d,e,g,h);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}d.I=3;d.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=Pu(a);return e(b,c,d,a)};d.w=e;return d}(),k=function(a,b,c,k){switch(arguments.length){case 0:return h.call(this);case 1:return g.call(this,a);case 2:return e.call(this,
a,b);case 3:return d.call(this,a,b,c);default:var m=null;if(3<arguments.length){for(var m=0,p=Array(arguments.length-3);m<p.length;)p[m]=arguments[m+3],++m;m=new H(p,0,null)}return l.w(a,b,c,m)}throw Error("Invalid arity: "+(arguments.length-1));};k.I=3;k.N=l.N;k.T=h;k.c=g;k.h=e;k.j=d;k.w=l.w;return k}()};
Ow.w=function(a,b,c,d){return function(a){return function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){b=U.h(I(a),b);for(var c=K(a);;)if(c)b=I(c).call(null,b),c=K(c);else return b}b.I=0;b.N=function(a){a=G(a);return c(a)};b.w=c;return b}()}(mw(lv(a,lv(b,lv(c,d)))))};Ow.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return Ow.w(b,a,c,d)};Ow.I=3;
var Pw=function Pw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Pw.c(arguments[0]);case 2:return Pw.h(arguments[0],arguments[1]);case 3:return Pw.j(arguments[0],arguments[1],arguments[2]);case 4:return Pw.L(arguments[0],arguments[1],arguments[2],arguments[3]);default:return Pw.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};Pw.c=function(a){return a};
Pw.h=function(a,b){return function(){function c(c,d,e){return a.L?a.L(b,c,d,e):a.call(null,b,c,d,e)}function d(c,d){return a.j?a.j(b,c,d):a.call(null,b,c,d)}function e(c){return a.h?a.h(b,c):a.call(null,b,c)}function g(){return a.c?a.c(b):a.call(null,b)}var h=null,k=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){return U.w(a,b,c,d,e,R([g],
0))}c.I=3;c.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=Pu(a);return d(b,c,e,a)};c.w=d;return c}(),h=function(a,b,h,u){switch(arguments.length){case 0:return g.call(this);case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,h);default:var l=null;if(3<arguments.length){for(var l=0,m=Array(arguments.length-3);l<m.length;)m[l]=arguments[l+3],++l;l=new H(m,0,null)}return k.w(a,b,h,l)}throw Error("Invalid arity: "+(arguments.length-1));};h.I=3;h.N=
k.N;h.T=g;h.c=e;h.h=d;h.j=c;h.w=k.w;return h}()};
Pw.j=function(a,b,c){return function(){function d(d,e,g){return a.ba?a.ba(b,c,d,e,g):a.call(null,b,c,d,e,g)}function e(d,e){return a.L?a.L(b,c,d,e):a.call(null,b,c,d,e)}function g(d){return a.j?a.j(b,c,d):a.call(null,b,c,d)}function h(){return a.h?a.h(b,c):a.call(null,b,c)}var k=null,l=function(){function d(a,b,c,d){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return e.call(this,a,b,c,g)}function e(d,e,g,h){return U.w(a,
b,c,d,e,R([g,h],0))}d.I=3;d.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=Pu(a);return e(b,c,d,a)};d.w=e;return d}(),k=function(a,b,c,k){switch(arguments.length){case 0:return h.call(this);case 1:return g.call(this,a);case 2:return e.call(this,a,b);case 3:return d.call(this,a,b,c);default:var m=null;if(3<arguments.length){for(var m=0,p=Array(arguments.length-3);m<p.length;)p[m]=arguments[m+3],++m;m=new H(p,0,null)}return l.w(a,b,c,m)}throw Error("Invalid arity: "+(arguments.length-
1));};k.I=3;k.N=l.N;k.T=h;k.c=g;k.h=e;k.j=d;k.w=l.w;return k}()};
Pw.L=function(a,b,c,d){return function(){function e(e,g,h){return a.Ac?a.Ac(b,c,d,e,g,h):a.call(null,b,c,d,e,g,h)}function g(e,g){return a.ba?a.ba(b,c,d,e,g):a.call(null,b,c,d,e,g)}function h(e){return a.L?a.L(b,c,d,e):a.call(null,b,c,d,e)}function k(){return a.j?a.j(b,c,d):a.call(null,b,c,d)}var l=null,m=function(){function e(a,b,c,d){var e=null;if(3<arguments.length){for(var e=0,h=Array(arguments.length-3);e<h.length;)h[e]=arguments[e+3],++e;e=new H(h,0,null)}return g.call(this,a,b,c,e)}function g(e,
g,h,k){return U.w(a,b,c,d,e,R([g,h,k],0))}e.I=3;e.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=Pu(a);return g(b,c,d,a)};e.w=g;return e}(),l=function(a,b,c,d){switch(arguments.length){case 0:return k.call(this);case 1:return h.call(this,a);case 2:return g.call(this,a,b);case 3:return e.call(this,a,b,c);default:var l=null;if(3<arguments.length){for(var l=0,p=Array(arguments.length-3);l<p.length;)p[l]=arguments[l+3],++l;l=new H(p,0,null)}return m.w(a,b,c,l)}throw Error("Invalid arity: "+
(arguments.length-1));};l.I=3;l.N=m.N;l.T=k;l.c=h;l.h=g;l.j=e;l.w=m.w;return l}()};Pw.w=function(a,b,c,d,e){return function(){function g(a){var b=null;if(0<arguments.length){for(var b=0,c=Array(arguments.length-0);b<c.length;)c[b]=arguments[b+0],++b;b=new H(c,0,null)}return h.call(this,b)}function h(g){return U.ba(a,b,c,d,Dw.h(e,g))}g.I=0;g.N=function(a){a=G(a);return h(a)};g.w=h;return g}()};Pw.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return Pw.w(b,a,c,d,e)};
Pw.I=4;
function Qw(){var a=Rw,b=Sw;return function(){function c(c,d,e){c=null==c?b:c;return a.j?a.j(c,d,e):a.call(null,c,d,e)}function d(c,d){var e=null==c?b:c;return a.h?a.h(e,d):a.call(null,e,d)}function e(c){c=null==c?b:c;return a.c?a.c(c):a.call(null,c)}var g=null,h=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){return U.ba(a,null==c?b:c,d,e,
g)}c.I=3;c.N=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=Pu(a);return d(b,c,e,a)};c.w=d;return c}(),g=function(a,b,g,p){switch(arguments.length){case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,g);default:var k=null;if(3<arguments.length){for(var k=0,l=Array(arguments.length-3);k<l.length;)l[k]=arguments[k+3],++k;k=new H(l,0,null)}return h.w(a,b,g,k)}throw Error("Invalid arity: "+(arguments.length-1));};g.I=3;g.N=h.N;g.c=e;g.h=d;g.j=c;g.w=
h.w;return g}()}function Tw(a,b){return function d(b,g){return new sw(null,function(){var e=G(g);if(e){if(Kv(e)){for(var k=vu(e),l=P(k),m=ww(l),p=0;;)if(p<l)zw(m,function(){var d=b+p,e=xt.h(k,p);return a.h?a.h(d,e):a.call(null,d,e)}()),p+=1;else break;return yw(m.ob(),d(b+l,wu(e)))}return lv(function(){var d=I(e);return a.h?a.h(b,d):a.call(null,b,d)}(),d(b+1,Pu(e)))}return null},null,null)}(0,b)}function Uw(a,b,c,d){this.state=a;this.meta=b;this.Hd=c;this.sb=d;this.Y=16386;this.C=6455296}f=Uw.prototype;
f.equiv=function(a){return this.O(null,a)};f.O=function(a,b){return this===b};f.Ce=function(){return this.state};f.sa=function(){return this.meta};f.yh=function(a,b,c){for(var d,e=G(this.sb),g=null,h=0,k=0;;)if(k<h)d=g.$(null,k),a=S(d,0,null),d=S(d,1,null),d.L?d.L(a,this,b,c):d.call(null,a,this,b,c),k+=1;else if(a=G(e))e=a,Kv(e)?(g=vu(e),e=wu(e),a=g,d=P(g),g=a,h=d):(g=I(e),a=S(g,0,null),d=S(g,1,null),d.L?d.L(a,this,b,c):d.call(null,a,this,b,c),e=K(e),g=null,h=0),k=0;else return null};
f.xh=function(a,b,c){this.sb=T.j(this.sb,b,c);return this};f.zh=function(a,b){return this.sb=wv.h(this.sb,b)};f.ha=function(){return qa(this)};function Vw(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return Ww(arguments[0]);default:return c=arguments[0],b=new H(b.slice(1),0,null),d=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b,b=F.h(d,ct),d=F.h(d,Yw),new Uw(c,b,d,null)}}function Ww(a){return new Uw(a,null,null,null)}
function Zw(a,b){if(a instanceof Uw){var c=a.Hd;if(null!=c&&!z(c.c?c.c(b):c.call(null,b)))throw Error("Validator rejected reference state");c=a.state;a.state=b;null!=a.sb&&nu(a,c,b);return b}return xu(a,b)}
var $w=function $w(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return $w.h(arguments[0],arguments[1]);case 3:return $w.j(arguments[0],arguments[1],arguments[2]);case 4:return $w.L(arguments[0],arguments[1],arguments[2],arguments[3]);default:return $w.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};
$w.h=function(a,b){if(a instanceof Uw){var c=a.state;c=b.c?b.c(c):b.call(null,c);c=Zw(a,c)}else c=yu.h(a,b);return c};$w.j=function(a,b,c){if(a instanceof Uw){var d=a.state;b=b.h?b.h(d,c):b.call(null,d,c);a=Zw(a,b)}else a=yu.j(a,b,c);return a};$w.L=function(a,b,c,d){if(a instanceof Uw){var e=a.state;b=b.j?b.j(e,c,d):b.call(null,e,c,d);a=Zw(a,b)}else a=yu.L(a,b,c,d);return a};$w.w=function(a,b,c,d,e){return a instanceof Uw?Zw(a,U.ba(b,a.state,c,d,e)):yu.ba(a,b,c,d,e)};
$w.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return $w.w(b,a,c,d,e)};$w.I=4;
function ax(a,b){return function d(b,g){return new sw(null,function(){var e=G(g);if(e){if(Kv(e)){for(var k=vu(e),l=P(k),m=ww(l),p=0;;)if(p<l){var u=function(){var d=b+p,e=xt.h(k,p);return a.h?a.h(d,e):a.call(null,d,e)}();null!=u&&m.add(u);p+=1}else break;return yw(m.ob(),d(b+l,wu(e)))}l=function(){var d=I(e);return a.h?a.h(b,d):a.call(null,b,d)}();return null==l?d(b+1,Pu(e)):lv(l,d(b+1,Pu(e)))}return null},null,null)}(0,b)}
var Z=function Z(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Z.c(arguments[0]);case 2:return Z.h(arguments[0],arguments[1]);case 3:return Z.j(arguments[0],arguments[1],arguments[2]);case 4:return Z.L(arguments[0],arguments[1],arguments[2],arguments[3]);default:return Z.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};
Z.c=function(a){return function(b){return function(){function c(c,d){var e=a.c?a.c(d):a.call(null,d);return b.h?b.h(c,e):b.call(null,c,e)}function d(a){return b.c?b.c(a):b.call(null,a)}function e(){return b.T?b.T():b.call(null)}var g=null,h=function(){function c(a,b,c){var e=null;if(2<arguments.length){for(var e=0,g=Array(arguments.length-2);e<g.length;)g[e]=arguments[e+2],++e;e=new H(g,0,null)}return d.call(this,a,b,e)}function d(c,d,e){d=U.j(a,d,e);return b.h?b.h(c,d):b.call(null,c,d)}c.I=2;c.N=
function(a){var b=I(a);a=K(a);var c=I(a);a=Pu(a);return d(b,c,a)};c.w=d;return c}(),g=function(a,b,g){switch(arguments.length){case 0:return e.call(this);case 1:return d.call(this,a);case 2:return c.call(this,a,b);default:var k=null;if(2<arguments.length){for(var k=0,l=Array(arguments.length-2);k<l.length;)l[k]=arguments[k+2],++k;k=new H(l,0,null)}return h.w(a,b,k)}throw Error("Invalid arity: "+(arguments.length-1));};g.I=2;g.N=h.N;g.T=e;g.c=d;g.h=c;g.w=h.w;return g}()}};
Z.h=function(a,b){return new sw(null,function(){var c=G(b);if(c){if(Kv(c)){for(var d=vu(c),e=P(d),g=ww(e),h=0;;)if(h<e)zw(g,function(){var b=xt.h(d,h);return a.c?a.c(b):a.call(null,b)}()),h+=1;else break;return yw(g.ob(),Z.h(a,wu(c)))}return lv(function(){var b=I(c);return a.c?a.c(b):a.call(null,b)}(),Z.h(a,Pu(c)))}return null},null,null)};
Z.j=function(a,b,c){return new sw(null,function(){var d=G(b),e=G(c);if(d&&e){var g=lv;var h=I(d);var k=I(e);h=a.h?a.h(h,k):a.call(null,h,k);d=g(h,Z.j(a,Pu(d),Pu(e)))}else d=null;return d},null,null)};Z.L=function(a,b,c,d){return new sw(null,function(){var e=G(b),g=G(c),h=G(d);if(e&&g&&h){var k=lv;var l=I(e);var m=I(g),p=I(h);l=a.j?a.j(l,m,p):a.call(null,l,m,p);e=k(l,Z.L(a,Pu(e),Pu(g),Pu(h)))}else e=null;return e},null,null)};
Z.w=function(a,b,c,d,e){var g=function k(a){return new sw(null,function(){var b=Z.h(G,a);return Lw(Xv,b)?lv(Z.h(I,b),k(Z.h(Pu,b))):null},null,null)};return Z.h(function(){return function(b){return U.h(a,b)}}(g),g(sv.w(e,d,R([c,b],0))))};Z.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return Z.w(b,a,c,d,e)};Z.I=4;
function bx(a,b){if("number"!==typeof a)throw Error("Assert failed: (number? n)");return new sw(null,function(){if(0<a){var c=G(b);return c?lv(I(c),bx(a-1,Pu(c))):null}return null},null,null)}function cx(a,b){if("number"!==typeof a)throw Error("Assert failed: (number? n)");return new sw(null,function(c){return function(){return c(a,b)}}(function(a,b){for(;;){var c=G(b);if(0<a&&c){var d=a-1,c=Pu(c);a=d;b=c}else return c}}),null,null)}function dx(a){return Z.j(function(a){return a},a,cx(2,a))}
function ex(a){return new sw(null,function(){return lv(a,ex(a))},null,null)}function fx(a,b){return U.h(Dw,U.j(Z,a,b))}function gx(a,b){return new sw(null,function(){var c=G(b);if(c){if(Kv(c)){for(var d=vu(c),e=P(d),g=ww(e),h=0;;)if(h<e){var k=xt.h(d,h);k=a.c?a.c(k):a.call(null,k);z(k)&&(k=xt.h(d,h),g.add(k));h+=1}else break;return yw(g.ob(),gx(a,wu(c)))}d=I(c);c=Pu(c);return z(a.c?a.c(d):a.call(null,d))?lv(d,gx(a,c)):gx(a,c)}return null},null,null)}function hx(a,b){return gx(Nw(a),b)}
function ix(a){return function c(a){return new sw(null,function(){return lv(a,z(Gv.c?Gv.c(a):Gv.call(null,a))?fx(c,R([G.c?G.c(a):G.call(null,a)],0)):null)},null,null)}(a)}function jx(a){return gx(function(a){return!Gv(a)},Pu(ix(a)))}
var Rw=function Rw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Rw.T();case 1:return Rw.c(arguments[0]);case 2:return Rw.h(arguments[0],arguments[1]);case 3:return Rw.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};Rw.T=function(){return tv};Rw.c=function(a){return a};
Rw.h=function(a,b){return null!=a?null!=a&&(a.Y&4||x===a.bu)?Zt(su(nt(ru,qu(a),b)),Av(a)):nt(vt,a,b):nt(sv,Qu,b)};Rw.j=function(a,b,c){return null!=a&&(a.Y&4||x===a.bu)?Zt(su(Yv(b,Ew,qu(a),c)),Av(a)):Yv(b,sv,a,c)};Rw.I=3;function kx(a,b){return su(nt(function(b,d){return Ew.h(b,a.c?a.c(d):a.call(null,d))},qu(tv),b))}function lx(a,b,c){return new sw(null,function(){var d=G(c);if(d){var e=bx(a,d);return a===P(e)?lv(e,lx(a,b,cx(b,d))):null}return null},null,null)}
function mx(a,b,c,d){return new sw(null,function(){var e=G(d);if(e){var g=bx(a,e);if(a===P(g))return lv(g,mx(a,b,c,cx(b,e)));e=bx(a,Dw.h(g,c));return vt(Qu,e)}return null},null,null)}function nx(a,b){return nt(F,a,b)}function ox(a,b,c){var d=Nv;for(b=G(b);;)if(null!=b){a=F.j(a,I(b),d);if(d===a)return c;b=K(b)}else return a}
var px=function px(b,c,d){c=G(c);var e=I(c),g=K(c);return g?T.j(b,e,function(){var c=F.h(b,e);return px.j?px.j(c,g,d):px.call(null,c,g,d)}()):T.j(b,e,d)},qx=function qx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 3:return qx.j(arguments[0],arguments[1],arguments[2]);case 4:return qx.L(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return qx.ba(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);case 6:return qx.Ac(arguments[0],
arguments[1],arguments[2],arguments[3],arguments[4],arguments[5]);default:return qx.w(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5],new H(c.slice(6),0,null))}};qx.j=function(a,b,c){b=G(b);var d=I(b);return(b=K(b))?T.j(a,d,qx.j(F.h(a,d),b,c)):T.j(a,d,function(){var b=F.h(a,d);return c.c?c.c(b):c.call(null,b)}())};qx.L=function(a,b,c,d){b=G(b);var e=I(b);return(b=K(b))?T.j(a,e,qx.L(F.h(a,e),b,c,d)):T.j(a,e,function(){var b=F.h(a,e);return c.h?c.h(b,d):c.call(null,b,d)}())};
qx.ba=function(a,b,c,d,e){b=G(b);var g=I(b);return(b=K(b))?T.j(a,g,qx.ba(F.h(a,g),b,c,d,e)):T.j(a,g,function(){var b=F.h(a,g);return c.j?c.j(b,d,e):c.call(null,b,d,e)}())};qx.Ac=function(a,b,c,d,e,g){b=G(b);var h=I(b);return(b=K(b))?T.j(a,h,qx.Ac(F.h(a,h),b,c,d,e,g)):T.j(a,h,function(){var b=F.h(a,h);return c.L?c.L(b,d,e,g):c.call(null,b,d,e,g)}())};qx.w=function(a,b,c,d,e,g,h){var k=G(b);b=I(k);return(k=K(k))?T.j(a,b,U.w(qx,F.h(a,b),k,c,d,R([e,g,h],0))):T.j(a,b,U.w(c,F.h(a,b),d,e,g,R([h],0)))};
qx.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),g=K(e),e=I(g),h=K(g),g=I(h),h=K(h);return qx.w(b,a,c,d,e,g,h)};qx.I=6;function rx(a,b,c){return T.j(a,b,function(){var d=F.h(a,b);return c.c?c.c(d):c.call(null,d)}())}function sx(a,b){var c=tx,d=Qw();return T.j(a,c,function(){var e=F.h(a,c);return d.h?d.h(e,b):d.call(null,e,b)}())}function ux(a,b){this.Ba=a;this.v=b}
function vx(a){return new ux(a,[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null])}function wx(a){return new ux(a.Ba,lt(a.v))}function xx(a){a=a.G;return 32>a?0:a-1>>>5<<5}function yx(a,b,c){for(;;){if(0===b)return c;var d=vx(a);d.v[0]=c;c=d;b-=5}}
var zx=function zx(b,c,d,e){var g=wx(d),h=b.G-1>>>c&31;5===c?g.v[h]=e:(d=d.v[h],null!=d?(c-=5,b=zx.L?zx.L(b,c,d,e):zx.call(null,b,c,d,e)):b=yx(null,c-5,e),g.v[h]=b);return g};function Ax(a,b){throw Error([D.c("No item "),D.c(a),D.c(" in vector of length "),D.c(b)].join(""));}function Bx(a,b){if(b>=xx(a))return a.Mb;for(var c=a.root,d=a.shift;;)if(0<d)var e=d-5,c=c.v[b>>>d&31],d=e;else return c.v}function Cx(a,b){return 0<=b&&b<a.G?Bx(a,b):Ax(b,a.G)}
var Dx=function Dx(b,c,d,e,g){var h=wx(d);if(0===c)h.v[e&31]=g;else{var k=e>>>c&31;c-=5;d=d.v[k];b=Dx.ba?Dx.ba(b,c,d,e,g):Dx.call(null,b,c,d,e,g);h.v[k]=b}return h},Ex=function Ex(b,c,d){var e=b.G-2>>>c&31;if(5<c){c-=5;var g=d.v[e];b=Ex.j?Ex.j(b,c,g):Ex.call(null,b,c,g);if(null==b&&0===e)return null;d=wx(d);d.v[e]=b;return d}if(0===e)return null;d=wx(d);d.v[e]=null;return d};function Fx(a,b,c,d,e,g){this.i=a;this.base=b;this.v=c;this.Cc=d;this.start=e;this.end=g}
Fx.prototype.hb=function(){return this.i<this.end};Fx.prototype.next=function(){32===this.i-this.base&&(this.v=Bx(this.Cc,this.i),this.base+=32);var a=this.v[this.i&31];this.i+=1;return a};function Gx(a,b,c){return new Fx(b,b-b%32,b<P(a)?Bx(a,b):null,a,b,c)}function V(a,b,c,d,e,g){this.meta=a;this.G=b;this.shift=c;this.root=d;this.Mb=e;this.K=g;this.C=167668511;this.Y=8196}f=V.prototype;f.cj=x;f.uh=function(a,b){return new V(null,2,5,X,[b,F.h(this,b)],null)};f.toString=function(){return Bu(this)};
f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return"number"===typeof b?this.Zb(null,b,c):c};
f.lg=function(a,b,c){a=0;for(var d=c;;)if(a<this.G){var e=Bx(this,a);c=e.length;a:for(var g=0;;)if(g<c)var h=g+a,k=e[g],d=b.j?b.j(d,h,k):b.call(null,d,h,k),g=g+1;else{e=d;break a}a+=c;d=e}else return d};f.$=function(a,b){return Cx(this,b)[b&31]};f.Zb=function(a,b,c){return 0<=b&&b<this.G?Bx(this,b)[b&31]:c};
f.wh=function(a,b,c){if(0<=b&&b<this.G)return xx(this)<=b?(a=lt(this.Mb),a[b&31]=c,new V(this.meta,this.G,this.shift,this.root,a,null)):new V(this.meta,this.G,this.shift,Dx(this,this.shift,this.root,b,c),this.Mb,null);if(b===this.G)return this.wa(null,c);throw Error([D.c("Index "),D.c(b),D.c(" out of bounds  [0,"),D.c(this.G),D.c("]")].join(""));};f.ic=function(){return Gx(this,0,this.G)};f.sa=function(){return this.meta};f.xa=function(){return this.G};f.Sl=function(){return this.$(null,0)};
f.Tl=function(){return this.$(null,1)};f.ng=function(){return 0<this.G?this.$(null,this.G-1):null};f.og=function(){if(0===this.G)throw Error("Can't pop empty vector");if(1===this.G)return Zt(tv,this.meta);if(1<this.G-xx(this))return new V(this.meta,this.G-1,this.shift,this.root,this.Mb.slice(0,-1),null);var a=Bx(this,this.G-2),b=Ex(this,this.shift,this.root),b=null==b?X:b,c=this.G-1;return 5<this.shift&&null==b.v[1]?new V(this.meta,c,this.shift-5,b.v[0],a,null):new V(this.meta,c,this.shift,b,a,null)};
f.mg=function(){return 0<this.G?new jv(this,this.G-1,null):null};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){if(b instanceof V)if(this.G===P(b))for(var c=this.ic(null),d=zu(b);;)if(c.hb()){var e=c.next(),g=d.next();if(!M.h(e,g))return!1}else return!0;else return!1;else return kv(this,b)};f.kg=function(){return new Hx(this.G,this.shift,Ix.c?Ix.c(this.root):Ix.call(null,this.root),Jx.c?Jx.c(this.Mb):Jx.call(null,this.Mb))};f.Ja=function(){return Zt(tv,this.meta)};
f.xb=function(a,b){return Zu(this,b)};f.yb=function(a,b,c){a=0;for(var d=c;;)if(a<this.G){var e=Bx(this,a);c=e.length;a:for(var g=0;;)if(g<c)var h=e[g],d=b.h?b.h(d,h):b.call(null,d,h),g=g+1;else{e=d;break a}a+=c;d=e}else return d};f.Ob=function(a,b,c){if("number"===typeof b)return this.wh(null,b,c);throw Error("Vector's key for assoc must be a number.");};f.sh=function(a,b){return Sv(b)?0<=b&&b<this.G:!1};
f.oa=function(){if(0===this.G)return null;if(32>=this.G)return new H(this.Mb,0,null);a:{var a=this.root;for(var b=this.shift;;)if(0<b)b-=5,a=a.v[0];else{a=a.v;break a}}return Kx?Kx(this,a,0,0):Lx.call(null,this,a,0,0)};f.ya=function(a,b){return new V(b,this.G,this.shift,this.root,this.Mb,this.K)};
f.wa=function(a,b){if(32>this.G-xx(this)){for(var c=this.Mb.length,d=Array(c+1),e=0;;)if(e<c)d[e]=this.Mb[e],e+=1;else break;d[c]=b;return new V(this.meta,this.G+1,this.shift,this.root,d,null)}c=(d=this.G>>>5>1<<this.shift)?this.shift+5:this.shift;d?(d=vx(null),d.v[0]=this.root,e=yx(null,this.shift,new ux(null,this.Mb)),d.v[1]=e):d=zx(this,this.shift,this.root,new ux(null,this.Mb));return new V(this.meta,this.G+1,c,d,[b],null)};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.$(null,c);case 3:return this.Zb(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.$(null,c)};a.j=function(a,c,d){return this.Zb(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.$(null,a)};f.h=function(a,b){return this.Zb(null,a,b)};
var X=new ux(null,[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null]),tv=new V(null,0,5,X,[],Vu);function Mx(a){var b=a.length;if(32>b)return new V(null,b,5,X,a,null);for(var c=32,d=(new V(null,32,5,X,a.slice(0,32),null)).kg(null);;)if(c<b)var e=c+1,d=Ew.h(d,a[c]),c=e;else return su(d)}V.prototype[kt]=function(){return Su(this)};function Nx(a){return gt(a)?Mx(a):su(nt(ru,qu(tv),a))}
var Ox=function Ox(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return Ox.w(0<c.length?new H(c.slice(0),0,null):null)};Ox.w=function(a){return a instanceof H&&0===a.i?Mx(a.v):Nx(a)};Ox.I=0;Ox.N=function(a){return Ox.w(G(a))};function Px(a,b,c,d,e,g){this.Dc=a;this.node=b;this.i=c;this.rb=d;this.meta=e;this.K=g;this.C=32375020;this.Y=1536}f=Px.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.$b=function(){if(this.rb+1<this.node.length){var a=this.Dc;var b=this.node,c=this.i,d=this.rb+1;a=Kx?Kx(a,b,c,d):Lx.call(null,a,b,c,d);return null==a?null:a}return this.Sp(null)};
f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(tv,this.meta)};f.xb=function(a,b){var c=this.Dc;var d=this.i+this.rb,e=P(this.Dc);c=Qx.j?Qx.j(c,d,e):Qx.call(null,c,d,e);return Zu(c,b)};f.yb=function(a,b,c){a=this.Dc;var d=this.i+this.rb,e=P(this.Dc);a=Qx.j?Qx.j(a,d,e):Qx.call(null,a,d,e);return $u(a,b,c)};f.ab=function(){return this.node[this.rb]};
f.zb=function(){if(this.rb+1<this.node.length){var a=this.Dc;var b=this.node,c=this.i,d=this.rb+1;a=Kx?Kx(a,b,c,d):Lx.call(null,a,b,c,d);return null==a?Qu:a}return this.bj(null)};f.oa=function(){return this};f.Rl=function(){var a=this.node;return new vw(a,this.rb,a.length)};f.bj=function(){var a=this.i+this.node.length;if(a<st(this.Dc)){var b=this.Dc,c=Bx(this.Dc,a);return Kx?Kx(b,c,a,0):Lx.call(null,b,c,a,0)}return Qu};
f.ya=function(a,b){return Rx?Rx(this.Dc,this.node,this.i,this.rb,b):Lx.call(null,this.Dc,this.node,this.i,this.rb,b)};f.wa=function(a,b){return lv(b,this)};f.Sp=function(){var a=this.i+this.node.length;if(a<st(this.Dc)){var b=this.Dc,c=Bx(this.Dc,a);return Kx?Kx(b,c,a,0):Lx.call(null,b,c,a,0)}return null};Px.prototype[kt]=function(){return Su(this)};
function Lx(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 3:return b=arguments[0],c=arguments[1],d=arguments[2],new Px(b,Cx(b,c),c,d,null,null);case 4:return Kx(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return Rx(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}function Kx(a,b,c,d){return new Px(a,b,c,d,null,null)}
function Rx(a,b,c,d,e){return new Px(a,b,c,d,e,null)}function Sx(a,b,c,d,e){this.meta=a;this.Cc=b;this.start=c;this.end=d;this.K=e;this.C=167666463;this.Y=8192}f=Sx.prototype;f.cj=x;f.uh=function(a,b){return new V(null,2,5,X,[b,F.h(this,b)],null)};f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return"number"===typeof b?this.Zb(null,b,c):c};
f.lg=function(a,b,c){a=this.start;for(var d=0;;)if(a<this.end){var e=d,g=xt.h(this.Cc,a);c=b.j?b.j(c,e,g):b.call(null,c,e,g);d+=1;a+=1}else return c};f.$=function(a,b){return 0>b||this.end<=this.start+b?Ax(b,this.end-this.start):xt.h(this.Cc,this.start+b)};f.Zb=function(a,b,c){return 0>b||this.end<=this.start+b?c:xt.j(this.Cc,this.start+b,c)};
f.wh=function(a,b,c){a=this.start+b;if(0>b||this.end+1<=a)throw Error([D.c("Index "),D.c(b),D.c(" out of bounds [0,"),D.c(this.xa(null)),D.c("]")].join(""));b=this.meta;c=T.j(this.Cc,a,c);var d=this.start,e=this.end;a+=1;a=e>a?e:a;return Tx.ba?Tx.ba(b,c,d,a,null):Tx.call(null,b,c,d,a,null)};f.ic=function(){return Gx(this.Cc,this.start,this.end)};f.sa=function(){return this.meta};f.xa=function(){return this.end-this.start};f.ng=function(){return xt.h(this.Cc,this.end-1)};
f.og=function(){if(this.start===this.end)throw Error("Can't pop empty vector");var a=this.meta,b=this.Cc,c=this.start,d=this.end-1;return Tx.ba?Tx.ba(a,b,c,d,null):Tx.call(null,a,b,c,d,null)};f.mg=function(){return this.start!==this.end?new jv(this,this.end-this.start-1,null):null};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(tv,this.meta)};f.xb=function(a,b){return Zu(this,b)};
f.yb=function(a,b,c){return $u(this,b,c)};f.Ob=function(a,b,c){if("number"===typeof b)return this.wh(null,b,c);throw Error("Subvec's key for assoc must be a number.");};f.oa=function(){var a=this;return function(b){return function d(e){return e===a.end?null:lv(xt.h(a.Cc,e),new sw(null,function(){return function(){return d(e+1)}}(b),null,null))}}(this)(a.start)};f.ya=function(a,b){return Tx.ba?Tx.ba(b,this.Cc,this.start,this.end,this.K):Tx.call(null,b,this.Cc,this.start,this.end,this.K)};
f.wa=function(a,b){var c=this.meta,d=Tt(this.Cc,this.end,b),e=this.start,g=this.end+1;return Tx.ba?Tx.ba(c,d,e,g,null):Tx.call(null,c,d,e,g,null)};f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.$(null,c);case 3:return this.Zb(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.$(null,c)};a.j=function(a,c,d){return this.Zb(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};
f.c=function(a){return this.$(null,a)};f.h=function(a,b){return this.Zb(null,a,b)};Sx.prototype[kt]=function(){return Su(this)};function Tx(a,b,c,d,e){for(;;)if(b instanceof Sx)c=b.start+c,d=b.start+d,b=b.Cc;else{if(!Jv(b))throw Error("v must satisfy IVector");var g=P(b);if(0>c||0>d||c>g||d>g)throw Error("Index out of bounds");return new Sx(a,b,c,d,e)}}
var Qx=function Qx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return Qx.h(arguments[0],arguments[1]);case 3:return Qx.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};Qx.h=function(a,b){return Qx.j(a,b,P(a))};Qx.j=function(a,b,c){return Tx(null,a,b,c,null)};Qx.I=3;function Ux(a,b){return a===b.Ba?b:new ux(a,lt(b.v))}function Ix(a){return new ux({},lt(a.v))}
function Jx(a){var b=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];Mv(a,0,b,0,a.length);return b}var Vx=function Vx(b,c,d,e){d=Ux(b.root.Ba,d);var g=b.G-1>>>c&31;if(5===c)b=e;else{var h=d.v[g];null!=h?(c-=5,b=Vx.L?Vx.L(b,c,h,e):Vx.call(null,b,c,h,e)):b=yx(b.root.Ba,c-5,e)}d.v[g]=b;return d};function Hx(a,b,c,d){this.G=a;this.shift=b;this.root=c;this.Mb=d;this.Y=88;this.C=275}f=Hx.prototype;
f.nf=function(a,b){if(this.root.Ba){if(32>this.G-xx(this))this.Mb[this.G&31]=b;else{var c=new ux(this.root.Ba,this.Mb),d=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];d[0]=b;this.Mb=d;if(this.G>>>5>1<<this.shift){var d=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null],e=this.shift+
5;d[0]=this.root;d[1]=yx(this.root.Ba,this.shift,c);this.root=new ux(this.root.Ba,d);this.shift=e}else this.root=Vx(this,this.shift,this.root,c)}this.G+=1;return this}throw Error("conj! after persistent!");};f.vh=function(){if(this.root.Ba){this.root.Ba=null;var a=this.G-xx(this),b=Array(a);Mv(this.Mb,0,b,0,a);return new V(null,this.G,this.shift,this.root,b,null)}throw Error("persistent! called twice");};
f.mf=function(a,b,c){if("number"===typeof b)return Wx(this,b,c);throw Error("TransientVector's key for assoc! must be a number.");};
function Wx(a,b,c){if(a.root.Ba){if(0<=b&&b<a.G){if(xx(a)<=b)a.Mb[b&31]=c;else{var d=function(){return function g(d,k){var h=Ux(a.root.Ba,k);if(0===d)h.v[b&31]=c;else{var m=b>>>d&31,p=g(d-5,h.v[m]);h.v[m]=p}return h}}(a).call(null,a.shift,a.root);a.root=d}return a}if(b===a.G)return a.nf(null,c);throw Error([D.c("Index "),D.c(b),D.c(" out of bounds for TransientVector of length"),D.c(a.G)].join(""));}throw Error("assoc! after persistent!");}
f.xa=function(){if(this.root.Ba)return this.G;throw Error("count after persistent!");};f.$=function(a,b){if(this.root.Ba)return Cx(this,b)[b&31];throw Error("nth after persistent!");};f.Zb=function(a,b,c){return 0<=b&&b<this.G?this.$(null,b):c};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return"number"===typeof b?this.Zb(null,b,c):c};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};function Xx(a,b){this.tg=a;this.ni=b}
Xx.prototype.hb=function(){var a=null!=this.tg&&G(this.tg);return a?a:(a=null!=this.ni)?this.ni.hb():a};Xx.prototype.next=function(){if(null!=this.tg){var a=I(this.tg);this.tg=K(this.tg);return a}if(null!=this.ni&&this.ni.hb())return this.ni.next();throw Error("No such element");};Xx.prototype.remove=function(){return Error("Unsupported operation")};function Yx(a,b,c,d){this.meta=a;this.qc=b;this.kd=c;this.K=d;this.C=31850572;this.Y=0}f=Yx.prototype;f.toString=function(){return Bu(this)};
f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.ab=function(){return I(this.qc)};
f.zb=function(){var a=K(this.qc);return a?new Yx(this.meta,a,this.kd,null):null==this.kd?this.Ja(null):new Yx(this.meta,this.kd,null,null)};f.oa=function(){return this};f.ya=function(a,b){return new Yx(b,this.qc,this.kd,this.K)};f.wa=function(a,b){return lv(b,this)};Yx.prototype[kt]=function(){return Su(this)};function Zx(a,b,c,d,e){this.meta=a;this.count=b;this.qc=c;this.kd=d;this.K=e;this.C=31858766;this.Y=8192}f=Zx.prototype;f.toString=function(){return Bu(this)};
f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,this.count.c?this.count.c(this):this.count.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.ic=function(){return new Xx(this.qc,zu(this.kd))};f.sa=function(){return this.meta};f.xa=function(){return this.count};f.ng=function(){return I(this.qc)};
f.og=function(){if(z(this.qc)){var a=K(this.qc);return a?new Zx(this.meta,this.count-1,a,this.kd,null):new Zx(this.meta,this.count-1,G(this.kd),tv,null)}return this};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt($x,this.meta)};f.ab=function(){return I(this.qc)};f.zb=function(){return Pu(G(this))};f.oa=function(){var a=G(this.kd),b=this.qc;return z(z(b)?b:a)?new Yx(null,this.qc,G(a),null):null};
f.ya=function(a,b){return new Zx(b,this.count,this.qc,this.kd,this.K)};f.wa=function(a,b){if(z(this.qc)){var c=this.kd;c=new Zx(this.meta,this.count+1,this.qc,sv.h(z(c)?c:tv,b),null)}else c=new Zx(this.meta,this.count+1,sv.h(this.qc,b),tv,null);return c};var $x=new Zx(null,0,null,tv,Vu);Zx.prototype[kt]=function(){return Su(this)};function ay(){this.C=2097152;this.Y=0}ay.prototype.equiv=function(a){return this.O(null,a)};ay.prototype.O=function(){return!1};var by=new ay;
function cy(a,b){return Qv(Hv(b)?P(a)===P(b)?Lw(function(a){return M.h(F.j(b,I(a),by),qv(a))},a):null:null)}function dy(a,b,c,d,e){this.i=a;this.Qv=b;this.Ip=c;this.gb=d;this.sq=e}dy.prototype.hb=function(){var a=this.i<this.Ip;return a?a:this.sq.hb()};dy.prototype.next=function(){if(this.i<this.Ip){var a=fv(this.gb,this.i);this.i+=1;return new V(null,2,5,X,[a,Dt.h(this.Qv,a)],null)}return this.sq.next()};dy.prototype.remove=function(){return Error("Unsupported operation")};
function ey(a){this.s=a}ey.prototype.next=function(){if(null!=this.s){var a=I(this.s),b=S(a,0,null),a=S(a,1,null);this.s=K(this.s);return{value:[b,a],done:!1}}return{value:null,done:!0}};function fy(a){this.s=a}fy.prototype.next=function(){if(null!=this.s){var a=I(this.s);this.s=K(this.s);return{value:[a,a],done:!1}}return{value:null,done:!0}};
function gy(a,b){if(b instanceof Y)a:{var c=a.length;for(var d=b.Sb,e=0;;){if(c<=e){c=-1;break a}if(a[e]instanceof Y&&d===a[e].Sb){c=e;break a}e+=2}}else if(da(b)||"number"===typeof b)a:for(c=a.length,d=0;;){if(c<=d){c=-1;break a}if(b===a[d]){c=d;break a}d+=2}else if(b instanceof Nu)a:for(c=a.length,d=b.Dd,e=0;;){if(c<=e){c=-1;break a}if(a[e]instanceof Nu&&d===a[e].Dd){c=e;break a}e+=2}else if(null==b)a:for(c=a.length,d=0;;){if(c<=d){c=-1;break a}if(null==a[d]){c=d;break a}d+=2}else a:for(c=a.length,
d=0;;){if(c<=d){c=-1;break a}if(M.h(b,a[d])){c=d;break a}d+=2}return c}function hy(a,b,c){this.v=a;this.i=b;this.dc=c;this.C=32374990;this.Y=0}f=hy.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.dc};f.$b=function(){return this.i<this.v.length-2?new hy(this.v,this.i+2,this.dc):null};f.xa=function(){return(this.v.length-this.i)/2};f.ha=function(){return Uu(this)};
f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.dc)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return new V(null,2,5,X,[this.v[this.i],this.v[this.i+1]],null)};f.zb=function(){return this.i<this.v.length-2?new hy(this.v,this.i+2,this.dc):Qu};f.oa=function(){return this};f.ya=function(a,b){return new hy(this.v,this.i,b)};f.wa=function(a,b){return lv(b,this)};hy.prototype[kt]=function(){return Su(this)};
function iy(a,b,c){this.v=a;this.i=b;this.G=c}iy.prototype.hb=function(){return this.i<this.G};iy.prototype.next=function(){var a=new V(null,2,5,X,[this.v[this.i],this.v[this.i+1]],null);this.i+=2;return a};function y(a,b,c,d){this.meta=a;this.G=b;this.v=c;this.K=d;this.C=16647951;this.Y=8196}f=y.prototype;f.cj=x;f.uh=function(a,b){var c=gy(this.v,b);return new V(null,2,5,X,[this.v[c],F.h(this,b)],null)};f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.keys=function(){return Su(jy.c?jy.c(this):jy.call(null,this))};f.entries=function(){return new ey(G(G(this)))};f.values=function(){return Su(ky.c?ky.c(this):ky.call(null,this))};f.has=function(a){return Tv(this,a)};f.get=function(a,b){return this.X(null,a,b)};
f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.$(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,Kv(d)?(e=vu(d),d=wu(d),c=e,b=P(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){a=gy(this.v,b);return-1===a?c:this.v[a+1]};
f.lg=function(a,b,c){a=this.v.length;for(var d=0;;)if(d<a){var e=this.v[d],g=this.v[d+1];c=b.j?b.j(c,e,g):b.call(null,c,e,g);d+=2}else return c};f.ic=function(){return new iy(this.v,0,2*this.G)};f.sa=function(){return this.meta};f.xa=function(){return this.G};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Wu(this)};
f.O=function(a,b){if(null!=b&&(b.C&1024||x===b.Wp)){var c=this.v.length;if(this.G===b.xa(null))for(var d=0;;)if(d<c){var e=b.X(null,this.v[d],Nv);if(e!==Nv)if(M.h(this.v[d+1],e))d+=2;else return!1;else return!1}else return!0;else return!1}else return cy(this,b)};f.kg=function(){return new ly({},this.v.length,lt(this.v))};f.Ja=function(){return Zt(Kw,this.meta)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};
f.lf=function(a,b){if(0<=gy(this.v,b)){var c=this.v.length,d=c-2;if(0===d)return this.Ja(null);for(var d=Array(d),e=0,g=0;;){if(e>=c)return new y(this.meta,this.G-1,d,null);M.h(b,this.v[e])||(d[g]=this.v[e],d[g+1]=this.v[e+1],g+=2);e+=2}}else return this};
f.Ob=function(a,b,c){a=gy(this.v,b);if(-1===a){if(this.G<my){a=this.v;for(var d=a.length,e=Array(d+2),g=0;;)if(g<d)e[g]=a[g],g+=1;else break;e[d]=b;e[d+1]=c;return new y(this.meta,this.G+1,e,null)}return Zt(Gt(Rw.h(ny,this),b,c),this.meta)}if(c===this.v[a+1])return this;b=lt(this.v);b[a+1]=c;return new y(this.meta,this.G,b,null)};f.sh=function(a,b){return-1!==gy(this.v,b)};f.oa=function(){var a=this.v;return 0<=a.length-2?new hy(a,0,null):null};f.ya=function(a,b){return new y(b,this.G,this.v,this.K)};
f.wa=function(a,b){if(Jv(b))return this.Ob(null,xt.h(b,0),xt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(Jv(e))c=c.Ob(null,xt.h(e,0),xt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};var Kw=new y(null,0,[],Xu),my=8;
function vv(a){for(var b=[],c=0;;)if(c<a.length){var d=a[c],e=a[c+1],g=gy(b,d);-1===g?(g=b,g.push(d),g.push(e)):b[g+1]=e;c+=2}else break;return new y(null,b.length/2,b,null)}y.prototype[kt]=function(){return Su(this)};function ly(a,b,c){this.sg=a;this.Of=b;this.v=c;this.C=258;this.Y=56}f=ly.prototype;f.xa=function(){if(z(this.sg))return dw(this.Of);throw Error("count after persistent!");};f.na=function(a,b){return this.X(null,b,null)};
f.X=function(a,b,c){if(z(this.sg))return a=gy(this.v,b),-1===a?c:this.v[a+1];throw Error("lookup after persistent!");};f.nf=function(a,b){if(z(this.sg)){if(null!=b?b.C&2048||x===b.iu||(b.C?0:C(Lt,b)):C(Lt,b))return this.mf(null,hw.c?hw.c(b):hw.call(null,b),iw.c?iw.c(b):iw.call(null,b));for(var c=G(b),d=this;;){var e=I(c);if(z(e))c=K(c),d=d.mf(null,hw.c?hw.c(e):hw.call(null,e),iw.c?iw.c(e):iw.call(null,e));else return d}}else throw Error("conj! after persistent!");};
f.vh=function(){if(z(this.sg))return this.sg=!1,new y(null,dw(this.Of),this.v,null);throw Error("persistent! called twice");};f.mf=function(a,b,c){if(z(this.sg)){a=gy(this.v,b);if(-1===a){if(this.Of+2<=2*my)return this.Of+=2,this.v.push(b),this.v.push(c),this;a=oy.h?oy.h(this.Of,this.v):oy.call(null,this.Of,this.v);return tu(a,b,c)}c!==this.v[a+1]&&(this.v[a+1]=c);return this}throw Error("assoc! after persistent!");};
function oy(a,b){for(var c=qu(ny),d=0;;)if(d<a)c=tu(c,b[d],b[d+1]),d+=2;else return c}function py(){this.Gd=!1}function qy(a,b){return a===b?!0:pw(a,b)?!0:M.h(a,b)}function ry(a,b,c){a=lt(a);a[b]=c;return a}function sy(a,b){var c=Array(a.length-2);Mv(a,0,c,0,2*b);Mv(a,2*(b+1),c,2*b,c.length-2*b);return c}function ty(a,b,c,d){a=a.vf(b);a.v[c]=d;return a}
function uy(a,b,c){for(var d=a.length,e=0,g=c;;)if(e<d){c=a[e];if(null!=c){var h=a[e+1];c=b.j?b.j(g,c,h):b.call(null,g,c,h)}else c=a[e+1],c=null!=c?c.$h(b,g):g;e+=2;g=c}else return g}function vy(a,b,c,d){this.v=a;this.i=b;this.bi=c;this.fd=d}vy.prototype.advance=function(){for(var a=this.v.length;;)if(this.i<a){var b=this.v[this.i],c=this.v[this.i+1];null!=b?b=this.bi=new V(null,2,5,X,[b,c],null):null!=c?(b=zu(c),b=b.hb()?this.fd=b:!1):b=!1;this.i+=2;if(b)return!0}else return!1};
vy.prototype.hb=function(){var a=null!=this.bi;return a?a:(a=null!=this.fd)?a:this.advance()};vy.prototype.next=function(){if(null!=this.bi){var a=this.bi;this.bi=null;return a}if(null!=this.fd)return a=this.fd.next(),this.fd.hb()||(this.fd=null),a;if(this.advance())return this.next();throw Error("No such element");};vy.prototype.remove=function(){return Error("Unsupported operation")};function wy(a,b,c){this.Ba=a;this.Da=b;this.v=c}f=wy.prototype;
f.vf=function(a){if(a===this.Ba)return this;var b=ew(this.Da),c=Array(0>b?4:2*(b+1));Mv(this.v,0,c,0,2*b);return new wy(a,this.Da,c)};f.Yh=function(){return xy?xy(this.v):yy.call(null,this.v)};f.$h=function(a,b){return uy(this.v,a,b)};f.Pe=function(a,b,c,d){var e=1<<(b>>>a&31);if(0===(this.Da&e))return d;var g=ew(this.Da&e-1),e=this.v[2*g],g=this.v[2*g+1];return null==e?g.Pe(a+5,b,c,d):qy(c,e)?g:d};
f.ed=function(a,b,c,d,e,g){var h=1<<(c>>>b&31),k=ew(this.Da&h-1);if(0===(this.Da&h)){var l=ew(this.Da);if(2*l<this.v.length){a=this.vf(a);b=a.v;g.Gd=!0;a:for(c=2*(l-k),g=2*k+(c-1),l=2*(k+1)+(c-1);;){if(0===c)break a;b[l]=b[g];--l;--c;--g}b[2*k]=d;b[2*k+1]=e;a.Da|=h;return a}if(16<=l){k=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];k[c>>>b&31]=zy.ed(a,b+5,c,d,e,g);for(e=d=0;;)if(32>d)0!==
(this.Da>>>d&1)&&(k[d]=null!=this.v[e]?zy.ed(a,b+5,Ku(this.v[e]),this.v[e],this.v[e+1],g):this.v[e+1],e+=2),d+=1;else break;return new Ay(a,l+1,k)}b=Array(2*(l+4));Mv(this.v,0,b,0,2*k);b[2*k]=d;b[2*k+1]=e;Mv(this.v,2*k,b,2*(k+1),2*(l-k));g.Gd=!0;a=this.vf(a);a.v=b;a.Da|=h;return a}l=this.v[2*k];h=this.v[2*k+1];if(null==l)return l=h.ed(a,b+5,c,d,e,g),l===h?this:ty(this,a,2*k+1,l);if(qy(d,l))return e===h?this:ty(this,a,2*k+1,e);g.Gd=!0;g=b+5;d=By?By(a,g,l,h,c,d,e):Cy.call(null,a,g,l,h,c,d,e);e=2*k;
k=2*k+1;a=this.vf(a);a.v[e]=null;a.v[k]=d;return a};
f.cd=function(a,b,c,d,e){var g=1<<(b>>>a&31),h=ew(this.Da&g-1);if(0===(this.Da&g)){var k=ew(this.Da);if(16<=k){h=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];h[b>>>a&31]=zy.cd(a+5,b,c,d,e);for(d=c=0;;)if(32>c)0!==(this.Da>>>c&1)&&(h[c]=null!=this.v[d]?zy.cd(a+5,Ku(this.v[d]),this.v[d],this.v[d+1],e):this.v[d+1],d+=2),c+=1;else break;return new Ay(null,k+1,h)}a=Array(2*(k+1));Mv(this.v,
0,a,0,2*h);a[2*h]=c;a[2*h+1]=d;Mv(this.v,2*h,a,2*(h+1),2*(k-h));e.Gd=!0;return new wy(null,this.Da|g,a)}var l=this.v[2*h],g=this.v[2*h+1];if(null==l)return k=g.cd(a+5,b,c,d,e),k===g?this:new wy(null,this.Da,ry(this.v,2*h+1,k));if(qy(c,l))return d===g?this:new wy(null,this.Da,ry(this.v,2*h+1,d));e.Gd=!0;e=this.Da;k=this.v;a+=5;a=Dy?Dy(a,l,g,b,c,d):Cy.call(null,a,l,g,b,c,d);c=2*h;h=2*h+1;d=lt(k);d[c]=null;d[h]=a;return new wy(null,e,d)};
f.Xh=function(a,b,c,d){var e=1<<(b>>>a&31);if(0===(this.Da&e))return d;var g=ew(this.Da&e-1),e=this.v[2*g],g=this.v[2*g+1];return null==e?g.Xh(a+5,b,c,d):qy(c,e)?new V(null,2,5,X,[e,g],null):d};
f.Zh=function(a,b,c){var d=1<<(b>>>a&31);if(0===(this.Da&d))return this;var e=ew(this.Da&d-1),g=this.v[2*e],h=this.v[2*e+1];return null==g?(a=h.Zh(a+5,b,c),a===h?this:null!=a?new wy(null,this.Da,ry(this.v,2*e+1,a)):this.Da===d?null:new wy(null,this.Da^d,sy(this.v,e))):qy(c,g)?new wy(null,this.Da^d,sy(this.v,e)):this};f.ic=function(){return new vy(this.v,0,null,null)};var zy=new wy(null,0,[]);function Ey(a,b,c){this.v=a;this.i=b;this.fd=c}
Ey.prototype.hb=function(){for(var a=this.v.length;;){if(null!=this.fd&&this.fd.hb())return!0;if(this.i<a){var b=this.v[this.i];this.i+=1;null!=b&&(this.fd=zu(b))}else return!1}};Ey.prototype.next=function(){if(this.hb())return this.fd.next();throw Error("No such element");};Ey.prototype.remove=function(){return Error("Unsupported operation")};function Ay(a,b,c){this.Ba=a;this.G=b;this.v=c}f=Ay.prototype;f.vf=function(a){return a===this.Ba?this:new Ay(a,this.G,lt(this.v))};
f.Yh=function(){return Fy?Fy(this.v):Gy.call(null,this.v)};f.$h=function(a,b){for(var c=this.v.length,d=0,e=b;;)if(d<c){var g=this.v[d];null!=g&&(e=g.$h(a,e));d+=1}else return e};f.Pe=function(a,b,c,d){var e=this.v[b>>>a&31];return null!=e?e.Pe(a+5,b,c,d):d};f.ed=function(a,b,c,d,e,g){var h=c>>>b&31,k=this.v[h];if(null==k)return a=ty(this,a,h,zy.ed(a,b+5,c,d,e,g)),a.G+=1,a;b=k.ed(a,b+5,c,d,e,g);return b===k?this:ty(this,a,h,b)};
f.cd=function(a,b,c,d,e){var g=b>>>a&31,h=this.v[g];if(null==h)return new Ay(null,this.G+1,ry(this.v,g,zy.cd(a+5,b,c,d,e)));a=h.cd(a+5,b,c,d,e);return a===h?this:new Ay(null,this.G,ry(this.v,g,a))};f.Xh=function(a,b,c,d){var e=this.v[b>>>a&31];return null!=e?e.Xh(a+5,b,c,d):d};
f.Zh=function(a,b,c){var d=b>>>a&31,e=this.v[d];if(null!=e){a=e.Zh(a+5,b,c);if(a===e)d=this;else if(null==a)if(8>=this.G)a:{e=this.v;a=e.length;b=Array(2*(this.G-1));c=0;for(var g=1,h=0;;)if(c<a)c!==d&&null!=e[c]&&(b[g]=e[c],g+=2,h|=1<<c),c+=1;else{d=new wy(null,h,b);break a}}else d=new Ay(null,this.G-1,ry(this.v,d,a));else d=new Ay(null,this.G,ry(this.v,d,a));return d}return this};f.ic=function(){return new Ey(this.v,0,null)};
function Hy(a,b,c){b*=2;for(var d=0;;)if(d<b){if(qy(c,a[d]))return d;d+=2}else return-1}function Iy(a,b,c,d){this.Ba=a;this.Zd=b;this.G=c;this.v=d}f=Iy.prototype;f.vf=function(a){if(a===this.Ba)return this;var b=Array(2*(this.G+1));Mv(this.v,0,b,0,2*this.G);return new Iy(a,this.Zd,this.G,b)};f.Yh=function(){return xy?xy(this.v):yy.call(null,this.v)};f.$h=function(a,b){return uy(this.v,a,b)};f.Pe=function(a,b,c,d){a=Hy(this.v,this.G,c);return 0>a?d:qy(c,this.v[a])?this.v[a+1]:d};
f.ed=function(a,b,c,d,e,g){if(c===this.Zd){b=Hy(this.v,this.G,d);if(-1===b){if(this.v.length>2*this.G)return b=2*this.G,c=2*this.G+1,a=this.vf(a),a.v[b]=d,a.v[c]=e,g.Gd=!0,a.G+=1,a;c=this.v.length;b=Array(c+2);Mv(this.v,0,b,0,c);b[c]=d;b[c+1]=e;g.Gd=!0;d=this.G+1;a===this.Ba?(this.v=b,this.G=d,a=this):a=new Iy(this.Ba,this.Zd,d,b);return a}return this.v[b+1]===e?this:ty(this,a,b+1,e)}return(new wy(a,1<<(this.Zd>>>b&31),[null,this,null,null])).ed(a,b,c,d,e,g)};
f.cd=function(a,b,c,d,e){return b===this.Zd?(a=Hy(this.v,this.G,c),-1===a?(a=2*this.G,b=Array(a+2),Mv(this.v,0,b,0,a),b[a]=c,b[a+1]=d,e.Gd=!0,new Iy(null,this.Zd,this.G+1,b)):M.h(this.v[a+1],d)?this:new Iy(null,this.Zd,this.G,ry(this.v,a+1,d))):(new wy(null,1<<(this.Zd>>>a&31),[null,this])).cd(a,b,c,d,e)};f.Xh=function(a,b,c,d){a=Hy(this.v,this.G,c);return 0>a?d:qy(c,this.v[a])?new V(null,2,5,X,[this.v[a],this.v[a+1]],null):d};
f.Zh=function(a,b,c){a=Hy(this.v,this.G,c);return-1===a?this:1===this.G?null:new Iy(null,this.Zd,this.G-1,sy(this.v,dw(a)))};f.ic=function(){return new vy(this.v,0,null,null)};
function Cy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 6:return Dy(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5]);case 7:return By(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5],arguments[6]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}
function Dy(a,b,c,d,e,g){var h=Ku(b);if(h===d)return new Iy(null,h,2,[b,c,e,g]);var k=new py;return zy.cd(a,h,b,c,k).cd(a,d,e,g,k)}function By(a,b,c,d,e,g,h){var k=Ku(c);if(k===e)return new Iy(null,k,2,[c,d,g,h]);var l=new py;return zy.ed(a,b,k,c,d,l).ed(a,b,e,g,h,l)}function Jy(a,b,c,d,e){this.meta=a;this.nodes=b;this.i=c;this.s=d;this.K=e;this.C=32374860;this.Y=0}f=Jy.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv(b,this)};
f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return null==this.s?new V(null,2,5,X,[this.nodes[this.i],this.nodes[this.i+1]],null):I(this.s)};f.zb=function(){var a=this,b=null==a.s?function(){var b=a.nodes,d=a.i+2;return Ky?Ky(b,d,null):yy.call(null,b,d,null)}():function(){var b=a.nodes,d=a.i,e=K(a.s);return Ky?Ky(b,d,e):yy.call(null,b,d,e)}();return null!=b?b:Qu};f.oa=function(){return this};f.ya=function(a,b){return new Jy(b,this.nodes,this.i,this.s,this.K)};
f.wa=function(a,b){return lv(b,this)};Jy.prototype[kt]=function(){return Su(this)};function yy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return xy(arguments[0]);case 3:return Ky(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}function xy(a){return Ky(a,0,null)}
function Ky(a,b,c){if(null==c)for(c=a.length;;)if(b<c){if(null!=a[b])return new Jy(null,a,b,null,null);var d=a[b+1];if(z(d)&&(d=d.Yh(),z(d)))return new Jy(null,a,b+2,d,null);b+=2}else return null;else return new Jy(null,a,b,c,null)}function Ly(a,b,c,d,e){this.meta=a;this.nodes=b;this.i=c;this.s=d;this.K=e;this.C=32374860;this.Y=0}f=Ly.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return nv(b,this)};
f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return I(this.s)};f.zb=function(){var a=this.nodes;var b=this.i,c=K(this.s);a=My?My(null,a,b,c):Gy.call(null,null,a,b,c);return null!=a?a:Qu};f.oa=function(){return this};f.ya=function(a,b){return new Ly(b,this.nodes,this.i,this.s,this.K)};f.wa=function(a,b){return lv(b,this)};Ly.prototype[kt]=function(){return Su(this)};
function Gy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return Fy(arguments[0]);case 4:return My(arguments[0],arguments[1],arguments[2],arguments[3]);default:throw Error([D.c("Invalid arity: "),D.c(b.length)].join(""));}}function Fy(a){return My(null,a,0,null)}
function My(a,b,c,d){if(null==d)for(d=b.length;;)if(c<d){var e=b[c];if(z(e)&&(e=e.Yh(),z(e)))return new Ly(a,b,c+1,e,null);c+=1}else return null;else return new Ly(a,b,c,d,null)}function Ny(a,b,c){this.Jb=a;this.Sr=b;this.oo=c}Ny.prototype.hb=function(){return!this.oo||this.Sr.hb()};Ny.prototype.next=function(){if(this.oo)return this.Sr.next();this.oo=!0;return new V(null,2,5,X,[null,this.Jb],null)};Ny.prototype.remove=function(){return Error("Unsupported operation")};
function Oy(a,b,c,d,e,g){this.meta=a;this.G=b;this.root=c;this.Ib=d;this.Jb=e;this.K=g;this.C=16123663;this.Y=8196}f=Oy.prototype;f.cj=x;f.uh=function(a,b){return this.Ib?new V(null,2,5,X,[null,this.Jb],null):this.root.Xh(0,Ku(b),b,null)};f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};f.keys=function(){return Su(jy.c?jy.c(this):jy.call(null,this))};f.entries=function(){return new ey(G(G(this)))};f.values=function(){return Su(ky.c?ky.c(this):ky.call(null,this))};
f.has=function(a){return Tv(this,a)};f.get=function(a,b){return this.X(null,a,b)};f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.$(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,Kv(d)?(e=vu(d),d=wu(d),c=e,b=P(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.X(null,b,null)};
f.X=function(a,b,c){return null==b?this.Ib?this.Jb:c:null==this.root?c:this.root.Pe(0,Ku(b),b,c)};f.lg=function(a,b,c){a=this.Ib?b.j?b.j(c,null,this.Jb):b.call(null,c,null,this.Jb):c;return null!=this.root?this.root.$h(b,a):a};f.ic=function(){var a=this.root?zu(this.root):Iw();return this.Ib?new Ny(this.Jb,a,!1):a};f.sa=function(){return this.meta};f.xa=function(){return this.G};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Wu(this)};f.O=function(a,b){return cy(this,b)};
f.kg=function(){return new Py({},this.root,this.G,this.Ib,this.Jb)};f.Ja=function(){return Zt(ny,this.meta)};f.lf=function(a,b){if(null==b)return this.Ib?new Oy(this.meta,this.G-1,this.root,!1,null,null):this;if(null==this.root)return this;var c=this.root.Zh(0,Ku(b),b);return c===this.root?this:new Oy(this.meta,this.G-1,c,this.Ib,this.Jb,null)};
f.Ob=function(a,b,c){if(null==b)return this.Ib&&c===this.Jb?this:new Oy(this.meta,this.Ib?this.G:this.G+1,this.root,!0,c,null);a=new py;b=(null==this.root?zy:this.root).cd(0,Ku(b),b,c,a);return b===this.root?this:new Oy(this.meta,a.Gd?this.G+1:this.G,b,this.Ib,this.Jb,null)};f.sh=function(a,b){return null==b?this.Ib:null==this.root?!1:this.root.Pe(0,Ku(b),b,Nv)!==Nv};f.oa=function(){if(0<this.G){var a=null!=this.root?this.root.Yh():null;return this.Ib?lv(new V(null,2,5,X,[null,this.Jb],null),a):a}return null};
f.ya=function(a,b){return new Oy(b,this.G,this.root,this.Ib,this.Jb,this.K)};f.wa=function(a,b){if(Jv(b))return this.Ob(null,xt.h(b,0),xt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(Jv(e))c=c.Ob(null,xt.h(e,0),xt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};var ny=new Oy(null,0,null,!1,null,Xu);Oy.prototype[kt]=function(){return Su(this)};
function Py(a,b,c,d,e){this.Ba=a;this.root=b;this.count=c;this.Ib=d;this.Jb=e;this.C=258;this.Y=56}function Qy(a,b,c){if(a.Ba){if(null==b)a.Jb!==c&&(a.Jb=c),a.Ib||(a.count+=1,a.Ib=!0);else{var d=new py;b=(null==a.root?zy:a.root).ed(a.Ba,0,Ku(b),b,c,d);b!==a.root&&(a.root=b);d.Gd&&(a.count+=1)}return a}throw Error("assoc! after persistent!");}f=Py.prototype;f.xa=function(){if(this.Ba)return this.count;throw Error("count after persistent!");};
f.na=function(a,b){return null==b?this.Ib?this.Jb:null:null==this.root?null:this.root.Pe(0,Ku(b),b)};f.X=function(a,b,c){return null==b?this.Ib?this.Jb:c:null==this.root?c:this.root.Pe(0,Ku(b),b,c)};
f.nf=function(a,b){a:if(this.Ba)if(null!=b?b.C&2048||x===b.iu||(b.C?0:C(Lt,b)):C(Lt,b))var c=Qy(this,hw.c?hw.c(b):hw.call(null,b),iw.c?iw.c(b):iw.call(null,b));else{c=G(b);for(var d=this;;){var e=I(c);if(z(e))c=K(c),d=Qy(d,hw.c?hw.c(e):hw.call(null,e),iw.c?iw.c(e):iw.call(null,e));else{c=d;break a}}}else throw Error("conj! after persistent");return c};
f.vh=function(){if(this.Ba){this.Ba=null;var a=new Oy(null,this.count,this.root,this.Ib,this.Jb,null)}else throw Error("persistent! called twice");return a};f.mf=function(a,b,c){return Qy(this,b,c)};var Xw=function Xw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return Xw.w(0<c.length?new H(c.slice(0),0,null):null)};Xw.w=function(a){for(var b=G(a),c=qu(ny);;)if(b){a=K(K(b));var d=I(b),b=qv(b),c=tu(c,d,b),b=a}else return su(c)};Xw.I=0;Xw.N=function(a){return Xw.w(G(a))};
var Ry=function Ry(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return Ry.w(0<c.length?new H(c.slice(0),0,null):null)};Ry.w=function(a){a=a instanceof H&&0===a.i?a.v:mt(a);return vv(a)};Ry.I=0;Ry.N=function(a){return Ry.w(G(a))};function Sy(a,b){this.ia=a;this.dc=b;this.C=32374988;this.Y=0}f=Sy.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.dc};f.$b=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.ej||(this.ia.C?0:C(Bt,this.ia)):C(Bt,this.ia))?this.ia.$b(null):K(this.ia);return null==a?null:new Sy(a,this.dc)};f.ha=function(){return Uu(this)};
f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.dc)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return this.ia.ab(null).Sl()};f.zb=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.ej||(this.ia.C?0:C(Bt,this.ia)):C(Bt,this.ia))?this.ia.$b(null):K(this.ia);return null!=a?new Sy(a,this.dc):Qu};f.oa=function(){return this};f.ya=function(a,b){return new Sy(this.ia,b)};f.wa=function(a,b){return lv(b,this)};
Sy.prototype[kt]=function(){return Su(this)};function jy(a){return(a=G(a))?new Sy(a,null):null}function hw(a){return Mt(a)}function Ty(a,b){this.ia=a;this.dc=b;this.C=32374988;this.Y=0}f=Ty.prototype;f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.sa=function(){return this.dc};f.$b=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.ej||(this.ia.C?0:C(Bt,this.ia)):C(Bt,this.ia))?this.ia.$b(null):K(this.ia);return null==a?null:new Ty(a,this.dc)};f.ha=function(){return Uu(this)};
f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.dc)};f.xb=function(a,b){return nv(b,this)};f.yb=function(a,b,c){return pv(b,c,this)};f.ab=function(){return this.ia.ab(null).Tl()};f.zb=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.ej||(this.ia.C?0:C(Bt,this.ia)):C(Bt,this.ia))?this.ia.$b(null):K(this.ia);return null!=a?new Ty(a,this.dc):Qu};f.oa=function(){return this};f.ya=function(a,b){return new Ty(this.ia,b)};f.wa=function(a,b){return lv(b,this)};
Ty.prototype[kt]=function(){return Su(this)};function ky(a){return(a=G(a))?new Ty(a,null):null}function iw(a){return Nt(a)}var Uy=function Uy(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return Uy.w(0<c.length?new H(c.slice(0),0,null):null)};Uy.w=function(a){return z(Mw(Xv,a))?Vv(function(a,c){return sv.h(z(a)?a:Kw,c)},a):null};Uy.I=0;Uy.N=function(a){return Uy.w(G(a))};
function Vy(a){for(var b=Kw,c=G(new V(null,3,5,X,[Wy,Xy,Yy],null));;)if(c)var d=I(c),e=F.j(a,d,Zy),b=Gw.h(e,Zy)?T.j(b,d,e):b,c=K(c);else return Zt(b,Av(a))}function $y(a){this.yn=a}$y.prototype.hb=function(){return this.yn.hb()};$y.prototype.next=function(){if(this.yn.hb())return this.yn.next().Mb[0];throw Error("No such element");};$y.prototype.remove=function(){return Error("Unsupported operation")};function az(a,b,c){this.meta=a;this.Ne=b;this.K=c;this.C=15077647;this.Y=8196}f=az.prototype;
f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};f.keys=function(){return Su(G(this))};f.entries=function(){return new fy(G(G(this)))};f.values=function(){return Su(G(this))};f.has=function(a){return Tv(this,a)};
f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.$(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,Kv(d)?(e=vu(d),d=wu(d),c=e,b=P(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return Ft(this.Ne,b)?b:c};f.ic=function(){return new $y(zu(this.Ne))};f.sa=function(){return this.meta};f.xa=function(){return st(this.Ne)};
f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Wu(this)};f.O=function(a,b){return Fv(b)&&P(this)===P(b)&&Lw(function(a){return function(b){return Tv(a,b)}}(this),b)};f.kg=function(){return new bz(qu(this.Ne))};f.Ja=function(){return Zt(cz,this.meta)};f.Ul=function(a,b){return new az(this.meta,Kt(this.Ne,b),null)};f.oa=function(){return jy(this.Ne)};f.ya=function(a,b){return new az(b,this.Ne,this.K)};f.wa=function(a,b){return new az(this.meta,T.j(this.Ne,b,null),null)};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};var cz=new az(null,Kw,Xu);
function dz(a){for(var b=a.length,c=qu(cz),d=0;;)if(d<b)ru(c,a[d]),d+=1;else break;return su(c)}az.prototype[kt]=function(){return Su(this)};function bz(a){this.xe=a;this.Y=136;this.C=259}f=bz.prototype;f.nf=function(a,b){this.xe=tu(this.xe,b,null);return this};f.vh=function(){return new az(null,su(this.xe),null)};f.xa=function(){return P(this.xe)};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return Dt.j(this.xe,b,Nv)===Nv?c:b};
f.call=function(){function a(a,b,c){return Dt.j(this.xe,b,Nv)===Nv?c:b}function b(a,b){return Dt.j(this.xe,b,Nv)===Nv?null:b}var c=null,c=function(c,e,g){switch(arguments.length){case 2:return b.call(this,0,e);case 3:return a.call(this,0,e,g)}throw Error("Invalid arity: "+(arguments.length-1));};c.h=b;c.j=a;return c}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return Dt.j(this.xe,a,Nv)===Nv?null:a};f.h=function(a,b){return Dt.j(this.xe,a,Nv)===Nv?b:a};
function ez(a){a=G(a);if(null==a)return cz;if(a instanceof H&&0===a.i)return dz(a.v);for(var b=qu(cz);;)if(null!=a){var c=K(a),b=b.nf(null,a.ab(null));a=c}else return su(b)}function fz(a){for(var b=tv;;)if(K(a))b=sv.h(b,I(a)),a=K(a);else return G(b)}function rw(a){if(null!=a&&(a.Y&4096||x===a.Xp))return a.name;if("string"===typeof a)return a;throw Error([D.c("Doesn't support name: "),D.c(a)].join(""));}
var gz=function gz(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return gz.h(arguments[0],arguments[1]);case 3:return gz.j(arguments[0],arguments[1],arguments[2]);default:return gz.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};gz.h=function(a,b){return b};gz.j=function(a,b,c){return(a.c?a.c(b):a.call(null,b))>(a.c?a.c(c):a.call(null,c))?b:c};
gz.w=function(a,b,c,d){return nt(function(b,c){return gz.j(a,b,c)},gz.j(a,b,c),d)};gz.N=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return gz.w(b,a,c,d)};gz.I=3;function hz(a,b,c){this.i=a;this.end=b;this.step=c}hz.prototype.hb=function(){return 0<this.step?this.i<this.end:this.i>this.end};hz.prototype.next=function(){var a=this.i;this.i+=this.step;return a};function iz(a,b,c,d,e){this.meta=a;this.start=b;this.end=c;this.step=d;this.K=e;this.C=32375006;this.Y=8192}f=iz.prototype;
f.toString=function(){return Bu(this)};f.equiv=function(a){return this.O(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return O(this,a,0);case 2:return O(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return O(this,a,0)};a.h=function(a,c){return O(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return hv(this,a,P(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return hv(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return hv(this,a,b)};return b}();f.$=function(a,b){if(b<this.xa(null))return this.start+b*this.step;if(this.start>this.end&&0===this.step)return this.start;throw Error("Index out of bounds");};
f.Zb=function(a,b,c){return b<this.xa(null)?this.start+b*this.step:this.start>this.end&&0===this.step?this.start:c};f.ic=function(){return new hz(this.start,this.end,this.step)};f.sa=function(){return this.meta};f.$b=function(){return 0<this.step?this.start+this.step<this.end?new iz(this.meta,this.start+this.step,this.end,this.step,null):null:this.start+this.step>this.end?new iz(this.meta,this.start+this.step,this.end,this.step,null):null};
f.xa=function(){return ht(this.oa(null))?0:Math.ceil((this.end-this.start)/this.step)};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=Uu(this)};f.O=function(a,b){return kv(this,b)};f.Ja=function(){return Zt(Qu,this.meta)};f.xb=function(a,b){return Zu(this,b)};f.yb=function(a,b,c){for(a=this.start;;)if(0<this.step?a<this.end:a>this.end)c=b.h?b.h(c,a):b.call(null,c,a),a+=this.step;else return c};f.ab=function(){return null==this.oa(null)?null:this.start};
f.zb=function(){return null!=this.oa(null)?new iz(this.meta,this.start+this.step,this.end,this.step,null):Qu};f.oa=function(){return 0<this.step?this.start<this.end?this:null:0>this.step?this.start>this.end?this:null:this.start===this.end?null:this};f.ya=function(a,b){return new iz(b,this.start,this.end,this.step,this.K)};f.wa=function(a,b){return lv(b,this)};iz.prototype[kt]=function(){return Su(this)};function jz(a){a:for(var b=a;;)if(G(b))b=K(b);else break a;return a}
function kz(a,b){if("string"===typeof b){var c=a.exec(b);return M.h(I(c),b)?1===P(c)?I(c):Nx(c):null}throw new TypeError("re-matches must match against a string.");}function lz(a,b){if("string"===typeof b){var c=a.exec(b);return null==c?null:1===P(c)?I(c):Nx(c)}throw new TypeError("re-find must match against a string.");}function mz(a){if(a instanceof RegExp)return a;var b=lz(/^\(\?([idmsux]*)\)/,a),c=S(b,0,null),b=S(b,1,null),c=P(c);return new RegExp(a.substring(c),z(b)?b:"")}
function nz(a,b,c,d,e,g,h){var k=Ys;Ys=null==Ys?null:Ys-1;try{if(null!=Ys&&0>Ys)return lu(a,"#");lu(a,c);if(0===et.c(g))G(h)&&lu(a,function(){var a=oz.c(g);return z(a)?a:"..."}());else{if(G(h)){var l=I(h);b.j?b.j(l,a,g):b.call(null,l,a,g)}for(var m=K(h),p=et.c(g)-1;;)if(!m||null!=p&&0===p){G(m)&&0===p&&(lu(a,d),lu(a,function(){var a=oz.c(g);return z(a)?a:"..."}()));break}else{lu(a,d);var u=I(m);c=a;h=g;b.j?b.j(u,c,h):b.call(null,u,c,h);var A=K(m);c=p-1;m=A;p=c}}return lu(a,e)}finally{Ys=k}}
function pz(a,b){for(var c,d=G(b),e=null,g=0,h=0;;)if(h<g)c=e.$(null,h),lu(a,c),h+=1;else if(d=G(d))e=d,Kv(e)?(d=vu(e),e=wu(e),c=d,g=P(d),d=e,e=c):(c=I(e),lu(a,c),d=K(e),e=null,g=0),h=0;else return null}var rz={'"':'\\"',"\\":"\\\\","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t"};function sz(a){return[D.c('"'),D.c(a.replace(RegExp('[\\\\"\b\f\n\r\t]',"g"),function(a){return rz[a]})),D.c('"')].join("")}
function tz(a,b){var c=Qv(F.h(a,ct));return c?(c=null!=b?b.C&131072||x===b.ju?!0:!1:!1)?null!=Av(b):c:c}
function uz(a,b,c){if(null==a)return lu(b,"nil");if(tz(c,a)){lu(b,"^");var d=Av(a);vz.j?vz.j(d,b,c):vz.call(null,d,b,c);lu(b," ")}if(a.Zp)return a.ou(b);if(null!=a&&(a.C&2147483648||x===a.Qa))return a.ja(null,b,c);if(!0===a||!1===a||"number"===typeof a)return lu(b,""+D.c(a));if(null!=a&&a.constructor===Object)return lu(b,"#js "),d=Z.h(function(b){return new V(null,2,5,X,[qw.c(b),a[b]],null)},Lv(a)),wz.L?wz.L(d,vz,b,c):wz.call(null,d,vz,b,c);if(gt(a))return nz(b,vz,"#js ["," ","]",c,a);if(da(a))return z(bt.c(c))?
lu(b,sz(a)):lu(b,a);if(oa(a)){var e=a.name;c=z(function(){var a=null==e;return a?a:Ba(e)}())?"Function":e;return pz(b,R(["#object[",c,' "',""+D.c(a),'"]'],0))}if(a instanceof Date)return c=function(a,b){for(var c=""+D.c(a);;)if(P(c)<b)c=[D.c("0"),D.c(c)].join("");else return c},pz(b,R(['#inst "',""+D.c(a.getUTCFullYear()),"-",c(a.getUTCMonth()+1,2),"-",c(a.getUTCDate(),2),"T",c(a.getUTCHours(),2),":",c(a.getUTCMinutes(),2),":",c(a.getUTCSeconds(),2),".",c(a.getUTCMilliseconds(),3),"-",'00:00"'],0));
if(a instanceof RegExp)return pz(b,R(['#"',a.source,'"'],0));if(z(a.constructor.jj))return pz(b,R(["#object[",a.constructor.jj.replace(RegExp("/","g"),"."),"]"],0));e=a.constructor.name;c=z(function(){var a=null==e;return a?a:Ba(e)}())?"Object":e;return pz(b,R(["#object[",c," ",""+D.c(a),"]"],0))}function vz(a,b,c){var d=xz.c(c);return z(d)?(c=T.j(c,yz,uz),d.j?d.j(a,b,c):d.call(null,a,b,c)):uz(a,b,c)}
function zz(a,b){var c=new Es;a:{var d=new Au(c);vz(I(a),d,b);for(var e=G(K(a)),g=null,h=0,k=0;;)if(k<h){var l=g.$(null,k);lu(d," ");vz(l,d,b);k+=1}else if(e=G(e))g=e,Kv(g)?(e=vu(g),g=wu(g),l=e,h=P(e),e=g,g=l):(l=I(g),lu(d," "),vz(l,d,b),e=K(g),g=null,h=0),k=0;else break a}return c}function Az(a,b){return Dv(a)?"":""+D.c(zz(a,b))}function Bz(a){var b=T.j($s(),bt,!1);a=Az(a,b);Ws.c?Ws.c(a):Ws.call(null);a=$s();Ws.c?Ws.c("\n"):Ws.call(null);return F.h(a,at),null}
function Cz(a,b,c,d,e){return nz(d,function(a,b,d){var e=Mt(a);c.j?c.j(e,b,d):c.call(null,e,b,d);lu(b," ");a=Nt(a);return c.j?c.j(a,b,d):c.call(null,a,b,d)},[D.c(a),D.c("{")].join(""),", ","}",e,G(b))}function wz(a,b,c,d){var e=S(null,0,null),g=S(null,1,null);return z(e)?Cz([D.c("#:"),D.c(e)].join(""),g,b,c,d):Cz(null,a,b,c,d)}H.prototype.Qa=x;H.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};sw.prototype.Qa=x;sw.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};
Jy.prototype.Qa=x;Jy.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};hy.prototype.Qa=x;hy.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};Px.prototype.Qa=x;Px.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};ow.prototype.Qa=x;ow.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};jv.prototype.Qa=x;jv.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};Oy.prototype.Qa=x;Oy.prototype.ja=function(a,b,c){return wz(this,vz,b,c)};
Ly.prototype.Qa=x;Ly.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};Sx.prototype.Qa=x;Sx.prototype.ja=function(a,b,c){return nz(b,vz,"["," ","]",c,this)};az.prototype.Qa=x;az.prototype.ja=function(a,b,c){return nz(b,vz,"#{"," ","}",c,this)};xw.prototype.Qa=x;xw.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};Uw.prototype.Qa=x;Uw.prototype.ja=function(a,b,c){lu(b,"#object [cljs.core.Atom ");vz(new y(null,1,[Dz,this.state],null),b,c);return lu(b,"]")};
Ty.prototype.Qa=x;Ty.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};V.prototype.Qa=x;V.prototype.ja=function(a,b,c){return nz(b,vz,"["," ","]",c,this)};Yx.prototype.Qa=x;Yx.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};lw.prototype.Qa=x;lw.prototype.ja=function(a,b){return lu(b,"()")};Zx.prototype.Qa=x;Zx.prototype.ja=function(a,b,c){return nz(b,vz,"#queue ["," ","]",c,G(this))};y.prototype.Qa=x;y.prototype.ja=function(a,b,c){return wz(this,vz,b,c)};
iz.prototype.Qa=x;iz.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};Sy.prototype.Qa=x;Sy.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};jw.prototype.Qa=x;jw.prototype.ja=function(a,b,c){return nz(b,vz,"("," ",")",c,this)};var Ez=null;function Fz(){null==Ez&&(Ez=Ww?Ww(0):Vw.call(null,0));return Ou.c([D.c("reagent"),D.c($w.h(Ez,Yu))].join(""))}function Gz(){}
var Hz=function Hz(b){if(null!=b&&null!=b.eu)return b.eu(b);var c=Hz[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Hz._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IEncodeJS.-clj-\x3ejs",b);};function Iz(a){return(null!=a?x===a.du||(a.kj?0:C(Gz,a)):C(Gz,a))?Hz(a):"string"===typeof a||"number"===typeof a||a instanceof Y||a instanceof Nu?Jz.c?Jz.c(a):Jz.call(null,a):Az(R([a],0),$s())}
var Jz=function Jz(b){if(null==b)return null;if(null!=b?x===b.du||(b.kj?0:C(Gz,b)):C(Gz,b))return Hz(b);if(b instanceof Y)return rw(b);if(b instanceof Nu)return""+D.c(b);if(Hv(b)){var c={};b=G(b);for(var d=null,e=0,g=0;;)if(g<e){var h=d.$(null,g),k=S(h,0,null),h=S(h,1,null);c[Iz(k)]=Jz.c?Jz.c(h):Jz.call(null,h);g+=1}else if(b=G(b))Kv(b)?(e=vu(b),b=wu(b),d=e,e=P(e)):(e=I(b),d=S(e,0,null),e=S(e,1,null),c[Iz(d)]=Jz.c?Jz.c(e):Jz.call(null,e),b=K(b),d=null,e=0),g=0;else break;return c}if(Ev(b)){c=[];b=
G(Z.h(Jz,b));d=null;for(g=e=0;;)if(g<e)k=d.$(null,g),c.push(k),g+=1;else if(b=G(b))d=b,Kv(d)?(b=vu(d),g=wu(d),d=b,e=P(b),b=g):(b=I(d),c.push(b),b=K(d),d=null,e=0),g=0;else break;return c}return b};function Kz(){}var Lz=function Lz(b,c){if(null!=b&&null!=b.cu)return b.cu(b,c);var d=Lz[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Lz._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IEncodeClojure.-js-\x3eclj",b);};
function Mz(a,b){var c=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b,d=F.h(c,Nz);return function(a,c,d,k){return function m(e){return(null!=e?x===e.Dz||(e.kj?0:C(Kz,e)):C(Kz,e))?Lz(e,U.h(Ry,b)):Pv(e)?jz(Z.h(m,e)):Ev(e)?Rw.h(null==e?null:tt(e),Z.h(m,e)):gt(e)?Nx(Z.h(m,e)):(null==e?null:e.constructor)===Object?Rw.h(Kw,function(){return function(a,b,c,d){return function L(g){return new sw(null,function(a,b,c,d){return function(){for(;;){var a=G(g);if(a){if(Kv(a)){var b=vu(a),c=P(b),h=ww(c);a:for(var k=0;;)if(k<
c){var p=xt.h(b,k),p=new V(null,2,5,X,[d.c?d.c(p):d.call(null,p),m(e[p])],null);h.add(p);k+=1}else{b=!0;break a}return b?yw(h.ob(),L(wu(a))):yw(h.ob(),null)}h=I(a);return lv(new V(null,2,5,X,[d.c?d.c(h):d.call(null,h),m(e[h])],null),L(Pu(a)))}return null}}}(a,b,c,d),null,null)}}(a,c,d,k)(Lv(e))}()):e}}(b,c,d,z(d)?qw:D)(a)}function Oz(a,b){this.ef=a;this.K=b;this.C=2153775104;this.Y=2048}f=Oz.prototype;f.toString=function(){return this.ef};f.equiv=function(a){return this.O(null,a)};
f.O=function(a,b){return b instanceof Oz&&this.ef===b.ef};f.ja=function(a,b){return lu(b,[D.c('#uuid "'),D.c(this.ef),D.c('"')].join(""))};f.ha=function(){null==this.K&&(this.K=Ku(this.ef));return this.K};function Pz(a,b,c){var d=Error(a);this.message=a;this.data=b;this.Lp=c;this.name=d.name;this.description=d.description;this.number=d.number;this.fileName=d.fileName;this.lineNumber=d.lineNumber;this.columnNumber=d.columnNumber;this.stack=d.stack;return this}Pz.prototype.__proto__=Error.prototype;
Pz.prototype.Qa=x;Pz.prototype.ja=function(a,b,c){lu(b,"#error {:message ");vz(this.message,b,c);z(this.data)&&(lu(b,", :data "),vz(this.data,b,c));z(this.Lp)&&(lu(b,", :cause "),vz(this.Lp,b,c));return lu(b,"}")};Pz.prototype.toString=function(){return Bu(this)};function Qz(a,b){return new Pz(a,b,null)};var Rz=new Y(null,"response","response",-1068424192),Sz=new Y(null,"tick","tick",-835886976),Tz=new Nu(null,"localTimeParser","localTimeParser",-1738135328,null),Uz=new Y(null,"description","description",-1428560544),Vz=new Y(null,"unnamed","unnamed",-26044928),Wz=new Y(null,"date-element-parser","date-element-parser",2072167040),Xz=new Y(null,"auth-token","auth-token",30990976),Yz=new Y(null,"add-event","add-event",938429088),Zz=new Y(null,"hour-minute","hour-minute",-1164421312),$z=new Y("common",
"default-response","common/default-response",107834336),aA=new Y(null,"yield","yield",177875009),bA=new Y(null,"ordinal-suffix","ordinal-suffix",-1311843199),cA=new Y(null,"labels","labels",-626734591),dA=new Y(null,"paused","paused",-1710376127),eA=new Y(null,"non-field-errors","non-field-errors",882146145),fA=new Y(null,"errors","errors",-908790718),gA=new Y(null,"month-name","month-name",-605509534),hA=new Y(null,"formatters","formatters",-1875637118),iA=new Y(null,"on-set","on-set",-140953470),
jA=new Y(null,"day","day",-274800446),kA=new Y(null,"format","format",-1306924766),lA=new Y("flash","messages","flash/messages",247896418),mA=new Y(null,"t-time","t-time",-42016318),nA=new Nu(null,"timeParser","timeParser",1585048034,null),oA=new Y(null,"basic-ordinal-date","basic-ordinal-date",243220162),pA=new Y(null,"date","date",-1463434462),qA=new Y(null,"hour","hour",-555989214),rA=new Y(null,"timezone","timezone",1831928099),sA=new Y(null,"long","long",-171452093),tA=new Y(null,"cljsLegacyRender",
"cljsLegacyRender",-1527295613),Jw=new Nu(null,"meta14186","meta14186",718375395,null),uA=new Y("common","on-load","common/on-load",-2052806173),vA=new Y(null,"time-no-ms","time-no-ms",870271683),wA=new Y(null,"weekyear-week-day","weekyear-week-day",-740233533),xA=new Y(null,"overrides","overrides",1738628867),yA=new Y(null,"idle","idle",-2007156861),zA=new Y(null,"where","where",-2044795965),AA=new Y(null,"week-date-time","week-date-time",540228836),BA=new Y(null,"date-hour-minute-second-fraction",
"date-hour-minute-second-fraction",1937143076),CA=new Y(null,"eventCategory","eventCategory",2015922692),DA=new Y(null,"group","group",582596132),ct=new Y(null,"meta","meta",1499536964),EA=new Y(null,"full","full",436801220),FA=new Y(null,"basic-date-time","basic-date-time",1525413604),GA=new Y(null,"date-time","date-time",177938180),HA=new Y(null,"basic-time-no-ms","basic-time-no-ms",-1720654076),IA=new Y(null,"dddd","dddd",217016228),JA=new Y(null,"date-parser","date-parser",-981534587),dt=new Y(null,
"dup","dup",556298533),KA=new Y(null,"loading-count","loading-count",-1446402651),LA=new Y(null,"basic-week-date","basic-week-date",1775847845),MA=new Y(null,"key","key",-1516042587),NA=new Y(null,"fsm-state","fsm-state",1656310533),OA=new Nu(null,"dateOptionalTimeParser","dateOptionalTimeParser",1783230854,null),PA=new Y(null,"parse-quoted","parse-quoted",1180570118),QA=new Y(null,"int","int",-1741416922),RA=new Y(null,"basic-t-time-no-ms","basic-t-time-no-ms",-424650106),SA=new Y(null,"local-time",
"local-time",-1873195290),TA=new Y(null,"\x3c-","\x3c-",760412998),UA=new Y(null,"date-time-no-ms","date-time-no-ms",1655953671),VA=new Y(null,"year-month-day","year-month-day",-415594169),WA=new Y(null,"db","db",993250759),XA=new Y(null,"sub","sub",-2093760025),YA=new Y(null,"date-opt-time","date-opt-time",-1507102105),ZA=new Y(null,"eventLabel","eventLabel",-31846777),$A=new Y("re-frame.std-interceptors","not-found","re-frame.std-interceptors/not-found",-1614827865),tx=new Y(null,"queue","queue",
1455835879),aB=new Y(null,"displayName","displayName",-809144601),bB=new Y("common","handle-default-response","common/handle-default-response",980613895),cB=new Y(null,"rfc822","rfc822",-404628697),Yw=new Y(null,"validator","validator",-1966190681),dB=new Y(null,"eventAction","eventAction",1999627175),eB=new Y(null,"sequential","sequential",-1082983960),fB=new Y(null,"illegal-pattern","illegal-pattern",-1810990520),gB=new Y(null,"warn","warn",-436710552),hB=new Y(null,"columns","columns",1998437288),
iB=new Y("auth","update-expires-in","auth/update-expires-in",1932168168),jB=new Y(null,"date-hour-minute-second-ms","date-hour-minute-second-ms",-425334775),kB=new Y(null,"name","name",1843675177),lB=new Y(null,"basic-ordinal-date-time","basic-ordinal-date-time",1054564521),mB=new Nu(null,"timeElementParser","timeElementParser",302132553,null),nB=new Y(null,"ordinal-date","ordinal-date",-77899447),oB=new Y(null,"meridiem","meridiem",1668960617),pB=new Y(null,"change-points","change-points",-349409815),
qB=new Y(null,"hour-minute-second-fraction","hour-minute-second-fraction",-1253038551),rB=new Y("form","finish-progress","form/finish-progress",937307977),sB=new Y(null,"value","value",305978217),tB=new Y(null,"date-hour-minute","date-hour-minute",1629918346),uB=new Y(null,"time","time",1385887882),vB=new Y(null,"component-did-mount","component-did-mount",-1126910518),wB=new Y(null,"fields","fields",-1932066230),xB=new Y("settings","app_settings","settings/app_settings",634678890),yB=new Y(null,"localtime",
"localtime",1720049482),zB=new Y(null,"basic-week-date-time","basic-week-date-time",-502077622),AB=new Y(null,"do-fx","do-fx",1194163050),BB=new Y(null,"net","net",344042539),CB=new Y(null,"months","months",-45571637),DB=new Y(null,"flash","flash",934660619),EB=new Y(null,"app_settings","app_settings",-682153397),FB=new Y(null,"PM","PM",1356677707),GB=new Y(null,"settings","settings",1556144875),HB=new Y(null,"month","month",-1960248533),IB=new Y(null,"component-did-update","component-did-update",
-1468549173),JB=new Y(null,"initial-messages","initial-messages",2057377771),KB=new Y(null,"days","days",-1394072564),Dz=new Y(null,"val","val",128701612),LB=new Y(null,"format-str","format-str",695206156),MB=new Y(null,"dispatch-n","dispatch-n",-504469236),NB=new Y(null,"weekyear","weekyear",-74064500),OB=new Y(null,"type","type",1174270348),PB=new Y(null,"abbr","abbr",2088591884),QB=new Y(null,"debug","debug",-1608172596),RB=new Y(null,"basic-time","basic-time",-923134899),SB=new Nu(null,"localDateParser",
"localDateParser",477820077,null),yz=new Y(null,"fallback-impl","fallback-impl",-1501286995),TB=new Y(null,"invalid-date","invalid-date",2030506573),at=new Y(null,"flush-on-newline","flush-on-newline",-151457939),UB=new Y(null,"componentWillUnmount","componentWillUnmount",1573788814),VB=new Y(null,"string","string",-1989541586),WB=new Y(null,"hour-minute-second","hour-minute-second",-1906654770),XB=new Y(null,"ordinal-date-time","ordinal-date-time",-1386753458),YB=new Y(null,"deregister-event-handler",
"deregister-event-handler",-1096518994),ZB=new Y("common","initialize","common/initialize",-132515154),$B=new Y(null,"seconds","seconds",-445266194),aC=new Y(null,"parse-error","parse-error",255902478),bC=new Nu(null,"dateParser","dateParser",-1248418930,null),cC=new Y(null,"array","array",-2080713842),dC=new Y(null,"ordinal-date-time-no-ms","ordinal-date-time-no-ms",-1539005490),eC=new Y("net","load-auth-token","net/load-auth-token",-541175633),fC=new Y(null,"on-click","on-click",1632826543),gC=
new Y(null,"hour-minute-second-ms","hour-minute-second-ms",1209749775),hC=new Y(null,"running","running",1554969103),iC=new Y(null,"hitType","hitType",-1417600145),jC=new Y(null,"shouldComponentUpdate","shouldComponentUpdate",1795750960),kC=new Y("flash","add-message","flash/add-message",905417936),lC=new Y(null,"time-parser","time-parser",-1636511536),mC=new Y(null,"flush-dom","flush-dom",-933676816),nC=new Y(null,"day-name","day-name",1806125744),oC=new Y(null,"div","div",1057191632),pC=new Y(null,
"trim-v","trim-v",-1274938640),bt=new Y(null,"readably","readably",1129599760),qC=new Y(null,"date-time-parser","date-time-parser",-656147568),oz=new Y(null,"more-marker","more-marker",-14717935),rC=new Y(null,"dispatch","dispatch",1319337009),sC=new Y(null,"year","year",335913393),Xy=new Y(null,"reagentRender","reagentRender",-358306383),tC=new Y(null,"token","token",-1211463215),uC=new Y(null,"t-time-no-ms","t-time-no-ms",990689905),vC=new Nu(null,"dateElementParser","dateElementParser",984800945,
null),wC=new Y(null,"basic-week-date-time-no-ms","basic-week-date-time-no-ms",-2043113679),xC=new Nu(null,"localDateOptionalTimeParser","localDateOptionalTimeParser",435955537,null),yC=new Y(null,"no-cache","no-cache",1588056370),Wy=new Y(null,"render","render",-1408033454),zC=new Y(null,"messages","messages",345434482),AC=new Y(null,"db-handler","db-handler",579530098),BC=new Y(null,"period-match-erroro","period-match-erroro",1058444722),CC=new Y(null,"basic-date","basic-date",1566551506),DC=new Y(null,
"event","event",301435442),EC=new Y(null,"after","after",594996914),FC=new Y(null,"reagent-render","reagent-render",-985383853),GC=new Y(null,"weekyear-week","weekyear-week",795291571),HC=new Y(null,"sub-type","sub-type",-997954412),IC=new Y(null,"invalid-format","invalid-format",-72676108),et=new Y(null,"print-length","print-length",1931866356),JC=new Y(null,"local-date","local-date",1829761428),KC=new Y(null,"basic-ordinal-date-time-no-ms","basic-ordinal-date-time-no-ms",-395135436),LC=new Y(null,
"id","id",-1388402092),MC=new Y(null,"class","class",-2030961996),NC=new Y(null,"effects","effects",-282369292),OC=new Y(null,"progress?","progress?",374452148),PC=new Y(null,"pm","pm",1813737428),QC=new Y(null,"bindto","bindto",583512213),RC=new Y(null,"year-month","year-month",735283381),SC=new Y(null,"auto-run","auto-run",1958400437),TC=new Nu(null,"/","/",-1371932971,null),UC=new Y(null,"run-queue","run-queue",-1701798027),VC=new Y(null,"severity","severity",175684886),WC=new Y("form","init-form",
"form/init-form",339616054),XC=new Y(null,"info","info",-317069002),YC=new Y(null,"stack","stack",-793405930),ZC=new Y("settings","settings","settings/settings",1867392726),$C=new Nu(null,"dateTimeParser","dateTimeParser",-1493718282,null),aD=new Y("flash","delete-message","flash/delete-message",1089438775),bD=new Y(null,"local-date-opt-time","local-date-opt-time",1178432599),cD=new Y(null,"display-name","display-name",694513143),dD=new Y(null,"right","right",-452581833),eD=new Y(null,"scheduled",
"scheduled",553898551),fD=new Y(null,"hours","hours",58380855),gD=new Y(null,"years","years",-1298579689),hD=new Y(null,"week-date","week-date",-1176745129),iD=new Y(null,"csrf-token","csrf-token",-1872302856),jD=new Y(null,"on-dispose","on-dispose",2105306360),kD=new Y(null,"pause","pause",-2095325672),lD=new Y(null,"error","error",-978969032),mD=new Y(null,"quoted","quoted",2117344952),Yy=new Y(null,"componentFunction","componentFunction",825866104),nD=new Y(null,"exception","exception",-335277064),
oD=new Y(null,"coeffects","coeffects",497912985),pD=new Y(null,"x","x",2099068185),qD=new Y(null,"form","form",-1624062471),rD=new Y(null,"date-hour","date-hour",-344234471),sD=new Y(null,"tag","tag",-1290361223),tD=new Y(null,"contents","contents",-1567174023),uD=new Y(null,"period","period",-352129191),vD=new Nu(null,"quote","quote",1377916282,null),wD=new Y(null,"set","set",304602554),xD=new Y(null,"minutes","minutes",1319166394),yD=new Y(null,"timeout","timeout",-318625318),zD=new Y(null,"not-implemented",
"not-implemented",1918806714),AD=new Y(null,"Authorization","Authorization",-1017527462),BD=new Y(null,"axis","axis",-1215390822),CD=new Y("net","loading-count","net/loading-count",-1446220870),DD=new Y(null,"groupEnd","groupEnd",-337721382),ED=new Y(null,"atom","atom",-397043653),FD=new Y(null,"trigger","trigger",103466139),GD=new Y(null,"autobind","autobind",-570650245),HD=new Y(null,"cofx","cofx",2013202907),ID=new Y("analytics","click","analytics/click",320864955),xz=new Y(null,"alt-impl","alt-impl",
670969595),JD=new Y(null,"goog.date.UtcDateTime","goog.date.UtcDateTime",731790075),KD=new Y(null,"resume","resume",-118572261),LD=new Y(null,"time-element-parser","time-element-parser",-2042883205),MD=new Y(null,"subchart","subchart",-126478469),ND=new Y(null,"ms","ms",-1152709733),OD=new Y(null,"enrich","enrich",-2108921925),PD=new Y(null,"fx","fx",-1237829572),QD=new Y(null,"auth-header-name","auth-header-name",759786620),RD=new Y(null,"date-hour-minute-second","date-hour-minute-second",-1565419364),
SD=new Y(null,"before","before",-1633692388),TD=new Y(null,"week-date-time-no-ms","week-date-time-no-ms",-1226853060),Nz=new Y(null,"keywordize-keys","keywordize-keys",1310784252),UD=new Y(null,"log","log",-1595516004),VD=new Nu(null,"deref","deref",1494944732,null),WD=new Y("re-frame.std-interceptors","untrimmed-event","re-frame.std-interceptors/untrimmed-event",-840935075),XD=new Y(null,"HOURS","HOURS",-1611068963),YD=new Y(null,"map","map",1371690461),ZD=new Y(null,"finish-run","finish-run",753148477),
$D=new Y(null,"basic-date-time-no-ms","basic-date-time-no-ms",-899402179),aE=new Y(null,"object","object",1474613949),bE=new Y(null,"componentWillMount","componentWillMount",-285327619),cE=new Y(null,"millis","millis",-1338288387),dE=new Y(null,"names","names",-1943074658),eE=new Y(null,"tooltip","tooltip",-1809677058),fE=new Y(null,"required","required",1807647006),gE=new Y(null,"created_at","created_at",1484050750),hE=new Y(null,"mysql","mysql",-1431590210),iE=new Y(null,"dispatch-later","dispatch-later",
291951390),jE=new Y("flash","add-initial-messages","flash/add-initial-messages",1637693695),kE=new Y(null,"basic-t-time","basic-t-time",191791391),lE=new Y(null,"in","in",-1531184865),mE=new Y(null,"left","left",-399115937),Zy=new Y("cljs.core","not-found","cljs.core/not-found",-1572889185),nE=new Y(null,"default-year","default-year",1658037695),oE=new Y(null,"text","text",-1790561697),pE=new Y(null,"show","show",-576705889),qE=new Y("views","set-view","views/set-view",473229023),rE=new Y(null,"data",
"data",-232669377);var sE=RegExp("([\\uD800-\\uDBFF])([\\uDC00-\\uDFFF])","g");function tE(a,b,c){var d=RegExp,e=b.source,g=z(b.ignoreCase)?[D.c("g"),D.c("i")].join(""):"g",g=z(b.multiline)?[D.c(g),D.c("m")].join(""):g;b=z(b.Pz)?[D.c(g),D.c("u")].join(""):g;d=new d(e,b);return a.replace(d,c)}
function uE(a){return function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){b=dx(b);if(M.h(P(b),1))return b=I(b),a.c?a.c(b):a.call(null,b);b=Nx(b);return a.c?a.c(b):a.call(null,b)}b.I=0;b.N=function(a){a=G(a);return c(a)};b.w=c;return b}()}
function vE(a,b,c){if("string"===typeof b)return a.replace(new RegExp(Ua(b),"g"),c);if(b instanceof RegExp)return"string"===typeof c?tE(a,b,c):tE(a,b,uE(c));throw[D.c("Invalid match arg: "),D.c(b)].join("");}function wE(a){var b=new Es;for(a=G(a);;)if(null!=a)b=b.append(""+D.c(I(a))),a=K(a);else return b.toString()}function xE(a,b){for(var c=new Es,d=G(b);;)if(null!=d)c.append(""+D.c(I(d))),d=K(d),null!=d&&c.append(a);else return c.toString()}
function yE(a){return 2>P(a)?a.toUpperCase():[D.c(a.substring(0,1).toUpperCase()),D.c(a.substring(1).toLowerCase())].join("")}function zE(a,b){var c="/(?:)/"===""+D.c(b)?sv.h(Nx(lv("",Z.h(D,G(a)))),""):Nx((""+D.c(a)).split(b));if(1<P(c))a:for(;;)if(""===Bv(c))c=null==c?null:Rt(c);else break a;return c}function AE(a){return Ca(a)};function BE(a){var b=Pa("\x26nbsp;");return xE(b,Z.h(function(a){return wE(mw(a))},mw(mx(3,3,null,a.replace(sE,"$2$1").split("").reverse().join("")))))}function CE(a){if(Ca(a))return null;var b=[D.c("%."),D.c(0),D.c("f")].join("");a=wm(b,a);a=zE(a,".");b=S(a,0,null);S(a,1,null);return[D.c(BE(b)),D.c(Pa("\x26nbsp;")),D.c("Kč")].join("")};var DE="undefined"!==typeof console;if("undefined"===typeof EE)var EE=Ww?Ww(null):Vw.call(null,null);
if("undefined"===typeof FE)var FE=function(){var a={};a.warn=function(){return function(){function a(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(a){return $w.w(EE,qx,new V(null,1,5,X,[gB],null),sv,R([U.h(D,a)],0))}a.I=0;a.N=function(a){a=G(a);return c(a)};a.w=c;return a}()}(a);a.error=function(){return function(){function a(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-
0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(a){return $w.w(EE,qx,new V(null,1,5,X,[lD],null),sv,R([U.h(D,a)],0))}a.I=0;a.N=function(a){a=G(a);return c(a)};a.w=c;return a}()}(a);return a}();if("undefined"===typeof GE){var HE;if("undefined"!==typeof React)HE=React;else{var IE;if("undefined"!==typeof require){var JE=require("react");if(z(JE))IE=JE;else throw Error("require('react') failed");}else throw Error("js/React is missing");HE=IE}var GE=HE}var KE=new az(null,new y(null,2,["aria",null,"data",null],null),null);function LE(a){return 2>P(a)?a.toUpperCase():[D.c(a.substring(0,1).toUpperCase()),D.c(a.substring(1))].join("")}
function ME(a){if("string"===typeof a)return a;a=rw(a);var b=zE(a,/-/);var c=G(b);b=I(c);c=K(c);return z(KE.c?KE.c(b):KE.call(null,b))?a:U.j(D,b,Z.h(LE,c))}function NE(a){var b=function(){var b=function(){var b=xv(a);return b?(b=a.displayName,z(b)?b:a.name):b}();if(z(b))return b;b=function(){var b=null!=a?a.Y&4096||x===a.Xp?!0:!1:!1;return b?rw(a):b}();if(z(b))return b;b=Av(a);return Hv(b)?kB.c(b):null}();return vE(""+D.c(b),"$",".")}var OE=!1;if("undefined"===typeof PE)var PE=0;function QE(a){return setTimeout(a,16)}var RE="undefined"===typeof window||null==window.document?QE:function(){var a=window,b=a.requestAnimationFrame;if(z(b))return b;b=a.webkitRequestAnimationFrame;if(z(b))return b;b=a.mozRequestAnimationFrame;if(z(b))return b;a=a.msRequestAnimationFrame;return z(a)?a:QE}();function SE(a,b){return a.cljsMountOrder-b.cljsMountOrder}if("undefined"===typeof TE)var TE=function(){return null};function UE(a){this.Ck=a}
function VE(a,b){var c=a[b];if(null==c)return null;a[b]=null;for(var d=c.length,e=0;;)if(e<d)c[e].call(null),e+=1;else return null}function WE(a){if(a.Ck)return null;a.Ck=!0;a=function(a){return function(){a.Ck=!1;VE(a,"beforeFlush");TE();var b=a.componentQueue;if(null!=b){a.componentQueue=null;b.sort(SE);for(var d=b.length,e=0;;)if(e<d){var g=b[e];!0===g.cljsIsDirty&&g.forceUpdate();e+=1}else break}return VE(a,"afterRender")}}(a);return RE.c?RE.c(a):RE.call(null,a)}
UE.prototype.enqueue=function(a,b){if(null==b)throw Error("Assert failed: (some? f)");null==this[a]&&(this[a]=[]);this[a].push(b);return WE(this)};function XE(a){return YE.enqueue("afterRender",a)}if("undefined"===typeof YE)var YE=new UE(!1);function ZE(a){if(z(a.cljsIsDirty))return null;a.cljsIsDirty=!0;return YE.enqueue("componentQueue",a)};function $E(a,b){var c=U.j(gz,a,b);return lv(c,hx(function(a){return function(b){return a===b}}(c),b))}function aF(a,b){return P(a)<P(b)?nt(sv,b,a):nt(sv,a,b)}var bF=function bF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return bF.c(arguments[0]);case 2:return bF.h(arguments[0],arguments[1]);default:return bF.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};bF.c=function(a){return a};
bF.h=function(a,b){for(;;)if(P(b)<P(a)){var c=a;a=b;b=c}else return nt(function(a,b){return function(a,c){return Tv(b,c)?a:Cv.h(a,c)}}(a,b),a,a)};bF.w=function(a,b,c){a=$E(function(a){return-P(a)},sv.w(c,b,R([a],0)));return nt(bF,I(a),Pu(a))};bF.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return bF.w(b,a,c)};bF.I=2;
var cF=function cF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return cF.c(arguments[0]);case 2:return cF.h(arguments[0],arguments[1]);default:return cF.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};cF.c=function(a){return a};cF.h=function(a,b){return P(a)<P(b)?nt(function(a,d){return Tv(b,d)?Cv.h(a,d):a},a,a):nt(Cv,a,b)};cF.w=function(a,b,c){return nt(cF,a,sv.h(c,b))};
cF.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return cF.w(b,a,c)};cF.I=2;var dF;if("undefined"===typeof eF)var eF=!1;if("undefined"===typeof fF)var fF=0;if("undefined"===typeof gF)var gF=Ww?Ww(0):Vw.call(null,0);function hF(a,b){var c=dF;dF=a;try{return b.T?b.T():b.call(null)}finally{dF=c}}
function iF(a,b){b.Vi=null;b.Mz=fF+=1;var c=hF(b,a),d=b.Vi;b.De=!1;a:{var e=b.cg;var g=null==d?0:d.length,h=g===(null==e?0:e.length);if(h)for(h=0;;){var k=h===g;if(k){e=k;break a}if(d[h]===e[h])h+=1;else{e=!1;break a}}else e=h}if(!e)a:{e=ez(d);g=ez(b.cg);b.cg=d;for(var d=G(cF.h(e,g)),h=null,l=k=0;;)if(l<k){var m=h.$(null,l);ou(m,b,jF);l+=1}else if(d=G(d))h=d,Kv(h)?(d=vu(h),l=wu(h),h=d,k=P(d),d=l):(d=I(h),ou(d,b,jF),d=K(h),h=null,k=0),l=0;else break;e=G(cF.h(g,e));d=null;for(k=h=0;;)if(k<h)g=d.$(null,
k),pu(g,b),k+=1;else if(e=G(e))d=e,Kv(d)?(e=vu(d),d=wu(d),g=e,h=P(e),e=d,d=g):(g=I(d),pu(g,b),e=K(d),d=null,h=0),k=0;else break a}return c}function kF(a){var b=dF;if(null!=b){var c=b.Vi;null==c?b.Vi=[a]:c.push(a)}}function lF(a,b){eF&&$w.j(gF,Zv,P(b)-P(a));return b}function mF(a,b,c){var d=a.sb;a.sb=lF(d,T.j(d,b,c));return a.Yo=null}function nF(a,b){var c=a.sb;a.sb=lF(c,wv.h(c,b));return a.Yo=null}
function oF(a,b,c){for(var d=a.Yo,d=null==d?a.Yo=Wv(function(){return function(a,b,c){a.push(b);a.push(c);return a}}(d),[],a.sb):d,e=d.length,g=0;;)if(g<e){var h=d[g],k=d[g+1];k.L?k.L(h,a,b,c):k.call(null,h,a,b,c);g=2+g}else return null}function pF(a,b,c,d){lu(b,[D.c("#\x3c"),D.c(d),D.c(" ")].join(""));a:{d=dF;dF=null;try{var e=Vt(a);break a}finally{dF=d}e=void 0}vz(e,b,c);return lu(b,"\x3e")}if("undefined"===typeof qF)var qF=null;
function rF(){for(;;){var a=qF;if(null==a)return null;qF=null;for(var b=a.length,c=0;;)if(c<b){var d=a[c];d.De&&null!=d.cg&&sF(d,!0);c+=1}else break}}TE=rF;function tF(a,b,c,d){this.state=a;this.meta=b;this.Hd=c;this.sb=d;this.C=2153938944;this.Y=114690}f=tF.prototype;f.sk=x;f.ja=function(a,b,c){return pF(this,b,c,"Atom:")};f.sa=function(){return this.meta};f.ha=function(){return qa(this)};f.O=function(a,b){return this===b};
f.jc=function(a,b){if(null!=this.Hd&&!z(this.Hd.c?this.Hd.c(b):this.Hd.call(null,b)))throw Error([D.c("Assert failed: "),D.c("Validator rejected reference state"),D.c("\n"),D.c("(validator new-value)")].join(""));var c=this.state;this.state=b;null!=this.sb&&oF(this,c,b);return b};f.fj=function(a,b){return this.jc(null,b.c?b.c(this.state):b.call(null,this.state))};f.gj=function(a,b,c){return this.jc(null,b.h?b.h(this.state,c):b.call(null,this.state,c))};
f.hj=function(a,b,c,d){return this.jc(null,b.j?b.j(this.state,c,d):b.call(null,this.state,c,d))};f.ij=function(a,b,c,d,e){return this.jc(null,U.ba(b,this.state,c,d,e))};f.yh=function(a,b,c){return oF(this,b,c)};f.xh=function(a,b,c){return mF(this,b,c)};f.zh=function(a,b){return nF(this,b)};f.Ce=function(){kF(this);return this.state};
var uF=function uF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return uF.c(arguments[0]);default:return uF.w(arguments[0],new H(c.slice(1),0,null))}};uF.c=function(a){return new tF(a,null,null,null)};uF.w=function(a,b){var c=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b,d=F.h(c,ct),c=F.h(c,Yw);return new tF(a,d,c,null)};uF.N=function(a){var b=I(a);a=K(a);return uF.w(b,a)};uF.I=1;
function vF(a,b,c,d){var e=b.reagReactionCache,g=null==e?Kw:e,h=g.h?g.h(c,null):g.call(null,c,null);if(null!=h)return Vt(h);if(null==dF)return a.T?a.T():a.call(null);var k=function(){var k=function(){return function(){eF&&$w.h(gF,aw);var a=wv.h(b.reagReactionCache,c);b.reagReactionCache=a;null!=d&&(d.gi=null);return null}}(a,jD,e,g,h);return wF.j?wF.j(a,jD,k):wF.call(null,a,jD,k)}(),l=Vt(k);b.reagReactionCache=T.j(g,c,k);eF&&$w.h(gF,Yu);null!=d&&(d.gi=k);return l}
function xF(a,b,c){this.Nc=a;this.ub=b;this.gi=c;this.C=2153807872;this.Y=0}f=xF.prototype;f.sk=x;f.Ce=function(){var a=this,b=a.gi;return null==b?vF(function(){return function(){return U.h(a.Nc,a.ub)}}(b,this),a.Nc,a.ub,this):Vt(b)};f.O=function(a,b){return b instanceof xF&&M.h(this.Nc,b.Nc)&&M.h(this.ub,b.ub)};f.ha=function(){return Ku(new V(null,2,5,X,[this.Nc,this.ub],null))};f.ja=function(a,b,c){return pF(this,b,c,"Track:")};
function yF(a,b,c,d,e){this.Ha=a;this.path=b;this.gi=c;this.state=d;this.sb=e;this.C=2153807872;this.Y=114690}function zF(a){var b=dF;dF=null;try{return a.Ce(null)}finally{dF=b}}function AF(a,b,c){b!==c&&(a.state=c,null!=a.sb&&oF(a,b,c))}f=yF.prototype;f.sk=x;f.ja=function(a,b,c){return pF(this,b,c,[D.c("Cursor: "),D.c(this.path)].join(""))};f.ha=function(){return Ku(new V(null,2,5,X,[this.Ha,this.path],null))};f.O=function(a,b){return b instanceof yF&&M.h(this.path,b.path)&&M.h(this.Ha,b.Ha)};
f.jc=function(a,b){AF(this,this.state,b);(null!=this.Ha?this.Ha.C&32768||x===this.Ha.Tp||(this.Ha.C?0:C(Ut,this.Ha)):C(Ut,this.Ha))?M.h(this.path,tv)?Zw.h?Zw.h(this.Ha,b):Zw.call(null,this.Ha,b):$w.L(this.Ha,px,this.path,b):this.Ha.h?this.Ha.h(this.path,b):this.Ha.call(null,this.path,b);return b};f.fj=function(a,b){var c=this;return c.jc(null,function(){var a=zF(c);return b.c?b.c(a):b.call(null,a)}())};
f.gj=function(a,b,c){var d=this;return d.jc(null,function(){var a=zF(d);return b.h?b.h(a,c):b.call(null,a,c)}())};f.hj=function(a,b,c,d){var e=this;return e.jc(null,function(){var a=zF(e);return b.j?b.j(a,c,d):b.call(null,a,c,d)}())};f.ij=function(a,b,c,d,e){return this.jc(null,U.ba(b,zF(this),c,d,e))};f.yh=function(a,b,c){return oF(this,b,c)};f.xh=function(a,b,c){return mF(this,b,c)};f.zh=function(a,b){return nF(this,b)};
f.Ce=function(){var a=this,b=this,c=a.state,d=function(){var d=a.gi;return null==d?(d=(null!=a.Ha?a.Ha.C&32768||x===a.Ha.Tp||(a.Ha.C?0:C(Ut,a.Ha)):C(Ut,a.Ha))?function(){return function(){return nx(N.c?N.c(a.Ha):N.call(null,a.Ha),a.path)}}(d,c,b):function(){return function(){return a.Ha.c?a.Ha.c(a.path):a.Ha.call(null,a.path)}}(d,c,b),vF(d,a.Ha,a.path,b)):Vt(d)}();AF(b,c,d);return d};
var BF=function BF(b){if(null!=b&&null!=b.ho)return b.ho();var c=BF[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=BF._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("IDisposable.dispose!",b);},CF=function CF(b,c){if(null!=b&&null!=b.Jr)return b.Jr(0,c);var d=CF[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=CF._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IDisposable.add-on-dispose!",b);};
function jF(a,b,c,d){c===d||a.De?a=null:null==a.od?(a.De=!0,null==qF&&(qF=[],!1===YE.Ck&&WE(YE)),a=qF.push(a)):a=!0===a.od?sF(a,!1):a.od.c?a.od.c(a):a.od.call(null,a);return a}function DF(a,b,c,d,e,g,h,k){this.Nc=a;this.state=b;this.De=c;this.kr=d;this.cg=e;this.sb=g;this.od=h;this.Ql=k;this.C=2153807872;this.Y=114690}function EF(a){var b=dF;dF=null;try{return a.Ce(null)}finally{dF=b}}
function sF(a,b){var c=a.state;if(z(b)){var d=a.Nc;try{a.Ql=null;var e=iF(d,a)}catch(g){e=g,a.state=e,a.Ql=e,e=a.De=!1}}else e=iF(a.Nc,a);a.kr||(a.state=e,null==a.sb||M.h(c,e)||oF(a,c,e));return e}function FF(a,b){var c=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b,d=F.h(c,SC),e=F.h(c,iA),g=F.h(c,jD),c=F.h(c,yC);null!=d&&(a.od=d);null!=e&&(a.yr=e);null!=g&&(a.wr=g);null!=c&&(a.kr=c)}f=DF.prototype;f.sk=x;f.ja=function(a,b,c){return pF(this,b,c,[D.c("Reaction "),D.c(Ku(this)),D.c(":")].join(""))};f.ha=function(){return qa(this)};
f.O=function(a,b){return this===b};f.ho=function(){var a=this.state,b=this.cg;this.od=this.state=this.cg=null;this.De=!0;for(var b=G(ez(b)),c=null,d=0,e=0;;)if(e<d){var g=c.$(null,e);pu(g,this);e+=1}else if(b=G(b))c=b,Kv(c)?(b=vu(c),e=wu(c),c=b,d=P(b),b=e):(b=I(c),pu(b,this),b=K(c),c=null,d=0),e=0;else break;null!=this.wr&&this.wr(a);a=this.xr;if(null==a)return null;b=a.length;for(c=0;;)if(c<b)a[c].call(null,this),c+=1;else return null};
f.Jr=function(a,b){var c=this.xr;return null==c?this.xr=[b]:c.push(b)};f.jc=function(a,b){if(!xv(this.yr))throw Error([D.c("Assert failed: "),D.c("Reaction is read only."),D.c("\n"),D.c("(fn? (.-on-set a))")].join(""));var c=this.state;this.state=b;this.yr(c,b);oF(this,c,b);return b};f.fj=function(a,b){var c=this;return c.jc(null,function(){var a=EF(c);return b.c?b.c(a):b.call(null,a)}())};f.gj=function(a,b,c){var d=this;return d.jc(null,function(){var a=EF(d);return b.h?b.h(a,c):b.call(null,a,c)}())};
f.hj=function(a,b,c,d){var e=this;return e.jc(null,function(){var a=EF(e);return b.j?b.j(a,c,d):b.call(null,a,c,d)}())};f.ij=function(a,b,c,d,e){return this.jc(null,U.ba(b,EF(this),c,d,e))};f.yh=function(a,b,c){return oF(this,b,c)};f.xh=function(a,b,c){return mF(this,b,c)};f.zh=function(a,b){var c=Dv(this.sb);nF(this,b);return!c&&Dv(this.sb)&&null==this.od?this.ho():null};
f.Ce=function(){var a=this.Ql;if(null!=a)throw a;(a=null==dF)&&rF();a&&null==this.od?this.De&&(a=this.state,this.state=this.Nc.T?this.Nc.T():this.Nc.call(null),null==this.sb||M.h(a,this.state)||oF(this,a,this.state)):(kF(this),this.De&&sF(this,!1));return this.state};
function wF(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;var c=arguments[0],b=1<b.length?new H(b.slice(1),0,null):null,e=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b,b=F.h(e,SC),d=F.h(e,iA),e=F.h(e,jD),c=new DF(c,null,!0,!1,null,null,null,null);FF(c,new y(null,3,[SC,b,iA,d,jD,e],null));return c}var GF=wF(null);
function HF(a,b){var c=IF,d=GF,e=iF(a,d);null!=d.cg&&(GF=wF(null),FF(d,c),d.Nc=a,d.od=function(){return function(){return ZE.c?ZE.c(b):ZE.call(null,b)}}(d,e),b.cljsRatom=d);return e}function JF(a){var b={};a=hF(b,a);return new V(null,2,5,X,[a,null!=b.Vi],null)};var KF;function LF(a,b){var c=b.argv;if(null==c){var c=X,d=a.constructor;a:for(var e=Lv(b),g=e.length,h=Kw,k=0;;)if(k<g)var l=e[k],h=T.j(h,qw.c(l),b[l]),k=k+1;else break a;c=new V(null,2,5,c,[d,h],null)}return c}function MF(a){var b;if(b=xv(a))a=null==a?null:a.prototype,b=null!=(null==a?null:a.reagentRender);return b}function NF(a){var b;if(b=xv(a))a=null==a?null:a.prototype,b=null!=(null==a?null:a.render);return b}if("undefined"===typeof OF)var OF=null;
function PF(a){for(;;){var b=a.reagentRender;if(Rv(b))var c=null;else throw Error("Assert failed: (ifn? f)");var d=!0===a.cljsLegacyRender?b.call(a,a):function(){var c=LF(a,a.props);switch(P(c)){case 1:return b.call(a);case 2:return b.call(a,fv(c,1));case 3:return b.call(a,fv(c,1),fv(c,2));case 4:return b.call(a,fv(c,1),fv(c,2),fv(c,3));case 5:return b.call(a,fv(c,1),fv(c,2),fv(c,3),fv(c,4));default:return b.apply(a,mt(c).slice(1))}}();if(Jv(d))return OF.c?OF.c(d):OF.call(null,d);if(Rv(d))c=MF(d)?
function(a,b,c,d){return function(){function a(a){var c=null;if(0<arguments.length){for(var c=0,d=Array(arguments.length-0);c<d.length;)d[c]=arguments[c+0],++c;c=new H(d,0,null)}return b.call(this,c)}function b(a){a=U.j(Ox,d,a);return OF.c?OF.c(a):OF.call(null,a)}a.I=0;a.N=function(a){a=G(a);return b(a)};a.w=b;return a}()}(a,b,c,d):d,a.reagentRender=c;else return d}}
var IF=new y(null,1,[yC,!0],null),RF=new y(null,1,[Wy,function(){var a=this.cljsRatom;this.cljsIsDirty=!1;return null==a?HF(function(a,c){return function(){a:{var a=KF;KF=c;try{var b=[!1];try{var g=PF(c);b[0]=!0;var h=g;break a}finally{z(b[0])||z(DE)&&(z(!1)?FE:console).error(""+D.c([D.c("Error rendering component"),D.c(QF.T?QF.T():QF.call(null))].join("")))}}finally{KF=a}h=void 0}return h}}(a,this),this):sF(a,!1)}],null);
function SF(a,b){var c=a instanceof Y?a.Sb:null;switch(c){case "getDefaultProps":throw Error([D.c("Assert failed: "),D.c("getDefaultProps not supported"),D.c("\n"),D.c("false")].join(""));case "getInitialState":return function(){return function(){var a=this.cljsState;a=null!=a?a:this.cljsState=uF.c(null);var c=b.call(this,this);return Zw.h?Zw.h(a,c):Zw.call(null,a,c)}}(a,c);case "componentWillReceiveProps":return function(){return function(a){return b.call(this,this,LF(this,a))}}(a,c);case "shouldComponentUpdate":return function(){return function(a){var c=
OE;if(c)return c;var c=this.props.argv,d=a.argv,h=null==c||null==d;return null==b?h||Gw.h(c,d):h?b.call(this,this,LF(this,this.props),LF(this,a)):b.call(this,this,c,d)}}(a,c);case "componentWillUpdate":return function(){return function(a){return b.call(this,this,LF(this,a))}}(a,c);case "componentDidUpdate":return function(){return function(a){return b.call(this,this,LF(this,a))}}(a,c);case "componentWillMount":return function(){return function(){this.cljsMountOrder=PE+=1;return null==b?null:b.call(this,
this)}}(a,c);case "componentDidMount":return function(){return function(){return b.call(this,this)}}(a,c);case "componentWillUnmount":return function(){return function(){var a=this.cljsRatom;null!=a&&BF(a);this.cljsIsDirty=!1;return null==b?null:b.call(this,this)}}(a,c);default:return null}}
function TF(a,b,c){var d=SF(a,b);if(z(z(d)?b:d)&&!Rv(b))throw Error([D.c("Assert failed: "),D.c([D.c("Expected function in "),D.c(c),D.c(a),D.c(" but got "),D.c(b)].join("")),D.c("\n"),D.c("(ifn? f)")].join(""));return z(d)?d:b}var UF=new y(null,3,[jC,null,bE,null,UB,null],null),VF=function(a){return function(b){return function(c){var d=F.h(N.c?N.c(b):N.call(null,b),c);if(null!=d)return d;d=a.c?a.c(c):a.call(null,c);$w.L(b,T,c,d);return d}}(function(){var a=Kw;return Ww?Ww(a):Vw.call(null,a)}())}(ME);
function WF(a){return Wv(function(a,c,d){return T.j(a,qw.c(VF.c?VF.c(c):VF.call(null,c)),d)},Kw,a)}
function XF(a){var b=Vy(a),c=I(ky(b));if(!(0<P(b)))throw Error([D.c("Assert failed: "),D.c("Missing reagent-render"),D.c("\n"),D.c("(pos? (count renders))")].join(""));if(1!==P(b))throw Error([D.c("Assert failed: "),D.c("Too many render functions supplied"),D.c("\n"),D.c("(\x3d\x3d 1 (count renders))")].join(""));if(!Rv(c))throw Error([D.c("Assert failed: "),D.c([D.c("Render must be a function, not "),D.c(Az(R([c],0),$s()))].join("")),D.c("\n"),D.c("(ifn? render-fun)")].join(""));var d=function(){var b=
Xy.c(a);return z(b)?b:Yy.c(a)}(),b=null==d,e=z(d)?d:Wy.c(a),g=""+D.c(function(){var b=aB.c(a);return z(b)?b:NE(e)}());a:switch(g){case "":c=""+D.c(Fz());break a;default:c=g}d=Wv(function(a,b,c,d,e){return function(a,b,c){return T.j(a,b,TF(b,c,e))}}(d,b,e,g,c),Kw,a);return T.w(d,aB,c,R([GD,!1,tA,b,Xy,e,Wy,Wy.c(RF)],0))}function YF(a){return Wv(function(a,c,d){a[rw(c)]=d;return a},{},a)}
function ZF(a){if(!Hv(a))throw Error("Assert failed: (map? body)");return GE.createClass(YF(XF(Uy.w(R([UF,WF(a)],0)))))}
var $F=function $F(b){var c=function(){var c=null==b?null:b._reactInternalInstance;c=z(c)?c:b;return null==c?null:c._currentElement}(),d=function(){var b=null==c?null:c.type;return null==b?null:b.displayName}(),e=function(){var b=null==c?null:c._owner,b=null==b?null:$F.c?$F.c(b):$F.call(null,b);return null==b?null:[D.c(b),D.c(" \x3e ")].join("")}(),d=[D.c(e),D.c(d)].join("");return Dv(d)?null:d};
function QF(){var a=KF;var b=$F(a);z(b)?a=b:(a=null==a?null:a.constructor,a=null==a?null:NE(a));return Dv(a)?"":[D.c(" (in "),D.c(a),D.c(")")].join("")}
function aG(a){if(!Rv(a))throw Error([D.c("Assert failed: "),D.c([D.c("Expected a function, not "),D.c(Az(R([a],0),$s()))].join("")),D.c("\n"),D.c("(ifn? f)")].join(""));NF(a)&&!MF(a)&&z(DE)&&(z(!1)?FE:console).warn([D.c("Warning: "),D.c("Using native React classes directly in Hiccup forms "),D.c("is not supported. Use create-element or "),D.c("adapt-react-class instead: "),D.c(function(){var b=NE(a);return Dv(b)?a:b}()),D.c(QF())].join(""));if(MF(a))return a.cljsReactClass=a;var b=Av(a),b=T.j(b,
FC,a),b=ZF(b);return a.cljsReactClass=b};function bG(a,b,c){if(kw(c))return c=U.h(nw,Z.h(a,c)),b.c?b.c(c):b.call(null,c);if(Pv(c))return c=jz(Z.h(a,c)),b.c?b.c(c):b.call(null,c);if(Iv(c))return c=nt(function(b,c){return sv.h(b,a.c?a.c(c):a.call(null,c))},c,c),b.c?b.c(c):b.call(null,c);Ev(c)&&(c=Rw.h(null==c?null:tt(c),Z.h(a,c)));return b.c?b.c(c):b.call(null,c)}var cG=function cG(b,c){return bG(Pw.h(cG,b),Xv,b.c?b.c(c):b.call(null,c))};var dG=/([^\s\.#]+)(?:#([^\s\.#]+))?(?:\.([^\s#]+))?/;function eG(a){return a instanceof Y||a instanceof Nu}var fG={"class":"className","for":"htmlFor",charset:"charSet"};function gG(a,b,c){if(eG(b)){var d=rw(b);d=fG.hasOwnProperty(d)?fG[d]:null;b=null==d?fG[rw(b)]=ME(b):d}a[b]=hG.c?hG.c(c):hG.call(null,c);return a}
function hG(a){return"object"!==n(a)?a:eG(a)?rw(a):Hv(a)?Wv(gG,{},a):Ev(a)?Jz(a):Rv(a)?function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){return U.h(a,b)}b.I=0;b.N=function(a){a=G(a);return c(a)};b.w=c;return b}():Jz(a)}function iG(a,b,c){a=null==a?{}:a;a[b]=c;return a}
var jG=new az(null,new y(null,6,["url",null,"tel",null,"text",null,"textarea",null,"password",null,"search",null],null),null),kG=function kG(b){var c=b.cljsInputElement;if(null==c)return null;b.cljsInputDirty=!1;var d=b.cljsRenderedValue,e=b.cljsDOMValue;if(Gw.h(d,e)){if(c===document.activeElement&&Tv(jG,c.type)&&"string"===typeof d&&"string"===typeof e){var g=c.value;if(Gw.h(g,e))return XE(function(){return function(){return kG.c?kG.c(b):kG.call(null,b)}}(g,d,e,c,c));e=P(g)-c.selectionStart;e=P(d)-
e;b.cljsDOMValue=d;c.value=d;c.selectionStart=e;return c.selectionEnd=e}b.cljsDOMValue=d;return c.value=d}return null};function lG(a,b,c){a.cljsDOMValue=c.target.value;z(a.cljsInputDirty)||(a.cljsInputDirty=!0,XE(function(){return kG(a)}));return b.c?b.c(c):b.call(null,c)}
function mG(a){var b=KF;if(z(function(){var b=null!=a;return b?(b=a.hasOwnProperty("onChange"),z(b)?a.hasOwnProperty("value"):b):b}())){var c=a.value,d=null==c?"":c,e=a.onChange;null==b.cljsInputElement&&(b.cljsDOMValue=d);b.cljsRenderedValue=d;delete a.value;a.defaultValue=d;a.onChange=function(a,c,d,e){return function(a){return lG(b,e,a)}}(a,c,d,e);a.ref=function(){return function(a){return b.cljsInputElement=a}}(a,c,d,e)}}
var nG=null,pG=new y(null,3,[cD,"ReagentInput",IB,kG,FC,function(a,b,c,d){mG(c);return oG.L?oG.L(a,b,c,d):oG.call(null,a,b,c,d)}],null);function qG(a){if(Hv(a))try{var b=F.h(a,MA)}catch(c){b=null}else b=null;return b}function rG(a){var b=qG(Av(a));return null==b?qG(S(a,1,null)):b}var sG={};
function tG(a,b,c){var d=a.name,e=S(b,c,null),g=null==e||Hv(e);var e=hG(g?e:null),h=a.id,e=null!=h&&null==(null==e?null:e.id)?iG(e,"id",h):e;a=a.className;null==a?a=e:(h=null==e?null:e.className,a=iG(e,"className",null==h?a:[D.c(a),D.c(" "),D.c(h)].join("")));c+=g?1:0;a:switch(d){case "input":case "textarea":g=!0;break a;default:g=!1}if(g)return g=X,null==nG&&(nG=ZF(pG)),b=zv(new V(null,5,5,g,[nG,b,d,a,c],null),Av(b)),uG.c?uG.c(b):uG.call(null,b);g=qG(Av(b));g=null==g?a:iG(a,"key",g);return oG.L?
oG.L(b,d,g,c):oG.call(null,b,d,g,c)}function vG(a){return""+D.c(cG(function(a){if(xv(a)){var b=NE(a);switch(b){case "":return a;default:return Ou.c(b)}}else return a},a))}function wG(a,b){return[D.c(U.h(D,b)),D.c(": "),D.c(vG(a)),D.c("\n"),D.c(QF())].join("")}
function xG(a){for(;;){if(!(0<P(a)))throw Error([D.c("Assert failed: "),D.c(wG(a,R(["Hiccup form should not be empty"],0))),D.c("\n"),D.c("(pos? (count v))")].join(""));var b=S(a,0,null);if(!eG(b)&&"string"!==typeof b&&!Rv(b))throw Error([D.c("Assert failed: "),D.c(wG(a,R(["Invalid Hiccup form"],0))),D.c("\n"),D.c("(valid-tag? tag)")].join(""));if(eG(b)||"string"===typeof b){var b=rw(b),c=b.indexOf("\x3e");switch(c){case -1:var c=b,b=sG,d=c,b=b.hasOwnProperty(d)?b[d]:null;if(null==b){var b=c,e=K(kz(dG,
rw(c))),d=S(e,0,null),g=S(e,1,null),e=S(e,2,null),e=null==e?null:vE(e,/\./," ");if(!z(d))throw Error([D.c("Assert failed: "),D.c([D.c("Invalid tag: '"),D.c(c),D.c("'"),D.c(QF())].join("")),D.c("\n"),D.c("tag")].join(""));b=sG[b]={name:d,id:g,className:e}}return tG(b,a,1);case 0:c=S(a,1,null);if(!M.h("\x3e",b))throw Error([D.c("Assert failed: "),D.c(wG(a,R(["Invalid Hiccup tag"],0))),D.c("\n"),D.c('(\x3d "\x3e" n)')].join(""));if("string"!==typeof c&&!xv(c))throw Error([D.c("Assert failed: "),D.c(wG(a,
R(["Expected React component in"],0))),D.c("\n"),D.c("(or (string? comp) (fn? comp))")].join(""));return tG({name:c},a,2);default:a=new V(null,2,5,X,[b.substring(0,c),T.j(a,0,b.substring(c+1))],null)}}else return c=b.cljsReactClass,b=null==c?aG(b):c,c={argv:a},a=rG(a),null!=a&&(c.key=a),GE.createElement(b,c)}}function uG(a){return"object"!==n(a)?a:Jv(a)?xG(a):Pv(a)?yG.c?yG.c(a):yG.call(null,a):eG(a)?rw(a):(null!=a?a.C&2147483648||x===a.Qa||(a.C?0:C(mu,a)):C(mu,a))?Az(R([a],0),$s()):a}OF=uG;
function yG(a){var b={},c=JF(function(b){return function(){for(var c=mt(a),d=c.length,e=0;;)if(e<d){var l=c[e];Jv(l)&&null==rG(l)&&(b["no-key"]=!0);c[e]=uG(l);e+=1}else break;return c}}(b)),d=S(c,0,null),c=S(c,1,null);z(c)&&z(DE)&&(z(!1)?FE:console).warn([D.c("Warning: "),D.c(wG(a,R(["Reactive deref not supported in lazy seq, ","it should be wrapped in doall"],0)))].join(""));z(b["no-key"])&&z(DE)&&(z(!1)?FE:console).warn([D.c("Warning: "),D.c(wG(a,R(["Every element in a seq should have a unique :key"],
0)))].join(""));return d}function oG(a,b,c,d){var e=P(a)-d;switch(e){case 0:return GE.createElement(b,c);case 1:return GE.createElement(b,c,uG(S(a,d,null)));default:return GE.createElement.apply(null,Wv(function(){return function(a,b,c){b>=d&&a.push(uG(c));return a}}(e),[b,c],a))}};if("undefined"===typeof zG)var zG=null;if("undefined"===typeof AG)var AG=null;function BG(){if(null!=AG)return AG;if("undefined"!==typeof ReactDOM)return AG=ReactDOM;if("undefined"!==typeof require){var a=AG=require("react-dom");if(z(a))return a;throw Error("require('react-dom') failed");}throw Error("js/ReactDOM is missing");}if("undefined"===typeof CG){var CG,DG=Kw;CG=Ww?Ww(DG):Vw.call(null,DG)}
function EG(a,b){var c=OE;OE=!0;try{return BG().render(a.T?a.T():a.call(null),b,function(){return function(){var c=OE;OE=!1;try{return $w.L(CG,T,b,new V(null,2,5,X,[a,b],null)),VE(YE,"afterRender"),null}finally{OE=c}}}(c))}finally{OE=c}}function FG(a,b){return EG(a,b)}function GG(a,b){rF();return EG(function(){return uG(xv(a)?a.T?a.T():a.call(null):a)},b)};function HG(a,b){return GG(a,b)}fa("reagent.core.force_update_all",function(){rF();rF();for(var a=G(ky(N.c?N.c(CG):N.call(null,CG))),b=null,c=0,d=0;;)if(d<c){var e=b.$(null,d);U.h(FG,e);d+=1}else if(a=G(a))b=a,Kv(b)?(a=vu(b),d=wu(b),b=a,c=P(a),a=d):(a=I(b),U.h(FG,a),a=K(b),b=null,c=0),d=0;else break;return VE(YE,"afterRender")});function IG(a){return XE(a)};var LG=function LG(b){if(null!=b&&null!=b.$p)return b.$p();var c=LG[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=LG._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("PushbackReader.read-char",b);},MG=function MG(b,c){if(null!=b&&null!=b.aq)return b.aq(0,c);var d=MG[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=MG._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("PushbackReader.unread",b);};
function NG(a,b,c){this.s=a;this.buffer=b;this.hn=c}NG.prototype.$p=function(){return 0===this.buffer.length?(this.hn+=1,this.s[this.hn]):this.buffer.pop()};NG.prototype.aq=function(a,b){return this.buffer.push(b)};function OG(a){var b=!/[^\t\n\r ]/.test(a);return z(b)?b:","===a}function PG(a){throw Error(U.h(D,a));}
function QG(a,b){for(var c=new Es(b),d=LG(a);;){var e;if(!(e=null==d||OG(d))){e=d;var g="#"!==e;e=g?(g="'"!==e)?(g=":"!==e)?RG.c?RG.c(e):RG.call(null,e):g:g:g}if(e)return MG(a,d),c.toString();c.append(d);d=LG(a)}}function SG(a){for(;;){var b=LG(a);if("\n"===b||"\r"===b||null==b)return a}}var TG=mz("^([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+))(N)?$"),UG=mz("^([-+]?[0-9]+)/([0-9]+)$"),VG=mz("^([-+]?[0-9]+(\\.[0-9]*)?([eE][-+]?[0-9]+)?)(M)?$"),WG=mz("^[:]?([^0-9/].*/)?([^0-9/][^/]*)$");
function XG(a,b){var c=a.exec(b);return null!=c&&c[0]===b?1===c.length?c[0]:c:null}var YG=mz("^[0-9A-Fa-f]{2}$"),ZG=mz("^[0-9A-Fa-f]{4}$");function $G(a,b,c){return z(kz(a,c))?c:PG(R(["Unexpected unicode escape \\",b,c],0))}
function aH(a){var b=LG(a),c="t"===b?"\t":"r"===b?"\r":"n"===b?"\n":"\\"===b?"\\":'"'===b?'"':"b"===b?"\b":"f"===b?"\f":null;z(c)?b=c:"x"===b?(a=(new Es(LG(a),LG(a))).toString(),b=String.fromCharCode(parseInt($G(YG,b,a),16))):"u"===b?(a=(new Es(LG(a),LG(a),LG(a),LG(a))).toString(),b=String.fromCharCode(parseInt($G(ZG,b,a),16))):b=/[^0-9]/.test(b)?PG(R(["Unexpected unicode escape \\",b],0)):String.fromCharCode(b);return b}
function bH(a,b){for(var c=[];;){a:{var d=b;for(var e=OG,g=LG(d);;)if(z(e.c?e.c(g):e.call(null,g)))g=LG(d);else{d=g;break a}}z(d)||PG(R(["EOF while reading"],0));if(a===d)return c;e=RG.c?RG.c(d):RG.call(null,d);z(e)?d=e.h?e.h(b,d):e.call(null,b,d):(MG(b,d),d=cH.L?cH.L(b,!0,null,!0):cH.call(null,b,!0,null));d!==b&&c.push(d)}}function dH(a,b){return PG(R(["Reader for ",b," not implemented yet"],0))}
function eH(a,b){var c=LG(a),d=fH.c?fH.c(c):fH.call(null,c);if(z(d))return d.h?d.h(a,b):d.call(null,a,b);d=gH.h?gH.h(a,c):gH.call(null,a,c);return z(d)?d:PG(R(["No dispatch macro for ",c],0))}function hH(a,b){return PG(R(["Unmatched delimiter ",b],0))}function iH(a){a=bH(")",a);for(var b=a.length,c=Qu;;)if(0<b)var d=b-1,c=c.wa(null,a[b-1]),b=d;else return c}function jH(a){return Nx(bH("]",a))}
function kH(a){a=bH("}",a);var b=a.length;if(!Sv(b))throw Error([D.c("Argument must be an integer: "),D.c(b)].join(""));0!==(b&1)&&PG(R(["Map literal must contain an even number of forms"],0));if(b<=2*my){for(var b=[],c=0;;)if(c<a.length){var d=a[c],e=a[c+1];if(-1===gy(b,d)){var g=b;g.push(d);g.push(e)}else throw Error([D.c("Duplicate key: "),D.c(d)].join(""));c+=2}else break;a=new y(null,a.length/2,a,null)}else{b=a.length;c=qu(ny);for(d=0;;)if(d<b){tu(c,a[d],a[d+1]);if(Gw.h(st(c),d/2+1))throw Error([D.c("Duplicate key: "),
D.c(a[d])].join(""));d+=2}else break;a=su(c)}return a}function lH(a){for(var b=new Es,c=LG(a);;){if(null==c)return PG(R(["EOF while reading"],0));if("\\"===c)b.append(aH(a));else{if('"'===c)return b.toString();b.append(c)}c=LG(a)}}
function mH(a){for(var b=new Es,c=LG(a);;){if(null==c)return PG(R(["EOF while reading"],0));if("\\"===c){b.append(c);var d=LG(a);if(null==d)return PG(R(["EOF while reading"],0));var e=function(){var a=b;a.append(d);return a}(),g=LG(a)}else{if('"'===c)return b.toString();e=function(){var a=b;a.append(c);return a}();g=LG(a)}b=e;c=g}}
function nH(a,b){var c=QG(a,b),d=Qa(c,"/");z(z(d)?1!==c.length:d)?c=Ou.h(fw(c,0,c.indexOf("/")),fw(c,c.indexOf("/")+1,c.length)):(d=Ou.c(c),c="nil"===c?null:"true"===c?!0:"false"===c?!1:"/"===c?TC:d);return c}
function oH(a,b){var c=QG(a,b),d=c.substring(1);return 1===d.length?d:"tab"===d?"\t":"return"===d?"\r":"newline"===d?"\n":"space"===d?" ":"backspace"===d?"\b":"formfeed"===d?"\f":"u"===d.charAt(0)?String.fromCharCode(parseInt(d.substring(1),16)):"o"===d.charAt(0)?dH(0,c):PG(R(["Unknown character literal: ",c],0))}
function pH(a){a=QG(a,LG(a));var b=XG(WG,a);a=b[0];var c=b[1],b=b[2];return void 0!==c&&":/"===c.substring(c.length-2,c.length)||":"===b[b.length-1]||-1!==a.indexOf("::",1)?PG(R(["Invalid token: ",a],0)):null!=c&&0<c.length?qw.h(c.substring(0,c.indexOf("/")),b):qw.c(a)}function qH(a){return function(b){b=cH.L?cH.L(b,!0,null,!0):cH.call(null,b,!0,null);b=vt(Qu,b);return vt(b,a)}}function rH(){return function(){return PG(R(["Unreadable form"],0))}}
function sH(a){var b=cH.L?cH.L(a,!0,null,!0):cH.call(null,a,!0,null);b=b instanceof Nu?new y(null,1,[sD,b],null):"string"===typeof b?new y(null,1,[sD,b],null):b instanceof Y?vv([b,!0]):b;Hv(b)||PG(R(["Metadata must be Symbol,Keyword,String or Map"],0));a=cH.L?cH.L(a,!0,null,!0):cH.call(null,a,!0,null);return(null!=a?a.C&262144||x===a.Jz||(a.C?0:C(Yt,a)):C(Yt,a))?zv(a,Uy.w(R([Av(a),b],0))):PG(R(["Metadata can only be applied to IWithMetas"],0))}
function tH(a){a=bH("}",a);for(var b=a.length,c=qu(cz),d=0;;)if(d<b){ru(c,a[d]);if(!M.h(P(c),d+1))throw Error([D.c("Duplicate key: "),D.c(a[d])].join(""));d+=1}else break;return su(c)}function uH(a){return mz(mH(a))}function vH(a){cH.L?cH.L(a,!0,null,!0):cH.call(null,a,!0,null);return a}function RG(a){return'"'===a?lH:":"===a?pH:";"===a?SG:"'"===a?qH(vD):"@"===a?qH(VD):"^"===a?sH:"`"===a?dH:"~"===a?dH:"("===a?iH:")"===a?hH:"["===a?jH:"]"===a?hH:"{"===a?kH:"}"===a?hH:"\\"===a?oH:"#"===a?eH:null}
function fH(a){return"{"===a?tH:"\x3c"===a?rH():'"'===a?uH:"!"===a?SG:"_"===a?vH:null}
function cH(a,b,c){for(;;){var d=LG(a);if(null==d)return z(b)?PG(R(["EOF while reading"],0)):c;if(!OG(d))if(";"===d)a=SG.h?SG.h(a,d):SG.call(null,a);else{var e=RG(d);if(z(e))e=e.h?e.h(a,d):e.call(null,a,d);else{var g,e=a;!(g=!/[^0-9]/.test(d))&&(g="+"===d||"-"===d)&&(g=LG(e),MG(e,g),g=!/[^0-9]/.test(g));if(g)a:for(e=a,d=new Es(d),g=LG(e);;){var h=null==g;h||(h=(h=OG(g))?h:RG.c?RG.c(g):RG.call(null,g));if(z(h)){MG(e,g);d=e=d.toString();z(XG(TG,d))?(d=XG(TG,d),g=d[2],null!=(M.h(g,"")?null:g)?d=0:(g=
z(d[3])?[d[3],10]:z(d[4])?[d[4],16]:z(d[5])?[d[5],8]:z(d[6])?[d[7],parseInt(d[6],10)]:[null,null],h=g[0],null==h?d=null:(g=parseInt(h,g[1]),d="-"===d[1]?-g:g))):z(XG(UG,d))?(d=XG(UG,d),d=parseInt(d[1],10)/parseInt(d[2],10)):d=z(XG(VG,d))?parseFloat(d):null;e=z(d)?d:PG(R(["Invalid number format [",e,"]"],0));break a}d.append(g);g=LG(e)}else e=nH(a,d)}if(e!==a)return e}}}
var wH=function(a,b){return function(c,d){return F.h(z(d)?b:a,c)}}(new V(null,13,5,X,[null,31,28,31,30,31,30,31,31,30,31,30,31],null),new V(null,13,5,X,[null,31,29,31,30,31,30,31,31,30,31,30,31],null)),xH=/(\d\d\d\d)(?:-(\d\d)(?:-(\d\d)(?:[T](\d\d)(?::(\d\d)(?::(\d\d)(?:[.](\d+))?)?)?)?)?)?(?:[Z]|([-+])(\d\d):(\d\d))?/;function yH(a){a=parseInt(a,10);return ht(isNaN(a))?a:null}
function zH(a,b,c,d){a<=b&&b<=c||PG(R([[D.c(d),D.c(" Failed:  "),D.c(a),D.c("\x3c\x3d"),D.c(b),D.c("\x3c\x3d"),D.c(c)].join("")],0));return b}
function AH(a){var b=kz(xH,a);S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null),g=S(b,4,null),h=S(b,5,null),k=S(b,6,null),l=S(b,7,null),m=S(b,8,null),p=S(b,9,null),u=S(b,10,null);if(ht(b))return PG(R([[D.c("Unrecognized date/time syntax: "),D.c(a)].join("")],0));var A=yH(c),B=function(){var a=yH(d);return z(a)?a:1}();a=function(){var a=yH(e);return z(a)?a:1}();var b=function(){var a=yH(g);return z(a)?a:0}(),c=function(){var a=yH(h);return z(a)?a:0}(),E=function(){var a=yH(k);return z(a)?a:
0}(),J=function(){a:if(M.h(3,P(l)))var a=l;else if(3<P(l))a=l.substring(0,3);else for(a=new Es(l);;)if(3>a.Be.length)a=a.append("0");else{a=a.toString();break a}a=yH(a);return z(a)?a:0}(),m=(M.h(m,"-")?-1:1)*(60*function(){var a=yH(p);return z(a)?a:0}()+function(){var a=yH(u);return z(a)?a:0}());return new V(null,8,5,X,[A,zH(1,B,12,"timestamp month field must be in range 1..12"),zH(1,a,function(){var a=0===cw(A,4);z(a)&&(a=ht(0===cw(A,100)),a=z(a)?a:0===cw(A,400));return wH.h?wH.h(B,a):wH.call(null,
B,a)}(),"timestamp day field must be in range 1..last day in month"),zH(0,b,23,"timestamp hour field must be in range 0..23"),zH(0,c,59,"timestamp minute field must be in range 0..59"),zH(0,E,M.h(c,59)?60:59,"timestamp second field must be in range 0..60"),zH(0,J,999,"timestamp millisecond field must be in range 0..999"),m],null)}
var BH,CH=Uy.w(R([new y(null,4,["inst",function(a){if("string"===typeof a){var b=AH(a);if(z(b)){a=S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null),g=S(b,4,null),h=S(b,5,null),k=S(b,6,null);b=S(b,7,null);b=new Date(Date.UTC(a,c-1,d,e,g,h,k)-6E4*b)}else b=PG(R([[D.c("Unrecognized date/time syntax: "),D.c(a)].join("")],0))}else b=PG(R(["Instance literal expects a string for its timestamp."],0));return b},"uuid",function(a){return"string"===typeof a?new Oz(a,null):PG(R(["UUID literal expects a string as its representation."],
0))},"queue",function(a){return Jv(a)?Rw.h($x,a):PG(R(["Queue literal expects a vector for its elements."],0))},"js",function(a){if(Jv(a)){var b=[];a=G(a);for(var c=null,d=0,e=0;;)if(e<d){var g=c.$(null,e);b.push(g);e+=1}else if(a=G(a))c=a,Kv(c)?(a=vu(c),e=wu(c),c=a,d=P(a),a=e):(a=I(c),b.push(a),a=K(c),c=null,d=0),e=0;else break;return b}if(Hv(a)){b={};a=G(a);c=null;for(e=d=0;;)if(e<d){var h=c.$(null,e),g=S(h,0,null),h=S(h,1,null);b[rw(g)]=h;e+=1}else if(a=G(a))Kv(a)?(d=vu(a),a=wu(a),c=d,d=P(d)):
(d=I(a),c=S(d,0,null),d=S(d,1,null),b[rw(c)]=d,a=K(a),c=null,d=0),e=0;else break;return b}return PG(R([[D.c("JS literal expects a vector or map containing "),D.c("only string or unqualified keyword keys")].join("")],0))}],null),Kw],0));BH=Ww?Ww(CH):Vw.call(null,CH);var DH=Ww?Ww(null):Vw.call(null,null);
function gH(a,b){var c=nH(a,b),d=F.h(N.c?N.c(BH):N.call(null,BH),""+D.c(c)),e=N.c?N.c(DH):N.call(null,DH);return z(d)?(c=cH(a,!0,null),d.c?d.c(c):d.call(null,c)):z(e)?(d=cH(a,!0,null),e.h?e.h(c,d):e.call(null,c,d)):PG(R(["Could not find tag parser for ",""+D.c(c)," in ",Az(R([jy(N.c?N.c(BH):N.call(null,BH))],0),$s())],0))}function EH(a,b){var c=""+D.c(a);F.h(N.c?N.c(BH):N.call(null,BH),c);$w.L(BH,T,c,b)};function FH(a){return Z.h(function(b){return a.item(b)},new iz(null,0,a.length,1,null))};function GH(a,b){try{var c=Mz(qd(a),R([Nz,!0],0)),d=b.c?b.c(c):b.call(null,c);return new V(null,2,5,X,[z(d)?c:a,d],null)}catch(e){return new V(null,2,5,X,[a,!1],null)}}function HH(a,b){var c=rw(kB.c(b));c=zE(c,"-");c=wE(Dw.h(new V(null,1,5,X,[I(c)],null),Z.h(yE,Pu(c))));return Sh(a,c)}
var IH=uF.c(new y(null,4,[VB,function(a,b){var c=null==a,d=c?a:""+D.c(a);if(z(b))c=new V(null,2,5,X,[d,!Ca(a)],null);else var e=da(a),c=new V(null,2,5,X,[d,z(e)?e:c],null);return c},aE,function(a,b){return ht(b)&&Ca(a)?new V(null,2,5,X,[null,!0],null):GH(a,Hv)},cC,function(a,b){return ht(b)&&Ca(a)?new V(null,2,5,X,[null,!0],null):GH(a,Jv)},QA,function(a,b){var c=(c="string"===typeof a)?lz(/^-?\d+$/,a):c;if(z(c)){if("string"!==typeof a)throw Error("Cannot read from non-string object.");c=cH(new NG(a,
[],-1),!1,null)}else c=null;return ht(b)&&Ca(a)?new V(null,2,5,X,[null,!0],null):new V(null,2,5,X,[c,Sv(c)],null)}],null));function JH(a,b,c){return F.h(N.c?N.c(IH):N.call(null,IH),a).call(null,HH(c,b),F.j(b,fE,!1))}function KH(a){return function(b,c){var d=OB.c(c);return T.j(b,kB.c(c),Tv(N.c?N.c(IH):N.call(null,IH),d)?JH(d,c,a):new V(null,2,5,X,[HH(a,c),!0],null))}};var LH=function(){var a=new y(null,5,[UD,console.log.bind(console),gB,console.warn.bind(console),lD,console.error.bind(console),DA,z(console.group)?console.group.bind(console):console.log.bind(console),DD,z(console.groupEnd)?console.groupEnd.bind(console):function(){return Qu}],null);return Ww?Ww(a):Vw.call(null,a)}();function MH(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;NH(arguments[0],1<b.length?new H(b.slice(1),0,null):null)}
function NH(a,b){if(!Tv(N.c?N.c(LH):N.call(null,LH),a))throw Error([D.c("Assert failed: "),D.c([D.c("re-frame: log called with unknown level: "),D.c(a)].join("")),D.c("\n"),D.c("(contains? (clojure.core/deref loggers) level)")].join(""));return U.h(function(){var b=N.c?N.c(LH):N.call(null,LH);return a.c?a.c(b):a.call(null,b)}(),b)};var OH=uF.c(tv),PH=uF.c(tv);function QH(a){a=I(F.h(a,JB));return Ca(a)?null:a}function RH(a,b){$w.j(PH,sv,new V(null,2,5,X,[a,b],null))}function SH(a,b,c,d){Ca(c.id)&&NH(gB,R([[D.c("Component with selector '"),D.c(a),D.c("' has no ID ")].join(""),c],0));return HG(new V(null,1,5,X,[function(){return b.h?b.h(c,d):b.call(null,c,d)}],null),c)}function TH(a){return Wv(function(a,c,d){var b=S(d,0,null);d=S(d,1,null);return z(d)?a:sv.h(a,[D.c(rw(c)),D.c(" -\x3e '"),D.c(b),D.c("'")].join(""))},tv,a)}
function UH(a){return Lw(Ov,Z.h(function(a){S(a,0,null);a=S(a,1,null);S(a,0,null);return S(a,1,null)},a))}function VH(a,b,c,d){return b.h?b.h(c,d):b.call(null,c,d)}function WH(a,b,c,d,e){var g=S(d,0,null);d=S(d,1,null);d=nt(KH(e),Kw,d);z(UH(d))?a.L?a.L(c,g,e,d):a.call(null,c,g,e,d):(a=NH,e=e.id,c=xE(" ",TH(d)),b=wm("Invalid arguments for %s #%s: %s",b,e,c),a(gB,R([b],0)))}
function XH(){for(var a,b,c,d=G(new V(null,2,5,X,[new V(null,3,5,X,[VH,"control",PH],null),new V(null,3,5,X,[SH,"component",OH],null)],null)),e=null,g=0,h=0;;)if(h<g){a=e.$(null,h);c=S(a,0,null);b=S(a,1,null);a=S(a,2,null);a=G(N.c?N.c(a):N.call(null,a));for(var k=null,l=0,m=0;;)if(m<l){var p=k.$(null,m),u=S(p,0,null),A=S(p,1,null);S(p,2,null);for(var p=FH(document.querySelectorAll(u)),p=G(p),B=null,E=0,J=0;;)if(J<E){var L=B.$(null,J);WH(c,b,u,A,L);J+=1}else if(p=G(p))B=p,Kv(B)?(p=vu(B),J=wu(B),B=
p,E=P(p),p=J):(p=I(B),WH(c,b,u,A,p),p=K(B),B=null,E=0),J=0;else break;m+=1}else if(a=G(a)){if(Kv(a))l=vu(a),a=wu(a),k=l,l=P(l);else{m=I(a);k=S(m,0,null);l=S(m,1,null);S(m,2,null);m=FH(document.querySelectorAll(k));m=G(m);u=null;for(p=A=0;;)if(p<A)B=u.$(null,p),WH(c,b,k,l,B),p+=1;else if(m=G(m))u=m,Kv(u)?(m=vu(u),p=wu(u),u=m,A=P(m),m=p):(m=I(u),WH(c,b,k,l,m),m=K(u),u=null,A=0),p=0;else break;a=K(a);k=null;l=0}m=0}else break;h+=1}else if(b=G(d)){d=b;if(Kv(d))c=vu(d),d=wu(d),b=c,c=P(c),e=b,g=c;else{e=
I(d);c=S(e,0,null);b=S(e,1,null);a=S(e,2,null);e=G(N.c?N.c(a):N.call(null,a));g=null;for(a=h=0;;)if(a<h){m=g.$(null,a);k=S(m,0,null);l=S(m,1,null);S(m,2,null);m=FH(document.querySelectorAll(k));m=G(m);u=null;for(p=A=0;;)if(p<A)B=u.$(null,p),WH(c,b,k,l,B),p+=1;else if(m=G(m))u=m,Kv(u)?(m=vu(u),p=wu(u),u=m,A=P(m),m=p):(m=I(u),WH(c,b,k,l,m),m=K(u),u=null,A=0),p=0;else break;a+=1}else if(e=G(e)){if(Kv(e))h=vu(e),e=wu(e),g=h,h=P(h);else{a=I(e);g=S(a,0,null);h=S(a,1,null);S(a,2,null);a=FH(document.querySelectorAll(g));
a=G(a);k=null;for(m=l=0;;)if(m<l)u=k.$(null,m),WH(c,b,g,h,u),m+=1;else if(a=G(a))k=a,Kv(k)?(a=vu(k),m=wu(k),k=a,l=P(a),a=m):(a=I(k),WH(c,b,g,h,a),a=K(k),k=null,l=0),m=0;else break;e=K(e);g=null;h=0}a=0}else break;d=K(d);e=null;g=0}h=0}else break};var Sw=Rw.h($x,tv);function YH(a,b){CF(a,b)}function ZH(a,b){setTimeout(a,b)}function $H(a){return null!=a&&x===a.sk?[D.c(z(Mu.h?Mu.h(tF,a):Mu.call(null,tF,a))?"ra":z(Mu.h?Mu.h(yF,a):Mu.call(null,yF,a))?"rc":z(Mu.h?Mu.h(DF,a):Mu.call(null,DF,a))?"rx":z(Mu.h?Mu.h(xF,a):Mu.call(null,xF,a))?"tr":"other"),D.c(Ku(a))].join(""):null};function aI(a){var b=null!=a&&(a.C&64||x===a.Pb)?U.h(Xw,a):a;a=F.h(b,LC);var c=F.h(b,SD),b=F.h(b,EC);return new y(null,3,[LC,z(a)?a:Vz,SD,c,EC,b],null)}function bI(a,b,c){return px(a,new V(null,2,5,X,[NC,b],null),c)}function cI(a,b){return nx(a,new V(null,2,5,X,[oD,b],null))}function dI(a,b){for(var c=a;;){var d=tx.c(c);if(Dv(d))return c;var e=Bv(d),g=YC.c(c),c=T.w(c,tx,null==d?null:Rt(d),R([YC,sv.h(g,e)],0)),e=F.h(e,b),c=z(e)?e.c?e.c(c):e.call(null,c):c}};Ww||Vw.call(null,0);var eI=Kw;Ww||Vw.call(null,eI);var fI=new az(null,new y(null,4,[XA,null,DC,null,HD,null,PD,null],null),null),gI,hI=Kw;gI=Ww?Ww(hI):Vw.call(null,hI);function iI(a,b){return F.h(F.h(N.c?N.c(gI):N.call(null,gI),a),b)}function jI(a,b,c){$w.L(gI,px,new V(null,2,5,X,[a,b],null),c);return c}
var kI=function kI(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return kI.T();case 1:return kI.c(arguments[0]);case 2:return kI.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};kI.T=function(){var a=Kw;return Zw.h?Zw.h(gI,a):Zw.call(null,gI,a)};kI.c=function(a){if(!z(fI.c?fI.c(a):fI.call(null,a)))throw Error("Assert failed: (kinds kind)");return $w.j(gI,wv,a)};
kI.h=function(a,b){if(!z(fI.c?fI.c(a):fI.call(null,a)))throw Error("Assert failed: (kinds kind)");return z(iI(a,b))?$w.w(gI,qx,new V(null,1,5,X,[a],null),wv,R([b],0)):NH(gB,R(["re-frame: can't clear",""+D.c(a),"handler for",[D.c(b),D.c(". Handler not found.")].join("")],0))};kI.I=2;var lI=function lI(b,c){var d=G(c),e=I(d);if(d=K(d)){var g=F.h(b,e);return z(g)?(d=lI.h?lI.h(g,d):lI.call(null,g,d),G(d)?T.j(b,e,d):wv.h(b,e)):b}return wv.h(b,e)};function mI(a){return Jv(a)?I(a):NH(lD,R(["re-frame: expected a vector, but got:",a],0))};var nI=uF.c(Kw);if(!z(fI.c?fI.c(DC):fI.call(null,DC)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var oI=null;
function pI(a){var b=mI(a),b=iI(DC,b);if(z(b))if(z(oI))NH(lD,R([[D.c('re-frame: while handling "'),D.c(oI),D.c('", dispatch-sync was called for "'),D.c(a),D.c("\". You can't call dispatch-sync within an event handler.")].join("")],0));else{var c=oI;oI=a;try{var d=px(Kw,new V(null,2,5,X,[oD,DC],null),a);var e=sx(d,b);var g=dI(e,SD),h=wv.h(g,tx),k=YC.c(g);var l=sx(h,k);dI(l,EC)}finally{oI=c}}};var qI=new y(null,2,[mC,function(a){function b(){return Wb.c?Wb.c(a):Wb.call(null,a)}return IG.c?IG.c(b):IG.call(null,b)},aA,Wb],null),rI=function rI(b,c){if(null!=b&&null!=b.Ir)return b.Ir(0,c);var d=rI[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=rI._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IEventQueue.push",b);},sI=function sI(b,c){if(null!=b&&null!=b.fo)return b.fo(0,c);var d=sI[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=sI._;
if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("IEventQueue.-call-post-event-callbacks",b);};function tI(a,b,c){this.Aj=a;this.Lb=b;this.Kv=c}function uI(a){for(var b=P(a.Lb);;){if(0===b)return vI(a,ZD,null);var c=Mw(qI,jy(Av(Bv(a.Lb))));if(z(c))return vI(a,kD,c);wI(a);--b}}tI.prototype.Ir=function(a,b){return vI(this,Yz,b)};function xI(a){a=function(a){return function(){return vI(a,UC,null)}}(a);return Wb.c?Wb.c(a):Wb.call(null,a)}
function vI(a,b,c){var d=function(){var d=new V(null,2,5,X,[a.Aj,b],null);if(M.h(new V(null,2,5,X,[yA,Yz],null),d))return new V(null,2,5,X,[eD,function(a,b){return function(){b.Lb=sv.h(b.Lb,c);return xI(b)}}(d,a)],null);if(M.h(new V(null,2,5,X,[eD,Yz],null),d))return new V(null,2,5,X,[eD,function(a,b){return function(){return b.Lb=sv.h(b.Lb,c)}}(d,a)],null);if(M.h(new V(null,2,5,X,[eD,UC],null),d))return new V(null,2,5,X,[hC,function(a,b){return function(){return uI(b)}}(d,a)],null);if(M.h(new V(null,
2,5,X,[hC,Yz],null),d))return new V(null,2,5,X,[hC,function(a,b){return function(){return b.Lb=sv.h(b.Lb,c)}}(d,a)],null);if(M.h(new V(null,2,5,X,[hC,kD],null),d))return new V(null,2,5,X,[dA,function(a,b){return function(){return yI(b,c)}}(d,a)],null);if(M.h(new V(null,2,5,X,[hC,nD],null),d))return new V(null,2,5,X,[yA,function(a,b){return function(){b.Lb=Sw;throw c;}}(d,a)],null);if(M.h(new V(null,2,5,X,[hC,ZD],null),d))return Dv(a.Lb)?new V(null,1,5,X,[yA],null):new V(null,2,5,X,[eD,function(a,
b){return function(){return xI(b)}}(d,a)],null);if(M.h(new V(null,2,5,X,[dA,Yz],null),d))return new V(null,2,5,X,[dA,function(a,b){return function(){return b.Lb=sv.h(b.Lb,c)}}(d,a)],null);if(M.h(new V(null,2,5,X,[dA,KD],null),d))return new V(null,2,5,X,[hC,function(a,b){return function(){wI(b);return uI(b)}}(d,a)],null);throw Qz([D.c("re-frame: router state transition not found. "),D.c(a.Aj),D.c(" "),D.c(b)].join(""),new y(null,2,[NA,a.Aj,FD,b],null));}();var e=S(d,0,null);d=S(d,1,null);a.Aj=e;return z(d)?
d.T?d.T():d.call(null):null}function yI(a,b){var c=function(a){return function(){return vI(a,KD,null)}}(a);return b.c?b.c(c):b.call(null,c)}function wI(a){var b=Bv(a.Lb);try{pI(b);var c=a.Lb;var d=null==c?null:Rt(c);a.Lb=d;a.fo(0,b)}catch(e){vI(a,nD,e)}}
tI.prototype.fo=function(a,b){for(var c,d=G(ky(this.Kv)),e=null,g=0,h=0;;)if(h<g)c=e.$(null,h),c.h?c.h(b,this.Lb):c.call(null,b,this.Lb),h+=1;else if(d=G(d))e=d,Kv(e)?(d=vu(e),e=wu(e),c=d,g=P(d),d=e,e=c):(c=I(e),c.h?c.h(b,this.Lb):c.call(null,b,this.Lb),d=K(e),e=null,g=0),h=0;else return null};var zI=new tI(yA,Sw,Kw);function AI(a){if(null==a)throw new Pz('re-frame: you called "dispatch" without an event vector.',Kw,null);rI(zI,a);return null}function BI(a){pI(a);sI(zI,a);return null};if(!z(fI.c?fI.c(PD):fI.call(null,PD)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");
var CI=Pw.h(jI,PD),DI=aI(R([LC,AB,EC,function(a){for(var b,c=G(NC.c(a)),d=null,e=0,g=0;;)if(g<e)a=d.$(null,g),b=S(a,0,null),a=S(a,1,null),b=iI(PD,b),z(b)&&(b.c?b.c(a):b.call(null,a)),g+=1;else if(a=G(c))c=a,Kv(c)?(d=vu(c),c=wu(c),a=d,b=P(d),d=a,e=b):(a=I(c),b=S(a,0,null),a=S(a,1,null),d=iI(PD,b),z(d)&&(d.c?d.c(a):d.call(null,a)),c=K(c),d=null,e=0),g=0;else return null}],0)),EI=function(a){return function(b){b=G(b);for(var c=null,d=0,e=0;;)if(e<d){var g=c.$(null,e),h=null!=g&&(g.C&64||x===g.Pb)?U.h(Xw,
g):g;var k=h;var l=F.h(h,ND);var m=F.h(h,rC);Dv(m)||"number"!==typeof l?NH(lD,R(["re-frame: ignoring bad :dispatch-later value:",k],0)):ZH(function(a,b,c,d,e,g,h,k,l){return function(){return AI(l)}}(b,c,d,e,g,h,k,l,m,a),l);e+=1}else if(h=G(b)){g=h;if(Kv(g))b=vu(g),l=wu(g),c=b,m=P(b),b=l,d=m;else{var p=I(g),u=null!=p&&(p.C&64||x===p.Pb)?U.h(Xw,p):p;k=u;l=F.h(u,ND);m=F.h(u,rC);Dv(m)||"number"!==typeof l?NH(lD,R(["re-frame: ignoring bad :dispatch-later value:",k],0)):ZH(function(a,b,c,d,e,g,h,k,l){return function(){return AI(l)}}(b,
c,d,e,p,u,k,l,m,g,h,a),l);b=K(g);c=null;d=0}e=0}else return null}}(iE);CI.h?CI.h(iE,EI):CI.call(null,iE,EI);function FI(a){return Jv(a)?AI(a):NH(lD,R(["re-frame: ignoring bad :dispatch value. Expected a vector, but got:",a],0))}CI.h?CI.h(rC,FI):CI.call(null,rC,FI);
function GI(a){Gv(a)||NH(lD,R(["re-frame: ignoring bad :dispatch-n value. Expected a collection, got got:",a],0));a=G(a);for(var b=null,c=0,d=0;;)if(d<c){var e=b.$(null,d);AI(e);d+=1}else if(a=G(a))b=a,Kv(b)?(a=vu(b),b=wu(b),e=a,c=P(a),a=b,b=e):(e=I(b),AI(e),a=K(b),b=null,c=0),d=0;else return null}CI.h?CI.h(MB,GI):CI.call(null,MB,GI);
function HI(a){var b=Pw.h(kI,DC);if(Gv(a)){a=G(Gv(a)?a:new V(null,1,5,X,[a],null));for(var c=null,d=0,e=0;;)if(e<d){var g=c.$(null,e);b.c?b.c(g):b.call(null,g);e+=1}else if(a=G(a))c=a,Kv(c)?(a=vu(c),c=wu(c),g=a,d=P(a),a=c,c=g):(g=I(c),b.c?b.c(g):b.call(null,g),a=K(c),c=null,d=0),e=0;else return null}else return null}CI.h?CI.h(YB,HI):CI.call(null,YB,HI);function II(a){return Zw.h?Zw.h(nI,a):Zw.call(null,nI,a)}CI.h?CI.h(WA,II):CI.call(null,WA,II);if(!z(fI.c?fI.c(HD):fI.call(null,HD)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var JI=Pw.h(jI,HD);function KI(a){return T.j(a,WA,N.c?N.c(nI):N.call(null,nI))}JI.h?JI.h(WA,KI):JI.call(null,WA,KI);var LI=function(a){return aI(R([LC,oD,SD,function(b){return rx(b,oD,iI(HD,a))}],0))}(WA);function MI(a,b){return M.h(a,b)?new V(null,3,5,X,[null,null,a],null):new V(null,3,5,X,[a,b,null],null)}function NI(a){return G(a)?nt(function(a,c){var b=S(c,0,null),e=S(c,1,null);return T.j(a,b,e)},Nx(bx(U.h(bw,jy(a)),ex(null))),a):null}
function OI(a,b,c){var d=F.h(a,c),e=F.h(b,c),g=PI.h?PI.h(d,e):PI.call(null,d,e),h=S(g,0,null),k=S(g,1,null),g=S(g,2,null);a=Tv(a,c);b=Tv(b,c);d=a&&b&&(null!=g||null==d&&null==e);return new V(null,3,5,X,[!a||null==h&&d?null:vv([c,h]),!b||null==k&&d?null:vv([c,k]),d?vv([c,g]):null],null)}
var QI=function QI(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return QI.h(arguments[0],arguments[1]);case 3:return QI.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};QI.h=function(a,b){return QI.j(a,b,aF(jy(a),jy(b)))};QI.j=function(a,b,c){return nt(function(a,b){return jz(Z.j(Uy,a,b))},new V(null,3,5,X,[null,null,null],null),Z.h(Pw.j(OI,a,b),c))};QI.I=3;
function RI(a,b){return Nx(Z.h(NI,QI.j(Jv(a)?a:Nx(a),Jv(b)?b:Nx(b),new iz(null,0,function(){var c=P(a),d=P(b);return c>d?c:d}(),1,null))))}function SI(a,b){return new V(null,3,5,X,[Hw(cF.h(a,b)),Hw(cF.h(b,a)),Hw(bF.h(a,b))],null)}
var TI=function TI(b){if(null!=b&&null!=b.qu)return b.qu(b);var c=TI[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=TI._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw it("EqualityPartition.equality-partition",b);},UI=function UI(b,c){if(null!=b&&null!=b.pu)return b.pu(b,c);var d=UI[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=UI._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw it("Diff.diff-similar",b);};TI["null"]=function(){return ED};
TI.string=function(){return ED};TI.number=function(){return ED};TI.array=function(){return eB};TI["function"]=function(){return ED};TI["boolean"]=function(){return ED};TI._=function(a){return(null!=a?a.C&1024||x===a.Wp||(a.C?0:C(Jt,a)):C(Jt,a))?YD:(null!=a?a.C&4096||x===a.nu||(a.C?0:C(Ot,a)):C(Ot,a))?wD:(null!=a?a.C&16777216||x===a.mu||(a.C?0:C(gu,a)):C(gu,a))?eB:ED};UI["null"]=function(a,b){return MI(a,b)};UI.string=function(a,b){return MI(a,b)};UI.number=function(a,b){return MI(a,b)};
UI.array=function(a,b){return RI(a,b)};UI["function"]=function(a,b){return MI(a,b)};UI["boolean"]=function(a,b){return MI(a,b)};UI._=function(a,b){return function(){var b=TI(a),b=b instanceof Y?b.Sb:null;switch(b){case "atom":return MI;case "set":return SI;case "sequential":return RI;case "map":return QI;default:throw Error([D.c("No matching clause: "),D.c(b)].join(""));}}().call(null,a,b)};function PI(a,b){return M.h(a,b)?new V(null,3,5,X,[null,null,a],null):M.h(TI(a),TI(b))?UI(a,b):MI(a,b)};aI(R([LC,QB,SD,function(a){NH(UD,R(["Handling re-frame event:",cI(a,DC)],0));return a},EC,function(a){var b=cI(a,DC),c=cI(a,WA);var d=ox(a,new V(null,2,5,X,[NC,WA],null),$A);M.h(d,$A)?NH(UD,R(["No :db changes caused by:",b],0)):(d=PI(c,d),c=S(d,0,null),d=S(d,1,null),null!=c||null!=d?(NH(DA,R(["db clojure.data/diff for:",b],0)),NH(UD,R(["only before:",c],0)),NH(UD,R(["only after :",d],0)),MH(DD)):NH(UD,R(["no app-db changes caused by:",b],0)));return a}],0));
var VI=aI(R([LC,pC,SD,function(a){return px(qx.L(a,new V(null,2,5,X,[oD,DC],null),Qx,1),new V(null,2,5,X,[oD,WD],null),cI(a,DC))},EC,function(a){return px(lI(a,new V(null,2,5,X,[oD,WD],null)),new V(null,2,5,X,[oD,DC],null),cI(a,WD))}],0));function WI(a){return aI(R([LC,AC,SD,function(b){var c=oD.c(b),d=null!=c&&(c.C&64||x===c.Pb)?U.h(Xw,c):c,c=F.h(d,WA),d=F.h(d,DC);return bI(b,WA,a.h?a.h(c,d):a.call(null,c,d))}],0))}
function XI(a){return aI(R([LC,OD,EC,function(b){var c=cI(b,DC);var d=nx(b,new V(null,2,5,X,[NC,WA],null));d=z(d)?d:cI(b,WA);return bI(b,WA,a.h?a.h(d,c):a.call(null,d,c))}],0))}function YI(a){return aI(R([LC,EC,EC,function(b){var c=nx(b,new V(null,2,5,X,[NC,WA],null));c=z(c)?c:cI(b,WA);var d=cI(b,DC);a.h?a.h(c,d):a.call(null,c,d);return b}],0))};if(!z(fI.c?fI.c(XA):fI.call(null,XA)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var ZI,$I=Kw;ZI=Ww?Ww($I):Vw.call(null,$I);function aJ(a,b,c){a=new V(null,2,5,X,[a,b],null);YH(c,function(a){return function(){$w.j(ZI,wv,a);return null}}(a));$w.L(ZI,T,a,c);return c}function bJ(a,b){return F.h(N.c?N.c(ZI):N.call(null,ZI),new V(null,2,5,X,[a,b],null))}
var cJ=function cJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return cJ.c(arguments[0]);case 2:return cJ.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};
cJ.c=function(a){var b=bJ(a,tv);if(z(b))return b;var c=mI(a);b=iI(XA,c);return null==b?NH(lD,R([[D.c('re-frame: no subscription handler registered for: "'),D.c(c),D.c('". Returning a nil subscription.')].join("")],0)):aJ(a,tv,b.h?b.h(nI,a):b.call(null,nI,a))};
cJ.h=function(a,b){var c=bJ(a,b);if(z(c))return c;var d=mI(a);var e=iI(XA,d);if(null==e)return NH(lD,R([[D.c('re-frame: no subscription handler registered for: "'),D.c(d),D.c('". Returning a nil subscription.')].join("")],0));var g=wF(function(){return function(){return kx(N,b)}}(d,e,c));var h=wF(function(b,c,d){return function(){var c=N.c?N.c(b):N.call(null,b);return d.j?d.j(nI,a,c):d.call(null,nI,a,c)}}(g,d,e,c));return aJ(a,b,wF(function(a,b){return function(){var a=N.c?N.c(b):N.call(null,b);return N.c?
N.c(a):N.call(null,a)}}(g,h,d,e,c)))};cJ.I=2;function dJ(a){return Rw.j(null==a?null:tt(a),Z.c(function(a){var b=S(a,0,null);a=S(a,1,null);return new V(null,2,5,X,[b,N.c?N.c(a):N.call(null,a)],null)}),a)}function eJ(a,b){if(Gv(a))var c=Z.h(N,a);else Hv(a)?c=dJ(a):(c=null!=a?a.C&32768||x===a.Tp?!0:a.C?!1:C(Ut,a):C(Ut,a),c=z(c)?N.c?N.c(a):N.call(null,a):NH(lD,R(["re-frame: in the reg-sub for ",b,", the input-signals function returns: ",a],0)));return c}
function fJ(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;return gJ(arguments[0],1<b.length?new H(b.slice(1),0,null):null)}
function gJ(a,b){var c=rv(b),d=fz(b),e=[D.c("re-frame: reg-sub for "),D.c(a),D.c(", ")].join(""),g=function(){var a=P(d);switch(a){case 0:return function(){return function(){var a=null,a=function(a,b){switch(arguments.length){case 1:return nI;case 2:return nI}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(){return nI};a.h=function(){return nI};return a}()}(a,c,d,e);case 1:return a=I(d),xv(a)||NH(lD,R([e,"2nd argument expected to be an inputs function, got:",a],0)),a;case 2:var b=
S(d,0,null),g=S(d,1,null);M.h(TA,b)||NH(lD,R([e,"expected :\x3c-, got:",b],0));return function(a,b,c){return function(){function a(){return cJ.c(c)}function b(){return cJ.c(c)}var d=null,d=function(c,d){switch(arguments.length){case 1:return b.call(this);case 2:return a.call(this)}throw Error("Invalid arity: "+(arguments.length-1));};d.c=b;d.h=a;return d}()}(d,b,g,a,c,d,e);default:var b=lx(2,2,d),g=Z.h(I,b),m=Z.h(rv,b);Lw(new az(null,new y(null,1,[TA,null],null),null),g)&&Lw(Jv,m)||NH(lD,R([e,"expected pairs of :\x3c- and vectors, got:",
b],0));return function(a,b,c){return function(){function a(){return Z.h(cJ,c)}function b(){return Z.h(cJ,c)}var d=null,d=function(c,d){switch(arguments.length){case 1:return b.call(this);case 2:return a.call(this)}throw Error("Invalid arity: "+(arguments.length-1));};d.c=b;d.h=a;return d}()}(b,g,m,a,c,d,e)}}();return jI(XA,a,function(b,c,d,e){return function(){function g(g,h,k){var l=e.h?e.h(h,k):e.call(null,h,k);g=Ww?Ww(null):Vw.call(null,null);var l=wF(function(b,c,d){return function(){var c=eJ(b,
a);return d.j?d.j(c,h,k):d.call(null,c,h,k)}}(l,g,b,c,d,e)),m=$H(l);Zw.h?Zw.h(g,m):Zw.call(null,g,m);return l}function h(g,h){var k=e.c?e.c(h):e.call(null,h),l=Ww?Ww(null):Vw.call(null,null),k=wF(function(b,c,d){return function(){var c=eJ(b,a);return d.h?d.h(c,h):d.call(null,c,h)}}(k,l,b,c,d,e)),m=$H(k);Zw.h?Zw.h(l,m):Zw.call(null,l,m);return k}var k=null,k=function(a,b,c){switch(arguments.length){case 2:return h.call(this,0,b);case 3:return g.call(this,0,b,c)}throw Error("Invalid arity: "+(arguments.length-
1));};k.h=h;k.j=g;return k}()}(c,d,e,g))};Pw.h(kI,XA);Pw.h(kI,PD);Pw.h(kI,HD);Pw.h(kI,DC);var hJ=function hJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return hJ.h(arguments[0],arguments[1]);case 3:return hJ.j(arguments[0],arguments[1],arguments[2]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};hJ.h=function(a,b){return hJ.j(a,null,b)};hJ.j=function(a,b,c){b=new V(null,4,5,X,[LI,DI,b,WI(c)],null);return jI(DC,a,hx(ft,jx(b)))};hJ.I=3;function iJ(){var a=I(hx(AE,Z.h(function(a){return Sh(a,"initial")},hx(ft,new V(null,2,5,X,[document.querySelector("body"),document.querySelector("#wrapper")],null)))));z(a)&&(a=new V(null,2,5,X,[ZB,Mz(qd(a),R([Nz,!0],0))],null),BI.c?BI.c(a):BI.call(null,a));XH();a=new V(null,1,5,X,[eC],null);BI.c?BI.c(a):BI.call(null,a);a=new V(null,1,5,X,[uA],null);return BI.c?BI.c(a):BI.call(null,a)};function jJ(a){return Jv(a)&&M.h(2,P(a))&&Gw.h(Mt(a),eA)}function kJ(a,b,c){return px(a,new V(null,3,5,X,[qD,b,OC],null),c)}function lJ(a,b){return Wv(function(a,b,e){return px(a,new V(null,3,5,X,[wB,b,sB],null),e)},a,b)};function mJ(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null),h=S(b,2,null);return qx.j(a,new V(null,3,5,X,[qD,h,fA],null),function(a,b,c,d){return function(){var a=I(hx(ft,new V(null,3,5,X,[Rz.c(d),d,Kw],null)));return Rw.h(Kw,gx(jJ,Dw.h(nx(a,new V(null,2,5,X,[zC,lD],null)),nx(a,new V(null,2,5,X,[zC,fA],null)))))}}(b,c,d,e,g,h))}XI.c?XI.c(mJ):XI.call(null,mJ);function nJ(a,b){S(b,0,null);var c=S(b,1,null);return kJ(a,c,!0)}XI.c?XI.c(nJ):XI.call(null,nJ);
function oJ(a,b){S(b,0,null);var c=S(b,1,null);S(c,0,null);S(c,1,null);c=S(b,2,null);return kJ(a,c,!1)}XI.c?XI.c(oJ):XI.call(null,oJ);function pJ(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null),h=S(b,2,null);return setTimeout(function(a,b,c,d,e,g){return function(){var a=new V(null,2,5,X,[rB,g],null);return AI.c?AI.c(a):AI.call(null,a)}}(b,c,d,e,g,h),15E3)}YI.c?YI.c(pJ):YI.call(null,pJ);
function qJ(a,b){S(b,0,null);var c=S(b,1,null);S(c,0,null);S(c,1,null);c=S(b,2,null);return null==c?a:px(a,new V(null,3,5,X,[qD,c,fA],null),null)}XI.c?XI.c(qJ):XI.call(null,qJ);function rJ(a,b){S(b,0,null);var c=S(b,1,null),d=S(b,2,null);return px(a,new V(null,2,5,X,[qD,LC.c(c)],null),lJ(c,d))}var sJ=XI.c?XI.c(rJ):XI.call(null,rJ);hJ.j(rB,VI,function(a,b){var c=S(b,0,null);return kJ(a,c,!1)});hJ.j(WC,new V(null,1,5,X,[sJ],null),function(a,b){S(b,0,null);S(b,1,null);S(b,2,null);return a});function tJ(a){return nx(a,new V(null,2,5,X,[GB,GB],null))}fJ.h?fJ.h(ZC,tJ):fJ.call(null,ZC,tJ);function uJ(a){return nx(a,new V(null,2,5,X,[GB,EB],null))}fJ.h?fJ.h(xB,uJ):fJ.call(null,xB,uJ);function vJ(a){return ox(a,new V(null,2,5,X,[BB,KA],null),0)}fJ.h?fJ.h(CD,vJ):fJ.call(null,CD,vJ);hJ.h(eC,function(a){return px(a,new V(null,2,5,X,[BB,Xz],null),(new Ur(document)).get(rw(ox(a,new V(null,2,5,X,[BB,QD],null),AD))))});var wJ=new y(null,1,[zC,tv],null);function xJ(a,b,c,d,e,g,h){this.id=a;this.text=b;this.severity=c;this.timeout=d;this.Ma=e;this.ea=g;this.K=h;this.C=2229667594;this.Y=8192}f=xJ.prototype;f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){switch(b instanceof Y?b.Sb:null){case "id":return this.id;case "text":return this.text;case "severity":return this.severity;case "timeout":return this.timeout;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return nz(b,function(){return function(a){return nz(b,vz,""," ","",c,a)}}(this),"#common.flash.db.FlashMessage{",", ","}",c,Dw.h(new V(null,4,5,X,[new V(null,2,5,X,[LC,this.id],null),new V(null,2,5,X,[oE,this.text],null),new V(null,2,5,X,[VC,this.severity],null),new V(null,2,5,X,[yD,this.timeout],null)],null),this.ea))};f.ic=function(){return new dy(0,this,4,new V(null,4,5,X,[LC,oE,VC,yD],null),z(this.ea)?zu(this.ea):Iw())};f.sa=function(){return this.Ma};
f.xa=function(){return 4+P(this.ea)};f.ha=function(){var a=this.K;return null!=a?a:this.K=a=gw(this)};f.O=function(a,b){return z(z(b)?this.constructor===b.constructor&&cy(this,b):b)?!0:!1};f.lf=function(a,b){return Tv(new az(null,new y(null,4,[LC,null,VC,null,yD,null,oE,null],null),null),b)?wv.h(Zt(Rw.h(Kw,this),this.Ma),b):new xJ(this.id,this.text,this.severity,this.timeout,this.Ma,Hw(wv.h(this.ea,b)),null)};
f.Ob=function(a,b,c){return z(pw.h?pw.h(LC,b):pw.call(null,LC,b))?new xJ(c,this.text,this.severity,this.timeout,this.Ma,this.ea,null):z(pw.h?pw.h(oE,b):pw.call(null,oE,b))?new xJ(this.id,c,this.severity,this.timeout,this.Ma,this.ea,null):z(pw.h?pw.h(VC,b):pw.call(null,VC,b))?new xJ(this.id,this.text,c,this.timeout,this.Ma,this.ea,null):z(pw.h?pw.h(yD,b):pw.call(null,yD,b))?new xJ(this.id,this.text,this.severity,c,this.Ma,this.ea,null):new xJ(this.id,this.text,this.severity,this.timeout,this.Ma,T.j(this.ea,
b,c),null)};f.oa=function(){return G(Dw.h(new V(null,4,5,X,[new V(null,2,5,X,[LC,this.id],null),new V(null,2,5,X,[oE,this.text],null),new V(null,2,5,X,[VC,this.severity],null),new V(null,2,5,X,[yD,this.timeout],null)],null),this.ea))};f.ya=function(a,b){return new xJ(this.id,this.text,this.severity,this.timeout,b,this.ea,this.K)};f.wa=function(a,b){return Jv(b)?this.Ob(null,xt.h(b,0),xt.h(b,1)):nt(vt,this,b)};
function yJ(a,b,c){return new xJ(Date.now(),b,F.j(new y(null,1,[fA,lD],null),a,a),c,null,null,null)}function zJ(a,b){return yJ(a,b,M.h(a,lD)?0:3E3)}function AJ(a,b,c,d){return qx.j(a,new V(null,2,5,X,[DB,zC],null),function(a){return sv.h(a,yJ(b,c,d))})}function BJ(a,b,c){return qx.j(a,new V(null,2,5,X,[DB,zC],null),function(a){return sv.h(a,zJ(b,c))})}function CJ(a){return nt(function(a,c){return sv.h(a,zJ(lD,c))},tv,nx(a,new V(null,3,5,X,[zC,lD,eA],null)))}
function DJ(a){var b=S(a,0,null),c=S(a,1,null);return"string"===typeof c?new V(null,1,5,X,[zJ(qw.c(b),c)],null):Gv(c)?Z.h(function(a,b){return function(a){return zJ(qw.c(b),a)}}(a,b,c),c):tv}function EJ(a){a=lD.c(a);return"string"===typeof a?new V(null,1,5,X,[zJ(lD,a)],null):tv}function FJ(a,b){return Dw.w(b,fx(Xv,R([Z.h(DJ,F.j(a,zC,Kw))],0)),R([CJ(a),EJ(a)],0))};function GJ(a){return qx.j(a,new V(null,2,5,X,[BB,KA],null),Yu)}XI.c?XI.c(GJ):XI.call(null,GJ);function HJ(a){return qx.j(a,new V(null,2,5,X,[BB,KA],null),function(a){return 0<a?a-1:0})}var IJ=XI.c?XI.c(HJ):XI.call(null,HJ);var JJ=uF.c(tv);hJ.h(uA,function(a){return nt(function(a,c){return c.c?c.c(a):c.call(null,a)},a,N.c?N.c(JJ):N.call(null,JJ))});hJ.j($z,new V(null,1,5,X,[IJ],null),function(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null);return qx.j(a,new V(null,2,5,X,[DB,zC],null),function(a,b,c,d){return function(a){return FJ(I(hx(ft,new V(null,3,5,X,[Rz.c(d),d,Kw],null))),a)}}(b,c,d,e,g))});
hJ.h(bB,function(a,b){S(b,0,null);var c=S(b,1,null),d=S(c,0,null);S(c,1,null);c=S(c,2,null);return M.h(-1,c)?BJ(a,lD,[D.c("Zdá se, že jste off-line. Prosíme, připojte se k Internetu a zkuste"),D.c(" to znovu")].join("")):M.h(401,c)?BJ(a,lD,"Vaše přihlášení vypršelo, prosíme, aktualizujte stránku."):M.h(404,c)?window.location.reload():M.h(429,c)?BJ(a,lD,ox(d,new V(null,2,5,X,[zC,lD],null),"Příliš mnoho požadavků.")):M.h(502,c)||M.h(503,c)?(d=new V(null,2,5,X,[qE,new V(null,2,5,X,["error","not-available"],
null)],null),AI.c?AI.c(d):AI.call(null,d)):400<=c?BJ(a,lD,[D.c("Objevila se interní chyba. Prosíme, aktualizujte stránku a zkuste"),D.c(" to znovu")].join("")):a});function KJ(a){return ox(a,new V(null,2,5,X,[DB,zC],null),tv)}fJ.h?fJ.h(lA,KJ):fJ.call(null,lA,KJ);function LJ(a,b){return xE(" ",Rw.h(new V(null,1,5,X,[a],null),Z.h(function(b){return[D.c(a),D.c("--"),D.c(b)].join("")},b)))};function MJ(a){return function(){var b=new V(null,2,5,X,[aD,a],null);return AI.c?AI.c(b):AI.call(null,b)}}var NJ=function NJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return NJ.w(arguments[0],arguments[1],2<c.length?new H(c.slice(2),0,null):null)};
NJ.w=function(a,b,c){return Rw.h(new V(null,2,5,X,[oC,new y(null,1,[MC,LJ([D.c(b),D.c("__msg")].join(""),new V(null,1,5,X,[rw(VC.c(a))],null))],null)],null),Dw.h(new V(null,1,5,X,[new V(null,3,5,X,[oC,new y(null,1,[MC,[D.c(b),D.c("__content")].join("")],null),oE.c(a)],null)],null),c))};NJ.I=2;NJ.N=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return NJ.w(b,a,c)};
function OJ(a,b){return ZF(new y(null,3,[cD,"close-button",vB,function(){if(0<yD.c(a)){var b=MJ(LC.c(a)),d=yD.c(a);return setTimeout(b,d)}return null},FC,function(a){return new V(null,3,5,X,[oC,new y(null,2,[MC,[D.c(b),D.c("__close")].join(""),fC,MJ(LC.c(a))],null),Pa("\x26times;")],null)}],null))}function PJ(a,b){return new V(null,4,5,X,[NJ,a,b,new V(null,3,5,X,[OJ,a,b],null)],null)}
function QJ(a){return new V(null,3,5,X,[oC,new y(null,1,[MC,LJ([D.c(a),D.c("__msg")].join(""),new V(null,1,5,X,["loading"],null))],null),new V(null,3,5,X,[oC,new y(null,1,[MC,[D.c(a),D.c("__content")].join("")],null),"Načítám..."],null)],null)}
function RJ(a,b,c){return new V(null,4,5,X,[oC,new y(null,1,[MC,c],null),0<b?new V(null,2,5,X,[QJ,c],null):null,G(a)?function(){return function e(a){return new sw(null,function(){for(;;){var b=G(a);if(b){if(Kv(b)){var g=vu(b),l=P(g),m=ww(l);a:for(var p=0;;)if(p<l){var u=xt.h(g,p),u=zv(new V(null,3,5,X,[PJ,u,c],null),new y(null,1,[MA,LC.c(u)],null));m.add(u);p+=1}else{g=!0;break a}return g?yw(m.ob(),e(wu(b))):yw(m.ob(),null)}m=I(b);return lv(zv(new V(null,3,5,X,[PJ,m,c],null),new y(null,1,[MA,LC.c(m)],
null)),e(Pu(b)))}return null}},null,null)}(a)}():null],null)};function SJ(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null);return qx.j(a,new V(null,2,5,X,[DB,zC],null),function(a,b,c,d){return function(a){return FJ(I(hx(ft,new V(null,3,5,X,[Rz.c(d),d,Kw],null))),a)}}(b,c,d,e,g))}XI.c?XI.c(SJ):XI.call(null,SJ);function TJ(a){return px(a,new V(null,2,5,X,[DB,zC],null),tv)}XI.c?XI.c(TJ):XI.call(null,TJ);
function UJ(a,b){var c=S(b,0,null),d=S(b,1,null);return qx.j(a,new V(null,2,5,X,[DB,zC],null),function(a,b,c){return function(d){return hx(function(a,b,c){return function(a){return M.h(LC.c(a),c)}}(a,b,c),d)}}(b,c,d))}var VJ=XI.c?XI.c(UJ):XI.call(null,UJ);function WJ(a,b){var c=S(b,0,null),d=S(b,1,null);return nt(function(){return function(a,b){return BJ(a,qw.c(function(){var a=VC.c(b);return z(a)?a:XC}()),oE.c(b))}}(b,c,d),a,d)}var XJ=XI.c?XI.c(WJ):XI.call(null,WJ);hJ.j(aD,new V(null,1,5,X,[VJ],null),function(a,b){S(b,0,null);S(b,1,null);return a});hJ.h(kC,function(a,b){S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null);return AJ(a,c,d,e)});hJ.j(jE,XJ,function(a,b){S(b,0,null);S(b,1,null);return a});$w.j(OH,sv,new V(null,2,5,X,["#sticky-flash-cljs",new V(null,2,5,X,[function(a,b){var c=function(){var a=new V(null,1,5,X,[CD],null);return cJ.c?cJ.c(a):cJ.call(null,a)}(),d=function(){var a=new V(null,1,5,X,[lA],null);return cJ.c?cJ.c(a):cJ.call(null,a)}(),e=QH(b);return ZF(new y(null,3,[cD,"flash-component",vB,function(a,b,c){return function(){if(G(c)){var a=new V(null,2,5,X,[jE,c],null);return AI.c?AI.c(a):AI.call(null,a)}return null}}(c,d,e),FC,function(a,b){return function(){return new V(null,
4,5,X,[RJ,N.c?N.c(b):N.call(null,b),N.c?N.c(a):N.call(null,a),"common-flash"],null)}}(c,d,e)],null))},new V(null,1,5,X,[new y(null,3,[kB,JB,fE,!1,OB,cC],null)],null)],null)],null));var YJ=null!=window.ga;function ZJ(a){var b=a.className,c=[D.c(window.location.pathname),D.c(window.location.hash)].join(""),d=Sh(a,"analytics");a=Ca(d)?Rf(a):xE("_",new V(null,2,5,X,[d,Rf(a)],null));return new y(null,4,[iC,"event",CA,b,dB,c,ZA,a],null)}function $J(a){a=a.currentTarget;a=Uy.w(R([new y(null,1,[dB,a.href],null),ZJ(a)],0));return z(YJ)?U.h(window.ga,R(["send",a],0)):NH(UD,R(["GA event: ",a],0))};function aK(a,b){S(b,0,null);var c=S(b,1,null);return v(c,"click",$J)}var bK=YI.c?YI.c(aK):YI.call(null,aK);hJ.j(ID,bK,function(a,b){S(b,0,null);S(b,1,null);return a});RH("a",new V(null,1,5,X,[function(a){return v(a,"click",$J)}],null));RH(".collapsible",new V(null,1,5,X,[function(a){return v(a,"click",function(){return function(){return eh(a,"expanded")}}(a,"click"))}],null));RH(".collapsible button",new V(null,1,5,X,[function(a){return a.disabled=!1}],null));if(M.h("default","nodejs")){var cK=require("@pupeno/xmlhttprequest").XMLHttpRequest;global.XMLHttpRequest=cK};var dK="undefined"!=typeof Object.keys?function(a){return Object.keys(a)}:function(a){return Ib(a)},eK="undefined"!=typeof Array.isArray?function(a){return Array.isArray(a)}:function(a){return"array"===n(a)};function fK(){Math.round(15*Math.random()).toString(16)};var gK=1;function hK(a,b){if(null==a)return null==b;if(a===b)return!0;if("object"===typeof a){if(eK(a)){if(eK(b)&&a.length===b.length){for(var c=0;c<a.length;c++)if(!hK(a[c],b[c]))return!1;return!0}return!1}if(a.Ah)return a.Ah(b);if(null!=b&&"object"===typeof b){if(b.Ah)return b.Ah(a);var c=0,d=dK(b).length,e;for(e in a)if(a.hasOwnProperty(e)&&(c++,!b.hasOwnProperty(e)||!hK(a[e],b[e])))return!1;return c===d}}return!1}function iK(a,b){return a^b+2654435769+(a<<6)+(a>>2)}var jK={},kK=0;
function lK(a){var b=0;if(null!=a.forEach)a.forEach(function(a,c){b=(b+(mK(c)^mK(a)))%4503599627370496});else for(var c=dK(a),d=0;d<c.length;d++)var e=c[d],g=a[e],b=(b+(mK(e)^mK(g)))%4503599627370496;return b}function nK(a){var b=0;if(eK(a))for(var c=0;c<a.length;c++)b=iK(b,mK(a[c]));else a.forEach&&a.forEach(function(a){b=iK(b,mK(a))});return b}
function mK(a){if(null==a)return 0;switch(typeof a){case "number":return a;case "boolean":return!0===a?1:0;case "string":var b=jK[a];if(null==b){for(var c=b=0;c<a.length;++c)b=31*b+a.charCodeAt(c),b%=4294967296;kK++;256<=kK&&(jK={},kK=1);jK[a]=b}a=b;return a;case "function":return b=a.transit$hashCode$,b||(b=gK,"undefined"!=typeof Object.defineProperty?Object.defineProperty(a,"transit$hashCode$",{value:b,enumerable:!1}):a.transit$hashCode$=b,gK++),b;default:return a instanceof Date?a.valueOf():eK(a)?
nK(a):a.Vl?a.Vl():lK(a)}};Qs("9007199254740991");Qs("-9007199254740991");Fs.prototype.equiv=function(a){return hK(this,a)};Fs.prototype.equiv=Fs.prototype.equiv;Fs.prototype.Ah=function(a){return a instanceof Fs&&this.pb(a)};Fs.prototype.Vl=function(){return this.Ei()};Date.prototype.Ah=function(a){return a instanceof Date?this.valueOf()===a.valueOf():!1};Date.prototype.Vl=function(){return this.valueOf()};(8|3&Math.round(14*Math.random())).toString(16);fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();fK();Oz.prototype.O=function(a,b){return b instanceof Oz?this.ef===b.ef:!1};Fs.prototype.O=function(a,b){return this.equiv(b)};Fs.prototype.gu=x;Fs.prototype.ha=function(){return mK(this)};var oK=tv;Ww||Vw.call(null,oK);function pK(a){return Bz(R(["CLJS-AJAX response:",a],0))}Ww||Vw.call(null,pK);function qK(a){return"undefined"!==typeof console?console.error(a):"undefined"!==typeof window?window.alert(""+D.c(a)):Bz(R(["CLJS-AJAX ERROR:",a],0))}Ww||Vw.call(null,qK);uF.c(null);uF.c(null);function rK(a,b,c,d,e,g){this.value=a;this.left=b;this.right=c;this.Ma=d;this.ea=e;this.K=g;this.C=2229667594;this.Y=8192}f=rK.prototype;f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){switch(b instanceof Y?b.Sb:null){case "value":return this.value;case "left":return this.left;case "right":return this.right;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return nz(b,function(){return function(a){return nz(b,vz,""," ","",c,a)}}(this),"#linked.map.Node{",", ","}",c,Dw.h(new V(null,3,5,X,[new V(null,2,5,X,[sB,this.value],null),new V(null,2,5,X,[mE,this.left],null),new V(null,2,5,X,[dD,this.right],null)],null),this.ea))};f.ic=function(){return new dy(0,this,3,new V(null,3,5,X,[sB,mE,dD],null),z(this.ea)?zu(this.ea):Iw())};f.sa=function(){return this.Ma};f.xa=function(){return 3+P(this.ea)};
f.ha=function(){var a=this.K;return null!=a?a:this.K=a=gw(this)};f.O=function(a,b){return z(z(b)?this.constructor===b.constructor&&cy(this,b):b)?!0:!1};f.lf=function(a,b){return Tv(new az(null,new y(null,3,[sB,null,dD,null,mE,null],null),null),b)?wv.h(Zt(Rw.h(Kw,this),this.Ma),b):new rK(this.value,this.left,this.right,this.Ma,Hw(wv.h(this.ea,b)),null)};
f.Ob=function(a,b,c){return z(pw.h?pw.h(sB,b):pw.call(null,sB,b))?new rK(c,this.left,this.right,this.Ma,this.ea,null):z(pw.h?pw.h(mE,b):pw.call(null,mE,b))?new rK(this.value,c,this.right,this.Ma,this.ea,null):z(pw.h?pw.h(dD,b):pw.call(null,dD,b))?new rK(this.value,this.left,c,this.Ma,this.ea,null):new rK(this.value,this.left,this.right,this.Ma,T.j(this.ea,b,c),null)};
f.oa=function(){return G(Dw.h(new V(null,3,5,X,[new V(null,2,5,X,[sB,this.value],null),new V(null,2,5,X,[mE,this.left],null),new V(null,2,5,X,[dD,this.right],null)],null),this.ea))};f.ya=function(a,b){return new rK(this.value,this.left,this.right,b,this.ea,this.K)};f.wa=function(a,b){return Jv(b)?this.Ob(null,xt.h(b,0),xt.h(b,1)):nt(vt,this,b)};function sK(a,b){this.head=a;this.rd=b;this.C=2314602255;this.Y=8192}f=sK.prototype;
f.toString=function(){var a=this;return[D.c("{"),D.c(xE(", ",function(){return function(a){return function d(b){return new sw(null,function(){return function(){for(;;){var a=G(b);if(a){if(Kv(a)){var e=vu(a),k=P(e),l=ww(k);a:for(var m=0;;)if(m<k){var p=xt.h(e,m),u=S(p,0,null),p=S(p,1,null),u=[D.c(u),D.c(" "),D.c(p)].join("");l.add(u);m+=1}else{e=!0;break a}return e?yw(l.ob(),d(wu(a))):yw(l.ob(),null)}e=I(a);l=S(e,0,null);e=S(e,1,null);return lv([D.c(l),D.c(" "),D.c(e)].join(""),d(Pu(a)))}return null}}}(a),
null,null)}}(a)(a)}())),D.c("}")].join("")};f.equiv=function(a){return this.O(null,a)};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){a=Uv(this.rd,b);return z(a)?sB.c(Nt(a)):c};f.lg=function(a,b,c){return nt(function(){return function(a,c){return U.h(Pw.h(b,a),c)}}(this),c,G(this))};f.ja=function(a,b){return lu(b,[D.c("#linked/map "),D.c(Rw.h(tv,this))].join(""))};f.sa=function(){return Av(this.rd)};f.xa=function(){return P(this.rd)};
f.mg=function(){return tK.c?tK.c(this):tK.call(null,this)};f.ha=function(){return Ku(Rw.h(Kw,this))};f.O=function(a,b){return cy(this,b)};f.Ja=function(){return Zt(uK,Av(this.rd))};f.lf=function(a,b){return vK.h?vK.h(this,b):vK.call(null,this,b)};f.Ob=function(a,b,c){return wK.j?wK.j(this,b,c):wK.call(null,this,b,c)};f.sh=function(a,b){return Tv(this.rd,b)};f.oa=function(){return xK.c?xK.c(this):xK.call(null,this)};f.ya=function(a,b){return new sK(this.head,zv(this.rd,b))};
f.wa=function(a,b){if(Jv(b))return this.Ob(null,xt.h(b,0),xt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(Jv(e))c=c.Ob(null,xt.h(e,0),xt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};
function wK(a,b,c){var d=a.head;a=a.rd;var e=Uv(a,b);if(z(e))return new sK(d,px(a,new V(null,2,5,X,[b,sB],null),c));if(Dv(a))return new sK(b,T.j(a,b,new rK(c,b,b,null,null,null)));e=nx(a,new V(null,2,5,X,[d,mE],null));return new sK(d,px(px(T.j(a,b,new rK(c,e,d,null,null,null)),new V(null,2,5,X,[d,mE],null),b),new V(null,2,5,X,[e,dD],null),b))}
function vK(a,b){var c=a.head,d=a.rd,e=Uv(d,b);if(z(e)){if(M.h(1,P(d)))return null==a?null:tt(a);var g=dD.c(Nt(e)),e=mE.c(Nt(e)),c=M.h(b,c)?g:c;return new sK(c,px(px(wv.h(d,b),new V(null,2,5,X,[g,mE],null),e),new V(null,2,5,X,[e,dD],null),g))}return a}function yK(a,b){return new V(null,2,5,X,[a,b],null)}
var zK=function zK(b,c,d,e){var g=Uv(b,c),h=S(g,0,null),k=S(g,1,null),l=yK(h,sB.c(k)),m=e.c?e.c(k):e.call(null,k);return M.h(c,d)?vt(Qu,l):lv(l,new sw(null,function(c,g,h,k,l){return function(){return zK.L?zK.L(b,l,d,e):zK.call(null,b,l,d,e)}}(g,h,k,l,m),null,null))};function xK(a){var b=a.rd;a=a.head;var c=nx(b,new V(null,2,5,X,[a,mE],null));return G(b)?zK(b,a,c,dD):null}function tK(a){var b=a.rd;a=a.head;var c=nx(b,new V(null,2,5,X,[a,mE],null));return G(b)?zK(b,c,a,mE):null}
var uK=new sK(null,ny),AK=Pw.h(Rw,uK);EH("linked/map",AK);function BK(a){this.Se=a;this.C=2313556239;this.Y=8192}f=BK.prototype;f.toString=function(){return[D.c("["),D.c(xE(" ",Z.h(D,this))),D.c("]")].join("")};f.equiv=function(a){return this.O(null,a)};f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){return Ft(this.Se,b)?b:c};f.ja=function(a,b){return lu(b,[D.c("#linked/set "),D.c(Rw.h(tv,this))].join(""))};f.sa=function(){return Av(this.Se)};f.xa=function(){return st(this.Se)};f.mg=function(){var a=ku(this.Se);return a?Z.h(hw,a):null};
f.ha=function(){return Ku(Rw.h(cz,this))};f.O=function(a,b){return Fv(b)&&P(this)===P(b)&&Lw(function(a){return function(b){return Tv(a,b)}}(this),b)};f.Ja=function(){return zv(CK,Av)};f.Ul=function(a,b){return new BK(Kt(this.Se,b))};f.oa=function(){var a=G(this.Se);return a?Z.h(hw,a):null};f.ya=function(a,b){return new BK(zv(this.Se,b))};f.wa=function(a,b){return new BK(T.j(this.Se,b,null))};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.X(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.X(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(lt(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.X(null,a,b)};var CK=new BK(uK),DK=Pw.h(Rw,CK);EH("linked/set",DK);var EK=new y(null,4,[QD,"Authorization",KA,0,iD,null,Xz,null],null);hJ.h(ZB,function(a,b){S(b,0,null);var c=S(b,1,null);return Uy.w(R([new y(null,2,[DB,wJ,BB,EK],null),c],0))});function FK(a,b,c,d,e,g,h){a=ea(a)?Date.UTC(a,b||0,c||1,d||0,e||0,g||0,h||0):a?a.getTime():wa();this.date=new Date(a)}r(FK,hj);f=FK.prototype;f.clone=function(){var a=new FK(this.date);a.Yc=this.Yc;a.xf=this.xf;return a};f.add=function(a){(a.years||a.months)&&cj.prototype.add.call(this,new Wi(a.years,a.months));this.date=new Date(this.date.getTime()+1E3*(a.seconds+60*(a.minutes+60*(a.hours+24*a.days))))};f.getTimezoneOffset=function(){return 0};f.getFullYear=hj.prototype.getUTCFullYear;
f.getMonth=hj.prototype.getUTCMonth;f.getDate=hj.prototype.getUTCDate;f.getHours=hj.prototype.getUTCHours;f.getMinutes=hj.prototype.getUTCMinutes;f.getSeconds=hj.prototype.getUTCSeconds;f.getMilliseconds=hj.prototype.getUTCMilliseconds;f.getDay=hj.prototype.getUTCDay;f.setFullYear=hj.prototype.setUTCFullYear;f.setMonth=hj.prototype.setUTCMonth;f.setDate=hj.prototype.setUTCDate;f.setHours=hj.prototype.setUTCHours;f.setMinutes=hj.prototype.setUTCMinutes;f.setSeconds=hj.prototype.setUTCSeconds;
f.setMilliseconds=hj.prototype.setUTCMilliseconds;var GK=new V(null,12,5,X,"January February March April May June July August September October November December".split(" "),null),HK=new V(null,7,5,X,"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),null);function IK(a){return Lw(function(a){return a instanceof cj},a)?U.h(M,Z.h(function(a){return a.getTime()},a)):U.h(M,a)}var JK=new V(null,12,5,X,[31,28,31,30,31,30,31,31,30,31,30,31],null);function KK(a){a=z(IK(R([a,1],0)))?11:a-1;return JK.c?JK.c(a):JK.call(null,a)}
function LK(a){a=null!=a&&(a.C&64||x===a.Pb)?U.h(Xw,a):a;var b=F.h(a,gD),c=F.h(a,CB),d=F.h(a,KB),e=F.h(a,fD),g=F.h(a,xD),h=F.h(a,$B),k=F.h(a,cE),l=z(c)?1<=c&&12>=c:null;if(z(b))if(z(c))if(z(l)){var m=KK(c);b=0===cw(b,400)?!0:0===cw(b,100)?!1:0===cw(b,4)?!0:!1;b=z(b)?IK(R([c,2],0)):b;m=z(b)?m+1:m}else m=l;else m=c;else m=z(c)?z(l)?KK(c):l:c;var p=z(d)?z(m)?1<=d&&d<=m:1<=d&&31>=d:null,u=z(e)?0<=e&&23>=e:null,A=z(g)?0<=g&&59>=g:null,b=z(h)?0<=h&&60>=h:null;m=z(k)?0<=k&&999>=k:null;if(!Lw(Ov,hx(ft,new V(null,
6,5,X,[l,p,u,A,b,m],null)))){var B=Kw,c=!1===l?T.j(B,CB,c):B,d=!1===p?T.j(c,KB,d):c,e=!1===u?T.j(d,fD,e):d,g=!1===A?T.j(e,xD,g):e,h=!1===b?T.j(g,$B,h):g,k=!1===m?T.j(h,cE,k):h;throw Qz("Date is not valid",new y(null,3,[OB,TB,pA,a,fA,k],null));}}function MK(a,b){return I(ax(function(a,d){return z(IK(R([d,b],0)))?a:null},a))}function NK(a){a=Z.h(function(a){return a instanceof Y||a instanceof Nu?""+D.c(a):a},a);return U.j(wm,"%s not implemented yet",a)};function OK(a,b){return vE("string"===typeof a?a:wE(a),b,"")}function PK(a){return new V(null,2,5,X,[mD,a],null)}function QK(a,b){var c=G(b);I(c);K(c);for(var c=b,d=tv;;){var e=c,g=G(e),c=I(g),g=K(g);var h=c;h=z(h)?a.c?a.c(c):a.call(null,c):h;if(z(h))d=sv.h(d,c),c=g;else return new V(null,2,5,X,[d,e],null)}}function RK(a,b){var c=QK(dz([a]),b),d=S(c,0,null),c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[tC,U.j(D,a,d)],null),c],null)}
function SK(a,b){var c=G(b),d=I(c),c=K(c);if(M.h(d,"'"))return new V(null,2,5,X,[PK("'"),c],null);c=QK(Nw(new az(null,new y(null,1,["'",null],null),null)),b);d=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[PK(d),Pu(c)],null)}function TK(a,b){var c=QK(function(a){return ht(lz(/[a-zA-Z']/,a))},b),d=S(c,0,null),c=S(c,1,null);return new V(null,2,5,X,[PK(U.j(D,a,d)),c],null)}
function UK(a){var b=a;for(a=tv;;){var c=G(b),b=I(c),c=K(c),d=M.h(b,"'")?SK:z(lz(/[a-zA-Z]/,""+D.c(b)))?RK:TK,b=d.h?d.h(b,c):d.call(null,b,c),c=S(b,0,null),b=S(b,1,null);a=sv.h(a,c);if(!G(b))return a}}
function VK(a,b){var c=I(a);var d=wE(Pu(a));var e=P(b)-1,g=[D.c(c),D.c(d.substring(0,e))].join("");d=M.h(b,g)?new V(null,2,5,X,[g,fw(d,e,P(d))],null):new V(null,2,5,X,[null,[D.c(c),D.c(d)].join("")],null);c=S(d,0,null);d=S(d,1,null);return z(c)?new V(null,2,5,X,[new V(null,2,5,X,[bA,c],null),d],null):null}
function WK(a,b,c){a=QK(function(a){return lz(/\d/,a)},a);var d=S(a,0,null);a=S(a,1,null);return P(d)>=b?new V(null,2,5,X,[function(){var a=U.h(D,bx(c,d));return parseInt(a)}(),Dw.h(cx(c,d),a)],null):new V(null,2,5,X,[function(){var a=U.h(D,d);return parseInt(a)}(),a],null)}function XK(a,b,c,d){c=WK(a,c,d);a=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[b,a],null),c],null)}function YK(a,b){return function(c){return XK(c,gD,a,b)}}
function ZK(a,b){return function(c){return XK(c,NB,a,b)}}function $K(a){return function(b){return XK(b,GC,a,2)}}function aL(){return function(a){return XK(a,CB,1,2)}}function bL(a,b){return function(c){return XK(c,KB,a,b)}}function cL(a){return function(b){return XK(b,fD,a,2)}}function dL(a){return function(b){return XK(b,XD,a,2)}}function eL(a){return function(b){return XK(b,xD,a,2)}}function fL(a){return function(b){return XK(b,$B,a,2)}}function gL(a,b){return function(c){return XK(c,cE,a,b)}}
function hL(a,b,c){b=60*parseInt(b,10)+parseInt(c,10);a=M.h(a,"+")?$v:Zv;return new V(null,2,5,X,[rA,new Wi(aj,a.c?a.c(b):a.call(null,b))],null)}
function iL(a){return function(b){var c=G(b);var d=I(c);var e=K(c),g=function(){return function(){return Qz([D.c("Invalid timezone format: "),D.c(b)].join(""),new y(null,1,[OB,aC],null))}}(b,c,d,e,d,e),h=function(a,b,c,d,e,g){return function(a){var b=wE(bx(4,g)),c=lz(/^(\d{2})(\d{2})/,b);return z(c)?(S(c,0,null),b=S(c,1,null),c=S(c,2,null),new V(null,2,5,X,[hL(a,b,c),cx(4,g)],null)):null}}(b,c,d,e,d,e,g),k=function(a,b,c,d,e,g){return function(a){var b=wE(bx(5,g)),c=lz(/^(\d{2}):(\d{2})/,b);return z(c)?
(S(c,0,null),b=S(c,1,null),c=S(c,2,null),new V(null,2,5,X,[hL(a,b,c),cx(5,g)],null)):null}}(b,c,d,e,d,e,g,h);if(z((new az(null,new y(null,2,["+",null,"-",null],null),null)).call(null,d)))switch(c=a instanceof Y?a.Sb:null,c){case "dddd":h=h(d);if(z(h))return h;d=k(d);if(z(d))return d;throw g();case "long":h=h(d);if(z(h))return h;d=k(d);if(z(d))return d;throw g();default:throw Error([D.c("No matching clause: "),D.c(c)].join(""));}else{if(M.h(d,"Z"))return new V(null,1,5,X,[new V(null,2,5,X,[rA,hL(Zv,
"0","0")],null)],null);var l=a instanceof Y?a.Sb:null;switch(l){case "abbr":var m=bx(3,b);d=QK(function(){return function(a){return lz(/[A-Z]/,a)}}(m,a,l,b,c,d,e,d,e,g,h,k),m);k=S(d,0,null);S(d,1,null);if(M.h(P(k),3))return new V(null,2,5,X,[new V(null,2,5,X,[rA,wE(k)],null),cx(3,b)],null);throw g();case "full":throw Qz([D.c("Cannot parse long form timezone:"),D.c(b)].join(""),new y(null,1,[OB,aC],null));default:throw g();}}}}
function jL(){return function(a){var b=new V(null,2,5,X,[bx(2,a),cx(2,a)],null);var c=S(b,0,null);a=S(c,0,null);c=S(c,1,null);b=S(b,1,null);var d=[D.c(a),D.c(c)].join(""),c=z((new az(null,new y(null,4,["AM",null,"am",null,"pm",null,"PM",null],null),null)).call(null,d))?new V(null,2,5,X,[d,b],null):z((new az(null,new y(null,2,["a",null,"p",null],null),null)).call(null,a))?new V(null,2,5,X,[(new y(null,2,["a","am","p","pm"],null)).call(null,a),lv(c,b)],null):z((new az(null,new y(null,2,["A",null,"P",
null],null),null)).call(null,a))?new V(null,2,5,X,[(new y(null,2,["A","am","P","pm"],null)).call(null,a),lv(c,b)],null):null;a=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[oB,qw.c(a)],null),wE(c)],null)}}
function kL(a,b,c){c=Dw.h(c,Z.h(function(a){return a.substring(0,3)},c));var d=I(hx(Ow.h(Pw.h(M,a),qv),Z.h(function(){return function(b){return new V(null,2,5,X,[b,OK(a,mz([D.c("^"),D.c(b)].join("")))],null)}}(c),c))),e=S(d,0,null),d=S(d,1,null);if(z(e))return new V(null,2,5,X,[new V(null,2,5,X,[b,cw(MK(c,e),12)],null),d],null);throw Qz([D.c("Could not parse "),D.c(rw(b)),D.c(" name")].join(""),new y(null,4,[OB,aC,HC,BC,uD,b,lE,d],null));}
function lL(){return function(a){return qx.j(kL(a,CB,GK),new V(null,2,5,X,[0,1],null),Yu)}}function mL(){return function(a){return kL(a,KB,HK)}}function nL(a){return function(b){var c=OK(b,mz([D.c("^"),D.c(a)].join("")));if(M.h(b,c))throw Qz("Quoted text not found",new y(null,2,[OB,aC,zA,PA],null));return new V(null,2,5,X,[new V(null,2,5,X,[mD,a],null),c],null)}}
function oL(){return function(a){var b=VK(a,"st");if(z(b))return b;b=VK(a,"nd");if(z(b))return b;b=VK(a,"rd");return z(b)?b:VK(a,"th")}}
function pL(a){var b=S(a,0,null);a=S(a,1,null);if(M.h(b,tC))switch(a){case "S":return gL(1,2);case "SSS":return gL(3,3);case "s":return fL(1);case "ss":return fL(2);case "m":return eL(1);case "mm":return eL(2);case "h":return cL(1);case "hh":return cL(2);case "H":return dL(1);case "HH":return dL(2);case "d":return bL(1,2);case "dd":return bL(2,2);case "D":return bL(1,3);case "DD":return bL(2,3);case "DDD":return bL(3,3);case "M":return aL();case "MM":return aL();case "MMM":return lL();case "MMMM":return lL();
case "y":return YK(1,4);case "yy":return YK(2,2);case "yyyy":return YK(4,4);case "Y":return YK(1,4);case "YY":return YK(2,2);case "YYYY":return YK(4,4);case "x":return ZK(1,4);case "xx":return ZK(2,2);case "xxxx":return ZK(4,4);case "w":return $K(1);case "ww":return $K(2);case "E":return mL();case "EEE":return mL();case "EEEE":return mL();case "a":return jL();case "A":return jL();case "Z":return iL(IA);case "ZZ":return iL(sA);case "ZZZ":return iL(PB);case "ZZZZ":return iL(PB);case "z":return iL(PB);
case "zz":return iL(PB);case "zzz":return iL(PB);case "zzzz":return iL(EA);case "o":return oL();default:throw Qz([D.c("Illegal pattern component: "),D.c(a)].join(""),new y(null,1,[OB,fB],null));}else return nL(a)}
function qL(a,b){for(var c=Z.h(pL,UK(a)),d=G(c),e=I(d),g=K(d),h=tv,k=b,l=c,m=h;;){var p=k,u=l,A=G(u),B=I(A),E=K(A),J=B,L=E,Q=m,k=function(a,c,d,e){return function(){return Qz([D.c("Invalid format: "),D.c(b),D.c(" is malformed at "),D.c(Az(R([e],0),$s()))].join(""),new y(null,2,[OB,aC,HC,IC],null))}}(k,l,m,p,u,A,B,E,J,L,Q,b,c,c,d,e,g,e,g,h);if(G(p)){if(null==J)throw k();l=J.c?J.c(p):J.call(null,p);k=S(l,0,null);l=S(l,1,null);Q=sv.h(Q,k);k=l;l=L;m=Q}else{if(z(J))throw k();return Q}}}
function rL(a,b,c){b=null!=b&&(b.C&64||x===b.Pb)?U.h(Xw,b):b;b=F.h(b,nE);c=Rw.h(Kw,hx(Ow.h(new az(null,new y(null,1,[mD,null],null),null),I),c));var d=null!=c&&(c.C&64||x===c.Pb)?U.h(Xw,c):c,e=F.h(d,xD),g=F.h(d,XD),h=F.h(d,cE);c=F.h(d,rA);var k=F.h(d,oB),l=F.h(d,CB),m=F.h(d,KB),p=F.h(d,$B),u=F.h(d,fD),A=F.h(d,gD),B=z(A)?A:z(b)?b:0,E=z(l)?l-1:null,J=z(k)?z((new az(null,new y(null,2,[FB,null,PC,null],null),null)).call(null,k))?function(){var a=u+12;return M.h(a,24)?12:a}():M.h(u,12)?0:u:g;b=wv.w(T.j(d,
fD,J),XD,R([oB],0));c=c instanceof Wi?c:new Wi(bj,0);LK(b);b=function(){var b=a instanceof Y?a.Sb:null;switch(b){case "goog.date.Date":return new cj(B,E,m);case "goog.date.DateTime":return new hj(B,E,m,J,e,p,h);case "goog.date.UtcDateTime":return new FK(B,E,m,J,e,p,h);default:throw Error([D.c("No matching clause: "),D.c(b)].join(""));}}();b.add(c);return b};function sL(a,b,c,d,e){c=1>d?""+D.c(c):[D.c(wE(bx(d-P(""+D.c(c)),ex("0")))),D.c(c)].join("");d=P(c);e=c.substring(d-e);return new V(null,2,5,X,[[D.c(a),D.c(e)].join(""),b],null)}var tL=function tL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return tL.c(arguments[0]);case 2:return tL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};tL.c=function(a){return tL.h(a,a)};
tL.h=function(a,b){return function(c,d){return sL(c,d,d.getMilliseconds(),a,b)}};tL.I=2;var uL=function uL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return uL.c(arguments[0]);case 2:return uL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};uL.c=function(a){return uL.h(a,a)};uL.h=function(a,b){return function(c,d){return sL(c,d,d.getSeconds(),a,b)}};uL.I=2;
var vL=function vL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return vL.c(arguments[0]);case 2:return vL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};vL.c=function(a){return vL.h(a,a)};vL.h=function(a,b){return function(c,d){return sL(c,d,d.getMinutes(),a,b)}};vL.I=2;
var wL=function wL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return wL.c(arguments[0]);case 2:return wL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};wL.c=function(a){return wL.h(a,a)};wL.h=function(a,b){return function(c,d){var e=cw(d.getHours(),12);return sL(c,d,0===e?12:e,a,b)}};wL.I=2;
var xL=function xL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return xL.c(arguments[0]);case 2:return xL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};xL.c=function(a){return xL.h(a,a)};xL.h=function(a,b){return function(c,d){return sL(c,d,d.getHours(),a,b)}};xL.I=2;
var yL=function yL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return yL.c(arguments[0]);case 2:return yL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};yL.c=function(a){return yL.h(a,a)};yL.h=function(a,b){return function(c,d){return sL(c,d,d.getDate(),a,b)}};yL.I=2;
var zL=function zL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return zL.c(arguments[0]);case 2:return zL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};zL.c=function(a){return zL.h(a,a)};zL.h=function(a,b){return function(c,d){return sL(c,d,d.getMonth()+1,a,b)}};zL.I=2;
var AL=function AL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return AL.c(arguments[0]);case 2:return AL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};AL.c=function(a){return AL.h(a,a)};AL.h=function(a,b){return function(c,d){return sL(c,d,d.getYear(),a,b)}};AL.I=2;function BL(a){return function(b,c){return new V(null,2,5,X,[U.j(D,b,a),c],null)}}
function CL(a,b,c,d,e){var g=Nx(function(){return z(e)?Z.h(function(){return function(a){return a.substring(0,3)}}(d),d):d}());return new V(null,2,5,X,[[D.c(a),D.c(g.c?g.c(c):g.call(null,c))].join(""),b],null)}function DL(a){return function(b,c){return CL(b,c,c.getMonth(),GK,a)}}function EL(a){return function(b,c){return CL(b,c,c.getDay(),HK,a)}}
var FL=function FL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return FL.c(arguments[0]);case 2:return FL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};FL.c=function(a){return FL.h(a,a)};FL.h=function(a,b){return function(c,d){var e=d.getYear(),g=d.getMonth(),h=d.getDate(),k=M.h(g,0),l=M.h(g,11),g=Ui(e,g,h),e=k&&52<=g?e-1:l&&M.h(g,1)?e+1:e;return sL(c,d,e,a,b)}};FL.I=2;
var GL=function GL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return GL.c(arguments[0]);case 2:return GL.h(arguments[0],arguments[1]);default:throw Error([D.c("Invalid arity: "),D.c(c.length)].join(""));}};GL.c=function(a){return GL.h(a,a)};GL.h=function(a,b){return function(c,d){var e=d.getYear(),g=d.getMonth(),h=d.getDate();return sL(c,d,Ui(e,g,h),a,b)}};GL.I=2;
function HL(a){return function(b,c){var d=c.getHours(),d=z(a)?12>d?"AM":"PM":12>d?"am":"pm";return new V(null,2,5,X,[[D.c(b),D.c(d)].join(""),c],null)}}function IL(){return function(a,b){return b instanceof FK?new V(null,2,5,X,[[D.c(a),D.c(gj(b))].join(""),b],null):new V(null,2,5,X,[a,b],null)}}
function JL(a){return function(b,c){var d=a.c?a.c(c):a.call(null,c),e=function(){switch(d){case 1:return"st";case 2:return"nd";case 3:return"rd";case 21:return"st";case 22:return"nd";case 23:return"rd";case 31:return"st";default:return"th"}}();return new V(null,2,5,X,[[D.c(b),D.c(e)].join(""),c],null)}}
function KL(a){var b=S(a,0,null);a=S(a,1,null);if(M.h(b,tC))switch(a){case "S":return new V(null,3,5,X,[cE,1,2],null);case "SSS":return new V(null,3,5,X,[cE,3,3],null);case "s":return new V(null,3,5,X,[$B,1,2],null);case "ss":return new V(null,3,5,X,[$B,2,2],null);case "m":return new V(null,3,5,X,[xD,1,2],null);case "mm":return new V(null,3,5,X,[xD,2,2],null);case "h":return new V(null,3,5,X,[fD,1,2],null);case "hh":return new V(null,3,5,X,[fD,2,2],null);case "H":return new V(null,3,5,X,[XD,1,2],
null);case "HH":return new V(null,3,5,X,[XD,2,2],null);case "d":return new V(null,3,5,X,[jA,1,2],null);case "dd":return new V(null,3,5,X,[jA,2,2],null);case "D":return new V(null,3,5,X,[jA,1,3],null);case "DD":return new V(null,3,5,X,[jA,2,3],null);case "DDD":return new V(null,3,5,X,[jA,3,3],null);case "M":return new V(null,3,5,X,[HB,1,2],null);case "MM":return new V(null,3,5,X,[HB,2,2],null);case "MMM":return new V(null,2,5,X,[gA,!0],null);case "MMMM":return new V(null,2,5,X,[gA,!1],null);case "y":return new V(null,
3,5,X,[sC,1,4],null);case "yy":return new V(null,3,5,X,[sC,2,2],null);case "yyyy":return new V(null,3,5,X,[sC,4,4],null);case "Y":return new V(null,3,5,X,[sC,1,4],null);case "YY":return new V(null,3,5,X,[sC,2,2],null);case "YYYY":return new V(null,3,5,X,[sC,4,4],null);case "x":return new V(null,3,5,X,[NB,1,4],null);case "xx":return new V(null,3,5,X,[NB,2,2],null);case "xxxx":return new V(null,3,5,X,[NB,4,4],null);case "w":return new V(null,3,5,X,[GC,1,2],null);case "ww":return new V(null,3,5,X,[GC,
2,2],null);case "E":return new V(null,2,5,X,[nC,!0],null);case "EEE":return new V(null,2,5,X,[nC,!0],null);case "EEEE":return new V(null,2,5,X,[nC,!1],null);case "a":return new V(null,2,5,X,[oB,!1],null);case "A":return new V(null,2,5,X,[oB,!0],null);case "Z":return new V(null,1,5,X,[rA],null);case "ZZ":return new V(null,1,5,X,[rA],null);case "o":return new V(null,1,5,X,[bA],null);default:throw Error([D.c("No matching clause: "),D.c(a)].join(""));}else return new V(null,2,5,X,[mD,a],null)}
function LL(a){var b=a instanceof Y?a.Sb:null;switch(b){case "millis":return function(){return function(a){return a.getMilliseconds()}}(a,b);case "seconds":return function(){return function(a){return a.getSeconds()}}(a,b);case "minutes":return function(){return function(a){return a.getMinutes()}}(a,b);case "hours":return function(){return function(a){return a.getHours()}}(a,b);case "HOURS":return function(){return function(a){return a.getHours()}}(a,b);case "day":return function(){return function(a){return a.getDate()}}(a,
b);case "month":return function(){return function(a){return a.getMonth()}}(a,b);case "year":return function(){return function(a){return a.getYear()}}(a,b);default:throw Error([D.c("No matching clause: "),D.c(b)].join(""));}}
function ML(a,b,c){var d=G(c);c=I(d);d=K(d);c=c instanceof Y?c.Sb:null;switch(c){case "ordinal-suffix":return--b,a=a.c?a.c(b):a.call(null,b),a=S(a,0,null),JL(LL(a));case "month-name":return U.h(DL,d);case "day":return U.h(yL,d);case "timezone":return U.h(IL,d);case "meridiem":return U.h(HL,d);case "month":return U.h(zL,d);case "weekyear":return U.h(FL,d);case "seconds":return U.h(uL,d);case "day-name":return U.h(EL,d);case "year":return U.h(AL,d);case "weekyear-week":return U.h(GL,d);case "hours":return U.h(wL,
d);case "quoted":return U.h(BL,d);case "minutes":return U.h(vL,d);case "HOURS":return U.h(xL,d);case "millis":return U.h(tL,d);default:throw Error([D.c("No matching clause: "),D.c(c)].join(""));}};var NL={id:"UTC",std_offset:0,names:new V(null,1,5,X,["UTC"],null),transitions:tv};function OL(a,b,c,d,e,g,h){this.fe=a;this.re=b;this.ce=c;this.we=d;this.Ma=e;this.ea=g;this.K=h;this.C=2229667594;this.Y=8192}f=OL.prototype;f.na=function(a,b){return this.X(null,b,null)};f.X=function(a,b,c){switch(b instanceof Y?b.Sb:null){case "format-str":return this.fe;case "overrides":return this.re;case "default-year":return this.ce;case "timezone":return this.we;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return nz(b,function(){return function(a){return nz(b,vz,""," ","",c,a)}}(this),"#cljs-time.format.Formatter{",", ","}",c,Dw.h(new V(null,4,5,X,[new V(null,2,5,X,[LB,this.fe],null),new V(null,2,5,X,[xA,this.re],null),new V(null,2,5,X,[nE,this.ce],null),new V(null,2,5,X,[rA,this.we],null)],null),this.ea))};f.ic=function(){return new dy(0,this,4,new V(null,4,5,X,[LB,xA,nE,rA],null),z(this.ea)?zu(this.ea):Iw())};f.sa=function(){return this.Ma};f.xa=function(){return 4+P(this.ea)};
f.ha=function(){var a=this.K;return null!=a?a:this.K=a=gw(this)};f.O=function(a,b){return z(z(b)?this.constructor===b.constructor&&cy(this,b):b)?!0:!1};f.lf=function(a,b){return Tv(new az(null,new y(null,4,[rA,null,xA,null,LB,null,nE,null],null),null),b)?wv.h(Zt(Rw.h(Kw,this),this.Ma),b):new OL(this.fe,this.re,this.ce,this.we,this.Ma,Hw(wv.h(this.ea,b)),null)};
f.Ob=function(a,b,c){return z(pw.h?pw.h(LB,b):pw.call(null,LB,b))?new OL(c,this.re,this.ce,this.we,this.Ma,this.ea,null):z(pw.h?pw.h(xA,b):pw.call(null,xA,b))?new OL(this.fe,c,this.ce,this.we,this.Ma,this.ea,null):z(pw.h?pw.h(nE,b):pw.call(null,nE,b))?new OL(this.fe,this.re,c,this.we,this.Ma,this.ea,null):z(pw.h?pw.h(rA,b):pw.call(null,rA,b))?new OL(this.fe,this.re,this.ce,c,this.Ma,this.ea,null):new OL(this.fe,this.re,this.ce,this.we,this.Ma,T.j(this.ea,b,c),null)};
f.oa=function(){return G(Dw.h(new V(null,4,5,X,[new V(null,2,5,X,[LB,this.fe],null),new V(null,2,5,X,[xA,this.re],null),new V(null,2,5,X,[nE,this.ce],null),new V(null,2,5,X,[rA,this.we],null)],null),this.ea))};f.ya=function(a,b){return new OL(this.fe,this.re,this.ce,this.we,b,this.ea,this.K)};f.wa=function(a,b){return Jv(b)?this.Ob(null,xt.h(b,0),xt.h(b,1)):nt(vt,this,b)};
function PL(a){a=new y(null,2,[LB,a,rA,NL],null);return new OL(LB.c(a),xA.c(a),nE.c(a),rA.c(a),null,wv.w(a,LB,R([xA,nE,rA],0)),null)}function QL(a){return function(){throw{name:zD,message:NK(R([rw(a)],0))};}}var RL;
a:for(var SL=[Wz,Zz,mA,oA,pA,qA,vA,wA,AA,BA,FA,GA,HA,JA,LA,RA,SA,UA,VA,YA,cB,jB,lB,nB,qB,tB,uB,zB,NB,RB,WB,XB,dC,gC,lC,qC,sC,uC,wC,CC,GC,JC,KC,RC,bD,hD,rD,LD,RD,TD,$D,hE,kE],TL=[QL(vC),PL("HH:mm"),PL("'T'HH:mm:ss.SSSZZ"),PL("yyyyDDD"),PL("yyyy-MM-dd"),PL("HH"),PL("HH:mm:ssZZ"),PL("xxxx-'W'ww-e"),PL("xxxx-'W'ww-e'T'HH:mm:ss.SSSZZ"),PL("yyyy-MM-dd'T'HH:mm:ss.SSS"),PL("yyyyMMdd'T'HHmmss.SSSZ"),PL("yyyy-MM-dd'T'HH:mm:ss.SSSZZ"),PL("HHmmssZ"),QL(bC),PL("xxxx'W'wwe"),PL("'T'HHmmssZ"),QL(Tz),PL("yyyy-MM-dd'T'HH:mm:ssZZ"),
PL("yyyy-MM-dd"),QL(OA),PL("EEE, dd MMM yyyy HH:mm:ss Z"),PL("yyyy-MM-dd'T'HH:mm:ss.SSS"),PL("yyyyDDD'T'HHmmss.SSSZ"),PL("yyyy-DDD"),PL("HH:mm:ss.SSS"),PL("yyyy-MM-dd'T'HH:mm"),PL("HH:mm:ss.SSSZZ"),PL("xxxx'W'wwe'T'HHmmss.SSSZ"),PL("xxxx"),PL("HHmmss.SSSZ"),PL("HH:mm:ss"),PL("yyyy-DDD'T'HH:mm:ss.SSSZZ"),PL("yyyy-DDD'T'HH:mm:ssZZ"),PL("HH:mm:ss.SSS"),PL(nA),QL($C),PL("yyyy"),PL("'T'HH:mm:ssZZ"),PL("xxxx'W'wwe'T'HHmmssZ"),PL("yyyyMMdd"),PL("xxxx-'W'ww"),QL(SB),PL("yyyyDDD'T'HHmmssZ"),PL("yyyy-MM"),
QL(xC),PL("xxxx-'W'ww-e"),PL("yyyy-MM-dd'T'HH"),QL(mB),PL("yyyy-MM-dd'T'HH:mm:ss"),PL("xxxx-'W'ww-e'T'HH:mm:ssZZ"),PL("yyyyMMdd'T'HHmmssZ"),PL("yyyy-MM-dd HH:mm:ss"),PL("'T'HHmmss.SSSZ")],UL=SL.length,VL=0,WL=qu(ny);;)if(VL<UL)var XL=VL+1,YL=WL.mf(null,SL[VL],TL[VL]),VL=XL,WL=YL;else{RL=su(WL);break a}var ZL=new az(null,new y(null,9,[Wz,null,JA,null,SA,null,YA,null,lC,null,qC,null,JC,null,bD,null,LD,null],null),null);cF.h(ez(jy(RL)),ZL);function $L(a,b){return rL(JD,a,qL(LB.c(a),b))}
function aM(a){return I(function(){return function c(d){return new sw(null,function(){for(var e=d;;)if(e=G(e)){if(Kv(e)){var g=vu(e),h=P(g),k=ww(h);a:for(var l=0;;)if(l<h){var m=xt.h(g,l);try{var p=$L(m,a)}catch(u){p=null}z(p)&&k.add(p);l+=1}else{p=!0;break a}return p?yw(k.ob(),c(wu(e))):yw(k.ob(),null)}g=I(e);try{k=$L(g,a)}catch(u){k=null}if(z(k))return lv(k,c(Pu(e)));e=Pu(e)}else return null},null,null)}(ky(RL))}())};PL("dd. MM. yyyy");var bM=PL("dd.MM.yyyy HH:mm");function cM(a,b){return F.h(I(gx(function(a){return M.h(b,F.h(a,"id"))},a)),"value")}function dM(a,b){return Z.h(function(a){var c=S(a,0,null),e=S(a,1,null);return lv(c,Z.h(function(a,b){return function(a){return F.j(a,b,0)}}(a,c,e),b))},a)}function eM(a){return Z.h(function(a){return new Date(gE.c(a))},a)}
function fM(a,b){var c=Mz(b,R([Nz,!1],0)),d=[D.c('\x3cdiv class\x3d"tooltip"\x3e'),D.c("\x3cdiv\x3e%s\x3c/div\x3e"),D.c("\x3cdiv\x3eBodový zisk: %s bodů\x3c/div\x3e"),D.c("\x3cdiv\x3eCelkový nákupní limit: %s\x3c/div\x3e"),D.c("\x3cdiv\x3eDostupný limit: %s\x3c/div\x3e"),D.c("\x3cdiv\x3eDůvod: %s\x3c/div\x3e"),D.c("\x3c/div\x3e")].join("");var e=(new hj(F.h(I(c),"x"))).toIsoString();if(Ca(e))e=null;else{e=aM(e);var g=null!=bM&&(bM.C&64||x===bM.Pb)?U.h(Xw,bM):bM,h=F.h(g,LB);F.h(g,hA);if(null==e)throw Error("Assert failed: (not (nil? dt))");
if(!(e instanceof cj))throw Error("Assert failed: (instance? goog.date.Date dt)");a:for(g=kx(KL,UK(h)),h=Tw(Pw.h(ML,g),g),g=G(h),I(g),K(g),g=e,e="";;){var k=G(h),h=I(k),l=K(k),k=h,h=l;if(null==k)break a;g=k.h?k.h(e,g):k.call(null,e,g);e=S(g,0,null);g=S(g,1,null)}}g=cM(c,"points");h=CE(cM(c,"price_limit"));k=CE(cM(c,"available_limit"));c=Uz.c(F.h(a,F.h(I(c),"index")));return wm(d,e,g,h,k,c)}
RH("#history-limit",new V(null,2,5,X,[function(a,b){var c=I(F.h(b,cA)),d=I(F.h(b,pB)),c=new y(null,5,[rE,new y(null,3,[pD,"x",dE,c,hB,lv(lv("x",eM(d)),dM(c,d))],null),BD,new y(null,1,[pD,new y(null,3,[OB,"timeseries",Sz,new y(null,1,[kA,"%d-%m-%Y"],null),yB,!0],null)],null),eE,new y(null,1,[tD,Pw.h(fM,d)],null),QC,a,MD,new y(null,1,[pE,!0],null)],null);return c3.generate(Jz(c))},new V(null,2,5,X,[new y(null,3,[kB,cA,OB,aE,fE,!0],null),new y(null,3,[kB,pB,OB,cC,fE,!0],null)],null)],null));fa("backend.core.run",function(){window.__dispatch=function(a){a=new V(null,2,5,X,[iB,a],null);return AI.c?AI.c(a):AI.call(null,a)};return iJ()});
})();
